package com.netapp.acquisition.aquisitionserverside.interfaces.data;

import javax.persistence.*;
import java.io.Serializable;

/**
 * Created by weis.
 * Date: Jun 8, 2010
 * Time: 2:34:16 PM
 */

@Entity
@Table(name = "au_attrs")
public class AcquisitionUnitAttribute implements Serializable {
    @Transient    
    private static long serialVersionUID = 1L;

    @Id
    @GeneratedValue
    private long id;

    @Column(name = "type", nullable = false)
    private String type;

    @Column(name = "subType", nullable = false)
    private String subType;

    @Column(name = "name", nullable = false)
    private String name;

    @Column(name = "value", nullable = false)
    private String value;

    @Version
    @Column(name = "version")
    private long version;

    @ManyToOne
    @JoinColumn(name = "au_id", nullable = false)
    private AcquisitionUnit acquisitionUnit;

    public AcquisitionUnitAttribute() {
    }

    public AcquisitionUnitAttribute(String type, String subType, String name, String value) {
        this.type = type;
        this.subType = subType;
        this.name = name;
        this.value = value;
    }

    public long getId() {
        return id;
    }

    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    public String getSubType() {
        return subType;
    }

    public void setSubType(String subType) {
        this.subType = subType;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getValue() {
        return value;
    }

    public void setValue(String value) {
        this.value = value;
    }

    public AcquisitionUnit getAcquisitionUnit() {
        return acquisitionUnit;
    }

    public void setAcquisitionUnit(AcquisitionUnit acquisitionUnit) {
        this.acquisitionUnit = acquisitionUnit;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        AcquisitionUnitAttribute that = (AcquisitionUnitAttribute) o;

        if (name != null ? !name.equals(that.name) : that.name != null) return false;
        if (subType != null ? !subType.equals(that.subType) : that.subType != null) return false;
        if (type != null ? !type.equals(that.type) : that.type != null) return false;
        if (value != null ? !value.equals(that.value) : that.value != null) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = type != null ? type.hashCode() : 0;
        result = 31 * result + (subType != null ? subType.hashCode() : 0);
        result = 31 * result + (name != null ? name.hashCode() : 0);
        result = 31 * result + (value != null ? value.hashCode() : 0);
        return result;
    }

    public AcquisitionUnitAttributeBusinessKey createBusinessKey() {
        return new AcquisitionUnitAttributeBusinessKey(type, subType, name, acquisitionUnit == null ? 0 : acquisitionUnit.getId());
    }

    public static class AcquisitionUnitAttributeBusinessKey implements Serializable {
        private String type;
        private String subType;
        private String name;
        private long acquisitionUnitId;

        public AcquisitionUnitAttributeBusinessKey(String type, String subType, String name, long acquisitionUnitId) {
            this.type = type;
            this.subType = subType;
            this.name = name != null ? name.toUpperCase() : null;
            this.acquisitionUnitId = acquisitionUnitId;
        }

        public String getType() {
            return type;
        }

        public String getSubType() {
            return subType;
        }

        public String getName() {
            return name;
        }

        public long getAcquisitionUnitId() {
            return acquisitionUnitId;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            AcquisitionUnitAttributeBusinessKey that = (AcquisitionUnitAttributeBusinessKey) o;

            if (acquisitionUnitId != that.acquisitionUnitId) return false;
            if (name != null ? !name.equals(that.name) : that.name != null) return false;
            if (subType != null ? !subType.equals(that.subType) : that.subType != null) return false;
            if (type != null ? !type.equals(that.type) : that.type != null) return false;

            return true;
        }

        @Override
        public int hashCode() {
            int result = type != null ? type.hashCode() : 0;
            result = 31 * result + (subType != null ? subType.hashCode() : 0);
            result = 31 * result + (name != null ? name.hashCode() : 0);
            result = 31 * result + (int) (acquisitionUnitId ^ (acquisitionUnitId >>> 32));
            return result;
        }


    }
}

