package com.netapp.acquisition.aquisitionserverside.interfaces.data;

/**
 * Types of updates that may be sent via the Acquisition Update JMS Topic.
 *
 * <p>Such messages will be "MapMessage"s with the following keys:
 * <ul>
 *      <li>UPDATE_TYPE:  Required.  Name off of this enum set as string.</li>
 *      <li>UPDATE_TIME:  Required.  Time (system) of this update as long.</li>
 *      <li>ACQUISITION_UNIT_ID:  Optional.  Acquisition Unit involved as long id. </li>
 *      <li>DATASOURCE_ID:  Optional.  DataSource ID involved as long. </li>
 *      <li>DATASOURCE_NAME:  Optional.  DataSource name involved as string.  </li>
 * </ul>
 *
 */
public enum AcquisitionUpdateType {

    /**
     * A DataSource was added.  Populated fields:  ALL
     */
    DATA_SOURCE_ADD,

    /**
     * A DataSource was updated.  Populated fields:  ALL
     */
    DATA_SOURCE_UPDATE,

    /**
     * A DataSource was removed.  Populated fields:  UPDATE_TYPE, UPDATE_TIME, ACQUISITION_UNIT_ID, DATASOURCE_NAME
     */
    DATA_SOURCE_REMOVE,

    /**
     * A DataSource crawl was initiated.  Populated fields:  ALL
     */
    DATA_SOURCE_CRAWL,

    /**
     * A DataSource "force acquisition" was initiated.  Populated fields:  ALL
     */
    DATA_SOURCE_FORCE,

    /**
     * A DataSource audit entry was created.  Populated fields:  UPDATE_TYPE, UPDATE_TIME
     */
    DATA_SOURCE_AUDIT_CREATE,

    /**
     * A DataSource audit entry was updated.  Populated fields:  UPDATE_TYPE, UPDATE_TIME
     */
    DATA_SOURCE_AUDIT_UPDATE,

    /**
     * A DataSource audit entry was closed.  Populated fields:  UPDATE_TYPE, UPDATE_TIME
     */
    DATA_SOURCE_AUDIT_CLOSE,

    /**
     * An acquisition unit was registered.  Populated fields:  UPDATE_TYPE, UPDATE_TIME, ACQUISITION_UNIT_ID
     */
    ACQUISITION_UNIT_REGISTER,

    /**
     * An acquisition unit was updated.  Populated fields:  UPDATE_TYPE, UPDATE_TIME, ACQUISITION_UNIT_ID
     */
    ACQUISITION_UNIT_UPDATE,

    /**
     * An acquisition unit was unregistered.  Populated fields:  UPDATE_TYPE, UPDATE_TIME, ACQUISITION_UNIT_ID
     */
    ACQUISITION_UNIT_UNREGISTER,

    /**
     * The lease of an existing acquisition unit was renewed. Populated fields:  UPDATE_TYPE, UPDATE_TIME, ACQUISITION_UNIT_ID
     */
    ACQUISITION_UNIT_LEASE_RENEW
    

}
