/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.acquisition.aquisitionserverside.interfaces.data;

import javax.persistence.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

/**
 * Created by weis.
 * Date: Jun 8, 2010
 * Time: 11:33:39 AM
 */

@SuppressWarnings("UnusedDeclaration")
@NamedQueries({
        @NamedQuery(name = "findDataSourcesToRunWithAttributesByAuId",
                query = "select ds from DataSource ds where ds.auId=:auId and ds.manual <> 2"),
        @NamedQuery(name = "findDataSourcesByAuId",
                query = "select ds from DataSource ds where ds.auId=:auId")
})

@Entity
@Table(name = "ds")
public class DataSource implements Serializable {
    @Transient
    private final static long serialVersionUID = 1L;

    @Id
    @GeneratedValue
    private long id;

    @Column(name = "ds_type_id", nullable = false)
    private long dsTypeId;

    @Column(name = "au_id", nullable = false)
    private long auId;

    @Column(name = "name", nullable = false)
    private String name;

    @Column(name = "vendor", nullable = false)
    private String vendor;

    @Column(name = "model", nullable = false)
    private String model;

    @Column(name = "status", nullable = false)
    //todo verify enum
    private String status;

    @Column(name = "statusext", nullable = false)
    private String statusExt;

    @Column(name = "time", nullable = false)
    private long time;

    @Column(name = "manual", nullable = true)
    private int manual;

    @Column(name = "active", nullable = true)
    private boolean active;

    @OneToMany(mappedBy = "dataSource", cascade = CascadeType.ALL)
    Set<DataSourceAttribute> attributes = new HashSet<DataSourceAttribute>();

    public DataSource() {
    }

    public DataSource(long dsTypeId, long auId, String name, String vendor, String model, String status, String statusExt, long time, int manual, boolean active) {
        this.dsTypeId = dsTypeId;
        this.auId = auId;
        this.name = name;
        this.vendor = vendor;
        this.model = model;
        this.status = status;
        this.statusExt = statusExt;
        this.time = time;
        this.manual = manual;
        this.active = active;
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public long getDsTypeId() {
        return dsTypeId;
    }

    public void setDsTypeId(long dsTypeId) {
        this.dsTypeId = dsTypeId;
    }

    public long getAuId() {
        return auId;
    }

    public void setAuId(long auId) {
        this.auId = auId;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getVendor() {
        return vendor;
    }

    public void setVendor(String vendor) {
        this.vendor = vendor;
    }

    public String getModel() {
        return model;
    }

    public void setModel(String model) {
        this.model = model;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getStatusExt() {
        return statusExt;
    }

    public void setStatusExt(String statusExt) {
        this.statusExt = statusExt;
    }

    public long getTime() {
        return time;
    }

    public void setTime(long time) {
        this.time = time;
    }

    public int getManual() {
        return manual;
    }

    public void setManual(int manual) {
        this.manual = manual;
    }

    public boolean isActive() {
        return active;
    }

    public void setActive(boolean active) {
        this.active = active;
    }

    public Set<DataSourceAttribute> getAttributes() {
        return attributes;
    }

    public void setAttributes(Set<DataSourceAttribute> attributes) {
        this.attributes = attributes;
    }

    public void addAttribute(DataSourceAttribute dsa) {
        dsa.setDataSource(this);
        attributes.add(dsa);
    }

    public void removeAttribute(DataSourceAttribute dsa) {
        dsa.setDataSource(null);
        attributes.remove(dsa);
    }

    public Properties getAttributesAsProperties() {
        Properties p = new Properties();
        for (DataSourceAttribute attribute : attributes) {
            p.put(attribute.getName(), attribute.getValue());
        }
        return p;
    }

    public static enum DSStatus {
        Stopped,
        Running
    }
}
