package com.netapp.acquisition.aquisitionserverside.interfaces.data;

import javax.persistence.*;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

/**
 * Created by weis.
 * Date: Jun 11, 2010
 * Time: 3:16:04 PM
 */
@Entity
@Table(name = "ds_audit")
public class DataSourceAudit implements Serializable {

    static public final long END_DATE = 5000000000000L; // Fri Jun 11 04:53:20 EDT 2128

    @Transient
    private final static long serialVersionUID = 1L;

    @Id
    @GeneratedValue
    private long id;

    @Column(name = "package", nullable = false)
    private String packageName;

    @Column(name = "status", nullable = false)
    private String status;

    @Column(name = "type", nullable = false)
    @Enumerated(EnumType.STRING)
    private Type type;

    @Column(name = "ds_id", nullable = false)
    private long dsId;

    @Column(name = "starttime", nullable = false)
    private long startTime;

    @Column(name = "endtime", nullable = false)
    private long endTime;

    @Column(name = "numberOfTimes", nullable = false)
    private int numberOfTimes;

    @OneToMany(mappedBy = "audit", cascade = CascadeType.ALL)
    private Collection<DataSourceAuditDetail> details = new ArrayList<DataSourceAuditDetail>();

    @Version
    @Column(name = "version")
    private long version;

    public DataSourceAudit() {
    }

    public DataSourceAudit(String packageName, String status, Type type, long dsId, long startTime, long endTime, int numberOfTimes) {
        this.packageName = packageName;
        this.status = status;
        this.type = type;
        this.dsId = dsId;
        this.startTime = startTime;
        this.endTime = endTime;
        this.numberOfTimes = numberOfTimes;
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getPackageName() {
        return packageName;
    }

    public void setPackageName(String packageName) {
        this.packageName = packageName;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public Type getType() {
        return type;
    }

    public void setType(Type type) {
        this.type = type;
    }

    public long getDsId() {
        return dsId;
    }

    public void setDsId(long dsId) {
        this.dsId = dsId;
    }

    public long getStartTime() {
        return startTime;
    }

    public void setStartTime(long startTime) {
        this.startTime = startTime;
    }

    public long getEndTime() {
        return endTime;
    }

    public void setEndTime(long endTime) {
        this.endTime = endTime;
    }

    public int getNumberOfTimes() {
        return numberOfTimes;
    }

    public void setNumberOfTimes(int numberOfTimes) {
        this.numberOfTimes = numberOfTimes;
    }

    public long getVersion() {
        return version;
    }

    public void setVersion(long version) {
        this.version = version;
    }

    public boolean isOpen() {
        return this.endTime == END_DATE;
    }

    public static enum Type{
        START,
        ACQUIRING,
        STANDBY,
        SUCCESS,
        ERROR,
        WARNING,
        DISABLED
    }

    public Collection<DataSourceAuditDetail> getDetails() {
        return Collections.unmodifiableCollection(details);
    }

    public void addAcquisitionAuditMessage(DataSourceAuditDetail msg){
        msg.setAudit(this);
        details.add(msg);
    }

    public void removeAcquisitionAuditMessage(DataSourceAuditDetail msg){
        msg.setAudit(null);
        details.remove(msg);
    }
}
