/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.acquisition.aquisitionserverside.interfaces.data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class DataSourceDoesNotExistException extends AcquisitionServerException {

    private static final long serialVersionUID = 1L;

    private final List<Long> missingIds;

    public DataSourceDoesNotExistException() {
        super();
        this.missingIds = new ArrayList<Long>();
    }

    public DataSourceDoesNotExistException(final Long dataSourceId) {
        this(Arrays.asList(dataSourceId));
    }

    public DataSourceDoesNotExistException(List<Long> dataSourceIds) {
        super(DataSourceDoesNotExistException.buildString(dataSourceIds));
        this.missingIds = new ArrayList<Long>();
        this.missingIds.addAll(dataSourceIds);
    }

    public List<Long> getMissingIds() {
        return missingIds;
    }

    public static String buildString(List<Long> dataSourceIds) {
        final StringBuilder missingStr = new StringBuilder("DataSource Ids {");
        final int size = dataSourceIds.size();
        int count = 0;
        for (final Long id : dataSourceIds) {
            missingStr.append(id);
            if (count < size -1) {
                missingStr.append(", ");
            }
        }
        missingStr.append("} not found.");
        return missingStr.toString();
    }

}
