package com.netapp.acquisition.aquisitionserverside.interfaces.data;

import javax.persistence.*;
import java.io.Serializable;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * Created by weis.
 * Date: Jun 28, 2010
 * Time: 4:30:55 PM
 */
@Entity
@Table(name = "ds_type")
public class DataSourceType implements Serializable {
    @Transient
    private static final long serialVersionUID = 1L;

    @Id
    private long id;

    @Column(name = "name", nullable = false)
    private String name;

    @Column(name = "description", nullable = true)
    private String description;

    @OneToMany(mappedBy = "dataSourceType", cascade = CascadeType.ALL)
    @MapKey(name="name")
    private Map<String, DataSourceTypeAttribute> attrs = new HashMap<String, DataSourceTypeAttribute>();

    @OneToMany(mappedBy = "dataSourceType", cascade = CascadeType.ALL)
    private List<DataSourceTypeVendorModel> vendorModels = new LinkedList<DataSourceTypeVendorModel>();

    @Version
    @Column(name = "version")
    private long version;

    
    public DataSourceType() {
    }

    public DataSourceType(final long id, final String name, final String description) {
        this.id = id;
        this.name = name;
        this.description = description;
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public long getVersion() {
        return version;
    }

    public void setVersion(long version) {
        this.version = version;
    }

    public void addAttribute(String name, DataSourceTypeAttribute dstAttribute){
        attrs.put(name, dstAttribute);
        dstAttribute.setDataSourceType(this);
    }

    public void addVendor(DataSourceTypeVendorModel vendor){
        vendorModels.add(vendor);
        vendor.setDataSourceType(this);
    }
}
