package com.netapp.acquisition.aquisitionserverside.interfaces.data;

import javax.persistence.*;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

/**
 * Created by weis.
 * Date: Jun 28, 2010
 * Time: 4:33:58 PM
 */
//todo make this embeddable
@Entity
@Table(name = "ds_type_attrs")
public class DataSourceTypeAttribute implements Serializable {
    @Transient
    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue
    private long id;

    @ManyToOne
    @JoinColumn(name = "ds_type_id", nullable = false)
    private DataSourceType dataSourceType;

    @Column(name = "name", nullable = false)
    private String name;

    @Column(name = "value", nullable = false)
    private String value;

    @Column(name = "description", nullable = true)
    private String description;

    @Column(name = "label", nullable = true)
    private String label;

    @Transient
    private Type type;

    @Column(name = "type", nullable = true)
    private String typeString;

    @Column(name = "min", nullable = true)
    private String min;

    @Column(name = "max", nullable = true)
    private String max;

    @Transient
    private Set<String> enums;  // Original name in XML is enum, but it has being changed because is a Java's keyword

    @Column(name = "enum", nullable = true)
    private String enumsStr;

    @Column(name = "role", nullable = true)
    private String role;

    @Column(name = "editable")
    private boolean editable;

    @Column(name = "encrypt")
    private boolean encrypt;

    @Column(name = "guiorder", nullable = true)
    private Integer guiorder;

    @Version
    @Column(name = "version")
    private long version;


    public DataSourceTypeAttribute() {
    }

    public DataSourceTypeAttribute(final long ds_type_id, final String name, final String value) {
        this(ds_type_id,
                name,
                value,
                null,
                null,
                "string",
                null,
                null,
                null,
                "implementation",
                null,
                null,
                "0");
    }

    public DataSourceTypeAttribute(final long ds_type_id, final Map<String, String> attributes) {
        this(ds_type_id,
                attributes.get("name"),
                attributes.get("value"),
                attributes.get("description"),
                attributes.get("label"),
                attributes.get("type"),
                attributes.get("min"),
                attributes.get("max"),
                attributes.get("enum"),
                attributes.get("role"),
                attributes.get("editable"),
                attributes.get("encrypt"),
                attributes.get("guiorder"));
    }

    public DataSourceTypeAttribute(final long ds_type_id,
                                   final String name,
                                   final String value,
                                   final String description,
                                   final String label,
                                   final String type,
                                   final String min,
                                   final String max,
                                   final String enums,
                                   final String role,
                                   final String editable,
                                   final String encrypt,
                                   final String guiorder) {
        this.dataSourceType = new DataSourceType();
        this.dataSourceType.setId(ds_type_id);

        // Mandatory fields with data on them.
        this.name = getStringValue("name", name, true, null);
        this.value = getStringValue("value", value, true, null);

        // Optional fields with data on them.
        this.description = getStringValue("description", description, false, "");
        this.label = getStringValue("label", label, false, "");
        this.type = Type.fromString(type);
        this.typeString = type;
        this.min = getStringValue("min", min, false, "");
        this.max = getStringValue("max", max, false, "");
        this.enumsStr = getStringValue("enum", enums, false, "");
        this.enums = translateEnums();
        this.role = getStringValue("role", role, false, "mandatory");
        this.editable = getBooleanValue("editable", editable, false, true);
        this.encrypt = getBooleanValue("encrypt", encrypt, false, false);
        this.guiorder = getIntValue("guiorder", guiorder, true, 0);
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public void setDataSourceType(DataSourceType dataSourceType) {
        this.dataSourceType = dataSourceType;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public void setValue(final String value) {
        this.value = value;
    }

    public void setDescription(final String description) {
        this.description = description;
    }

    public void setLabel(final String label) {
        this.label = label;
    }

    public void setType(final Type type) {
        this.type = type;
    }

    public void setMin(final String min) {
        this.min = min;
    }

    public void setMax(final String max) {
        this.max = max;
    }

    public void setEnums(final Set<String> enums) {
        this.enums = enums;
    }

    public void setEnumsStr(final String enumsStr) {
        this.enumsStr = enumsStr;
    }

    public void setRole(final String role) {
        this.role = role;
    }

    public void setEditable(final boolean editable) {
        this.editable = editable;
    }

    public void setEncrypt(final boolean encrypt) {
        this.encrypt = encrypt;
    }


    public DataSourceType getDataSourceType() {
        return dataSourceType;
    }

    public String getName() {
        return name;
    }

    public String getValue() {
        return value;
    }

    public String getDescription() {
        return description;
    }

    public String getLabel() {
        return label;
    }

    public Type getType() {
        return type;
    }

    public String getMin() {
        return min;
    }

    public String getMax() {
        return max;
    }

    public Set<String> getEnums() {
        return enums;
    }

    public String getEnumsStr() {
        return enumsStr;
    }

    public String getRole() {
        return role;
    }

    public boolean isEditable() {
        return editable;
    }

    public boolean isEncrypt() {
        return encrypt;
    }

    public int getGuiorder() {
        return guiorder;
    }

    public boolean isMandatory() {
        return role.equals("mandatory");
    }

    public boolean isHidden() {
        return role.equals("hidden") || role.equals("implementation");
    }

    public boolean isSystemAttribute() {
        return name.startsWith("transport");
    }

    /**
     * @return A set of enum values or null if not available
     */
    private Set<String> translateEnums() {
        if (enumsStr == null || enumsStr.length() == 0)
            return null;

        enums = new HashSet<String>(10);

        for (final StringTokenizer tok = new StringTokenizer(enumsStr, ";"); tok.hasMoreTokens();) {
            final String enumValue = tok.nextToken();
            enums.add(enumValue);
        }

        return enums;
    }

    /**
     * Returns an Java's boolean primitive.
     * If <code>attrValue</code> is null then a 0 (zero) will be returned.
     *
     * @param attrName
     * @param attrValue
     * @param isMandatory
     * @param defaultValueInCaseOfNull
     * @return boolean primitive value
     */
    private boolean getBooleanValue(final String attrName,
                                    final String attrValue,
                                    boolean isMandatory,
                                    boolean defaultValueInCaseOfNull) {

        boolean value;
        if (attrValue == null) {
            if (isMandatory)
                throw new IllegalArgumentException("Missing value for a mandatroy attribute [" + attrName + "]");

            // Let's take the default value in case of null.
            value = defaultValueInCaseOfNull;
        } else {
            // If the <code>attrValue</code> is not true neither false then
            // calling Boolean.parBoolean(attrValue) will always going to be false.
            // If we can not match with TRUE or FALSE then the defaultValue is used.
            if (attrValue.equalsIgnoreCase(Boolean.TRUE.toString()))
                value = true;
            else if (attrValue.equalsIgnoreCase(Boolean.FALSE.toString()))
                value = false;
            else
                value = defaultValueInCaseOfNull;
        }

        // return the value as boolena primity.
        return value;
    }

    /**
     * Returns an Java's int primitive.
     *
     * @param attrName
     * @param attrValue
     * @param isMandatory
     * @param defaultValueInCaseOfNull
     * @return int primitive value
     */
    private int getIntValue(final String attrName,
                            final String attrValue,
                            boolean isMandatory,
                            int defaultValueInCaseOfNull) {

        int value;
        if (attrValue == null) {
            if (isMandatory)
                throw new IllegalArgumentException("Missing value for a mandatroy attribute [" + attrName + "]");

            // Let's take the default value in case of null.
            value = defaultValueInCaseOfNull;
        } else {
            value = Integer.parseInt(attrValue);
        }

        // return the value as boolena primity.
        return value;
    }

    /**
     * Returns an Java's integer primitive  as a result of extracting the attribute name from <code>Element</code>.
     * If no value is found then <code>defaultValue</code> is returned.
     *
     * @param attrName
     * @param attrValue
     * @param isMandatory
     * @param defaultValueInCaseOfNull
     */
    private String getStringValue(final String attrName,
                                  final String attrValue,
                                  boolean isMandatory,
                                  final String defaultValueInCaseOfNull) {

        if (attrValue == null) {
            if (isMandatory)
                throw new IllegalArgumentException("Missing value for a mandatroy attribute [" + attrName + "]");

            // Let's take the default value in case of null.
            return defaultValueInCaseOfNull;
        }

        return attrValue;
    }

    /**
     * Given a specific value, checks if this value can be successfully set in this attribute (type-wise)
     */
    public boolean checkValueType(final String valueToCheck) {
        if (valueToCheck == null || valueToCheck.length() == 0)
            return true;

        switch (getType()) {
            case INTEGER:
                try {
                    Integer.parseInt(valueToCheck);
                    return true;
                }
                catch (NumberFormatException e) {
                    return false;
                }

            case BOOLEAN:
                //noinspection RedundantIfStatement
                if (valueToCheck.equals(Boolean.TRUE.toString()) || valueToCheck.equals(Boolean.FALSE.toString()))
                    return true;

                return false;

            case FLOAT:
                try {
                    Float.parseFloat(valueToCheck);
                    return true;
                }
                catch (NumberFormatException e) {
                    return false;
                }

            case ENUM:
                return enums.contains(valueToCheck);

            case STRING:
            default:
                return true;
        }
    }

    public void toXML(final StringBuilder builder) {    /**
     * Marshal the attribute in an XML format. This is used when we need to deploy the configuration of the data source's
     * to the database.
     */
        builder.append("<property ");
        builder.append("name=\"").append(name).append("\" ");
        builder.append("value=\"").append(value).append("\" ");

        if (description != null && description.length() > 0)
            builder.append("description=\"").append(description).append("\" ");

        if (label != null && label.length() > 0)
            builder.append("label=\"").append(label).append("\" ");

        if (type != null)
            builder.append("type=\"").append(type).append("\" ");

        if (min != null && min.length() > 0)
            builder.append("min=\"").append(min).append("\" ");

        if (max != null && max.length() > 0)
            builder.append("max=\"").append(max).append("\" ");

        if (enums != null && enums.size() > 0) {
            builder.append("enum=\"");

            for (final String enumValue : enums)
                builder.append(enumValue).append(";");

            builder.deleteCharAt(builder.length() - 1);

            builder.append("\" ");
        }

        if (!isMandatory())
            builder.append("mandatory=\"").append("false").append("\" ");
        else
            builder.append("mandatory=\"").append("true").append("\" ");

        if (editable)
            builder.append("editable=\"").append("true").append("\" ");
        else
            builder.append("editable=\"").append("false").append("\" ");

        if (isEncrypt())
            builder.append("password=\"").append("true").append("\" ");
        else
            builder.append("password=\"").append("false").append("\" ");

        if (isHidden())
            builder.append("hidden=\"").append("true").append("\" ");
        else
            builder.append("hidden=\"").append("false").append("\" ");


        builder.append("/>");
    }

    public static enum Type {
        STRING("string"),
        INTEGER("integer"),
        FLOAT("float"),
        BOOLEAN("boolean"),
        ENUM("enum"),
        LIST("list");

        private String e;

        private Type(final String e) {
            this.e = e;
        }

        public static Type fromString(final String s) {
            if (s == null || s.length() == 0)
                return STRING;

            for (Type type : Type.values()) {
                if (type.e.equals(s))
                    return type;
            }

            throw new IllegalArgumentException("Invalid argument type '" + s + "'");
        }

        public String toString() {
            return e;
        }
    }
}
