/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.acquisition.aquisitionserverside.interfaces.impl;

import com.netapp.acquisition.aquisitionserverside.interfaces.data.AcquisitionUpdateType;
import com.netapp.oci.platform.common.jms.JmsUtils;
import org.slf4j.Logger;

import javax.jms.*;
import java.util.Map;
import java.util.Properties;

/**
 * @author Al Velella
 *         Date: 3/15/13
 *         Time: 1:01 PM
 */
public class AcquisitionJms {
    private static final Logger logger =org.slf4j.LoggerFactory.getLogger("com.netapp.acquisition.aquisitionserverside.interfaces.impl.AcquisitionServerSessionBean");

    private javax.jms.Connection jmsConnection;
    private Session session;
    private MessageProducer sender;

    public static final AcquisitionJms INSTANCE = new AcquisitionJms();

    private AcquisitionJms() {
        this.jmsConnection = null;
        this.session = null;
        this.sender = null;
    }

  public void send(ConnectionFactory connectionFactory, Topic updateTopic, AcquisitionUpdateType updateType, long updateTime, Long auId, Long dsId, String dsName)
  {
    send( connectionFactory, updateTopic, updateType, updateTime, auId, dsId, dsName, null );
  }
    public synchronized void send(ConnectionFactory connectionFactory, Topic updateTopic, AcquisitionUpdateType updateType, long updateTime, Long auId, Long dsId, String dsName, final Properties properties) {
        try {
            setupSession(connectionFactory, updateTopic);

            final MapMessage mapMessage = session.createMapMessage();
            mapMessage.setString("UPDATE_TYPE", updateType.name());
            mapMessage.setLong("UPDATE_TIME", updateTime);
            if (auId != null) {
                mapMessage.setLong("ACQUISITION_UNIT_ID", auId);
            }
            if (dsId != null) {
                mapMessage.setLong("DATASOURCE_ID", dsId);
            }
            if (dsName != null) {
                mapMessage.setString("DATASOURCE_NAME", dsName);
            }

            //add other properties for the call to the message
            if( properties != null )
            {
              for( Map.Entry<Object, Object> property : properties.entrySet() )
                mapMessage.setString( String.valueOf( property.getKey() ), String.valueOf(  property.getValue() ) );
            }

            sender.send(mapMessage);
        } catch (JMSException e) {
            logger.error("Caught error trying to send acquisition update event.", e);
            destroySession();
        }
    }

    private void setupSession(ConnectionFactory connectionFactory, Topic notificationTopic) throws JMSException {
        if (jmsConnection == null) {
            synchronized (this) {
                if (jmsConnection == null) {
                    jmsConnection = connectionFactory.createConnection();
                    session = jmsConnection.createSession(false, Session.AUTO_ACKNOWLEDGE);
                    sender = session.createProducer(notificationTopic);
                }
            }
        }
    }

    private void destroySession() {
        JmsUtils.close(jmsConnection);
        jmsConnection = null;
        session = null;
        sender = null;
    }
}
