/*
 * $Id: //depot/prod/acquisition/main/tools/config/eclipse/.settings/org.eclipse.jdt.ui.prefs#1 $
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.acquisition.aquisitionserverside.interfaces.impl;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.attribute.PosixFilePermission;
import java.util.HashSet;
import java.util.Set;

import com.netapp.acquisition.aquisitionserverside.interfaces.data.AcquisitionServerException;
import com.netapp.crypto.KeyStoreWrapper;
import com.netapp.crypto.SymmetricCipherFacade;

public final class AcquisitionServerUtil {

    private static SymmetricCipherFacade cipherFacade;

    private AcquisitionServerUtil() { }

    public static synchronized SymmetricCipherFacade getCipherFacade() throws AcquisitionServerException {
        if (cipherFacade != null) {
            return cipherFacade;
        }
        // Initialize OCIE encryption algorithms
        try {
            // called netapp.keystore because encryption maintained by NetApp's crypto-util.jar
            final String keystoreFileName = "netapp.keystore";
            final File onaroHome = new File(System.getProperty("onaro.home", "."));
            final File certsHome = new File(onaroHome, "jboss/server/onaro/cert/originator/");
            final File keystoreFile = new File(certsHome, keystoreFileName);
            final KeyStoreWrapper keyStoreWrapper = KeyStoreWrapper.createWrapper(keystoreFile);
            cipherFacade = new SymmetricCipherFacade(keyStoreWrapper);

            // The netapp.keystore will only be created on the first call to the
            // SymmetricCipherFacade.encrypt().  So we want to call this method
            // if the file does not exist.
            String systemOS = System.getProperty("os.name");
            if (!keystoreFile.exists() && !systemOS.contains("Window")) {
                cipherFacade.encrypt("test");

                // Now that the file has been created, we need to unset the
                // global permissions (600)
                Set<PosixFilePermission> perms = new HashSet<PosixFilePermission>();
                // Add owners permission
                perms.add(PosixFilePermission.OWNER_READ);
                perms.add(PosixFilePermission.OWNER_WRITE);

                Files.setPosixFilePermissions(Paths.get(keystoreFile.getPath()), perms);
            }

            return cipherFacade;
        } catch (Exception e) {
            final AcquisitionServerException ase =
                    new AcquisitionServerException("Failed to initialize SymmetricCipherFacade " + e.getMessage(), e);
            throw ase;
        }
    }

}
