/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp;

import java.sql.Connection;

import com.netapp.oci.platform.common.param.ParamException;
import com.netapp.oci.platform.common.param.PluginParam;

/**
 * @author Al Velella
 *         Date: Oct 10, 2012
 *         Time: 3:20:57 PM
 */
public final class Param {

    public enum Key {
        DB_PREFIX,
        CHANGES_LIFETIME,
        HISTORY_LIFETIME,
        SAMPLE_TABLES_LIFESPAN_DAYS,
        SAMPLE_RAW_TABLES_LIFESPAN_DAYS,
        SUMMARY_TABLES_LIFESPAN_DAYS,
        PERFORMANCE_UPDATE_CURRENT_SAMPLE,
        PRESERVE_OFFLINE_OBJECTS,
        RECORD_FULL_CLUSTER_CHANGE,
        SOFT_DELETE_PURGE_LIFETIME_DAYS,
        UPDATE_HISTORY_LIFESPAN_DAYS,
        APIS_COLLECTION_FLAG
    }

    public static final PluginParam INSTANCE = new PluginParam("netapp_model");

    private Param() {
    }

    public static <T> T get(Key key, Class<T> clazz) throws ParamException {
        final String keyName = key.name();
        return get(keyName, clazz);
    }

    private static <T> T get(String keyName, Class<T> clazz) throws ParamException {
        if (clazz == long.class) {
            //noinspection unchecked
            return (T) Long.valueOf(INSTANCE.getLong(keyName));
        } else if (clazz == int.class) {
            //noinspection unchecked
            return (T) Integer.valueOf(INSTANCE.getInt(keyName));
        } else if (clazz == boolean.class) {
            //noinspection unchecked
            return (T) Boolean.valueOf(INSTANCE.getBoolean(keyName));
        } else if (clazz == String.class) {
            //noinspection unchecked
            return (T) INSTANCE.getString(keyName);
        } else {
            throw new IllegalArgumentException("Unexpected param type " + clazz);
        }
    }

    /**
     * Get a value with a default backup.  NOTE!!  Don't use this method without adding an entry to the param
     * table via an upgrader.   The default here is really a fail safe in case somebody accidently deletes
     * the param table entry.
     *
     * @param key          param key
     * @param clazz        param type class
     * @param defaultValue default value of correct type
     * @param <T>          the type
     * @return the value
     */
    public static <T> T get(Key key, Class<T> clazz, T defaultValue) {
        T val;
        try {
            val = get(key, clazz);
        } catch (Exception e) {
            val = defaultValue;
        }
        return val;
    }

    /**
     * Get a value with a default backup.  NOTE!!  Don't use this method without adding an entry to the param
     * table via an upgrader.   The default here is really a fail safe in case somebody accidently deletes
     * the param table entry.
     *
     * @param keyName      param key
     * @param clazz        param type class
     * @param defaultValue default value of correct type
     * @param <T>          the type
     * @return the value
     */
    public static <T> T get(String keyName, Class<T> clazz, T defaultValue) {
        T val;
        try {
            val = get(keyName, clazz);
        } catch (Exception e) {
            val = defaultValue;
        }
        return val;
    }

    public static void set(Connection connection, String keyName, String valueToSet) throws ParamException {
        INSTANCE.set(connection, keyName, valueToSet);

    }

    public static void set(Connection connection, String keyName, boolean valueToSet) throws ParamException {
        INSTANCE.set(connection, keyName, valueToSet);

    }

    public static void set(Connection connection, String keyName, long valueToSet) throws ParamException {
        INSTANCE.set(connection, keyName, valueToSet);

    }

    public static void set(Connection connection, String keyName, int valueToSet) throws ParamException {
        INSTANCE.set(connection, keyName, valueToSet);

    }

}
