package com.netapp.oci.netapp.client.interfaces.impl;

import com.netapp.oci.netapp.Param;
import com.netapp.oci.netapp.client.interfaces.local.ParamLocal;
import com.netapp.oci.netapp.client.interfaces.remote.ParamRemote;
import com.netapp.oci.platform.common.ServerConstant;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.onaro.sanscreen.server.interfaces.data.ServerInterfaceException;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.annotation.security.RunAs;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.sql.DataSource;
import java.sql.Connection;

/**
 * @author Iti Behari
 *         Date: 7/8/13
 */
@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
@RunAs("admin")
public class ParamBean implements ParamRemote, ParamLocal {


    @Resource(name = "jdbc/OnaroDataSource",
            mappedName = "java:/onaro_pool",
            type = DataSource.class,
            authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    @Override
    @RolesAllowed({"guest", "acquisition", "admin"})
    public String get(String paramKey, String defaultValue) {
        return Param.get(paramKey, String.class, defaultValue);
    }

    @Override
    @RolesAllowed({"guest", "acquisition", "admin"})
    public boolean get(String paramKey, boolean defaultValue) {
        return Param.get(paramKey, boolean.class, defaultValue);

    }

    @Override
    @RolesAllowed({"guest", "acquisition", "admin"})
    public int get(String paramKey, int defaultValue) {
        return Param.get(paramKey, int.class, defaultValue);

    }

    @Override
    @RolesAllowed({"guest", "acquisition", "admin"})
    public long get(String paramKey, long defaultValue) {
        return Param.get(paramKey, long.class, defaultValue);

    }

    @Override
    @RolesAllowed({"acquisition", "admin"})
    public void set(String paramKey, String value) throws ServerInterfaceException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            Param.set(connection, paramKey, value);

        } catch (Exception e) {
            throw new ServerInterfaceException("Unable to set value " + value + " for param key " + paramKey, e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @RolesAllowed({"acquisition", "admin"})
    public void set(String paramKey, boolean value) throws ServerInterfaceException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            Param.set(connection, paramKey, value);

        } catch (Exception e) {
            throw new ServerInterfaceException("Unable to set value " + value + " for param key " + paramKey, e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @RolesAllowed({"acquisition", "admin"})
    public void set(String paramKey, int value) throws ServerInterfaceException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            Param.set(connection, paramKey, value);

        } catch (Exception e) {
            throw new ServerInterfaceException("Unable to set value " + value + " for param key " + paramKey, e);
        } finally {
            ConnectionHelper.close(connection);
        }

    }

    @Override
    @RolesAllowed({"acquisition", "admin"})
    public void set(String paramKey, long value) throws ServerInterfaceException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            Param.set(connection, paramKey, value);

        } catch (Exception e) {
            throw new ServerInterfaceException("Unable to set value " + value + " for param key " + paramKey, e);
        } finally {
            ConnectionHelper.close(connection);
        }

    }

}
