package com.netapp.oci.netapp.client.interfaces.impl;

import com.netapp.acquisition.aquisitionserverside.interfaces.local.AcquisitionServerSessionLocal;
import com.netapp.oci.netapp.client.interfaces.data.ZAPIManagerException;
import com.netapp.oci.netapp.client.interfaces.local.ZAPIManagerLocal;
import com.netapp.oci.netapp.client.interfaces.remote.ZAPIManagerRemote;
import com.netapp.oci.platform.common.ServerConstant;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import java.util.Set;


@Stateless
@RolesAllowed({"guest", "user", "admin", "scheduler", "acquisition"})
public class ZAPIManagerBean implements ZAPIManagerRemote, ZAPIManagerLocal {

    /**
     * used for a connection to the database. It is passed to the updates checker class to retrieve proxy settings
     */
    @Resource(name = "jdbc/OnaroDataSource",
              mappedName = "java:/onaro_pool",
              type = javax.sql.DataSource.class,
              authenticationType = Resource.AuthenticationType.CONTAINER)
    private javax.sql.DataSource dataSource;

    @EJB
    AcquisitionServerSessionLocal acquisitionServerSessionLocal;


    @Override
    public void assignDefaultPolicyToVolumes(final long clusterObjId, final Set<Long> targetVolumeObjIds, final long qosPolicyGroupId) throws ZAPIManagerException {
        final ZAPIManagerImpl zapiManagerImpl = ZAPIManagerImpl.INSTANCE;
        zapiManagerImpl.assignDefaultPolicyToVolumes(dataSource, acquisitionServerSessionLocal, clusterObjId, targetVolumeObjIds, qosPolicyGroupId);
    }

    @Override
    public void assignDefaultPolicyToVolumes(long clusterObjId, Set<Long> targetVolumeObjIds, long qosPolicyGroupId, String userName, String password) throws ZAPIManagerException {
        final ZAPIManagerImpl zapiManagerImpl = ZAPIManagerImpl.INSTANCE;
        zapiManagerImpl.assignDefaultPolicyToVolumes(dataSource, acquisitionServerSessionLocal, clusterObjId, targetVolumeObjIds, qosPolicyGroupId,
                userName, password);
    }
    
    @Override
    public void removePolicyFromVolumes(final long clusterObjId, final Set<Long> targetVolumeObjIds) throws ZAPIManagerException {
        final ZAPIManagerImpl zapiManagerImpl = ZAPIManagerImpl.INSTANCE;
        zapiManagerImpl.assignDefaultPolicyToVolumes(dataSource, acquisitionServerSessionLocal, clusterObjId, targetVolumeObjIds, Long.MIN_VALUE);
    }

    @Override
    public void removePolicyFromVolumes(long clusterObjId, Set<Long> targetVolumeObjIds, String userName, String password) throws ZAPIManagerException {
        final ZAPIManagerImpl zapiManagerImpl = ZAPIManagerImpl.INSTANCE;
        zapiManagerImpl.assignDefaultPolicyToVolumes(dataSource, acquisitionServerSessionLocal, clusterObjId, targetVolumeObjIds, Long.MIN_VALUE,
                userName, password);
    }
}
