package com.netapp.oci.netapp.client.interfaces.remote;

import com.netapp.oci.netapp.client.interfaces.data.ZAPIManagerException;

import javax.ejb.Remote;
import java.util.Set;

@Remote
public interface ZAPIManagerRemote {

    public String JNDI_NAME = "ejb:mega-app/server-core//ZAPIManagerBean!com.netapp.oci.netapp.client.interfaces.remote.ZAPIManagerRemote";

    /**
     * Assign the default QOS policy group to the volumes identified by the given object IDs.
     * All target volumes must exist in the same cluster.
     * @param clusterObjId the object ID of the cluster hosting the volumes to be configured
     * @param targetVolumeObjIds the set of Volume object IDs to assign to the default QOS policy group
     * @param qosPolicyGroupId the objid of the policy group to assign (probably objid of "_Performance_Monitor_volumes")
     * @throws com.netapp.oci.netapp.client.interfaces.data.ZAPIManagerException if configuring ANY volume fails
     */
    public void assignDefaultPolicyToVolumes(final long clusterObjId, final Set<Long> targetVolumeObjIds, final long qosPolicyGroupId) throws ZAPIManagerException;


    /**
     * Assign the default QOS policy group to the volumes identified by the given object IDs.
     * All target volumes must exist in the same cluster.
     * @param clusterObjId the object ID of the cluster hosting the volumes to be configured
     * @param targetVolumeObjIds the set of Volume object IDs to assign to the default QOS policy group
     * @param qosPolicyGroupId the objid of the policy group to assign (probably objid of "_Performance_Monitor_volumes")
     * @param userName ZAPI username used to assign policy group
     * @param password ZAPI password used to assign policy group
     * @throws com.netapp.oci.netapp.client.interfaces.data.ZAPIManagerException if configuring ANY volume fails
     */
    public void assignDefaultPolicyToVolumes(final long clusterObjId, final Set<Long> targetVolumeObjIds, final long qosPolicyGroupId,
                                             final String userName, final String password) throws ZAPIManagerException;
    
    /**
     * Given a set of volumes removes the policy groups assigned to them. The policy group name for those volumes after removal will be "none".
     * All target volumes must exist in the same cluster.
     * @param clusterObjId the object ID of the cluster hosting the volumes to be configured
     * @param targetVolumeObjIds the set of Volume object IDs to assign to the default QOS policy group 
     * @throws ZAPIManagerException if configuring ANY volume fails
     */
    public void removePolicyFromVolumes(final long clusterObjId, final Set<Long> targetVolumeObjIds) throws ZAPIManagerException;
    
    /**
     * Given a set of volumes removes the policy groups assigned to them. The policy group name for those volumes after removal will be "none".
     * All target volumes must exist in the same cluster.
     * @param clusterObjId
     * @param targetVolumeObjIds
     * @param userName ZAPI username used to remove policy group
     * @param password ZAPI password used to remove policy group
     * @throws ZAPIManagerException
     */
    public void removePolicyFromVolumes(final long clusterObjId, final Set<Long> targetVolumeObjIds, final String userName, final String password) throws ZAPIManagerException;
}
