/*
 * $Id: //depot/prod/acquisition/main/tools/config/eclipse/.settings/org.eclipse.jdt.ui.prefs#1 $
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.oci.netapp.ems;

/**
 * Utility class to compare two ONTAP version strings. Currently we only take version generation and major.
 */
public final class NetappEmsUtil {

    /**
     * Compare two Ontap version string. Version format is "generation.major".
     *
     * @param version1
     * @param version2
     * @return 1  - version1 is greater than version2
     *         0  - version1 is same as version2
     *         -1 - version1 is less than version2
     */
    public static int compareOntapVersion(String version1, String version2) {
        if (version1 == null || version2 == null) {
            throw new IllegalArgumentException("Each of the Ontap comparable versions cannot be null, [" + version1 + "," + version2 + "]");
        }

        String[] vs1 =  version1.split("\\.");
        if (vs1.length != 2) {
            throw new IllegalArgumentException("Ontap version format generation.major. " + version1 + " is invalid!");
        }

        String[] vs2 = version2.split("\\.");
        if (vs2.length != 2) {
            throw new IllegalArgumentException("Ontap version format generation.major. " + version2 + " is invalid!");
        }

        int generation1 = Integer.valueOf(vs1[0]), major1 = Integer.valueOf(vs1[1]);
        int generation2 = Integer.valueOf(vs2[0]), major2 = Integer.valueOf(vs2[1]);

        return Integer.compare(generation1, generation2) != 0 ?  Integer.compare(generation1, generation2) : Integer.compare(major1, major2);
    }

}
