/*
 * $Id: //depot/prod/acquisition/main/tools/config/eclipse/.settings/org.eclipse.jdt.ui.prefs#1 $
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.oci.netapp.ems.shared.interfaces.data;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

@Entity
@Table(name = "ems_event")
public class NetappEmsEvent implements Serializable {
    @Transient
    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue
    private Long id;

    @Column(name = "messageName", nullable = false)
    private String messageName;

    @Column(name = "severity", nullable = false)
    private String severity;

    @Column(name = "time", nullable = false)
    private Long time;

    @Column(name = "event", nullable = false)
    private String event;

    @Column(name = "emsSeverity", nullable = false)
    private String emsSeverity;

    @Column(name = "source", nullable = false)
    private String source;

    @Column(name = "version", nullable = false)
    private int version;

    @Column(name = "nodeUuid", nullable = true)
    private String nodeUuid;

    @Column(name = "clusterUuid", nullable = false)
    private String clusterUuid;

    @OneToMany(cascade = CascadeType.ALL, mappedBy = "event")
    private Set<NetappEmsEventParameter> parameters = new HashSet<NetappEmsEventParameter>();

    /**
     * This is for the EntityManager.
     */
    public NetappEmsEvent() { }

    public static class Builder {
        private String messageName;
        private String severity;
        private Long time;
        private String event;
        private String emsSeverity;
        private String source;
        private int version;
        private String nodeUuid;
        private String clusterUuid;

        public Builder withMessageName(String messageName) {
            this.messageName = messageName;
            return this;
        }

        public Builder withSeverity(String severity) {
            this.severity = severity;
            return this;
        }

        public Builder withTime(long time) {
            this.time = time;
            return this;
        }

        public Builder withEvent(String event) {
            this.event = event;
            return this;
        }

        public Builder withEmsSeverity(String emsSeverity) {
            this.emsSeverity = emsSeverity;
            return this;
        }

        public Builder withSource(String source) {
            this.source = source;
            return this;
        }

        public Builder withVersion(int version) {
            this.version = version;
            return this;
        }

        public Builder withNodeUuid(String nodeUuid) {
            this.nodeUuid = nodeUuid;
            return this;
        }

        public Builder withClusterUuid(String clusterUuid) {
            this.clusterUuid = clusterUuid;
            return this;
        }

        public NetappEmsEvent build() {
            return new NetappEmsEvent(this);
        }
    }

    private NetappEmsEvent(Builder builder) {
        this.setId(null);   /* auto-generated by database*/
        this.setMessageName(builder.messageName);
        this.setSeverity(builder.severity);
        this.setTime(builder.time);
        this.setEvent(builder.event);
        this.setEmsSeverity(builder.emsSeverity);
        this.setSource(builder.source);
        this.setVersion(builder.version);
        this.setNodeUuid(builder.nodeUuid);
        this.setClusterUuid(builder.clusterUuid);
    }


    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getMessageName() {
        return messageName;
    }


    public void setMessageName(String messageName) {
        this.messageName = messageName;
    }

    public String getSeverity() {
        return severity;
    }


    public void setSeverity(String severity) {
        this.severity = severity;
    }


    public Long getTime() {
        return time;
    }

    public void setTime(Long time) {
        this.time = time;
    }

    public String getEvent() {
        return event;
    }


    public void setEvent(String event) {
        this.event = event;
    }


    public String getEmsSeverity() {
        return emsSeverity;
    }


    public void setEmsSeverity(String emsSeverity) {
        this.emsSeverity = emsSeverity;
    }


    public String getSource() {
        return source;
    }


    public void setSource(String source) {
        this.source = source;
    }


    public int getVersion() {
        return version;
    }


    public void setVersion(int version) {
        this.version = version;
    }


    public String getNodeUuid() {
        return nodeUuid;
    }


    public void setNodeUuid(String nodeUuid) {
        this.nodeUuid = nodeUuid;
    }


    public String getClusterUuid() {
        return clusterUuid;
    }


    public void setClusterUuid(String clusterUuid) {
        this.clusterUuid = clusterUuid;
    }

    public Set<NetappEmsEventParameter> getParameters() {
        return parameters;
    }

    public void addEventParameter(NetappEmsEventParameter parameter) {
        parameters.add(parameter);
        parameter.setEvent(this);
    }

    @Override
    public String toString() {
        return "NetAppEmsEvent(id="
                + this.getId()
                + ", clusterUuid="
                + this.getClusterUuid()
                + ", nodeUuid="
                + this.getNodeUuid()
                + ", messageName="
                + this.getMessageName()
                + ", time="
                + this.getTime()
                + ")";
    }

}
