/*
 * $Id: //depot/prod/acquisition/main/tools/config/eclipse/.settings/org.eclipse.jdt.ui.prefs#1 $
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.oci.netapp.ems.shared.interfaces.impl;

import java.util.List;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.netapp.oci.netapp.ems.shared.interfaces.data.NetAppEmsEventException;
import com.netapp.oci.netapp.ems.shared.interfaces.data.NetappEmsEvent;
import com.netapp.oci.netapp.ems.shared.interfaces.local.NetAppEmsEventPersistence;

@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
@RolesAllowed({"guest", "user", "admin", "scheduler", "acquisition"})
public class NetAppEmsEventPersistenceBean implements NetAppEmsEventPersistence {

    @PersistenceContext(unitName = "onaro_model_persistence")
    private EntityManager entityManager;

    private static final Logger logger = LoggerFactory.getLogger("com.netapp.oci.netapp.ems.shared.interfaces.impl.NetAppEmsEventPersistenceBean");

    private static final String SQL_SELECT_EMS_EVENTS_BY_CLUSTER_UUID =
            "SELECT e FROM NetappEmsEvent e WHERE e.clusterUuid = :clusterUuid";

    @Override
    public void insertEvent(NetappEmsEvent emsEntity) throws NetAppEmsEventException {
        persistEmsEvent(emsEntity);
    }

    @Override
    public void insertEvents(List<NetappEmsEvent> emsEntities) throws NetAppEmsEventException {
        for (NetappEmsEvent emsEntity : emsEntities) {
            persistEmsEvent(emsEntity);
        }
    }

    private void persistEmsEvent(NetappEmsEvent event) throws NetAppEmsEventException {
        try {
            entityManager.persist(event);
            logger.info("Persisted ems event in netapp_model and retrieved id is " + event.getId());
        } catch (Exception e) {
            throw new NetAppEmsEventException("Error in persisting ems event", e);
        }
    }

    @Override
    public long getLastEventTimestamp(String uuid) {
        long lastEventtimestamp = 0;
        final String sql = "SELECT MAX(time) from NetappEmsEvent where clusterUuid=\'" + uuid + "\'";
        try {
            lastEventtimestamp = (Long)entityManager.createQuery(sql).getSingleResult();
        } catch (Exception e) {
            logger.debug("event timestamp retrieval exception - " + e);
            logger.warn("Could not retrieve last ems timestamp for cluster uuid " + uuid);
        }
        return lastEventtimestamp;
    }

    @Override
    @SuppressWarnings("unchecked")
    public void deleteEmsEventsByCluserUuid(String clusterUuid) throws NetAppEmsEventException {
        try {
            // delete emsEvent and its parameters
            List<NetappEmsEvent> emsEvents = entityManager.createQuery(SQL_SELECT_EMS_EVENTS_BY_CLUSTER_UUID)
                                                          .setParameter("clusterUuid", clusterUuid)
                                                          .getResultList();

            emsEvents.forEach(ems->entityManager.remove(ems));

            entityManager.flush();
            logger.debug("All ems events associated with cluster {} have been purged.", clusterUuid);
        } catch (Exception e) {
            throw new NetAppEmsEventException("Error in purging ems event", e);
        }
    }

}
