/*
 * $Id: //depot/prod/acquisition/main/tools/config/eclipse/.settings/org.eclipse.jdt.ui.prefs#1 $
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.oci.netapp.ems.shared.interfaces.impl;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.SQLException;

import javax.annotation.security.RunAs;
import javax.sql.DataSource;

import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.netapp.oci.platform.common.table.ConnectionHelper;

@RunAs("admin")
@PersistJobDataAfterExecution
@DisallowConcurrentExecution
public class NetappEmsEventPurgeJob implements Job {

    public static final String CRON_TRIGGER = "0 30 4 * * ?";
    public static final String JOB_NAME = "NetappEmsEventPurgeJob";

    private static final Logger logger = LoggerFactory.getLogger(NetappEmsEventPurgeJob.class);

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        Connection connection = null;
        CallableStatement callableStatement = null;

        try {
            final long startTime = System.currentTimeMillis();

            DataSource dataSource = (DataSource) jobExecutionContext.getJobDetail().getJobDataMap().get("dataSourceModelOnaroPool");
            connection = ConnectionHelper.newConnection(dataSource);

            logger.debug("Started EMS event purge");
            callableStatement = connection.prepareCall("{call netapp_performance.purge_ems_event()}");
            callableStatement.execute();
            logger.info("Completed Purge of EMS event in " + (System.currentTimeMillis() - startTime) + " msec.");
        } catch (SQLException e) {
            throw new JobExecutionException(e);
        } finally {
            ConnectionHelper.close(callableStatement);
            ConnectionHelper.close(connection);
        }
    }

}

