/*
 * $Id: //depot/prod/acquisition/main/tools/config/eclipse/.settings/org.eclipse.jdt.ui.prefs#1 $
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.oci.netapp.ems.shared.interfaces.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Singleton;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.interceptor.Interceptors;

import com.onaro.sanscreen.acquisition.sessions.AcquisitionFacadeSessionLocal;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceNew;

import com.netapp.oci.netapp.ems.shared.interfaces.local.EmsMsgListener;
import com.netapp.oci.netapp.ems.shared.interfaces.local.NetappEmsMsgListenerRegistry;
import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.platform.common.ServerInterfaceInterceptor;
import com.netapp.oci.platform.common.table.TableException;

@Singleton
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@RolesAllowed({"guest", "user", "admin", "scheduler", "acquisition"})
@Interceptors(ServerInterfaceInterceptor.class)
public class NetappEmsMsgListenerRegistryBean implements NetappEmsMsgListenerRegistry {

    private final ConcurrentHashMap<String, EmsMsgListener> emsMsgListeners = new ConcurrentHashMap<>(2, 0.75f, 1);

    private static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(NetappEmsMsgListenerRegistryBean.class);

    @EJB(name = AcquisitionFacadeSessionLocal.JNDI_NAME)
    private AcquisitionFacadeSessionLocal acquisitionLocal;

    @Override
    public synchronized void register(String listenerName, EmsMsgListener listener, boolean withCertRegenerated) throws NetappInterfaceException {
        if (listenerName == null || listener == null) {
            throw new NetappInterfaceException("listenerName and listener are required to register an EMS listener!");
        }

        EmsMsgListener existListener = emsMsgListeners.get(listenerName);
        if (existListener == null) {
            emsMsgListeners.put(listenerName, listener);
        } else {
            throw new NetappInterfaceException("Listener [" + listenerName + "] is an existing EMS listener! Please choose a different name.");
        }

        // check if there is any EMS REST destination registration to be done
        try {
            List<DataSourceNew> failedDss;
            failedDss = acquisitionLocal.checkEmsRegistrationForUpgrade();
            if (!failedDss.isEmpty()) {
                logger.error("Unable to register with clusters {} for EMS REST push.", failedDss);
            }
        } catch (TableException e) {
            throw new NetappInterfaceException("Unable to check EMS configurations to register REST destinations", e);
        }

        // handle EMS registration due to the server certificate regeneration
        if (withCertRegenerated) {
            acquisitionLocal.handleEmsRegistryForCertRegeneration();
        }
    }

    @Override
    public void pullEvents() throws NetappInterfaceException {
        // Pull any missed EMS events by making zapi call
        List<EmsMsgListener> listeners = new ArrayList<>(emsMsgListeners.values());
        acquisitionLocal.getZapiEmsEvents(listeners);
    }



    @Override
    public synchronized void unregister(String listenerName) throws NetappInterfaceException {
        if (listenerName == null) {
            throw new NetappInterfaceException("listenerName is required to unregister an EMS listener!");
        }
        emsMsgListeners.remove(listenerName);
    }


    @Override
    public Map<String, EmsMsgListener> getEmsMsgListeners() {
        return emsMsgListeners;
    }

}
