package com.netapp.oci.netapp.foundation.qosvolumeworkloaduuidconvertion.interfaces.impl;

import com.netapp.oci.netapp.foundation.qosvolumeworkloaduuidconvertion.interfaces.remote.UuidConversionRemote;
import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.originator.handler.ClusterHandler;
import com.netapp.oci.netapp.originator.handler.QosWorkloadHandler;
import com.netapp.oci.netapp.startup.NetAppPlugin;
import com.netapp.oci.platform.common.ServerInterfaceInterceptor;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.originator.OriginatorStateCache;
import com.netapp.oci.platform.originator.OriginatorStateCacheFactory;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.key.KeyCache;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.SimpleUuidKey;
import com.netapp.oci.platform.originator.key.UuidKey;
import com.onaro.sanscreen.util.Constant;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.interceptor.Interceptors;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

/**
 * The class is to convert static uuids to dynamic uuids in QosWorkload table for FS cluster
 */

@Stateless
        @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
        @RolesAllowed({"guest", "user", "admin", "scheduler", "acquisition"})
@Interceptors(ServerInterfaceInterceptor.class)
public class UuidConversionBean implements UuidConversionRemote {
    private static final Logger logger = LoggerFactory.getLogger(UuidConversionBean.class);

    @Resource(name = "jdbc/OnaroDataSource", mappedName = "java:/onaro_pool", type = javax.sql.DataSource.class, authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    @Override
    public void convert(String clusterKey, Map<String, String> mapToConvert) throws NetappInterfaceException {
        KeyCache keyCache = KeyCache.get();
        long originId =  keyCache.findOriginatorByClusterKey(new SimpleUuidKey(clusterKey));
        Long clusterId = keyCache.findId(originId, ClusterHandler.TAG, clusterKey);
        Connection connection = ConnectionHelper.newConnection(dataSource);

        if (clusterId == null) {
            logger.warn("Failed to find cluster id from clusterKey {}" + clusterKey );
        }
        else {
            try {
                String updateSQL =  "UPDATE netapp_model.qos_workload SET uuid = ?, resourceKey = CONCAT(SUBSTRING_INDEX(resourceKey, '=', 2), '=', ?) WHERE clusterId = ? and name = ? and holderType='VOLUME'";
                PreparedStatementWrapper preparedStatement = PreparedStatementWrapper.createForUpdate(connection, updateSQL);
                 // update database
                for(Map.Entry<String, String> entry : mapToConvert.entrySet())   {
                    preparedStatement.setString(1, clusterKey + "." + entry.getValue());
                    preparedStatement.setString(2, entry.getValue());
                    preparedStatement.setLong(3, clusterId);
                    preparedStatement.setString(4, entry.getKey());
                    preparedStatement.addBatchEx();
                }

                preparedStatement.executeBatchEx();

                // update KeyCache & stateCache
                OriginatorStateCache stateCache = OriginatorStateCacheFactory.getOriginatorCache(connection, originId, (new NetAppPlugin().getRegistry()));
                Map<UuidKey, UuidKey> uuidKeyMap = new HashMap<UuidKey, UuidKey>();
                for(Map.Entry<String, String> entry : mapToConvert.entrySet()) {
                    String newUuid = clusterKey + "." + entry.getValue();
                    String oldUuid = clusterKey + "." + entry.getKey();
                    UuidKey newKey = new SimpleUuidKey(newUuid);
                    UuidKey oldKey = new SimpleUuidKey(oldUuid);
                    uuidKeyMap.put(oldKey, newKey);
                }
                keyCache.modifyUuidKeys(originId, QosWorkloadHandler.TAG, uuidKeyMap);
                stateCache.modifyUuidKeys(QosWorkloadHandler.TAG, uuidKeyMap);
                
                // reset uuid conversion flag
                updateSQL = "UPDATE acquisition.ds_attrs a SET value = 'false' WHERE a.name = 'netappfoundation.upgrader.1_2_0_0' AND a.ds_id = (SELECT datasourceId FROM netapp_model.cluster b "
                        + "WHERE b.uuid = '" + clusterKey + "')";
                preparedStatement = PreparedStatementWrapper.createForUpdate(connection, updateSQL);
                preparedStatement.executeUpdateEx();
                        
            }
            catch (Exception e) {
                throw new NetappInterfaceException("Failed to update qos_workload table, clusterKey:" + clusterKey, e);
            }
            finally {
                ConnectionHelper.close(connection);
            } 
        }
     }
}
