/*
 * $Id: //depot/prod/acquisition/main/tools/config/eclipse/.settings/org.eclipse.jdt.ui.prefs#1 $
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.oci.netapp.interfaces.data;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.Transient;

import com.netapp.oci.netapp.ems.shared.interfaces.data.NetappEmsEventParameter;

public class NetappEmsEventDto implements Serializable {
    @Transient
    private static final long serialVersionUID = 1L;

    private Long id;

    private String messageName;

    private String severity;

    private Long time;

    private String event;

    private String emsSeverity;

    private String source;

    private int version;

    private String nodeUuid;

    private String clusterUuid;

    private Set<NetappEmsEventParameter> parameters = new HashSet<NetappEmsEventParameter>();

    public static class Builder {
        private Long id;
        private String messageName;
        private String severity;
        private Long time;
        private String event;
        private String emsSeverity;
        private String source;
        private int version;
        private String nodeUuid;
        private String clusterUuid;
        private Set<NetappEmsEventParameter> parameters;

        public Builder withId(long id) {
            this.id = id;
            return this;
        }

        public Builder withMessageName(String messageName) {
            this.messageName = messageName;
            return this;
        }

        public Builder withSeverity(String severity) {
            this.severity = severity;
            return this;
        }

        public Builder withTime(long time) {
            this.time = time;
            return this;
        }

        public Builder withEvent(String event) {
            this.event = event;
            return this;
        }

        public Builder withEmsSeverity(String emsSeverity) {
            this.emsSeverity = emsSeverity;
            return this;
        }

        public Builder withSource(String source) {
            this.source = source;
            return this;
        }

        public Builder withVersion(int version) {
            this.version = version;
            return this;
        }

        public Builder withNodeUuid(String nodeUuid) {
            this.nodeUuid = nodeUuid;
            return this;
        }

        public Builder withClusterUuid(String clusterUuid) {
            this.clusterUuid = clusterUuid;
            return this;
        }

        public Builder withParameters(Set<NetappEmsEventParameter> parameters) {
            this.parameters = parameters;
            return this;
        }

        public NetappEmsEventDto build() {
            return new NetappEmsEventDto(this);
        }
    }

    private NetappEmsEventDto(Builder builder) {
        this.setId(builder.id);   /* auto-generated by database*/
        this.setMessageName(builder.messageName);
        this.setSeverity(builder.severity);
        this.setTime(builder.time);
        this.setEvent(builder.event);
        this.setEmsSeverity(builder.emsSeverity);
        this.setSource(builder.source);
        this.setVersion(builder.version);
        this.setNodeUuid(builder.nodeUuid);
        this.setClusterUuid(builder.clusterUuid);
        this.setParameters(builder.parameters);
    }


    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getMessageName() {
        return messageName;
    }


    public void setMessageName(String messageName) {
        this.messageName = messageName;
    }

    public String getSeverity() {
        return severity;
    }


    public void setSeverity(String severity) {
        this.severity = severity;
    }


    public Long getTime() {
        return time;
    }

    public void setTime(Long time) {
        this.time = time;
    }

    public String getEvent() {
        return event;
    }


    public void setEvent(String event) {
        this.event = event;
    }


    public String getEmsSeverity() {
        return emsSeverity;
    }


    public void setEmsSeverity(String emsSeverity) {
        this.emsSeverity = emsSeverity;
    }


    public String getSource() {
        return source;
    }


    public void setSource(String source) {
        this.source = source;
    }


    public int getVersion() {
        return version;
    }


    public void setVersion(int version) {
        this.version = version;
    }


    public String getNodeUuid() {
        return nodeUuid;
    }


    public void setNodeUuid(String nodeUuid) {
        this.nodeUuid = nodeUuid;
    }


    public String getClusterUuid() {
        return clusterUuid;
    }


    public void setClusterUuid(String clusterUuid) {
        this.clusterUuid = clusterUuid;
    }

    public Set<NetappEmsEventParameter> getParameters() {
        return this.parameters;
    }

    public void setParameters(Set<NetappEmsEventParameter> parameters) {
        this.parameters = parameters;
    }

    @Override
    public String toString() {
        return "NetAppEmsEvent(id="
                + this.getId()
                + ", clusterUuid="
                + this.getClusterUuid()
                + ", nodeUuid="
                + this.getNodeUuid()
                + ", messageName="
                + this.getMessageName()
                + ", time="
                + this.getTime()
                + ")";
    }

}
