/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator;

import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.onaro.sanscreen.server.interfaces.data.ServerInterfaceException;

import java.io.Serializable;
import java.sql.Connection;

/**
 * Helper class for <code>NetAppPlugin</code> to perform actions post originator update.
 * The runner performs the following action:-
 * 1) updates 'lastupdateTime' when any object on cluster is updated.
 *
 * @author Iti Behari
 *         Date: 12/21/12
 */
public class PostOriginatorUpdate implements Serializable {

    private static final long serialVersionUID = -1L;

    public void postProcess(Connection connection, long originId, long datasourceId, long time) throws ServerInterfaceException {
        String sql =  "UPDATE netapp_model.cluster SET lastUpdateTime = ?, datasourceId = ? WHERE originId = ? ";
        try {
            PreparedStatementWrapper.createForUpdate(connection, sql).setNext(time).setNext(datasourceId).setNext(originId).executeUpdateEx();
        } catch (TableException e) {
            throw new ServerInterfaceException("Error while updating cluster's updateTime", e);
        }
    }
}
