/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.netapp.oci.netapp.originator.handler.*;
import com.netapp.oci.platform.originator.PlatformRegistry;
import com.netapp.oci.platform.originator.handler.XmlChildTagDetails;
import com.netapp.oci.platform.originator.key.UuidKeyFactory;

/*
 * @author Al Velella
 *         Date: Sep 26, 2011
 *         Time: 10:40:34 AM
 */
public class Registry extends PlatformRegistry {
    public static final Registry INSTANCE = new Registry();

    /**
     * For dependent objects where the offline object and the parent object is not the same
     * (e.g: a volume has a vserver parent but can be offlined by an aggregate)
     */
    public final Map<String, List<String>> tagToChangeDetailAttributes;

    private Registry() {
        super();

        // Handlers must be registered in table-hierarchy order. This order will be respected when updated/deleting
        // data else foreign key violations will occur.
        addXmlHandler(
                new ClusterHandler(),
                "cluster",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new NodeHandler(),
                "cluster_node",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new FlashDeviceHandler(),
                "flash_device",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new StorageShelfHandler(),
                "storage_shelf",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new StorageShelfCableHandler(),
                "storage_shelf_cable",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new ServiceProcessorHandler(),
                "service_processor",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new StorageSwitchHandler(),
                "storage_switch",
                UuidKeyFactory.COLON_KEY);
        addXmlHandler(
                new LicenseHandler(),
                "license",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new LicenseV2EntitlementRiskHandler(),
                "license_v2_entitlement_risk",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new NodeLicenseUsageHandler(),
                "node_license_usage",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new JobScheduleHandler(),
                "ontap_job_schedule",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new SnapshotPolicyHandler(),
                "snapshot_policy",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new SnapshotPolicyScheduleHandler(),
                "snapshot_policy_schedule",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new QosPolicyGroupHandler(),
                "qos_policy_group",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new NetworkIpSpaceHandler(),
                "network_ip_space",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new VserverHandler(),
                "vserver",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new VserverNameMappingHandler(),
                "vserver_name_mapping",
                UuidKeyFactory.DOT_KEY);
        addSelfRelationshipXmlHandler(
                new QosPolicyGroupVserverRelationshipHandler(),
                QosPolicyGroupHandler.TAG);
        addXmlHandler(
                new ProcessorHandler(),
                "processor",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new AggregateHandler(),
                "aggregate",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
            new ResourceAggregateHandler(),
            "resource_aggregate",
            UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new MetroClusterOperationHandler(),
                "metrocluster_opreation",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new PlexHandler(),
                "plex",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new RaidGroupHandler(),
                "raid_group",
                UuidKeyFactory.DASH_KEY);
        addXmlHandler(
                new ExportPolicyHandler(),
                "export_policy",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new ExportRuleHandler(),
                "export_rule",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new SisPolicyHandler(),
                "sis_policy",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
               new NetworkFirewallPolicyHandler(),
               "network_firewall_policy",
               UuidKeyFactory.DOT_KEY);
        addXmlHandler(
               new NetworkFirewallPolicyServiceHandler(),
               "network_firewall_policy_service",
               UuidKeyFactory.DOT_KEY);
        addXmlHandler(
               new LDAPClientHandler(),
               "ldap_client",
               UuidKeyFactory.DOT_KEY);
        addXmlHandler(
               new NSSwitchHandler(),
               "nameservice_nsswitch",
               UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new StorageServiceHandler(),
                "infinitevol_storage_service",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new QuotaPolicyHandler(),
                "quota_policy",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new VolumeHandler(),
                "volume",
                UuidKeyFactory.DASH_KEY);
        addSelfRelationshipXmlHandler(
                new VolumeSelfRelationshipHandler(),
                VolumeHandler.TAG);
        addXmlHandler(
                new VolumeFlexCacheHandler(),
                "flex_cache",
                UuidKeyFactory.SIMPLE_KEY);
        addSelfRelationshipXmlHandler(
                new VolumeFlexgroupConstituentRelationshipHandler(),
                VolumeHandler.TAG);
        addSelfRelationshipXmlHandler(
                new StorageServiceVolumeRelationshipHandler(),
                StorageServiceHandler.TAG);
        addXmlHandler(
                new SnapshotHandler(),
                "snapshot",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new SnapMirrorPolicyHandler(),
                "snap_mirror_policy",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new SnapMirrorPolicyRuleHandler(),
                "snap_mirror_policy_rule",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new SnapMirrorHandler(),
                "snap_mirror",
                UuidKeyFactory.SIMPLE_KEY);
        addSelfRelationshipXmlHandler(
                new SnapMirrorSelfRelationshipHandler(),
                SnapMirrorHandler.TAG);
        addXmlHandler(
                new QtreeHandler(),
                "qtree",
                UuidKeyFactory.DASH_KEY);
        addXmlHandler(
                new CifsShareHandler(),
                "cifs_share",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new CifsShareACLHandler(),
                "cifs_share_acl",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new LunHandler(),
                "lun",
                UuidKeyFactory.DASH_KEY);
        addXmlHandler(
                new NamespaceHandler(),
                "namespace",
                UuidKeyFactory.DASH_KEY);
        addXmlHandler(
                new ObjectstoreConfigHandler(),
                "objectstore_config",
                UuidKeyFactory.SIMPLE_KEY
        );
        addXmlHandler(
                new StoragePoolHandler(),
                "storage_pool",
                UuidKeyFactory.COLON_KEY
        );
        addXmlHandler(
                new StoragePoolAvailableSizeHandler(),
		"storage_pool_available",
                UuidKeyFactory.COLON_KEY
        );
        addXmlHandler(
                new StoragePoolAggregateHandler(),
                "storage_pool_aggregate_relationship",
                UuidKeyFactory.COLON_KEY
        );
        addXmlHandler(
                new DiskHandler(),
                "disk",
                UuidKeyFactory.COLON_KEY);
        addXmlHandler(
                new DiskAggregateRelationshipHandler(),
                "disk_aggregate_relationship",
                UuidKeyFactory.COLON_KEY);
        addXmlHandler(
                new LunImportHandler(),
                "lun_import",
                UuidKeyFactory.DASH_KEY);
        addXmlHandler(
                new NetworkPortBroadcastDomainHandler(),
                "network_port_broadcast_domain",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new NetworkSubnetHandler(),
                "network_subnet",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new NetworkPortHandler(),
                "network_port",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new NetworkVlanHandler(),
                "network_vlan",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new FcpPortHandler(),
                "fcp_port",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new RoutingGroupHandler(),
                "routing_group",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new RoutingGroupDestinationHandler(),
                "routing_group_destination",
                UuidKeyFactory.DOT_KEY);

        //Network failovergroups must before LIF's, LIF's have reference to failovergroups
        addXmlHandler(
              new NetworkFailoverGroupHandler(),
              "network_failover_group",
              UuidKeyFactory.DOT_KEY);

        // add LifHandler for NetworkLif
        addHierarchyXmlHandler(
                new LifHandler(LifHandler.SubTag.NETWORK_LIF),
                "network_lif",
                UuidKeyFactory.SIMPLE_KEY);
        // add LifHandler for FcpLif
        addHierarchyXmlHandler(
                new LifHandler(LifHandler.SubTag.FCP_LIF),
                "fcp_lif",
                UuidKeyFactory.SIMPLE_KEY);
        // add LifHandler for NvmfFcLif
        addHierarchyXmlHandler(
                new LifHandler(LifHandler.SubTag.NVMF_FC_LIF),
                "nvmf_fc_lif",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new HaPairHandler(),
                null,
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new IgroupHandler(),
                "igroup",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new IgroupInitiatorHandler(),
                "igroup_initiator",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new LunMapHandler(),
                "lun_map",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new LUNMapReportingNodeHandler(),
                "lun_map_reporting_node",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new StorageAdapterHandler(),
                "storage_adapter",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new DiskPathHandler(),
                "disk_path",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new IscsiPortalGroupHandler(),
                "iscsi_portal_group",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new IscsiPortalGroupLifHandler(),
                "iscsi_portal_group_lif",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new IscsiSessionHandler(),
                "iscsi_session",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new IscsiSessionConnectionHandler(),
                "iscsi_session_connection",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new IscsiSecurityEntryHandler(),
                "iscsi_security_entry",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new PortsetHandler(),
                "portset",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new PortsetIgroupRefHandler(),
                null,
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new PortsetPortHandler(),
                "portset_port",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new TreeQuotaHandler(),
                "tree_quota",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new UserQuotaHandler(),
                "user_quota",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new QuotaPolicyRuleHandler(),
                "quota_policy_rule",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new QuotaUserHandler(),
                "quota_user",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new VolumeMoveJobHandler(),
                "volume_move",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new NisDomainHandler(),
                "nis_domain",
                UuidKeyFactory.DOT_KEY);
        addSelfRelationshipXmlHandler(
                new NetworkPortRelationshipHandler(),
                NetworkPortHandler.TAG);
        addXmlHandler(
                new VserverAggregateRelationshipHandler(),
                null,
                UuidKeyFactory.DOT_KEY);
        addSelfRelationshipXmlHandler(
                new SnapshotPolicyVserverRelationshipHandler(),
                SnapshotPolicyHandler.TAG_NAME);
        addXmlHandler(
                new QosWorkloadHandler(),
                "qos_workload",
                UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new QosWorkloadNodeRelationshipHandler(),
                "qos_workload_node_relationship",
                UuidKeyFactory.DOT_KEY);
        addXmlHandler(
                new QosServiceCenterHandler(),
                "qos_resource",
                UuidKeyFactory.SIMPLE_KEY);
        addSelfRelationshipXmlHandler(
                new QosServiceCenterRelationshipHandler(),
                QosServiceCenterHandler.TAG);
        addXmlHandler(
        		new QosWorkloadDetailHandler(),
        		"qos_workload_detail",
        		UuidKeyFactory.SIMPLE_KEY);
        addXmlHandler(
                new ApplicationRecordHandler(),
                "application_record",
                UuidKeyFactory.SIMPLE_KEY);

        addXmlHandler(
                new SystemHealthAlertHandler(),
                "system_health_alert",
                UuidKeyFactory.DOT_KEY);

      addXmlHandler(
              new NetworkFailoverGroupNetworkPortRelationshipHandler(),
              "network_failover_group_network_port_relationship",
              UuidKeyFactory.DOT_KEY);

        addXmlHandler(
                new ClusterPeerHandler(),
                "cluster_peer",
                UuidKeyFactory.SIMPLE_KEY);


        addXmlHandler(
                new VServerPeerHandler(),
                "vserver_peer",
                UuidKeyFactory.SIMPLE_KEY);

        addXmlHandler(
                new SnapMirrorHistoryHandler(),
                "snap_mirror_history",
                UuidKeyFactory.SIMPLE_KEY);

        addXmlHandler(
                new SnapLockHandler(),
                "snaplock",
                UuidKeyFactory.SIMPLE_KEY);

        addXmlHandler(
                new NetworkRouteHandler(),
                "network_route",
                UuidKeyFactory.DOT_KEY);

        addXmlHandler(
                new NetworkRouteLifHandler(),
                "network_route_lif",
                UuidKeyFactory.DOT_KEY);

        addXmlHandler(
                new AggregateObjectstoreConfigMappingHandler(),
                "aggregate_objectstore_config_mapping",
                UuidKeyFactory.DOT_KEY);

        addXmlHandler(
            new ObjectstoreSpaceDetailHandler(),
            "objectstore_space_detail",
            UuidKeyFactory.DOT_KEY);

        addXmlHandler(
                new FileHandler(),
                "file",
                UuidKeyFactory.DOT_KEY);

        /**
         * PLEASE NOTE: There is no particular order running  the following db runners.
         * Please avoid introducing dependencies of runners.
         */

        //make sure in a failover group change the lif is updated with the correct failover group
        addDbRunner(LifHandler.SubTag.NETWORK_LIF.getTagName(), "com/netapp/oci/netapp/originator/runner/network_failover_group_link_network_lif.sql", null);
        addDbRunner(LifHandler.SubTag.FCP_LIF.getTagName(), "com/netapp/oci/netapp/originator/runner/network_failover_group_link_fcp_lif.sql", null);

        //make sure the lif is updated when a failover group is deleted
        addDbRunner(NetworkFailoverGroupHandler.TAG, null, "com/netapp/oci/netapp/originator/runner/network_failover_group_unlink_lif.sql");


        addDbRunner(HaPairHandler.TAG,
                "com/netapp/oci/netapp/originator/runner/ha_pair_link_node_partner.sql",
                "com/netapp/oci/netapp/originator/runner/ha_pair_unlink_node_partner.sql");
        addDbRunner(VolumeHandler.TAG,
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/volume_activate_volume_move.sql",
                        "com/netapp/oci/netapp/originator/runner/volume_link_vserver_root.sql",
                        "com/netapp/oci/netapp/originator/runner/volume_link_snap_mirror.sql"),
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/volume_soft_delete_volume_move.sql",
                        "com/netapp/oci/netapp/originator/runner/volume_unlink_vserver_root.sql",
                        "com/netapp/oci/netapp/originator/runner/volume_unlink_snap_mirror.sql",
                        "com/netapp/oci/netapp/originator/runner/volume_unlink_cifs_share.sql"));
        addDbRunner(VolumeFlexCacheHandler.TAG,
                "com/netapp/oci/netapp/originator/runner/volume_flex_cache_link.sql",
                "com/netapp/oci/netapp/originator/runner/volume_flex_cache_unlink.sql");
        addDbRunner(SnapMirrorHandler.TAG,
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/snap_mirror_link_source.sql",
                        "com/netapp/oci/netapp/originator/runner/snap_mirror_vserver_link_source.sql"),
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/snap_mirror_soft_delete_snap_mirror_history.sql"));

        addDbRunner(QtreeHandler.TAG,
                null,
                "com/netapp/oci/netapp/originator/runner/qtree_unlink_cifs_share.sql");
        addDbRunner(VserverHandler.TAG,
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/vserver_link_snap_mirror.sql",
                        "com/netapp/oci/netapp/originator/runner/vserver_link_vserver_peer.sql"),
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/vserver_unlink_snap_mirror.sql",
                        "com/netapp/oci/netapp/originator/runner/vserver_unlink_vserver_peer.sql"));

        addDbRunner(ClusterHandler.TAG,
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/cluster_link_drPartner.sql",
                        "com/netapp/oci/netapp/originator/runner/cluster_drPartner_linkback.sql",
                        "com/netapp/oci/netapp/originator/runner/cluster_link_cluster_peer.sql",
                        "com/netapp/oci/netapp/originator/runner/cluster_update_drPartner.sql"),
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/cluster_unlink_drPartner.sql",
                        "com/netapp/oci/netapp/originator/runner/cluster_unlink_cluster_peer.sql"));

        addDbRunner(NodeHandler.TAG,
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/node_link_drPartner.sql",
                        "com/netapp/oci/netapp/originator/runner/node_drPartner_linkback.sql",
                        "com/netapp/oci/netapp/originator/runner/node_update_drPartner.sql",
                        "com/netapp/oci/netapp/originator/runner/mcc_disk_remote_home_owner_active_nodeId_linkback.sql"),
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/node_unlink_drPartner.sql",
                        "com/netapp/oci/netapp/originator/runner/mcc_disk_remote_home_owner_active_nodeId_unlink.sql",
                        "com/netapp/oci/netapp/originator/runner/node_soft_delete_qos_workload_node_relationship.sql"));

        addDbRunner(DiskHandler.TAG,
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/mcc_disk_remote_home_owner_active_nodeId_link.sql"
                ),
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/disk_soft_delete_disk_aggregate_relationship.sql"));

        addDbRunner(ClusterPeerHandler.TAG,
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/cluster_peer_link_remote_cluster.sql"
                ),
                Collections.<String>emptyList());

        addDbRunner(VServerPeerHandler.TAG,
                Arrays.asList(
                        "com/netapp/oci/netapp/originator/runner/vserver_peer_link_peer_vserver.sql"
                ),
                Collections.<String>emptyList());

        addDbRunner(ExportPolicyHandler.TAG, null, "com/netapp/oci/netapp/originator/runner/volume_qtree_unset_exportpolicyid.sql");

        addDbRunner("Igroup", null, "com/netapp/oci/netapp/originator/runner/lunmap_unset_igroupid.sql");

        //initialize tag to change detail attribute list.
        tagToChangeDetailAttributes = Collections.unmodifiableMap(initializeTagChangeDetailRecord());

        setTagToColumnsToUnlinkOnDelete(Collections.unmodifiableMap(initializeTagToColumnsToUnlinkOnDelete()));
        setTagToDependentChildrenForOfflineTags(Collections.unmodifiableMap(initializeTagToDependentChildrenForOfflineTags()));

    }

    private Map<String, List<XmlChildTagDetails>> initializeTagToDependentChildrenForOfflineTags() {
        final Map<String, List<XmlChildTagDetails>> map = new HashMap<String, List<XmlChildTagDetails>>();

        //******************************************************************************
        //                          ADD QosWorkload's DEPENDENT CHILDREN
        //******************************************************************************
        final List<XmlChildTagDetails> childrenTagDetailsForQosWorkload = new ArrayList<XmlChildTagDetails>();

        // Add QosWorkloadNodeRelationship relationship
        childrenTagDetailsForQosWorkload.add(new XmlChildTagDetails(QosWorkloadNodeRelationshipHandler.TAG, QosWorkloadNodeRelationshipHandler.TABLE_NAME,
                Collections.singletonMap(QosWorkloadNodeRelationshipHandler.TABLE_NAME, "workloadId")));

        //******************************************************************************
        //                          ADD VOLUME'S DEPENDENT CHILDREN
        //******************************************************************************
        List<XmlChildTagDetails> childrenTagDetailsForVolume = new ArrayList<XmlChildTagDetails>();

        // Add Lun relationship
        childrenTagDetailsForVolume.add(new XmlChildTagDetails(LunHandler.TAG, LunHandler.TABLE_NAME,
                Collections.singletonMap(LunHandler.TABLE_NAME, "volumeId"),
                VserverHandler.TABLE_NAME, VserverHandler.CHILD_FOREIGN_KEY, VserverHandler.TAG));


        // Add Namespace relationship
        childrenTagDetailsForVolume.add(new XmlChildTagDetails(NamespaceHandler.TAG, NamespaceHandler.TABLE_NAME,
                Collections.singletonMap(NamespaceHandler.TABLE_NAME, "volumeId"),
                VserverHandler.TABLE_NAME, VserverHandler.CHILD_FOREIGN_KEY, VserverHandler.TAG));


        // Add Qtree relationship
        childrenTagDetailsForVolume.add(new XmlChildTagDetails(QtreeHandler.TAG, QtreeHandler.TABLE_NAME,
                Collections.singletonMap(QtreeHandler.TABLE_NAME, "volumeId"),
                VserverHandler.TABLE_NAME, VserverHandler.CHILD_FOREIGN_KEY, VserverHandler.TAG));

        // Add qtree_quota relationship
        childrenTagDetailsForVolume.add(new XmlChildTagDetails(TreeQuotaHandler.TAG, TreeQuotaHandler.TABLE_NAME,
                Collections.singletonMap(TreeQuotaHandler.TABLE_NAME, "volumeId"),
                VserverHandler.TABLE_NAME, VserverHandler.CHILD_FOREIGN_KEY, VserverHandler.TAG));

        // Add user_quota relationship
        childrenTagDetailsForVolume.add(new XmlChildTagDetails(UserQuotaHandler.TAG, UserQuotaHandler.TABLE_NAME,
                Collections.singletonMap(UserQuotaHandler.TABLE_NAME, "volumeId"),
                VserverHandler.TABLE_NAME, VserverHandler.CHILD_FOREIGN_KEY, VserverHandler.TAG));

        // Add SnapLock relationship
        childrenTagDetailsForVolume.add(new XmlChildTagDetails(SnapLockHandler.TAG, SnapLockHandler.TABLE_NAME,
                Collections.singletonMap(SnapLockHandler.TABLE_NAME, "volumeId")));

        // Add snapmirror relationship
        childrenTagDetailsForVolume.add(new XmlChildTagDetails(SnapMirrorHandler.TAG, SnapMirrorHandler.TABLE_NAME,
                Collections.singletonMap(SnapMirrorHandler.TABLE_NAME, "volumeId")));

        //Add CifsShare
        childrenTagDetailsForVolume.add(
                new XmlChildTagDetails(CifsShareHandler.TAG, CifsShareHandler.TABLE_NAME,
                        Collections.singletonMap(CifsShareHandler.TABLE_NAME, "volumeId")));


        // Add quota_user relationship
        Map<String, String> quotaUserRelationshipMap = new LinkedHashMap<String, String>();
        quotaUserRelationshipMap.put(UserQuotaHandler.TABLE_NAME, "volumeId");
        quotaUserRelationshipMap.put(QuotaUserHandler.TABLE_NAME, "userQuotaId");

        childrenTagDetailsForVolume.add(new XmlChildTagDetails(QuotaUserHandler.TAG, QuotaUserHandler.TABLE_NAME,
                Collections.unmodifiableMap(quotaUserRelationshipMap),
                VserverHandler.TABLE_NAME, VserverHandler.CHILD_FOREIGN_KEY, VserverHandler.TAG));

        // Add lun_map relationship
        Map<String, String> relationshipMap = new LinkedHashMap<String, String>();
        relationshipMap.put(LunHandler.TABLE_NAME, "volumeId");
        relationshipMap.put(LunMapHandler.TABLE_NAME, "lunId");
        childrenTagDetailsForVolume.add(new XmlChildTagDetails(LunMapHandler.TAG, LunMapHandler.TABLE_NAME,
                Collections.unmodifiableMap(relationshipMap),
                VserverHandler.TABLE_NAME, VserverHandler.CHILD_FOREIGN_KEY, VserverHandler.TAG));

         //Add lun_map_reporting_node
        Map<String, String> lunMapRelMap = new LinkedHashMap<>();
        lunMapRelMap.put(LunHandler.TABLE_NAME, "volumeId");
        lunMapRelMap.put(LunMapHandler.TABLE_NAME, "lunId");
        lunMapRelMap.put(LUNMapReportingNodeHandler.TABLE_NAME, "lunMapId");
        childrenTagDetailsForVolume.add(new XmlChildTagDetails(
                LUNMapReportingNodeHandler.TAG,
                LUNMapReportingNodeHandler.TABLE_NAME,
                Collections.unmodifiableMap(lunMapRelMap)));

        //Add VolumeFlexgroupConstituentRelationship
        childrenTagDetailsForVolume.add(
                new XmlChildTagDetails(VolumeFlexgroupConstituentRelationshipHandler.TAG, VolumeFlexgroupConstituentRelationshipHandler.TABLE_NAME,
                        Collections.singletonMap(VolumeFlexgroupConstituentRelationshipHandler.TABLE_NAME, "objid")));



        // Code below is a hack which is relying on current implementation of ObjectQueryRunner's buildSimpleSql()
        // and buildSql() which use FK column in expressions like: format("JOIN ON %s.%s", tableName, FK)
        // As result code below injects SQL condition in this statement but it should not have any knowledge about
        // SQL expressions in other packages
        childrenTagDetailsForVolume.add(new XmlChildTagDetails(QosWorkloadHandler.TAG, QosWorkloadHandler.TABLE_NAME,
                Collections.singletonMap(QosWorkloadHandler.TABLE_NAME, "holderId and holderType='VOLUME'")));

        Map<String, String> lunToWlRelMap = new LinkedHashMap<String, String>();
        lunToWlRelMap.put(LunHandler.TABLE_NAME, "volumeId");
        lunToWlRelMap.put(QosWorkloadHandler.TABLE_NAME, "holderId and holderType='LUN'");
        childrenTagDetailsForVolume.add(new XmlChildTagDetails(QosWorkloadHandler.TAG, QosWorkloadHandler.TABLE_NAME,
                Collections.unmodifiableMap(lunToWlRelMap)));

        for (XmlChildTagDetails qosWorkloadChildDetails : childrenTagDetailsForQosWorkload) {
            Map<String, String> childToParentRelationshipMap = new LinkedHashMap<String, String>();
            childToParentRelationshipMap.put(QosWorkloadHandler.TABLE_NAME, "holderId and holderType='VOLUME'");
            for (Map.Entry<String, String> entry : qosWorkloadChildDetails.getDependentTableToForeignKeyColumn().entrySet()) {
                childToParentRelationshipMap.put(entry.getKey(), entry.getValue());
            }
            childrenTagDetailsForVolume.add(new XmlChildTagDetails(qosWorkloadChildDetails.getChildTagName(), qosWorkloadChildDetails.getChildTable(),
                    Collections.unmodifiableMap(childToParentRelationshipMap),
                    qosWorkloadChildDetails.getParentTable(), qosWorkloadChildDetails.getParentFK(), qosWorkloadChildDetails.getParentTag()));
        }

        for (XmlChildTagDetails qosWorkloadChildDetails : childrenTagDetailsForQosWorkload) {
            Map<String, String> childToParentRelationshipMap = new LinkedHashMap<String, String>();
            childToParentRelationshipMap.put(LunHandler.TABLE_NAME, "volumeId");
            childToParentRelationshipMap.put(QosWorkloadHandler.TABLE_NAME, "holderId and holderType='LUN'");
            for (Map.Entry<String, String> entry : qosWorkloadChildDetails.getDependentTableToForeignKeyColumn().entrySet()) {
                childToParentRelationshipMap.put(entry.getKey(), entry.getValue());
            }
            childrenTagDetailsForVolume.add(new XmlChildTagDetails(qosWorkloadChildDetails.getChildTagName(), qosWorkloadChildDetails.getChildTable(),
                    Collections.unmodifiableMap(childToParentRelationshipMap)));
        }
        // end of hack

        map.put(VolumeHandler.TAG, Collections.unmodifiableList(childrenTagDetailsForVolume));

        //******************************************************************************
        //                          ADD AGGREGATE'S DEPENDENT CHILDREN
        //******************************************************************************
        // Add volume relationship
        List<XmlChildTagDetails> childrenTagDetailsForAggregate = new ArrayList<XmlChildTagDetails>();
        childrenTagDetailsForAggregate.add(new XmlChildTagDetails(VolumeHandler.TAG, VolumeHandler.TABLE_NAME,
                Collections.singletonMap(VolumeHandler.TABLE_NAME, "aggregateId"),
                VserverHandler.TABLE_NAME, VserverHandler.CHILD_FOREIGN_KEY, VserverHandler.TAG));

        // Add volume's children relationship
        for (XmlChildTagDetails volumeChildDetails : childrenTagDetailsForVolume) {
            Map<String, String> childToParentRelationshipMap = new LinkedHashMap<String, String>();
            childToParentRelationshipMap.put(VolumeHandler.TABLE_NAME, "aggregateId");
            for (Map.Entry<String, String> entry : volumeChildDetails.getDependentTableToForeignKeyColumn().entrySet()) {
                childToParentRelationshipMap.put(entry.getKey(), entry.getValue());
            }
            childrenTagDetailsForAggregate.add(new XmlChildTagDetails(volumeChildDetails.getChildTagName(), volumeChildDetails.getChildTable(),
                    childToParentRelationshipMap,
                    volumeChildDetails.getParentTable(), volumeChildDetails.getParentFK(), volumeChildDetails.getParentTag()));
        }

        // Add Raid group relationship
        childrenTagDetailsForAggregate.add(new XmlChildTagDetails(RaidGroupHandler.TAG, RaidGroupHandler.TABLE_NAME,
                Collections.singletonMap(RaidGroupHandler.TABLE_NAME, "aggregateId")));
        //Add Plex
        childrenTagDetailsForAggregate.add(new XmlChildTagDetails(PlexHandler.TAG, PlexHandler.TABLE_NAME,
                Collections.singletonMap(PlexHandler.TABLE_NAME, "aggregateId")));

        //Add Disk Aggregate Relationship
        childrenTagDetailsForAggregate.add(
                new XmlChildTagDetails(DiskAggregateRelationshipHandler.TAG, DiskAggregateRelationshipHandler.TABLE_NAME,
                Collections.singletonMap(DiskAggregateRelationshipHandler.TABLE_NAME, "aggregateId")));

        //Add Storage Pool Aggregate Relationship
        childrenTagDetailsForAggregate.add(
                new XmlChildTagDetails(StoragePoolAggregateHandler.TAG, StoragePoolAggregateHandler.TABLE_NAME,
                        Collections.singletonMap(StoragePoolAggregateHandler.TABLE_NAME, "aggregateId")));

        //Add Vserver To Aggregate
        childrenTagDetailsForAggregate.add(
                new XmlChildTagDetails(VserverAggregateRelationshipHandler.TAG, VserverAggregateRelationshipHandler.TABLE_NAME,
                        Collections.singletonMap(VserverAggregateRelationshipHandler.TABLE_NAME, "aggregateId")));

        //Add Qos Service Center
        childrenTagDetailsForAggregate.add(
                new XmlChildTagDetails(QosServiceCenterHandler.TAG, QosServiceCenterHandler.TABLE_NAME,
                        Collections.singletonMap(QosServiceCenterHandler.TABLE_NAME, "aggregateId")));

        //Add AggregateObjectstoreConfigMapping
        childrenTagDetailsForAggregate.add(
                new XmlChildTagDetails(AggregateObjectstoreConfigMappingHandler.TAG, AggregateObjectstoreConfigMappingHandler.TABLE_NAME,
                        Collections.singletonMap(AggregateObjectstoreConfigMappingHandler.TABLE_NAME, "aggregateId")));

        //Add Qos workload detail
        Map<String, String> workloadDetailRelationshipMap = new LinkedHashMap<>();
        workloadDetailRelationshipMap.put(QosServiceCenterHandler.TABLE_NAME, "aggregateId");
        workloadDetailRelationshipMap.put(QosWorkloadDetailHandler.TABLE_NAME, "serviceCenterId");

        childrenTagDetailsForAggregate.add(
                new XmlChildTagDetails(QosWorkloadDetailHandler.TAG, QosWorkloadDetailHandler.TABLE_NAME,
                        workloadDetailRelationshipMap));

        map.put(AggregateHandler.TAG, Collections.unmodifiableList(childrenTagDetailsForAggregate));

        //******************************************************************************
        //                          ADD NODE's DEPENDENT CHILDREN
        //******************************************************************************
        final List<XmlChildTagDetails> childrenTagDetailsForNode = new ArrayList<XmlChildTagDetails>();

        // Add disk relationship
        childrenTagDetailsForNode.add(new XmlChildTagDetails(DiskHandler.TAG, DiskHandler.TABLE_NAME,
                Collections.singletonMap(DiskHandler.TABLE_NAME, "activeNodeId")));

        // Add FCP Port relationship
        childrenTagDetailsForNode.add(new XmlChildTagDetails(FcpPortHandler.TAG, FcpPortHandler.TABLE_NAME,
                Collections.singletonMap(FcpPortHandler.TABLE_NAME, "nodeId")));

        // Add Network Port relationship
        childrenTagDetailsForNode.add(new XmlChildTagDetails(NetworkPortHandler.TAG, NetworkPortHandler.TABLE_NAME,
                Collections.singletonMap(NetworkPortHandler.TABLE_NAME, "nodeId")));

        // Add aggregate relationship
        childrenTagDetailsForNode.add(new XmlChildTagDetails(AggregateHandler.TAG, AggregateHandler.TABLE_NAME,
                Collections.singletonMap(AggregateHandler.TABLE_NAME, "nodeId")));

        // Add aggregate children relationship
        for (XmlChildTagDetails aggregateChildDetails : childrenTagDetailsForAggregate) {
            Map<String, String> childToParentRelationshipMap = new LinkedHashMap<String, String>();
            childToParentRelationshipMap.put(AggregateHandler.TABLE_NAME, "nodeId");
            for (Map.Entry<String, String> entry : aggregateChildDetails.getDependentTableToForeignKeyColumn().entrySet()) {
                childToParentRelationshipMap.put(entry.getKey(), entry.getValue());
            }
            childrenTagDetailsForNode.add(new XmlChildTagDetails(aggregateChildDetails.getChildTagName(), aggregateChildDetails.getChildTable(),
                    childToParentRelationshipMap,
                    aggregateChildDetails.getParentTable(), aggregateChildDetails.getParentFK(), aggregateChildDetails.getParentTag()));
        }

        map.put(NodeHandler.TAG, Collections.unmodifiableList(childrenTagDetailsForNode));

        return map;
    }

    private Map<String, List<String>> initializeTagChangeDetailRecord() {
        Map<String, List<String>> tagToChangeDetailAttributes = new HashMap<String, List<String>>();

        //cluster attribute tracking.
        List<String> clusterChangeDetailAttributes = new ArrayList<String>();
        clusterChangeDetailAttributes.add("Name");
        clusterChangeDetailAttributes.add("MtConfigurationState");
        clusterChangeDetailAttributes.add("MtConfigurationType");
        clusterChangeDetailAttributes.add("MtMode");
        clusterChangeDetailAttributes.add("MtDrPartnerClusterUUID");
        clusterChangeDetailAttributes.add("VersionGeneration");
        clusterChangeDetailAttributes.add("VersionMajor");
        clusterChangeDetailAttributes.add("VersionMinor");
        tagToChangeDetailAttributes.put(ClusterHandler.TAG, clusterChangeDetailAttributes);

        //Node attribute tracking
        List<String> nodeChangeDetailAttributes = new ArrayList<String>();
        nodeChangeDetailAttributes.add("Name");
        nodeChangeDetailAttributes.add("FailoverState");
        nodeChangeDetailAttributes.add("MtDrOperationState");
        nodeChangeDetailAttributes.add("NodeStorageConfigurationRaw");
        tagToChangeDetailAttributes.put(NodeHandler.TAG, nodeChangeDetailAttributes);

        //NetworkLif attribute tracking.
        List<String> networkLifChangeDetailAttributes = new ArrayList<String>();
        networkLifChangeDetailAttributes.add("IsHome");
        tagToChangeDetailAttributes.put(LifHandler.SubTag.NETWORK_LIF.getTagName(), networkLifChangeDetailAttributes);

        // volume attributes for which detailed change info is needed.
        List<String> volumeChangeDetailAttributes = new ArrayList<String>();
        volumeChangeDetailAttributes.add("ExportPolicyId");
        volumeChangeDetailAttributes.add("SnapshotPolicyId");
        volumeChangeDetailAttributes.add("SisPolicyId");
        volumeChangeDetailAttributes.add("AggregateId");
        volumeChangeDetailAttributes.add("QosPolicyGroupId");
        volumeChangeDetailAttributes.add("IsJunctionActive");
        volumeChangeDetailAttributes.add("IsSisInlineCompressionEnabled");
        volumeChangeDetailAttributes.add("IsSpaceGuaranteeEnabled");
        volumeChangeDetailAttributes.add("JunctionPath");
        volumeChangeDetailAttributes.add("Name");
        volumeChangeDetailAttributes.add("PercentageFractionalReserve");
        volumeChangeDetailAttributes.add("PercentageSnapshotReserve");
        volumeChangeDetailAttributes.add("SisStatus");
        volumeChangeDetailAttributes.add("SpaceGuarantee");
        volumeChangeDetailAttributes.add("SpaceGuaranteeRaw");
        volumeChangeDetailAttributes.add("IsManagedByService");
        volumeChangeDetailAttributes.add("AutoSizeMaximumSize");
        volumeChangeDetailAttributes.add("AutoSizeIncrementSize");
        volumeChangeDetailAttributes.add("AutoSizeModeRaw");
        volumeChangeDetailAttributes.add("SnapshotAutoDeleteCommitmentRaw");
        volumeChangeDetailAttributes.add("SnapshotAutoDeleteDeleteOrderRaw");
        volumeChangeDetailAttributes.add("SnapshotAutoDeleteDeferDeleteRaw");
        volumeChangeDetailAttributes.add("SnapshotAutoDeleteTargetFreeSpace");
        volumeChangeDetailAttributes.add("SnapshotAutoDeleteTriggerRaw");
        volumeChangeDetailAttributes.add("SnapshotAutoDeletePrefix");
        volumeChangeDetailAttributes.add("SnapshotAutoDeleteDestroyList");
        volumeChangeDetailAttributes.add("IsSnapshotAutoDeleteEnabled");
        volumeChangeDetailAttributes.add("State");
        tagToChangeDetailAttributes.put(VolumeHandler.TAG, volumeChangeDetailAttributes);

        List<String> volumeRelationshipAttributes = new ArrayList<String>();
        volumeRelationshipAttributes.add("CloneParentId");
        tagToChangeDetailAttributes.put(VolumeSelfRelationshipHandler.TAG, volumeRelationshipAttributes);

        //Aggregate change attributes.
        List<String> aggregateChangeDetailAttributes = new ArrayList<String>();
        aggregateChangeDetailAttributes.add("Name");
        aggregateChangeDetailAttributes.add("IsHybridEnabled");
        aggregateChangeDetailAttributes.add("IsInactiveDataReportingEnabled");
        aggregateChangeDetailAttributes.add("NodeId");
        tagToChangeDetailAttributes.put(AggregateHandler.TAG, aggregateChangeDetailAttributes);

        //Vserver change attributes.
        List<String> vserverChangeDetailAttributes = new ArrayList<String>();
        vserverChangeDetailAttributes.add("Name");
        vserverChangeDetailAttributes.add("CIFSDomain");
        vserverChangeDetailAttributes.add("ActiveNISDomainName");
        tagToChangeDetailAttributes.put(VserverHandler.TAG, vserverChangeDetailAttributes);

        //QoS Policy group attributes
        List<String> qosPolicyGroupChangeDetailAttributes = new ArrayList<String>();
        qosPolicyGroupChangeDetailAttributes.add("MaxThroughput");
        qosPolicyGroupChangeDetailAttributes.add("MinThroughput");
        tagToChangeDetailAttributes.put(QosPolicyGroupHandler.TAG, qosPolicyGroupChangeDetailAttributes);

        //Raid group attributes.
        List<String> raidGroupChangeDetailAttributes = new ArrayList<String>();
        raidGroupChangeDetailAttributes.add("IsReconstructing");
        tagToChangeDetailAttributes.put(RaidGroupHandler.TAG, raidGroupChangeDetailAttributes);

        //Qtree attribute tracking
        List<String> qtreeChangeDetailAttributes = new ArrayList<String>();
        qtreeChangeDetailAttributes.add("Name");
        tagToChangeDetailAttributes.put(QtreeHandler.TAG, qtreeChangeDetailAttributes);

        //Lun attribute tracking
        List<String> lunChangeDetailAttributes = new ArrayList<String>();
        lunChangeDetailAttributes.add("IsOnline");
        lunChangeDetailAttributes.add("VolumeId");
        lunChangeDetailAttributes.add("QosPolicyGroupId");
        tagToChangeDetailAttributes.put(LunHandler.TAG, lunChangeDetailAttributes);

/* MJK: PENDING ONTAP
        //Namespace attribute tracking
        List<String> namespaceChangeDetailAttributes = new ArrayList<String>();
        namespaceChangeDetailAttributes.add("IsOnline");
        namespaceChangeDetailAttributes.add("VolumeId");
        namespaceChangeDetailAttributes.add("QosPolicyGroupId");
        tagToChangeDetailAttributes.put(NamespaceHandler.TAG, namespaceChangeDetailAttributes);
*/

        //Flash device tracking
        List<String> flashDeviceChangeDetailAttributes = new ArrayList<>();
        flashDeviceChangeDetailAttributes.add("Status");
        tagToChangeDetailAttributes.put(FlashDeviceHandler.TAG, flashDeviceChangeDetailAttributes);

        return tagToChangeDetailAttributes;
    }

    public Map<String, Map<String, List<String>>> initializeTagToColumnsToUnlinkOnDelete() {
        Map<String, Map<String, List<String>>> tagToColumnsToUnlinkMap = new HashMap<String, Map<String, List<String>>>();

        // ExportPolicy
        tagToColumnsToUnlinkMap.put(ExportPolicyHandler.TAG, Collections.unmodifiableMap(Collections.singletonMap(VolumeHandler.TABLE_NAME, Collections.singletonList("exportPolicyId"))));

        // JobSchedule
        tagToColumnsToUnlinkMap.put(JobScheduleHandler.TAG_NAME, Collections.unmodifiableMap(Collections.singletonMap(SnapMirrorHandler.TABLE_NAME, Collections.singletonList("jobScheduleId"))));

        // Node
        tagToColumnsToUnlinkMap.put(NodeHandler.TAG, Collections.unmodifiableMap(Collections.singletonMap(SnapMirrorHandler.TABLE_NAME, Collections.singletonList("nodeId"))));

        // QosPolicyGroup
        Map<String, List<String>> map = new HashMap<String, List<String>>();
        map.put(VolumeHandler.TABLE_NAME, Collections.singletonList("qosPolicyGroupId"));
        map.put(LunHandler.TABLE_NAME, Collections.singletonList("qosPolicyGroupId"));
/* MJK: PENDING ONTAP
        map.put(NamespaceHandler.TABLE_NAME, Collections.singletonList("qosPolicyGroupId"));
*/

        tagToColumnsToUnlinkMap.put(QosPolicyGroupHandler.TAG, Collections.unmodifiableMap(map));

        // RaidGroupHandler
        tagToColumnsToUnlinkMap.put(RaidGroupHandler.TAG, Collections.unmodifiableMap(Collections.singletonMap(DiskHandler.TABLE_NAME, Collections.singletonList("raidGroupId"))));

        // SisPolicyHandler
        tagToColumnsToUnlinkMap.put(SisPolicyHandler.TAG_NAME, Collections.unmodifiableMap(Collections.singletonMap(VolumeHandler.TABLE_NAME, Collections.singletonList("sisPolicyId"))));

        // SnapMirrorPolicyHandler
        tagToColumnsToUnlinkMap.put(SnapMirrorPolicyHandler.TAG_NAME, Collections.unmodifiableMap(Collections.singletonMap(SnapMirrorHandler.TABLE_NAME, Collections.singletonList("snapMirrorPolicyId"))));

        // SnapshotHandler
        tagToColumnsToUnlinkMap.put(SnapshotHandler.TAG_NAME, Collections.unmodifiableMap(Collections.singletonMap(SnapMirrorHandler.TABLE_NAME, Arrays.asList("exportedSnapshotId", "newestSnapshotId"))));

        // SnapshotPolicyHandler
        tagToColumnsToUnlinkMap.put(SnapshotPolicyHandler.TAG_NAME, Collections.unmodifiableMap(Collections.singletonMap(VolumeHandler.TABLE_NAME, Collections.singletonList("snapshotPolicyId"))));

        //StorageServiceHandler
        tagToColumnsToUnlinkMap.put(StorageServiceHandler.TAG, Collections.unmodifiableMap(Collections.singletonMap(VolumeHandler.TABLE_NAME, Collections.singletonList("storageServiceId"))));

        return tagToColumnsToUnlinkMap;
    }

    public final Map<String, List<String>> getTagToChangeDetailAttributes() {
        return tagToChangeDetailAttributes;
    }

}
