/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

/**
 * @author Al Velella
 *         Date: Oct 24, 2011
 *         Time: 2:51:41 PM
 */
public class AggregateHandler extends XmlHandler {
    public static final String TAG = "Aggregate";
    public static final String TABLE_NAME = "netapp_model.aggregate";



    public AggregateHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long nodeId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(1);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterId", clusterId)
                .column("NodeId", nodeId)
                .column("Type", String.class)
                .column("IsHybrid", boolean.class)
                .column("IsHybridEnabled", boolean.class)
                .column("UsesSharedDisks", boolean.class)
                .column("State", Enum.class)
                .column("StateRaw", String.class)
                .column("SnapshotSizeTotal", long.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("SnapshotSizeUsed", long.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("SnapshotSizeAvail", long.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("HybridCacheSizeTotal", long.class)
                .column("SizeTotal", long.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("SizeUsed", long.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("SizeUsedPercent", int.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("SizeAvail", long.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("SizeAvailPercent", int.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("TotalCommitted", long.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("TotalReservedSpace", long.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("IsInactiveDataReportingEnabled", boolean.class)
                .column("RaidStatus", String.class)
                .column("RaidType", Enum.class)
                .column("RaidTypeRaw", String.class)
                .column("MirrorStatus", Enum.class)
                .column("MirrorStatusRaw", String.class)
                .column("BlockType", Enum.class)
                .column("BlockTypeRaw", String.class)
                .column("Is64BitUpgradeInProgress", boolean.class)
                .column("VolumeDedupeSpaceSavings", long.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("VolumeCompressionSpaceSavings", long.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("RaidSize", long.class)
                .column("IsRootAggregate", boolean.class)
                .column("HasLocalRoot", boolean.class)
                .column("HasPartnerRoot", boolean.class)
                .column("AggregateType", Enum.class)
                .column("AggregateTypeRaw", String.class)
                .column("AggregateDerivedType", String.class)
                .column("IsCftPreCommit", boolean.class)
                .column("IsSnapLock", boolean.class)
                .column("SnapLockType", Enum.class)
                .column("SnapLockTypeRaw", String.class)
                .column("TotalStorageEfficiencyRatio", String.class)
                .column("IsComposite", boolean.class)
                .column("TotalUsedExternalLicensedSpace", long.class, EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE));
    }

    @Override
    public boolean isObjectOffline(OriginatorUpdate originatorUpdate, Map<String, String> objectProperties) {
        final String stateAttr = "State";
        return this.isObjectOffline(stateAttr, "OFFLINE", objectProperties) ||
                isObjectOffline(stateAttr, "RESTRICTED", objectProperties) ||
                isObjectOffline(stateAttr, "FAILED", objectProperties);
    }
}
