/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

public class AggregateObjectstoreConfigMappingHandler extends XmlHandler {
    public static final String TAG = "AggregateObjectstoreConfigMapping";
    public static final String TABLE_NAME = "netapp_model.aggregate_objectstore_config_mapping";


    public AggregateObjectstoreConfigMappingHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long objectstoreId = originatorUpdate.getRequiredObjectIdByTagname("ObjectstoreConfig", properties.get("ObjectstoreUUID"), getTagName());
        Long aggregateId = originatorUpdate.getRequiredObjectIdByTagname("Aggregate", properties.get("AggregateUUID"), getTagName());


        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("AggregateId", aggregateId)
                .column("AggregateName", String.class)
                .column("TieringFullnessThreshold", long.class)
                .column("ObjectstoreId", objectstoreId)
                .column("ObjectstoreName", String.class)
                .column("BinUuid", String.class)
                .column("AvailabilityState", Enum.class)
                .column("AvailabilityStateRaw", String.class);
    }
}

