/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.sql.Timestamp;
import java.util.EnumSet;
import java.util.Map;

/**
 *  * @author Iti Behari
  *         Date: July 11, 2013
 */
public class ApplicationRecordHandler extends XmlHandler {
    public static final String TABLE_NAME = "netapp_model.application_record";
    public static final String TAG = "ApplicationRecord";

    public ApplicationRecordHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ClusterId", clusterId)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("RecordName", String.class)
                .column("RecordValue", String.class, EnumSet.of(ColumnAttr.STRING_512))
                .column("ModifiedFrom", String.class)
                .column("LastModifiedTimestamp", Timestamp.class);
    }
}