/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: Feb 1, 2012
 *         Time: 5:22:51 PM
 */
public class CifsShareHandler extends XmlHandler {
    public static final String TAG = "CIFSShare";
    public static final String TABLE_NAME = "netapp_model.cifs_share";


    public CifsShareHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId(1);
        Long vserverId = originatorUpdate.peekObjectId();
        Long volumeId = originatorUpdate.getObjectIdByTagname("Volume", properties.get("VolumeUUID"));
        Long qtreeId = originatorUpdate.getObjectIdByTagname("Qtree", properties.get("QtreeUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterId", clusterId)
                .column("VserverId", vserverId)
                .column("VolumeId", volumeId)
                .column("QtreeId", qtreeId)
                .column("Path", String.class)
                .column("Comment", String.class)
                .column("ShareProperties", String.class);
    }
}