/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

/**
 * @author Al Velella
 *         Date: Sep 22, 2011
 *         Time: 12:00:18 PM
 */
public class ClusterHandler extends XmlHandler {

    public static final String TAG = "Cluster";

    public ClusterHandler() {
        super(TAG, "netapp_model.cluster");
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties, Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("SerialNumber", String.class)
                .column("ManagementIP", String.class)
                .column("Version", String.class)
                .column("VersionGeneration", int.class)
                .column("VersionMajor", int.class)
                .column("VersionMinor", int.class)
                .column("RawDiskBytesTotal", long.class)
                .column("RawDiskBytesUsed", long.class)
                .column("TotalExternalLicensedSpace", long.class)
                .column("Contact", String.class)
                .column("Location", String.class)
                .column("DiagnosisStatus", Enum.class)
                .column("DiagnosisStatusRaw", String.class)
                .column("HaConfigured", boolean.class)
                .column("TimeZone", String.class)
                .column("TimeZoneUTC", String.class)
                .column("TimeZoneVersion", String.class)
                .column("IsMetrocluster", boolean.class)
                .column("SslFipsEnabled", boolean.class)
                .column("MtDrPartnerClusterUUID", String.class)
                .column("MtConfigurationState", Enum.class)
                .column("MtConfigurationStateRaw", String.class)
                .column("MtMode", Enum.class)
                .column("MtModeRaw", String.class)
                .column("MtRemoteConfigurationState", Enum.class)
                .column("MtRemoteConfigurationStateRaw", String.class)
                .column("MtRemoteMode", Enum.class)
                .column("MtRemoteModeRaw", String.class)
                .column("MtConfigurationType",String.class);
    }
}
