package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;


public class ClusterPeerHandler extends XmlHandler {
    public static final String TAG = "ClusterPeer";
    public static final String TABLE_NAME = "netapp_model.cluster_peer";

    public ClusterPeerHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long networkIpSpaceId = originatorUpdate.getObjectIdByTagname("NetworkIpSpace", properties.get("IpSpaceUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("ActiveAddresses", String.class, EnumSet.of(ColumnAttr.TEXT))
                .column("AddressFamily", Enum.class)
                .column("AddressFamilyRaw", String.class)
                .column("AuthStatusAdmin", String.class)
                .column("AuthStatusOperational", Enum.class)
                .column("AuthStatusOperationalRaw", String.class)
                .column("Availability", Enum.class)
                .column("AvailabilityRaw", String.class)
                .column("RemoteClusterName", String.class)
                .column("RemoteClusterUUID", String.class)
                .column("NetworkIpSpaceId", networkIpSpaceId)
                .column("IsClusterHealthy", boolean.class)
                .column("PeerAddresses", String.class, EnumSet.of(ColumnAttr.TEXT))
                .column("RemoteClusterNodes", String.class, EnumSet.of(ColumnAttr.TEXT))
                .column("SerialNumber", String.class)
                .column("UnreachableLocalNodes", String.class, EnumSet.of(ColumnAttr.TEXT))
                .column("EncryptRaw", String.class)
        		.column("Encrypt", Enum.class);        	
    }
}
