/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

public class DiskAggregateRelationshipHandler extends XmlHandler {
    public static final String TAG = "DiskAggregateRelationship";
    public static final String TABLE_NAME = "netapp_model.disk_aggregate_relationship";


    public DiskAggregateRelationshipHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long diskId = originatorUpdate.getRequiredObjectIdByTagname("Disk", properties.get("DiskUUID"), getTagName());
        Long aggregateId = originatorUpdate.getRequiredObjectIdByTagname("Aggregate", properties.get("AggregateUUID"), getTagName());


        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("DiskId", diskId)
                .column("AggregateId", aggregateId)
                .column("IsCache", boolean.class);
    }
}
