/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: Oct 24, 2011
 *         Time: 2:51:50 PM
 */
public class DiskHandler extends XmlHandler {
    public static final String TAG = "Disk";
    public static final String TABLE_NAME = "netapp_model.disk";

    public DiskHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long homeNodeId = originatorUpdate.getObjectIdByTagname("Node", properties.get("HomeNodeUUID"));
        Long ownerNodeId = originatorUpdate.getObjectIdByTagname("Node", properties.get("OwnerNodeUUID"));
        Long activeNodeId = originatorUpdate.getObjectIdByTagname("Node", properties.get("ActiveNodeUUID"));
        Long raidGroupId = originatorUpdate.getObjectIdByTagname("RaidGroup", properties.get("RaidGroupUUID"));
        Long storagePoolId = originatorUpdate.getObjectIdByTagname("StoragePool", properties.get("StoragePoolUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterId", clusterId)
                .column("HomeNodeId", homeNodeId)
                .column("OwnerNodeId", ownerNodeId)
                .column("ActiveNodeId", activeNodeId)
                .column("RaidGroupId", raidGroupId)
                .column("StoragePoolId", storagePoolId)
                .column("InterfaceType", Enum.class)
                .column("Model", String.class)
                .column("FirmwareRevision", String.class)
                .column("Rpm", int.class)
                .column("SerialNumber", String.class)
                .column("Shelf", String.class)
                .column("ShelfBay", String.class)
                .column("Vendor", String.class)
                .column("ContainerType", Enum.class)
                .column("IsFailed", boolean.class)
                .column("PhysicalBlocks", long.class)
                .column("RaidPosition", Enum.class)
                .column("IsVirtual", boolean.class)
                .column("BytesPerSector", long.class)
                .column("UsedBlocks", long.class)
                .column("Uid", String.class)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("IsZeroed", boolean.class)
                .column("IsZeroing", boolean.class)
                .column("Pool", int.class)
                .column("ZeroingPercent", int.class)
                .column("ChecksumCompatibility", String.class)
                .column("UsedBytes", long.class)
                .column("TotalBytes", long.class)
                .column("ContainerTypeRaw", String.class)
                .column("RaidPositionRaw", String.class)
                .column("InterfaceTypeRaw", String.class)
                .column("IsMediaScrubbing", boolean.class)
                .column("IsOffline", boolean.class)
                .column("IsPrefailed", boolean.class)
                .column("IsReconstructing", boolean.class)
                .column("IsReplacing", boolean.class)
                .column("IsShared", boolean.class)
                .column("EffectiveInterfaceType", Enum.class)
                .column("EffectiveInterfaceTypeRaw", String.class)
                .column("CapacitySectors", long.class)
                .column("RightSizeSectors", long.class)
                .column("IsInFdr", boolean.class)
                .column("FailedReason", String.class)
        		.column("UsableDataSize", long.class)
        		.column("UsableRootSize", long.class)
                .column("HomeNodeName", String.class)
                .column("OwnerNodeName", String.class)
                .column("ActiveNodeName", String.class)
                .column("IsLocalAttach", boolean.class)
                .column("PercentSparesConsumed", int.class)
                .column("PercentRatedLifeUsed", int.class)
                .column("PartitioningType", Enum.class)
                .column("PartitioningTypeRaw", String.class)
        		;
    }
}
