/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: Jan 23, 2012
 *         Time: 1:25:46 P
 */
public class DiskPathHandler extends XmlHandler {

    public static final String TABLE_NAME = "netapp_model.disk_path";
    private static final String TAG = "DiskPath";

    public DiskPathHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long nodeId = originatorUpdate.getRequiredObjectIdByTagname("Node", properties.get("NodeUUID"), getTagName());
        Long diskId = originatorUpdate.getRequiredObjectIdByTagname("Disk", properties.get("DiskUUID"), getTagName());
        Long initiatorPortId = originatorUpdate.getObjectIdByTagname("StorageAdapter", properties.get("InitiatorPortUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("NodeId", nodeId)
                .column("DiskId", diskId)
                .column("InitiatorPortId", initiatorPortId)
                .column("ArrayName", String.class)
                .column("DiskPort", String.class)
                .column("DiskPortName", String.class)
                .column("InitiatorSideSwitchPort", String.class)
                .column("LUN", int.class)
                .column("LUNPathUseState", Enum.class)
                .column("LUNPathUseStateRaw", String.class)
                .column("TargetWWPN", String.class)
                .column("TargetSideSwitchPort", String.class)
                .column("TPGN", long.class)
                .column("TargetWWNN", String.class);
    }


}
