/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

/**
 * @author Al Velella
 *         Date: Feb 1, 2012
 *         Time: 5:28:52 PM
 */
public class ExportRuleHandler extends XmlHandler {

    public ExportRuleHandler() {
        super("ExportRule", "netapp_model.export_rule");
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId(2);
        Long vserverId = originatorUpdate.peekObjectId(1);
        Long exportPolicyId = originatorUpdate.peekObjectId();

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("VserverId", vserverId)
                .column("ExportPolicyId", exportPolicyId)
                .column("RuleIndex", int.class)
                .column("ClientMatch", String.class)
                .column("AccessProtocol", String.class)
                .column("RORule", String.class)
                .column("RWRule", String.class)
                .column("SuperUserSecurity", String.class)
                .column("AnonymousUserId", String.class)
                .column("IsAllowSetUidEnabled", boolean.class)
                .column("IsAllowDevIsEnabled", boolean.class);
    }
}