/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 *
 * Author: jsprague
 */
public class FcpPortHandler extends XmlHandler {
    public static final String TAG = "FcpPort";
    public static final String TABLE_NAME = "netapp_model.fcp_port";

    public FcpPortHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long nodeId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(1);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("NodeID", nodeId)
                .column("ClusterID", clusterId)
                .column("NodeName", String.class)
                .column("PortName", String.class)
                .column("Adapter", String.class)
                .column("FabricEstablished", boolean.class)
                .column("FirmwareRev", String.class)
                .column("InfoName", String.class)
                .column("MaxSpeed", int.class)
                .column("Speed", Enum.class)
                .column("SpeedRaw", String.class)                
                .column("PhysicalProtocol", Enum.class)
                .column("PhysicalProtocolRaw", String.class)
                .column("State", Enum.class)
                .column("StateRaw", String.class)
                .column("Status", Enum.class)
                .column("StatusRaw", String.class)
                .column("SwitchPort", String.class)
                .column("DataLinkRate", int.class)
                .column("MediaType", Enum.class)
                .column("MediaTypeRaw", String.class)
                .column("StatusAdmin", Enum.class)
                .column("StatusAdminRaw", String.class);
    }
}
