/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

/**
 * Created by jianwei on 8/25/17.
 */
public class FileHandler extends XmlHandler {
    public static final String TAG = "File";
    public static final String TABLE_NAME = "netapp_model.file";

    public FileHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long volumeId = originatorUpdate.peekObjectId();
        Long vserverId = originatorUpdate.peekObjectId(1);
        Long clusterId = originatorUpdate.peekObjectId(2);
        Long qosPolicyGroupId = originatorUpdate.getObjectIdByTagname("QosPolicyGroup", properties.get("QosPolicyGroupUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("VolumeId", volumeId)
                .column("VserverId", vserverId)
                .column("QosPolicyGroupId", qosPolicyGroupId)
                .column("Path", String.class)
                .column("IsOnline", boolean.class)
                .column("Size", long.class)
                .column("SizeUsed", long.class)
                .column("IsSpaceReservationEnabled", boolean.class)
                .column("IsSpaceAllocEnabled", boolean.class)
                .column("IsMapped", boolean.class)
                .column("SerialNumber", String.class)
                .column("MultiProtocolType", String.class)
                .column("AlignmentRaw", String.class)
                .column("PrefixSize", long.class)
                .column("SuffixSize", long.class)
                .column("ClassRaw", String.class)
                .column("Comment", String.class);
    }
}
