/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Iti Behari
 *         Date: 1/4/13
 */
public class FlashDeviceHandler extends XmlHandler {
    public static final String TAG = "FlashDevice";
    public static final String TABLE_NAME = "netapp_model.flash_device";

    public FlashDeviceHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long nodeId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(1);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("NodeId", nodeId)
                .column("SlotNumber", int.class)
                .column("SerialNumber", String.class)
                .column("ModelName", String.class)
                .column("FirmwareRevision", String.class)
                .column("CapacityGB", int.class)
                .column("PercentOnline", int.class)
                .column("Status", Enum.class)
                .column("StatusRaw", String.class)
                .column("ServiceTime", int.class)
                .column("ThresholdProfile", String.class);


    }
}
