/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: Jan 12, 2012
 *         Time: 10:09:18 AM
 */
public class IgroupInitiatorHandler extends XmlHandler {

    public IgroupInitiatorHandler() {
        super("IgroupInitiator", "netapp_model.igroup_initiator");
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long igroupId = originatorUpdate.peekObjectId();
        Long vserverId = originatorUpdate.peekObjectId(1);
        Long clusterId = originatorUpdate.peekObjectId(2);
        String name = properties.get("Name");

        // generate internal UUID
        properties.put("UUID", buildCompoundUUID(clusterId, igroupId, name));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterID", clusterId)
                .column("VserverID", vserverId)
                .column("IgroupID", igroupId)
                .column("Name", String.class);
    }

}
