/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: Feb 6, 2012
 *         Time: 5:21:36 PM
 */
public class IscsiPortalGroupLifHandler extends XmlHandler {

    public IscsiPortalGroupLifHandler() {
        super("ISCSIPortalGroupLIF", "netapp_model.iscsi_portal_group_lif");
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId(2);
        Long vserverId = originatorUpdate.peekObjectId(1);
        Long iscsiPortalGroupId = originatorUpdate.peekObjectId();
        Long lifId = originatorUpdate.getRequiredObjectIdByTagname("NetworkLIF", properties.get("LIFUUID"), getTagName());

        // generate internal UUID
        properties.put("UUID", buildCompoundUUID(clusterId, iscsiPortalGroupId, lifId));
        
        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("VserverId", vserverId)
                .column("TargetPortalGroupId", iscsiPortalGroupId)
                .column("LifId", lifId)
                .column("InterfaceName", String.class)
                .column("IPAddress", String.class)
                .column("IPPort", String.class);
    }
}
