/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: Feb 7, 2012
 *         Time: 2:52:47 PM
 */
public class IscsiSessionConnectionHandler extends XmlHandler {

    public IscsiSessionConnectionHandler() {
        super("ISCSISessionConnection", "netapp_model.iscsi_session_connection");
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId(3);
        Long iscsiSessionId = originatorUpdate.peekObjectId();
        Long lifId = originatorUpdate.getRequiredObjectIdByTagname("NetworkLIF", properties.get("LIFUUID"), getTagName());

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("IscsiSessionId", iscsiSessionId)
                .column("LifId", lifId)
                .column("ConnectionId", int.class)
                .column("LocalIPAddress", String.class)
                .column("LocalIPPort", String.class)
                .column("RemoteIPAddress", String.class)
                .column("AuthenticationStyle", Enum.class);
    }
}
