package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

/**
 * Handler for LUNMapReportingNode TAG, maps the data to the netapp_model.lun_map_reporting_node table.
 */
public class LUNMapReportingNodeHandler extends XmlHandler {
    public static final String TAG = "LUNMapReportingNode";
    public static final String TABLE_NAME = "netapp_model.lun_map_reporting_node";

    public LUNMapReportingNodeHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long lunMapId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(2);
        Long nodeId = originatorUpdate.getRequiredObjectIdByTagname("Node", properties.get("ReportingingNodeUUID"), getTagName());
        
        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState, false, EnumSet.of(ColumnAttr.STRING_512))
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterID", clusterId)
                .column("LUNMapId", lunMapId)
                .column("NodeId", nodeId);
    }
}
