/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.sql.Timestamp;
import java.util.EnumSet;
import java.util.Map;

/**
 * @author Nancy Walia
 */
public class LicenseHandler extends XmlHandler {
    public static final String TABLE_NAME = "netapp_model.license";

    public LicenseHandler() {
        super("License", TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();

        Long nodeId = originatorUpdate.getObjectIdByTagname(NodeHandler.TAG, properties.get("OwnerNodeUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ClusterId", clusterId)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("OwnerNodeId", nodeId)
                .column("OwnerType", Enum.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("OwnerName", String.class)
                .column("Package", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("SerialNumber", String.class)
                .column("ExpirationTime", Timestamp.class)
                .column("Description", String.class)
                .column("LicenseType", Enum.class)
                .column("LicenseTypeRaw", String.class);
    }
}