package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;
import java.util.EnumSet;
import java.util.Map;

public class LunImportHandler extends XmlHandler {
    public static final String TAG = "LunImport";
    public static final String TABLE_NAME = "netapp_model.lun_import";

    public LunImportHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        String vServerUUID = properties.get("VserverUUID");
        Long vServerId = originatorUpdate.getObjectIdByTagname("Vserver", vServerUUID); 
        String foreignDiskUUID = properties.get("ForeignDiskUUID");
        Long foreignDiskId = originatorUpdate.getObjectIdByTagname("Disk", foreignDiskUUID);
        Long importCurrentNodeId = originatorUpdate.getObjectIdByTagname("Node", properties.get("ImportCurrentNodeUUID"));
        Long importHomeNodeId = originatorUpdate.getObjectIdByTagname("Node", properties.get("ImportHomeNodeUUID"));
        
        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("VserverId", vServerId)
                .column("AdministrativeStatus", Enum.class)
                .column("AdministrativeStatusRaw", String.class)
                .column("ComparedBlocks", long.class)
                .column("EstimatedRemainingDuration", long.class)
                .column("FailureReason", String.class)
                .column("ForeignDiskId", foreignDiskId)
                .column("ImportCurrentNodeId", importCurrentNodeId)
                .column("ImportHomeNodeId", importHomeNodeId)
                .column("ImportedBlocks", long.class)
                .column("OperationInProgress", Enum.class)
                .column("OperationInProgressRaw", String.class)
                .column("OperationalStatus", Enum.class)
                .column("OperationalStatusRaw", String.class)
                .column("Path", String.class)
                .column("PercentComplete", long.class)
                .column("TotalBlocks", long.class);
    }
}

