/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: Jan 12, 2012
 *         Time: 10:11:45 AM
 */
public class LunMapHandler extends XmlHandler {

    public static final String TAG = "LUNMap";
    public static final String TABLE_NAME = "netapp_model.lun_map";

    public LunMapHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long vserverId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(1);
        Long lunId = originatorUpdate.getRequiredObjectIdByTagname("LUN", properties.get("LUNUUID"), getTagName());
        Long igroupId = originatorUpdate.getRequiredObjectIdByTagname("Igroup", properties.get("IgroupUUID"), getTagName());
        Long owningNodeId = originatorUpdate.getObjectIdByTagname("Node", properties.get("OwningNodeUUID"));
        
        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterID", clusterId)
                .column("VserverID", vserverId)
                .column("LunID", lunId)
                .column("IgroupID", igroupId)
                .column("LUN", int.class)
                .column("Path", String.class)
                .column("OwningNodeId", owningNodeId);
    }
}
