package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;
import javax.xml.soap.Text;
import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;


public class MetroClusterOperationHandler extends XmlHandler {
    public static final String TAG = "MetroClusterOperation";
    public static final String TABLE_NAME = "netapp_model.metrocluster_operation";
    
    public MetroClusterOperationHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long nodeId = originatorUpdate.getObjectIdByTagname("Node", properties.get("NodeUUID"));
        
        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("NodeId", nodeId)
                .column("StartTime", long.class)
                .column("EndTime", long.class)
                .column("ErrorList", String.class, EnumSet.of(ColumnAttr.TEXT))
                .column("jobId", long.class)
                .column("Operation", Enum.class)
                .column("OperationRaw", String.class)
                .column("State", Enum.class)
                .column("StateRaw", String.class);
    }
}
