/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: Nov 8, 2011
 *         Time: 10:02:13 AM
 */
public class NamespaceHandler extends XmlHandler {
    public static final String TAG = "Namespace";
    public static final String TABLE_NAME = "netapp_model.namespace";

    public NamespaceHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long volumeId = originatorUpdate.peekObjectId();
        Long vserverId = originatorUpdate.peekObjectId(1);
        Long clusterId = originatorUpdate.peekObjectId(2);
        Long qtreeId = originatorUpdate.getObjectIdByTagname("Qtree", properties.get("QtreeUUID"));
/* MJK: PENDING ONTAP
        Long qosPolicyGroupId = originatorUpdate.getObjectIdByTagname("QosPolicyGroup", properties.get("QosPolicyGroupUUID"));
*/

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("QtreeId", qtreeId)
                .column("VolumeId", volumeId)
                .column("VserverId", vserverId)
/* MJK: PENDING ONTAP
                .column("QosPolicyGroupId", qosPolicyGroupId)
*/
                .column("Path", String.class)
                .column("Subsystem", String.class)
                .column("IsReadOnly", boolean.class)
                .column("RestoreInaccessible", boolean.class)
                .column("Size", long.class)
                .column("SizeUsed", long.class)
                .column("BlockSize", long.class)
                .column("OsType", Enum.class)
                .column("OsTypeRaw", String.class)
                .column("State", Enum.class)
                .column("StateRaw", String.class)
                .column("Comment", String.class);
    }
}
