/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * Handler for mapping the Network Failover group attributes with the network_failover_group table.
 */
public class NetworkFailoverGroupHandler extends XmlHandler {

    public static final String TAG = "NetworkFailoverGroup";
    public static final String TABLE_NAME = "netapp_model.network_failover_group";

    public NetworkFailoverGroupHandler() {
        super(TAG, TABLE_NAME);
    }    

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long networkIpSpaceId = originatorUpdate.getObjectIdByTagname("NetworkPortBroadcastDomain", properties.get("NetworkPortBroadcastDomainUUID"));
        Long vserverId = originatorUpdate.getObjectIdByTagname("Vserver", properties.get("VserverUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterId", clusterId)
                .column("NetworkPortBroadcastDomainId", networkIpSpaceId)
                .column("vserverId", vserverId);
    }
}
