/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

public class NetworkFailoverGroupNetworkPortRelationshipHandler extends XmlHandler {
    public static final String TAG = "NetworkFailoverGroupNetworkPortRelationship";
    public static final String TABLE_NAME = "netapp_model.network_failover_group_network_port_relationship";


    public NetworkFailoverGroupNetworkPortRelationshipHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long nodeId = originatorUpdate.getRequiredObjectIdByTagname("Node", properties.get("NodeUUID"), getTagName());
        Long networkFailoverGroupId = originatorUpdate.getRequiredObjectIdByTagname("NetworkFailoverGroup", properties.get("NetworkFailoverGroupUUID"), getTagName());
        Long networkPortId = originatorUpdate.getRequiredObjectIdByTagname("NetworkPort", properties.get("NetworkPortUUID"), getTagName());


        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("NetworkFailoverGroupId", networkFailoverGroupId)
                .column("NetworkPortId", networkPortId);
    }
}
