/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Collections;
import java.util.EnumSet;
import java.util.List;
import java.util.Map;

/**
 * Handler for mapping the NetworkIpSpace attributes with the network_ip_space table.
 */
public class NetworkIpSpaceHandler extends XmlHandler {

    public static final String TAG = "NetworkIpSpace";
    public static final String TABLE_NAME = "netapp_model.network_ip_space";

    public NetworkIpSpaceHandler() {
        super(TAG, TABLE_NAME);
    }    

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterId", clusterId)
                .column("NetworkIpSpaceId", long.class);
    }
    
    @Override
    public String alternateKeyToPrimaryKey(Connection connection, OriginatorUpdate originatorUpdate,
            Map<String, String> map) throws OriginatorUpdateException {
        Long originId = originatorUpdate.getOriginId();
        String name = map.get("name");
        List<String> results = Collections.<String> emptyList();

        try {
            StringBuilder sb = new StringBuilder();
            sb.append("SELECT ").append(getKeyAttr());
            sb.append(" FROM netapp_model.network_ip_space WHERE originid = ? AND name = ?");
            
            PreparedStatementWrapper<String> preparedStatement = PreparedStatementWrapper.create(connection, sb.toString(), TableResultParserDefault.STRING);
            preparedStatement.setLong(1, originId);
            preparedStatement.setString(2, name);
            
            results = preparedStatement.executeQueryEx();
        }
        catch (SQLException | TableException e) {
            throw new OriginatorUpdateException(String.format(
                    "Failed to retrieve NetworkIPSapce uuid for originId: %d, and name: %s", originId, name), e);
        }
        
        return (results != null && !results.isEmpty()) ? results.get(0) : null;
    }
}
