/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * Handler for mapping the Network Port Broadcast Domain attributes with the network_port_broadcast_domain table.
 */
public class NetworkPortBroadcastDomainHandler extends XmlHandler {

    public static final String TAG = "NetworkPortBroadcastDomain";
    public static final String TABLE_NAME = "netapp_model.network_port_broadcast_domain";

    public NetworkPortBroadcastDomainHandler() {
        super(TAG, TABLE_NAME);
    }    

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long networkIpSpaceId = originatorUpdate.getObjectIdByTagname("NetworkIpSpace", properties.get("NetworkIpSpaceUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState, false, EnumSet.of(ColumnAttr.STRING_512))
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterId", clusterId)
                .column("MTU", long.class)
                .column("NetworkIpSpaceId", networkIpSpaceId);
    }
}
