/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: Nov 8, 2011
 *         Time: 10:02:01 AM
 */
public class NetworkPortHandler extends XmlHandler {
    
    public static final String TAG = "NetworkPort";
    public static final String TABLE_NAME = "netapp_model.network_port";

    public NetworkPortHandler() {
        super(TAG, TABLE_NAME);
    }    

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long nodeId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(1);
        Long networkPortBroadcastDomainId = originatorUpdate.getObjectIdByTagname("NetworkPortBroadcastDomain", properties.get("NetworkPortBroadcastDomainUUID"));
        Long networkIpSpaceId = originatorUpdate.getObjectIdByTagname("NetworkIpSpace", properties.get("NetworkIpSpaceUUID"));


        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterId", clusterId)
                .column("NodeID", nodeId)
                .column("MACAddress", String.class)
                .column("AdministrativeSpeed", Enum.class)
                .column("AdministrativeSpeedRaw", String.class)
                .column("OperationalSpeed", Enum.class)
                .column("OperationalSpeedRaw", String.class)
                .column("LinkStatus", Enum.class)
                .column("LinkStatusRaw", String.class)
                .column("Role", Enum.class)
                .column("RoleRaw", String.class)
                .column("PortType", Enum.class)
                .column("PortTypeRaw", String.class)
                .column("VLANIdentifier", long.class)
                .column("MTU", long.class)
                .column("OperationalDuplex", Enum.class)
                .column("OperationalDuplexRaw", String.class)
                .column("networkIpSpaceId", networkIpSpaceId)
                .column("networkPortBroadcastDomainId", networkPortBroadcastDomainId)
                .column("HealthStatusRaw", String.class)
                .column("HealthStatus", Enum.class)
                .column("IgnoreHealthStatus", boolean.class)
                .column("HealthDegradedReasonsRaw", String.class)
                .column("HealthDegradedReasons", Enum.class)
                .column("IsAdministrativeUp", boolean.class);
    }
}
