package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

public class NetworkRouteHandler extends XmlHandler {
    public static final String TAG = "NetworkRoute";
    public static final String TABLE_NAME = "netapp_model.network_route";

    public NetworkRouteHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long vserverId = originatorUpdate.getRequiredObjectIdByTagname("Vserver", properties.get("VserverUUID"), getTagName());

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("AddressFamily", Enum.class)
                .column("AddressFamilyRaw", String.class)
                .column("Destination", String.class)
                .column("Gateway", String.class)
                .column("Metric", long.class)
                .column("VserverId", vserverId)
                .column("ClusterId", clusterId);
    }
}
