package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

public class NetworkRouteLifHandler extends XmlHandler {
    public static final String TAG = "NetworkRouteLif";
    public static final String TABLE_NAME = "netapp_model.network_route_lif";

    public NetworkRouteLifHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long lifId = originatorUpdate.getRequiredObjectIdByTagname("NetworkLIF", properties.get("LifUUID"), getTagName());
        Long networkRouteId = originatorUpdate.getRequiredObjectIdByTagname("NetworkRoute", properties.get("NetworkRouteUUID"), getTagName());

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("NetworkRouteId", networkRouteId)
                .column("LifId", lifId);
    }
}
