package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * Handler for mapping the NetworkIpSpace attributes with the network_ip_space table.
 */
public class NetworkSubnetHandler extends XmlHandler {
    public static final String TAG = "NetworkSubnet";
    public static final String TABLE_NAME = "netapp_model.network_subnet";

    public NetworkSubnetHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long broadcastDomainId = originatorUpdate.getObjectIdByTagname(NetworkPortBroadcastDomainHandler.TAG, properties.get("BroadcastDomainUUID"));
        Long ipspaceId = originatorUpdate.getObjectIdByTagname(NetworkIpSpaceHandler.TAG, properties.get("IpspaceUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("SubnetName", String.class)
                .column("ClusterId", clusterId)
                .column("Subnet", String.class)
                .column("BroadcastDomainId", broadcastDomainId)
                .column("IpspaceId", ipspaceId)
                .column("AvailableCount", long.class)
                .column("ForceUpdateLifAssociations", boolean.class)
                .column("Gateway", String.class)
                .column("Ipranges", String.class)
                .column("TotalCount", long.class)
                .column("UsedCount", long.class);

    }
}
