/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;


public class NetworkVlanHandler extends XmlHandler {
    public static final String TABLE_NAME = "netapp_model.network_vlan";
    public static final String TAG_NAME = "NetworkVlan";

    public NetworkVlanHandler() {
        super(TAG_NAME, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState)
            throws OriginatorUpdateException, KeyCacheException {

        Long nodeId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(1);
        Long portId = originatorUpdate.getObjectIdByTagname("NetworkPort", properties.get("NetworkPortUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("NodeId", nodeId)
                .column("PortId", portId)
                .column("VlanID", long.class)
                .column("InterfaceName", String.class)
                .column("IsGvrpEnabled", boolean.class);
    }
}

