/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;


public class NodeHandler extends XmlHandler {
    public static final String TAG = "Node";
    public static final String TABLE_NAME = "netapp_model.node";

    public NodeHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("clusterId", clusterId)
                .column("SerialNumber", String.class,  EnumSet.of(ColumnAttr.NULL_PERSISTS_LAST_VALUE))
                .column("NodeStorageConfiguration", Enum.class)
                .column("NodeStorageConfigurationRaw", String.class)
                .column("Version", String.class)
                .column("Model", String.class)
                .column("Uptime", long.class)
                .column("Vendor", String.class)
                .column("Location", String.class)
                .column("FailoverState", String.class)
                .column("CPUProcessorType", String.class)
                .column("CPUProcessorId", String.class)
                .column("NumberOfProcessors", int.class)
                .column("MemorySizeMB", long.class)
                .column("ProductType", String.class)
                .column("BoardType", String.class)
                .column("NVRAMId", String.class)
                .column("CPUFirmwareRelease", String.class)
                .column("IsOverTemperature", boolean.class)
                .column("FailedFanCount", int.class)
                .column("FailedPowerSupplyCount", int.class)
                .column("NVRAMBatteryStatus", Enum.class)
                .column("NVRAMBatteryStatusRaw", String.class)
                .column("RawDiskBytesTotal", long.class)
                .column("RawDiskBytesUsed", long.class)
                .column("AggregateBytesTotal", long.class)
                .column("AggregateBytesUsed", long.class)
                .column("IsFailoverEnabled", boolean.class)
                .column("IsTakeOverPossible", boolean.class)
                .column("PartnerFirmwareState", String.class)
                .column("LocalFirmwareState", String.class)
                .column("IsInterconnectUp", boolean.class)
                .column("InterconnectLinks", String.class)
                .column("InterconnectType", String.class)
                .column("VersionGeneration", int.class)
                .column("VersionMajor", int.class)
                .column("VersionMinor", int.class)
                .column("Owner", String.class)
                .column("IsNodeHealthy", boolean.class)
                .column("IsEpsilonNode", boolean.class)
                .column("IsAllFlashOptimized", boolean.class)
                .column("EnvFailedFanMessage", String.class)
                .column("EnvFailedPowerSupplyMessage", String.class)
                .column("GiveBackState", String.class)
                .column("CurrentMode", String.class)
                .column("TakeoverByPartnerNotPossibleReason", String.class, EnumSet.of(ColumnAttr.STRING_512))
                .column("TakeoverOfPartnerNotPossibleReason", String.class, EnumSet.of(ColumnAttr.STRING_512))
                .column("TakeoverFailureReason", String.class)
                .column("MtDrPartnerNodeUUID", String.class)
                .column("MtDrOperationState", Enum.class)
                .column("MtDrOperationStateRaw", String.class)
                .column("MtAutomaticUso", boolean.class);
    }

    @Override
    public boolean isObjectOffline(OriginatorUpdate originatorUpdate, Map<String, String> objectProperties) {
        return this.isObjectOffline("IsNodeHealthy", String.valueOf(Boolean.FALSE), objectProperties);
    }


}
