package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

public class NodeLicenseUsageHandler extends XmlHandler {
    public static final String TAG = "NodeLicenseUsage";
    public static final String TABLE_NAME = "netapp_model.node_license_usage";

    public NodeLicenseUsageHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override

    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {

        Long nodeId = originatorUpdate.getRequiredObjectIdByTagname("Node", properties.get("NodeUUID"), getTagName());
        Long licenseV2EntitlementRiskId = originatorUpdate.getRequiredObjectIdByTagname("LicenseV2EntitlementRisk", properties.get("LicenseV2EntitlementRiskUUID"), getTagName());
        Long clusterId = originatorUpdate.peekObjectId(1);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("InUse", boolean.class)
                .column("IsLicensed", boolean.class)
                .column("NodeName", String.class)
                .column("SerialNumber", String.class)
                .column("LicenseV2EntitlementRiskId", licenseV2EntitlementRiskId)
                .column("NodeId", nodeId);
    }
}
