/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

public class ObjectstoreConfigHandler extends XmlHandler {
    public static final String TAG = "ObjectstoreConfig";
    public static final String TABLE_NAME = "netapp_model.objectstore_config";


    public ObjectstoreConfigHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterId", clusterId)
                .column("containerName", String.class)
                .column("Port", int.class)
                .column("Server", String.class)
                .column("AccessKey", String.class)
                .column("ProviderTypeRaw", String.class)
                .column("ProviderType", Enum.class)
                .column("IsSslEnabled", boolean.class)
                .column("IpSpace", String.class)
                .column("AzureAccount", String.class)
                .column("AuthType", String.class);
    }
}
