/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Nancy Walia
 */
public class PlexHandler extends XmlHandler {

    public static final String TAG = "Plex";
    public static final String TABLE_NAME = "netapp_model.plex";

    public PlexHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        long aggregateId = originatorUpdate.peekObjectId();
        long nodeId = originatorUpdate.peekObjectId(1);
        long clusterId = originatorUpdate.peekObjectId(2);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("AggregateId", aggregateId)
                .column("ClusterId", clusterId)
                .column("NodeId", nodeId)
                .column("IsOnline", boolean.class)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Status", Enum.class)
                .column("StatusRaw", String.class)
                .column("ResyncPercent", int.class);
    }
}

