/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.Map;

/**
 * @author Al Velella
 *         Date: Jan 12, 2012
 *         Time: 10:23:06 AM
 */
public class PortsetIgroupRefHandler extends XmlHandler {

    public PortsetIgroupRefHandler() {
        super("PortsetIgroupRef", "netapp_model.portset_to_igroup");
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long portsetId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(2);
        Long igroupId = originatorUpdate.getRequiredObjectIdByTagname("Igroup", properties.get("IgroupUUID"), getTagName());

        // generate internal UUID
        properties.put("UUID", buildCompoundUUID(clusterId, portsetId, igroupId));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ClusterID", clusterId)
                .column("PortsetID", portsetId)
                .column("IgroupID", igroupId);
    }
}
