/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.Map;

/**
 * @author Al Velella
 *         Date: Jan 12, 2012
 *         Time: 10:21:10 AM
 */
public class PortsetPortHandler extends XmlHandler {
    public PortsetPortHandler() {
        super("PortsetPort", "netapp_model.portset_port");
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long portsetId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(2);
        String name = properties.get("Name");
        Long fcpLifId = originatorUpdate.getObjectIdByTagname("FCPLIF", properties.get("FcpLifUUID"));
        Long iScsiPortalGroupId = originatorUpdate.getObjectIdByTagname("ISCSIPortalGroup", properties.get("IScsiPortalGroupUUID"));

        // generate internal UUID
        properties.put("UUID", buildCompoundUUID(clusterId, portsetId, name));
        
        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ClusterID", clusterId)
                .column("PortsetID", portsetId)
                .column("Name", String.class)
                .column("FcpLifID", fcpLifId)
                .column("IScsiPortalGroupID",iScsiPortalGroupId);
    }
}
