/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

public class QosPolicyGroupHandler extends XmlHandler {
    public static final String TAG = "QosPolicyGroup";

    public QosPolicyGroupHandler() {
        super(TAG, "netapp_model.qos_policy_group");
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("PolicyGroup", String.class)
                .column("PgIdentifier", long.class)
                .column("PolicyGroupClass", Enum.class)
                .column("PolicyGroupClassRaw", String.class)
                .column("MaxThroughput", String.class)
                .column("MinThroughput", String.class)
                .column("IsShared", boolean.class)
                .column("IsAdaptive", boolean.class)
                .column("MinIOPsAllocation", String.class)
                .column("EnforceFloor", boolean.class)
                .column("Allocation", Enum.class)
                .column("MinAllocation", Enum.class)
                .column("BlockSize", String.class)
                .column("CapacityUnit", Enum.class);
    }


}
