/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

public class QosServiceCenterHandler extends XmlHandler {
    public static final String TAG = "QosServiceCenter";
    public static final String TABLE_NAME = "netapp_model.qos_service_center";

    public QosServiceCenterHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
            Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long nodeId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(1);
        Long aggregateId = originatorUpdate.getObjectIdByTagname("Aggregate", properties.get("AggregateUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState, false, EnumSet.of(ColumnAttr.STRING_512))
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterID", clusterId)
                .column("NodeID", nodeId)
                .column("AggregateID", aggregateId)
                .column("Type", Enum.class, EnumSet.of(ColumnAttr.REQUIRED));
    }
}
