/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

public class QosWorkloadHandler extends XmlHandler {
    public static final String TAG = "QosWorkload";
    public static final String TABLE_NAME = "netapp_model.qos_workload";
    public static final String CHILD_FOREIGN_KEY = "holderId";

    public QosWorkloadHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
            Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long vserverId = originatorUpdate.getObjectIdByTagname("Vserver", properties.get("VserverUUID"));
        Long volumeId = originatorUpdate.getObjectIdByTagname("Volume", properties.get("VolumeUUID"));
        Long lunId = originatorUpdate.getObjectIdByTagname("LUN", properties.get("LunUUID"));
        Long fileId = originatorUpdate.getObjectIdByTagname("File", properties.get("FileUUID"));

        String holderTypeStr = properties.get("HolderType");

        Long holderId; 
        if(holderTypeStr.equals("LUN")) {
            holderId = lunId;
        } else if (holderTypeStr.equals("VOLUME")) {
            holderId = volumeId;
        } else if (holderTypeStr.equals("FILE")) {
            holderId = fileId;
        } else {
            holderId = null;
        }

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterID", clusterId)
                .column("VserverID", vserverId)
                .column("HolderId", holderId)
                .column("WorkloadClass", Enum.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("WorkloadClassRaw", String.class)
                .column("IsAdaptive", boolean.class)
                .column("HolderType", Enum.class);
    }
}
