/*
 * Copyright (c) 2014 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * Handler for QosWorkloadNodeRelationship TAG, maps the data to the netapp_model.qos_workload_node_relationship table.
 */
public class QosWorkloadNodeRelationshipHandler extends XmlHandler {
    public static final String TAG = "QosWorkloadNodeRelationship";
    public static final String TABLE_NAME = "netapp_model.qos_workload_node_relationship";

    public QosWorkloadNodeRelationshipHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long nodeId = originatorUpdate.getRequiredObjectIdByTagname("Node", properties.get("NodeUUID"), getTagName());
        Long workloadId = originatorUpdate.getRequiredObjectIdByTagname("QosWorkload", properties.get("QosWorkloadUUID"), getTagName());

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState, false, EnumSet.of(ColumnAttr.STRING_512))
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterID", clusterId)
                .column("NodeId", nodeId)
                .column("WorkloadId", workloadId);
    }
}
