/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

/**
 * @author Al Velella
 *         Date: Nov 8, 2011
 *         Time: 10:02:19 AM
 */
public class QtreeHandler extends XmlHandler {

    public static final String TAG = "Qtree";
    public static final String TABLE_NAME = "netapp_model.qtree";

    public QtreeHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long volumeId = originatorUpdate.peekObjectId();
        Long vserverId = originatorUpdate.peekObjectId(1);
        Long clusterId = originatorUpdate.peekObjectId(2);
        Long exportPolicyId = originatorUpdate.getObjectIdByTagname("ExportPolicy", properties.get("ExportPolicyUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("QtreeIdentifier", int.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterId", clusterId)
                .column("VolumeId", volumeId)
                .column("VserverId", vserverId)
                .column("Status", Enum.class)
                .column("StatusRaw", String.class)
                .column("SecurityStyle", Enum.class)
                .column("SecurityStyleRaw", String.class)
                .column("Oplocks", boolean.class)
                .column("ExportPolicyId", exportPolicyId)
                .column("IsExportPolicyInherited", boolean.class)
                .column("Mode", String.class);
    }
}
