/*
 * Copyright (c) 2017 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

public class QuotaPolicyRuleHandler extends XmlHandler {

    public static final String TAG = "QuotaPolicyRule";
    public static final String TABLE_NAME = "netapp_model.quota_policy_rule";

    public QuotaPolicyRuleHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long volumeId = originatorUpdate.peekObjectId();
        Long vserverId = originatorUpdate.peekObjectId(1);
        Long clusterId = originatorUpdate.peekObjectId(2);
        Long qtreeId = originatorUpdate.getObjectIdByTagname("Qtree", properties.get("QtreeUUID"));
        Long quotaPolicyId = originatorUpdate.getObjectIdByTagname("QuotaPolicy", properties.get("QuotaPolicyUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("VserverId", vserverId)
                .column("VolumeId", volumeId)
                .column("qtreeId", qtreeId)
                .column("quotaPolicyId", quotaPolicyId)
                .column("QuotaType", Enum.class)
                .column("QuotaTypeRaw", String.class)
                .column("QuotaTarget", String.class)
                .column("DiskLimit", long.class)
                .column("IsDiskLimitUnlimited", boolean.class)
                .column("FileLimit", long.class)
                .column("IsFileLimitUnlimited", boolean.class)
                .column("SoftDiskLimit", long.class)
                .column("IsSoftDiskLimitUnlimited", boolean.class)
                .column("SoftFileLimit", long.class)
                .column("IsSoftFileLimitUnlimited", boolean.class)
                .column("Threshold", long.class)
                .column("IsThresholdUnlimited", boolean.class)
                .column("IsPerformUserMapping", boolean.class);
    }
}