/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * User: onur
 * Date: Feb 10, 2012
 */
public class QuotaUserHandler extends XmlHandler {

    public static final String TAG = "QuotaUser";
    public static final String TABLE_NAME = "netapp_model.quota_user";

    public QuotaUserHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long quotaId = originatorUpdate.peekObjectId();
        Long vserverId = originatorUpdate.peekObjectId(2);
        Long clusterId = originatorUpdate.peekObjectId(3);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("VserverId", vserverId)
                .column("UserQuotaId", quotaId)
                .column("QuotaUserId", String.class)
                .column("QuotaUserName", String.class)
                .column("QuotaUserType", Enum.class);
    }
}