/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.*;

/**
 * @author Nancy Walia
 */
public class RaidGroupHandler extends XmlHandler {

    public static final String TAG = "RaidGroup";
    public static final String TABLE_NAME = "netapp_model.raid_group";

    public RaidGroupHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        long plexId = originatorUpdate.peekObjectId();
        long aggregateId = originatorUpdate.peekObjectId(1);
        long nodeId = originatorUpdate.peekObjectId(2);
        long clusterId = originatorUpdate.peekObjectId(3);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("PlexId", plexId)
                .column("AggregateId", aggregateId)
                .column("ClusterId", clusterId)
                .column("NodeId", nodeId)
                .column("IsReconstructing", boolean.class)
                .column("RaidType", Enum.class)
                .column("RaidTypeRaw", String.class)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512));
    }

}


