/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: 2/4/13
 *         Time: 11:45 AM
 */
public class RoutingGroupDestinationHandler extends XmlHandler {
    public static final String TAG = "RoutingGroupDestination";
    public static final String TABLE_NAME = "netapp_model.routing_group_destination";

    public RoutingGroupDestinationHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long routingGroupId = originatorUpdate.peekObjectId();
        Long vserverId = originatorUpdate.peekObjectId(1);
        Long clusterId = originatorUpdate.peekObjectId(2);

        // generate internal UUID
        String resourceKey = originatorUpdate.peekObjectUuid() + "." + properties.get("DestinationAddress");
        properties.put("UUID", resourceKey);
        properties.put("ResourceKey", resourceKey);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("VserverId", vserverId)
                .column("RoutingGroupId", routingGroupId)
                .column("DestinationAddress", String.class)
                .column("GatewayAddress", String.class);
    }
}

