/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;


public class SisPolicyHandler extends XmlHandler {
    public static final String TABLE_NAME = "netapp_model.sis_policy";
    public static final String TAG_NAME = "SisPolicy";

    public SisPolicyHandler() {
        super(TAG_NAME, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState)
            throws OriginatorUpdateException, KeyCacheException {

        Long clusterId = originatorUpdate.peekObjectId(1);
        Long vserverId = originatorUpdate.peekObjectId();

        final String jobScheduleUuid = properties.get("JobScheduleUUID");
        Long jobScheduleId = null;
        if (jobScheduleUuid != null) {
            jobScheduleId = originatorUpdate.getObjectIdByTagname(JobScheduleHandler.TAG_NAME, jobScheduleUuid);
        }

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterId", clusterId)
                .column("VserverId", vserverId)
                .column("JobScheduleId", jobScheduleId)
                .column("IsEnabled", boolean.class)
                .column("Comment", String.class)
                .column("Duration", String.class)
                .column("QosPolicy", String.class)
                .column("QosPolicyRaw", String.class)
                .column("PolicyType", String.class)
                .column("PolicyTypeRaw", String.class);
    }


}

