/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * SNAP Lock Handler
 */
public class SnapLockHandler extends XmlHandler {

    public static final String TAG = "SnapLock";
    public static final String TABLE_NAME = "netapp_model.snaplock";

    public SnapLockHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId(2); // Cluster is great grandparent, vserver is grandParent
        Long volumeId = originatorUpdate.peekObjectId(0); // Volume is immediate parent

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("clusterId", clusterId)
                .column("volumeId", volumeId)
                .column("Type", Enum.class)
                .column("TypeRaw", String.class)
                .column("MinimumRetentionPeriod", String.class)
                .column("DefaultRetentionPeriod", String.class)
                .column("MaximumRetentionPeriod", String.class)
                .column("AutoCommitPeriod", String.class)
                .column("VolumeExpiryTime", String.class)
                .column("PrivilegedDeleteState", String.class)
                .column("VolumeComplianceClock", long.class);
    }
}