package com.netapp.oci.netapp.originator.handler;


import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;
import java.util.EnumSet;
import java.util.Map;


public class SnapMirrorHistoryHandler extends XmlHandler {
    public static final String TAG = "SnapMirrorHistory";
    public static final String TABLE_NAME = "netapp_model.snap_mirror_history";

    public SnapMirrorHistoryHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public boolean keepHistory() {
        return true;
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long snapMirrorId = originatorUpdate.getRequiredObjectIdByTagname("SnapMirror", properties.get("SnapMirrorUUID"), getTagName()); 

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("SnapMirrorId", snapMirrorId)
                .column("OperationType", Enum.class)
                .column("OperationTypeRaw", String.class)
                .column("Result", Enum.class)
                .column("ResultRaw", String.class)
                .column("AdditionalInfo", String.class)
                .column("StartTime", long.class)
                .column("EndTime", long.class)
                .column("TransferSize", long.class);
    }
}
