/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.*;

/**
 * @author Shan Ponnusamy
 */
public class SnapMirrorPolicyHandler extends XmlHandler {

    public static final String TAG_NAME = "SnapMirrorPolicy";

    private static final String TABLE_NAME = "netapp_model.snap_mirror_policy";

    public SnapMirrorPolicyHandler() {
        super(TAG_NAME, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState)
            throws OriginatorUpdateException, KeyCacheException {

        Long clusterId = originatorUpdate.peekObjectId();

        Long vserverId = null;
        final String vserverUuid = properties.get("VserverUUID");
        if (vserverUuid != null) {
            vserverId = originatorUpdate.getObjectIdByTagname("Vserver", vserverUuid);
        }

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("VserverId", vserverId)
                .column("Name", String.class)
                .column("Comment", String.class)
                .column("CreateSnapshot", boolean.class)
                .column("IgnoreAccessTime", boolean.class)
                .column("IsNetworkCompressionEnabled", boolean.class)
                .column("Restart", Enum.class)
                .column("RestartRaw", String.class)
                .column("TotalKeep", long.class)
                .column("TransferPriority", Enum.class)
                .column("TransferPriorityRaw", String.class)
                .column("Tries", String.class)
                .column("Type", Enum.class)
                .column("TypeRaw", Enum.class);
    }

}
