/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Shan Ponnusamy
 */
public class SnapMirrorPolicyRuleHandler extends XmlHandler {

    public static final String TAG_NAME = "SnapMirrorPolicyRule";

    private static final String TABLE_NAME = "netapp_model.snap_mirror_policy_rule";

    public SnapMirrorPolicyRuleHandler() {
        super(TAG_NAME, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState)
            throws OriginatorUpdateException, KeyCacheException {

        Long clusterId = originatorUpdate.peekObjectId(1);
        Long snapMirrorPolicyId = originatorUpdate.peekObjectId();

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("SnapMirrorPolicyId", snapMirrorPolicyId)
                .column("Keep", String.class)
                .column("Preserve", boolean.class)
                .column("SnapMirrorLabel", String.class)
                .column("Warn", String.class);
    }
}
