/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.sql.Timestamp;
import java.util.*;

/**
 *  @author Shan Ponnusamy
 */

public class SnapshotHandler extends XmlHandler {
    public static final String TABLE_NAME = "netapp_model.snapshot";
    public static final String TAG_NAME = "Snapshot";

    public SnapshotHandler() {
        super(TAG_NAME, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState)
            throws OriginatorUpdateException, KeyCacheException {

        Long vserverId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(1);

        final String volumeUuid = properties.get("VolumeUUID");
        long volumeId = originatorUpdate.getRequiredObjectIdByTagname("Volume", volumeUuid, getTagName());

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("VolumeId", volumeId)
                .column("VserverId", vserverId)
                .column("ClusterId", clusterId)
                .column("InstanceUUID", String.class)
                .column("VersionUUID", String.class)
                .column("IsBusy", boolean.class)
                .column("AccessTime", Timestamp.class)
                .column("Dependency", String.class)
                .column("PercentageOfTotalBlocks", int.class)
                .column("CumulativePercentageOfTotalBlocks", int.class);                
    }


}
