/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: Jan 12, 2012
 *         Time: 3:40:46 PM
 */
public class StorageAdapterHandler extends XmlHandler {
    public static final String TAG = "StorageAdapter";
    public static final String TABLE_NAME = "netapp_model.storage_adapter";

    public StorageAdapterHandler() {
        super(TAG,TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long nodeId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(1);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterID", clusterId)
                .column("NodeID", nodeId)
                .column("Name", String.class)
                .column("FcNodeName", String.class)
                .column("FcPortName", String.class)
                .column("IsInUse", boolean.class)
                .column("Model", String.class)
                .column("FcLinkRate", int.class)
                .column("Status", Enum.class)
                .column("AdapterType", Enum.class)
                .column("State", Enum.class)
                .column("StateRaw", String.class)
                .column("CableEndIdentifier", String.class)
                .column("CableLength", String.class)
                .column("CableManufacturer", String.class)
                .column("CablePartNumber", String.class)
                .column("CableSerialNumber", String.class)
                .column("AttachedSerialNumber", String.class)
                .column("CableTechnology", String.class);
    }
}
