/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

public class StoragePoolAvailableSizeHandler extends XmlHandler {
    public static final String TAG = "StoragePoolAvailableSize";
    public static final String TABLE_NAME = "netapp_model.storage_pool_available";


    public StoragePoolAvailableSizeHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long storagePoolId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(1);
        Long nodeId = originatorUpdate.getRequiredObjectIdByTagname("Node", properties.get("NodeUUID"), getTagName());


        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("NodeId", nodeId)
                .column("StoragePoolId", storagePoolId)
                .column("AllocationUnitCount", long.class)
                .column("AllocationUnitSize", long.class)
                .column("AvailableSize", long.class)
                .column("StorageType", Enum.class)
                .column("StorageTypeRaw", String.class)
                .column("SyncmirrorPool", String.class);
    }
}
