/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.*;

public class StorageServiceHandler extends XmlHandler {

    public static final String TAG = "StorageService";

    public StorageServiceHandler() {
        super(TAG, "netapp_model.storage_service");
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {

        Long vserverId = originatorUpdate.peekObjectId();
        Long clusterId = originatorUpdate.peekObjectId(1);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                        .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                        .column("StorageServiceIdentifier", long.class, EnumSet.of(ColumnAttr.REQUIRED))
                        .column("ClusterId", clusterId)
                        .column("VserverId", vserverId)
                        .column("StorageServiceName", String.class)
                        .column("SizeUsed", long.class)
                        .column("SizeTotal", long.class)
                        .column("SizeUsedBySnapshots", long.class)
                        .column("SnapshotReserveSize", long.class)
                        .column("DeduplicationSpaceSaved", long.class)
                        .column("CompressionSpaceSaved", long.class);
    }


}
