/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.Map;

public class StorageServiceVolumeRelationshipHandler extends XmlHandler {
    private static final String TABLE_NAME = "netapp_model.storage_service_volume_relationship";
    private static final String TAG_NAME = "StorageServiceVolumeRelationship";

    public StorageServiceVolumeRelationshipHandler() {
        super(TAG_NAME, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState)
            throws OriginatorUpdateException, KeyCacheException {

        final String storageServiceUUID = properties.get("StorageServiceUUID");
        Long storageServiceId = originatorUpdate.getObjectIdByTagname(StorageServiceHandler.TAG, storageServiceUUID);
        final String volumeUUID = properties.get("VolumeUUID");
        Long volumeId = originatorUpdate.getObjectIdByTagname(VolumeHandler.TAG, volumeUUID);

        // construct the UUID from storage service for table consistency.
        properties.put("UUID", storageServiceUUID);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("StorageServiceId", storageServiceId)
                .column("VolumeId", volumeId);
    }
}