package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Iti Behari
 *         Date: 7/23/13
 */
public class StorageShelfCableHandler extends XmlHandler {

    public static final String TAG = "StorageShelfCable";
    public static final String TABLE_NAME = "netapp_model.storage_shelf_cable";

    public StorageShelfCableHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long shelfId = originatorUpdate.peekObjectId();
        Long nodeId = originatorUpdate.getObjectIdByTagname("Node", properties.get("NodeUUID"));
        Long clusterId = originatorUpdate.peekObjectId(1);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("NodeId", nodeId)
                .column("ShelfId", shelfId)
                .column("ChannelName", String.class)
                .column("IsShelfChannelFailure", boolean.class)
                .column("EndIdentifier", String.class)
                .column("Length", String.class)
                .column("Manufacturer", String.class)
                .column("PartNumber", String.class)
                .column("SerialNumber", String.class)
                .column("AttachedSerialNumber", String.class)
                .column("Technology", String.class)
                .column("Type", String.class)
                .column("ConnectorDesignator", String.class)
                .column("ConnectorNumber", int.class)
                .column("IsCableConnected", boolean.class)
                .column("IsConnectorError", boolean.class)
                .column("SwapCount", int.class)
                .column("ShelfModule", String.class);
    }
}

