/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: 1/25/13
 *         Time: 11:48 AM
 */
public class StorageShelfHandler extends XmlHandler {

    public static final String TAG = "StorageShelf";
    public static final String TABLE_NAME = "netapp_model.storage_shelf";

    public StorageShelfHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long nodeId = originatorUpdate.getRequiredObjectIdByTagname("Node", properties.get("NodeUUID"), TAG);
        Long partnerNodeId = originatorUpdate.getObjectIdByTagname("Node", properties.get("PartnerNodeUUID"));
        Long clusterId = originatorUpdate.peekObjectId(0);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("NodeId", nodeId)
                .column("PartnerNodeId", partnerNodeId)
                .column("ShelfUUID", String.class)
                .column("ShelfIdentifier", int.class)
                .column("SESProductID", String.class)
                .column("FirmwareRevisionA", String.class)
                .column("FirmwareRevisionB", String.class)
                .column("Module", String.class)
                .column("ChannelName", String.class)
                .column("PartnerChannelName", String.class)
                .column("ShelfType", String.class)
                .column("State", Enum.class)
                .column("StateRaw", String.class)
                .column("StackId",String.class)
                .column("IsLocalAttach", boolean.class)
                .column("TemperatureSensorCount", int.class)
                .column("TemperatureSensorFailedCount", int.class)
                .column("CoolingElementCount", int.class)
                .column("CoolingElementFailedCount", int.class)
                .column("PowerSupplyCount", int.class)
                .column("PowerSupplyFailedCount", int.class)
                .column("CurrentSensorCount", int.class)
                .column("CurrentSensorFailedCount", int.class)
                .column("VoltageSensorCount", int.class)
                .column("VoltageSensorFailedCount", int.class);
    }
}
