/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: 10/21/13
 *         Time: 1:10 PM
 */
public class StorageSwitchHandler extends XmlHandler {
    public static final String TAG = "StorageSwitch";
    public static final String TABLE_NAME = "netapp_model.storage_switch";

    public StorageSwitchHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterId", clusterId)
                .column("WWN", String.class)
                .column("Vendor", String.class)
                .column("FirmwareVersion", String.class)
                .column("IPAddress", String.class)
                .column("Model", String.class)
                .column("Role", Enum.class)
                .column("RoleRaw", String.class)
                .column("Status", Enum.class)
                .column("StatusRaw", String.class)
                .column("SensorCount", int.class)
                .column("PowerSupplyCount", int.class)
                .column("FanCount", int.class);
    }
}
