package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.sql.Timestamp;
import java.util.EnumSet;
import java.util.Map;

/**
 * Created with IntelliJ IDEA.
 * User: jianwei
 * Date: 1/10/14
 * Time: 4:40 PM
 * To change this template use File | Settings | File Templates.
 */
public class SystemHealthAlertHandler extends XmlHandler {

    public static final String TAG = "SystemHealthAlert";
    public static final String TABLE_NAME = "netapp_model.system_health_alert";


    public SystemHealthAlertHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long nodeId = originatorUpdate.getRequiredObjectIdByTagname("Node",  properties.get("NodeUUID"), getTagName());

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState,  false, EnumSet.of(ColumnAttr.STRING_512))
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("AlertId", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("AlertingResource", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("AlertingResourceName", String.class)
                .column("IndicationTime", Timestamp.class)
                .column("ClusterId", clusterId)
                .column("NodeId", nodeId)
                .column("Monitor", Enum.class)
                .column("MonitorRaw", String.class)
                .column("PerceivedSeverity", String.class)
                .column("Policy", String.class)
                .column("PossibleEffect", String.class, EnumSet.of(ColumnAttr.STRING_512))
                .column("CorrectiveActions", String.class, EnumSet.of(ColumnAttr.TEXT))
                .column("ProbableCause", String.class, EnumSet.of(ColumnAttr.STRING_512))
                .column("ProbableCauseDescription", String.class, EnumSet.of(ColumnAttr.TEXT))
                .column("Subsystem", String.class)
                .column("AdditionalInfo", String.class, EnumSet.of(ColumnAttr.TEXT))
                .column("Acknowledge", boolean.class)
                .column("Acknowledger", String.class)
                .column("Suppress", boolean.class)
                .column("Suppressor", String.class);
    }
}
