/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.EnumSet;
import java.util.Map;

/**
 * User: onur
 * Date: Feb 10, 2012
 */
public class TreeQuotaHandler extends XmlHandler {

    public static final String TAG = "TreeQuota";
    public static final String TABLE_NAME = "netapp_model.qtree_quota";

    public TreeQuotaHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long volumeId = originatorUpdate.peekObjectId();
        Long vserverId = originatorUpdate.peekObjectId(1);
        Long clusterId = originatorUpdate.peekObjectId(2);
        Long qtreeId = originatorUpdate.getObjectIdByTagname("Qtree", properties.get("QtreeUUID"));


        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("VserverId", vserverId)
                .column("VolumeId", volumeId)
                .column("QtreeId", qtreeId)
                .column("QuotaTarget", String.class)
                .column("QuotaType", Enum.class)
                .column("DiskLimit", long.class)
                .column("DiskLimitUnlimited", boolean.class)
                .column("DiskUsed", long.class)
                .column("DiskUsedUnlimited", boolean.class)
                .column("FileLimit", long.class)
                .column("FileLimitUnlimited", boolean.class)
                .column("FileUsed", long.class)
                .column("FileUsedUnlimited", boolean.class)
                .column("SoftDiskLimit", long.class)
                .column("SoftDiskLimitUnlimited", boolean.class)
                .column("SoftFileLimit", long.class)
                .column("SoftFileLimitUnlimited", boolean.class)
                .column("Threshold", long.class)
                .column("ThresholdUnlimited", boolean.class);
    }
}
