package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;
import java.util.EnumSet;
import java.util.Map;


public class VServerPeerHandler extends XmlHandler {
    public static final String TAG = "VServerPeer";
    public static final String TABLE_NAME = "netapp_model.vserver_peer";

    public VServerPeerHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long vServerId = originatorUpdate.getObjectIdByTagname("Vserver", properties.get("VServerUUID")); 

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("ClusterId", clusterId)
                .column("VServerId", vServerId)
                .column("PeerVServerUUID", String.class)
                .column("PeerClusterName", String.class)
                .column("PeerVServerName", String.class)
                .column("PeerState", Enum.class)
                .column("PeerStateRaw", String.class)
                .column("Applications", String.class, EnumSet.of(ColumnAttr.TEXT));
    }
}

