/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.Map;

/**
 * @author Jason Sprague
 *         Date: Jul 23, 2012
 *         Time: 8:30:00 AM
 */
public class VolumeFlexCacheHandler extends XmlHandler {
    public static final String TAG = "VolumeFlexCache";
    public static final String TABLE_NAME = "netapp_model.volume_flexcache";

    public VolumeFlexCacheHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        // generate internal UUID
        final String flexCacheUUID = properties.get("VolumeFlexCacheUUID");
        final String originUUID  = properties.get("VolumeOriginUUID");
        properties.put("UUID", buildCompoundUUID(flexCacheUUID, originUUID));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("flexCacheId", originatorUpdate.getRequiredObjectIdByTagname("Volume", flexCacheUUID, getTagName()))
                .column("originVolumeId", originatorUpdate.getRequiredObjectIdByTagname("Volume", originUUID, getTagName()));
    }
}
