/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Shan Ponnusamy
 */
public class VolumeMoveJobHandler extends XmlHandler {
    private static final String TABLE_NAME = "netapp_model.volume_move_job";
    private static final String TAG_NAME = "VolumeMoveJob";

    public VolumeMoveJobHandler() {
        super(TAG_NAME, TABLE_NAME);
    }
    
    @Override
    public boolean keepHistory() {
        return true;
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState)
            throws OriginatorUpdateException, KeyCacheException {

        Long clusterId = originatorUpdate.peekObjectId(1);

        final String volumeUuid = properties.get("VolumeUUID");
        long volumeId = originatorUpdate.getRequiredObjectIdByTagname("Volume", volumeUuid, TAG_NAME);

        final String sourceAggregateUuid = properties.get("SourceAggregateUUID");
        long sourceAggregateId = originatorUpdate.getRequiredObjectIdByTagname("Aggregate", sourceAggregateUuid, TAG_NAME);

        final String destinationAggregateUuid = properties.get("DestinationAggregateUUID");
        long destinationAggregateId = originatorUpdate.getRequiredObjectIdByTagname("Aggregate", destinationAggregateUuid, TAG_NAME);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("VolumeId", volumeId)
                .column("SourceAggregateId", sourceAggregateId)
                .column("DestinationAggregateId", destinationAggregateId)
                .column("ClusterId", clusterId)
                .column("JobIdentifier", long.class)                
                .column("StartTime", Timestamp.class)
                .column("CompletionTime", Timestamp.class)
                .column("Phase", Enum.class)
                .column("PhaseRaw", String.class)
                .column("State", Enum.class)
                .column("StateRaw", String.class)
                .column("Details", String.class)
                .column("PercentComplete", int.class)
                .column("EstimatedCompletionTime", Timestamp.class);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void adjustDeletedObjects (Connection connection, Map<String, Set<Long>> deletedObjs, Map<String, Set<Long>> deletedHistoryObjs) throws OriginatorUpdateException { 
        Set<Long> deletedVolumeIds = deletedObjs.get(VolumeHandler.TAG);
        
        if (deletedVolumeIds == null || deletedVolumeIds.isEmpty()) {
        	return;
        }
        
        StringBuilder sb = new StringBuilder();
        sb.append("SELECT objid FROM netapp_model.volume_move_job WHERE volumeId IN (");
        for (int i = 0; i < deletedVolumeIds.size(); i++) {
        	if (i > 0) {
        		sb.append(", ");
        	}
        	sb.append("?");
        }
        sb.append(")");
        
        try {
        	PreparedStatementWrapper<Long> preparedStatement = PreparedStatementWrapper.create(connection, sb.toString(), TableResultParserDefault.LONG);
        	int i = 1;
        	for (Long id : deletedVolumeIds) {
        		 preparedStatement.setLong(i++, id);
        	}
        	
        	List<Long> volumeMoveJobObjIds = preparedStatement.executeQueryEx();
        	Set<Long> deletedVolumeMoveSet = deletedObjs.get(VolumeMoveJobHandler.TAG_NAME);
        	Set<Long> deletedHistoryVolumeMoveSet = deletedHistoryObjs.get(VolumeMoveJobHandler.TAG_NAME);
        	
			for (Long id : volumeMoveJobObjIds) {
				if (deletedHistoryVolumeMoveSet != null) {
					deletedHistoryVolumeMoveSet.remove(id);
				}
				if (deletedVolumeMoveSet == null) {
					deletedVolumeMoveSet = new HashSet<Long>();
					deletedObjs.put(VolumeMoveJobHandler.TAG_NAME, deletedVolumeMoveSet);
				}
				deletedVolumeMoveSet.add(id);
			}
        	
        }
        catch (SQLException | TableException e) {
            throw new OriginatorUpdateException("Failed to adjust deleted objects", e);
        }
        
    }
    
}
