/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.Map;

/**
 * @author Shan Ponnusamy
 */
public class VserverAggregateRelationshipHandler extends XmlHandler {
    public static final String TABLE_NAME = "netapp_model.vserver_to_aggregate";
    public static final String TAG = "VserverAggregateRelationship";

    public VserverAggregateRelationshipHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState)
            throws OriginatorUpdateException, KeyCacheException {

        Long clusterId = originatorUpdate.peekObjectId();

        final String vserverUuid = properties.get("VserverUUID");
        Long vserverId = originatorUpdate.getObjectIdByTagname(VserverHandler.TAG, vserverUuid);
        final String aggregateUuid = properties.get("AggregateUUID");
        Long aggregateId = originatorUpdate.getObjectIdByTagname(AggregateHandler.TAG, aggregateUuid);

        // construct the UUID artificially
        String uuid = vserverUuid + "." + aggregateUuid;
        properties.put("UUID", uuid);

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ClusterId", clusterId)
                .column("VserverId", vserverId)
                .column("AggregateId", aggregateId);
    }
}
