/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.originator.handler;

import java.util.EnumSet;
import java.util.Map;

import com.netapp.oci.platform.originator.ObjState;
import com.netapp.oci.platform.originator.OriginatorUpdate;
import com.netapp.oci.platform.originator.OriginatorUpdateException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.HandlerResult;
import com.netapp.oci.platform.originator.handler.XmlHandler;
import com.netapp.oci.platform.originator.key.KeyCacheException;
import com.netapp.oci.platform.originator.key.UuidKey;

/**
 * @author Al Velella
 *         Date: Sep 22, 2011
 *         Time: 12:00:29 PM
 */
public class VserverHandler extends XmlHandler {
    public static final String TAG = "Vserver";
    public static final String TABLE_NAME = "netapp_model.vserver";
    public static final String CHILD_FOREIGN_KEY = "vserverId";

    public VserverHandler() {
        super(TAG, TABLE_NAME);
    }

    @Override
    public HandlerResult parseProperties(OriginatorUpdate originatorUpdate, Map<String, String> properties,
                                         Map<UuidKey, ObjState> keyToObjState) throws OriginatorUpdateException, KeyCacheException {
        Long clusterId = originatorUpdate.peekObjectId();
        Long snapshotPolicyId = originatorUpdate.getObjectIdByTagname("SnapshotPolicy", properties.get("SnapshotPolicyUUID"));
        Long qosPolicyGroupId = originatorUpdate.getObjectIdByTagname("QosPolicyGroup", properties.get("QosPolicyGroupUUID"));
        Long networkIpSpaceId = originatorUpdate.getObjectIdByTagname("NetworkIpSpace", properties.get("NetworkIpSpaceUUID"));
        Long quotaPolicyId = originatorUpdate.getObjectIdByTagname("QuotaPolicy", properties.get("QuotaPolicyUUID"));

        return new HandlerResult(originatorUpdate, this, properties, "UUID", keyToObjState)
                .column("ResourceKey", String.class, EnumSet.of(ColumnAttr.REQUIRED, ColumnAttr.STRING_512))
                .column("Name", String.class, EnumSet.of(ColumnAttr.REQUIRED))
                .column("ClusterId", clusterId)
                .column("SnapshotPolicyId", snapshotPolicyId)
                .column("QuotaPolicyId", quotaPolicyId)
                .column("QosPolicyGroupId", qosPolicyGroupId)
                .column("State", Enum.class)
                .column("StateRaw", String.class)
                .column("OperationalState", Enum.class)
                .column("OperationalStateRaw", String.class)
                .column("Type", Enum.class)
                .column("TypeRaw", String.class)
                .column("Subtype", Enum.class)
                .column("SubtypeRaw", String.class)
                .column("AllowedProtocols", String.class)
                .column("DisallowedProtocols", String.class)
                .column("NetworkIpSpaceId", networkIpSpaceId)
                .column("NFSDefaultWindowsUser", String.class)
                .column("NFSChownMode", Enum.class)
                .column("NFSChownModeRaw", String.class)
                .column("NFSV4IdDomain", String.class)
                .column("NFSV4GraceSeconds", long.class)
                .column("NFSMountdPort", long.class)
                .column("NFSNlmPort", long.class)
                .column("NFSNsmPort", long.class)
                .column("NFSEnabled", boolean.class)
                .column("NFSRootonlyEnabled", boolean.class)
                .column("NFSShowmount", boolean.class)
                .column("NFSVstorageEnabled", boolean.class)
                .column("NFSV2Supported", boolean.class)
                .column("NFSV3Supported", boolean.class)
                .column("NFSV4Supported", boolean.class)
                .column("NFSV41Supported", boolean.class)
                .column("NFSV41PnfsEnabled", boolean.class)
                .column("NFS41AclEnabled", boolean.class)
                .column("NFSV4ReadDelegationEnabled", boolean.class)
                .column("NFSV4WriteDelegationEnabled", boolean.class)
                .column("NFSV4MigrationEnabled", boolean.class)
                .column("NFSV4ReferralsEnabled", boolean.class)
                .column("NFSV4AclEnabled", boolean.class)
                .column("NFSV4NumericIdsEnabled", boolean.class)
                .column("NFSV41MigrationEnabled", boolean.class)
                .column("NFSV41ReferralsEnabled", boolean.class)
                .column("NFSV41AclPreserveEnabled", boolean.class)
                .column("NFSV41ReadDelegationEnabled", boolean.class)
                .column("NFSV41WriteDelegationEnabled", boolean.class)
                .column("CIFSEnabled", boolean.class)
                .column("CIFSServer", String.class)
                .column("CIFSAuthenticationStyle", Enum.class)
                .column("CIFSAuthenticationStyleRaw", String.class)
                .column("CIFSDomain", String.class)
                .column("ExportPolicyEnabled", boolean.class)
                .column("DefaultUnixUser", String.class)
                .column("DefaultUnixGroup", String.class)
                .column("ISCSIEnabled", boolean.class)
                .column("ISCSINodeName", String.class)
                .column("ISCSIAliasName", String.class)
                .column("NVMeEnabled", boolean.class)
                .column("FCPEnabled", boolean.class)
                .column("FCPNodeName", String.class)
                .column("NISEnabled", boolean.class)
                .column("ActiveNISDomainName", String.class)
                .column("ActiveNISServers", String.class, EnumSet.of(ColumnAttr.TEXT))
                .column("DNSEnabled", boolean.class)
                .column("DNSServers", String.class)
                .column("DNSDomainNames", String.class)
                .column("DNSTimeout", long.class)
                .column("DNSAttempts", long.class)
                .column("DomainName", String.class)
                .column("IsEnabled", boolean.class)
                .column("UseSecure", boolean.class)
                .column("Ttl", long.class)
                .column("MaximumVolumes", int.class)
                .column("Language", String.class)
                .column("NameServerSwitch", String.class)
                .column("VolumeBytesUsed", long.class)
                .column("VolumeBytesAvail", long.class)
                .column("VolumeBytesTotal", long.class)
                .column("IsRepository", boolean.class)
                .column("IsConfigLockedForChanges", boolean.class)
                .column("LdapClientEnabled", boolean.class)
                .column("LdapClientConfig", String.class)
                .column("IsKerberosEnabled", boolean.class)
                .column("IsSmbEncryptionRequired", boolean.class)
                .column("CifsWorkgroup", String.class)
                .column("RootVolumeName", String.class);
    }
}
