/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance;

import com.netapp.oci.netapp.Param;
import com.netapp.oci.platform.common.param.ParamException;
import com.netapp.oci.platform.framework.performance.tables.PerformanceTableUtils;
import org.slf4j.Logger;

/**
 * @author Anton Sergeev
 */
public class PerformanceConstants {
    /**
     * Performance database name
     */
    public static String NETAPP_PERFORMANCE_DATABASE = "netapp_performance";

    /**
     * Prefix for performance package name to use in datasource registry and for logger
     */
    public static final String PACKAGE_PREFIX = "com.netapp.oci.netapp.performance.";
    
    public static final String RAW_PACKAGE_SUFFIX = "_raw";

    /**
     * Prefix for query performance package logging
     */
    public static final String QUERY_PACKAGE_LOGGER = "com.netapp.oci.netapp.query.performance";

    /**
     * Max elapsed time in milliseconds that the sample listener can process the samples.
     * A warning message is logged when the sample processing time exceeds this limit. 
     * Is 100 millis a proper number or should we make it configurable?
     */
    public static final long SAMPLE_LISTENER_MAX_ELAPSED_TIME = 100; 
    
    public static String SAMPLE_RAW_PACKAGE_PATTERN = ".+_raw$";
    
    private static org.slf4j.Logger logger =org.slf4j.LoggerFactory.getLogger(PerformanceConstants.class);
    
    /**
     * Get the number of days to keep performance data for a given {@code Param.Key}.
     *
     * If no value is set or if an invalid value is set, then returns its default value.
     */
    public static int getTablesLifeSpan(Param.Key key) {
        try {
            int tablesLifespan = Param.get(key, int.class);
            if (tablesLifespan < 1) {
                logger.warn("Invalid value was set for param '" + key.name() + "'; set value is: " + tablesLifespan
                        + ".  Using default value for the param; default value is: " + getDefaultTablesLifeSpan(key));
                return getDefaultTablesLifeSpan(key);
            }
            return tablesLifespan;
        }
        catch (ParamException e) {
            return getDefaultTablesLifeSpan(key);
        }
    }

    private static int getDefaultTablesLifeSpan(Param.Key key) {
        switch (key) {
            case SAMPLE_TABLES_LIFESPAN_DAYS:
                return PerformanceTableUtils.DEFAULT_SAMPLE_TABLES_LIFESPAN_DAYS;
            case SAMPLE_RAW_TABLES_LIFESPAN_DAYS:
                return PerformanceTableUtils.DEFAULT_SAMPLE_RAW_TABLES_LIFESPAN_DAYS;           
            case SUMMARY_TABLES_LIFESPAN_DAYS:
                return PerformanceTableUtils.DEFAULT_SUMMARY_TABLES_LIFESPAN_DAYS;
            case UPDATE_HISTORY_LIFESPAN_DAYS:
                return PerformanceTableUtils.DEFAULT_UPDATE_HISTORY_LIFESPAN_DAYS;
            default:
                logger.warn("The default life span is not defined for param '" + key.name());
                return Integer.MAX_VALUE;
        }
    }
}
