/*
 * $Id$
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.oci.netapp.performance;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

import javax.transaction.SystemException;
import javax.transaction.UserTransaction;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.netapp.oci.netapp.performance.aggregate.interfaces.impl.AggregatePerformanceBean;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.impl.CifsVserverPerformanceBean;
import com.netapp.oci.netapp.performance.cluster.interfaces.impl.ClusterPerformanceBean;
import com.netapp.oci.netapp.performance.disk.interfaces.impl.DiskPerformanceBean;
import com.netapp.oci.netapp.performance.extcacheobj.interfaces.impl.ExtCacheObjPerformanceBean;
import com.netapp.oci.netapp.performance.nvmffclif.interfaces.impl.NvmfFcLifPerformanceBean;
import com.netapp.oci.netapp.performance.nvmffclif.interfaces.impl.NvmfFcLifVserverPerformanceBean;
import com.netapp.oci.netapp.performance.fcplif.interfaces.impl.FcpLifPerformanceBean;
import com.netapp.oci.netapp.performance.fcplif.interfaces.impl.FcpLifVserverPerformanceBean;
import com.netapp.oci.netapp.performance.fcpport.interfaces.impl.FcpPortPerformanceBean;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.impl.IscsiLifPerformanceBean;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.impl.IscsiLifVserverPerformanceBean;
import com.netapp.oci.netapp.performance.lun.interfaces.impl.LunPerformanceBean;
import com.netapp.oci.netapp.performance.namespace.interfaces.impl.NamespacePerformanceBean;
import com.netapp.oci.netapp.performance.networklif.interfaces.impl.NetworkLifPerformanceBean;
import com.netapp.oci.netapp.performance.networklif.interfaces.impl.NetworkLifVserverPerformanceBean;
import com.netapp.oci.netapp.performance.networkport.interfaces.impl.NetworkPortPerformanceBean;
import com.netapp.oci.netapp.performance.nfs.interfaces.impl.NfsPerformanceBean;
import com.netapp.oci.netapp.performance.nfs.interfaces.impl.Nfsv3PerformanceBean;
import com.netapp.oci.netapp.performance.nfs.interfaces.impl.Nfsv41PerformanceBean;
import com.netapp.oci.netapp.performance.nfs.interfaces.impl.Nfsv4PerformanceBean;
import com.netapp.oci.netapp.performance.nic.interfaces.impl.NicPerformanceBean;
import com.netapp.oci.netapp.performance.node.interfaces.impl.NodePerformanceBean;
import com.netapp.oci.netapp.performance.opmheadroom.interfaces.impl.OpmHeadRoomAggrPerformanceBean;
import com.netapp.oci.netapp.performance.opmheadroom.interfaces.impl.OpmHeadRoomCPUPerformanceBean;
import com.netapp.oci.netapp.performance.processor.interfaces.impl.ProcessorPerformanceBean;
import com.netapp.oci.netapp.performance.qosservicecenter.interfaces.impl.QosServiceCenterPerformanceBean;
import com.netapp.oci.netapp.performance.qosworkload.interfaces.impl.QosFileWorkloadPerformanceBean;
import com.netapp.oci.netapp.performance.qosworkload.interfaces.impl.QosLunWorkloadPerformanceBean;
import com.netapp.oci.netapp.performance.qosworkload.interfaces.impl.QosVolumeWorkloadPerformanceBean;
import com.netapp.oci.netapp.performance.qosworkloaddblade.interfaces.impl.QosWorkloadQueueDBladePerformanceBean;
import com.netapp.oci.netapp.performance.qosworkloaddetail.interfaces.impl.QosWorkloadDetailPerformanceBean;
import com.netapp.oci.netapp.performance.qosworkloadnblade.interfaces.impl.QosWorkloadQueueNBladePerformanceBean;
import com.netapp.oci.netapp.performance.resourceheadroom.interfaces.impl.ResourceHeadRoomAggrBean;
import com.netapp.oci.netapp.performance.resourceheadroom.interfaces.impl.ResourceHeadRoomCPUBean;
import com.netapp.oci.netapp.performance.shared.AbstractPerformanceComponentsFactory;
import com.netapp.oci.netapp.performance.shared.ComponentsFactoryRepository;
import com.netapp.oci.netapp.performance.vserver.interfaces.impl.VserverPerformanceBean;
import com.netapp.oci.netapp.performance.wafl.interfaces.impl.WaflPerformanceBean;
import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.framework.performance.util.AbstractTableManagement;

public class SummaryCalculator {

    private enum PerformancePackages {

        AGGREGATE(AggregatePerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getAggregatePerformanceComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        CIFS_VSERVER(CifsVserverPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getCifsVserverPerformanceComponentsFactory()),
        NETWORK_LIF_VSERVER(NetworkLifVserverPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getLifVserverPerformanceComponentsFactory()),
        NFSV3(Nfsv3PerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getNfsv3PerformanceComponentsFactory()),
        NFSV4(Nfsv4PerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getNfsv4PerformanceComponentsFactory()),
        NFSV41(Nfsv41PerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getNfsv41PerformanceComponentsFactory()),
        NFS(NfsPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getNfsPerformanceComponentsFactory()),
        DISK(DiskPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getDiskPerformanceComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        NODE(NodePerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getNodePerformanceComponentsFactory()),
        NVMF_FC_LIF(NvmfFcLifPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getNvmfFcLifPerformanceComponentsFactory()),
        NVMF_FC_LIF_VSERVER(NvmfFcLifVserverPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getNvmfFcLifVserverPerformanceComponentsFactory()),
        FCP_LIF(FcpLifPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getFcpLifPerformanceComponentsFactory()),
        FCP_LIF_VSERVER(FcpLifVserverPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getFcpLifVserverPerformanceComponentsFactory()),
        ISCSI_LIF(IscsiLifPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getIscsiLifPerformanceComponentsFactory()),
        ISCSI_LIF_VSERVER(IscsiLifVserverPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getIscsiLifVserverPerformanceComponentsFactory()),
        NETWORK_LIF(NetworkLifPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getNetworkLifPerformanceComponentsFactory()),
        NETWORK_PORT(NetworkPortPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getNetworkPortPerformanceComponentsFactory()),
        NIC(NicPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getNicPerformanceComponentsFactory()),
        FCP_PORT(FcpPortPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getFcpPortPerformanceComponentsFactory()),
        VSERVER(VserverPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getVserverPerformanceComponentsFactory()),
        PROCESSOR(ProcessorPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getProcessorPerformanceComponentsFactory()),
        QOS_VOLUME_WORKLOAD(QosVolumeWorkloadPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getQosVolumeWorkloadPerformanceComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        QOS_LUN_WORKLOAD(QosLunWorkloadPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getQosLunWorkloadPerformanceComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        QOS_FILE_WORKLOAD(QosFileWorkloadPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getQosFileWorkloadPerformanceComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        QOS_WORKLOAD_QUEUE_DBLADE(QosWorkloadQueueDBladePerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getQosWorkloadQueueDBladePerformanceComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        QOS_WORKLOAD_QUEUE_NBLADE(QosWorkloadQueueNBladePerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getQosWorkloadQueueNBladePerformanceComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        QOS_WORKLOAD_DETAIL(QosWorkloadDetailPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getQosWorkloadDetailPerformanceComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        QOS_SERVICE_CENTER(QosServiceCenterPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getQosServiceCenterPerformanceComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        WAFL(WaflPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getWaflPerformanceComponentsFactory()),
        CLUSTER(ClusterPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getClusterPerformanceComponentsFactory()),
        LUN(LunPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getLunPerformanceComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        NAMESPACE(NamespacePerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getNamespacePerformanceComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        EXT_CACHE_OBJ(ExtCacheObjPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getExtCacheObjPerformanceComponentsFactory()),
        RESOURCE_HEADROOM_AGGR(ResourceHeadRoomAggrBean.PACKAGE_NAME, ComponentsFactoryRepository.getResourceHeadRoomAggrComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        RESOURCE_HEADROOM_CPU(ResourceHeadRoomCPUBean.PACKAGE_NAME, ComponentsFactoryRepository.getResourceHeadRoomCPUComponentsFactory()),
        OPM_HEADROOM_AGGR(OpmHeadRoomAggrPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getOpmHeadRoomAggrComponentsFactory(), CONTAINER_TYPE.CLUSTER),
        OPM_HEADROOM_CPU(OpmHeadRoomCPUPerformanceBean.PACKAGE_NAME, ComponentsFactoryRepository.getOpmHeadRoomCPUComponentsFactory());


        private String objectName;
        private AbstractPerformanceComponentsFactory componentsFactory;
        private CONTAINER_TYPE containerType = CONTAINER_TYPE.NONE;

        private PerformancePackages(String objectName, AbstractPerformanceComponentsFactory componentsFactory) {
            this.objectName = objectName;
            this.componentsFactory = componentsFactory;
        }

        private PerformancePackages(String objectName, AbstractPerformanceComponentsFactory componentsFactory, CONTAINER_TYPE containerType) {
            this.objectName = objectName;
            this.componentsFactory = componentsFactory;
            this.containerType = containerType;
        }

        private String getObjectName() {
            return objectName;
        }

        private AbstractPerformanceComponentsFactory getComponentsFactory() {
            return componentsFactory;
        }

        private CONTAINER_TYPE getContainerType() {
            return containerType;
        }
    }

    private enum CONTAINER_TYPE {
        CLUSTER,
        VSERVER,
        NONE,
    }

    private final static Logger logger = LoggerFactory.getLogger(SummaryCalculator.class);

    private static final String CLUSTER_QUERY = "SELECT objid, uuid FROM netapp_model.cluster WHERE datasourceId = ?";
    private static final String VSERVER_QUERY = "SELECT v.objid, v.uuid FROM netapp_model.vserver v INNER JOIN netapp_model.cluster c ON v.clusterId = c.objId WHERE c.datasourceId = ?";

    public static void calculateSummary(UserTransaction utx, Connection connection, long dsId, Long summaryHour) throws PerformanceException {
        final long startTime = System.currentTimeMillis();

        long clusterId = -1;
        String clusterKey = null;
        //Retrieve Cluster Key
        PreparedStatement preparedStatement = null;
        try {
            preparedStatement = connection.prepareStatement(CLUSTER_QUERY);
            preparedStatement.setLong(1, dsId);
            ResultSet rs = preparedStatement.executeQuery();
            if (rs.next()) {
                clusterId = rs.getLong("objid");
                clusterKey = rs.getString("uuid");
            }
        } catch (SQLException e) {
            throw new PerformanceException("Failed to retrieve cluster id and cluster key.", e);
        } finally {
            if (preparedStatement != null) {
                try {
                    preparedStatement.close();
                } catch (Exception ignored) {
                }
            }
        }
        //Retrieve vserver key
        Map<Long, String> vserverKeyMap = new HashMap<>();
        try {
            preparedStatement = connection.prepareStatement(VSERVER_QUERY);
            preparedStatement.setLong(1, dsId);
            ResultSet rs = preparedStatement.executeQuery();
            while (rs.next()) {
                vserverKeyMap.put(rs.getLong("objid"), rs.getString("uuid"));
            }
        } catch (SQLException e) {
            throw new PerformanceException("Failed to retrieve vserver id and vserver key.", e);
        } finally {
            if (preparedStatement != null) {
                try {
                    preparedStatement.close();
                } catch (Exception ignored) {
                }
            }
        }

        for (PerformancePackages performancePkg : PerformancePackages.values()) {
            if (CONTAINER_TYPE.VSERVER.equals(performancePkg.getContainerType())) {
                for (Map.Entry<Long, String> vServerKey : vserverKeyMap.entrySet()) {
                    if (AbstractTableManagement.isMarkCreated(getFullSampleTableTable(performancePkg.getObjectName(), vServerKey.getKey()))) {
                        insertSummaries(utx, connection, dsId, performancePkg.getComponentsFactory(), vServerKey.getKey(), vServerKey.getValue(),
                                summaryHour);
                    }
                }
            } else if (CONTAINER_TYPE.CLUSTER.equals(performancePkg.getContainerType())) {
                if (AbstractTableManagement.isMarkCreated(getFullSampleTableTable(performancePkg.getObjectName(), clusterId))) {
                    insertSummaries(utx, connection, dsId, performancePkg.getComponentsFactory(), clusterId, clusterKey, summaryHour);
                }
            } else {
                if (AbstractTableManagement.isMarkCreated(getFullSampleTableTable(performancePkg.getObjectName()))) {
                    insertSummaries(utx, connection, dsId, performancePkg.getComponentsFactory(), clusterId, clusterKey, summaryHour);
                }
            }
        }
        logger.info(String.format("CalculateSummary data_source_id=%d,time_taken=%d", dsId,
                System.currentTimeMillis() - startTime));
    }

    private static void insertSummaries(UserTransaction utx, Connection connection, long dsId,
                                        AbstractPerformanceComponentsFactory componentsFactory, Long containerId, String containerKey,
                                        Long summaryHour) {
        long startTime = System.currentTimeMillis();
        try {
            utx.begin();
            componentsFactory.createPersistenceFacade(connection, containerKey, containerId).insertHourlySummaries(connection, summaryHour);
            utx.commit();
        } catch (Exception genExe) {
            logger.error(String.format("Error creating summary records, data_source_id=%d,pkg=%s,insert_time=%d", dsId,
                    componentsFactory.getPerformancePackageId(containerId), System.currentTimeMillis() - startTime), genExe);
            try {
                utx.rollback();
            } catch (SystemException sysExe) {
                logger.warn("Error rolling back transaction", sysExe);
            }
        } finally {
            logger.info(String.format("SummaryJob data_source_id=%d,pkg=%s,insert_time=%d", dsId,
                    componentsFactory.getPerformancePackageId(containerId), System.currentTimeMillis() - startTime));
        }
    }

    private static String getFullSampleTableTable(String packageName, Long containerId) {
        StringBuilder sb = new StringBuilder();
        sb.append("netapp_performance.sample_").append(packageName);
        if (containerId != null) {
            sb.append("_").append(containerId);
        }
        return sb.toString();
    }

    private static String getFullSampleTableTable(String packageName) {
        return getFullSampleTableTable(packageName, null);
    }
}

