package com.netapp.oci.netapp.performance.aggregate;

import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateMeasurements;
import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

public class AggregateMetaData {
	public static final Column[] RAW_COLUMNS = new Column[]{
		new Column("cp_read_blocks", "bigint unsigned"),
		new Column("cp_reads", "bigint unsigned"),
		new Column("total_transfers", "bigint unsigned"),
		new Column("user_read_blocks", "bigint unsigned"),
		new Column("user_reads", "bigint unsigned"),
		new Column("user_write_blocks", "bigint unsigned"),
		new Column("user_writes", "bigint unsigned"),
        new Column("total_transfers_hdd", "bigint unsigned"),
        new Column("total_transfers_ssd", "bigint unsigned"),
        new Column("user_read_blocks_hdd", "bigint unsigned"),
        new Column("user_read_blocks_ssd", "bigint unsigned"),
        new Column("user_reads_hdd", "bigint unsigned"),
        new Column("user_reads_ssd", "bigint unsigned"),
        new Column("user_write_blocks_hdd", "bigint unsigned"),
        new Column("user_write_blocks_ssd", "bigint unsigned"),
        new Column("user_writes_hdd", "bigint unsigned"),
        new Column("user_writes_ssd", "bigint unsigned")
    };

	public static final Column[] SAMPLE_COLUMNS = new Column[]{
		new Column("cpReadBlocks", "float"),
		new Column("cpReads", "float"),
		new Column("totalTransfers", "float"),
		new Column("userReadBlocks", "float"),
		new Column("userReads", "float"),
		new Column("userWriteBlocks", "float"),
		new Column("userWrites", "float"),
        new Column("totalTransfersHdd", "float"),
        new Column("totalTransfersSsd", "float"),
        new Column("userReadBlocksHdd", "float"),
        new Column("userReadBlocksSsd", "float"),
        new Column("userReadsHdd", "float"),
        new Column("userReadsSsd", "float"),
        new Column("userWriteBlocksHdd", "float"),
        new Column("userWriteBlocksSsd", "float"),
        new Column("userWritesHdd", "float"),
        new Column("userWritesSsd", "float"),
        new Column("opmAggrThroughput", "float"),
        new Column("opmAggrLatency", "float"),
        new Column("opmAggrUserReadLatency", "float"),
        new Column("opmAggrUserWriteLatency", "float"),
        new Column("opmAggrCpReadLatency", "float"),
        new Column("opmAggrUtilization", "float")
	};

	public static final Column[] SUMMARY_COLUMNS = new Column[]{
		new Column("cpReadBlocks", "float"),
		new Column("cpReads", "float"),
		new Column("totalTransfers", "float"),
		new Column("userReadBlocks", "float"),
		new Column("userReads", "float"),
		new Column("userWriteBlocks", "float"),
		new Column("userWrites", "float"),
        new Column("totalTransfersHdd", "float"),
        new Column("totalTransfersSsd", "float"),
        new Column("userReadBlocksHdd", "float"),
        new Column("userReadBlocksSsd", "float"),
        new Column("userReadsHdd", "float"),
        new Column("userReadsSsd", "float"),
        new Column("userWriteBlocksHdd", "float"),
        new Column("userWriteBlocksSsd", "float"),
        new Column("userWritesHdd", "float"),
        new Column("userWritesSsd", "float"),
        new Column("opmAggrThroughput", "float"),
        new Column("opmAggrLatency", "float"),
        new Column("opmAggrUserReadLatency", "float"),
        new Column("opmAggrUserWriteLatency", "float"),
        new Column("opmAggrCpReadLatency", "float"),
        new Column("opmAggrUtilization", "float")
    };
	
    private static ObjectParser<AggregateRawMeasurements> rawParser;
    private static ObjectParser<AggregateMeasurements> sampleParser;

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG}

    };
    
    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG
    };
    
    static {
    	rawParser = new ObjectParser<AggregateRawMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, AggregateRawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpReadBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpReads());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalTransfers());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserReadBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserReads());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWriteBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWrites());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalTransfersHdd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalTransfersSsd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserReadBlocksHdd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserReadBlocksSsd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserReadsHdd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserReadsSsd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWriteBlocksHdd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWriteBlocksSsd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWritesHdd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWritesSsd());
            }

            public AggregateRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new AggregateRawMeasurements()
                        .setCpReadBlocks(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setCpReads(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setTotalTransfers(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setUserReadBlocks(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setUserReads(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setUserWriteBlocks(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setUserWrites(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setTotalTransfersHdd(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setTotalTransfersSsd(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setUserReadBlocksHdd(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setUserReadBlocksSsd(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setUserReadsHdd(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setUserReadsSsd(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setUserWriteBlocksHdd(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setUserWriteBlocksSsd(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setUserWritesHdd(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                        .setUserWritesSsd(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
            }
        };
        
        sampleParser = new ObjectParser<AggregateMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, AggregateMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpReadBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpReads());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalTransfers());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserReadBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserReads());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWriteBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWrites());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalTransfersHdd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalTransfersSsd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserReadBlocksHdd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserReadBlocksSsd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserReadsHdd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserReadsSsd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWriteBlocksHdd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWriteBlocksSsd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWritesHdd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWritesSsd());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmAggrThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmAggrLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmAggrUserReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmAggrUserWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmAggrCpReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmAggrUtilization());
            }

            public AggregateMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new AggregateMeasurements()
                        .setCpReadBlocks(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setCpReads(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setTotalTransfers(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setUserReadBlocks(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setUserReads(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setUserWriteBlocks(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setUserWrites(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setTotalTransfersHdd(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setTotalTransfersSsd(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setUserReadBlocksHdd(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setUserReadBlocksSsd(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setUserReadsHdd(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setUserReadsSsd(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setUserWriteBlocksHdd(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setUserWriteBlocksSsd(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setUserWritesHdd(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setUserWritesSsd(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmAggrThroughput(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmAggrLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmAggrUserReadLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmAggrUserWriteLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmAggrCpReadLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmAggrUtilization(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }
    
    public static ObjectParser<AggregateRawMeasurements> getRawParser() {
        return rawParser;
    }
    
    public static ObjectParser<AggregateMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<AggregateMeasurements> getSampleParser() {
        return sampleParser;
    }
}
