/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.aggregate.interfaces.data;


import java.io.Serializable;

public class WaflHyaAggrMeasurements implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * Average of raid io latency on read hit.
     */
    private float hya_read_hit_latency_average;

    /**
     * Average read miss latency.
     */
    private float hya_read_miss_latency_average;

    /**
     * Average write latency to HDD.
     */
    private float hya_write_hdd_latency_average;

    /**
     * Average of raid io latency on write to ssd.
     */
    private float hya_write_ssd_latency_average;

    /**
     * Number of block misses in SSD cache.
     */
    private float read_miss_blks_rate;

    /**
     * Number of HDD read operations replaced by SSD reads per second.
     */
    private float read_ops_replaced;

    /**
     * Percentage of HDD read operations replace by SSD.
     */
    private float read_ops_replaced_percent;

    /**
     * Total read cached SSD blocks.
     */
    private float ssd_read_cached;

    /**
     * Total SSD blocks.
     */
    private float ssd_total;

    /**
     * Total SSD blocks used.
     */
    private float ssd_total_used;

    /**
     * Total write cached SSD blocks.
     */
    private float ssd_write_cached;


    /**
     * Number of HDD write blocks replaced by SSD writes per second.
     */
    private float write_blks_replaced;


    public WaflHyaAggrMeasurements() {
    }


    public float getHyaReadHitLatencyAverage() {
        return hya_read_hit_latency_average;
    }

    public void setHyaReadHitLatencyAverage(final float hya_read_hit_latency_average) {
        this.hya_read_hit_latency_average = hya_read_hit_latency_average;
    }

    public float getHyaReadMissLatencyAverage() {
        return hya_read_miss_latency_average;
    }

    public void setHyaReadMissLatencyAverage(final float hya_read_miss_latency_average) {
        this.hya_read_miss_latency_average = hya_read_miss_latency_average;
    }

    public float getHyaWriteHddLatencyAverage() {
        return hya_write_hdd_latency_average;
    }

    public void setHyaWriteHddLatencyAverage(final float hya_write_hdd_latency_average) {
        this.hya_write_hdd_latency_average = hya_write_hdd_latency_average;
    }

    public float getHyaWriteSsdLatencyAverage() {
        return hya_write_ssd_latency_average;
    }

    public void setHyaWriteSsdLatencyAverage(final float hya_write_ssd_latency_average) {
        this.hya_write_ssd_latency_average = hya_write_ssd_latency_average;
    }

    public float getReadMissBlksRate() {
        return read_miss_blks_rate;
    }

    public void setReadMissBlksRate(final float read_miss_blks_rate) {
        this.read_miss_blks_rate = read_miss_blks_rate;
    }

    public float getReadOpsReplaced() {
        return read_ops_replaced;
    }

    public void setReadOpsReplaced(final float read_ops_replaced) {
        this.read_ops_replaced = read_ops_replaced;
    }

    public float getReadOpsReplacedPercent() {
        return read_ops_replaced_percent;
    }

    public void setReadOpsReplacedPercent(final float read_ops_replaced_percent) {
        this.read_ops_replaced_percent = read_ops_replaced_percent;
    }

    public float getSsdReadCached() {
        return ssd_read_cached;
    }

    public void setSsdReadCached(final float ssd_read_cached) {
        this.ssd_read_cached = ssd_read_cached;
    }

    public float getSsdTotal() {
        return ssd_total;
    }

    public void setSsdTotal(final float ssd_total) {
        this.ssd_total = ssd_total;
    }

    public float getSsdTotalUsed() {
        return ssd_total_used;
    }

    public void setSsdTotalUsed(final float ssd_total_used) {
        this.ssd_total_used = ssd_total_used;
    }

    public float getSsdWriteCached() {
        return ssd_write_cached;
    }

    public void setSsdWriteCached(final float ssd_write_cached) {
        this.ssd_write_cached = ssd_write_cached;
    }

    public float getWriteBlksReplaced() {
        return write_blks_replaced;
    }

    public void setWritesBlksReplaced(final float write_blks_replaced) {
        this.write_blks_replaced = write_blks_replaced;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        WaflHyaAggrMeasurements that = (WaflHyaAggrMeasurements) o;

        if (Float.compare(that.hya_read_hit_latency_average, hya_read_hit_latency_average) != 0) return false;
        if (Float.compare(that.hya_read_miss_latency_average, hya_read_miss_latency_average) != 0) return false;
        if (Float.compare(that.hya_write_hdd_latency_average, hya_write_hdd_latency_average) != 0) return false;
        if (Float.compare(that.hya_write_ssd_latency_average, hya_write_ssd_latency_average) != 0) return false;
        if (Float.compare(that.read_miss_blks_rate, read_miss_blks_rate) != 0) return false;
        if (Float.compare(that.read_ops_replaced, read_ops_replaced) != 0) return false;
        if (Float.compare(that.read_ops_replaced_percent, read_ops_replaced_percent) != 0) return false;
        if (Float.compare(that.ssd_read_cached, ssd_read_cached) != 0) return false;
        if (Float.compare(that.ssd_total, ssd_total) != 0) return false;
        if (Float.compare(that.ssd_total_used, ssd_total_used) != 0) return false;
        if (Float.compare(that.ssd_write_cached, ssd_write_cached) != 0) return false;
        if (Float.compare(that.write_blks_replaced, write_blks_replaced) != 0) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = (hya_read_hit_latency_average != +0.0f ? Float.floatToIntBits(hya_read_hit_latency_average) : 0);
        result = 31 * result + (hya_read_miss_latency_average != +0.0f ? Float.floatToIntBits(hya_read_miss_latency_average) : 0);
        result = 31 * result + (hya_write_hdd_latency_average != +0.0f ? Float.floatToIntBits(hya_write_hdd_latency_average) : 0);
        result = 31 * result + (hya_write_ssd_latency_average != +0.0f ? Float.floatToIntBits(hya_write_ssd_latency_average) : 0);
        result = 31 * result + (read_miss_blks_rate != +0.0f ? Float.floatToIntBits(read_miss_blks_rate) : 0);
        result = 31 * result + (read_ops_replaced != +0.0f ? Float.floatToIntBits(read_ops_replaced) : 0);
        result = 31 * result + (read_ops_replaced_percent != +0.0f ? Float.floatToIntBits(read_ops_replaced_percent) : 0);
        result = 31 * result + (ssd_read_cached != +0.0f ? Float.floatToIntBits(ssd_read_cached) : 0);
        result = 31 * result + (ssd_total != +0.0f ? Float.floatToIntBits(ssd_total) : 0);
        result = 31 * result + (ssd_total_used != +0.0f ? Float.floatToIntBits(ssd_total_used) : 0);
        result = 31 * result + (ssd_write_cached != +0.0f ? Float.floatToIntBits(ssd_write_cached) : 0);
        result = 31 * result + (write_blks_replaced != +0.0f ? Float.floatToIntBits(write_blks_replaced) : 0);
        return result;
    }

    @Override
    public String toString() {
        return "WaflHyaAggrMeasurements{" +
                "hya_read_hit_latency_average=" + hya_read_hit_latency_average +
                ", hya_read_miss_latency_average=" + hya_read_miss_latency_average +
                ", hya_write_hdd_latency_average=" + hya_write_hdd_latency_average +
                ", hya_write_ssd_latency_average=" + hya_write_ssd_latency_average +
                ", read_miss_blks_rate=" + read_miss_blks_rate +
                ", read_ops_replaced=" + read_ops_replaced +
                ", read_ops_replaced_percent=" + read_ops_replaced_percent +
                ", ssd_read_cached=" + ssd_read_cached +
                ", ssd_total=" + ssd_total +
                ", ssd_total_used=" + ssd_total_used +
                ", ssd_write_cached=" + ssd_write_cached +
                ", write_blks_replaced=" + write_blks_replaced +
                '}';
    }
}
