package com.netapp.oci.netapp.performance.aggregate.interfaces.remote;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.Remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

@Remote
public interface AggregatePerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//AggregatePerformanceQueryBean!com.netapp.oci.netapp.performance.aggregate.interfaces.remote.AggregatePerformanceQueryRemote";

    /**
     * Gets aggregate samples for period of time
     *
     * @param aggrIds set of node ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of node id to list ordered by time of aggregate samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<AggregateMeasurements>>> getSamples(Set<Long> aggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;
    
    /**
     * Gets aggregate samples for period of time
     *
     * @param containerId container id
     * @param aggrIds set of aggregate ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of aggregate id to list ordered by time of aggregate samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<AggregateMeasurements>>> getSamples(Long containerId, Set<Long> aggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;
    
    /**
     * Gets aggregate samples for period of time
     *
     * @param containerId container id
     * @param aggrId aggregate id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of aggregate samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<AggregateMeasurements>> getSamples(Long containerId, Long aggrId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given aggregate id's (aggrIds).  The map returned will contain only aggregate ids that has a sample
     * aggregate ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the aggregate ids passed.
     *
     * @param clusterId cluster id
     * @param aggrIds set of aggregate ids
     * @return map of aggregate id to latest sample SampleInfo<AggregateMeasurements> aggregate ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<AggregateMeasurements>> getLatestSample(Long clusterId, Set<Long> aggrIds) throws NetappInterfaceException;

    /**
     * Gets aggregate summaries for aggregate ids
     *
     * @param aggrIds set of aggregate ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of aggregate id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<AggregateMeasurements>>> getSummaries(Set<Long> aggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets aggregate summaries for aggregate ids
     *
     * @param clusterId cluster id
     * @param aggrIds set of aggregate ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of aggregate id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<AggregateMeasurements>>> getSummaries(Long clusterId, Set<Long> aggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;
    
}
