package com.netapp.oci.netapp.performance.cifsvserver;

import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverMeasurements;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

public class CifsVserverMetaData {
	public static final Column[] RAW_COLUMNS = new Column[]{
		new Column("cifs_latency", "bigint unsigned"),
		new Column("cifs_latency_base", "bigint unsigned"),
		new Column("cifs_ops", "bigint unsigned"),
		new Column("cifs_read_latency", "bigint unsigned"),
		new Column("cifs_read_ops", "bigint unsigned"),
		new Column("cifs_write_latency", "bigint unsigned"),
		new Column("cifs_write_ops", "bigint unsigned"),
		new Column("cifs_read_size_histo", "varchar"),
		new Column("cifs_write_size_histo", "varchar"),
        new Column("cifs_read_throughput", "bigint unsigned"),
        new Column("cifs_write_throughput", "bigint unsigned"),
	};
	
	public static final Column[] SAMPLE_COLUMNS = new Column[]{
	    new Column("cifsLatencyBase", "bigint unsigned"),
        new Column("cifsLatency", "float"),
        new Column("cifsOps", "float"),
        new Column("cifsReadLatency", "float"),
        new Column("cifsReadOps", "float"),
        new Column("cifsWriteLatency", "float"),
        new Column("cifsWriteOps", "float"),
        new Column("cifsReadThroughput", "float"),
        new Column("cifsWriteThroughput", "float"),
        new Column("opmCifsThroughput", "float")
    };
	
	public static final Column[] SUMMARY_COLUMNS = new Column[]{
	    new Column("cifsLatencyBase", "bigint unsigned"),
        new Column("cifsLatency", "float"),
        new Column("cifsOps", "float"),
        new Column("cifsReadLatency", "float"),
        new Column("cifsReadOps", "float"),
        new Column("cifsWriteLatency", "float"),
        new Column("cifsWriteOps", "float"),
        new Column("cifsReadThroughput", "float"),
        new Column("cifsWriteThroughput", "float"),
        new Column("opmCifsThroughput", "float")
    };

	  public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
	        {SqlMethod.SUM},
	        {new RatioMethod("cifsLatencyBase")},
	        {SqlMethod.AVG},
	        {new RatioMethod("cifsReadOps")},
	        {SqlMethod.AVG},
	        {new RatioMethod("cifsWriteOps")},
	        {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG}
	    };

	    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
	        SqlMethod.SUM,
	        new RatioMethod("cifsLatencyBase"),
	        SqlMethod.AVG,
	        new RatioMethod("cifsReadOps"),
	        SqlMethod.AVG,
	        new RatioMethod("cifsWriteOps"),
	        SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG
	    };

	private static ObjectParser<CifsVserverRawMeasurements> rawParser;
	private static ObjectParser<CifsVserverMeasurements> sampleParser;

    static {
    	rawParser = new ObjectParser<CifsVserverRawMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, CifsVserverRawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsLatencyBase());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsReadSizeHisto());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsWriteSizeHisto());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsReadThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsWriteThroughput());
            }

            public CifsVserverRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new CifsVserverRawMeasurements(
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getStringNullable(resultSet, ++i),
                        PerformanceTableUtils.getStringNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));

            }
        };
        
        sampleParser = new ObjectParser<CifsVserverMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, CifsVserverMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsLatencyBase());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsReadThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsWriteThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsThroughput());
            }

            public CifsVserverMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new CifsVserverMeasurements(
                        PerformanceTableUtils.getLongNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }
    
    public static ObjectParser<CifsVserverRawMeasurements> getRawParser() {
        return rawParser;
    }

    public static ObjectParser<CifsVserverMeasurements> getSampleParser() {
        return sampleParser;
    }

    public static ObjectParser<CifsVserverMeasurements> getSummaryParser() {
        return sampleParser;
    }

}
