package com.netapp.oci.netapp.performance.cifsvserver.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Anton Sergeev
 */
@Remote
public interface CifsVserverPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//CifsVserverPerformanceQueryBean!com.netapp.oci.netapp.performance.cifsvserver.interfaces.remote.CifsVserverPerformanceQueryLocal";

    /**
     * Gets cifs samples for period of time
     *
     * @param cifsIds set of cifs ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of cifs id to list ordered by time of cifs samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<CifsVserverMeasurements>>> getSamples(Set<Long> cifsIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets cifs samples for period of time
     *
     * @param containerId container id
     * @param cifsIds set of cifs ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of cifs id to list ordered by time of cifs samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<CifsVserverMeasurements>>> getSamples(Long containerId, Set<Long> cifsIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets cifs samples for period of time
     *
     * @param containerId container id
     * @param cifsId cifs id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of cifs samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<CifsVserverMeasurements>> getSamples(Long containerId, Long cifsId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given cifs id's (cifsIds). The map returned will contain only cifs ids that has a sample.
     * Disk ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the Disk ids passed.
     *
     * @param clusterId cluster id
     * @param cifsIds set of cifs ids
     * @return map of cifs id to latest sample SampleInfo<CifsVserverMeasurements>. Disk ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<CifsVserverMeasurements>> getLatestSample(Long clusterId, Set<Long> cifsIds) throws NetappInterfaceException;


    /**
     * Gets cifs summaries for cifs ids
     *
     * @param cifsIds set of cifs ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of cifs id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<CifsVserverMeasurements>>> getSummaries(Set<Long> cifsIds, Date fromTime, Date toTime) throws NetappInterfaceException;

   /**
     * Gets cifs summaries for cifs ids
     *
     * @param clusterId cluster id
     * @param cifsIds set of cifs ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of cifs id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<CifsVserverMeasurements>>> getSummaries(Long clusterId, Set<Long> cifsIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
