package com.netapp.oci.netapp.performance.cifsvserver.interfaces.remote;

import java.util.Collection;

import javax.ejb.Remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverMeasurements;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

@Remote
public interface CifsVserverPerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//CifsVserverPerformanceBean!com.netapp.oci.netapp.performance.cifsvserver.interfaces.remote.CifsVserverPerformanceRemote";
    
    /**
     * To report raw samples for cifs:vserver performance
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw aggregate samples
     * @param leaseTime - the time for datasource to be registered in the in the registry (in milliseconds)
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportCifsVserverRawData(Long datasourceId, String clusterKey, long time, Collection<RawSample<CifsVserverRawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
    
    /**
     * To report samples for cifs:vserver performance
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw aggregate samples
     * @param leaseTime - the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical --flag showing the samples are for historical poll or regular poll
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportCifsVserverData(Long datasourceId, String clusterKey, long time, Collection<RawSample<CifsVserverMeasurements>> rawSamples, long leaseTime, boolean isHistorical) throws NetappInterfaceException;

}
