/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.cluster;

import java.sql.ResultSet;
import java.sql.SQLException;

import com.netapp.oci.netapp.performance.cluster.interfaces.data.ClusterMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

public class ClusterMetaData {

    public static final Column[] SAMPLE_COLUMNS = new Column[] {
        new Column("clusterOps", "float"),
        new Column("clusterThroughput", "float"),
        new Column("clusterMaxNodeUtilization", "float"),
        new Column("clusterMaxAggregateUtilization", "float")};

    public static final Column[] SUMMARY_COLUMNS = new Column[] {
    	new Column("clusterOps", "float"),
        new Column("clusterThroughput", "float"),
        new Column("clusterMaxNodeUtilization", "float"),
        new Column("clusterMaxAggregateUtilization", "float")};

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][] {
    	{SqlMethod.AVG},
    	{SqlMethod.AVG},
        {SqlMethod.AVG}, 
        {SqlMethod.AVG}, 
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[] {
    	SqlMethod.AVG,
    	SqlMethod.AVG,
        SqlMethod.AVG, 
        SqlMethod.AVG, 
    };

    private static ObjectParser<ClusterMeasurements> sampleParser;

    static {
        sampleParser = new ObjectParser<ClusterMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement,
                    ClusterMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getClusterOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getClusterThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getClusterMaxNodeUtilization());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getClusterMaxAggregateUtilization());
            }

            public ClusterMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                final ClusterMeasurements clusterMeasurements = new ClusterMeasurements();
                int i = startIndex - 1;
                clusterMeasurements.setClusterOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                clusterMeasurements.setClusterThroughput(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                clusterMeasurements.setClusterMaxNodeUtilization(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                clusterMeasurements.setClusterMaxAggregateUtilization(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                return clusterMeasurements;
            }
        };
    }

    public static ObjectParser<ClusterMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<ClusterMeasurements> getSampleParser() {
        return sampleParser;
    }

}
