package com.netapp.oci.netapp.performance.disk.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.disk.interfaces.data.DiskMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;
import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Anton Sergeev
 */
@Remote
public interface DiskPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//DiskPerformanceQueryBean!com.netapp.oci.netapp.performance.disk.interfaces.remote.DiskPerformanceQueryRemote";

    /**
     * Gets disk samples for period of time
     *
     * @param diskIds set of disk ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of disk id to list ordered by time of disk samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<DiskMeasurements>>> getSamples(Set<Long> diskIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets disk samples for period of time
     *
     * @param containerId container id
     * @param diskIds set of disk ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of disk id to list ordered by time of disk samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<DiskMeasurements>>> getSamples(Long containerId, Set<Long> diskIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets disk samples for period of time
     *
     * @param containerId container id
     * @param diskId disk id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of disk samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<DiskMeasurements>> getSamples(Long containerId, Long diskId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given disk id's (diskIds). The map returned will contain only disk ids that has a sample.
     * Disk ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the Disk ids passed.
     *
     * @param clusterId cluster id
     * @param diskIds set of disk ids
     * @return map of disk id to latest sample SampleInfo<DiskMeasurements>. Disk ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<DiskMeasurements>> getLatestSample(Long clusterId, Set<Long> diskIds) throws NetappInterfaceException;

    /**
     * Gets disk summaries for disk ids
     *
     * @param diskIds set of disk ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of disk id to disk samplesummary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<DiskMeasurements>>> getSummaries(Set<Long> diskIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets disk summaries for disk ids
     *
     * @param containerId container id
     * @param diskIds set of disk ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of disk id to disk samplesummary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<DiskMeasurements>>> getSummaries(Long containerId, Set<Long> diskIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    
    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
