package com.netapp.oci.netapp.performance.disk.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.disk.interfaces.data.DiskMeasurements;
import com.netapp.oci.netapp.performance.disk.interfaces.data.DiskRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

import javax.ejb.Remote;
import java.util.Collection;

/**
 * @author Anton Sergeev
 */
@Remote
public interface DiskPerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//DiskPerformanceBean!com.netapp.oci.netapp.performance.disk.interfaces.remote.DiskPerformanceRemote";

    /**
     * The Acq use this method to report on Disk performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw disk samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportDiskSamples(Long datasourceId, String clusterKey, long time, Collection<RawSample<DiskMeasurements>> rawSamples, long leaseTime, boolean isHistorical) throws NetappInterfaceException;

    /**
     * To report raw samples for disk performance
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw disk samples
     * @param leaseTime - the time for datasource to be registered in the in the registry (in milliseconds)
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportDiskRawSamples(Long datasourceId, String clusterKey, long time, Collection<RawSample<DiskRawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
}
