/*
 * Copyright (c) 2015 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.extcacheobj;

import java.sql.ResultSet;
import java.sql.SQLException;

import com.netapp.oci.netapp.performance.extcacheobj.interfaces.data.ExtCacheObjMeasurements;
import com.netapp.oci.netapp.performance.extcacheobj.interfaces.data.ExtCacheObjRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

public class ExtCacheObjMetaData {
	public static final Column[] RAW_COLUMNS = new Column[]{
		new Column("accesses", "bigint unsigned"),
		new Column("hitPercent", "bigint unsigned"),
		new Column("diskReadsReplaced", "bigint unsigned"),
		new Column("readIOBlocks", "bigint unsigned"),
	};

    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("accesses", "float"),
            new Column("hitPercent", "float"),
            new Column("diskReadsReplaced", "float"),
            new Column("cacheReadThroughput", "float"),
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("accesses", "float"),
            new Column("hitPercent", "float"),
            new Column("diskReadsReplaced", "float"),
            new Column("cacheReadThroughput", "float"),
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {SqlMethod.AVG},
            {new RatioMethod("accesses")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            SqlMethod.AVG,
            new RatioMethod("accesses"),
            SqlMethod.AVG,
            SqlMethod.AVG,
    };

    private static ObjectParser<ExtCacheObjRawMeasurements> rawParser;
    private static ObjectParser<ExtCacheObjMeasurements> sampleParser;

    static {
    	rawParser = new ObjectParser<ExtCacheObjRawMeasurements>() {
            @Override
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, ExtCacheObjRawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAccesses());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getHitPercent());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDiskReadsReplaced());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadIOBlocks());
            }

            @Override
            public ExtCacheObjRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new ExtCacheObjRawMeasurements(
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
            }
        };

        sampleParser = new ObjectParser<ExtCacheObjMeasurements>() {
            @Override
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, ExtCacheObjMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAccesses());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getHitPercent());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDiskReadsReplaced());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCacheReadThroughput());
            }

            @Override
            public ExtCacheObjMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new ExtCacheObjMeasurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<ExtCacheObjMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<ExtCacheObjMeasurements> getSampleParser() {
        return sampleParser;
    }

    public static ObjectParser<ExtCacheObjRawMeasurements> getRawParser() {
        return rawParser;
    }
}
