/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.extcacheobj.interfaces.data;

import java.io.Serializable;

@SuppressWarnings("UnusedDeclaration")
public class ExtCacheObjMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;
    
    /**
     *  Estimated number of disk reads per second replaced by cache
     */
    private float diskReadsReplaced;

    /**
     *  External cache hit rate
     */
    private float hitPercent;
    
    /**
     * External cache accesses per second
     */
    private float accesses;
    
    /**
     * Number of buffers served off the external cache per sec.
     * Measurement unit is KBps, aligning with units of node counter opmSysReadThroughput 
     */
    private float cacheReadThroughput;

    public ExtCacheObjMeasurements() {
    }

    public ExtCacheObjMeasurements(float diskReadsReplaced, float hitPercent, float accesses, float cacheReadThroughput) {
        this.hitPercent = hitPercent;
        this.diskReadsReplaced = diskReadsReplaced;
        this.accesses = accesses;
        this.cacheReadThroughput = cacheReadThroughput;
    }

    public float getDiskReadsReplaced() {
        return diskReadsReplaced;
    }

    public void setDiskReadsReplaced(final float diskReadsReplaced) {
        this.diskReadsReplaced = diskReadsReplaced;
    }

    public float getHitPercent() {
        return hitPercent;
    }

    public void setHitPercent(final float hitPercent) {
        this.hitPercent = hitPercent;
    }
    
    public float getAccesses(){
        return this.accesses;
    }

    public void setAccesses(float accesses){
        this.accesses = accesses;
    }
    
    public void setCacheReadThroughput(float cacheReadThroughput){
        this.cacheReadThroughput = cacheReadThroughput;
    }
    
    public float getCacheReadThroughput(){
        return this.cacheReadThroughput;
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof ExtCacheObjMeasurements)) {
            return false;
        }

        ExtCacheObjMeasurements that = (ExtCacheObjMeasurements) o;

        return Float.compare(that.diskReadsReplaced, diskReadsReplaced) == 0
                       && Float.compare(that.hitPercent, hitPercent) == 0
                       && Float.compare(that.cacheReadThroughput, cacheReadThroughput) == 0
                       && Float.compare(that.accesses, accesses) == 0;
    }

    @Override
    public int hashCode() {
        int result = (diskReadsReplaced != +0.0f ? Float.floatToIntBits(diskReadsReplaced) : 0);
        result = 31 * result + (hitPercent != +0.0f ? Float.floatToIntBits(hitPercent) : 0);
        result = 31 * result + (cacheReadThroughput != +0.0f ? Float.floatToIntBits(cacheReadThroughput) : 0);
        result = 31 * result + (accesses != +0.0f ? Float.floatToIntBits(accesses) : 0);
        return result;

    }

    @Override
    public String toString() {
        return "ExtCacheObjMeasurements{" +
               "diskReadsReplaced=" + diskReadsReplaced +
               ", hitPercent=" + hitPercent +
               ", accesses=" + accesses +
               ", cacheReadThroughput=" + cacheReadThroughput +
               '}';
    }
}