/*
 * Copyright (c) 2015 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.extcacheobj.interfaces.data;

import java.io.Serializable;
import java.math.BigInteger;

@SuppressWarnings("UnusedDeclaration")
public class ExtCacheObjRawMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     *  Estimated number of disk reads per second replaced by cache
     */
    private BigInteger diskReadsReplaced;

    /**
     *  External cache hit rate
     */
    private BigInteger hitPercent;

    /**
     * External cache accesses per second
     */
    private BigInteger accesses;

    /**
     * Number of wafl blocks in all cache read I/Os
     */
    private BigInteger readIOBlocks;

    public ExtCacheObjRawMeasurements() {
    }

    public ExtCacheObjRawMeasurements(BigInteger diskReadsReplaced, BigInteger hitPercent, BigInteger accesses,
        BigInteger readIOBlocks) {
        this.hitPercent = hitPercent;
        this.diskReadsReplaced = diskReadsReplaced;
        this.accesses = accesses;
        this.readIOBlocks = readIOBlocks;
    }

    public BigInteger getDiskReadsReplaced() {
        return diskReadsReplaced;
    }

    public void setDiskReadsReplaced(final BigInteger diskReadsReplaced) {
        this.diskReadsReplaced = diskReadsReplaced;
    }

    public BigInteger getHitPercent() {
        return hitPercent;
    }

    public void setHitPercent(final BigInteger hitPercent) {
        this.hitPercent = hitPercent;
    }

    public BigInteger getAccesses(){
        return this.accesses;
    }

    public void setAccesses(BigInteger accesses){
        this.accesses = accesses;
    }

    public BigInteger getReadIOBlocks() {
        return readIOBlocks;
    }

    public void setReadIOBlocks(BigInteger readIOBlocks) {
        this.readIOBlocks = readIOBlocks;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result
                + ((accesses == null) ? 0 : accesses.hashCode());
        result = prime
                * result
                + ((diskReadsReplaced == null) ? 0 : diskReadsReplaced
                        .hashCode());
        result = prime * result
                + ((hitPercent == null) ? 0 : hitPercent.hashCode());
        result = prime * result
                + ((readIOBlocks == null) ? 0 : readIOBlocks.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        ExtCacheObjRawMeasurements other = (ExtCacheObjRawMeasurements) obj;
        if (accesses == null) {
            if (other.accesses != null)
                return false;
        } else if (!accesses.equals(other.accesses))
            return false;
        if (diskReadsReplaced == null) {
            if (other.diskReadsReplaced != null)
                return false;
        } else if (!diskReadsReplaced.equals(other.diskReadsReplaced))
            return false;
        if (hitPercent == null) {
            if (other.hitPercent != null)
                return false;
        } else if (!hitPercent.equals(other.hitPercent))
            return false;
        if (readIOBlocks == null) {
            if (other.readIOBlocks != null)
                return false;
        } else if (!readIOBlocks.equals(other.readIOBlocks))
            return false;
        return true;
    }

    @Override
    public String toString() {
        return "ExtCacheObjRawMeasurements{" +
               "diskReadsReplaced=" + diskReadsReplaced +
               ", hitPercent=" + hitPercent +
               ", accesses=" + accesses +
               ", readIOBlocks=" + readIOBlocks +
               '}';
    }
}