package com.netapp.oci.netapp.performance.extcacheobj.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.extcacheobj.interfaces.data.ExtCacheObjMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;
import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Remote
public interface ExtCacheObjPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//ExtCacheObjPerformanceQueryBean!com.netapp.oci.netapp.performance.extcacheobj.interfaces.remote.ExtCacheObjPerformanceQueryRemote";

    /**
     * Gets node extcacheobj samples for period of time
     *
     * @param nodeIds set of node ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of node id to list ordered by time of extcacheobj samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<ExtCacheObjMeasurements>>> getSamples(Set<Long> nodeIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets node extcacheobj samples for period of time
     *
     * @param containerId container id
     * @param nodeIds set of node ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of node id to list ordered by time of extcacheobj samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<ExtCacheObjMeasurements>>> getSamples(Long containerId, Set<Long> nodeIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets node extcacheobj samples for period of time
     *
     * @param containerId container id
     * @param nodeId node id
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return list ordered by time of node extcacheobj samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<ExtCacheObjMeasurements>> getSamples(Long containerId, Long nodeId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest extcacheobj sample for the given node Ids. The map returned will contain only node ids that has a sample.
     * Node ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the node ids passed.
     *
     * @param clusterId cluster id
     * @param nodeIds set of node ids
     * @return map of node id to latest sample SampleInfo<ExtCacheObjMeasurements>. Node ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<ExtCacheObjMeasurements>> getLatestSample(Long clusterId, Set<Long> nodeIds) throws NetappInterfaceException;

    /**
     * Gets node extcacheobj summaries for node ids
     *
     * @param nodeIds set of node ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of node id to extcacheobj summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<ExtCacheObjMeasurements>>> getSummaries(Set<Long> nodeIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets node extcacheobj summaries for node ids
     *
     * @param containerId container id
     * @param nodeIds set of node ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of node id to extcacheobj summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<ExtCacheObjMeasurements>>> getSummaries(Long containerId, Set<Long> nodeIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    
    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
