package com.netapp.oci.netapp.performance.extcacheobj.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.extcacheobj.interfaces.data.ExtCacheObjMeasurements;
import com.netapp.oci.netapp.performance.extcacheobj.interfaces.data.ExtCacheObjRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

import javax.ejb.Remote;
import java.util.Collection;

@Remote
public interface ExtCacheObjPerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//ExtCacheObjPerformanceBean!com.netapp.oci.netapp.performance.extcacheobj.interfaces.remote.ExtCacheObjPerformanceRemote";

    /**
     * The Acq use this method to report on external cache objects performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw extCacheObj samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportExtCacheObjSamples(Long datasourceId, String clusterKey, long time, Collection<RawSample<ExtCacheObjMeasurements>> rawSamples, long leaseTime, boolean isHistorical) throws NetappInterfaceException;

    /**
     * To report raw samples for external cache object performance
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw extCacheObj samples
     * @param leaseTime - the time for datasource to be registered in the in the registry (in milliseconds)
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportExtCacheObjRawSamples(Long datasourceId, String clusterKey, long time, Collection<RawSample<ExtCacheObjRawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
}
