package com.netapp.oci.netapp.performance.fcplif;

import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

public class FcpLifVserverMetaData {
	public static final Column[] RAW_COLUMNS = new Column[]{
		new Column("avg_latency", "bigint unsigned"),
		new Column("total_ops", "bigint unsigned"),
		new Column("avg_other_latency", "bigint unsigned"),
		new Column("other_ops", "bigint unsigned"),
		new Column("read_data", "bigint unsigned"),
		new Column("avg_read_latency", "bigint unsigned"),
		new Column("read_ops", "bigint unsigned"),
		new Column("write_data", "bigint unsigned"),
		new Column("avg_write_latency", "bigint unsigned"),
		new Column("write_ops", "bigint unsigned"),
	};

	public static final Column[] SAMPLE_COLUMNS = new Column[]{
		new Column("avgLatency", "float"),
		new Column("totalOps", "float"),
		new Column("otherOps", "float"),
		new Column("readData", "float"),
		new Column("readOps", "float"),
		new Column("writeData", "float"),
		new Column("writeOps", "float"),
		new Column("avgOtherLatency", "float"),
		new Column("avgReadLatency", "float"),
		new Column("avgWriteLatency", "float"),
		new Column("opmFcpLifThroughput", "float")
	};

	public static final Column[] SUMMARY_COLUMNS = new Column[]{
		new Column("avgLatency", "float"),
		new Column("totalOps", "float"),    	
		new Column("otherOps", "float"),
		new Column("readData", "float"),
		new Column("readOps", "float"),
		new Column("writeData", "float"),
		new Column("writeOps", "float"),
		new Column("avgOtherLatency", "float"),
		new Column("avgReadLatency", "float"),
		new Column("avgWriteLatency", "float"),
		new Column("opmFcpLifThroughput", "float")
	};

	public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
		{new RatioMethod("totalOps")},
		{SqlMethod.AVG},
		{SqlMethod.AVG},
		{SqlMethod.AVG},
		{SqlMethod.AVG},
		{SqlMethod.AVG},
		{SqlMethod.AVG},
		{new RatioMethod("otherOps")},
		{new RatioMethod("readOps")},
		{new RatioMethod("writeOps")},
		{SqlMethod.AVG}
	};

	public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
		new RatioMethod("totalOps"),
		SqlMethod.AVG,    	
		SqlMethod.AVG,
		SqlMethod.AVG,
		SqlMethod.AVG,
		SqlMethod.AVG,
		SqlMethod.AVG,
		new RatioMethod("otherOps"),
		new RatioMethod("readOps"),
		new RatioMethod("writeOps"),
		SqlMethod.AVG
	};

	private static ObjectParser<FcpLifVserverRawMeasurements> rawParser;
	private static ObjectParser<FcpLifVserverMeasurements> sampleParser;

    static {
		rawParser = new ObjectParser<FcpLifVserverRawMeasurements>() {
			public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, FcpLifVserverRawMeasurements objectData) throws TableException {
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgOtherLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
			}

			public FcpLifVserverRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
				int i = startIndex - 1;
				return new FcpLifVserverRawMeasurements(
						PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
						PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
						PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
						PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
						PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
						PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
						PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
						PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
						PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
						PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
			}
		};


		sampleParser = new ObjectParser<FcpLifVserverMeasurements>() {
			public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, FcpLifVserverMeasurements objectData) throws TableException {
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgOtherLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getThroughput());
			}

			public FcpLifVserverMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
				int i = startIndex - 1;
				return new FcpLifVserverMeasurements(
						PerformanceTableUtils.getFloatNullable(resultSet, ++i),
						PerformanceTableUtils.getFloatNullable(resultSet, ++i),
						PerformanceTableUtils.getFloatNullable(resultSet, ++i),
						PerformanceTableUtils.getFloatNullable(resultSet, ++i),
						PerformanceTableUtils.getFloatNullable(resultSet, ++i),
						PerformanceTableUtils.getFloatNullable(resultSet, ++i),
						PerformanceTableUtils.getFloatNullable(resultSet, ++i),
						PerformanceTableUtils.getFloatNullable(resultSet, ++i),
						PerformanceTableUtils.getFloatNullable(resultSet, ++i),
						PerformanceTableUtils.getFloatNullable(resultSet, ++i),
						PerformanceTableUtils.getFloatNullable(resultSet, ++i));
			}
		};
	}

	public static ObjectParser<FcpLifVserverRawMeasurements> getRawParser() {
		return rawParser;
	}

	public static ObjectParser<FcpLifVserverMeasurements> getSummaryParser() {
		return sampleParser;
	}

	public static ObjectParser<FcpLifVserverMeasurements> getSampleParser() {
		return sampleParser;
	}

}
