package com.netapp.oci.netapp.performance.fcplif.interfaces.data;

import java.io.Serializable;
/**
 * @author Anton Sergeev
 */
public class FcpLifMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * Average latency for FCP operations.
     */
    private float avgLatency;

    /**
     * Average Read latency for FCP operations.
     */
    private float avgReadLatency;

    /**
     * Average Write latency for FCP operations.
     */
    private float avgWriteLatency;

    /**
     * Average Other latency for FCP operations.
     */
    private float avgOtherLatency;
    
    /**
     * Total number of operations
     */
    private float totalOps;
    
    /**
     * Number of operations that are not read or write.
     */
    private float otherOps;

    /**
     * Amount of data read from the storage system
     */
    private float readData;

    /**
     * Number of read operations
     */
    private float readOps;

    /**
     * Amount of data written to the storage system
     */
    private float writeData;

    /**
     * Number of write operations
     */
    private float writeOps;

    /**
     * Lif through put = readData + writeData
     */
    private float throughput;

    public FcpLifMeasurements() {
    }

    public FcpLifMeasurements(float avgLatency, float totalOps, float otherOps, float readData, float readOps, float writeData,
            float writeOps, float avgOtherLatency, float avgReadLatency, float avgWriteLatency, 
                                float throughput) {
    	this.avgLatency = avgLatency;
        this.avgReadLatency = avgReadLatency;
        this.avgWriteLatency = avgWriteLatency;
        this.avgOtherLatency = avgOtherLatency;
    	this.totalOps = totalOps;
        this.otherOps = otherOps;
        this.readData = readData;
        this.readOps = readOps;
        this.writeData = writeData;
        this.writeOps = writeOps;
        this.throughput = throughput;
    }
    
    public float getAvgLatency() {
		return avgLatency;
	}

	public void setAvgLatency(float avgLatency) {
		this.avgLatency = avgLatency;
	}

    public float getAvgReadLatency() {
        return avgReadLatency;
    }

    public void setAvgReadLatency(float avgReadLatency) {
        this.avgReadLatency = avgReadLatency;
    }

    public float getAvgWriteLatency() {
        return avgWriteLatency;
    }

    public void setAvgWriteLatency(float avgWriteLatency) {
        this.avgWriteLatency = avgWriteLatency;
    }

    public float getAvgOtherLatency() {
        return avgOtherLatency;
    }

    public void setAvgOtherLatency(float avgOtherLatency) {
        this.avgOtherLatency = avgOtherLatency;
    }

	public float getTotalOps() {
		return totalOps;
	}

	public void setTotalOps(float totalOps) {
		this.totalOps = totalOps;
	}

	public float getOtherOps() {
        return otherOps;
    }

    public void setOtherOps(float otherOps) {
        this.otherOps = otherOps;
    }

    public float getReadData() {
        return readData;
    }

    public void setReadData(float readData) {
        this.readData = readData;
    }

    public float getReadOps() {
        return readOps;
    }

    public void setReadOps(float readOps) {
        this.readOps = readOps;
    }

    public float getWriteData() {
        return writeData;
    }

    public void setWriteData(float writeData) {
        this.writeData = writeData;
    }

    public float getWriteOps() {
        return writeOps;
    }

    public void setWriteOps(float writeOps) {
        this.writeOps = writeOps;
    }

    public float getThroughput() {
        return throughput;
    }

    public void setThroughput(float throughput) {
        this.throughput = throughput;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof FcpLifMeasurements)) return false;

        FcpLifMeasurements that = (FcpLifMeasurements) o;
        if (Float.compare(that.avgLatency, avgLatency) != 0) return false;
        if (Float.compare(that.avgReadLatency, avgReadLatency) != 0) return false;
        if (Float.compare(that.avgWriteLatency, avgWriteLatency) != 0) return false;
        if (Float.compare(that.avgOtherLatency, avgOtherLatency) != 0) return false;
        if (Float.compare(that.totalOps, totalOps) != 0) return false;
        if (Float.compare(that.otherOps, otherOps) != 0) return false;
        if (Float.compare(that.readData, readData) != 0) return false;
        if (Float.compare(that.readOps, readOps) != 0) return false;
        if (Float.compare(that.writeData, writeData) != 0) return false;
        if (Float.compare(that.writeOps, writeOps) != 0) return false;
        if (Float.compare(that.throughput, throughput) != 0) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = (otherOps != +0.0f ? Float.floatToIntBits(otherOps) : 0);
        result = 31 * result + (avgLatency != +0.0f ? Float.floatToIntBits(avgLatency) : 0);
        result = 31 * result + (avgReadLatency != +0.0f ? Float.floatToIntBits(avgReadLatency) : 0);
        result = 31 * result + (avgWriteLatency != +0.0f ? Float.floatToIntBits(avgWriteLatency) : 0);
        result = 31 * result + (avgOtherLatency != +0.0f ? Float.floatToIntBits(avgOtherLatency) : 0);
        result = 31 * result + (totalOps != +0.0f ? Float.floatToIntBits(totalOps) : 0);
        result = 31 * result + (readData != +0.0f ? Float.floatToIntBits(readData) : 0);
        result = 31 * result + (readOps != +0.0f ? Float.floatToIntBits(readOps) : 0);
        result = 31 * result + (writeData != +0.0f ? Float.floatToIntBits(writeData) : 0);
        result = 31 * result + (writeOps != +0.0f ? Float.floatToIntBits(writeOps) : 0);
        result = 31 * result + (throughput != +0.0f ? Float.floatToIntBits(throughput) : 0);
        return result;
    }

    @Override
    public String toString() {
        return "FcpLifMeasurements{" +
        		"avgLatency=" + avgLatency +
                "avgReadLatency=" + avgReadLatency +
                "avgWriteLatency=" + avgWriteLatency +
                "avgOtherLatency=" + avgOtherLatency +
        		", totalOps=" + totalOps +
                ", otherOps=" + otherOps +
                ", readData=" + readData +
                ", readOps=" + readOps +
                ", writeData=" + writeData +
                ", writeOps=" + writeOps +
                ", throughput=" + throughput +
                '}';
    }
}
