package com.netapp.oci.netapp.performance.fcplif.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

import javax.ejb.Remote;
import java.util.Collection;

/**
 * @author Anton Sergeev
 */
@Remote
public interface FcpLifPerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//FcpLifPerformanceBean!com.netapp.oci.netapp.performance.fcplif.interfaces.remote.FcpLifPerformanceRemote";

    /**
     * The Acq use this method to report on FcpLif performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param vserverKey - the vserver Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw fcplif samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws NetappInterfaceException if failed to report
     */
    void reportFcpLifSamples(Long datasourceId, 
                             String clusterKey, 
                             String vserverKey, 
                             long time, 
                             Collection<RawSample<FcpLifMeasurements>> rawSamples, 
                             long leaseTime, boolean isHistorical) throws NetappInterfaceException;

    /**
     * The Acq use this method to report on FcpLif raw performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param vserverKey - the vserver Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw fcplif samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     *
     * @throws NetappInterfaceException if failed to report
     */
    void reportFcpLifRawSamples(Long datasourceId, 
                                String clusterKey, 
                                String vserverKey, 
                                long time, 
                                Collection<RawSample<FcpLifRawMeasurements>> rawSamples, 
                                long leaseTime) throws NetappInterfaceException;
    
}
