package com.netapp.oci.netapp.performance.fcplif.interfaces.remote;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.Remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

@Remote
public interface FcpLifVserverPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//FcpLifVserverPerformanceQueryBean!com.netapp.oci.netapp.performance.fcplif.interfaces.remote.FcpLifVserverPerformanceQueryRemote";

    /**
     * Gets fcplifVserver samples for period of time
     *
     * @param fcpLifIds set of fcplif ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of fcplif id to list ordered by time of fcplif samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<FcpLifVserverMeasurements>>> getSamples(Set<Long> fcpLifIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * Gets fcplifVserver samples for period of time
     *
     * @param containerId container id
     * @param fcpLifIds set of fcplif ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of fcplif id to list ordered by time of fcplif samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<FcpLifVserverMeasurements>>> getSamples(Long containerId, Set<Long> fcpLifIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets fcplifVserver samples for period of time
     *
     * @param containerId container id
     * @param fcpLifId fcpLif id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of fcplif samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    public List<SampleInfo<FcpLifVserverMeasurements>> getSamples(Long containerId, Long fcpLifId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given fcpLif id's (fcpLifIds). The map returned will contain only fcpLif ids that has a sample.
     * FCP Lif ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the fcpLif ids passed.
     *
     * @param vServerId vServer id
     * @param fcpLifIds set of fcp Lif ids
     * @return map of fcpLif id to latest sample SampleInfo<FcpLifMeasurements> . FCPLif ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<FcpLifVserverMeasurements>> getLatestSample(Long vServerId, Set<Long> fcpLifIds) throws NetappInterfaceException;

    /**
     * Gets fcplifVserver summaries for fcplif ids
     *
     * @param vServerIds set of fcplif ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of fcplif id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long,  List<Summary<FcpLifVserverMeasurements>>> getSummaries(Set<Long> vServerIds, Date fromTime, Date toTime) throws NetappInterfaceException;

   /**
     * Gets fcplifVserver summaries for fcplif ids
     *
     * @param clusterId cluster id
     * @param vServerIds set of fcplif ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of fcplif id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<FcpLifVserverMeasurements>>> getSummaries(Long clusterId, Set<Long> vServerIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
