package com.netapp.oci.netapp.performance.fcplif.interfaces.remote;

import java.util.Collection;

import javax.ejb.Remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifRawMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

@Remote
public interface FcpLifVserverPerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//FcpLifVserverPerformanceBean!com.netapp.oci.netapp.performance.fcplif.interfaces.remote.FcpLifVserverPerformanceRemote";

    /**
     * The Acq use this method to report on FcpLifVserver performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw fcplif samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws NetappInterfaceException if failed to report
     */
    void reportFcpLifVserverSamples(Long datasourceId, 
                             String clusterKey, 
                             long time, 
                             Collection<RawSample<FcpLifVserverMeasurements>> rawSamples, 
                             long leaseTime, boolean isHistorical) throws NetappInterfaceException;

    /**
     * The Acq use this method to report on FcpLifVserver raw performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw fcplif samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     *
     * @throws NetappInterfaceException if failed to report
     */
    void reportFcpLifVserverRawSamples(Long datasourceId, 
                                String clusterKey, 
                                long time, 
                                Collection<RawSample<FcpLifVserverRawMeasurements>> rawSamples, 
                                long leaseTime) throws NetappInterfaceException;

}
