/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.fcpport;

import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortMeasurements;
import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @author Nancy Walia
 */

public class FcpPortMetaData {
    public static final Column[] RAW_COLUMNS = new Column[]{
    	new Column("avg_other_latency", "bigint unsigned"),
    	new Column("avg_read_latency", "bigint unsigned"),
    	new Column("avg_write_latency", "bigint unsigned"),
        new Column("other_ops", "bigint unsigned"),
        new Column("read_ops", "bigint unsigned"),
        new Column("write_ops", "bigint unsigned")
    };

    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("otherOps", "float"),
            new Column("readData", "float"),
            new Column("readOps", "float"),
            new Column("writeData", "float"),
            new Column("writeOps", "float"),
            new Column("avgOtherLatency", "float"),
            new Column("avgReadLatency", "float"),
            new Column("avgWriteLatency", "float"),
            new Column("totalOps", "float"),
            new Column("readThroughput", "float"),
            new Column("writeThroughput", "float"),
            new Column("throughput", "float"),
            new Column("utilization", "float")
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("otherOps", "float"),
            new Column("readData", "float"),
            new Column("readOps", "float"),
            new Column("writeData", "float"),
            new Column("writeOps", "float"),
            new Column("avgOtherLatency", "float"),
            new Column("avgReadLatency", "float"),
            new Column("avgWriteLatency", "float"),
            new Column("totalOps", "float"),
            new Column("readThroughput", "float"),
            new Column("writeThroughput", "float"),
            new Column("throughput", "float"),
            new Column("utilization", "float")
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("otherOps")},
            {new RatioMethod("readOps")},
            {new RatioMethod("writeOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("otherOps"),
            new RatioMethod("readOps"),
            new RatioMethod("writeOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG
    };

    private static ObjectParser<FcpPortRawMeasurements> rawParser;
    private static ObjectParser<FcpPortMeasurements> sampleParser;

    static {
        rawParser = new ObjectParser<FcpPortRawMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, FcpPortRawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
            }

            public FcpPortRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new FcpPortRawMeasurements(
                		PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                		PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                		PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                		PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                		PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));

            }
        };
    	
        sampleParser = new ObjectParser<FcpPortMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, FcpPortMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUtilization());
            }

            public FcpPortMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new FcpPortMeasurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<FcpPortRawMeasurements> getRawParser() {
        return rawParser;
    }
        
    public static ObjectParser<FcpPortMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<FcpPortMeasurements> getSampleParser() {
        return sampleParser;
    }
}
