/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.fcpport.interfaces.data;

import java.io.Serializable;

/**
 * @author Nancy Walia
 */
public class FcpPortMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * Number of operations that are not read or write.
     */
    private float otherOps;

    /**
     * Amount of data read from the storage system
     */
    private float readData;

    /**
     * Number of read operations
     */
    private float readOps;

    /**
     * Amount of data written to the storage system
     */
    private float writeData;

    /**
     * Number of write operations
     */
    private float writeOps;

    /**
     * Number of total operations = readOps + writeOps + otherOps
     */
    private float totalOps;

    /**
     * Average latency on the fcp port
     */
    private float avgOtherLatency;

    /**
     * Average read latency on the fcp port
     */
    private float avgReadLatency;

    /**
     * Average write latency on the fcp port
     */    
    private float avgWriteLatency;

    /**
     *  Sum of fcp_lif read_data  ops per second at this port
     */
    private float readThroughput;

    /**
     *  Sum of fcp_lif write_data  ops per second at this port
     */
    private float writeThroughput;

    /**
     *  Sum of read and write throughput this port
     */
    private float throughput;

    /**
     * opm_throughput / link_speed
     */
    private float utilization;
    
    public FcpPortMeasurements() {
    }
    
    public FcpPortMeasurements(float otherOps, float readData, float readOps, float writeData, float writeOps,
                               float totalOps, float avgOtherLatency, float avgReadLatency, float avgWriteLatency,
                               float readThroughput, float writeThroughput, float throughput, float utilization) {
        this.otherOps = otherOps;
        this.readData = readData;
        this.readOps = readOps;
        this.writeData = writeData;
        this.writeOps = writeOps;
        this.totalOps = totalOps;
        this.avgOtherLatency = avgOtherLatency;
        this.avgReadLatency = avgReadLatency;
        this.avgWriteLatency = avgWriteLatency;
        this.readThroughput = readThroughput;
        this.writeThroughput = writeThroughput;
        this.throughput = throughput;
        this.utilization = utilization;
    }

    public float getOtherOps() {
        return otherOps;
    }

    public void setOtherOps(float otherOps) {
        this.otherOps = otherOps;
    }

    public float getTotalOps() {
        return totalOps;
    }

    public void setTotalOps(float totalOps) {
        this.totalOps = totalOps;
    }

    public float getReadData() {
        return readData;
    }

    public void setReadData(float readData) {
        this.readData = readData;
    }

    public float getReadOps() {
        return readOps;
    }

    public void setReadOps(float readOps) {
        this.readOps = readOps;
    }

    public float getWriteData() {
        return writeData;
    }

    public void setWriteData(float writeData) {
        this.writeData = writeData;
    }

    public float getWriteOps() {
        return writeOps;
    }

    public void setWriteOps(float writeOps) {
        this.writeOps = writeOps;
    }
    
    public float getAvgOtherLatency() {
		return avgOtherLatency;
	}

	public void setAvgOtherLatency(float avgOtherLatency) {
		this.avgOtherLatency = avgOtherLatency;
	}

	public float getAvgReadLatency() {
		return avgReadLatency;
	}

	public void setAvgReadLatency(float avgReadLatency) {
		this.avgReadLatency = avgReadLatency;
	}

	public float getAvgWriteLatency() {
		return avgWriteLatency;
	}

	public void setAvgWriteLatency(float avgWriteLatency) {
		this.avgWriteLatency = avgWriteLatency;
	}

    public float getReadThroughput() {
        return readThroughput;
    }

    public void setReadThroughput(float readThroughput) {
        this.readThroughput = readThroughput;
    }

    public float getWriteThroughput() {
        return writeThroughput;
    }

    public void setWriteThroughput(float writeThroughput) {
        this.writeThroughput = writeThroughput;
    }

    public float getThroughput() {
        return throughput;
    }

    public void setThroughput(float throughput) {
        this.throughput = throughput;
    }

    public float getUtilization() {
        return utilization;
    }

    public void setUtilization(float utilization) {
        this.utilization = utilization;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        FcpPortMeasurements that = (FcpPortMeasurements) o;

        return Float.compare(that.otherOps, otherOps) == 0
                && Float.compare(that.readData, readData) == 0
                && Float.compare(that.readOps, readOps) == 0
                && Float.compare(that.writeData, writeData) == 0
                && Float.compare(that.writeOps, writeOps) == 0
                && Float.compare(that.totalOps, totalOps) == 0
                && Float.compare(that.avgOtherLatency, avgOtherLatency) == 0
                && Float.compare(that.avgReadLatency, avgReadLatency) == 0
                && Float.compare(that.avgWriteLatency, avgWriteLatency) == 0
                && Float.compare(that.readThroughput, readThroughput) == 0
                && Float.compare(that.writeThroughput, writeThroughput) == 0
                && Float.compare(that.throughput, throughput) == 0
                && Float.compare(that.utilization, utilization) == 0;
    }

    @Override
    public int hashCode() {
        int result = (otherOps != +0.0f ? Float.floatToIntBits(otherOps) : 0);
        result = 31 * result + (readData != +0.0f ? Float.floatToIntBits(readData) : 0);
        result = 31 * result + (readOps != +0.0f ? Float.floatToIntBits(readOps) : 0);
        result = 31 * result + (writeData != +0.0f ? Float.floatToIntBits(writeData) : 0);
        result = 31 * result + (writeOps != +0.0f ? Float.floatToIntBits(writeOps) : 0);
        result = 31 * result + (totalOps != +0.0f ? Float.floatToIntBits(totalOps) : 0);
        result = 31 * result + (avgOtherLatency != +0.0f ? Float.floatToIntBits(avgOtherLatency) : 0);
        result = 31 * result + (avgReadLatency != +0.0f ? Float.floatToIntBits(avgReadLatency) : 0);
        result = 31 * result + (avgWriteLatency != +0.0f ? Float.floatToIntBits(avgWriteLatency) : 0);
        result = 31 * result + (readThroughput != +0.0f ? Float.floatToIntBits(readThroughput) : 0);
        result = 31 * result + (writeThroughput != +0.0f ? Float.floatToIntBits(writeThroughput) : 0);
        result = 31 * result + (throughput != +0.0f ? Float.floatToIntBits(throughput) : 0);
        result = 31 * result + (utilization != +0.0f ? Float.floatToIntBits(utilization) : 0);
        return result;
    }

    @Override
    public String toString() {
        return "FcpPortMeasurements{" +
                "otherOps=" + otherOps +
                ", readData=" + readData +
                ", readOps=" + readOps +
                ", writeData=" + writeData +
                ", writeOps=" + writeOps +
                ", writeOps=" + totalOps +
                ", avgOtherLatency=" + avgOtherLatency +
                ", avgReadLatency=" + avgReadLatency +
                ", avgWriteLatency=" + avgWriteLatency +
                ", avgWriteLatency=" + readThroughput +
                ", avgWriteLatency=" + writeThroughput +
                ", avgWriteLatency=" + throughput +
                ", avgWriteLatency=" + utilization +
                '}';
    }
}

