package com.netapp.oci.netapp.performance.fcpport.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Nancy Walia
 */

@Remote
public interface FcpPortPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//FcpPortPerformanceQueryRemote!com.netapp.oci.netapp.performance.fcpport.interfaces.remote.FcpPortPerformanceQueryRemote";

    /**
     * Gets fcpPort samples for period of time
     *
     * @param fcpPortIds set of fcpPort ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of fcpPort id to list ordered by time of fcpPort samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<FcpPortMeasurements>>> getSamples(Set<Long> fcpPortIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets fcpPort samples for period of time
     *
     * @param containerId container id
     * @param fcpPortIds set of fcpPort ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of fcpPort id to list ordered by time of fcpPort samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<FcpPortMeasurements>>> getSamples(Long containerId, Set<Long> fcpPortIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets fcpPort samples for period of time
     *
     * @param containerId container id
     * @param fcpPortId fcpPort id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of fcpPort samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if anything went wrong
     */
    public List<SampleInfo<FcpPortMeasurements>> getSamples(Long containerId, Long fcpPortId, Date fromTime, Date toTime) throws NetappInterfaceException ;

    /**
     * Retrieve the latest sample for the given fcpPort id's (fcpPortIds). The map returned will contain only fcpPort ids that has a sample.
     * fcpPort ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the fcpPort ids passed.
     *
     * @param clusterId cluster id
     * @param fcpPortIds set of fcpPort ids
     * @return map of fcpPort id to latest sample SampleInfo<FcpPortMeasurements>. fcpPort ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<FcpPortMeasurements>> getLatestSample(Long clusterId, Set<Long> fcpPortIds) throws NetappInterfaceException;

    /**
     * Gets fcpPort summaries for fcpPort ids
     *
     * @param fcpPortIds set of fcpPort ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of fcpPort id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<FcpPortMeasurements>>> getSummaries(Set<Long> fcpPortIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets fcpPort summaries for fcpPort ids
     *
     * @param clusterId cluster id
     * @param fcpPortIds set of fcpPort ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of fcpPort id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<FcpPortMeasurements>>> getSummaries(Long clusterId, Set<Long> fcpPortIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
