package com.netapp.oci.netapp.performance.fcpport.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortMeasurements;
import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

import javax.ejb.Remote;
import java.util.Collection;

/**
 * @author Nancy Walia
 */
@Remote
public interface FcpPortPerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//FcpPortPerformanceBean!com.netapp.oci.netapp.performance.fcpport.interfaces.remote.FcpPortPerformanceRemote";

    /**
     * The Acq use this method to report on FcpPort performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluter Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw fcpPort samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportFcpPortSamples(Long datasourceId, String clusterKey, long time, Collection<RawSample<FcpPortMeasurements>> rawSamples, long leaseTime, boolean isHistorical) throws NetappInterfaceException;
    
    /**
     * The Acq use this method to report on FcpPort raw performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluter Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw fcpPort samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     *
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportFcpPortRawSamples(Long datasourceId, String clusterKey, long time, Collection<RawSample<FcpPortRawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
    
}

