/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.iscsilif.interfaces.data;

import java.io.Serializable;

/**
 * @author Al Velella
 *         Date: Jul 31, 2012
 *         Time: 4:17:18 PM
 */
public class IscsiLifMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * Number of blocks sent from iSCSI sessions on this LIF
     */
    private long dataInBlocks;

    /**
     * Number of blocks received from iSCSI sessions on this LIF
     */
    private long dataOutBlocks;

    /**
     * iSCSI read operations per second on this LIF
     */
    private float iscsiReadOps;

    /**
     * iSCSI write operations per second on this LIF
     */
    private float iscsiWriteOps;

    /**
     * iSCSI other operations per second on this LIF
     */
    private float iscsiOtherOps;

    /**
     * Number of protocol errors from iSCSI sessions on this LIF
     */
    private float protocolErrors;

    /**
     * Number of read data from iSCSI sessions on this LIF
     */
    private float readData;

    /**
     * Number of write data to iSCSI sessions on this LIF
     */
    private float writeData;

    /**
     * command transfered from iSCSI sessions on this LIF
     */
    private long cmdTransfered;

    /**
     * Average latency through iSCSI sessions on this LIF
     */
    private float avgOtherLatency;

    /**
     * Average Read latency through iSCSI sessions on this LIF
     */
    private float avgReadLatency;

    /**
     * Average write latency through iSCSI sessions on this LIF
     */
    private float avgWriteLatency;

    /**
     * Average latency through iSCSI sessions on this LIF
     */
    private float avgLatency;

    /**
     *  Oerations per second on this LIF
     */
    private float ops;

    /**
     * Throughput at on this LIF
     */
    private float throughput;
    
    public IscsiLifMeasurements() {
    }

    public IscsiLifMeasurements(long dataInBlocks, long dataOutBlocks, float iscsiReadOps, float iscsiWriteOps,
                                float protocolErrors, float readData, float writeData, long cmdTransfered, float avgLatency,
                                float avgOtherLatency, float avgReadLatency, float avgWriteLatency, float iscsiOtherOps,
                                float ops, float throughput) {
        this.dataInBlocks = dataInBlocks;
        this.dataOutBlocks = dataOutBlocks;
        this.iscsiReadOps = iscsiReadOps;
        this.iscsiWriteOps = iscsiWriteOps;
        this.protocolErrors = protocolErrors;
        this.readData = readData;
        this.writeData = writeData;
        this.cmdTransfered = cmdTransfered;
        this.avgLatency = avgLatency;
        this.avgOtherLatency = avgOtherLatency;
        this.avgReadLatency = avgReadLatency;
        this.avgWriteLatency = avgWriteLatency;
        this.iscsiOtherOps = iscsiOtherOps;
        this.ops = ops;
        this.throughput = throughput;
    }

    public long getDataInBlocks() {
        return dataInBlocks;
    }

    public void setDataInBlocks(long dataInBlocks) {
        this.dataInBlocks = dataInBlocks;
    }

    public long getDataOutBlocks() {
        return dataOutBlocks;
    }

    public void setDataOutBlocks(long dataOutBlocks) {
        this.dataOutBlocks = dataOutBlocks;
    }

    public float getIscsiReadOps() {
        return iscsiReadOps;
    }

    public void setIscsiReadOps(float iscsiReadOps) {
        this.iscsiReadOps = iscsiReadOps;
    }

    public float getIscsiOtherOps() {
        return iscsiOtherOps;
    }

    public void setIscsiOtherOps(float iscsiOtherOps) {
        this.iscsiOtherOps = iscsiOtherOps;
    }

    public float getIscsiWriteOps() {
        return iscsiWriteOps;
    }

    public void setIscsiWriteOps(float iscsiWriteOps) {
        this.iscsiWriteOps = iscsiWriteOps;
    }

    public float getProtocolErrors() {
        return protocolErrors;
    }

    public void setProtocolErrors(float protocolErrors) {
        this.protocolErrors = protocolErrors;
    }

    
    public float getReadData() {
		return readData;
	}

	public void setReadData(float readData) {
		this.readData = readData;
	}

	public float getWriteData() {
		return writeData;
	}

	public void setWriteData(float writeData) {
		this.writeData = writeData;
	}

	public long getCmdTransfered() {
		return cmdTransfered;
	}

	public void setCmdTransfered(long cmdTransfered) {
		this.cmdTransfered = cmdTransfered;
	}

	public float getAvgLatency() {
		return avgLatency;
	}

	public void setAvgLatency(float avgLatency) {
		this.avgLatency = avgLatency;
	}

    public float getAvgOtherLatency() {
        return avgOtherLatency;
    }

    public void setAvgOtherLatency(float avgOtherLatency) {
        this.avgOtherLatency = avgOtherLatency;
    }
    public float getAvgReadLatency() {
        return avgReadLatency;
    }

    public void setAvgReadLatency(float avgReadLatency) {
        this.avgReadLatency = avgReadLatency;
    }
    public float getAvgWriteLatency() {
        return avgWriteLatency;
    }

    public void setAvgWriteLatency(float avgWriteLatency) {
        this.avgWriteLatency = avgWriteLatency;
    }
    public float getOps() {
        return ops;
    }

    public void setOps(float ops) {
        this.ops = ops;
    }

    public float getThroughput() {
        return throughput;
    }

    public void setThroughput(float throughput) {
        this.throughput = throughput;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        IscsiLifMeasurements that = (IscsiLifMeasurements) o;

        return that.dataInBlocks ==  dataInBlocks 
                && that.dataOutBlocks == dataOutBlocks
                && Float.compare(that.iscsiReadOps, iscsiReadOps) == 0
                && Float.compare(that.iscsiWriteOps, iscsiWriteOps) == 0
                && Float.compare(that.iscsiOtherOps, iscsiOtherOps) == 0
                && Float.compare(that.protocolErrors, protocolErrors) == 0
                && Float.compare(that.readData, readData) == 0
                && Float.compare(that.writeData, writeData) == 0
                && that.cmdTransfered == cmdTransfered
                && Float.compare(that.avgLatency, avgLatency) == 0
                && Float.compare(that.avgReadLatency, avgReadLatency) == 0
                && Float.compare(that.avgWriteLatency, avgWriteLatency) == 0
                && Float.compare(that.avgOtherLatency, avgOtherLatency) == 0
                && Float.compare(that.ops, ops) == 0
                && Float.compare(that.throughput, throughput) == 0;
    }

    @Override
    public int hashCode() {
        int result = (dataInBlocks != +0.0f ? Float.floatToIntBits(dataInBlocks) : 0);
        result = 31 * result + (int) (dataInBlocks ^ dataInBlocks >>> 32);
        result = 31 * result + (int) (dataOutBlocks ^ dataOutBlocks >>> 32);
        result = 31 * result + (iscsiReadOps != +0.0f ? Float.floatToIntBits(iscsiReadOps) : 0);
        result = 31 * result + (iscsiWriteOps != +0.0f ? Float.floatToIntBits(iscsiWriteOps) : 0);
        result = 31 * result + (iscsiOtherOps != +0.0f ? Float.floatToIntBits(iscsiOtherOps) : 0);
        result = 31 * result + (protocolErrors != +0.0f ? Float.floatToIntBits(protocolErrors) : 0);
        result = 31 * result + (readData != +0.0f ? Float.floatToIntBits(readData) : 0);
        result = 31 * result + (writeData != +0.0f ? Float.floatToIntBits(writeData) : 0);
        result = 31 * result + (int) (cmdTransfered ^ cmdTransfered >>> 32);
        result = 31 * result + (avgLatency != +0.0f ? Float.floatToIntBits(avgLatency) : 0);
        result = 31 * result + (avgReadLatency != +0.0f ? Float.floatToIntBits(avgReadLatency) : 0);
        result = 31 * result + (avgWriteLatency != +0.0f ? Float.floatToIntBits(avgWriteLatency) : 0);
        result = 31 * result + (avgOtherLatency != +0.0f ? Float.floatToIntBits(avgOtherLatency) : 0);
        result = 31 * result + (ops != +0.0f ? Float.floatToIntBits(ops) : 0);
        result = 31 * result + (throughput != +0.0f ? Float.floatToIntBits(throughput) : 0);

        return result;        
    }

    @Override
    public String toString() {
        return "IscsiLifMeasurements{" +
                "dataInBlocks=" + dataInBlocks +
                ", dataOutBlocks=" + dataOutBlocks +
                ", iscsiReadOps=" + iscsiReadOps +
                ", iscsiWriteOps=" + iscsiWriteOps +
                ", iscsiOtherOps=" + iscsiOtherOps +
                ", protocolErrors=" + protocolErrors +
                ", readData=" + readData +
                ", writeData=" + writeData +
                ", cmdTransfered=" + cmdTransfered +
                ", avgLatency=" + avgLatency +
                ", avgLatency=" + avgReadLatency +
                ", avgLatency=" + avgWriteLatency +
                ", avgLatency=" + avgOtherLatency +
                ", ops=" + ops +
                ", throughput=" + throughput +
                '}';
    }
}
