/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.iscsilif.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Al Velella
 *         Date: Jul 31, 2012
 *         Time: 5:31:02 PM
 */
@Remote
public interface IscsiLifPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//IscsiLifPerformanceQueryBean!com.netapp.oci.netapp.performance.iscsilif.interfaces.remote.IscsiLifPerformanceQueryRemote";

    /**
     * Gets iSCSI LIF samples for period of time
     *
     * @param iscsiLif set of iSCSI LIF ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of iSCSI LIF id to list ordered by time of iSCSI LIF samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<IscsiLifMeasurements>>> getSamples(Set<Long> iscsiLif, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets iSCSI LIF samples for period of time
     *
     * @param containerId
     * @param iscsiLif set of iSCSI LIF ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of iSCSI LIF id to list ordered by time of iSCSI LIF samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<IscsiLifMeasurements>>> getSamples(Long containerId, Set<Long> iscsiLif, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets iSCSI LIF samples for period of time
     *
     * @param containerId container id
     * @param iscsiLifId iscsLif id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return  list ordered by time of iSCSI LIF samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if anything went wrong
     */
     List<SampleInfo<IscsiLifMeasurements>> getSamples(Long containerId, Long iscsiLifId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given iscsiLif id's (iscsiLifIds).  The map returned will contain only iscsiLif ids that has a sample.
     * iscsiLif ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the iscsiLif ids passed.
     *
     * @param vServerId vServer id
     * @param iscsiLifIds set of iscsiLif ids
     * @return map of iscsiLif id to latest sample SampleInfo<IscsiLifMeasurements>. iscsiLif ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<IscsiLifMeasurements>> getLatestSample(Long vServerId, Set<Long> iscsiLifIds) throws NetappInterfaceException;

    /**
     * Get iSCSI LIF summaries for iscsiLif ids
     *
     * @param iscsiLif set of iSCSI LIF ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of iSCSI LIF id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<IscsiLifMeasurements>>> getSummaries(Set<Long> iscsiLif, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Get iSCSI LIF summaries for iscsiLif ids
     *
     * @param vServerId vServer id
     * @param iscsiLif set of iSCSI LIF ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of iSCSI LIF id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<IscsiLifMeasurements>>> getSummaries(Long vServerId, Set<Long> iscsiLif, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
