package com.netapp.oci.netapp.performance.lun;

import com.netapp.oci.netapp.performance.lun.interfaces.data.LunMeasurements;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.WmaBaseRated;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @author Anton Sergeev
 */
public class LunMetaData {
    public static final Column[] RAW_COLUMNS = new Column[]{
        new Column("avg_latency", "bigint unsigned"),
        new Column("avg_read_latency", "bigint unsigned"),
        new Column("avg_write_latency", "bigint unsigned"),
        new Column("other_ops", "bigint unsigned"),
        new Column("read_data", "bigint unsigned"),
        new Column("read_ops", "bigint unsigned"),
        new Column("total_ops", "bigint unsigned"),
        new Column("write_data", "bigint unsigned"),
        new Column("write_ops", "bigint unsigned"),
        new Column("read_partial_blocks", "bigint unsigned"),
        new Column("write_partial_blocks", "bigint unsigned"),
    };
	
    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("avgLatency", "float"),
            new Column("avgReadLatency", "float"),
            new Column("avgWriteLatency", "float"),
            new Column("otherOps", "float"),
            new Column("readData", "float"),
            new Column("readOps", "float"),
            new Column("readPartialBlocks", "float"),
            new Column("totalOps", "float"),
            new Column("writeData", "float"),
            new Column("writeOps", "float"),
            new Column("writePartialBlocks", "float"),
            new Column("opmLunThroughput", "float")
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("avgLatency", "float"),
            new Column("avgReadLatency", "float"),
            new Column("avgWriteLatency", "float"),
            new Column("otherOps", "float"),
            new Column("readData", "float"),
            new Column("readOps", "float"),
            new Column("readPartialBlocks", "float"),
            new Column("totalOps", "float"),
            new Column("writeData", "float"),
            new Column("writeOps", "float"),
            new Column("writePartialBlocks", "float"),
            new Column("opmLunThroughput", "float")
    };

    public static final Column[] DAILY_SUMMARY_COLUMNS = new Column[]{
            new Column("minAvgLatency", "float"),
            new Column("maxAvgLatency", "float"),
            new Column("avgLatency", "float"),
            new Column("minAvgReadLatency", "float"),
            new Column("maxAvgReadLatency", "float"),
            new Column("avgReadLatency", "float"),
            new Column("minAvgWriteLatency", "float"),
            new Column("maxAvgWriteLatency", "float"),
            new Column("avgWriteLatency", "float"),
            new Column("minReadPartialBlocks", "float"),
            new Column("maxReadPartialBlocks", "float"),
            new Column("readPartialBlocks", "float"),
            new Column("minWritePartialBlocks", "float"),
            new Column("maxWritePartialBlocks", "float"),
            new Column("writePartialBlocks", "float"),
            new Column("minReadOps", "float"),
            new Column("maxReadOps", "float"),
            new Column("readOps", "float"),
            new Column("minWriteOps", "float"),
            new Column("maxWriteOps", "float"),
            new Column("writeOps", "float"),
            new Column("minOtherOps", "float"),
            new Column("maxOtherOps", "float"),
            new Column("otherOps", "float"),
            new Column("minTotalOps", "float"),
            new Column("maxTotalOps", "float"),
            new Column("totalOps", "float"),
            new Column("minReadData", "float"),
            new Column("maxReadData", "float"),
            new Column("readData", "float"),
            new Column("minWriteData", "float"),
            new Column("maxWriteData", "float"),
            new Column("writeData", "float"),
            new Column("minOpmLunThroughput", "float"),
            new Column("maxOpmLunThroughput", "float"),
            new Column("opmLunThroughput", "float")
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {new RatioMethod("totalOps")},
            {new RatioMethod("readOps")},
            {new RatioMethod("writeOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("readOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("writeOps")},
            {SqlMethod.AVG}
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            new RatioMethod("totalOps"),
            new RatioMethod("readOps"),
            new RatioMethod("writeOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("readOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("writeOps"),
            SqlMethod.AVG
    };

    public static final SqlMethod[] DAILY_SUMMARY_METHODS = new SqlMethod[]{
            SqlMethod.M_MIN,//minAvgLatency
            SqlMethod.M_MAX,//maxAvgLatency
            new WmaBaseRated("totalOps"),//avgLatency
            SqlMethod.M_MIN,//minAvgReadLatency
            SqlMethod.M_MAX,//maxAvgReadLatency
            new WmaBaseRated("readOps"),//avgReadLatency
            SqlMethod.M_MIN,//minAvgWriteLatency
            SqlMethod.M_MAX,//maxAvgWriteLatency
            new WmaBaseRated("writeOps"),//avgWriteLatency
            SqlMethod.M_MIN,//minReadPartialBlocks
            SqlMethod.M_MAX,//maxReadPartialBlocks
            new WmaBaseRated("readOps"),//readPartialBlocks
            SqlMethod.M_MIN,//minWritePartialBlocks
            SqlMethod.M_MAX,//maxWritePartialBlocks
            new WmaBaseRated("writeOps"),//writePartialBlocks
            SqlMethod.M_MIN,//minReadOps
            SqlMethod.M_MAX,//maxReadOps
            SqlMethod.CMA,//readOps
            SqlMethod.M_MIN,//minWriteOps
            SqlMethod.M_MAX,//maxWriteOps
            SqlMethod.CMA,//writeOps
            SqlMethod.M_MIN,//minOtherOps
            SqlMethod.M_MAX,//maxOtherOps
            SqlMethod.CMA,//otherOps
            SqlMethod.M_MIN,//minTotalOps
            SqlMethod.M_MAX,//maxTotalOps
            SqlMethod.CMA,//totalOps
            SqlMethod.M_MIN,//minReadData
            SqlMethod.M_MAX,//maxReadData
            SqlMethod.CMA,//readData
            SqlMethod.M_MIN,//minWriteData
            SqlMethod.M_MAX,//maxWriteData
            SqlMethod.CMA,//writeData
            SqlMethod.M_MIN,//minOpmLunThroughput
            SqlMethod.M_MAX,//maxOpmLunThroughput
            SqlMethod.CMA,//opmLunThroughput
    };

    private static ObjectParser<LunRawMeasurements> rawParser;
    private static ObjectParser<LunMeasurements> sampleParser;
    private static ObjectParser<LunMeasurements> dailySummaryParser;

    static {
        sampleParser = new ObjectParser<LunMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, LunMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadPartialBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWritePartialBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmLunThroughput());
            }

            public LunMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new LunMeasurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };

        dailySummaryParser = new ObjectParser<LunMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, LunMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadPartialBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadPartialBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadPartialBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWritePartialBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWritePartialBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWritePartialBlocks());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmLunThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmLunThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmLunThroughput());
            }

            public LunMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                throw new UnsupportedOperationException("Reading of daily summary is not supported");
            }
        };

        rawParser = new ObjectParser<LunRawMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, LunRawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadPartialBlocks());                		
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWritePartialBlocks());
            }

            public LunRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new LunRawMeasurements(
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));

            }
        };
    }

    public static ObjectParser<LunMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<LunMeasurements> getSampleParser() {
        return sampleParser;
    }

    public static ObjectParser<LunRawMeasurements> getRawParser() {
        return rawParser;
    }

    public static ObjectParser<LunMeasurements> getDailySummaryParser() {
        return dailySummaryParser;
    }
    
}
