/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.lun.interfaces.data;

import java.io.Serializable;

/**
 * @author Anton Sergeev
 */
public class LunMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * Average latency in milliseconds for all operations on the LUN
     */
    private float avgLatency;

    /**
     * Average read latency in milliseconds for all operations on the LUN
     */
    private float avgReadLatency;

    /**
     * Average write latency in milliseconds for all operations on the LUN
     */
    private float avgWriteLatency;

    /**
     * Number of other operations
     */
    private float otherOps;

    /**
     * Read bytes
     */
    private float readData;

    /**
     * Number of read operations
     */
    private float readOps;

    /**
     * Percent of reads whose size is not a multiple of WAFL block size
     */
    private float readPartialBlocks;

    /**
     * Total number of operations on the LUN
     */
    private float totalOps;

    /**
     * Write bytes
     */
    private float writeData;

    /**
     * Number of write operations
     */
    private float writeOps;

    /**
     * Percent of writes whose size is not a multiple of WAFL block size
     */
    private float writePartialBlocks;

    /**
     * Synthetic counter = readData + writeData
     */
    private float opmLunThroughput;
    
    public LunMeasurements() {
    }

    public LunMeasurements(float avgLatency, float avgReadLatency, float avgWriteLatency, float otherOps, float readData,
                           float readOps, float readPartialBlocks, float totalOps, float writeData, float writeOps, 
                           float writePartialBlocks, float opmLunThroughput) {
        this.avgLatency = avgLatency;
        this.avgReadLatency = avgReadLatency;
        this.avgWriteLatency = avgWriteLatency;
        this.otherOps = otherOps;
        this.readData = readData;
        this.readOps = readOps;
        this.readPartialBlocks = readPartialBlocks;
        this.totalOps = totalOps;
        this.writeData = writeData;
        this.writeOps = writeOps;
        this.writePartialBlocks = writePartialBlocks;
        this.opmLunThroughput = opmLunThroughput;
    }

    public float getAvgLatency() {
        return avgLatency;
    }

    public void setAvgLatency(float avgLatency) {
        this.avgLatency = avgLatency;
    }

    public float getAvgReadLatency() {
        return avgReadLatency;
    }

    public void setAvgReadLatency(float avgReadLatency) {
        this.avgReadLatency = avgReadLatency;
    }

    public float getAvgWriteLatency() {
        return avgWriteLatency;
    }

    public void setAvgWriteLatency(float avgWriteLatency) {
        this.avgWriteLatency = avgWriteLatency;
    }

    public float getOtherOps() {
        return otherOps;
    }

    public void setOtherOps(float otherOps) {
        this.otherOps = otherOps;
    }

    public float getReadData() {
        return readData;
    }

    public void setReadData(float readData) {
        this.readData = readData;
    }

    public float getReadOps() {
        return readOps;
    }

    public void setReadOps(float readOps) {
        this.readOps = readOps;
    }

    public float getReadPartialBlocks() {
        return readPartialBlocks;
    }

    public void setReadPartialBlocks(float readPartialBlocks) {
        this.readPartialBlocks = readPartialBlocks;
    }

    public float getTotalOps() {
        return totalOps;
    }

    public void setTotalOps(float totalOps) {
        this.totalOps = totalOps;
    }

    public float getWriteData() {
        return writeData;
    }

    public void setWriteData(float writeData) {
        this.writeData = writeData;
    }

    public float getWriteOps() {
        return writeOps;
    }

    public void setWriteOps(float writeOps) {
        this.writeOps = writeOps;
    }

    public float getWritePartialBlocks() {
        return writePartialBlocks;
    }

    public void setWritePartialBlocks(float writePartialBlocks) {
        this.writePartialBlocks = writePartialBlocks;
    }
    
    public float getOpmLunThroughput() {
		return opmLunThroughput;
	}

	public void setOpmLunThroughput(float opmLunThroughput) {
		this.opmLunThroughput = opmLunThroughput;
	}

	@Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof LunMeasurements)) return false;

        LunMeasurements that = (LunMeasurements) o;

        if (Float.compare(that.avgLatency, avgLatency) != 0) return false;
        if (Float.compare(that.avgReadLatency, avgReadLatency) != 0) return false;
        if (Float.compare(that.avgWriteLatency, avgWriteLatency) != 0) return false;
        if (Float.compare(that.otherOps, otherOps) != 0) return false;
        if (Float.compare(that.readData, readData) != 0) return false;
        if (Float.compare(that.readOps, readOps) != 0) return false;
        if (Float.compare(that.readPartialBlocks, readPartialBlocks) != 0) return false;
        if (Float.compare(that.totalOps, totalOps) != 0) return false;
        if (Float.compare(that.writeData, writeData) != 0) return false;
        if (Float.compare(that.writeOps, writeOps) != 0) return false;
        if (Float.compare(that.writePartialBlocks, writePartialBlocks) != 0) return false;
        if (Float.compare(that.opmLunThroughput, opmLunThroughput) != 0) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = (avgLatency != +0.0f ? Float.floatToIntBits(avgLatency) : 0);
        result = 31 * result + (avgReadLatency != +0.0f ? Float.floatToIntBits(avgReadLatency) : 0);
        result = 31 * result + (avgWriteLatency != +0.0f ? Float.floatToIntBits(avgWriteLatency) : 0);
        result = 31 * result + (otherOps != +0.0f ? Float.floatToIntBits(otherOps) : 0);
        result = 31 * result + (readData != +0.0f ? Float.floatToIntBits(readData) : 0);
        result = 31 * result + (readOps != +0.0f ? Float.floatToIntBits(readOps) : 0);
        result = 31 * result + (readPartialBlocks != +0.0f ? Float.floatToIntBits(readPartialBlocks) : 0);
        result = 31 * result + (totalOps != +0.0f ? Float.floatToIntBits(totalOps) : 0);
        result = 31 * result + (writeData != +0.0f ? Float.floatToIntBits(writeData) : 0);
        result = 31 * result + (writeOps != +0.0f ? Float.floatToIntBits(writeOps) : 0);
        result = 31 * result + (writePartialBlocks != +0.0f ? Float.floatToIntBits(writePartialBlocks) : 0);
        result = 31 * result + (opmLunThroughput != +0.0f ? Float.floatToIntBits(opmLunThroughput) : 0);
        return result;
    }

    @Override
    public String toString() {
        return "LunMeasurements{" +
                "avgLatency=" + avgLatency +
                ", avgReadLatency=" + avgReadLatency +
                ", avgWriteLatency=" + avgWriteLatency +
                ", otherOps=" + otherOps +
                ", readData=" + readData +
                ", readOps=" + readOps +
                ", readPartialBlocks=" + readPartialBlocks +
                ", totalOps=" + totalOps +
                ", writeData=" + writeData +
                ", writeOps=" + writeOps +
                ", writePartialBlocks=" + writePartialBlocks +
                ", opmLunThroughput=" + opmLunThroughput +
                '}';
    }
}
