package com.netapp.oci.netapp.performance.lun.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Anton Sergeev
 */
@Remote
public interface LunPerformanceQueryRemote {

    String JNDI_NAME = "ejb:mega-app/server-core//LunPerformanceQueryBean!com.netapp.oci.netapp.performance.lun.interfaces.remote.LunPerformanceQueryRemote";

    /**
     * Gets lun samples for period of time
     *
     * @param lunIds set of lun ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of lun id to list ordered by time of lun samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<LunMeasurements>>> getSamples(Set<Long> lunIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets lun samples for period of time
     *
     * @param containerId container id
     * @param lunIds set of lun ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of lun id to list ordered by time of lun samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<LunMeasurements>>> getSamples(Long containerId, Set<Long> lunIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets lun samples for period of time
     *
     * @param containerId container id
     * @param lunId lun id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of lun samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<LunMeasurements>> getSamples(Long containerId, Long lunId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given lun id's (lunIds). The map returned will contain only lun ids that has a sample.
     * lun ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the lun ids passed.
     *
     * @param containerId
     * @param lunIds set of lun ids
     * @return map of lun id to latest sample SampleInfo<LunMeasurements>. lun ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<LunMeasurements>> getLatestSample(Long containerId, Set<Long> lunIds) throws NetappInterfaceException;

    /**
     * Gets lun summaries for lun ids
     *
     * @param lunIds set of lun ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of lun id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<LunMeasurements>>>  getSummaries(Set<Long> lunIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets lun summaries for lun ids
     *
     * @param lunIds set of lun ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of lun id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<LunMeasurements>>>  getSummaries(Long containerId, Set<Long> lunIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
