package com.netapp.oci.netapp.performance.lun.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunMeasurements;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

import javax.ejb.Remote;
import java.util.Collection;

/**
 * @author Anton Sergeev
 */
@Remote
public interface LunPerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//LunPerformanceBean!com.netapp.oci.netapp.performance.lun.interfaces.remote.LunPerformanceRemote";

    /**
     * The Acq use this method to report on LUN performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw lun samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws NetappInterfaceException if failed to report
     */
    void reportLunSamples(Long datasourceId, String clusterKey, long time, Collection<RawSample<LunMeasurements>> rawSamples, long leaseTime, boolean isHistorical) throws NetappInterfaceException;
    
    /**
     * The Acq use this method to report on LUN raw performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw lun samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     *
     * @throws NetappInterfaceException if failed to report
     */
    void reportLunRawSamples(Long datasourceId, String clusterKey, long time, Collection<RawSample<LunRawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
    
}
