/*
 * $Id$
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.oci.netapp.performance.namespace;

import com.netapp.oci.netapp.performance.namespace.interfaces.data.NamespaceMeasurements;
import com.netapp.oci.netapp.performance.namespace.interfaces.data.NamespaceRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.WmaBaseRated;

import java.sql.ResultSet;
import java.sql.SQLException;

public class NamespaceMetaData {
    public static final Column[] RAW_COLUMNS = new Column[]{
        new Column("avg_latency", "bigint unsigned"),
        new Column("avg_read_latency", "bigint unsigned"),
        new Column("avg_write_latency", "bigint unsigned"),
        new Column("avg_other_latency", "bigint unsigned"),
        new Column("total_data", "bigint unsigned"),
        new Column("read_data", "bigint unsigned"),
        new Column("write_data", "bigint unsigned"),
        new Column("total_ops", "bigint unsigned"),
        new Column("read_ops", "bigint unsigned"),
        new Column("write_ops", "bigint unsigned"),
        new Column("other_ops", "bigint unsigned")
    };
	
    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("avgLatency", "float"),
            new Column("avgReadLatency", "float"),
            new Column("avgWriteLatency", "float"),
            new Column("avgOtherLatency", "float"),
            new Column("totalData", "float"),
            new Column("readData", "float"),
            new Column("writeData", "float"),
            new Column("totalOps", "float"),
            new Column("readOps", "float"),
            new Column("writeOps", "float"),
            new Column("otherOps", "float")
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("avgLatency", "float"),
            new Column("avgReadLatency", "float"),
            new Column("avgWriteLatency", "float"),
            new Column("avgOtherLatency", "float"),
            new Column("totalData", "float"),
            new Column("readData", "float"),
            new Column("writeData", "float"),
            new Column("totalOps", "float"),
            new Column("readOps", "float"),
            new Column("writeOps", "float"),
            new Column("otherOps", "float")
    };

    public static final Column[] DAILY_SUMMARY_COLUMNS = new Column[]{
            new Column("minAvgLatency", "float"),
            new Column("maxAvgLatency", "float"),
            new Column("avgLatency", "float"),
            new Column("minAvgReadLatency", "float"),
            new Column("maxAvgReadLatency", "float"),
            new Column("avgReadLatency", "float"),
            new Column("minAvgWriteLatency", "float"),
            new Column("maxAvgWriteLatency", "float"),
            new Column("avgWriteLatency", "float"),
            new Column("minAvgOtherLatency", "float"),
            new Column("maxAvgOtherLatency", "float"),
            new Column("avgOtherLatency", "float"),
            new Column("minTotalData", "float"),
            new Column("maxTotalData", "float"),
            new Column("totalData", "float"),
            new Column("minReadData", "float"),
            new Column("maxReadData", "float"),
            new Column("readData", "float"),
            new Column("minWriteData", "float"),
            new Column("maxWriteData", "float"),
            new Column("writeData", "float"),
            new Column("minTotalOps", "float"),
            new Column("maxTotalOps", "float"),
            new Column("totalOps", "float"),
            new Column("minReadOps", "float"),
            new Column("maxReadOps", "float"),
            new Column("readOps", "float"),
            new Column("minWriteOps", "float"),
            new Column("maxWriteOps", "float"),
            new Column("writeOps", "float"),
            new Column("minOtherOps", "float"),
            new Column("maxOtherOps", "float"),
            new Column("otherOps", "float")
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {new RatioMethod("totalOps")},  // avgLatency
            {new RatioMethod("readOps")},   // avgReadLatency
            {new RatioMethod("writeOps")},  // avgWriteLatency
            {new RatioMethod("otherOps")},  // avgOtherLatency
            {SqlMethod.AVG},                // totalData
            {SqlMethod.AVG},                // readData
            {SqlMethod.AVG},                // writeData
            {SqlMethod.AVG},                // totalOps
            {SqlMethod.AVG},                // readOps
            {SqlMethod.AVG},                // writeOps
            {SqlMethod.AVG}                 // otherOps
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            new RatioMethod("totalOps"),    // avgLatency
            new RatioMethod("readOps"),     // avgReadLatency
            new RatioMethod("writeOps"),    // avgWriteLatency
            new RatioMethod("otherOps"),    // avgOtherLatency
            SqlMethod.AVG,                  // totalData
            SqlMethod.AVG,                  // readData
            SqlMethod.AVG,                  // writeData
            SqlMethod.AVG,                  // totalOps
            SqlMethod.AVG,                  // readOps
            SqlMethod.AVG,                  // writeOps
            SqlMethod.AVG                   // otherOps
    };

    public static final SqlMethod[] DAILY_SUMMARY_METHODS = new SqlMethod[]{
            SqlMethod.M_MIN,                // minAvgLatency
            SqlMethod.M_MAX,                // maxAvgLatency
            new WmaBaseRated("totalOps"),   // avgLatency
            SqlMethod.M_MIN,                // minAvgReadLatency
            SqlMethod.M_MAX,                // maxAvgReadLatency
            new WmaBaseRated("readOps"),    // avgReadLatency
            SqlMethod.M_MIN,                // minAvgWriteLatency
            SqlMethod.M_MAX,                // maxAvgWriteLatency
            new WmaBaseRated("writeOps"),   // avgWriteLatency
            SqlMethod.M_MIN,                // minAvgOtherLatency
            SqlMethod.M_MAX,                // maxAvgOtherLatency
            new WmaBaseRated("otherOps"),   // avgOtherLatency
            SqlMethod.M_MIN,                // minTotalData
            SqlMethod.M_MAX,                // maxTotalData
            SqlMethod.CMA,                  // totalData
            SqlMethod.M_MIN,                // minReadData
            SqlMethod.M_MAX,                // maxReadData
            SqlMethod.CMA,                  // readData
            SqlMethod.M_MIN,                // minWriteData
            SqlMethod.M_MAX,                // maxWriteData
            SqlMethod.CMA,                  // writeData
            SqlMethod.M_MIN,                // minTotalOps
            SqlMethod.M_MAX,                // maxTotalOps
            SqlMethod.CMA,                  // totalOps
            SqlMethod.M_MIN,                // minReadOps
            SqlMethod.M_MAX,                // maxReadOps
            SqlMethod.CMA,                  // readOps
            SqlMethod.M_MIN,                // minWriteOps
            SqlMethod.M_MAX,                // maxWriteOps
            SqlMethod.CMA,                  // writeOps
            SqlMethod.M_MIN,                // minOtherOps
            SqlMethod.M_MAX,                // maxOtherOps
            SqlMethod.CMA,                  // otherOps
    };

    private static ObjectParser<NamespaceRawMeasurements> rawParser;
    private static ObjectParser<NamespaceMeasurements> sampleParser;
    private static ObjectParser<NamespaceMeasurements> dailySummaryParser;

    static {
        sampleParser = new ObjectParser<NamespaceMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NamespaceMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
            }

            public NamespaceMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NamespaceMeasurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };

        dailySummaryParser = new ObjectParser<NamespaceMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NamespaceMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
            }

            public NamespaceMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                throw new UnsupportedOperationException("Reading of daily summary is not supported");
            }
        };

        rawParser = new ObjectParser<NamespaceRawMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NamespaceRawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
            }

            public NamespaceRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NamespaceRawMeasurements(
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<NamespaceMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<NamespaceMeasurements> getSampleParser() {
        return sampleParser;
    }

    public static ObjectParser<NamespaceRawMeasurements> getRawParser() {
        return rawParser;
    }

    public static ObjectParser<NamespaceMeasurements> getDailySummaryParser() {
        return dailySummaryParser;
    }
    
}
