/*
 * $Id$
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.oci.netapp.performance.namespace.interfaces.data;

import java.io.Serializable;

public class NamespaceMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * Average latency in milliseconds for all operations on the Namespace
     */
    private float avgLatency;

    /**
     * Average read latency in milliseconds for all operations on the Namespace
     */
    private float avgReadLatency;

    /**
     * Average write latency in milliseconds for all operations on the Namespace
     */
    private float avgWriteLatency;

    /**
     * Average other latency in milliseconds for all operations on the Namespace
     */
    private float avgOtherLatency;

    /**
     * Total bytes
     */
    private float totalData;

    /**
     * Read bytes
     */
    private float readData;

    /**
     * Write bytes
     */
    private float writeData;

    /**
     * Total number of operations on the Namespace
     */
    private float totalOps;

    /**
     * Number of read operations
     */
    private float readOps;

    /**
     * Number of write operations
     */
    private float writeOps;

    /**
     * Number of other operations
     */
    private float otherOps;


    public NamespaceMeasurements() {
    }

    public NamespaceMeasurements(float avgLatency, float avgReadLatency, float avgWriteLatency, float avgOtherLatency,
                                 float totalData, float readData, float writeData,
                                 float totalOps, float readOps, float writeOps, float otherOps) {
        this.avgLatency = avgLatency;
        this.avgReadLatency = avgReadLatency;
        this.avgWriteLatency = avgWriteLatency;
        this.avgOtherLatency = avgOtherLatency;
        this.totalData = totalData;
        this.readData = readData;
        this.writeData = writeData;
        this.totalOps = totalOps;
        this.readOps = readOps;
        this.writeOps = writeOps;
        this.otherOps = otherOps;
    }

    public float getAvgLatency() {
        return avgLatency;
    }

    public void setAvgLatency(float avgLatency) {
        this.avgLatency = avgLatency;
    }

    public float getAvgReadLatency() {
        return avgReadLatency;
    }

    public void setAvgReadLatency(float avgReadLatency) {
        this.avgReadLatency = avgReadLatency;
    }

    public float getAvgWriteLatency() {
        return avgWriteLatency;
    }

    public void setAvgWriteLatency(float avgWriteLatency) {
        this.avgWriteLatency = avgWriteLatency;
    }

    public float getAvgOtherLatency() {
        return avgOtherLatency;
    }

    public void setAvgOtherLatency(float avgOtherLatency) {
        this.avgOtherLatency = avgOtherLatency;
    }

    public float getTotalData() {
        return totalData;
    }

    public void setTotalData(float totalData) {
        this.totalData = totalData;
    }

    public float getReadData() {
        return readData;
    }

    public void setReadData(float readData) {
        this.readData = readData;
    }

    public float getWriteData() {
        return writeData;
    }

    public void setWriteData(float writeData) {
        this.writeData = writeData;
    }

    public float getTotalOps() {
        return totalOps;
    }

    public void setTotalOps(float totalOps) {
        this.totalOps = totalOps;
    }

    public float getReadOps() {
        return readOps;
    }

    public void setReadOps(float readOps) {
        this.readOps = readOps;
    }

    public float getWriteOps() {
        return writeOps;
    }

    public void setWriteOps(float writeOps) {
        this.writeOps = writeOps;
    }

    public float getOtherOps() {
        return otherOps;
    }

    public void setOtherOps(float otherOps) {
        this.otherOps = otherOps;
    }

	@Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof NamespaceMeasurements)) return false;

        NamespaceMeasurements that = (NamespaceMeasurements) o;

        if (Float.compare(that.avgLatency, avgLatency) != 0) return false;
        if (Float.compare(that.avgReadLatency, avgReadLatency) != 0) return false;
        if (Float.compare(that.avgWriteLatency, avgWriteLatency) != 0) return false;
        if (Float.compare(that.avgOtherLatency, avgOtherLatency) != 0) return false;
        if (Float.compare(that.totalOps, totalOps) != 0) return false;
        if (Float.compare(that.readData, readData) != 0) return false;
        if (Float.compare(that.writeData, writeData) != 0) return false;
        if (Float.compare(that.totalOps, totalOps) != 0) return false;
        if (Float.compare(that.readOps, readOps) != 0) return false;
        if (Float.compare(that.writeOps, writeOps) != 0) return false;
        if (Float.compare(that.otherOps, otherOps) != 0) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = (avgLatency != +0.0f ? Float.floatToIntBits(avgLatency) : 0);
        result = 31 * result + (avgReadLatency != +0.0f ? Float.floatToIntBits(avgReadLatency) : 0);
        result = 31 * result + (avgWriteLatency != +0.0f ? Float.floatToIntBits(avgWriteLatency) : 0);
        result = 31 * result + (avgOtherLatency != +0.0f ? Float.floatToIntBits(avgOtherLatency) : 0);
        result = 31 * result + (totalData != +0.0f ? Float.floatToIntBits(totalData) : 0);
        result = 31 * result + (readData != +0.0f ? Float.floatToIntBits(readData) : 0);
        result = 31 * result + (writeData != +0.0f ? Float.floatToIntBits(writeData) : 0);
        result = 31 * result + (totalOps != +0.0f ? Float.floatToIntBits(totalOps) : 0);
        result = 31 * result + (readOps != +0.0f ? Float.floatToIntBits(readOps) : 0);
        result = 31 * result + (writeOps != +0.0f ? Float.floatToIntBits(writeOps) : 0);
        result = 31 * result + (otherOps != +0.0f ? Float.floatToIntBits(otherOps) : 0);
        return result;
    }

    @Override
    public String toString() {
        return "NamespaceMeasurements{" +
                "avgLatency=" + avgLatency +
                ", avgReadLatency=" + avgReadLatency +
                ", avgWriteLatency=" + avgWriteLatency +
                ", avgOtherLatency=" + avgOtherLatency +
                ", totalData=" + totalData +
                ", readData=" + readData +
                ", writeData=" + writeData +
                ", totalOps=" + totalOps +
                ", readOps=" + readOps +
                ", writeOps=" + writeOps +
                ", otherOps=" + otherOps +
                '}';
    }
}
