/*
 * $Id$
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.oci.netapp.performance.namespace.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.namespace.interfaces.data.NamespaceMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Remote
public interface NamespacePerformanceQueryRemote {

    String JNDI_NAME = "ejb:mega-app/server-core//NamespacePerformanceQueryBean!com.netapp.oci.netapp.performance.namespace.interfaces.remote.NamespacePerformanceQueryRemote";

    /**
     * Gets Namespace samples for period of time
     *
     * @param namespaceIds set of Namespace ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of Namespace id to list ordered by time of Namespace samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<NamespaceMeasurements>>> getSamples(Set<Long> namespaceIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets Namespace samples for period of time
     *
     * @param containerId container id
     * @param namespaceIds set of Namespace ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of Namespace id to list ordered by time of Namespace samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<NamespaceMeasurements>>> getSamples(Long containerId, Set<Long> namespaceIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets Namespace samples for period of time
     *
     * @param containerId container id
     * @param namespaceId Namespace id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of Namespace samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<NamespaceMeasurements>> getSamples(Long containerId, Long namespaceId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given Namespace id's (namespaceIds). The map returned will contain only Namespace ids that has a sample.
     * Namespace ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the Namespace ids passed.
     *
     * @param containerId
     * @param namespaceIds set of Namespace ids
     * @return map of Namespace id to latest sample SampleInfo<NamespaceMeasurements>. Namespace ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<NamespaceMeasurements>> getLatestSample(Long containerId, Set<Long> namespaceIds) throws NetappInterfaceException;

    /**
     * Gets Namespace summaries for Namespace ids
     *
     * @param namespaceIds set of Namespace ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of Namespace id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<NamespaceMeasurements>>>  getSummaries(Set<Long> namespaceIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets Namespace summaries for Namespace ids
     *
     * @param namespaceIds set of Namespace ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of Namespace id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<NamespaceMeasurements>>>  getSummaries(Long containerId, Set<Long> namespaceIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
