/*
 * $Id$
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.oci.netapp.performance.namespace.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.namespace.interfaces.data.NamespaceMeasurements;

import com.netapp.oci.netapp.performance.namespace.interfaces.data.NamespaceRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

import javax.ejb.Remote;
import java.util.Collection;

@Remote
public interface NamespacePerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//NamespacePerformanceBean!com.netapp.oci.netapp.performance.namespace.interfaces.remote.NamespacePerformanceRemote";

    /**
     * The Acq use this method to report on Namespace performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw Namespace samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws NetappInterfaceException if failed to report
     */
    void reportNamespaceSamples(Long datasourceId, String clusterKey, long time, Collection<RawSample<NamespaceMeasurements>> rawSamples, long leaseTime, boolean isHistorical) throws NetappInterfaceException;
    
    /**
     * The Acq use this method to report on Namespace raw performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw Namespace samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     *
     * @throws NetappInterfaceException if failed to report
     */
    void reportNamespaceRawSamples(Long datasourceId, String clusterKey, long time, Collection<RawSample<NamespaceRawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
    
}
