package com.netapp.oci.netapp.performance.networklif;

import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @author Nancy Walia
 *         Date: Feb 13, 2012
 */
public class NetworkLifMetaData {
	public static final Column[] RAW_COLUMNS = new Column[]{
		new Column("recv_data", "bigint unsigned"),
		new Column("recv_errors", "bigint unsigned"),
		new Column("recv_packet", "bigint unsigned"),
		new Column("sent_data", "bigint unsigned"),
		new Column("sent_errors", "bigint unsigned"),
		new Column("sent_packet", "bigint unsigned"),

	};
	
    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("recvData", "float"),
            new Column("recvErrors", "float"),
            new Column("recvPacket", "float"),
            new Column("sentData", "float"),
            new Column("sentErrors", "float"),
            new Column("sentPacket", "float"),
            new Column("opmLifPackets", "float"),
            new Column("opmLifThroughput", "float"),
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("recvData", "float"),
            new Column("recvErrors", "float"),
            new Column("recvPacket", "float"),
            new Column("sentData", "float"),
            new Column("sentErrors", "float"),
            new Column("sentPacket", "float"),
            new Column("opmLifPackets", "float"),
            new Column("opmLifThroughput", "float"),
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
    };

    private static ObjectParser<NetworkLifRawMeasurements> rawParser;
    private static ObjectParser<NetworkLifMeasurements> sampleParser;

    static {
    	rawParser = new ObjectParser<NetworkLifRawMeasurements>() {
    		public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NetworkLifRawMeasurements objectData) throws TableException {
    			PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvData());
    			PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvErrors());
    			PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvPacket());
    			PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSentData());
    			PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSentErrors());
    			PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSentPacket());

    		}

    		public NetworkLifRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
    			int i = startIndex - 1;
    			return new NetworkLifRawMeasurements(
    					PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
    					PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
    					PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
    					PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
    					PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
    					PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
    		}
    	};

        sampleParser = new ObjectParser<NetworkLifMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NetworkLifMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvErrors());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvPacket());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSentData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSentErrors());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSentPacket());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmLifPackets());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmLifThroughput());
                
            }

            public NetworkLifMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NetworkLifMeasurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<NetworkLifRawMeasurements> getRawParser() {
        return rawParser;
    }
    
    public static ObjectParser<NetworkLifMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<NetworkLifMeasurements> getSampleParser() {
        return sampleParser;
    }

}

