package com.netapp.oci.netapp.performance.networklif;

import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

public class NetworkLifVserverMetaData {
	public static final Column[] RAW_COLUMNS = new Column[]{
		new Column("recv_data", "bigint unsigned"),
		new Column("recv_errors", "bigint unsigned"),
		new Column("recv_packet", "bigint unsigned"),
		new Column("sent_data", "bigint unsigned"),
		new Column("sent_errors", "bigint unsigned"),
		new Column("sent_packet", "bigint unsigned"),
	};
	
    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("recvData", "float"),
            new Column("recvErrors", "float"),
            new Column("recvPacket", "float"),
            new Column("sentData", "float"),
            new Column("sentErrors", "float"),
            new Column("sentPacket", "float"),
            new Column("opmLifPackets", "float"),
            new Column("opmLifThroughput", "float")            
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("recvData", "float"),
            new Column("recvErrors", "float"),
            new Column("recvPacket", "float"),
            new Column("sentData", "float"),
            new Column("sentErrors", "float"),
            new Column("sentPacket", "float"),
            new Column("opmLifPackets", "float"),
            new Column("opmLifThroughput", "float")            
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG}            
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG            
    };

    private static ObjectParser<NetworkLifVserverRawMeasurements> rawParser;
    private static ObjectParser<NetworkLifVserverMeasurements> sampleParser;

    static {
    	rawParser = new ObjectParser<NetworkLifVserverRawMeasurements>() {
    		public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NetworkLifVserverRawMeasurements objectData) throws TableException {
    			PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvData());
    			PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvErrors());
    			PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvPacket());
    			PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSentData());
    			PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSentErrors());
    			PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSentPacket());
    		}

    		public NetworkLifVserverRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
    			int i = startIndex - 1;
    			return new NetworkLifVserverRawMeasurements(
    					PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
    					PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
    					PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
    					PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
    					PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
    					PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
    		}
    	};

        sampleParser = new ObjectParser<NetworkLifVserverMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NetworkLifVserverMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvErrors());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvPacket());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSentData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSentErrors());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSentPacket());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmLifPackets());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmLifThroughput());
            }

            public NetworkLifVserverMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NetworkLifVserverMeasurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),                        
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<NetworkLifVserverRawMeasurements> getRawParser() {
        return rawParser;
    }
    
    public static ObjectParser<NetworkLifVserverMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<NetworkLifVserverMeasurements> getSampleParser() {
        return sampleParser;
    }

}

