package com.netapp.oci.netapp.performance.networklif.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Nancy Walia
 *         Date: Feb 13, 2012
 */

@Remote
public interface NetworkLifPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//NetworkLifPerformanceQueryBean!com.netapp.oci.netapp.performance.networklif.interfaces.remote.NetworkLifPerformanceQueryRemote";

    /**
     * Gets networklif samples for period of time
     *
     * @param networkLifIds set of networklif ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of networklif id to list ordered by time of networklif samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<NetworkLifMeasurements>>> getSamples(Set<Long> networkLifIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets networklif samples for period of time
     *
     * @param containerId container id
     * @param networkLifIds set of networklif ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of networklif id to list ordered by time of networklif samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<NetworkLifMeasurements>>> getSamples(Long containerId, Set<Long> networkLifIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets networklif samples for period of time
     *
     * @param containerId container id
     * @param networkLifId network lif id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of networklif samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if anything went wrong
     */
     List<SampleInfo<NetworkLifMeasurements>> getSamples(Long containerId, Long networkLifId, Date fromTime, Date toTime) throws NetappInterfaceException ;

    /**
     * Retrieve the latest sample for the given networkLif id's (networkLifIds). The map returned will contain only networkLif ids that has a sample.
     * networkLif ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the networkLif ids passed.
     *
     * @param vServerId vServer id
     * @param networkLifIds set of networkLif ids
     * @return map of networkLif id to latest sample SampleInfo<NetworkLifMeasurements>. networkLif ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<NetworkLifMeasurements>> getLatestSample(Long vServerId, Set<Long> networkLifIds) throws NetappInterfaceException;

    /**
     * Gets networklif summaries for networklif ids
     *
     * @param networkLifIds set of networklif ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of networklif id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<NetworkLifMeasurements>>> getSummaries(Set<Long> networkLifIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets networklif summaries for networklif ids
     *
     * @param vServerId vServer Id
     * @param networkLifIds set of networklif ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of networklif id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<NetworkLifMeasurements>>> getSummaries(Long vServerId, Set<Long> networkLifIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}

