package com.netapp.oci.netapp.performance.networklif.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

import javax.ejb.Remote;
import java.util.Collection;

/**
 * @author Nancy Walia
 *         Date: Feb 13, 2012
 */
@Remote
public interface NetworkLifPerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//NetworkLifPerformanceBean!com.netapp.oci.netapp.performance.networklif.interfaces.remote.NetworkLifPerformanceRemote";

    /**
     * The Acq use this method to report on NetworkLif performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param vserverKey - the vserver Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw network samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportNetworkLifSamples(Long datasourceId, String clusterKey, String vserverKey, long time, Collection<RawSample<NetworkLifMeasurements>> rawSamples, long leaseTime, boolean isHistorical) throws NetappInterfaceException;
    
    /**
     * The Acq use this method to report on NetworkLif performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param vserverKey - the vserver Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw network samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     *
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportNetworkLifRawSamples(Long datasourceId, String clusterKey, String vserverKey, long time, Collection<RawSample<NetworkLifRawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;

}

