package com.netapp.oci.netapp.performance.networklif.interfaces.remote;

import java.util.Collection;

import javax.ejb.Remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

@Remote
public interface NetworkLifVserverPerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//NetworkLifVserverPerformanceBean!com.netapp.oci.netapp.performance.networklif.interfaces.remote.NetworkLifVserverPerformanceRemote";
    
    /**
     * To report raw samples for lif:vserver performance
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw aggregate samples
     * @param leaseTime - the time for datasource to be registered in the in the registry (in milliseconds)
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportLifVserverRawData(Long datasourceId, String clusterKey, long time, Collection<RawSample<NetworkLifVserverRawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
    
    /**
     * To report samples for lif:vserver performance
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw aggregate samples
     * @param leaseTime - the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportLifVserverData(Long datasourceId, String clusterKey, long time, Collection<RawSample<NetworkLifVserverMeasurements>> rawSamples, long leaseTime, boolean isHistorical) throws NetappInterfaceException;

}
