/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.networkport;

import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.netapp.performance.networkport.interfaces.data.NetworkPortMeasurements;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @author Nancy Walia
 */
public class NetworkPortMetaData {
    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("recvData", "float"),
            new Column("recvPackets", "float"),
            new Column("sendData", "float"),
            new Column("sendPackets", "float"),
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("recvData", "float"),
            new Column("recvPackets", "float"),
            new Column("sendData", "float"),
            new Column("sendPackets", "float"),
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
    };

    private static ObjectParser<NetworkPortMeasurements> sampleParser;

    static {
        sampleParser = new ObjectParser<NetworkPortMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NetworkPortMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRecvPackets());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSendData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSendPackets());
            }

            public NetworkPortMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NetworkPortMeasurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<NetworkPortMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<NetworkPortMeasurements> getSampleParser() {
        return sampleParser;
    }
}
