package com.netapp.oci.netapp.performance.networkport.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.networkport.interfaces.data.NetworkPortMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Nancy Walia
 */
@Remote
public interface NetworkPortPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//NetworkPortPerformanceQueryBean!com.netapp.oci.netapp.performance.networkport.interfaces.remote.NetworkPortPerformanceQueryRemote";

    /**
     * Gets networkPort samples for period of time
     *
     * @param networkPortIds   set of networkPort ids
     * @param fromTime begining of the time period
     * @param toTime   end of the time period
     * @return map of networkPort id to list ordered by time of networkPort samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<NetworkPortMeasurements>>> getSamples(Set<Long> networkPortIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets networkPort samples for period of time
     *
     * @param containerId container id
     * @param networkPortIds   set of networkPort ids
     * @param fromTime begining of the time period
     * @param toTime   end of the time period
     * @return map of networkPort id to list ordered by time of networkPort samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<NetworkPortMeasurements>>> getSamples(Long containerId, Set<Long> networkPortIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets networkPort samples for period of time
     *
     * @param containerId container id
     * @param networkPortId networkPort id
     * @param fromTime begining of the time period
     * @param toTime   end of the time period
     * @return list ordered by time of networkPort samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    List<SampleInfo<NetworkPortMeasurements>> getSamples(Long containerId, Long networkPortId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given networkPort id's (networkPortIds). The map returned will contain only networkPort ids that has a sample.
     * networkPort ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the networkPort ids passed.
     *
     * @param clusterId cluster id
     * @param networkPortIds set of networkPort ids
     * @return map of networkPort id to latest sample SampleInfo<NetworkPortMeasurements>. networkPort ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<NetworkPortMeasurements>> getLatestSample(Long clusterId, Set<Long> networkPortIds) throws NetappInterfaceException;

    /**
     * Gets networkPort summaries for networkPort ids
     *
     * @param networkPortIds   set of networkPort ids
     * @param fromTime begining of the time period
     * @param toTime   end of the time period
     * @return map of networkPort id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<NetworkPortMeasurements>>> getSummaries(Set<Long> networkPortIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets networkPort summaries for networkPort ids
     *
     * @param clusterId cluster id
     * @param networkPortIds   set of networkPort ids
     * @param fromTime begining of the time period
     * @param toTime   end of the time period
     * @return map of networkPort id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<NetworkPortMeasurements>>> getSummaries(Long clusterId, Set<Long> networkPortIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}


