package com.netapp.oci.netapp.performance.nfs;

import com.netapp.oci.netapp.performance.nfs.interfaces.data.NfsMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

public class NfsMetaData {

    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("opmNfsOps", "float"),
            new Column("opmNfsReadOps", "float"),
            new Column("opmNfsWriteOps", "float"),
            new Column("opmNfsThroughput", "float"),
            new Column("opmNfsReadThroughput", "float"),
            new Column("opmNfsWriteThroughput", "float"),
            new Column("opmNfsLatency", "float"),
            new Column("opmNfsReadLatency", "float"),
            new Column("opmNfsWriteLatency", "float")
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("opmNfsOps", "float"),
            new Column("opmNfsReadOps", "float"),
            new Column("opmNfsWriteOps", "float"),
            new Column("opmNfsThroughput", "float"),
            new Column("opmNfsReadThroughput", "float"),
            new Column("opmNfsWriteThroughput", "float"),
            new Column("opmNfsLatency", "float"),
            new Column("opmNfsReadLatency", "float"),
            new Column("opmNfsWriteLatency", "float")
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("opmNfsOps")},
            {new RatioMethod("opmNfsReadOps")},
            {new RatioMethod("opmNfsWriteOps")}
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("opmNfsOps"),
            new RatioMethod("opmNfsReadOps"),
            new RatioMethod("opmNfsWriteOps")
    };

    private static ObjectParser<NfsMeasurements> sampleParser;

    static {
        sampleParser = new ObjectParser<NfsMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NfsMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNfsOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNfsReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNfsWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNfsThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNfsReadThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNfsWriteThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNfsLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNfsReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNfsWriteLatency());
            }

            public NfsMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NfsMeasurements().setOpmNfsOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmNfsReadOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmNfsWriteOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmNfsThroughput(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmNfsReadThroughput(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmNfsWriteThroughput(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmNfsLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmNfsReadLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmNfsWriteLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<NfsMeasurements> getSampleParser() {
        return sampleParser;
    }

    public static ObjectParser<NfsMeasurements> getSummaryParser() {
        return sampleParser;
    }
}
