package com.netapp.oci.netapp.performance.nfs;

import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv3Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv3RawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

public class Nfsv3MetaData {
	public static final Column[] RAW_COLUMNS = new Column[]{
		new Column("nfsv3_ops", "bigint unsigned"),
		new Column("nfsv3_read_ops", "bigint unsigned"),
		new Column("nfsv3_write_ops", "bigint unsigned"),
		new Column("read_total", "bigint unsigned"),
		new Column("nfsv3_read_throughput", "bigint unsigned"),
		new Column("nfsv3_throughput", "bigint unsigned"),
		new Column("write_total", "bigint unsigned"),
		new Column("nfsv3_write_throughput", "bigint unsigned"),
		new Column("read_avg_latency", "bigint unsigned"),
		new Column("write_avg_latency", "bigint unsigned"),
        new Column("latency", "bigint unsigned"),
        new Column("total_ops", "bigint unsigned")
	};
	
	public static final Column[] SAMPLE_COLUMNS = new Column[]{
		new Column("nfsv3Ops", "float"),
		new Column("nfsv3ReadOps", "float"),
		new Column("nfsv3WriteOps", "float"),
		new Column("readTotal", "float"),
		new Column("nfsv3ReadThroughput", "float"),
		new Column("nfsv3Throughput", "float"),
		new Column("writeTotal", "float"),
		new Column("nfsv3WriteThroughput", "float"),
		new Column("readAvgLatency", "float"),
		new Column("writeAvgLatency", "float"),
        new Column("latency", "float"),
        new Column("totalOps", "float")
	};
	
	public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("nfsv3Ops", "float"),
            new Column("nfsv3ReadOps", "float"),
            new Column("nfsv3WriteOps", "float"),
            new Column("readTotal", "float"),
            new Column("nfsv3ReadThroughput", "float"),
            new Column("nfsv3Throughput", "float"),
            new Column("writeTotal", "float"),
            new Column("nfsv3WriteThroughput", "float"),
            new Column("readAvgLatency", "float"),
            new Column("writeAvgLatency", "float"),
            new Column("latency", "float"),
            new Column("totalOps", "float")
    };
	
	 public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
         {SqlMethod.AVG},
         {SqlMethod.AVG},
         {SqlMethod.AVG},
         {SqlMethod.SUM},
         {SqlMethod.AVG},
         {SqlMethod.AVG},
         {SqlMethod.SUM},
         {SqlMethod.AVG},
         {new RatioMethod("readTotal")},
         {new RatioMethod("writeTotal")},
         {new RatioMethod("totalOps")},
         {SqlMethod.AVG}
	 };

	 public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
		 SqlMethod.AVG,
		 SqlMethod.AVG,
		 SqlMethod.AVG,
		 SqlMethod.SUM,
		 SqlMethod.AVG,
		 SqlMethod.AVG,
		 SqlMethod.SUM,
		 SqlMethod.AVG,
		 new RatioMethod("readTotal"),
		 new RatioMethod("writeTotal"),
         new RatioMethod("totalOps"),
         SqlMethod.AVG
	 };

    private static ObjectParser<Nfsv3RawMeasurements> rawParser;
    private static ObjectParser<Nfsv3Measurements> sampleParser;

    static {
    	rawParser = new ObjectParser<Nfsv3RawMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, Nfsv3RawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv3Ops());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv3ReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv3WriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadTotal());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv3ReadThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv3Throughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteTotal());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv3WriteThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
            }

            public Nfsv3RawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                Nfsv3RawMeasurements m = new Nfsv3RawMeasurements(
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
                m.setLatency(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
                m.setTotalOps(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
                return m;
            }
        };
        
        sampleParser = new ObjectParser<Nfsv3Measurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, Nfsv3Measurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv3Ops());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv3ReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv3WriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadTotal());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv3ReadThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv3Throughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteTotal());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv3WriteThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
            }

            public Nfsv3Measurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                Nfsv3Measurements m =  new Nfsv3Measurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                m.setLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                m.setTotalOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                return m;
            }
        };
    }
    
    public static ObjectParser<Nfsv3RawMeasurements> getRawParser() {
        return rawParser;
    }
    
    public static ObjectParser<Nfsv3Measurements> getSampleParser() {
        return sampleParser;
    }
	
    public static ObjectParser<Nfsv3Measurements> getSummaryParser() {
        return sampleParser;
    }
}
