package com.netapp.oci.netapp.performance.nfs;

import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv41Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv41RawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

public class Nfsv41MetaData {
	public static final Column[] RAW_COLUMNS = new Column[]{
		new Column("read_avg_latency", "bigint unsigned"),
		new Column("read_total", "bigint unsigned"),
		new Column("nfs41_read_throughput", "bigint unsigned"),
		new Column("nfs41_throughput", "bigint unsigned"),
		new Column("write_avg_latency", "bigint unsigned"),
		new Column("write_total", "bigint unsigned"),
		new Column("nfs41_write_throughput", "bigint unsigned"),
		new Column("nfsv41_ops", "bigint unsigned"),
        new Column("latency", "bigint unsigned"),
        new Column("total_ops", "bigint unsigned"),
	};

	public static final Column[] SAMPLE_COLUMNS = new Column[]{
		new Column("nfsv41Ops", "float"),
		new Column("readTotal", "float"),
		new Column("nfsv41ReadThroughput", "float"),
		new Column("nfsv41Throughput", "float"),
		new Column("writeTotal", "float"),
		new Column("nfsv41WriteThroughput", "float"),
		new Column("readAvgLatency", "float"),
		new Column("writeAvgLatency", "float"),
        new Column("latency", "float"),
        new Column("totalOps", "float")
	};
	
	public static final Column[] SUMMARY_COLUMNS = new Column[]{
		new Column("nfsv41Ops", "float"),
		new Column("readTotal", "float"),
		new Column("nfsv41ReadThroughput", "float"),
		new Column("nfsv41Throughput", "float"),
		new Column("writeTotal", "float"),
		new Column("nfsv41WriteThroughput", "float"),
		new Column("readAvgLatency", "float"),
		new Column("writeAvgLatency", "float"),
        new Column("latency", "float"),
        new Column("totalOps", "float")
	};
	
	 public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
         {SqlMethod.AVG},
         {SqlMethod.SUM},
         {SqlMethod.AVG},
         {SqlMethod.AVG},
         {SqlMethod.SUM},
         {SqlMethod.AVG},
         {new RatioMethod("readTotal")},
         {new RatioMethod("writeTotal")},
         {new RatioMethod("totalOps")},
         {SqlMethod.AVG}
     };

	 public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
		 SqlMethod.AVG,
		 SqlMethod.SUM,
		 SqlMethod.AVG,
		 SqlMethod.AVG,
		 SqlMethod.SUM,
		 SqlMethod.AVG,
		 new RatioMethod("readTotal"),
		 new RatioMethod("writeTotal"),
         new RatioMethod("totalOps"),
         SqlMethod.AVG
     };

    private static ObjectParser<Nfsv41RawMeasurements> rawParser;
    private static ObjectParser<Nfsv41Measurements> sampleParser;

    static {
    	rawParser = new ObjectParser<Nfsv41RawMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, Nfsv41RawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadTotal());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfs41ReadThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfs41Throughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteTotal());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfs41WriteThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv41Ops());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
            }

            public Nfsv41RawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                Nfsv41RawMeasurements nfsv41RawMeasurements = new Nfsv41RawMeasurements(
                        PerformanceTableUtils.getStringNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
                nfsv41RawMeasurements.setLatency(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
                nfsv41RawMeasurements.setTotalOps(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
                return nfsv41RawMeasurements;
            }
        };
        
        sampleParser = new ObjectParser<Nfsv41Measurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, Nfsv41Measurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv41Ops());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadTotal());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv41ReadThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv41Throughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteTotal());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsv41WriteThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
            }

            public Nfsv41Measurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                Nfsv41Measurements nfsv41Measurements = new Nfsv41Measurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                nfsv41Measurements.setLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                nfsv41Measurements.setTotalOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                return nfsv41Measurements;
            }
        };
    }
    
    public static ObjectParser<Nfsv41RawMeasurements> getRawParser() {
        return rawParser;
    }
	
    public static ObjectParser<Nfsv41Measurements> getSampleParser() {
        return sampleParser;
    }
    public static ObjectParser<Nfsv41Measurements> getSummaryParser() {
        return sampleParser;
    }
}
