package com.netapp.oci.netapp.performance.nfs.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.NfsMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Remote
public interface NfsPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//NfsPerformanceQueryBean!com.netapp.oci.netapp.performance.nfs.interfaces.remote.NfsPerformanceQueryRemote";

    /**
     * Gets nfs samples for period of time
     *
     * @param vServerIds set of vServer ids
     * @param fromTime   beginning of the time period
     * @param toTime     end of the time period
     * @return map of vServer id to list ordered by time of nfs samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<NfsMeasurements>>> getSamples(Set<Long> vServerIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets nfs samples for period of time
     *
     * @param clusterId  Cluster id
     * @param vServerIds set of vServer ids
     * @param fromTime   beginning of the time period
     * @param toTime     end of the time period
     * @return map of vServer id to list ordered by time of nfs samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<NfsMeasurements>>> getSamples(Long clusterId, Set<Long> vServerIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets nfs samples for period of time
     *
     * @param clusterId Cluster id
     * @param vServerId vServer id
     * @param fromTime  beginning of the time period
     * @param toTime    end of the time period
     * @return list ordered by time of nfs samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    List<SampleInfo<NfsMeasurements>> getSamples(Long clusterId, Long vServerId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given vServer id's (vServerIds). The map returned will contain only vServer
     * ids that has a sample. vServer ids with no samples will not be returned in the map. An empty map will be returned
     * if no samples are found for all the vServer ids passed.
     *
     * @param clusterId  cluster id
     * @param vServerIds set of vServer ids
     * @return map of vServer id to latest sample SampleInfo<NfsMeasurements>. vServer ids with no samples will not be included
     *         in this map.
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, SampleInfo<NfsMeasurements>> getLatestSample(Long clusterId, Set<Long> vServerIds) throws NetappInterfaceException;


    /**
     * Gets nfs summaries for vServer ids
     *
     * @param vServerIds set of vServer ids
     * @param fromTime   beginning of the time period
     * @param toTime     end of the time period
     * @return map of vServer id to summary data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<Summary<NfsMeasurements>>> getSummaries(Set<Long> vServerIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets nfs summaries for vServer ids
     *
     * @param clusterId  Cluster Id
     * @param vServerIds set of vServer ids
     * @param fromTime   beginning of the time period
     * @param toTime     end of the time period
     * @return map of vServer id to summary data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<Summary<NfsMeasurements>>> getSummaries(Long clusterId, Set<Long> vServerIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param clusterId  Cluster Id
     * @param vServerIds set of vServer ids
     * @param fromTime   beginning of the time period
     * @param toTime     end of the time period
     * @return true if samples exist
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long clusterId, Long vServerIds, Date fromTime, Date toTime) throws NetappInterfaceException;

}
