package com.netapp.oci.netapp.performance.nfs.interfaces.remote;

import java.util.Collection;

import javax.ejb.Remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv3Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv3RawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

@Remote
public interface Nfsv3PerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//Nfsv3PerformanceBean!com.netapp.oci.netapp.performance.nfs.interfaces.remote.Nfsv3PerformanceRemote";
    
    /**
     * To report raw samples for nfsv3 performance
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw aggregate samples
     * @param leaseTime - the time for datasource to be registered in the in the registry (in milliseconds)
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportNfsv3RawData(Long datasourceId, String clusterKey, long time, Collection<RawSample<Nfsv3RawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;

    /**
     * To report summary samples for nfsv3 performance
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw aggregate samples
     * @param leaseTime - the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportNfsv3Data(Long datasourceId, String clusterKey, long time, Collection<RawSample<Nfsv3Measurements>> rawSamples, long leaseTime, boolean isHistorical) throws NetappInterfaceException;
}
