package com.netapp.oci.netapp.performance.nfs.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv41Measurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Anton Sergeev
 */
@Remote
public interface Nfsv41PerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//Nfsv41PerformanceQueryBean!com.netapp.oci.netapp.performance.nfs.interfaces.remote.Nfsv41PerformanceQueryRemote";

    /**
     * Gets nfs samples for period of time
     *
     * @param nfsIds set of nfs ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of nfs id to list ordered by time of nfs samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<Nfsv41Measurements>>> getSamples(Set<Long> nfsIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets nfs samples for period of time
     *
     * @param containerId container id
     * @param nfsIds set of nfs ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of nfs id to list ordered by time of nfs samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<Nfsv41Measurements>>> getSamples(Long containerId, Set<Long> nfsIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets nfs samples for period of time
     *
     * @param containerId container id
     * @param nfsId nfs id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of nfs samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<Nfsv41Measurements>> getSamples(Long containerId, Long nfsId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given nfs id's (nfsIds). The map returned will contain only nfs ids that has a sample.
     * Nfsv41 ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the Nfsv41 ids passed.
     *
     * @param clusterId cluster id
     * @param nfsIds set of nfs ids
     * @return map of nfs id to latest sample SampleInfo<Nfsv41Measurements>. Nfsv41 ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<Nfsv41Measurements>> getLatestSample(Long clusterId, Set<Long> nfsIds) throws NetappInterfaceException;


    /**
     * Gets nfs summaries for nfs ids
     *
     * @param nfsIds set of nfs ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of nfs id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<Nfsv41Measurements>>>  getSummaries(Set<Long> nfsIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets nfs summaries for nfs ids
     *
     * @param clusterId cluster id
     * @param nfsIds set of nfs ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of nfs id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<Nfsv41Measurements>>>  getSummaries(Long clusterId, Set<Long> nfsIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
