package com.netapp.oci.netapp.performance.nic;

import com.netapp.oci.netapp.performance.nic.interfaces.data.NicMeasurements;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

public class NicMetaData {
    public static final Column[] RAW_COLUMNS = new Column[]{
        new Column("rx_bytes", "bigint unsigned"),
        new Column("rx_errors", "bigint unsigned"),
        new Column("tx_bytes", "bigint unsigned"),
        new Column("tx_errors", "bigint unsigned"),
    };
    

    public static final Column[] SAMPLE_COLUMNS = new Column[]{
        new Column("rxBytes", "float"),
        new Column("rxErrors", "float"),
        new Column("txBytes", "float"),
        new Column("txErrors", "float"),
        new Column("opmNicTotalBytePerSec", "float"),
        new Column("opmNicTotalErrorRate", "float"),
        new Column("opmNicUtilization", "float")
    };
    
    public static final Column[] SUMMARY_COLUMNS = new Column[]{
        new Column("rxBytes", "float"),
        new Column("rxErrors", "float"),
        new Column("txBytes", "float"),
        new Column("txErrors", "float"),
        new Column("opmNicTotalBytePerSec", "float"),
        new Column("opmNicTotalErrorRate", "float"),
        new Column("opmNicUtilization", "float")
    };
    
   
    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG}
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG
    };

    private static ObjectParser<NicRawMeasurements> rawParser;
    private static ObjectParser<NicMeasurements> sampleParser;

    static {
        rawParser = new ObjectParser<NicRawMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NicRawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRxBytes());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRxErrors());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTxBytes());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTxErrors());
            }

            public NicRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NicRawMeasurements()
                .setRxBytes(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                .setRxErrors(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                .setTxBytes(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                .setTxErrors(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
            }
        };
        
        sampleParser = new ObjectParser<NicMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NicMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRxBytes());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRxErrors());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTxBytes());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTxErrors());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNicTotalBytePerSec());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNicTotalErrorRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNicUtilization());
            }

            public NicMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NicMeasurements()
                .setRxBytes(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setRxErrors(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setTxBytes(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setTxErrors(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setOpmNicTotalBytePerSec(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setOpmNicTotalErrorRate(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setOpmNicUtilization(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                
            }
        };
    }
    
    public static ObjectParser<NicRawMeasurements> getRawParser() {
        return rawParser;
    }
    
    public static ObjectParser<NicMeasurements> getSampleParser() {
        return sampleParser;
    }
    
    public static ObjectParser<NicMeasurements> getSummaryParser() {
        return sampleParser;
    }
}
