package com.netapp.oci.netapp.performance.nic.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Anton Sergeev
 */
@Remote
public interface NicPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//NicPerformanceQueryBean!com.netapp.oci.netapp.performance.nic.interfaces.remote.NicPerformanceQueryRemote";

    /**
     * Gets nic samples for period of time
     *
     * @param nicIds set of nic ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of nic id to list ordered by time of nic samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<NicMeasurements>>> getSamples(Set<Long> nicIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets nic samples for period of time
     *
     * @param containerId container id
     * @param nicIds set of nic ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of nic id to list ordered by time of nic samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<NicMeasurements>>> getSamples(Long containerId, Set<Long> nicIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets nic samples for period of time
     *
     * @param containerId container id
     * @param nicId nic id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of nic samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<NicMeasurements>> getSamples(Long containerId, Long nicId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given nic id's (nicIds).  The map returned will contain only nic ids that has a sample.
     * nic ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the nic ids passed.
     *
     * @param clusterId cluster id
     * @param nicIds set of nic ids
     * @return map of nic id to latest sample SampleInfo<NicMeasurements> nic ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<NicMeasurements>> getLatestSample(Long clusterId, Set<Long> nicIds) throws NetappInterfaceException;

    /**
     * Gets nic summaries for nic ids
     *
     * @param nicIds set of nic ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of nic id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<NicMeasurements>>> getSummaries(Set<Long> nicIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets nic summaries for nic ids
     *
     * @param nicIds set of nic ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of nic id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<NicMeasurements>>> getSummaries(Long clusterId, Set<Long> nicIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;
}
