/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.node;

import com.netapp.oci.netapp.performance.node.interfaces.data.NodeMeasurements;
import com.netapp.oci.netapp.performance.node.interfaces.data.NodeRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.PercentCleanTimeUtil;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @author Anton Sergeev
 */
public class NodeMetaData {
    public static final Column[] RAW_COLUMNS = new Column[]{
    	new Column("avg_processor_busy", "bigint unsigned"),
    	new Column("cpu_busy", "bigint unsigned"),
    	new Column("cpu_elapsed_time", "bigint unsigned"),
    	new Column("net_data_recv", "bigint unsigned"),
    	new Column("net_data_sent", "bigint unsigned"),
    	new Column("cifs_ops", "bigint unsigned"),
    	new Column("fcp_ops", "bigint unsigned"),
    	new Column("iscsi_ops", "bigint unsigned"),
    	new Column("nfs_ops", "bigint unsigned"),
    	new Column("other_ops", "bigint unsigned"),
    	new Column("read_ops", "bigint unsigned"),
        new Column("avg_latency", "bigint unsigned"),
        new Column("read_latency", "bigint unsigned"),
        new Column("write_latency", "bigint unsigned"),
    	new Column("total_ops", "bigint unsigned"),
    	new Column("write_ops", "bigint unsigned"),
    	new Column("fcp_data_recv", "bigint unsigned"),
    	new Column("fcp_data_sent", "bigint unsigned"),
    	new Column("nvmf_ops", "bigint unsigned"),
    	new Column("nvmf_data_recv", "bigint unsigned"),
    	new Column("nvmf_data_sent", "bigint unsigned"),
    };
    
    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("cpuBusy", "float"),
            new Column("avgProcessorBusy", "float"),
        	new Column("cpuElapsedTime", "bigint unsigned"),
        	new Column("netDataRecv", "float"),
        	new Column("netDataSent", "float"),
            new Column("readOps", "float"),
            new Column("writeOps", "float"),
            new Column("totalOps", "float"),
            new Column("otherOps", "float"),
            new Column("cifsOps", "float"),
            new Column("fcpOps", "float"),
            new Column("iscsiOps", "float"),
            new Column("nfsOps", "float"),
            new Column("readLatency", "float"),
            new Column("writeLatency", "float"),
            new Column("avgLatency", "float"),
            new Column("fcpDataRecv", "float"),
            new Column("fcpDataSent", "float"),
            new Column("opmSysReadThroughput", "float"),
            new Column("opmSysWriteThroughput", "float"),
            new Column("opmSysThroughput", "float"),
            new Column("opmSysUtilization", "float"),
            new Column("totalCpMSecs", "bigint unsigned"),
            new Column("p2Flush", "bigint unsigned"),
            new Column("totalKahunaBusy", "bigint unsigned"),
            new Column("maxProcessorElapsedTime", "bigint unsigned"),
            new Column("deltaPollTimeInMSecs", "bigint unsigned"),
            new Column("maxAggregateUtilization", "float"),
            new Column("percentCleanTimeUtil", "float"),
            new Column("diskShelfFibreChannelLoopUtilization", "float"),
            new Column("nvmfOps", "float"),
            new Column("nvmfDataRecv", "float"),
            new Column("nvmfDataSent", "float"),
            new Column("localUserOps", "float"),
            new Column("localUserReadOps", "float"),
            new Column("localUserWriteOps", "float"),
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("cpuBusy", "float"),
            new Column("avgProcessorBusy", "float"),
        	new Column("cpuElapsedTime", "bigint unsigned"),
        	new Column("netDataRecv", "float"),
        	new Column("netDataSent", "float"),        	
            new Column("readOps", "float"),
            new Column("writeOps", "float"),
            new Column("totalOps", "float"),
            new Column("otherOps", "float"),
            new Column("cifsOps", "float"),
            new Column("fcpOps", "float"),
            new Column("iscsiOps", "float"),
            new Column("nfsOps", "float"),
            new Column("readLatency", "float"),
            new Column("writeLatency", "float"),
            new Column("avgLatency", "float"),
            new Column("fcpDataRecv", "float"),
            new Column("fcpDataSent", "float"),
            new Column("opmSysReadThroughput", "float"),
            new Column("opmSysWriteThroughput", "float"),
            new Column("opmSysThroughput", "float"),
            new Column("opmSysUtilization", "float"),
            new Column("totalCpMSecs", "bigint unsigned"),
            new Column("p2Flush", "bigint unsigned"),
            new Column("totalKahunaBusy", "bigint unsigned"),
            new Column("maxProcessorElapsedTime", "bigint unsigned"),
            new Column("deltaPollTimeInMSecs", "bigint unsigned"),
            new Column("maxAggregateUtilization", "float"),
            new Column("percentCleanTimeUtil", "float"),
            new Column("diskShelfFibreChannelLoopUtilization", "float"),
            new Column("nvmfOps", "float"),
            new Column("nvmfDataRecv", "float"),
            new Column("nvmfDataSent", "float"),
            new Column("localUserOps", "float"),
            new Column("localUserReadOps", "float"),
            new Column("localUserWriteOps", "float"),
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {new RatioMethod("cpuElapsedTime")},
            {new RatioMethod("cpuElapsedTime")},
            {SqlMethod.SUM},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("readOps")},
            {new RatioMethod("writeOps")},
            {new RatioMethod("totalOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.SUM},
            {SqlMethod.SUM},
            {SqlMethod.SUM},
            {SqlMethod.SUM},
            {SqlMethod.SUM},
            {SqlMethod.AVG},
            {new PercentCleanTimeUtil("totalCpMSecs", "p2Flush", "maxAggregateUtilization", "deltaPollTimeInMSecs")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            new RatioMethod("cpuElapsedTime"),
            new RatioMethod("cpuElapsedTime"),
            SqlMethod.SUM,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("readOps"),
            new RatioMethod("writeOps"),
            new RatioMethod("totalOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.SUM,
            SqlMethod.SUM,
            SqlMethod.SUM,
            SqlMethod.SUM,
            SqlMethod.SUM,
            SqlMethod.AVG,
            new PercentCleanTimeUtil("totalCpMSecs", "p2Flush", "maxAggregateUtilization", "deltaPollTimeInMSecs"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
    };

    private static ObjectParser<NodeRawMeasurements> rawParser;
    private static ObjectParser<NodeMeasurements> sampleParser;

    static {
    	rawParser = new ObjectParser<NodeRawMeasurements>() {
    		
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NodeRawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgProcessorBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuElapsedTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNetDataRecv());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNetDataSent());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSysAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSysReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSysWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpDataRecv());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpDataSent());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNvmfOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNvmfDataRecv());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNvmfDataSent());
            }

            public NodeRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NodeRawMeasurements()
                    .setAvgProcessorBusy(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setCpuBusy(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setCpuElapsedTime(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setNetDataRecv(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setNetDataSent(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setCifsOps(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setFcpOps(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setIscsiOps(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setNfsOps(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setOtherOps(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setReadOps(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setSysAvgLatency(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setSysReadLatency(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setSysWriteLatency(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setTotalOps(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setWriteOps(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                	.setFcpDataRecv(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                	.setFcpDataSent(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                    .setNvmfOps(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                	.setNvmfDataRecv(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                	.setNvmfDataSent(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
            }
		};
		
        sampleParser = new ObjectParser<NodeMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NodeMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgProcessorBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuElapsedTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNetDataRecv());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNetDataSent());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSysReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSysWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSysAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpDataRecv());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpDataSent());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmSysReadThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmSysWriteThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmSysThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmSysUtilization());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalCpMSecs());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getP2Flush());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalKahunaBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMaxProcessorElapsedTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDeltaPollTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMaxAggregateUtilization());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getPercentCleanTimeUtil());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDiskShelfFibreChannelLoopUtilization());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNvmfOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNvmfDataRecv());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNvmfDataSent());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLocalUserOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLocalUserReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLocalUserWriteOps());
            }

            public NodeMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NodeMeasurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getLongNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getLongNullable(resultSet, ++i),
                        PerformanceTableUtils.getLongNullable(resultSet, ++i),
                        PerformanceTableUtils.getLongNullable(resultSet, ++i),
                        PerformanceTableUtils.getLongNullable(resultSet, ++i),
                        PerformanceTableUtils.getLongNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<NodeMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<NodeMeasurements> getSampleParser() {
        return sampleParser;
    }

    public static ObjectParser<NodeRawMeasurements> getRawParser() {
        return rawParser;
    }
}
