/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.node.interfaces.data;

import java.io.Serializable;

/**
 * @author Anton Sergeev
 */
public class NodeMeasurements implements Serializable {
	private static final long serialVersionUID = 2L;

    /**
     * System CPU resource utilization
     */
    private float cpuBusy;

    /**
     * Average processor utilization across all processors in the system
     */
    private float avgProcessorBusy;

    /**
     * CPU elapsed time in the system
     */
    private long cpuElapsedTime;
    
    /**
     * Net data received in the system
     */
    private float netDataRecv;

    /**
     * Net data sent in the system
     */
    private float netDataSent;
    
    /**
     * Number of read operations initiated each second for storing data or metadata associated with the node
     */    
    private float readOps;
    
    /**
     * Number of write operations initiated each second for storing data or metadata associated with the node
     */    
    private float writeOps;

    /**
     * Number of total operations initiated each second for storing data or metadata associated with the node
     */    
    private float totalOps;
    
    /**
     * Number of other operations initiated each second for storing data or metadata associated with the node
     */    
    private float otherOps;
    
    /**
     * Number of cifs operations initiated each second for storing data or metadata associated with the node
     */    
    private float cifsOps;
    
    /**
     * Number of fcp operations initiated each second for storing data or metadata associated with the node
     */
    private float fcpOps;

    /**
     * Number of iscsi operations initiated each second for storing data or metadata associated with the node
     */    
    private float iscsiOps;
    
    /**
     * Number of nvmf operations initiated each second for storing data or metadata associated with the node
     */    
    private float nvmfOps;
    
    /**
     * Number of nfs operations initiated each second for storing data or metadata associated with the node
     */    
    private float nfsOps;
    
    /**
     * latency in microseconds for system read
     */    
    private float sysReadLatency;
    
    /**
     * latency in microseconds for system write
     */        
    private float sysWriteLatency;

    /**
     * Average latency in microseconds for system operation
     */        
    private float sysAvgLatency;
    
    /**
     * Fcp Data that is sent and recv
     */
    private float fcpDataSent;
    
    private float fcpDataRecv;
    
    /**
     * Nvmf Data that is sent and recv
     */
    private float nvmfDataSent;
    
    private float nvmfDataRecv;
    
    /**
     * The synthetic counters for OPM
     */
    
    private float opmSysThroughput;
    
    private float opmSysReadThroughput;
    
    private float opmSysWriteThroughput;
    
    private float opmSysUtilization;

    private long totalCpMSecs;
    
    private long p2Flush;
    
    private long maxProcessorElapsedTime;
    
    private long totalKahunaBusy;
    
    private long deltaPollTime;
    
    private float maxAggregateUtilization;
    
    private float percentCleanTimeUtil;
    
    private float diskShelfFibreChannelLoopUtilization;

    private float localUserOps;

    private float localUserReadOps;

    private float localUserWriteOps;
    

	public NodeMeasurements() {
    }
    
    public NodeMeasurements(float cpuBusy, float avgProcessorBusy,
			long cpuElapsedTime, float netDataRecv, float netDataSent,
			float readOps, float writeOps, float totalOps, float otherOps,
			float cifsOps, float fcpOps, 
			float iscsiOps, float nfsOps, float sysReadLatency,
			float sysWriteLatency, float sysAvgLatency, float fcpDataRecv, float fcpDataSent, float opmSysReadThroughput,
			float opmSysWriteThroughput, float opmSysThroughput, float opmSysUtilization,
			long totalCpMSecs, long p2Flush, long totalKahunaBusy, long maxProcessorElapsedTime, long deltaPollTime, float maxAggregateUtilization,
			float percentCleanTimeUtil, float diskShelfFibreChannelLoopUtilization,
            float nvmfOps, float nvmfDataRecv, float nvmfDataSent, float localOps, float localReadOps, float localWriteOps) {
		super();
		this.cpuBusy = cpuBusy;
		this.avgProcessorBusy = avgProcessorBusy;
		this.cpuElapsedTime = cpuElapsedTime;
		this.netDataRecv = netDataRecv;
		this.netDataSent = netDataSent;
		this.readOps = readOps;
		this.writeOps = writeOps;
		this.totalOps = totalOps;
		this.otherOps = otherOps;
		this.cifsOps = cifsOps;
		this.fcpOps = fcpOps;
		this.iscsiOps = iscsiOps;
		this.nfsOps = nfsOps;
		this.sysReadLatency = sysReadLatency;
		this.sysWriteLatency = sysWriteLatency;
		this.sysAvgLatency = sysAvgLatency;
		this.fcpDataRecv = fcpDataRecv;
		this.fcpDataSent = fcpDataSent;
		this.opmSysThroughput = opmSysThroughput;
		this.opmSysReadThroughput = opmSysReadThroughput;
		this.opmSysWriteThroughput = opmSysWriteThroughput;
		this.opmSysUtilization = opmSysUtilization;
		this.totalCpMSecs = totalCpMSecs;
		this.p2Flush = p2Flush;
		this.totalKahunaBusy = totalKahunaBusy;
		this.maxProcessorElapsedTime = maxProcessorElapsedTime;
		this.deltaPollTime = deltaPollTime;
		this.maxAggregateUtilization = maxAggregateUtilization;
		this.percentCleanTimeUtil = percentCleanTimeUtil;
		this.diskShelfFibreChannelLoopUtilization = diskShelfFibreChannelLoopUtilization;
		this.nvmfOps = nvmfOps;
		this.nvmfDataRecv = nvmfDataRecv;
		this.nvmfDataSent = nvmfDataSent;
		this.localUserOps = localOps;
		this.localUserReadOps = localReadOps;
		this.localUserWriteOps = localWriteOps;
	}

	public float getCpuBusy() {
        return cpuBusy;
    }

    public void setCpuBusy(float cpuBusy) {
        this.cpuBusy = cpuBusy;
    }

    public float getAvgProcessorBusy() {
        return avgProcessorBusy;
    }

    public void setAvgProcessorBusy(final float avgProcessorBusy) {
        this.avgProcessorBusy = avgProcessorBusy;
    }

    public float getReadOps() {
		return readOps;
	}

	public void setReadOps(float readOps) {
		this.readOps = readOps;
	}

	public float getWriteOps() {
		return writeOps;
	}

	public void setWriteOps(float writeOps) {
		this.writeOps = writeOps;
	}

	public float getTotalOps() {
		return totalOps;
	}

	public void setTotalOps(float totalOps) {
		this.totalOps = totalOps;
	}

	public float getOtherOps() {
		return otherOps;
	}
	
	public void setOtherOps(float otherOps) {
		this.otherOps = otherOps;
	}

	public float getCifsOps() {
		return cifsOps;
	}

	public void setCifsOps(float cifsOps) {
		this.cifsOps = cifsOps;
	}

	public float getFcpOps() {
		return fcpOps;
	}

	public void setFcpOps(float fcpOps) {
		this.fcpOps = fcpOps;
	}

	public float getIscsiOps() {
		return iscsiOps;
	}

	public void setIscsiOps(float iscsiOps) {
		this.iscsiOps = iscsiOps;
	}

	public float getNfsOps() {
		return nfsOps;
	}

	public void setNfsOps(float nfsOps) {
		this.nfsOps = nfsOps;
	}

	public float getSysReadLatency() {
		return sysReadLatency;
	}

	public void setSysReadLatency(float sysReadLatency) {
		this.sysReadLatency = sysReadLatency;
	}

	public float getSysWriteLatency() {
		return sysWriteLatency;
	}

	public void setSysWriteLatency(float sysWriteLatency) {
		this.sysWriteLatency = sysWriteLatency;
	}

	public float getSysAvgLatency() {
		return sysAvgLatency;
	}

	public void setSysAvgLatency(float sysAvgLatency) {
		this.sysAvgLatency = sysAvgLatency;
	}
    	
    public long getCpuElapsedTime() {
		return cpuElapsedTime;
	}

	public void setCpuElapsedTime(long cpuElapsedTime) {
		this.cpuElapsedTime = cpuElapsedTime;
	}

	public float getNetDataRecv() {
		return netDataRecv;
	}

	public void setNetDataRecv(float netDataRecv) {
		this.netDataRecv = netDataRecv;
	}

	public float getNetDataSent() {
		return netDataSent;
	}

	public void setNetDataSent(float netDataSent) {
		this.netDataSent = netDataSent;
	}

	
	public float getFcpDataSent() {
		return fcpDataSent;
	}

	public void setFcpDataSent(float fcpDataSent) {
		this.fcpDataSent = fcpDataSent;
	}

	public float getFcpDataRecv() {
		return fcpDataRecv;
	}

	public void setFcpDataRecv(float fcpDataRecv) {
		this.fcpDataRecv = fcpDataRecv;
	}

	public float getOpmSysThroughput() {
		return opmSysThroughput;
	}

	public void setOpmSysThroughput(float opmSysThroughput) {
		this.opmSysThroughput = opmSysThroughput;
	}

	public float getOpmSysReadThroughput() {
		return opmSysReadThroughput;
	}

	public void setOpmSysReadThroughput(float opmSysReadThroughput) {
		this.opmSysReadThroughput = opmSysReadThroughput;
	}

	public float getOpmSysWriteThroughput() {
		return opmSysWriteThroughput;
	}

	public void setOpmSysWriteThroughput(float opmSysWriteThroughput) {
		this.opmSysWriteThroughput = opmSysWriteThroughput;
	}

	public float getOpmSysUtilization() {
		return opmSysUtilization;
	}

	public void setOpmSysUtilization(float opmSysUtilization) {
		this.opmSysUtilization = opmSysUtilization;
	}

	public long getTotalCpMSecs() {
		return totalCpMSecs;
	}

	public void setTotalCpMSecs(long totalCpMSecs) {
		this.totalCpMSecs = totalCpMSecs;
	}

	public long getP2Flush() {
		return p2Flush;
	}

	public void setP2Flush(long p2Flush) {
		this.p2Flush = p2Flush;
	}

	public long getMaxProcessorElapsedTime() {
		return maxProcessorElapsedTime;
	}

	public void setMaxProcessorElapsedTime(long maxProcessorElapsedTime) {
		this.maxProcessorElapsedTime = maxProcessorElapsedTime;
	}

	public long getTotalKahunaBusy() {
		return totalKahunaBusy;
	}

	public void setTotalKahunaBusy(long totalKahunaBusy) {
		this.totalKahunaBusy = totalKahunaBusy;
	}

	public long getDeltaPollTime() {
		return deltaPollTime;
	}

	public void setDeltaPollTime(long deltaPollTime) {
		this.deltaPollTime = deltaPollTime;
	}

	public float getMaxAggregateUtilization() {
		return maxAggregateUtilization;
	}

	public void setMaxAggregateUtilization(float maxAggregateUtilization) {
		this.maxAggregateUtilization = maxAggregateUtilization;
	}

	public float getPercentCleanTimeUtil() {
		return percentCleanTimeUtil;
	}

	public void setPercentCleanTimeUtil(float percentCleanTimeUtil) {
		this.percentCleanTimeUtil = percentCleanTimeUtil;
	}

	public float getDiskShelfFibreChannelLoopUtilization() {
		return diskShelfFibreChannelLoopUtilization;
	}

	public void setDiskShelfFibreChannelLoopUtilization(
			float diskShelfFibreChannelLoopUtilization) {
		this.diskShelfFibreChannelLoopUtilization = diskShelfFibreChannelLoopUtilization;
	}

	public float getNvmfOps() {
		return nvmfOps;
	}

	public void setNvmfOps(float nvmfOps) {
		this.nvmfOps = nvmfOps;
	}

	public float getNvmfDataSent() {
		return nvmfDataSent;
	}

	public void setNvmfDataSent(float nvmfDataSent) {
		this.nvmfDataSent = nvmfDataSent;
	}

	public float getNvmfDataRecv() {
		return nvmfDataRecv;
	}

	public void setNvmfDataRecv(float nvmfDataRecv) {
		this.nvmfDataRecv = nvmfDataRecv;
	}

	public float getLocalUserOps() {
		return localUserOps;
	}

	public void setLocalUserOps(float localOps) {
		this.localUserOps = localOps;
	}

	public float getLocalUserReadOps() {
		return this.localUserReadOps;
	}

	public void setLocalUserReadOps(float localUserReadOps) {
		this.localUserReadOps = localUserReadOps;
	}

	public float getLocalUserWriteOps() {
		return this.localUserWriteOps;
	}

	public void setLocalUserWriteOps(float localWriteOps) {
		this.localUserWriteOps = localWriteOps;
	}

	@Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof NodeMeasurements)) {
            return false;
        }

        NodeMeasurements that = (NodeMeasurements) o;

        return Float.compare(that.cpuBusy, cpuBusy) == 0
               && Float.compare(that.avgProcessorBusy, avgProcessorBusy) == 0
               && that.cpuElapsedTime == cpuElapsedTime
               && Float.compare(that.netDataRecv, netDataRecv) == 0
               && Float.compare(that.netDataSent, netDataSent) == 0
               && Float.compare(that.cifsOps, cifsOps) == 0
               && Float.compare(that.fcpOps, fcpOps) == 0
               && Float.compare(that.iscsiOps, iscsiOps) == 0
               && Float.compare(that.nfsOps, nfsOps) == 0
               && Float.compare(that.otherOps, otherOps) == 0
               && Float.compare(that.readOps, readOps) == 0
               && Float.compare(that.sysAvgLatency, sysAvgLatency) == 0
               && Float.compare(that.sysReadLatency, sysReadLatency) == 0
               && Float.compare(that.sysWriteLatency, sysWriteLatency) == 0
               && Float.compare(that.writeOps, writeOps) == 0
               && Float.compare(that.totalOps, totalOps) == 0
               && Float.compare(that.fcpDataRecv, fcpDataRecv) == 0
               && Float.compare(that.fcpDataSent, fcpDataSent) == 0
               && Float.compare(that.opmSysReadThroughput, opmSysReadThroughput) == 0
               && Float.compare(that.opmSysThroughput, opmSysThroughput) == 0
               && Float.compare(that.opmSysWriteThroughput, opmSysWriteThroughput) == 0
               && Float.compare(that.opmSysUtilization, opmSysUtilization) == 0
               && Long.compare(that.totalCpMSecs, totalCpMSecs) == 0
               && Long.compare(that.p2Flush, p2Flush) == 0
               && Long.compare(that.maxProcessorElapsedTime, maxProcessorElapsedTime) == 0
               && Long.compare(that.totalKahunaBusy, totalKahunaBusy) == 0
               && Long.compare(that.deltaPollTime, deltaPollTime) == 0
               && Float.compare(that.maxAggregateUtilization, maxAggregateUtilization) == 0
               && Float.compare(that.percentCleanTimeUtil, percentCleanTimeUtil) == 0
               && Float.compare(that.diskShelfFibreChannelLoopUtilization, diskShelfFibreChannelLoopUtilization) == 0
               && Float.compare(that.nvmfOps, nvmfOps) == 0
               && Float.compare(that.nvmfDataRecv, nvmfDataRecv) == 0
               && Float.compare(that.nvmfDataSent, nvmfDataSent) == 0
				&& Float.compare(that.localUserOps, localUserOps) == 0
				&& Float.compare(that.localUserReadOps, localUserReadOps) == 0
				&& Float.compare(that.localUserWriteOps, localUserWriteOps) == 0
               ;
    }

    @Override
    public int hashCode() {
        int result = (cpuBusy != +0.0f ? Float.floatToIntBits(cpuBusy) : 0);
        result = 31 * result + (avgProcessorBusy != +0.0f ? Float.floatToIntBits(avgProcessorBusy) : 0);
        result = 31 * result + (int) (cpuElapsedTime ^ cpuElapsedTime >>> 32);
        result = 31 * result + (netDataRecv != +0.0f ? Float.floatToIntBits(netDataRecv) : 0);
        result = 31 * result + (netDataSent != +0.0f ? Float.floatToIntBits(netDataSent) : 0);
        result = 31 * result + (cifsOps != +0.0f ? Float.floatToIntBits(cifsOps) : 0);
        result = 31 * result + (fcpOps != +0.0f ? Float.floatToIntBits(fcpOps) : 0);
        result = 31 * result + (iscsiOps != +0.0f ? Float.floatToIntBits(iscsiOps) : 0);
        result = 31 * result + (nfsOps != +0.0f ? Float.floatToIntBits(nfsOps) : 0);
        result = 31 * result + (otherOps != +0.0f ? Float.floatToIntBits(otherOps) : 0);
        result = 31 * result + (readOps != +0.0f ? Float.floatToIntBits(readOps) : 0);
        result = 31 * result + (sysAvgLatency != +0.0f ? Float.floatToIntBits(sysAvgLatency) : 0);
        result = 31 * result + (sysReadLatency != +0.0f ? Float.floatToIntBits(sysReadLatency) : 0);
        result = 31 * result + (sysWriteLatency != +0.0f ? Float.floatToIntBits(sysWriteLatency) : 0);
        result = 31 * result + (writeOps != +0.0f ? Float.floatToIntBits(writeOps) : 0);
        result = 31 * result + (totalOps != +0.0f ? Float.floatToIntBits(totalOps) : 0);
        result = 31 * result + (fcpDataRecv != +0.0f ? Float.floatToIntBits(fcpDataRecv) : 0);
        result = 31 * result + (fcpDataSent != +0.0f ? Float.floatToIntBits(fcpDataSent) : 0);
        result = 31 * result + (opmSysReadThroughput != +0.0f ? Float.floatToIntBits(opmSysReadThroughput) : 0);
        result = 31 * result + (opmSysWriteThroughput != +0.0f ? Float.floatToIntBits(opmSysWriteThroughput) : 0);
        result = 31 * result + (opmSysThroughput != +0.0f ? Float.floatToIntBits(opmSysThroughput) : 0);
        result = 31 * result + (opmSysUtilization != +0.0f ? Float.floatToIntBits(opmSysUtilization) : 0);
        result = 31 * result + (int)(totalCpMSecs ^ totalCpMSecs >>> 32);
        result = 31 * result + (int)(p2Flush ^ p2Flush >>> 32);
        result = 31 * result + (int)(totalKahunaBusy ^ totalKahunaBusy >>> 32);
        result = 31 * result + (int)(maxProcessorElapsedTime ^ maxProcessorElapsedTime >>> 32);
        result = 31 * result + (int)(deltaPollTime ^ deltaPollTime >>> 32);
        result = 31 * result + (maxAggregateUtilization != +0.0f ? Float.floatToIntBits(maxAggregateUtilization) : 0);
        result = 31 * result + (percentCleanTimeUtil != +0.0f ? Float.floatToIntBits(percentCleanTimeUtil) : 0);
        result = 31 * result + (diskShelfFibreChannelLoopUtilization != +0.0f ? Float.floatToIntBits(diskShelfFibreChannelLoopUtilization) : 0);
        result = 31 * result + (nvmfOps != +0.0f ? Float.floatToIntBits(nvmfOps) : 0);
        result = 31 * result + (nvmfDataRecv != +0.0f ? Float.floatToIntBits(nvmfDataRecv) : 0);
        result = 31 * result + (nvmfDataSent != +0.0f ? Float.floatToIntBits(nvmfDataSent) : 0);
        result = 31 * result + (localUserOps != +0.0f ? Float.floatToIntBits(localUserOps) : 0);
		result = 31 * result + (localUserReadOps != +0.0f ? Float.floatToIntBits(localUserReadOps) : 0);
		result = 31 * result + (localUserWriteOps != +0.0f ? Float.floatToIntBits(localUserWriteOps) : 0);
        return result;
    }

	@Override
    public String toString() {
        return "NodeMeasurements{" +
               "cpuBusy=" + cpuBusy +
               ", avgProcessorBusy=" + avgProcessorBusy +
               ", cpuElapsedTime=" + avgProcessorBusy +
               ", netDataRecv=" + netDataRecv +
               ", netDataSent=" + netDataSent +
               ", cifsOps=" + cifsOps +
               ", fcpOps=" + fcpOps +
               ", iscsiOps=" + iscsiOps +
               ", nfsOps=" + nfsOps +
               ", otherOps=" + otherOps +
               ", readOps=" + readOps +
               ", sysAvgLatency=" + sysAvgLatency +
               ", sysReadLatency=" + sysReadLatency +
               ", sysWriteLatency=" + sysWriteLatency +
               ", writeOps=" + writeOps +
               ", totalOps=" + totalOps +
               ", fcpDataRecv=" + fcpDataRecv +
               ", fcpDataSent=" + fcpDataSent +
               ", opmSysReadThroughput=" + opmSysReadThroughput +
               ", opmSysWriteThroughput=" + opmSysWriteThroughput +
               ", opmSysThroughput=" + opmSysThroughput +
               ", opmSysUtilization=" + opmSysUtilization +
               ", totalCpMSecs=" + totalCpMSecs +
               ", p2Flush=" + p2Flush +
               ", totalKahunaBusy=" + totalKahunaBusy +
               ", maxProcessorElapsedTime=" + maxProcessorElapsedTime +
               ", deltaPollTime=" + deltaPollTime + 
               ", maxAggregateUtilization=" + maxAggregateUtilization + 
               ", percentCleanTimeUtil=" + percentCleanTimeUtil + 
               ", diskShelfFibreChannelLoopUtilization=" + diskShelfFibreChannelLoopUtilization + 
               ", nvmfOps=" + nvmfOps +
               ", nvmfDataRecv=" + nvmfDataRecv +
               ", nvmfDataSent=" + nvmfDataSent +
				", localOps=" + localUserOps +
				", localReadOps=" + localUserReadOps +
				", localWriteOps=" + localUserWriteOps +
               '}';
    }
}
