package com.netapp.oci.netapp.performance.node.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.node.interfaces.data.NodeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Anton Sergeev
 */
@Remote
public interface NodePerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//NodePerformanceQueryBean!com.netapp.oci.netapp.performance.node.interfaces.remote.NodePerformanceQueryRemote";

    /**
     * Gets node samples for period of time
     *
     * @param nodeIds set of node ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of node id to list ordered by time of node samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<NodeMeasurements>>> getSamples(Set<Long> nodeIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets node samples for period of time
     *
     * @param containerId container id
     * @param nodeIds set of node ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of node id to list ordered by time of node samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<NodeMeasurements>>> getSamples(Long containerId, Set<Long> nodeIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets node samples for period of time
     *
     * @param containerId container id
     * @param nodeId node id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of node samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<NodeMeasurements>> getSamples(Long containerId, Long nodeId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given node id's (nodeIds).  The map returned will contain only node ids that has a sample.
     * node ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the node ids passed.
     *
     * @param clusterId cluster id
     * @param nodeIds set of node ids
     * @return map of node id to latest sample SampleInfo<NodeMeasurements> node ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<NodeMeasurements>> getLatestSample(Long clusterId, Set<Long> nodeIds) throws NetappInterfaceException;



    /**
     * Gets node summaries for node ids
     *
     * @param nodeIds set of node ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of node id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<NodeMeasurements>>> getSummaries(Set<Long> nodeIds, Date fromTime, Date toTime) throws NetappInterfaceException;

   /**
     * Gets node summaries for node ids
     *
     * @param nodeIds set of node ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of node id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
   Map<Long, List<Summary<NodeMeasurements>>> getSummaries(Long clusterId, Set<Long> nodeIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;
}
