package com.netapp.oci.netapp.performance.node.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.node.interfaces.data.NodeMeasurements;
import com.netapp.oci.netapp.performance.node.interfaces.data.NodeRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

import javax.ejb.Remote;
import java.util.Collection;

/**
 * @author Anton Sergeev
 */
@Remote
public interface NodePerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//NodePerformanceBean!com.netapp.oci.netapp.performance.node.interfaces.remote.NodePerformanceRemote";

    /**
     * The Acq use this method to report on Node performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw node samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws NetappInterfaceException if failed to report
     */
    void reportNodeSamples(Long datasourceId, String clusterKey, long time, Collection<RawSample<NodeMeasurements>> rawSamples, long leaseTime, boolean isHistorical) throws NetappInterfaceException;
    
    void reportNodeRawSamples(Long datasourceId, String clusterKey, long time, Collection<RawSample<NodeRawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
}
