package com.netapp.oci.netapp.performance.nvmffclif;

import com.netapp.oci.netapp.performance.nvmffclif.interfaces.data.NvmfFcLifMeasurements;
import com.netapp.oci.netapp.performance.nvmffclif.interfaces.data.NvmfFcLifRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @author Anton Sergeev
 */
public class NvmfFcLifMetaData {
    public static final Column[] RAW_COLUMNS = new Column[]{
            new Column("avg_latency", "bigint unsigned"),
            new Column("total_ops", "bigint unsigned"),
            new Column("avg_other_latency", "bigint unsigned"),
            new Column("other_ops", "bigint unsigned"),
            new Column("read_data", "bigint unsigned"),
            new Column("avg_read_latency", "bigint unsigned"),
            new Column("read_ops", "bigint unsigned"),
            new Column("write_data", "bigint unsigned"),
            new Column("avg_write_latency", "bigint unsigned"),
            new Column("write_ops", "bigint unsigned"),
    };

    public static final Column[] SAMPLE_COLUMNS = new Column[]{
    		new Column("avgLatency", "float"),
    		new Column("totalOps", "float"),
            new Column("otherOps", "float"),
            new Column("readData", "float"),
            new Column("readOps", "float"),
            new Column("writeData", "float"),
            new Column("writeOps", "float"),
            new Column("avgOtherLatency", "float"),
            new Column("avgReadLatency", "float"),
            new Column("avgWriteLatency", "float"),
            new Column("opmNvmfFcLifThroughput", "float")
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
    		new Column("avgLatency", "float"),
    		new Column("totalOps", "float"),
            new Column("otherOps", "float"),
            new Column("readData", "float"),
            new Column("readOps", "float"),
            new Column("writeData", "float"),
            new Column("writeOps", "float"),
            new Column("avgOtherLatency", "float"),
            new Column("avgReadLatency", "float"),
            new Column("avgWriteLatency", "float"),
            new Column("opmNvmfFcLifThroughput", "float"),
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
    		{new RatioMethod("totalOps")},
    		{SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("otherOps")},
            {new RatioMethod("readOps")},
            {new RatioMethod("writeOps")},
            {SqlMethod.AVG}
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
    		new RatioMethod("totalOps"),	
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
    		SqlMethod.AVG,    	
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("otherOps"),
            new RatioMethod("readOps"),
            new RatioMethod("writeOps"),
            SqlMethod.AVG
    };

    private static ObjectParser<NvmfFcLifRawMeasurements> rawParser;
    private static ObjectParser<NvmfFcLifMeasurements> sampleParser;

    static {
        rawParser = new ObjectParser<NvmfFcLifRawMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NvmfFcLifRawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
            }

            public NvmfFcLifRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NvmfFcLifRawMeasurements(
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
            }
        };


        sampleParser = new ObjectParser<NvmfFcLifMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NvmfFcLifMeasurements objectData) throws TableException {
            	PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
            	PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getThroughput());
            }

            public NvmfFcLifMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NvmfFcLifMeasurements(
                		PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                		PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<NvmfFcLifRawMeasurements> getRawParser() {
        return rawParser;
    }
    
    public static ObjectParser<NvmfFcLifMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<NvmfFcLifMeasurements> getSampleParser() {
        return sampleParser;
    }
    
}
