package com.netapp.oci.netapp.performance.nvmffclif.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.nvmffclif.interfaces.data.NvmfFcLifMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Anton Sergeev
 */
@Remote
public interface NvmfFcLifPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//NvmfFcLifPerformanceQueryBean!com.netapp.oci.netapp.performance.nvmffclif.interfaces.remote.NvmfFcLifPerformanceQueryRemote";

    /**
     * Gets nvmffclif samples for period of time
     *
     * @param nvmfFcLifIds set of nvmffclif ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of nvmfFclif id to list ordered by time of nvmffclif samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<NvmfFcLifMeasurements>>> getSamples(Set<Long> nvmfFcLifIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * Gets nvmffclif samples for period of time
     *
     * @param containerId container id
     * @param nvmfFcLifIds set of nvmffclif ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of nvmfFclif id to list ordered by time of nvmffclif samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<NvmfFcLifMeasurements>>> getSamples(Long containerId, Set<Long> nvmfFcLifIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets nvmffclif samples for period of time
     *
     * @param containerId container id
     * @param nvmfFcLifId nvmfFcLif id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of nvmffclif samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    public List<SampleInfo<NvmfFcLifMeasurements>> getSamples(Long containerId, Long nvmfFcLifId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given nvmfFcLif id's (nvmfFcLifIds). The map returned will contain only nvmfFcLif ids that has a sample.
     * NVMf FC Lif ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the nvmfFcLif ids passed.
     *
     * @param vServerId vServer id
     * @param nvmfFcLifIds set of NVMf FC Lif ids
     * @return map of nvmfFcLif id to latest sample SampleInfo<NvmfFcLifMeasurements> . NVMf FC Lif ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<NvmfFcLifMeasurements>> getLatestSample(Long vServerId, Set<Long> nvmfFcLifIds) throws NetappInterfaceException;

    /**
     * Gets nvmffclif summaries for nvmffclif ids
     *
     * @param nvmfFcLifIds set of nvmffclif ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of nvmffclif id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long,  List<Summary<NvmfFcLifMeasurements>>> getSummaries(Set<Long> nvmfFcLifIds, Date fromTime, Date toTime) throws NetappInterfaceException;

   /**
     * Gets nvmffclif summaries for nvmffclif ids
     *
     * @param vServerId vServer id
     * @param nvmfFcLifIds set of nvmffclif ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of nvmffclif id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<NvmfFcLifMeasurements>>> getSummaries(Long vServerId, Set<Long> nvmfFcLifIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
