package com.netapp.oci.netapp.performance.nvmffclif.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.nvmffclif.interfaces.data.NvmfFcLifMeasurements;
import com.netapp.oci.netapp.performance.nvmffclif.interfaces.data.NvmfFcLifRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

import javax.ejb.Remote;
import java.util.Collection;

/**
 * @author Anton Sergeev
 */
@Remote
public interface NvmfFcLifPerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//NvmfFcLifPerformanceBean!com.netapp.oci.netapp.performance.nvmffclif.interfaces.remote.NvmfFcLifPerformanceRemote";

    /**
     * The Acq use this method to report on NvmfFcLif performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param vserverKey - the vserver Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw nvmffclif samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws NetappInterfaceException if failed to report
     */
    void reportNvmfFcLifSamples(Long datasourceId, 
                                String clusterKey, 
                                String vserverKey, 
                                long time, 
                                Collection<RawSample<NvmfFcLifMeasurements>> rawSamples, 
                                long leaseTime, boolean isHistorical) throws NetappInterfaceException;

    /**
     * The Acq use this method to report on NvmfFcLif raw performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param vserverKey - the vserver Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw nvmffclif samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     *
     * @throws NetappInterfaceException if failed to report
     */
    void reportNvmfFcLifRawSamples(Long datasourceId, 
                                   String clusterKey, 
                                   String vserverKey, 
                                   long time, 
                                   Collection<RawSample<NvmfFcLifRawMeasurements>> rawSamples, 
                                   long leaseTime) throws NetappInterfaceException;
    
}
