package com.netapp.oci.netapp.performance.nvmffclif.interfaces.remote;

import java.util.Collection;

import javax.ejb.Remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.nvmffclif.interfaces.data.NvmfFcLifMeasurements;
import com.netapp.oci.netapp.performance.nvmffclif.interfaces.data.NvmfFcLifRawMeasurements;
import com.netapp.oci.netapp.performance.nvmffclif.interfaces.data.NvmfFcLifVserverMeasurements;
import com.netapp.oci.netapp.performance.nvmffclif.interfaces.data.NvmfFcLifVserverRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

@Remote
public interface NvmfFcLifVserverPerformanceRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//NvmfFcLifVserverPerformanceBean!com.netapp.oci.netapp.performance.nvmffclif.interfaces.remote.NvmfFcLifVserverPerformanceRemote";

    /**
     * The Acq use this method to report on NvmfFcLifVserver performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw nvmffclif samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     * @param isHistorical - flag showing that the samples are for historical poll or for regular poll
     * @throws NetappInterfaceException if failed to report
     */
    void reportNvmfFcLifVserverSamples(Long datasourceId, 
                             String clusterKey, 
                             long time, 
                             Collection<RawSample<NvmfFcLifVserverMeasurements>> rawSamples, 
                             long leaseTime, boolean isHistorical) throws NetappInterfaceException;

    /**
     * The Acq use this method to report on NvmfFcLifVserver raw performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw nvmffclif samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     *
     * @throws NetappInterfaceException if failed to report
     */
    void reportNvmfFcLifVserverRawSamples(Long datasourceId, 
                                String clusterKey, 
                                long time, 
                                Collection<RawSample<NvmfFcLifVserverRawMeasurements>> rawSamples, 
                                long leaseTime) throws NetappInterfaceException;

}
