package com.netapp.oci.netapp.performance.opmheadroom;

import java.sql.ResultSet;
import java.sql.SQLException;

import com.netapp.oci.netapp.performance.opmheadroom.interfaces.data.OpmHeadRoomAggrMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;


public class OpmHeadRoomAggrMetaData{
	public static final Column[] SAMPLE_COLUMNS = new Column[]{
		new Column("serviceTime", "float"),
		new Column("obOptimalPointUtil", "float"),
		new Column("obOptimalPointLatency", "float"),
		new Column("obOptimalPointOps", "float"),
		new Column("obcOptimalPointUtil", "float"),
		new Column("obcOptimalPointLatency", "float"),
		new Column("obcOptimalPointOps", "float"),
		new Column("obConfidenceLevel", "float"),
		new Column("mOptimalPointUtil", "float"),
		new Column("mOptimalPointLatency", "float"),
		new Column("mOptimalPointOps", "float"),
		new Column("mcOptimalPointUtil", "float"),
		new Column("mcOptimalPointLatency", "float"),
		new Column("mcOptimalPointOps", "float"),
		new Column("mConfidenceLevel", "float"),
		new Column("cOperationalPointAllWorkloads", "float"),
		new Column("cOperationalPointInternalWorkloads", "float"),
		new Column("oOperationalPointAllWorkloads", "float"),
		new Column("oOperationalPointInternalWorkloads", "float"),
		new Column("cHRoomUsedPercent", "float"),
		new Column("cHRoomUsedNoInternalPercent", "float"),
		new Column("isObservationBased", "bool"),
		new Column("availableOps", "float"),
	};

	public static final Column[] SUMMARY_COLUMNS = new Column[]{
		new Column("cOperationalPointAllWorkloads", "float"),
		new Column("cHRoomUsedPercent", "float"),
		new Column("cHRoomUsedNoInternalPercent", "float"),
		new Column("availableOps", "float")
	};

	private static ObjectParser<OpmHeadRoomAggrMeasurements> sampleParser;
	private static ObjectParser<OpmHeadRoomAggrMeasurements> summaryParser;

	public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{SqlMethod.AVG},
		{null},
		{null},
		{null},
		{SqlMethod.AVG},
		{SqlMethod.AVG},
		{null},
		{SqlMethod.AVG},
	};


	public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		SqlMethod.AVG,
		null,
		null,
		null,
		SqlMethod.AVG,
		SqlMethod.AVG,
		null,
		SqlMethod.AVG
	};

	static {
		sampleParser = new ObjectParser<OpmHeadRoomAggrMeasurements>() {
			@Override
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, OpmHeadRoomAggrMeasurements objectData) throws TableException {
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getServiceTime());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObOptimalPointUtil());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObOptimalPointLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObOptimalPointOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObcOptimalPointUtil());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObcOptimalPointLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObcOptimalPointOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObConfidenceLevel());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getmOptimalPointUtil());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getmOptimalPointLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getmOptimalPointOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMcOptimalPointUtil());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMcOptimalPointLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMcOptimalPointOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getmConfidenceLevel());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcOperationalPointAllWorkloads());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcOperationalPointInternalWorkloads());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getoOperationalPointAllWorkloads());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getoOperationalPointInternalWorkloads());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcHRoomUsedPercent());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcHRoomUsedNoInternalPercent());
				preparedStatement.setNext(objectData.isObservationBased());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvailableOps());
			}

			@Override
            public OpmHeadRoomAggrMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
				int i = startIndex - 1;
				return new OpmHeadRoomAggrMeasurements()
					.setServiceTime(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setObOptimalPointUtil(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
	                .setObOptimalPointLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setObOptimalPointOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setObcOptimalPointUtil(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
	                .setObcOptimalPointLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setObcOptimalPointOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setObConfidenceLevel(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setmOptimalPointUtil(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
	                .setmOptimalPointLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setmOptimalPointOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setMcOptimalPointUtil(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                    .setMcOptimalPointLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setMcOptimalPointOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setmConfidenceLevel(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setcOperationalPointAllWorkloads(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setcOperationalPointInternalWorkloads(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setoOperationalPointAllWorkloads(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setoOperationalPointInternalWorkloads(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setcHRoomUsedPercent(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setcHRoomUsedNoInternalPercent(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setObservationBased(resultSet.getBoolean(++i))
					.setAvailableOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
			}
		};

		summaryParser = new ObjectParser<OpmHeadRoomAggrMeasurements>() {
			@Override
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, OpmHeadRoomAggrMeasurements objectData) throws TableException {
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getoOperationalPointAllWorkloads());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcHRoomUsedPercent());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcHRoomUsedNoInternalPercent());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvailableOps());
			}

			@Override
            public OpmHeadRoomAggrMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
				int i = startIndex - 1;
				return new OpmHeadRoomAggrMeasurements()
					.setoOperationalPointAllWorkloads(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setcHRoomUsedPercent(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setcHRoomUsedNoInternalPercent(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setAvailableOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
			}
		};
	}
		public static ObjectParser<OpmHeadRoomAggrMeasurements> getSummaryParser() {
			return summaryParser;
		}

		public static ObjectParser<OpmHeadRoomAggrMeasurements> getSampleParser() {
			return sampleParser;
		}
}