package com.netapp.oci.netapp.performance.opmheadroom;

import java.sql.ResultSet;
import java.sql.SQLException;

import com.netapp.oci.netapp.performance.opmheadroom.interfaces.data.OpmHeadRoomCPUMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;


public class OpmHeadRoomCPUMetaData{
	public static final Column[] SAMPLE_COLUMNS = new Column[]{
		new Column("serviceTime", "float"),
		new Column("obOptimalPointUtil", "float"),
		new Column("obOptimalPointLatency", "float"),
		new Column("obOptimalPointOps", "float"),
		new Column("obcOptimalPointUtil", "float"),
		new Column("obcOptimalPointLatency", "float"),
		new Column("obcOptimalPointOps", "float"),
		new Column("obConfidenceLevel", "float"),
		new Column("mOptimalPointUtil", "float"),
		new Column("mOptimalPointLatency", "float"),
		new Column("mOptimalPointOps", "float"),
		new Column("mcOptimalPointUtil", "float"),
		new Column("mcOptimalPointLatency", "float"),
		new Column("mcOptimalPointOps", "float"),
		new Column("mConfidenceLevel", "float"),
		new Column("cOperationalPointAllWorkloads", "float"),
		new Column("cOperationalPointInternalWorkloads", "float"),
		new Column("oOperationalPointAllWorkloads", "float"),
		new Column("oOperationalPointInternalWorkloads", "float"),
		new Column("cHRoomUsedPercent", "float"),
		new Column("cHRoomUsedNoInternalPercent", "float"),
		new Column("haLatencyProjection", "float"),
		new Column("haHRoomUsedProjection", "float"),
		new Column("haUtilProjection", "float"),
		new Column("haOpsProjection", "float"),
		new Column("isObservationBased", "tinyint(1)"),
		new Column("haHRoomUsedNoInternalProjection", "float"),
		new Column("availableOps", "float"),
	};

	public static final Column[] SUMMARY_COLUMNS = new Column[]{
        new Column("cOperationalPointAllWorkloads", "float"),
        new Column("cHRoomUsedPercent", "float"),
        new Column("cHRoomUsedNoInternalPercent", "float"),
        new Column("haLatencyProjection", "float"),
        new Column("haHRoomUsedProjection", "float"),
        new Column("haUtilProjection", "float"),
        new Column("haOpsProjection", "float"),
        new Column("haHRoomUsedNoInternalProjection", "float"),
        new Column("availableOps", "float"),
	};

	private static ObjectParser<OpmHeadRoomCPUMeasurements> sampleParser;
	private static ObjectParser<OpmHeadRoomCPUMeasurements> summaryParser;

	public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{null},
		{SqlMethod.AVG},
		{null},
		{null},
		{null},
		{SqlMethod.AVG},
		{SqlMethod.AVG},
		{SqlMethod.AVG},
		{SqlMethod.AVG},
		{SqlMethod.AVG},
		{SqlMethod.AVG},
		{null},
		{SqlMethod.AVG},
		{SqlMethod.AVG},
	};


	public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		null,
		SqlMethod.AVG,
		null,
		null,
		null,
		SqlMethod.AVG,
		SqlMethod.AVG,
		SqlMethod.AVG,
		SqlMethod.AVG,
		SqlMethod.AVG,
		SqlMethod.AVG,
		null,
		SqlMethod.AVG,
		SqlMethod.AVG
	};

	static {
		sampleParser = new ObjectParser<OpmHeadRoomCPUMeasurements>() {
			@Override
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, OpmHeadRoomCPUMeasurements objectData) throws TableException {
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getServiceTime());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObOptimalPointUtil());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObOptimalPointLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObOptimalPointOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObcOptimalPointUtil());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObcOptimalPointLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObcOptimalPointOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getObConfidenceLevel());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getmOptimalPointUtil());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getmOptimalPointLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getmOptimalPointOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMcOptimalPointUtil());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMcOptimalPointLatency());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMcOptimalPointOps());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getmConfidenceLevel());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcOperationalPointAllWorkloads());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcOperationalPointInternalWorkloads());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getoOperationalPointAllWorkloads());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getoOperationalPointInternalWorkloads());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcHRoomInUsePercent());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcHRoomUsedNoInternalPercent());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getHaLatencyProjection());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getHaHRoomUsedProjection());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getHaUtilProjection());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getHaOpsProjection());
				preparedStatement.setNext(objectData.isObservationBased());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getHaHRoomUsedNoInternalProjection());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvailableOps());
			}

			@Override
            public OpmHeadRoomCPUMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
				int i = startIndex - 1;
				return new OpmHeadRoomCPUMeasurements()
					.setServiceTime(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setObOptimalPointUtil(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setObOptimalPointLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
	                .setObOptimalPointOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setObcOptimalPointUtil(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setObcOptimalPointLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
	                .setObcOptimalPointOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setObConfidenceLevel(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setmOptimalPointUtil(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setmOptimalPointLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
	                .setmOptimalPointOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setMcOptimalPointUtil(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setMcOptimalPointLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
	                .setMcOptimalPointOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setmConfidenceLevel(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setcOperationalPointAllWorkloads(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setcOperationalPointInternalWorkloads(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setoOperationalPointAllWorkloads(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setoOperationalPointInternalWorkloads(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setcHRoomInUsePercent(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setcHRoomUsedNoInternalPercent(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setHaLatencyProjection(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setHaHRoomUsedProjection(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setHaUtilProjection(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setHaOpsProjection(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setObservationBased(resultSet.getBoolean(++i))
					.setHaHRoomUsedNoInternalProjection(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setAvailableOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
			}
		};

		summaryParser = new ObjectParser<OpmHeadRoomCPUMeasurements>() {
			@Override
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, OpmHeadRoomCPUMeasurements objectData) throws TableException {

				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcOperationalPointAllWorkloads());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcHRoomInUsePercent());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getcHRoomUsedNoInternalPercent());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getHaLatencyProjection());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getHaHRoomUsedProjection());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getHaUtilProjection());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getHaOpsProjection());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getHaHRoomUsedNoInternalProjection());
				PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvailableOps());
			}

			@Override
            public OpmHeadRoomCPUMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
				int i = startIndex - 1;
				return new OpmHeadRoomCPUMeasurements()
					.setcOperationalPointAllWorkloads(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setcHRoomInUsePercent(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setcHRoomUsedNoInternalPercent(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setHaLatencyProjection(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setHaHRoomUsedProjection(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setHaUtilProjection(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setHaOpsProjection(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
					.setHaHRoomUsedNoInternalProjection(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
				    .setAvailableOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
			}
		};

	}
		public static ObjectParser<OpmHeadRoomCPUMeasurements> getSummaryParser() {
			return summaryParser;
		}

		public static ObjectParser<OpmHeadRoomCPUMeasurements> getSampleParser() {
			return sampleParser;
		}
}