package com.netapp.oci.netapp.performance.opmheadroom.interfaces.impl;
/*
 * Copyright (c) 2015 NetApp
 * All rights reserved
 */

import com.netapp.oci.netapp.db.NetAppModel;
import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.PerformanceConstants;
import com.netapp.oci.netapp.performance.opmheadroom.interfaces.data.OpmHeadRoomCPUMeasurements;
import com.netapp.oci.netapp.performance.opmheadroom.interfaces.local.OpmHeadRoomCPUPerformanceQueryLocal;
import com.netapp.oci.netapp.performance.opmheadroom.interfaces.remote.OpmHeadRoomCPUPerformanceQueryRemote;
import com.netapp.oci.netapp.performance.shared.ComponentsFactoryRepository;
import com.netapp.oci.platform.common.ServerInterfaceInterceptor;
import com.netapp.oci.platform.common.TimeSpan;
import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.framework.performance.data.Summary;
import com.netapp.oci.platform.framework.performance.util.PerformanceUtils;
import com.onaro.commons.util.CollectionPrinter;
import com.onaro.sanscreen.util.Constant;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.interceptor.Interceptors;
import javax.sql.DataSource;
import java.sql.Connection;
import java.util.*;

@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
@RolesAllowed({"guest", "user", "admin", "scheduler", "acquisition"})
@Interceptors(ServerInterfaceInterceptor.class)
public class OpmHeadRoomCPUPerformanceQueryBean implements OpmHeadRoomCPUPerformanceQueryRemote, OpmHeadRoomCPUPerformanceQueryLocal {
    @Resource(name = "jdbc/OnaroDataSource", mappedName = "java:/onaro_pool", type = javax.sql.DataSource.class, authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    private static final Logger logger = LoggerFactory.getLogger(PerformanceConstants.QUERY_PACKAGE_LOGGER);

    @Override
    public Map<Long, List<SampleInfo<OpmHeadRoomCPUMeasurements>>> getSamples(Set<Long> nodeIds, Date fromTime, Date toTime) throws NetappInterfaceException {
        long startTime = System.currentTimeMillis();
        logger.info("Query " + nodeIds.size() + "node samples from " + fromTime + " to " + toTime);

        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            Map<Long, List<SampleInfo<OpmHeadRoomCPUMeasurements>>> containerIdToSamples = new HashMap<Long, List<SampleInfo<OpmHeadRoomCPUMeasurements>>>();
            Set<Long> remainingIds = new HashSet<Long>(nodeIds);

            Map<Long, Set<Long>> clusterIdToNodeIds = NetAppModel.INSTANCE.getClusterIdByNodeIds(connection, nodeIds);
            for (Map.Entry<Long, Set<Long>> entry : clusterIdToNodeIds.entrySet()) {
                Long containerId = entry.getKey();
                Set<Long> ids = entry.getValue();

                remainingIds.removeAll(ids);
                containerIdToSamples.putAll(getSamples(connection, containerId, ids, fromTime, toTime));
            }

            if (!remainingIds.isEmpty()) {
                throw new NetappInterfaceException("Failed to find node with id " + CollectionPrinter.LONG.toString(remainingIds));
            }

            return containerIdToSamples;
        } catch (Exception e) {
            throw new NetappInterfaceException("Failed to get node samples", e);
        } finally {
            ConnectionHelper.close(connection);
            logger.info("Total Execution time to query node samples: " + (System.currentTimeMillis() - startTime));
        }
    }

    @Override
    public Map<Long, List<SampleInfo<OpmHeadRoomCPUMeasurements>>> getSamples(Long containerId, Set<Long> nodeIds, Date fromTime, Date toTime) throws NetappInterfaceException {
        long startTime = System.currentTimeMillis();
        logger.info("Query " + nodeIds.size() + "node samples from " + fromTime + " to " + toTime + " for containerId:" + containerId);

        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            return getSamples(connection, containerId, nodeIds, fromTime, toTime);
        } catch (Exception e) {
            throw new NetappInterfaceException("Failed to get node samples", e);
        } finally {
            ConnectionHelper.close(connection);
            logger.info("Total Execution time to query node samples: " + (System.currentTimeMillis() - startTime));
        }
    }

    @Override
    public List<SampleInfo<OpmHeadRoomCPUMeasurements>> getSamples(Long containerId, Long nodeId, Date fromTime, Date toTime) throws NetappInterfaceException {
        return getSamples(containerId, Collections.singleton(nodeId), fromTime, toTime).get(nodeId);

    }

    private Map<Long, List<SampleInfo<OpmHeadRoomCPUMeasurements>>> getSamples(Connection connection, Long containerId, Set<Long> nodeIds, Date fromTime, Date toTime) throws PerformanceException {
        return PerformanceUtils.toSampleInfos(
                ComponentsFactoryRepository.getOpmHeadRoomCPUComponentsFactory().createPersistenceFacade(connection, containerId)
                        .getSamples(nodeIds, new TimeSpan(fromTime, toTime)));
    }

    @Override
    public Map<Long, SampleInfo<OpmHeadRoomCPUMeasurements>> getLatestSample(Long clusterId, Set<Long> nodeIds) throws NetappInterfaceException {
        long startTime = System.currentTimeMillis();
        logger.info("Query " + nodeIds.size() + "node samples for cluster:" + clusterId);
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            return PerformanceUtils.toSampleInfo(
                    ComponentsFactoryRepository.getOpmHeadRoomCPUComponentsFactory().createPersistenceFacade(connection, clusterId)
                            .getLatestSample(nodeIds));
        } catch (Exception e) {
            throw new NetappInterfaceException("Failed to get node samples", e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public Map<Long, List<Summary<OpmHeadRoomCPUMeasurements>>> getSummaries(Set<Long> nodeIds, Date fromTime, Date toTime) throws NetappInterfaceException {
        logger.info("Query " + nodeIds.size() + "node summaries from " + fromTime + " to " + toTime);

        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            Map<Long, List<Summary<OpmHeadRoomCPUMeasurements>>> idToSummaries = new HashMap<>();
            Set<Long> remainingIds = new HashSet<Long>(nodeIds);

            Map<Long, Set<Long>> clusterIdTofcPortIds = NetAppModel.INSTANCE.getClusterIdByNodeIds(connection, nodeIds);
            for (Map.Entry<Long, Set<Long>> entry : clusterIdTofcPortIds.entrySet()) {
                Long containerId = entry.getKey();
                Set<Long> ids = entry.getValue();

                remainingIds.removeAll(ids);
                List<Summary<OpmHeadRoomCPUMeasurements>> summaries = ComponentsFactoryRepository.getOpmHeadRoomCPUComponentsFactory()
                        .createPersistenceFacade(connection, containerId)
                        .getSummaries(new TimeSpan(fromTime, toTime), ids);
                PerformanceUtils.addSummariesToMap(idToSummaries, summaries);
            }

            if (!remainingIds.isEmpty()) {
                throw new NetappInterfaceException("Failed to find node with id " + CollectionPrinter.LONG.toString(remainingIds));
            }
            return idToSummaries;
        } catch (Exception e) {
            throw new NetappInterfaceException("Failed to get node summaries", e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public Map<Long, List<Summary<OpmHeadRoomCPUMeasurements>>> getSummaries(Long clusterId, Set<Long> nodeIds, Date fromTime, Date toTime) throws NetappInterfaceException {
        logger.info("Query " + nodeIds.size() + "node summaries from " + fromTime + " to " + toTime + " for cluster " +
        clusterId);

        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            Map<Long, List<Summary<OpmHeadRoomCPUMeasurements>>> idToSummaries = new HashMap<>();
            List<Summary<OpmHeadRoomCPUMeasurements>> summaries = ComponentsFactoryRepository.getOpmHeadRoomCPUComponentsFactory()
                        .createPersistenceFacade(connection, clusterId)
                        .getSummaries(new TimeSpan(fromTime, toTime), nodeIds);
            PerformanceUtils.addSummariesToMap(idToSummaries, summaries);
            return idToSummaries;
        } catch (Exception e) {
            throw new NetappInterfaceException("Failed to get node summaries", e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
      public Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException {
          logger.info("Query if " + objId + "node has counters from " + fromTime + " to " + toTime);

          Connection connection = null;
          try {
              connection = ConnectionHelper.newConnection(dataSource);
              return ComponentsFactoryRepository.getOpmHeadRoomCPUComponentsFactory()
                      .createPersistenceFacade(connection, containerId).doSamplesExist(new TimeSpan(fromTime, toTime), objId);
          } catch (Exception e) {
              throw new NetappInterfaceException("Failed to assert if node has counters ", e);
          } finally {
              ConnectionHelper.close(connection);
          }
      }
}
