package com.netapp.oci.netapp.performance.opmheadroom.interfaces.remote;
/*
 * Copyright (c) 2014 NetApp
 * All rights reserved
 */


import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.opmheadroom.interfaces.data.OpmHeadRoomAggrMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Remote
public interface OpmHeadRoomAggrPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//OpmHeadRoomAggrPerformanceQueryBean!com.netapp.oci.netapp.performance.opmheadroom.interfaces.remote.OpmHeadRoomAggrPerformanceQueryRemote";

    /**
     * Gets opm headroom aggr samples for period of time
     *
     * @param opmHeadRoomAggrIds set of opm head room Aggr ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of opm headroom Aggr id to list ordered by time of opm headroom aggr samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<OpmHeadRoomAggrMeasurements>>> getSamples(Set<Long> opmHeadRoomAggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * Gets opm headroom aggr samples for period of time
     *
     * @param clusterId                      cluster id
     * @param opmHeadRoomAggrIds 		 opm head room Aggr Ids 
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of opm headroom Aggr id to list ordered by time of opm headroom aggr samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<OpmHeadRoomAggrMeasurements>>> getSamples(Long clusterId, Set<Long> opmHeadRoomAggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets opm headroom aggr for period of time
     *
     * @param clusterId                     cluster id
     * @param opmHeadRoomAggrId  		opmHeadRoomAggrId
     * @param fromTime                      beginning of the time period
     * @param toTime                        end of the time period
     * @return list ordered by time of opm headroom aggr samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    List<SampleInfo<OpmHeadRoomAggrMeasurements>> getSamples(Long clusterId, Long opmHeadRoomAggrId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given opm headroom aggr Ids (opmHeadRoomAggrIds). The map returned will contain only opm head room aggr Ids that has a sample.
     * opm headroom aggr ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the opm headroom aggr ids passed.
     *
     * @param clusterId                      cluster id
     * @param opmHeadRoomAggrIds set of opm headroom aggr ids
     * @return map of opmHeadRoom Aggr id to latest sample SampleInfo<OpmHeadRoomAggrMeasurements>. opmHeadRoom aggr ids with no samples will not be included in this map.
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, SampleInfo<OpmHeadRoomAggrMeasurements>> getLatestSample(Long clusterId, Set<Long> opmHeadRoomAggrIds) throws NetappInterfaceException;

    /**
     * Gets QoS workload queue nBlade summaries for opm headroom aggr ids
     *
     * @param opmHeadRoomAggrIds set of opm headroom aggr Ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of opm headroom Aggr id to summary data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<Summary<OpmHeadRoomAggrMeasurements>>> getSummaries(Set<Long> opmHeadRoomAggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS workload queue nBlade summaries for opm headroom aggr ids
     *
     * @param clusterId Cluster Id
     * @param opmHeadRoomAggrIds set of opm headroom aggr Ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of opm headroom aggr id to summary data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<Summary<OpmHeadRoomAggrMeasurements>>> getSummaries(Long clusterId, Set<Long> opmHeadRoomAggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param clusterId
     * @param opmHeadRoomAggrId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long clusterId, Long opmHeadRoomAggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;
}
