/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.processor;

import com.netapp.oci.netapp.performance.processor.interfaces.data.ProcessorMeasurements;
import com.netapp.oci.netapp.performance.processor.interfaces.data.ProcessorSummary;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @author Nancy Walia
 */
public class ProcessorMetaData {
    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("processorBusy", "float"),
            new Column("processorElapsedTime", "bigint(20)"),
            new Column("dBladeBusy", "float"),
            new Column("nBladeBusy", "float"),
            new Column("kahunaBusy", "float"),
            new Column("storageBusy", "float"),
            new Column("raidBusy", "float"),
            new Column("targetBusy", "float"),
            new Column("cifsBusy", "float"),
            new Column("clusterBusy", "float"),
            new Column("protocolBusy", "float"),
            new Column("nwkExclusiveBusy", "float"),
            new Column("nwkLegacyBusy", "float"),
};

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("processorBusy", "float"),
            new Column("processorElapsedTime", "bigint(20)"),
            new Column("dBladeBusy", "float"),
            new Column("nBladeBusy", "float"),
            new Column("kahunaBusy", "float"),
            new Column("storageBusy", "float"),
            new Column("raidBusy", "float"),
            new Column("targetBusy", "float"),
            new Column("cifsBusy", "float"),
            new Column("clusterBusy", "float"),
            new Column("protocolBusy", "float"),
            new Column("nwkExclusiveBusy", "float"),
            new Column("nwkLegacyBusy", "float"),
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {new RatioMethod("processorElapsedTime")},
            {SqlMethod.SUM},
            {new RatioMethod("processorElapsedTime")},
            {new RatioMethod("processorElapsedTime")},
            {new RatioMethod("processorElapsedTime")},
            {new RatioMethod("processorElapsedTime")},
            {new RatioMethod("processorElapsedTime")},
            {new RatioMethod("processorElapsedTime")},
            {new RatioMethod("processorElapsedTime")},
            {new RatioMethod("processorElapsedTime")},
            {new RatioMethod("processorElapsedTime")},
            {new RatioMethod("processorElapsedTime")},
            {new RatioMethod("processorElapsedTime")},
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            new RatioMethod("processorElapsedTime"),
            SqlMethod.SUM,
            new RatioMethod("processorElapsedTime"),
            new RatioMethod("processorElapsedTime"),
            new RatioMethod("processorElapsedTime"),
            new RatioMethod("processorElapsedTime"),
            new RatioMethod("processorElapsedTime"),
            new RatioMethod("processorElapsedTime"),
            new RatioMethod("processorElapsedTime"),
            new RatioMethod("processorElapsedTime"),
            new RatioMethod("processorElapsedTime"),
            new RatioMethod("processorElapsedTime"),
            new RatioMethod("processorElapsedTime"),
    };

    private static ObjectParser<ProcessorMeasurements> sampleParser;

    static {
        sampleParser = new ObjectParser<ProcessorMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, ProcessorMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getProcessorBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getProcessorElapsedTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDBladeBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNBladeBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getKahunaBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getStorageBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRaidBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTargetBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getClusterBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getProtocolBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNwkExclusiveBusy());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNwkLegacyBusy());
            }

            public ProcessorMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new ProcessorMeasurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getLongNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<ProcessorMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<ProcessorMeasurements> getSampleParser() {
        return sampleParser;
    }
}