/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.processor.interfaces.data;

import java.io.Serializable;

/**
 * @author Nancy Walia
 */
@SuppressWarnings("UnusedDeclaration")
public class ProcessorSummary implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * Percentage of elapsed time that the processor is executing non-idle processes
     */
    private float processorBusy;

    /**
     * Derived : Sum of elements from domain_busy representing 'd_blade' usage:  exempt, kahuna, raid, sm_exempt, storage, wafl_exempt, wafl_xcleaner
     */
    private float dBladeBusy;

    /**
     * Derived : Sum of elements from domain_busy representing 'n_blade' usage: cifs, cluster, dnscache, nwk_exclusive, nwk_exempt, nwk_legacy, protocol, target
     */
    private float nBladeBusy;

    /**
     * Percentage of elapsed time that the processor spent in busy domain 'kahuna'
     */
    private float kahunaBusy;

    /**
     * Percentage of elapsed time that the processor spent in busy domain 'storage'
     */
    private float storageBusy;

    /**
     * Percentage of elapsed time that the processor spent in busy domain 'raid'
     */
    private float raidBusy;

    /**
     * Percentage of elapsed time that the processor spent in busy domain 'target'
     */
    private float targetBusy;

    /**
     * Percentage of elapsed time that the processor spent in busy domain 'cifs'
     */
    private float cifsBusy;

    /**
     * Percentage of elapsed time that the processor spent in busy domain 'cluster'
     */
    private float clusterBusy;

    /**
     * Percentage of elapsed time that the processor spent in busy domain 'protocol'
     */
    private float protocolBusy;

    /**
     * Percentage of elapsed time that the processor spent in busy domain 'nwk_exclusive'
     */
    private float nwkExclusiveBusy;

    /**
     * Percentage of elapsed time that the processor spent in busy domain 'nwk_legacy'
     */
    private float nwkLegacyBusy;

    public ProcessorSummary() {
    }

    public ProcessorSummary(float processorBusy, float dBladeBusy, float nBladeBusy, float kahunaBusy, float storageBusy, float raidBusy, float targetBusy, float cifsBusy, float clusterBusy, float protocolBusy, float nwkExclusiveBusy, float nwkLegacyBusy) {
        this.processorBusy = processorBusy;
        this.dBladeBusy = dBladeBusy;
        this.nBladeBusy = nBladeBusy;
        this.kahunaBusy = kahunaBusy;
        this.storageBusy = storageBusy;
        this.raidBusy = raidBusy;
        this.targetBusy = targetBusy;
        this.cifsBusy = cifsBusy;
        this.clusterBusy = clusterBusy;
        this.protocolBusy = protocolBusy;
        this.nwkExclusiveBusy = nwkExclusiveBusy;
        this.nwkLegacyBusy = nwkLegacyBusy;
    }

    public float getProcessorBusy() {
        return processorBusy;
    }

    public void setProcessorBusy(float processorBusy) {
        this.processorBusy = processorBusy;
    }

    public float getDBladeBusy() {
        return dBladeBusy;
    }

    public void setDBladeBusy(float dBladeBusy) {
        this.dBladeBusy = dBladeBusy;
    }

    public float getNBladeBusy() {
        return nBladeBusy;
    }

    public void setNBladeBusy(float nBladeBusy) {
        this.nBladeBusy = nBladeBusy;
    }

    public float getdBladeBusy() {
        return dBladeBusy;
    }

    public void setdBladeBusy(float dBladeBusy) {
        this.dBladeBusy = dBladeBusy;
    }

    public float getnBladeBusy() {
        return nBladeBusy;
    }

    public void setnBladeBusy(float nBladeBusy) {
        this.nBladeBusy = nBladeBusy;
    }

    public float getKahunaBusy() {
        return kahunaBusy;
    }

    public void setKahunaBusy(float kahunaBusy) {
        this.kahunaBusy = kahunaBusy;
    }

    public float getStorageBusy() {
        return storageBusy;
    }

    public void setStorageBusy(float storageBusy) {
        this.storageBusy = storageBusy;
    }

    public float getRaidBusy() {
        return raidBusy;
    }

    public void setRaidBusy(float raidBusy) {
        this.raidBusy = raidBusy;
    }

    public float getTargetBusy() {
        return targetBusy;
    }

    public void setTargetBusy(float targetBusy) {
        this.targetBusy = targetBusy;
    }

    public float getCifsBusy() {
        return cifsBusy;
    }

    public void setCifsBusy(float cifsBusy) {
        this.cifsBusy = cifsBusy;
    }

    public float getClusterBusy() {
        return clusterBusy;
    }

    public void setClusterBusy(float clusterBusy) {
        this.clusterBusy = clusterBusy;
    }

    public float getProtocolBusy() {
        return protocolBusy;
    }

    public void setProtocolBusy(float protocolBusy) {
        this.protocolBusy = protocolBusy;
    }

    public float getNwkExclusiveBusy() {
        return nwkExclusiveBusy;
    }

    public void setNwkExclusiveBusy(float nwkExclusiveBusy) {
        this.nwkExclusiveBusy = nwkExclusiveBusy;
    }

    public float getNwkLegacyBusy() {
        return nwkLegacyBusy;
    }

    public void setNwkLegacyBusy(float nwkLegacyBusy) {
        this.nwkLegacyBusy = nwkLegacyBusy;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        ProcessorSummary that = (ProcessorSummary) o;

        return Float.compare(that.cifsBusy, cifsBusy) == 0
                && Float.compare(that.clusterBusy, clusterBusy) == 0
                && Float.compare(that.dBladeBusy, dBladeBusy) == 0
                && Float.compare(that.kahunaBusy, kahunaBusy) == 0
                && Float.compare(that.nBladeBusy, nBladeBusy) == 0
                && Float.compare(that.nwkExclusiveBusy, nwkExclusiveBusy) == 0
                && Float.compare(that.nwkLegacyBusy, nwkLegacyBusy) == 0
                && Float.compare(that.processorBusy, processorBusy) == 0
                && Float.compare(that.protocolBusy, protocolBusy) == 0
                && Float.compare(that.raidBusy, raidBusy) == 0
                && Float.compare(that.storageBusy, storageBusy) == 0
                && Float.compare(that.targetBusy, targetBusy) == 0;
    }

    @Override
    public int hashCode() {
        int result = (processorBusy != +0.0f ? Float.floatToIntBits(processorBusy) : 0);
        result = 31 * result + (dBladeBusy != +0.0f ? Float.floatToIntBits(dBladeBusy) : 0);
        result = 31 * result + (nBladeBusy != +0.0f ? Float.floatToIntBits(nBladeBusy) : 0);
        result = 31 * result + (kahunaBusy != +0.0f ? Float.floatToIntBits(kahunaBusy) : 0);
        result = 31 * result + (storageBusy != +0.0f ? Float.floatToIntBits(storageBusy) : 0);
        result = 31 * result + (raidBusy != +0.0f ? Float.floatToIntBits(raidBusy) : 0);
        result = 31 * result + (targetBusy != +0.0f ? Float.floatToIntBits(targetBusy) : 0);
        result = 31 * result + (cifsBusy != +0.0f ? Float.floatToIntBits(cifsBusy) : 0);
        result = 31 * result + (clusterBusy != +0.0f ? Float.floatToIntBits(clusterBusy) : 0);
        result = 31 * result + (protocolBusy != +0.0f ? Float.floatToIntBits(protocolBusy) : 0);
        result = 31 * result + (nwkExclusiveBusy != +0.0f ? Float.floatToIntBits(nwkExclusiveBusy) : 0);
        result = 31 * result + (nwkLegacyBusy != +0.0f ? Float.floatToIntBits(nwkLegacyBusy) : 0);
        return result;
    }

    @Override
    public String toString() {
        return "ProcessorSummary{" +
                "processorBusy=" + processorBusy +
                ", dBladeBusy=" + dBladeBusy +
                ", nBladeBusy=" + nBladeBusy;
    }
}