package com.netapp.oci.netapp.performance.processor.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.processor.interfaces.data.ProcessorMeasurements;
import com.netapp.oci.netapp.performance.processor.interfaces.data.ProcessorSummary;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Nancy Walia
 */

@Remote
public interface ProcessorPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//ProcessorPerformanceQueryBean!com.netapp.oci.netapp.performance.processor.interfaces.remote.ProcessorPerformanceQueryRemote";

    /**
     * Gets processor samples for period of time
     *
     * @param processorIds set of processor ids
     * @param fromTime     begining of the time period
     * @param toTime       end of the time period
     * @return map of processor id to list ordered by time of processor samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<ProcessorMeasurements>>> getSamples(Set<Long> processorIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets processor samples for period of time
     *
     * @param containerId  container id
     * @param processorIds set of processor ids
     * @param fromTime     begining of the time period
     * @param toTime       end of the time period
     * @return map of processor id to list ordered by time of processor samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<ProcessorMeasurements>>> getSamples(Long containerId, Set<Long> processorIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets processor samples for period of time
     *
     * @param containerId container id
     * @param processorId processor id
     * @param fromTime    begining of the time period
     * @param toTime      end of the time period
     * @return list ordered by time of processor samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    public List<SampleInfo<ProcessorMeasurements>> getSamples(Long containerId, Long processorId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given processor id's (processorIds).  The map returned will contain only processor ids that has a sample.
     * processor ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the processor ids passed.
     *
     * @param clusterId cluster id
     * @param processorIds set of processor ids
     * @return map of processor id to latest sample SampleInfo<ProcessorMeasurements>. processor ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<ProcessorMeasurements>> getLatestSample(Long clusterId, Set<Long> processorIds) throws NetappInterfaceException;

    /**
     * Gets processor summaries for processor ids
     *
     * @param processorIds set of processor ids
     * @param fromTime     begining of the time period
     * @param toTime       end of the time period
     * @return map of processor id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<ProcessorMeasurements>>> getSummaries(Set<Long> processorIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets processor summaries for processor ids
     *
     * @param clusterId cluster id
     * @param processorIds set of processor ids
     * @param fromTime     begining of the time period
     * @param toTime       end of the time period
     * @return map of processor id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<ProcessorMeasurements>>> getSummaries(Long clusterId, Set<Long> processorIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;
}