/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.qosservicecenter;

import java.sql.ResultSet;
import java.sql.SQLException;
import com.netapp.oci.netapp.performance.qosservicecenter.interfaces.data.QosServiceCenterMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.MeanRatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

public class QosServiceCenterMetaData {

    public static final Column[] SAMPLE_COLUMNS = new Column[] {
        new Column("waitTime", "bigint(20)"),
        new Column("utilization", "float"),
        new Column("collectionPeriod", "bigint(20)"),
        //These were added after the collection period to avoid changes in the OPM 1.1 migration tool
        new Column("cpuElapsedTime", "bigint(20)"),
        new Column("numInstances", "bigint(20)")};

    public static final Column[] SUMMARY_COLUMNS = new Column[] {
        new Column("waitTime", "bigint(20)"),
        new Column("utilization", "float"),
        new Column("collectionPeriod", "bigint(20)"),
        //These were added after the collection period to avoid changes in the OPM 1mr.1 migration tool
        new Column("cpuElapsedTime", "bigint(20)"),
        //If the number of instances changes during the summary period, then we average the number of instances
        new Column("numInstances", "float")};

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][] {
        {SqlMethod.SUM}, // waitTime
        {new MeanRatioMethod("cpuElapsedTime", "numInstances")}, // utilization
        {SqlMethod.SUM},  // collectionPeriod
        {SqlMethod.SUM},  // CPU Elapsed Time
        {SqlMethod.AVG},  // Number of Instances
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[] {
        SqlMethod.AVG, // waitTime
        new MeanRatioMethod("cpuElapsedTime", "numInstances"), // utilization
        SqlMethod.AVG,  // collectionPeriod
        SqlMethod.SUM,  // CPU Elapsed Time
        SqlMethod.AVG,  // Number of Instances
    };

    private static ObjectParser<QosServiceCenterMeasurements> sampleParser;

    static {
        sampleParser = new ObjectParser<QosServiceCenterMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement,
                    QosServiceCenterMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaitTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUtilization());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCollectionPeriod());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuElapsedTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNumInstances());
            }

            public QosServiceCenterMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                final QosServiceCenterMeasurements qosServiceCenterMeasurements = new QosServiceCenterMeasurements();
                int i = startIndex - 1;
                qosServiceCenterMeasurements.setWaitTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setUtilization(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setCollectionPeriod(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setCpuElapsedTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setNumInstances(PerformanceTableUtils.getLongNullable(resultSet, ++i));
                return qosServiceCenterMeasurements;
            }
        };
    }

    public static ObjectParser<QosServiceCenterMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<QosServiceCenterMeasurements> getSampleParser() {
        return sampleParser;
    }

}
