package com.netapp.oci.netapp.performance.qosservicecenter.interfaces.data;

import java.io.Serializable;


public class QosServiceCenterMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * Summation of Service Center child Workload Details' wait_time for delay service centers (target object-name
     * "resource", undefined for non-delay service centers
     */
    private long waitTime;

    /**
     * Service Center busy_percent (derived from cpu_elapsed_time) for non-delay service centers (target object-name
     * "resource"), undefined for delay service centers
     */
    private float utilization;

    /**
     * Elapsed time since boot
     */
    private long cpuElapsedTime;

    /**
     * The number of instances (processors, disks, and so on) in the resource.
     */
    private long numInstances;

    /**
     * Number of milliseconds between the previous and the current poll. For non-delay service centers this is the time
     * between the resource object collections and for delay centers this is the time between the workload detail object
     * (SN) or workload queue N/D blade object (FS and above) collections.
     */
    private long collectionPeriod;


    public QosServiceCenterMeasurements() {
    }

    public QosServiceCenterMeasurements(final long waitTime, final float utilization, final long cpuElapsedTime,
                                        final long numInstances,  final long collectionPeriod) {
        this.waitTime = waitTime;
        this.utilization = utilization;
        this.cpuElapsedTime = cpuElapsedTime;
        this.numInstances = numInstances;
        this.collectionPeriod = collectionPeriod;
    }

    public long getWaitTime() {
        return waitTime;
    }

    public QosServiceCenterMeasurements setWaitTime(long waitTime) {
        this.waitTime = waitTime;
        return this;
    }

    public float getUtilization() {
        return utilization;
    }

    public QosServiceCenterMeasurements setUtilization(float utilization) {
        this.utilization = utilization;
        return this;
    }

    public long getCpuElapsedTime() {
        return cpuElapsedTime;
    }

    public QosServiceCenterMeasurements setCpuElapsedTime(long cpuElapsedTime) {
        this.cpuElapsedTime = cpuElapsedTime;
        return this;
    }

    public long getNumInstances() {
        return numInstances;
    }

    public QosServiceCenterMeasurements setNumInstances(long numInstances) {
        this.numInstances = numInstances;
        return this;
    }

    public long getCollectionPeriod() {
        return collectionPeriod;
    }

    public QosServiceCenterMeasurements setCollectionPeriod(long collectionPeriod) {
        this.collectionPeriod = collectionPeriod;
        return this;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        QosServiceCenterMeasurements that = (QosServiceCenterMeasurements) o;

        if (collectionPeriod != that.collectionPeriod) return false;
        if (cpuElapsedTime != that.cpuElapsedTime) return false;
        if (numInstances != that.numInstances) return false;
        if (Float.compare(that.utilization, utilization) != 0) return false;
        if (waitTime != that.waitTime) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = (int) (waitTime ^ (waitTime >>> 32));
        result = 31 * result + (utilization != +0.0f ? Float.floatToIntBits(utilization) : 0);
        result = 31 * result + (int) (cpuElapsedTime ^ (cpuElapsedTime >>> 32));
        result = 31 * result + (int) (numInstances ^ (numInstances >>> 32));
        result = 31 * result + (int) (collectionPeriod ^ (collectionPeriod >>> 32));
        return result;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("QosServiceCenterMeasurements{");
        sb.append("waitTime=").append(waitTime);
        sb.append(", utilization=").append(utilization);
        sb.append(", cpuElapsedTime=").append(cpuElapsedTime);
        sb.append(", numInstances=").append(numInstances);
        sb.append(", collectionPeriod=").append(collectionPeriod);
        sb.append('}');
        return sb.toString();
    }
}
