/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.qosservicecenter.interfaces.remote;


import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.qosservicecenter.interfaces.data.QosServiceCenterMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Remote
public interface QosServiceCenterPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//QosServiceCenterPerformanceQueryBean!com.netapp.oci.netapp.performance.qosservicecenter.interfaces.remote.QosServiceCenterPerformanceQueryRemote";

    /**
     * Gets QoS service center samples for period of time
     *
     * @param qosServiceCenterIds set of QoS service center ids
     * @param fromTime            beginning of the time period
     * @param toTime              end of the time period
     * @return map of QoS service center id to list ordered by time of QoS service center samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<QosServiceCenterMeasurements>>> getSamples(Set<Long> qosServiceCenterIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS service center samples for period of time
     *
     * @param containerId         container id
     * @param qosServiceCenterIds set of QoS service center ids
     * @param fromTime            beginning of the time period
     * @param toTime              end of the time period
     * @return map of QoS service center id to list ordered by time of QoS service center samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<QosServiceCenterMeasurements>>> getSamples(Long containerId, Set<Long> qosServiceCenterIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS service center samples for period of time
     *
     * @param containerId        container id
     * @param qosServiceCenterId QoS service center id
     * @param fromTime           beginning of the time period
     * @param toTime             end of the time period
     * @return list ordered by time of QoS service center samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<QosServiceCenterMeasurements>> getSamples(Long containerId, Long qosServiceCenterId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given qosServiceCenter id's (qosServiceCenterIds).  The map returned will contain only qosServiceCenter ids that has a sample.
     * qosServiceCenter ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the qosServiceCenter ids passed.
     *
     * @param clusterId cluster id
     * @param qosServiceCenterIds set of qosServiceCenter ids
     * @return map of qosServiceCenter id to latest sample SampleInfo<QosServiceCenterMeasurements>. qosServiceCenter ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<QosServiceCenterMeasurements>> getLatestSample(Long clusterId, Set<Long> qosServiceCenterIds) throws NetappInterfaceException;

    /**
     * Gets QoS service center summaries for QoS service center ids
     *
     * @param qosServiceCenterIds set of QoS service center ids
     * @param fromTime            beginning of the time period
     * @param toTime              end of the time period
     * @return map of QoS service center id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<QosServiceCenterMeasurements>>> getSummaries(Set<Long> qosServiceCenterIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS service center summaries for QoS service center ids
     *
     * @param clusterId Cluster Id
     * @param qosServiceCenterIds set of QoS service center ids
     * @param fromTime            beginning of the time period
     * @param toTime              end of the time period
     * @return map of QoS service center id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<QosServiceCenterMeasurements>>> getSummaries(Long clusterId, Set<Long> qosServiceCenterIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
