/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.oci.netapp.performance.qosworkload;

import java.sql.ResultSet;
import java.sql.SQLException;

import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosLunWorkloadMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

/**
 * Created by jianwei on 5/25/17.
 */
public class QosLunWorkloadMetaData {

    public static final Column[] SAMPLE_COLUMNS = new Column[] {
        new Column("latency", "float"),
        new Column("ops", "float"),
        new Column("totalData", "float"),
        new Column("readData", "float"),
        new Column("readLatency", "float"),
        new Column("readOps", "float"),
        new Column("writeData", "float"),
        new Column("writeLatency", "float"),
        new Column("writeOps", "float"),
        new Column("logicalSize", "bigint(20)"),
        new Column("opsPerTB", "float"),
        new Column("maxIOPS", "float"),
        new Column("maxMBPS", "float"),
        new Column("minIOPS", "float"),
        new Column("maxIOPSPerTB", "float"),
        new Column("minIOPSPerTB", "float")};


    public static final Column[] SUMMARY_COLUMNS = new Column[] {
        new Column("latency", "float"),
        new Column("ops", "float"),
        new Column("totalData", "float"),
        new Column("readData", "float"),
        new Column("readLatency", "float"),
        new Column("readOps", "float"),
        new Column("writeData", "float"),
        new Column("writeLatency", "float"),
        new Column("writeOps", "float"),
        new Column("logicalSize", "bigint(20)"),
        new Column("opsPerTB", "float"),
        new Column("maxIOPS", "float"),
        new Column("maxMBPS", "float"),
        new Column("minIOPS", "float"),
        new Column("maxIOPSPerTB", "float"),
        new Column("minIOPSPerTB", "float")};


    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][] {
        {new RatioMethod("ops")}, //latency
        {SqlMethod.AVG}, // ops
        {SqlMethod.AVG}, // totalData
        {SqlMethod.AVG}, // readData
        {new RatioMethod("readOps")}, // readLatency
        {SqlMethod.AVG}, // readOps
        {SqlMethod.AVG}, // writeData
        {new RatioMethod("writeOps")}, // writeLatency
        {SqlMethod.AVG}, // writeOps
        {SqlMethod.AVG}, //logicalSize
        {SqlMethod.AVG}, // opsPerTB
        {SqlMethod.AVG}, // maxIOPS
        {SqlMethod.AVG}, // maxMBPS
        {SqlMethod.AVG}, // minIOPS
        {SqlMethod.AVG}, // maxIOPSPerTB
        {SqlMethod.AVG}, // minIOPSPerTB
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[] {
        new RatioMethod("ops"), //latency
        SqlMethod.AVG, // ops
        SqlMethod.AVG, // totalData
        SqlMethod.AVG, // readData
        new RatioMethod("readOps"), // readLatency
        SqlMethod.AVG, // readOps
        SqlMethod.AVG, // writeData
        new RatioMethod("writeOps"), // writeLatency
        SqlMethod.AVG, // writeOps
        SqlMethod.AVG, //logicalSize
        SqlMethod.AVG, // opsPerTB
        SqlMethod.AVG, // maxIOPS
        SqlMethod.AVG, // maxMBPS
        SqlMethod.AVG, //minIOPS
        SqlMethod.AVG, // maxIOPSPerTB
        SqlMethod.AVG, // minIOPSPerTB
    };


    private static ObjectParser<QosLunWorkloadMeasurements> sampleParser;

    static {
        sampleParser = new ObjectParser<QosLunWorkloadMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement,
                                               QosLunWorkloadMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLogicalSize());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpsPerTB());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMaxIOPS());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMaxMBPS());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMinIOPS());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMaxIOPSPerTB());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMinIOPSPerTB());
            }

            public QosLunWorkloadMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;

                QosLunWorkloadMeasurements qosLunWorkloadMeasurements =  new QosLunWorkloadMeasurements()
                        .setLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setTotalData(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setReadData(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setReadLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setReadOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setWriteData(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setWriteLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setWriteOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setLogicalSize(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setOpsPerTB(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setMaxIOPS(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setMaxMBPS(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setMinIOPS(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setMaxIOPSPerTB(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setMinIOPSPerTB(PerformanceTableUtils.getFloatNullable(resultSet, ++i));

                return qosLunWorkloadMeasurements;
            }
        };
    }

    public static ObjectParser<QosLunWorkloadMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<QosLunWorkloadMeasurements> getSampleParser() {
        return sampleParser;
    }
}
