/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.qosworkload;

import java.sql.ResultSet;
import java.sql.SQLException;

import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosVolumeWorkloadMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;
import com.netapp.oci.platform.framework.performance.tables.WmaBaseNotRated;
import com.netapp.oci.platform.framework.performance.tables.WmaBaseRated;

public class QosVolumeWorkloadMetaData {
    public static final Column[] SAMPLE_COLUMNS = new Column[] {
        new Column("latency", "float"),
        new Column("ops", "float"),
        new Column("totalData", "float"),
        new Column("readData", "float"),
        new Column("readLatency", "float"),
        new Column("readOps", "float"),
        new Column("writeData", "float"),
        new Column("writeLatency", "float"),
        new Column("writeOps", "float"),
        new Column("cacheMissRate", "float"),
        new Column("cacheMissRateBase", "bigint(20)"),
        new Column("logicalSize", "bigint(20)"),
        new Column("opsPerTB", "float"),
        new Column("maxIOPS", "float"),
        new Column("maxMBPS", "float"),
        new Column("minIOPS", "float"),
        new Column("maxIOPSPerTB", "float"),
        new Column("minIOPSPerTB", "float")};

    public static final Column[] DAILY_SUMMARY_COLUMNS = new Column[]{
            new Column("minLatency", "float"),
            new Column("maxLatency", "float"),
            new Column("latency", "float"),
            new Column("minOps", "float"),
            new Column("maxOps", "float"),
            new Column("ops", "float"),
            new Column("minTotalData", "float"),
            new Column("maxTotalData", "float"),
            new Column("totalData", "float"),
            new Column("minReadData", "float"),
            new Column("maxReadData", "float"),
            new Column("readData", "float"),
            new Column("minReadLatency", "float"),
            new Column("maxReadLatency", "float"),
            new Column("readLatency", "float"),
            new Column("minReadOps", "float"),
            new Column("maxReadOps", "float"),
            new Column("readOps", "float"),
            new Column("minWriteData", "float"),
            new Column("maxWriteData", "float"),
            new Column("writeData", "float"),
            new Column("minWriteLatency", "float"),
            new Column("maxWriteLatency", "float"),
            new Column("writeLatency", "float"),
            new Column("minWriteOps", "float"),
            new Column("maxWriteOps", "float"),
            new Column("writeOps", "float"),
            new Column("minCacheMissRate", "float"),
            new Column("maxCacheMissRate", "float"),
            new Column("cacheMissRate", "float"),
            new Column("cacheMissRateBase", "bigint(20)"),
            new Column("minIOPSPerTB", "float"),
            new Column("maxIOPSPerTB", "float"),
            new Column("opsPerTB", "float")};


    public static final Column[] SUMMARY_COLUMNS = new Column[] {
        new Column("latency", "float"),
        new Column("ops", "float"),
        new Column("totalData", "float"),
        new Column("readData", "float"),
        new Column("readLatency", "float"),
        new Column("readOps", "float"),
        new Column("writeData", "float"),
        new Column("writeLatency", "float"),
        new Column("writeOps", "float"),
        new Column("cacheMissRate", "float"),
        new Column("cacheMissRateBase", "bigint(20)"),
        new Column("logicalSize", "bigint(20)"),
        new Column("opsPerTB", "float"),
        new Column("maxIOPS", "float"),
        new Column("maxMBPS", "float"),
        new Column("minIOPS", "float"),
        new Column("maxIOPSPerTB", "float"),
        new Column("minIOPSPerTB", "float")};


public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][] {
        {new RatioMethod("ops")}, //latency
        {SqlMethod.AVG}, // ops
        {SqlMethod.AVG}, // totalData
        {SqlMethod.AVG}, // readData
        {new RatioMethod("readOps")}, // readLatency
        {SqlMethod.AVG}, // readOps
        {SqlMethod.AVG}, // writeData
        {new RatioMethod("writeOps")}, // writeLatency
        {SqlMethod.AVG}, // writeOps
        {new RatioMethod("cacheMissRateBase")}, // cacheMissRate
        {SqlMethod.SUM}, // cacheMissRateBase
        {SqlMethod.AVG}, //logicalSize
        {SqlMethod.AVG}, // opsPerTB
        {SqlMethod.AVG}, // maxIOPS
        {SqlMethod.AVG}, // maxMBPS
        {SqlMethod.AVG}, // minIOPS
        {SqlMethod.AVG}, // maxIOPSPerTB
        {SqlMethod.AVG}, // minIOPSPerTB
};

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[] {
        new RatioMethod("ops"), //latency
        SqlMethod.AVG, // ops
        SqlMethod.AVG, // totalData
        SqlMethod.AVG, // readData
        new RatioMethod("readOps"), // readLatency
        SqlMethod.AVG, // readOps
        SqlMethod.AVG, // writeData
        new RatioMethod("writeOps"), // writeLatency
        SqlMethod.AVG, // writeOps
        new RatioMethod("cacheMissRateBase"), // cacheMissRate
        SqlMethod.SUM, // cacheMissRateBase
        SqlMethod.AVG, //logicalSize
        SqlMethod.AVG, // opsPerTB
        SqlMethod.AVG, // maxIOPS
        SqlMethod.AVG, // maxMBPS
        SqlMethod.AVG, //minIOPS
        SqlMethod.AVG, // maxIOPSPerTB
        SqlMethod.AVG, // minIOPSPerTB
    };

    public static final SqlMethod[] DAILY_SUMMARY_METHODS = new SqlMethod[]{
            SqlMethod.M_MIN,//minLatency
            SqlMethod.M_MAX,//maxLatency
            new WmaBaseRated("ops"),//latency
            SqlMethod.M_MIN,//minOps
            SqlMethod.M_MAX,//maxOps
            SqlMethod.CMA,//ops
            SqlMethod.M_MIN,//minTotalData
            SqlMethod.M_MAX,//maxTotalData
            SqlMethod.CMA,//totalData
            SqlMethod.M_MIN,//minReadData
            SqlMethod.M_MAX,//maxReadData
            SqlMethod.CMA,//readData
            SqlMethod.M_MIN,//minReadLatency
            SqlMethod.M_MAX,//maxReadLatency
            new WmaBaseRated("readOps"),//readLatency
            SqlMethod.M_MIN,//minReadOps
            SqlMethod.M_MAX,//maxReadOps
            SqlMethod.CMA,//readOps
            SqlMethod.M_MIN,//minWriteData
            SqlMethod.M_MAX,//maxWriteData
            SqlMethod.CMA,//writeData
            SqlMethod.M_MIN,//minWriteLatency
            SqlMethod.M_MAX,//maxWriteLatency
            new WmaBaseRated("writeOps"),//writeLatency
            SqlMethod.M_MIN,//minWriteOps
            SqlMethod.M_MAX,//maxWriteOps
            SqlMethod.CMA,//writeOps
            SqlMethod.M_MIN,//minCacheMissRate
            SqlMethod.M_MAX,//maxCacheMissRate
            new WmaBaseNotRated("cacheMissRateBase"),//cacheMissRate
            SqlMethod.C_SUM,//cacheMissRateBase
            SqlMethod.M_MIN,//minOpsPerTB
            SqlMethod.M_MAX,//maxOpsPerTB
            SqlMethod.CMA,//opsPerTB
    };


    private static ObjectParser<QosVolumeWorkloadMeasurements> sampleParser;
    private static ObjectParser<QosVolumeWorkloadMeasurements> dailySummaryParser;

    static {
        sampleParser = new ObjectParser<QosVolumeWorkloadMeasurements>() {
            @Override
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement,
                    QosVolumeWorkloadMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCacheMissRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCacheMissRateBase());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLogicalSize());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpsPerTB());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMaxIOPS());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMaxMBPS());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMinIOPS());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMaxIOPSPerTB());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getMinIOPSPerTB());
            }

            @Override
            public QosVolumeWorkloadMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new QosVolumeWorkloadMeasurements()
                        .setLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setTotalData(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setReadData(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setReadLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setReadOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setWriteData(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setWriteLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setWriteOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setCacheMissRate(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setCacheMissRateBase(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setLogicalSize(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setOpsPerTB(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setMaxIOPS(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setMaxMBPS(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setMinIOPS(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setMaxIOPSPerTB(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setMinIOPSPerTB(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };

        dailySummaryParser = new ObjectParser<QosVolumeWorkloadMeasurements>() {
            @Override
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement,
                    QosVolumeWorkloadMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCacheMissRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCacheMissRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCacheMissRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCacheMissRateBase());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpsPerTB());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpsPerTB());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpsPerTB());
            }

            @Override
            public QosVolumeWorkloadMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                throw new UnsupportedOperationException("Reading of daily summary is not supported");
            }
        };
    }

    public static ObjectParser<QosVolumeWorkloadMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<QosVolumeWorkloadMeasurements> getSampleParser() {
        return sampleParser;
    }

    public static ObjectParser<QosVolumeWorkloadMeasurements> getDailySummaryParser() {
        return dailySummaryParser;
    }
}
