/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.oci.netapp.performance.qosworkload.interfaces.data;

import java.io.Serializable;

/**
 * This class holds common counters for QosLunWorkloadMeasuremnts and QosFileWorkloadMeasurements
 */

public class QosLunFileWorkloadMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * Average latency in microseconds for the WFL Filesystem to process all the operations on the LUN/File; not including
     * request processing or network communication
     */
    protected float latency;

    /**
     * Number of operations per second serviced by the lun/file
     */
    protected float ops;

    /**
     * Total bytes per second to and from the lun/file
     */
    protected float totalData;

    /**
     * Bytes read per second from the lun/file
     */
    protected float readData;

    /**
     * Average latency in microseconds for the WAFL filesystem to process read request to the lun/file; not including request
     * processing or network communication
     */
    protected float readLatency;

    /**
     * Number of reads per second to the lun/file
     */
    protected float readOps;

    /**
     * Bytes written per second to the lun/file
     */
    protected float writeData;

    /**
     * Average latency in microseconds for the WAFL filesystem to process write request to the lun/file; not including request
     * processing or network communication
     */
    protected float writeLatency;

    /**
     * Number of writes per second to the lun/file
     */
    protected float writeOps;

    /**
     * This is lun/file logical size in bytes
     */
    protected long logicalSize;

    /**
     * This is generation number associated with the logical_block_count counter. The algorithm is:
     * update logicalSize only when logical_block_cp_count is higher than previous value
     */
    protected long logicalBlockCpCount;

    /**
     * Number of operations per second per TB serviced by the lun/file
     */
    protected float opsPerTB;

    /**
     * Maximum throughput in terms of IOPS defined by qosPolicyGroup used by this lun/file workload
     */
    protected float maxIOPS;

    /**
     * Maximum throughput in terms of MBPS defined by qosPolicyGroup used by this lun/file workload
     */
    protected float maxMBPS;

    /**
     * Minimum throughput in terms of IOPS defined by qosPolicyGroup used by this lun/file workload
     */
    protected float minIOPS;

    /**
     * Number of maxIOPS per TB serviced by the lun/file
     */
    protected float maxIOPSPerTB;

    /**
     * Number of minIOPS per TB serviced by the lun/file
     */
    protected float minIOPSPerTB;

}
