/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.qosworkload.interfaces.data;

import java.io.Serializable;

@SuppressWarnings("UnusedDeclaration")
public class QosVolumeWorkloadMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * Average latency in microseconds for the WFL Filesystem to process all the operations on the volume; not including
     * request processing or network communication
     */
    private float latency;

    /**
     * Number of operations per second serviced by the volume
     */
    private float ops;

    /**
     * Total bytes per second to and from the volume
     */
    private float totalData;

    /**
     * Bytes read per second from the volume
     */
    private float readData;

    /**
     * Average latency in microseconds for the WAFL filesystem to process read request to the volume; not including request
     * processing or network communication
     */
    private float readLatency;

    /**
     * Number of reads per second to the volume
     */
    private float readOps;

    /**
     * Bytes written per second to the volume
     */
    private float writeData;

    /**
     * Average latency in microseconds for the WAFL filesystem to process write request to the volume; not including request
     * processing or network communication
     */
    private float writeLatency;

    /**
     * Number of writes per second to the volume
     */
    private float writeOps;
    
    /**
     * This is the percentage of cache misses. The sum of read_io_type DISK and HYA_HDD
     */
    private float cacheMissRate;

    /**
     * This is the total number of WAFL read requests. It is used as a base counter (or denominator) for the
     * cache_miss_rate counter. This counter is exactly similar to the read_io_type_base counter from the workload
     * object.
     */
    private long cacheMissRateBase;

    /**
     * This is the volume's logicalSize= sized_used + compression_space_saved + deduplication_space_shared. It is used for calculate opsPerTB by ops/logicalSize
     */
    private long logicalSize;

    /**
     * Number of operations per second per TB serviced by the volume
     */
    private float opsPerTB;

     /**
     * Maximum throughput in terms of IOPS defined by qosPolicyGroup used by this volume workload
     */
    private float maxIOPS;

    /**
     * Maximum throughput in terms of MBPS defined by qosPolicyGroup used by this volume workload
     */
    private float maxMBPS;

    /**
     * Minimum throughput in terms of IOPS defined by qosPolicyGroup used by this volume workload
     */
    private float minIOPS;

    /**
     * Number of maxIOPS per TB serviced by the volume
     */
    private float maxIOPSPerTB;

    /**
     * Number of minIOPS per TB serviced by the volume
     */
    private float minIOPSPerTB;



    public float getLatency() {
        return latency;
    }

    public QosVolumeWorkloadMeasurements setLatency(float latency) {
        this.latency = latency;
        return this;
    }

    public float getOps() {
        return ops;
    }

    public QosVolumeWorkloadMeasurements setOps(float ops) {
        this.ops = ops;
        return this;
    }

    public float getTotalData() {
        return totalData;
    }

    public QosVolumeWorkloadMeasurements setTotalData(float totalData) { 
        this.totalData = totalData;
        return this;
    }

    public float getReadData() {
        return readData;
    }

    public QosVolumeWorkloadMeasurements setReadData(float readData) {
        this.readData = readData;
        return this;
    }

    public float getReadLatency() {
        return readLatency;
    }

    public QosVolumeWorkloadMeasurements setReadLatency(float readLatency) {
        this.readLatency = readLatency;
        return this;
    }

    public float getReadOps() {
        return readOps;
    }

    public QosVolumeWorkloadMeasurements setReadOps(float readOps) {
        this.readOps = readOps;
        return this;
    }

    public float getWriteData() {
        return writeData;
    }

    public QosVolumeWorkloadMeasurements setWriteData(float writeData) {
        this.writeData = writeData;
		return this;
    }

    public float getWriteLatency() {
        return writeLatency;
    }

    public QosVolumeWorkloadMeasurements setWriteLatency(float writeLatency) {
        this.writeLatency = writeLatency;
		return this;
    }

    public float getWriteOps() {
        return writeOps;
    }

    public QosVolumeWorkloadMeasurements setWriteOps(float writeOps) {
        this.writeOps = writeOps;
		return this;
    }

    public float getCacheMissRate(){
	   return cacheMissRate;
    }
    
    public QosVolumeWorkloadMeasurements setCacheMissRate(float cacheMissRate){
    	this.cacheMissRate =  cacheMissRate;
    	return this;
    }

    public long getCacheMissRateBase() {
        return cacheMissRateBase;
    }

    public QosVolumeWorkloadMeasurements setCacheMissRateBase(long cacheMissRateBase) {
        this.cacheMissRateBase = cacheMissRateBase;
        return this;
    }

    public long getLogicalSize() {
        return logicalSize;
    }

    public QosVolumeWorkloadMeasurements setLogicalSize(long logicalSize) {
        this.logicalSize = logicalSize;
        return this;
    }

    public float getOpsPerTB() {
        return opsPerTB;

    }

    public QosVolumeWorkloadMeasurements setOpsPerTB(float opsPerTB) {
        this.opsPerTB = opsPerTB;
        return this;
    }

    public float getMaxIOPS() {
        return maxIOPS;
    }

    public QosVolumeWorkloadMeasurements setMaxIOPS(float maxIOPS) {
        this.maxIOPS = maxIOPS;
        return this;
    }

    public float getMinIOPS() {
        return minIOPS;
    }

    public QosVolumeWorkloadMeasurements setMinIOPS(float minIOPS) {
        this.minIOPS = minIOPS;
        return this;
    }

    public float getMaxMBPS() {
        return maxMBPS;
    }

    public QosVolumeWorkloadMeasurements setMaxMBPS(float maxMBPS) {
        this.maxMBPS = maxMBPS;
        return this;
    }

    public float getMaxIOPSPerTB() {
        return maxIOPSPerTB;
    }

    public QosVolumeWorkloadMeasurements setMaxIOPSPerTB(float maxIOPSPerTB) {
        this.maxIOPSPerTB = maxIOPSPerTB;
        return this;
    }

    public float getMinIOPSPerTB() {
        return minIOPSPerTB;
    }

    public QosVolumeWorkloadMeasurements setMinIOPSPerTB(float minIOPSPerTB) {
        this.minIOPSPerTB = minIOPSPerTB;
        return this;
    }


    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        QosVolumeWorkloadMeasurements that = (QosVolumeWorkloadMeasurements) o;

        if (Float.compare(that.cacheMissRate, cacheMissRate) != 0) return false;
        if (cacheMissRateBase != that.cacheMissRateBase) return false;
        if (Float.compare(that.latency, latency) != 0) return false;
        if (Float.compare(that.ops, ops) != 0) return false;
        if (Float.compare(that.readData, readData) != 0) return false;
        if (Float.compare(that.readLatency, readLatency) != 0) return false;
        if (Float.compare(that.readOps, readOps) != 0) return false;
        if (Float.compare(that.totalData, totalData) != 0) return false;
        if (Float.compare(that.writeData, writeData) != 0) return false;
        if (Float.compare(that.writeLatency, writeLatency) != 0) return false;
        if (Float.compare(that.writeOps, writeOps) != 0) return false;
        if (Float.compare(that.opsPerTB, opsPerTB) !=0) return false;
        if (logicalSize != that.logicalSize) return false;
        if (Float.compare(that.maxIOPS, maxIOPS) !=0) return false;
        if (Float.compare(that.maxMBPS, maxMBPS) !=0) return false;
        if (Float.compare(that.minIOPS, minIOPS) !=0) return false;
        if (Float.compare(that.maxIOPSPerTB, maxIOPSPerTB) !=0) return false;
        if (Float.compare(that.minIOPSPerTB, minIOPSPerTB) !=0) return false;
        return true;
    }

    @Override
    public int hashCode() {
        int result = (latency != +0.0f ? Float.floatToIntBits(latency) : 0);
        result = 31 * result + (ops != +0.0f ? Float.floatToIntBits(ops) : 0);
        result = 31 * result + (totalData != +0.0f ? Float.floatToIntBits(totalData) : 0);
        result = 31 * result + (readData != +0.0f ? Float.floatToIntBits(readData) : 0);
        result = 31 * result + (readLatency != +0.0f ? Float.floatToIntBits(readLatency) : 0);
        result = 31 * result + (readOps != +0.0f ? Float.floatToIntBits(readOps) : 0);
        result = 31 * result + (writeData != +0.0f ? Float.floatToIntBits(writeData) : 0);
        result = 31 * result + (writeLatency != +0.0f ? Float.floatToIntBits(writeLatency) : 0);
        result = 31 * result + (writeOps != +0.0f ? Float.floatToIntBits(writeOps) : 0);
        result = 31 * result + (cacheMissRate != +0.0f ? Float.floatToIntBits(cacheMissRate) : 0);
        result = 31 * result + (int) (cacheMissRateBase ^ (cacheMissRateBase >>> 32));
        result = 31 * result + (opsPerTB != +0.0f ? Float.floatToIntBits(opsPerTB) : 0);
        result = 31 * result + (int) (logicalSize ^ (logicalSize>>> 32));
        result = 31 * result + (maxIOPS != +0.0f ? Float.floatToIntBits(maxIOPS) : 0);
        result = 31 * result + (maxMBPS != +0.0f ? Float.floatToIntBits(maxMBPS) : 0);
        result = 31 * result + (minIOPS != +0.0f ? Float.floatToIntBits(minIOPS) : 0);
        result = 31 * result + (maxIOPSPerTB != +0.0f ? Float.floatToIntBits(maxIOPSPerTB) : 0);
        result = 31 * result + (minIOPSPerTB != +0.0f ? Float.floatToIntBits(minIOPSPerTB) : 0);
        return result;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("QosVolumeWorkloadMeasurements{");
        sb.append("latency=").append(latency);
        sb.append(", ops=").append(ops);
        sb.append(", totalData=").append(totalData);
        sb.append(", readData=").append(readData);
        sb.append(", readLatency=").append(readLatency);
        sb.append(", readOps=").append(readOps);
        sb.append(", writeData=").append(writeData);
        sb.append(", writeLatency=").append(writeLatency);
        sb.append(", writeOps=").append(writeOps);
        sb.append(", cacheMissRate=").append(cacheMissRate);
        sb.append(", cacheMissRateBase=").append(cacheMissRateBase);
        sb.append(", logicalSize=").append(logicalSize);
        sb.append(", opsPerTB=").append(opsPerTB);
        sb.append(", maxIOPS=").append(maxIOPS);
        sb.append(", maxMBPS=").append(maxMBPS);
        sb.append(", minIOPS=").append(minIOPS);
        sb.append(", maxIOPSPerTB=").append(maxIOPSPerTB);
        sb.append(", minIOPSPerTB=").append(minIOPSPerTB);
        sb.append('}');
        return sb.toString();
    }
}
