/*
 * $Id$
 *
 * Copyright (c) 2015 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.oci.netapp.performance.qosworkload.interfaces.remote;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.Remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosLunFileWorkloadMeasurements;
import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosLunWorkloadMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

@Remote
public interface QosLunWorkloadPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//QosLunWorkloadPerformanceQueryBean!com.netapp.oci.netapp.performance.qosworkload.interfaces.remote.QosLunWorkloadPerformanceQueryRemote";

    /**
     * Gets QoS lun workload samples for period of time
     *
     * @param qosWorkloadIds set of QoS workload ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of QoS workload id to list ordered by time of QoS workload samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<QosLunWorkloadMeasurements>>> getSamples(Set<Long> qosWorkloadIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * Gets QoS lun workload samples for period of time
     *
     * @param containerId container id
     * @param qosWorkloadIds set of QoS lun workload ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of QoS lun workload id to list ordered by time of QoS lun workload samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<QosLunWorkloadMeasurements>>> getSamples(Long containerId, Set<Long> qosWorkloadIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS lun workload samples for period of time
     *
     * @param containerId container id
     * @param qosWorkloadId Qos lun workload id
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return list ordered by time of QoS lun workload samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<QosLunWorkloadMeasurements>> getSamples(Long containerId, Long qosWorkloadId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given qoslun id's (qoslunIds). The map returned will contain only qoslunWorkload ids that has a sample.
     * qoslunWorkload ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the qoslunWorkload ids passed.
     *
     * @param clusterId cluster id
     * @param qosWorkloadIds set of qoslunWorkload ids
     * @return map of qoslunWorkload id to latest sample SampleInfo<QoslunWorkloadMeasurements>. qoslunWorkload ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<QosLunWorkloadMeasurements>> getLatestSample(Long clusterId, Set<Long> qosWorkloadIds) throws NetappInterfaceException;

    /**
     * Gets QoS lun workload summaries for QoS lun workload ids
     *
     * @param qosWorkloadIds set of QoS lun workload ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of QoS lun workload id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<QosLunWorkloadMeasurements>>> getSummaries(Set<Long> qosWorkloadIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS lun workload summaries for QoS lun workload ids
     *
     * @param clusterId cluster id
     * @param qosWorkloadIds set of QoS lun workload ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of QoS lun workload id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<QosLunWorkloadMeasurements>>> getSummaries(Long clusterId, Set<Long> qosWorkloadIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     *
     * @param clusterId
     * @param qosWorkloadId
     * @param time
     * @return sample of QosLunFileWorkloadMeasurements
     * @throws NetappInterfaceException
     */
    SampleInfo<QosLunWorkloadMeasurements> getFirstSampleAfterTime(Long clusterId, Long qosWorkloadId, long time) throws NetappInterfaceException;

    /**
     *
     * @param clusterId
     * @param qosWorkloadId
     * @param time
     * @return sample of QosLunFileWorkloadMeasurements
     * @throws NetappInterfaceException
     */
    SampleInfo<QosLunWorkloadMeasurements> getLastSampleBeforeTime(Long clusterId, Long qosWorkloadId, long time) throws NetappInterfaceException;

    /**
     *
     * @param clusterId
     * @param qosWorkloadId
     * @param time
     * @return summary of QosLunFileWorkloadMeasurements
     * @throws NetappInterfaceException
     */
    Summary<QosLunWorkloadMeasurements> getFirstSummaryAfterTime(Long clusterId, Long qosWorkloadId, long time) throws NetappInterfaceException;

    /**
     *
     * @param clusterId
     * @param qosWorkloadId
     * @param time
     * @return summary of QosLunFileWorkloadMeasurements
     * @throws NetappInterfaceException
     */
    Summary<QosLunWorkloadMeasurements>  getLastSummaryBeforeTime(Long clusterId, Long qosWorkloadId, long time) throws NetappInterfaceException;


}
