/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.qosworkload.interfaces.remote;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.Remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosVolumeWorkloadMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

@Remote
public interface QosVolumeWorkloadPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//QosVolumeWorkloadPerformanceQueryBean!com.netapp.oci.netapp.performance.qosworkload.interfaces.remote.QosVolumeWorkloadPerformanceQueryRemote";

    /**
     * Gets QoS volume workload samples for period of time
     *
     * @param qosVolumeWorkloadIds set of QoS volume workload ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of QoS volume workload id to list ordered by time of QoS volume workload samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<QosVolumeWorkloadMeasurements>>> getSamples(Set<Long> qosVolumeWorkloadIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * Gets QoS volume workload samples for period of time
     *
     * @param containerId container id
     * @param qosVolumeWorkloadIds set of QoS volume workload ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of QoS volume workload id to list ordered by time of QoS volume workload samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<QosVolumeWorkloadMeasurements>>> getSamples(Long containerId, Set<Long> qosVolumeWorkloadIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS volume workload samples for period of time
     *
     * @param containerId container id
     * @param qosVolumeWorkloadId Qos Volume workload id
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return list ordered by time of QoS volume workload samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<QosVolumeWorkloadMeasurements>> getSamples(Long containerId, Long qosVolumeWorkloadId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given qosVolume id's (qosVolumeIds). The map returned will contain only qosVolumeWorkload ids that has a sample.
     * qosVolumeWorkload ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the qosVolumeWorkload ids passed.
     *
     * @param clusterId cluster id
     * @param qosVolumeWorkloadIds set of qosVolumeWorkload ids
     * @return map of qosVolumeWorkload id to latest sample SampleInfo<QosVolumeWorkloadMeasurements>. qosVolumeWorkload ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<QosVolumeWorkloadMeasurements>> getLatestSample(Long clusterId, Set<Long> qosVolumeWorkloadIds) throws NetappInterfaceException;

    /**
     * Gets QoS volume workload summaries for QoS volume workload ids
     *
     * @param qosVolumeWorkloadIds set of QoS volume workload ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of QoS volume workload id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<QosVolumeWorkloadMeasurements>>> getSummaries(Set<Long> qosVolumeWorkloadIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS volume workload summaries for QoS volume workload ids
     *
     * @param clusterId cluster id
     * @param qosVolumeWorkloadIds set of QoS volume workload ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of QoS volume workload id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<QosVolumeWorkloadMeasurements>>> getSummaries(Long clusterId, Set<Long> qosVolumeWorkloadIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     *
     * @param clusterId
     * @param qosVolumeWorkloadId
     * @param time
     * @return sample of QosVolumeWorkloadMeasurements
     * @throws NetappInterfaceException
     */
    SampleInfo<QosVolumeWorkloadMeasurements> getFirstSampleAfterTime(Long clusterId, Long qosVolumeWorkloadId, long time) throws NetappInterfaceException;

    /**
     *
     * @param clusterId
     * @param qosVolumeWorkloadId
     * @param time
     * @return sample of QosVolumeWorkloadMeasurements
     * @throws NetappInterfaceException
     */
    SampleInfo<QosVolumeWorkloadMeasurements> getLastSampleBeforeTime(Long clusterId, Long qosVolumeWorkloadId, long time) throws NetappInterfaceException;

    /**
     *
     * @param clusterId
     * @param qosVolumeWorkloadId
     * @param time
     * @return summary of QosVolumeWorkloadMeasurements
     * @throws NetappInterfaceException
     */
    Summary<QosVolumeWorkloadMeasurements> getFirstSummaryAfterTime(Long clusterId, Long qosVolumeWorkloadId, long time) throws NetappInterfaceException;

    /**
     *
     * @param clusterId
     * @param qosVolumeWorkloadId
     * @param time
     * @return summary of QosVolumeWorkloadMeasurements
     * @throws NetappInterfaceException
     */
    Summary<QosVolumeWorkloadMeasurements>  getLastSummaryBeforeTime(Long clusterId, Long qosVolumeWorkloadId, long time) throws NetappInterfaceException;


}
