package com.netapp.oci.netapp.performance.qosworkloadconstituent.interfaces.data;

import java.io.Serializable;

/**
 * Measurement class for workload:constituent counters
 *
 */
public class QosWorkloadConstituentMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * The number of dirty buffers that were accumulated by the workload constituent
     */
    private long dirtyBuffers;
    /**
     * 
     * Average latency in microseconds for the WFL Filesystem to process all the operations on the volume; not including
     * request processing or network communication
     */
    private float latency;
    /**
     * The number of free blocks that were accumulated by the workload constituent
     */
    private long freedBlocks;

    /**
     * The number of cross-stripe read ahead messages that were issued on behalf of the workload constituent.
     */
    private long readAheadXStripe;

    /**
     * Number of operations per second serviced by the volume
     */
    private float ops;

    /**
     * Bytes read per second from the volume
     */
    private float readData;

    /**
     * Average latency in microseconds for the WAFL filesystem to process read request to the volume; not including request
     * processing or network communication
     */
    private float readLatency;

    /**
     * Number of reads per second to the volume
     */
    private float readOps;

    /**
     * Bytes written per second to the volume
     */
    private float writeData;

    /**
     * Average latency in microseconds for the WAFL filesystem to process write request to the volume; not including request
     * processing or network communication
     */
    private float writeLatency;

    /**
     * Number of writes per second to the volume
     */
    private float writeOps;
    
    /**
     * Total data read and written per second by the workload
     */
    private float totalData;

    /**
     * Average latency in microseconds where otherLatency = latency - (readLatency + writeLatency)
     */
    private float otherLatency;
    
    /**
     * The read_io_type_base
     */
    private long read_io_type_base;
    
    /**
     * Sum of Disk and Hya HDD counters in the read_io_type array
     */
    private long sumDiskAndHyaHDD;
    
    /**
     * Collection period in milliseconds.
     */
    private long collectionPeriod;

    public float getLatency() {
        return latency;
    }

    public QosWorkloadConstituentMeasurements setLatency(float latency) {
        this.latency = latency;
        return this;
    }

    public long getDirtyBuffers() {
        return dirtyBuffers;
    }

    public QosWorkloadConstituentMeasurements setDirtyBuffers(long dirtyBuffers) {
        this.dirtyBuffers = dirtyBuffers;
        return this;
    }

    public long getFreedBlocks() {
        return freedBlocks;
    }

    public QosWorkloadConstituentMeasurements setFreedBlocks(long freedBlocks) {
        this.freedBlocks = freedBlocks;
        return this;
    }

     public long getReadAheadXStripe() {
        return readAheadXStripe;
    }

    public QosWorkloadConstituentMeasurements setReadAheadXStripe(long readAheadXStripe) {
        this.readAheadXStripe = readAheadXStripe;
        return this;
    }

    public float getOps() {
        return ops;
    }

    public QosWorkloadConstituentMeasurements setOps(float ops) {
        this.ops = ops;
        return this;
    }

    public float getReadData() {
        return readData;
    }

    public QosWorkloadConstituentMeasurements setReadData(float readData) {
        this.readData = readData;
        return this;
    }

    public float getReadLatency() {
        return readLatency;
    }

    public QosWorkloadConstituentMeasurements setReadLatency(float readLatency) {
        this.readLatency = readLatency;
        return this;
    }

    public float getTotalData() {
        return totalData;
    }

    public QosWorkloadConstituentMeasurements setTotalData(float totalData) {
        this.totalData = totalData;
        return this;
    }

    public float getReadOps() {
        return readOps;
    }

    public QosWorkloadConstituentMeasurements setReadOps(float readOps) {
        this.readOps = readOps;
        return this;
    }

    public float getWriteData() {
        return writeData;
    }

    public QosWorkloadConstituentMeasurements setWriteData(float writeData) {
        this.writeData = writeData;
        return this;
    }

    public float getWriteLatency() {
        return writeLatency;
    }

    public QosWorkloadConstituentMeasurements setWriteLatency(float writeLatency) {
        this.writeLatency = writeLatency;
        return this;
    }

    public float getWriteOps() {
        return writeOps;
    }

    public QosWorkloadConstituentMeasurements setWriteOps(float writeOps) {
        this.writeOps = writeOps;
        return this;
    }

    public float getOtherLatency() {
        return otherLatency;
    }

    public QosWorkloadConstituentMeasurements setOtherLatency(float otherLatency) {
        this.otherLatency = otherLatency;
        return this;
    }
    
    public long getReadIOTypeBase(){
    	return read_io_type_base;
    }
    
    public QosWorkloadConstituentMeasurements setReadIOTypeBase(long read_io_type_base){
    	this.read_io_type_base = read_io_type_base;
    	return this;
    }
    
    public long getSumDiskAndHyaHDD(){
    	return sumDiskAndHyaHDD;
    }
    
    public QosWorkloadConstituentMeasurements setSumDiskAndHyaHDD(long sumDiskAndHyaHDD){
    	this.sumDiskAndHyaHDD = sumDiskAndHyaHDD;
    	return this;
    }

    public long getCollectionPeriod() {
        return collectionPeriod;
    }

    public QosWorkloadConstituentMeasurements setCollectionPeriod(long collectionPeriod) {
        this.collectionPeriod = collectionPeriod;
        return this;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        QosWorkloadConstituentMeasurements that = (QosWorkloadConstituentMeasurements) o;

        if (collectionPeriod != that.collectionPeriod) return false;
        if (dirtyBuffers != that.dirtyBuffers) return false;
        if (freedBlocks != that.freedBlocks) return false;
        if (Float.compare(that.latency, latency) != 0) return false;
        if (Float.compare(that.ops, ops) != 0) return false;
        if (Float.compare(that.otherLatency, otherLatency) != 0) return false;
        if (readAheadXStripe != that.readAheadXStripe) return false;
        if (Float.compare(that.readData, readData) != 0) return false;
        if (Float.compare(that.readLatency, readLatency) != 0) return false;
        if (Float.compare(that.readOps, readOps) != 0) return false;
        if (read_io_type_base != that.read_io_type_base) return false;
        if (sumDiskAndHyaHDD != that.sumDiskAndHyaHDD) return false;
        if (Float.compare(that.totalData, totalData) != 0) return false;
        if (Float.compare(that.writeData, writeData) != 0) return false;
        if (Float.compare(that.writeLatency, writeLatency) != 0) return false;
        if (Float.compare(that.writeOps, writeOps) != 0) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = (int) (dirtyBuffers ^ (dirtyBuffers >>> 32));
        result = 31 * result + (latency != +0.0f ? Float.floatToIntBits(latency) : 0);
        result = 31 * result + (int) (freedBlocks ^ (freedBlocks >>> 32));
        result = 31 * result + (int) (readAheadXStripe ^ (readAheadXStripe >>> 32));
        result = 31 * result + (ops != +0.0f ? Float.floatToIntBits(ops) : 0);
        result = 31 * result + (readData != +0.0f ? Float.floatToIntBits(readData) : 0);
        result = 31 * result + (readLatency != +0.0f ? Float.floatToIntBits(readLatency) : 0);
        result = 31 * result + (readOps != +0.0f ? Float.floatToIntBits(readOps) : 0);
        result = 31 * result + (writeData != +0.0f ? Float.floatToIntBits(writeData) : 0);
        result = 31 * result + (writeLatency != +0.0f ? Float.floatToIntBits(writeLatency) : 0);
        result = 31 * result + (writeOps != +0.0f ? Float.floatToIntBits(writeOps) : 0);
        result = 31 * result + (totalData != +0.0f ? Float.floatToIntBits(totalData) : 0);
        result = 31 * result + (otherLatency != +0.0f ? Float.floatToIntBits(otherLatency) : 0);
        result = 31 * result + (int) (read_io_type_base ^ (read_io_type_base >>> 32));
        result = 31 * result + (int) (sumDiskAndHyaHDD ^ (sumDiskAndHyaHDD >>> 32));
        result = 31 * result + (int) (collectionPeriod ^ (collectionPeriod >>> 32));
        return result;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("QosWorkloadConstituentMeasurements{");
        sb.append("dirtyBuffers=").append(dirtyBuffers);
        sb.append(", latency=").append(latency);
        sb.append(", freedBlocks=").append(freedBlocks);
        sb.append(", readAheadXStripe=").append(readAheadXStripe);
        sb.append(", ops=").append(ops);
        sb.append(", readData=").append(readData);
        sb.append(", readLatency=").append(readLatency);
        sb.append(", readOps=").append(readOps);
        sb.append(", writeData=").append(writeData);
        sb.append(", writeLatency=").append(writeLatency);
        sb.append(", writeOps=").append(writeOps);
        sb.append(", totalData=").append(totalData);
        sb.append(", otherLatency=").append(otherLatency);
        sb.append(", read_io_type_base=").append(read_io_type_base);
        sb.append(", sumDiskAndHyaHDD=").append(sumDiskAndHyaHDD);
        sb.append(", collectionPeriod=").append(collectionPeriod);
        sb.append('}');
        return sb.toString();
    }
}
