/*
 * Copyright (c) 2014 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.qosworkloaddblade;

import java.sql.ResultSet;
import java.sql.SQLException;

import com.netapp.oci.netapp.performance.qosworkloaddblade.interfaces.data.QosWorkloadQueueDBladeMeasurements;
import com.netapp.oci.netapp.performance.qosworkloaddblade.interfaces.data.QosWorkloadQueueDBladeSummary;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

/**
 * Meta data class for handling persistence for {@link QosWorkloadQueueDBladeMeasurements} and
 * {@link QosWorkloadQueueDBladeSummary}
 */
public class QosWorkloadQueueDBladeMetaData {
    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("cpuDBladeResidenceTime", "bigint(20)"),
            new Column("cpuDBladeUtilServiceTime", "bigint(20)"),
            new Column("cpuDBladeBackgroundServiceTime", "bigint(20)"),
            new Column("cpuDBladeBackgroundWAFLCPServiceTime", "bigint(20)"),
            new Column("cpuDBladeBackgroundBackgroundDeleteServiceTime", "bigint(20)"),
            new Column("cpuDBladeBackgroundRAStripeXServiceTime", "bigint(20)"),
            new Column("diskHDDBackgroundServiceTime", "bigint(20)"),
            new Column("diskHDDBackgroundWAFLCPServiceTime", "bigint(20)"),
            new Column("diskHDDBackgroundBackgroundDeleteServiceTime", "bigint(20)"),
            new Column("diskHDDBackgroundRAStripeXServiceTime", "bigint(20)"),
            new Column("diskSSDBackgroundServiceTime", "bigint(20)"),
            new Column("diskSSDBackgroundWAFLCPServiceTime", "bigint(20)"),
            new Column("diskSSDBackgroundBackgroundDeleteServiceTime", "bigint(20)"),
            new Column("diskSSDBackgroundRAStripeXServiceTime", "bigint(20)"),
            new Column("delayWAFLSuspCPWaitTime", "bigint(20)"),
            new Column("delayDiskIOWaitTime", "bigint(20)"),
            new Column("delayCloudIOWaitTime", "bigint(20)"),
            new Column("delayNvlogTransferWaitTime", "bigint(20)"),
            new Column("cacheMissRate", "float"),
            new Column("collectionPeriod", "bigint(20)"),
            new Column("cacheMissRateBase", "bigint(20)"),
            new Column("delayFlexcacheRalWaitTime", "bigint(20)"),
            new Column("delayFlexcacheSpinhiWaitTime", "bigint(20)"),
            new Column("delaySyncReplWaitTime", "bigint(20)")};

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("cpuDBladeResidenceTime", "bigint(20)"),
            new Column("cpuDBladeUtilServiceTime", "bigint(20)"),
            new Column("cpuDBladeBackgroundServiceTime", "bigint(20)"),
            new Column("cpuDBladeBackgroundWAFLCPServiceTime", "bigint(20)"),
            new Column("cpuDBladeBackgroundBackgroundDeleteServiceTime", "bigint(20)"),
            new Column("cpuDBladeBackgroundRAStripeXServiceTime", "bigint(20)"),
            new Column("diskHDDBackgroundServiceTime", "bigint(20)"),
            new Column("diskHDDBackgroundWAFLCPServiceTime", "bigint(20)"),
            new Column("diskHDDBackgroundBackgroundDeleteServiceTime", "bigint(20)"),
            new Column("diskHDDBackgroundRAStripeXServiceTime", "bigint(20)"),
            new Column("diskSSDBackgroundServiceTime", "bigint(20)"),
            new Column("diskSSDBackgroundWAFLCPServiceTime", "bigint(20)"),
            new Column("diskSSDBackgroundBackgroundDeleteServiceTime", "bigint(20)"),
            new Column("diskSSDBackgroundRAStripeXServiceTime", "bigint(20)"),
            new Column("delayWAFLSuspCPWaitTime", "bigint(20)"),
            new Column("delayDiskIOWaitTime", "bigint(20)"),
            new Column("delayCloudIOWaitTime", "bigint(20)"),
            new Column("delayNvlogTransferWaitTime", "bigint(20)"),
            new Column("cacheMissRate", "float"),
            new Column("collectionPeriod", "bigint(20)"),
            new Column("cacheMissRateBase", "bigint(20)"),
            new Column("delayFlexcacheRalWaitTime", "bigint(20)"),
            new Column("delayFlexcacheSpinhiWaitTime", "bigint(20)"),
            new Column("delaySyncReplWaitTime", "bigint(20)")};

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {SqlMethod.SUM}, // cpuDBladeResidenceTime
            {SqlMethod.SUM}, // cpuDBladeUtilServiceTime
            {SqlMethod.SUM}, // cpuDBladeBackgroundServiceTime
            {SqlMethod.SUM}, // cpuDBladeBackgroundWAFLCPServiceTime
            {SqlMethod.SUM}, // cpuDBladeBackgroundBackgroundDeleteServiceTime
            {SqlMethod.SUM}, // cpuDBladeBackgroundRAStripeXServiceTime
            {SqlMethod.SUM}, // diskHDDBackgroundServiceTime
            {SqlMethod.SUM}, // diskHDDBackgroundWAFLCPServiceTime
            {SqlMethod.SUM}, // diskHDDBackgroundBackgroundDeleteServiceTime
            {SqlMethod.SUM}, // diskHDDBackgroundRAStripeXServiceTime
            {SqlMethod.SUM}, // diskSSDBackgroundServiceTime
            {SqlMethod.SUM}, // diskSSDBackgroundWAFLCPServiceTime
            {SqlMethod.SUM}, // diskSSDBackgroundBackgroundDeleteServiceTime
            {SqlMethod.SUM}, // diskSSDBackgroundRAStripeXServiceTime
            {SqlMethod.SUM}, // delayWAFLSuspCPWaitTime
            {SqlMethod.SUM}, // delayDiskIOWaitTime
            {SqlMethod.SUM}, // delayCloudIOWaitTime
            {SqlMethod.SUM}, // delayNvlogTransferWaitTime
            {new RatioMethod("cacheMissRateBase")}, // cacheMissRate
            {SqlMethod.SUM}, // collectionPeriod
            {SqlMethod.SUM}, // cacheMissRateBase
            {SqlMethod.SUM}, // delayFlexcacheRalWaitTime
            {SqlMethod.SUM}, // delayFlexcacheSpinhiWaitTime
            {SqlMethod.SUM}, // delaySyncReplWaitTime
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            SqlMethod.SUM, // cpuDBladeResidenceTime
            SqlMethod.SUM, // cpuDBladeUtilServiceTime
            SqlMethod.SUM, // cpuDBladeBackgroundServiceTime
            SqlMethod.SUM, // cpuDBladeBackgroundWAFLCPServiceTime
            SqlMethod.SUM, // cpuDBladeBackgroundBackgroundDeleteServiceTime
            SqlMethod.SUM, // cpuDBladeBackgroundRAStripeXServiceTime
            SqlMethod.SUM, // diskHDDBackgroundServiceTime
            SqlMethod.SUM, // diskHDDBackgroundWAFLCPServiceTime
            SqlMethod.SUM, // diskHDDBackgroundBackgroundDeleteServiceTime
            SqlMethod.SUM, // diskHDDBackgroundRAStripeXServiceTime
            SqlMethod.SUM, // diskSSDBackgroundServiceTime
            SqlMethod.SUM, // diskSSDBackgroundWAFLCPServiceTime
            SqlMethod.SUM, // diskSSDBackgroundBackgroundDeleteServiceTime
            SqlMethod.SUM, // diskSSDBackgroundRAStripeXServiceTime
            SqlMethod.SUM, // delayWAFLSuspCPWaitTime
            SqlMethod.SUM, // delayDiskIOWaitTime
            SqlMethod.SUM, // delayCloudIOWaitTime
            SqlMethod.SUM, // delayNvlogTransferWaitTime
            new RatioMethod("cacheMissRateBase"), // cacheMissRate
            SqlMethod.SUM, // collectionPeriod
            SqlMethod.SUM, // cacheMissRateBase
            SqlMethod.SUM, // delayFlexcacheRalWaitTime
            SqlMethod.SUM, // delayFlexcacheSpinhiWaitTime
            SqlMethod.SUM, // delaySyncReplWaitTime
    };

    private static ObjectParser<QosWorkloadQueueDBladeMeasurements> sampleParser;

    static {
        sampleParser = new ObjectParser<QosWorkloadQueueDBladeMeasurements>() {
            @Override
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement,
                                                   QosWorkloadQueueDBladeMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuDBladeResidenceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuDBladeUtilServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuDBladeBackgroundServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuDBladeBackgroundWAFLCPServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuDBladeBackgroundBackgroundDeleteServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuDBladeBackgroundRAStripXServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDiskHDDBackgroundServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDiskHDDBackgroundWAFLCPServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDiskHDDBackgroundBackgroundDeleteServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDiskHDDBackgroundRAStripeXServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDiskSSDBackgroundServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDiskSSDBackgroundWAFLCPServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDiskSSDBackgroundBackgroundDeleteServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDiskSSDBackgroundRAStripeXServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDelayWAFLSuspCPWaitTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDelayDiskIOWaitTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDelayCloudIOWaitTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDelayNvlogTransferWaitTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCacheMissRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCollectionPeriod());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCacheMissRateBase());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDelayFlexcacheRalWaitTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDelayFlexcacheSpinhiWaitTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDelaySyncReplWaitTime());
            }

            @Override
            public QosWorkloadQueueDBladeMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new QosWorkloadQueueDBladeMeasurements()
                        .setCpuDBladeResidenceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setCpuDBladeUtilServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setCpuDBladeBackgroundServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setCpuDBladeBackgroundWAFLCPServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setCpuDBladeBackgroundBackgroundDeleteServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setCpuDBladeBackgroundRAStripXServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDiskHDDBackgroundServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDiskHDDBackgroundWAFLCPServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDiskHDDBackgroundBackgroundDeleteServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDiskHDDBackgroundRAStripeXServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDiskSSDBackgroundServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDiskSSDBackgroundWAFLCPServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDiskSSDBackgroundBackgroundDeleteServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDiskSSDBackgroundRAStripeXServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDelayWAFLSuspCPWaitTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDelayDiskIOWaitTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDelayCloudIOWaitTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDelayNvlogTransferWaitTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setCacheMissRate(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setCollectionPeriod(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setCacheMissRateBase(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDelayFlexcacheRalWaitTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDelayFlexcacheSpinhiWaitTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDelaySyncReplWaitTime(PerformanceTableUtils.getLongNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<QosWorkloadQueueDBladeMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<QosWorkloadQueueDBladeMeasurements> getSampleParser() {
        return sampleParser;
    }
}
