/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.qosworkloaddblade.interfaces.data;

import java.io.Serializable;

/**
 * Measurement class for workload_queue_dblade counters
 */
@SuppressWarnings("UnusedDeclaration")
public class QosWorkloadQueueDBladeMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * This is the CPU service time plus the wait time for operations done for this workload in the Dblade.
     * Units are microseconds..
     */
    private long cpuDBladeResidenceTime;

    /**
     * This is the CPU service time for operations done for this workload in the Dblade. Units are microseconds.
     */
    private long cpuDBladeUtilServiceTime;

    /**
     * The raw CPU Dblade background service time. This is the CPU service time for background operations done for
     * this workload in the Dblade. Represents service time directly tallied for the workload.Units are
     * microseconds.
     */
    private long cpuDBladeBackgroundServiceTime;

    /**
     * The raw CPU Dblade background service time. Wafl_cp represents service time contributed by the WAFL_CP
     * workload on behalf of this workload. Units are microseconds.
     */
    private long cpuDBladeBackgroundWAFLCPServiceTime;

    /**
     * The raw CPU Dblade background service time. Background_delete represents service time contributed by the
     * BACKGROUND_DELETE workload on behalf of this workload. Units are microseconds.
     */
    private long cpuDBladeBackgroundBackgroundDeleteServiceTime;

    /**
     * The raw CPU Dblade background service time. Readahead_x_stripe represents service time contributed by the
     * RA_STRIPE_X workload on behalf of this workload. Units are microseconds.
     */
    private long cpuDBladeBackgroundRAStripXServiceTime;

    /**
     * The raw Disk HDD background service time. This is the amount of time HDDs spend servicing requests by this
     * workload. Represents service time directly tallied for the workload. Units are microseconds.
     */
    private long diskHDDBackgroundServiceTime;

    /**
     * The raw Disk HDD background service time. This is the amount of time HDDs spend servicing requests by this
     * workload. Wafl_cp represents service time contributed by the WAFL_CP workload on behalf of this workload.
     * Units are microseconds.
     */
    private long diskHDDBackgroundWAFLCPServiceTime;

    /**
     * The raw Disk HDD background service time. This is the amount of time HDDs spend servicing requests by this
     * workload. Background_delete represents service time contributed by the BACKGROUND_DELETE workload on behalf
     * of this workload. Units are microseconds.
     */
    private long diskHDDBackgroundBackgroundDeleteServiceTime;

    /**
     * The raw Disk HDD background service time. This is the amount of time HDDs spend servicing requests by this
     * workload. Readahead_x_stripe represents service time contributed by the RA_STRIPE_X workload on behalf of
     * this workload. Units are microseconds.
     */
    private long diskHDDBackgroundRAStripeXServiceTime;

    /**
     * The raw disk SSD background service time. This is the amount of time SSDs spend servicing requests by this
     * workload. Represents service time directly tallied for the workload. Units are microseconds.
     */
    private long diskSSDBackgroundServiceTime;

    /**
     * The raw disk SSD background service time. This is the amount of time SSDs spend servicing requests by this
     * workload.  Wafl_cp represents service time contributed by the WAFL_CP workload on behalf of this workload.
     * Units are microseconds.
     */
    private long diskSSDBackgroundWAFLCPServiceTime;

    /**
     * The raw disk SSD background service time. This is the amount of time SSDs spend servicing requests by this
     * workload. Background_delete represents service time contributed by the BACKGROUND_DELETE workload on behalf
     * of this workload.Units are microseconds.
     */
    private long diskSSDBackgroundBackgroundDeleteServiceTime;

    /**
     * The raw disk SSD background service time. This is the amount of time SSDs spend servicing requests by this
     * workload. Readahead_x_stripe represents service time contributed by the RA_STRIPE_X workload on behalf of
     * this workload.Units are microseconds.
     */
    private long diskSSDBackgroundRAStripeXServiceTime;

    /**
     * This is the time spent in WAFL waiting for a CP to complete for this workload. Units are microseconds.
     */
    private long delayWAFLSuspCPWaitTime;

    /**
     * This is the time spent waiting for a read from disk to complete for this workload.
     */
    private long delayDiskIOWaitTime;

    /**
     * This is the time spent waiting for a read from the cloud to complete for this workload.
     */
    private long delayCloudIOWaitTime;

    /**
     * NVLog transfer wait time. This is the time spent waiting for a dma transfer
     * to local nvram and remote nvram for writes. Units are microseconds.
     */
    private long delayNvlogTransferWaitTime;

    /**
     * Time spent in RAL fetching from the remote volume for flexcache.(microseconds)
     */
    private long delayFlexcacheRalWaitTime;

    /**
     * Time spent in spinhi performing the remote op for flexcache.(microseconds)
     */
    private long delayFlexcacheSpinhiWaitTime;

    /**
     * Time spent secondary path for sync replication.(microseconds)
     */
    private long delaySyncReplWaitTime;

    /**
     * This is the percentage of cache misses. The sum of read_io_type DISK and HYA_HDD.
     */
    private float cacheMissRate;

    /**
     * This is the total number of WAFL read requests. It is used as a base counter (or denominator) for the
     * cache_miss_rate counter. This counter is exactly similar to the read_io_type_base counter from the workload
     * object.
     */
    private long cacheMissRateBase;

    /**
     * Time in milliseconds between the current and the previous poll.
     */
    private long collectionPeriod;

    public long getCpuDBladeResidenceTime() {
        return cpuDBladeResidenceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setCpuDBladeResidenceTime(long cpuDBladeResidenceTime) {
        this.cpuDBladeResidenceTime = cpuDBladeResidenceTime;
        return this;
    }

    public long getCpuDBladeUtilServiceTime() {
        return cpuDBladeUtilServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setCpuDBladeUtilServiceTime(long cpuDBladeUtilServiceTime) {
        this.cpuDBladeUtilServiceTime = cpuDBladeUtilServiceTime;
        return this;
    }

    public long getCpuDBladeBackgroundServiceTime() {
        return cpuDBladeBackgroundServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setCpuDBladeBackgroundServiceTime(long cpuDBladeBackgroundServiceTime) {
        this.cpuDBladeBackgroundServiceTime = cpuDBladeBackgroundServiceTime;
        return this;
    }

    public long getDiskHDDBackgroundServiceTime() {
        return diskHDDBackgroundServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDiskHDDBackgroundServiceTime(long diskHDDBackgroundServiceTime) {
        this.diskHDDBackgroundServiceTime = diskHDDBackgroundServiceTime;
        return this;
    }

    public long getDiskSSDBackgroundServiceTime() {
        return diskSSDBackgroundServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDiskSSDBackgroundServiceTime(long diskSSDBackgroundServiceTime) {
        this.diskSSDBackgroundServiceTime = diskSSDBackgroundServiceTime;
        return this;
    }

    public long getDelayWAFLSuspCPWaitTime() {
        return delayWAFLSuspCPWaitTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDelayWAFLSuspCPWaitTime(long delayWAFLSuspCPWaitTime) {
        this.delayWAFLSuspCPWaitTime = delayWAFLSuspCPWaitTime;
        return this;
    }

    public long getDelayDiskIOWaitTime() {
        return delayDiskIOWaitTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDelayDiskIOWaitTime(long delayDiskIOWaitTime) {
        this.delayDiskIOWaitTime = delayDiskIOWaitTime;
        return this;
    }

    public long getDelayNvlogTransferWaitTime(){
        return delayNvlogTransferWaitTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDelayNvlogTransferWaitTime(long delayNvlogTransferWaitTime){
        this.delayNvlogTransferWaitTime = delayNvlogTransferWaitTime;
        return this;
    }

    public float getCacheMissRate(){
        return this.cacheMissRate;
    }

    public QosWorkloadQueueDBladeMeasurements setCacheMissRate(float cacheMissRate){
        this.cacheMissRate = cacheMissRate;
        return this;
    }

    public long getCpuDBladeBackgroundWAFLCPServiceTime() {
        return cpuDBladeBackgroundWAFLCPServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setCpuDBladeBackgroundWAFLCPServiceTime(long cpuDBladeBackgroundWAFLCPServiceTime) {
        this.cpuDBladeBackgroundWAFLCPServiceTime = cpuDBladeBackgroundWAFLCPServiceTime;
        return this;
    }

    public long getCpuDBladeBackgroundBackgroundDeleteServiceTime() {
        return cpuDBladeBackgroundBackgroundDeleteServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setCpuDBladeBackgroundBackgroundDeleteServiceTime(long cpuDBladeBackgroundBackgroundDeleteServiceTime) {
        this.cpuDBladeBackgroundBackgroundDeleteServiceTime = cpuDBladeBackgroundBackgroundDeleteServiceTime;
        return this;
    }

    public long getCpuDBladeBackgroundRAStripXServiceTime() {
        return cpuDBladeBackgroundRAStripXServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setCpuDBladeBackgroundRAStripXServiceTime(long cpuDBladeBackgroundRAStripXServiceTime) {
        this.cpuDBladeBackgroundRAStripXServiceTime = cpuDBladeBackgroundRAStripXServiceTime;
        return this;
    }

    public long getDiskHDDBackgroundWAFLCPServiceTime() {
        return diskHDDBackgroundWAFLCPServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDiskHDDBackgroundWAFLCPServiceTime(long diskHDDBackgroundWAFLCPServiceTime) {
        this.diskHDDBackgroundWAFLCPServiceTime = diskHDDBackgroundWAFLCPServiceTime;
        return this;
    }

    public long getDiskHDDBackgroundBackgroundDeleteServiceTime() {
        return diskHDDBackgroundBackgroundDeleteServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDiskHDDBackgroundBackgroundDeleteServiceTime(long diskHDDBackgroundBackgroundDeleteServiceTime) {
        this.diskHDDBackgroundBackgroundDeleteServiceTime = diskHDDBackgroundBackgroundDeleteServiceTime;
        return this;
    }

    public long getDiskHDDBackgroundRAStripeXServiceTime() {
        return diskHDDBackgroundRAStripeXServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDiskHDDBackgroundRAStripeXServiceTime(long diskHDDBackgroundRAStripeXServiceTime) {
        this.diskHDDBackgroundRAStripeXServiceTime = diskHDDBackgroundRAStripeXServiceTime;
        return this;
    }

    public long getDiskSSDBackgroundWAFLCPServiceTime() {
        return diskSSDBackgroundWAFLCPServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDiskSSDBackgroundWAFLCPServiceTime(long diskSSDBackgroundWAFLCPServiceTime) {
        this.diskSSDBackgroundWAFLCPServiceTime = diskSSDBackgroundWAFLCPServiceTime;
        return this;
    }

    public long getDiskSSDBackgroundBackgroundDeleteServiceTime() {
        return diskSSDBackgroundBackgroundDeleteServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDiskSSDBackgroundBackgroundDeleteServiceTime(long diskSSDBackgroundBackgroundDeleteServiceTime) {
        this.diskSSDBackgroundBackgroundDeleteServiceTime = diskSSDBackgroundBackgroundDeleteServiceTime;
        return this;
    }

    public long getDiskSSDBackgroundRAStripeXServiceTime() {
        return diskSSDBackgroundRAStripeXServiceTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDiskSSDBackgroundRAStripeXServiceTime(long diskSSDBackgroundRAStripeXServiceTime) {
        this.diskSSDBackgroundRAStripeXServiceTime = diskSSDBackgroundRAStripeXServiceTime;
        return this;
    }

    public long getCacheMissRateBase() {
        return cacheMissRateBase;
    }

    public QosWorkloadQueueDBladeMeasurements setCacheMissRateBase(long cacheMissRateBase) {
        this.cacheMissRateBase = cacheMissRateBase;
        return this;
    }

    public long getCollectionPeriod() {
        return collectionPeriod;
    }

    public QosWorkloadQueueDBladeMeasurements setCollectionPeriod(long collectionPeriod) {
        this.collectionPeriod = collectionPeriod;
        return this;
    }


    /**
     * Get delayCloudIOWaitTime.
     *
     * @return delayCloudIOWaitTime as long.
     */
    public long getDelayCloudIOWaitTime() {
        return delayCloudIOWaitTime;
    }

    /**
     * Set delayCloudIOWaitTime.
     *
     * @param delayCloudIOWaitTime the value to set.
     * @return QosWorkloadQueueDBladeMeasurements
     */
    public QosWorkloadQueueDBladeMeasurements setDelayCloudIOWaitTime(long delayCloudIOWaitTime) {
        this.delayCloudIOWaitTime = delayCloudIOWaitTime;
        return this;
    }


    public long getDelayFlexcacheRalWaitTime() {
        return delayFlexcacheRalWaitTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDelayFlexcacheRalWaitTime(long delayFlexcacheRalWaitTime) {
        this.delayFlexcacheRalWaitTime = delayFlexcacheRalWaitTime;
        return this;
    }

    public long getDelayFlexcacheSpinhiWaitTime() {
        return delayFlexcacheSpinhiWaitTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDelayFlexcacheSpinhiWaitTime(long delayFlexcacheSpinhiWaitTime) {
        this.delayFlexcacheSpinhiWaitTime = delayFlexcacheSpinhiWaitTime;
        return this;
    }

    public long getDelaySyncReplWaitTime() {
        return delaySyncReplWaitTime;
    }

    public QosWorkloadQueueDBladeMeasurements setDelaySyncReplWaitTime(long delaySyncReplWaitTime) {
        this.delaySyncReplWaitTime = delaySyncReplWaitTime;
        return this;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        QosWorkloadQueueDBladeMeasurements that = (QosWorkloadQueueDBladeMeasurements) o;

        if (Float.compare(that.cacheMissRate, cacheMissRate) != 0) return false;
        if (cacheMissRateBase != that.cacheMissRateBase) return false;
        if (collectionPeriod != that.collectionPeriod) return false;
        if (cpuDBladeBackgroundBackgroundDeleteServiceTime != that.cpuDBladeBackgroundBackgroundDeleteServiceTime)
            return false;
        if (cpuDBladeBackgroundRAStripXServiceTime != that.cpuDBladeBackgroundRAStripXServiceTime) return false;
        if (cpuDBladeBackgroundServiceTime != that.cpuDBladeBackgroundServiceTime) return false;
        if (cpuDBladeBackgroundWAFLCPServiceTime != that.cpuDBladeBackgroundWAFLCPServiceTime) return false;
        if (cpuDBladeResidenceTime != that.cpuDBladeResidenceTime) return false;
        if (cpuDBladeUtilServiceTime != that.cpuDBladeUtilServiceTime) return false;
        if (delayDiskIOWaitTime != that.delayDiskIOWaitTime) return false;
        if (delayCloudIOWaitTime != that.delayCloudIOWaitTime) return false;
        if (delayNvlogTransferWaitTime != that.delayNvlogTransferWaitTime) return false;
        if (delayWAFLSuspCPWaitTime != that.delayWAFLSuspCPWaitTime) return false;
        if (diskHDDBackgroundBackgroundDeleteServiceTime != that.diskHDDBackgroundBackgroundDeleteServiceTime)
            return false;
        if (diskHDDBackgroundRAStripeXServiceTime != that.diskHDDBackgroundRAStripeXServiceTime) return false;
        if (diskHDDBackgroundServiceTime != that.diskHDDBackgroundServiceTime) return false;
        if (diskHDDBackgroundWAFLCPServiceTime != that.diskHDDBackgroundWAFLCPServiceTime) return false;
        if (diskSSDBackgroundBackgroundDeleteServiceTime != that.diskSSDBackgroundBackgroundDeleteServiceTime)
            return false;
        if (diskSSDBackgroundRAStripeXServiceTime != that.diskSSDBackgroundRAStripeXServiceTime) return false;
        if (diskSSDBackgroundServiceTime != that.diskSSDBackgroundServiceTime) return false;
        if (diskSSDBackgroundWAFLCPServiceTime != that.diskSSDBackgroundWAFLCPServiceTime) return false;
        if (delayFlexcacheRalWaitTime != that.delayFlexcacheRalWaitTime) return false;
        if (delayFlexcacheSpinhiWaitTime != that.delayFlexcacheSpinhiWaitTime) return false;
        if (delaySyncReplWaitTime != that.delaySyncReplWaitTime) return false;

        return true;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("QosWorkloadQueueDBladeMeasurements{");
        sb.append("cpuDBladeResidenceTime=").append(cpuDBladeResidenceTime);
        sb.append(", cpuDBladeUtilServiceTime=").append(cpuDBladeUtilServiceTime);
        sb.append(", cpuDBladeBackgroundServiceTime=").append(cpuDBladeBackgroundServiceTime);
        sb.append(", cpuDBladeBackgroundWAFLCPServiceTime=").append(cpuDBladeBackgroundWAFLCPServiceTime);
        sb.append(", cpuDBladeBackgroundBackgroundDeleteServiceTime=").append(cpuDBladeBackgroundBackgroundDeleteServiceTime);
        sb.append(", cpuDBladeBackgroundRAStripXServiceTime=").append(cpuDBladeBackgroundRAStripXServiceTime);
        sb.append(", diskHDDBackgroundServiceTime=").append(diskHDDBackgroundServiceTime);
        sb.append(", diskHDDBackgroundWAFLCPServiceTime=").append(diskHDDBackgroundWAFLCPServiceTime);
        sb.append(", diskHDDBackgroundBackgroundDeleteServiceTime=").append(diskHDDBackgroundBackgroundDeleteServiceTime);
        sb.append(", diskHDDBackgroundRAStripeXServiceTime=").append(diskHDDBackgroundRAStripeXServiceTime);
        sb.append(", diskSSDBackgroundServiceTime=").append(diskSSDBackgroundServiceTime);
        sb.append(", diskSSDBackgroundWAFLCPServiceTime=").append(diskSSDBackgroundWAFLCPServiceTime);
        sb.append(", diskSSDBackgroundBackgroundDeleteServiceTime=").append(diskSSDBackgroundBackgroundDeleteServiceTime);
        sb.append(", diskSSDBackgroundRAStripeXServiceTime=").append(diskSSDBackgroundRAStripeXServiceTime);
        sb.append(", delayWAFLSuspCPWaitTime=").append(delayWAFLSuspCPWaitTime);
        sb.append(", delayDiskIOWaitTime=").append(delayDiskIOWaitTime);
        sb.append(", delayCloudIOWaitTime=").append(delayCloudIOWaitTime);
        sb.append(", delayNvlogTransferWaitTime=").append(delayNvlogTransferWaitTime);
        sb.append(", delayFlexcacheRalWaitTime=").append(delayFlexcacheRalWaitTime);
        sb.append(", delayFlexcacheSpinhiWaitTime=").append(delayFlexcacheSpinhiWaitTime);
        sb.append(", delaySyncReplWaitTime=").append(delaySyncReplWaitTime);
        sb.append(", cacheMissRate=").append(cacheMissRate);
        sb.append(", cacheMissRateBase=").append(cacheMissRateBase);
        sb.append(", collectionPeriod=").append(collectionPeriod);
        sb.append('}');
        return sb.toString();
    }

    @Override
    public int hashCode() {
        int result = (int) (cpuDBladeResidenceTime ^ (cpuDBladeResidenceTime >>> 32));
        result = 31 * result + (int) (cpuDBladeUtilServiceTime ^ (cpuDBladeUtilServiceTime >>> 32));
        result = 31 * result + (int) (cpuDBladeBackgroundServiceTime ^ (cpuDBladeBackgroundServiceTime >>> 32));
        result = 31 * result + (int) (cpuDBladeBackgroundWAFLCPServiceTime ^ (cpuDBladeBackgroundWAFLCPServiceTime >>> 32));
        result = 31 * result + (int) (cpuDBladeBackgroundBackgroundDeleteServiceTime ^ (cpuDBladeBackgroundBackgroundDeleteServiceTime >>> 32));
        result = 31 * result + (int) (cpuDBladeBackgroundRAStripXServiceTime ^ (cpuDBladeBackgroundRAStripXServiceTime >>> 32));
        result = 31 * result + (int) (diskHDDBackgroundServiceTime ^ (diskHDDBackgroundServiceTime >>> 32));
        result = 31 * result + (int) (diskHDDBackgroundWAFLCPServiceTime ^ (diskHDDBackgroundWAFLCPServiceTime >>> 32));
        result = 31 * result + (int) (diskHDDBackgroundBackgroundDeleteServiceTime ^ (diskHDDBackgroundBackgroundDeleteServiceTime >>> 32));
        result = 31 * result + (int) (diskHDDBackgroundRAStripeXServiceTime ^ (diskHDDBackgroundRAStripeXServiceTime >>> 32));
        result = 31 * result + (int) (diskSSDBackgroundServiceTime ^ (diskSSDBackgroundServiceTime >>> 32));
        result = 31 * result + (int) (diskSSDBackgroundWAFLCPServiceTime ^ (diskSSDBackgroundWAFLCPServiceTime >>> 32));
        result = 31 * result + (int) (diskSSDBackgroundBackgroundDeleteServiceTime ^ (diskSSDBackgroundBackgroundDeleteServiceTime >>> 32));
        result = 31 * result + (int) (diskSSDBackgroundRAStripeXServiceTime ^ (diskSSDBackgroundRAStripeXServiceTime >>> 32));
        result = 31 * result + (int) (delayWAFLSuspCPWaitTime ^ (delayWAFLSuspCPWaitTime >>> 32));
        result = 31 * result + (int) (delayDiskIOWaitTime ^ (delayDiskIOWaitTime >>> 32));
        result = 31 * result + (int) (delayCloudIOWaitTime ^ (delayCloudIOWaitTime >>> 32));
        result = 31 * result + (int) (delayNvlogTransferWaitTime ^ (delayNvlogTransferWaitTime >>> 32));
        result = 31 * result + (int) (delayFlexcacheRalWaitTime ^ (delayFlexcacheRalWaitTime >>> 32));
        result = 31 * result + (int) (delayFlexcacheSpinhiWaitTime ^ (delayFlexcacheSpinhiWaitTime >>> 32));
        result = 31 * result + (int) (delaySyncReplWaitTime ^ (delaySyncReplWaitTime >>> 32));
        result = 31 * result + (cacheMissRate != +0.0f ? Float.floatToIntBits(cacheMissRate) : 0);
        result = 31 * result + (int) (cacheMissRateBase ^ (cacheMissRateBase >>> 32));
        result = 31 * result + (int) (collectionPeriod ^ (collectionPeriod >>> 32));
        return result;
    }
}
