/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.qosworkloaddetail.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.qosworkloaddetail.interfaces.data.QosWorkloadDetailMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Remote
public interface QosWorkloadDetailPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//QosWorkloadDetailPerformanceQueryBean!com.netapp.oci.netapp.performance.qosworkloaddetail.interfaces.remote.QosWorkloadDetailPerformanceQueryRemote";

    /**
     * Gets QoS workload detail samples for period of time
     *
     * @param qosWorkloadDetailIds set of QoS workload detail ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of QoS volume workload detail id to list ordered by time of QoS volume workload detail samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<QosWorkloadDetailMeasurements>>> getSamples(Set<Long> qosWorkloadDetailIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS workload detail samples for period of time
     *
     * @param containerId container id
     * @param qosWorkloadDetailIds set of QoS workload detail ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of QoS volume workload detail id to list ordered by time of QoS volume workload detail samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<QosWorkloadDetailMeasurements>>> getSamples(Long containerId, Set<Long> qosWorkloadDetailIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS workload detail samples for period of time
     *
     * @param containerId container id
     * @param qosWorkloadDetailId QoS workload detail id
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return list ordered by time of QoS volume workload detail samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<QosWorkloadDetailMeasurements>> getSamples(Long containerId, Long qosWorkloadDetailId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given qosWorkloadDetail id's (qosWorkloadDetailIds). The map returned will contain only qosWorkloadDetail id's that has a sample.
     * qosWorkloadDetail ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the qosWorkloadDetail ids passed.
     *
     * @param clusterId cluster id
     * @param qosWorkloadDetailIds set of qosWorkloadDetail ids
     * @return map of qosWorkloadDetail id to latest sample SampleInfo<QosWorkloadDetailMeasurements>. qosWorkloadDetail ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<QosWorkloadDetailMeasurements>> getLatestSample(Long clusterId, Set<Long> qosWorkloadDetailIds) throws NetappInterfaceException;

    /**
     * Gets QoS workload detail summaries for QoS volume workload ids
     *
     * @param qosWorkloadDetailIds set of QoS workload detail ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of QoS volume workload detail id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<QosWorkloadDetailMeasurements>>> getSummaries(Set<Long> qosWorkloadDetailIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS workload detail summaries for QoS volume workload ids
     *
     * @param clusterId cluster id
     * @param qosWorkloadDetailIds set of QoS workload detail ids
     * @param fromTime beginning of the time period
     * @param toTime end of the time period
     * @return map of QoS volume workload detail id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<QosWorkloadDetailMeasurements>>> getSummaries(Long clusterId, Set<Long> qosWorkloadDetailIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * returns true if samples exist over the time period
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}

