/*
 * Copyright (c) 2014 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.qosworkloadnblade;

import com.netapp.oci.netapp.performance.qosworkloadnblade.interfaces.data.QosWorkloadQueueNBladeMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Meta data class for handling persistence for {@link QosWorkloadQueueNBladeMeasurements}
 */
public class QosWorkloadQueueNBladeMetaData {
    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("readOps", "float"),
            new Column("writeOps", "float"),
            new Column("otherOps", "float"),
            new Column("readLatency", "float"),
            new Column("writeLatency", "float"),
            new Column("otherLatency", "float"),
            new Column("readData", "float"),
            new Column("writeData", "float"),
            new Column("cpuNBladeResidenceTime", "bigint(20)"),
            new Column("cpuNBladeUtilServiceTime", "bigint(20)"),
            new Column("delayClusterInterconnectWaitTime", "bigint(20)"),
            new Column("delayQosLimitWaitTime", "bigint(20)"),
            new Column("delayNetworkWaitTime", "bigint(20)"),
            new Column("collectionPeriod", "bigint(20)")};

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("readOps", "float"),
            new Column("writeOps", "float"),
            new Column("otherOps", "float"),
            new Column("readLatency", "float"),
            new Column("writeLatency", "float"),
            new Column("otherLatency", "float"),
            new Column("readData", "float"),
            new Column("writeData", "float"),
            new Column("cpuNBladeResidenceTime", "bigint(20)"),
            new Column("cpuNBladeUtilServiceTime", "bigint(20)"),
            new Column("delayClusterInterconnectWaitTime", "bigint(20)"),
            new Column("delayQosLimitWaitTime", "bigint(20)"),
            new Column("delayNetworkWaitTime", "bigint(20)"),
            new Column("collectionPeriod", "bigint(20)")};

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {SqlMethod.AVG}, // readOps
            {SqlMethod.AVG}, // writeOps
            {SqlMethod.AVG}, // otherOps
            {new RatioMethod("readOps")}, // readLatency
            {new RatioMethod("writeOps")}, // writeLatency
            {new RatioMethod("otherOps")}, // otherLatency
            {SqlMethod.AVG}, // readData
            {SqlMethod.AVG}, // writeData ////
            {SqlMethod.SUM}, // cpuNBladeResidenceTime
            {SqlMethod.SUM}, // cpuNBladeUtilServiceTime
            {SqlMethod.SUM}, // delayClusterInterconnectWaitTime
            {SqlMethod.SUM}, // delayQosLimitWaitTime
            {SqlMethod.SUM}, // delayNetworkWaitTime
            {SqlMethod.SUM}, // collectionPeriod
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            SqlMethod.AVG, // readOps
            SqlMethod.AVG, // writeOps
            SqlMethod.AVG, // otherOps
            SqlMethod.AVG, // readLatency
            SqlMethod.AVG, // writeLatency
            SqlMethod.AVG, // otherLatency
            SqlMethod.AVG, // readData
            SqlMethod.AVG, // writeData   ///
            SqlMethod.SUM, // cpuNBladeResidenceTime
            SqlMethod.SUM, // cpuNBladeUtilServiceTime
            SqlMethod.SUM, // delayClusterInterconnectWaitTime
            SqlMethod.SUM, // delayQosLimitWaitTime
            SqlMethod.SUM, // delayNetworkWaitTime
            SqlMethod.SUM, // collectionPeriod
    };

    private static ObjectParser<QosWorkloadQueueNBladeMeasurements> sampleParser;

    static {
        sampleParser = new ObjectParser<QosWorkloadQueueNBladeMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement,
                                                   QosWorkloadQueueNBladeMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuNBladeResidenceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpuNBladeUtilServiceTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDelayClusterInterconnectWaitTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDelayQosLimitWaitTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getDelayNetworkWaitTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCollectionPeriod());
            }

            public QosWorkloadQueueNBladeMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new QosWorkloadQueueNBladeMeasurements()
                        .setReadOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setWriteOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOtherOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setReadLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setWriteLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOtherLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setReadData(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setWriteData(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setCpuNBladeResidenceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setCpuNBladeUtilServiceTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDelayClusterInterconnectWaitTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDelayQosLimitWaitTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setDelayNetworkWaitTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                        .setCollectionPeriod(PerformanceTableUtils.getLongNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<QosWorkloadQueueNBladeMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<QosWorkloadQueueNBladeMeasurements> getSampleParser() {
        return sampleParser;
    }
}
