/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.qosworkloadnblade.interfaces.data;

import java.io.Serializable;

/**
 * Measurement class for workload_queue_nblade counters
 */
@SuppressWarnings("UnusedDeclaration")
public class QosWorkloadQueueNBladeMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * The workload's read operations that completed during the measurement interval; measured per second.
     */
    private float readOps;

    /**
     * The workload's write operations that completed during the measurement interval; measured per second.
     */
    private float writeOps;

    /**
     * The workload's (other) operations that completed during the measurement interval; measured per second.
     */
    private float otherOps;

    /**
     * The average read response time for requests that were initiated by the workload.
     */
    private float readLatency;

    /**
     * The average write response time for requests that were initiated by the workload.
     */
    private float writeLatency;

    /**
     * The average (other) response time for requests that were initiated by the workload.
     */
    private float otherLatency;

    /**
     * The data read per second by the workload.
     */
    private float readData;

    /**
     * The data written per second by the workload.
     */
    private float writeData;

    /**
     * The raw CPU Nblade residence time. Units are microseconds, not microseconds/visit.
     */
    private long cpuNBladeResidenceTime;

    /**
     * The raw CPU Nblade util service time. Units are microseconds, not microseconds/visit.
     */
    private long cpuNBladeUtilServiceTime;

    /**
     * The raw Cluster interconnect wait time. Units are in microseconds, not microseconds/visit.
     */
    private long delayClusterInterconnectWaitTime;

    /**
     * The raw QoS wait time. Units are in microseconds, not microseconds/visit.
     */
    private long delayQosLimitWaitTime;

    /**
     * The raw Network wait time. Units are in microseconds, not microseconds/visit.
     */
    private long delayNetworkWaitTime;

    /**
     * Collection period in milliseconds.
     */
    private long collectionPeriod;

    public float getReadOps() {
        return readOps;
    }

    public QosWorkloadQueueNBladeMeasurements setReadOps(float readOps) {
        this.readOps = readOps;
        return this;
    }

    public float getWriteOps() {
        return writeOps;
    }

    public QosWorkloadQueueNBladeMeasurements setWriteOps(float writeOps) {
        this.writeOps = writeOps;
        return this;
    }

    public float getOtherOps() {
        return otherOps;
    }

    public QosWorkloadQueueNBladeMeasurements setOtherOps(float otherOps) {
        this.otherOps = otherOps;
        return this;
    }

    public float getReadLatency() {
        return readLatency;
    }

    public QosWorkloadQueueNBladeMeasurements setReadLatency(float readLatency) {
        this.readLatency = readLatency;
        return this;
    }

    public float getWriteLatency() {
        return writeLatency;
    }

    public QosWorkloadQueueNBladeMeasurements setWriteLatency(float writeLatency) {
        this.writeLatency = writeLatency;
        return this;
    }

    public float getOtherLatency() {
        return otherLatency;
    }

    public QosWorkloadQueueNBladeMeasurements setOtherLatency(float otherLatency) {
        this.otherLatency = otherLatency;
        return this;
    }

    public float getReadData() {
        return readData;
    }

    public QosWorkloadQueueNBladeMeasurements setReadData(float readData) {
        this.readData = readData;
        return this;
    }

    public float getWriteData() {
        return writeData;
    }

    public QosWorkloadQueueNBladeMeasurements setWriteData(float writeData) {
        this.writeData = writeData;
        return this;
    }

    public long getCpuNBladeResidenceTime() {
        return cpuNBladeResidenceTime;
    }

    public QosWorkloadQueueNBladeMeasurements setCpuNBladeResidenceTime(long cpuNBladeResidenceTime) {
        this.cpuNBladeResidenceTime = cpuNBladeResidenceTime;
        return this;
    }

    public long getCpuNBladeUtilServiceTime() {
        return cpuNBladeUtilServiceTime;
    }

    public QosWorkloadQueueNBladeMeasurements setCpuNBladeUtilServiceTime(long cpuNBladeUtilServiceTime) {
        this.cpuNBladeUtilServiceTime = cpuNBladeUtilServiceTime;
        return this;
    }

    public long getDelayClusterInterconnectWaitTime() {
        return delayClusterInterconnectWaitTime;
    }

    public QosWorkloadQueueNBladeMeasurements setDelayClusterInterconnectWaitTime(long delayClusterInterconnectWaitTime) {
        this.delayClusterInterconnectWaitTime = delayClusterInterconnectWaitTime;
        return this;
    }

    public long getDelayQosLimitWaitTime() {
        return delayQosLimitWaitTime;
    }

    public QosWorkloadQueueNBladeMeasurements setDelayQosLimitWaitTime(long delayQosLimitWaitTime) {
        this.delayQosLimitWaitTime = delayQosLimitWaitTime;
        return this;
    }

    public long getDelayNetworkWaitTime() {
        return delayNetworkWaitTime;
    }

    public QosWorkloadQueueNBladeMeasurements setDelayNetworkWaitTime(long delayNetworkWaitTime) {
        this.delayNetworkWaitTime = delayNetworkWaitTime;
        return this;
    }

    public long getCollectionPeriod() {
        return collectionPeriod;
    }

    public QosWorkloadQueueNBladeMeasurements setCollectionPeriod(long collectionPeriod) {
        this.collectionPeriod = collectionPeriod;
        return this;
    }

    @Override
    public int hashCode() {
        int result = Float.floatToIntBits(readOps);
        result = 31 * result + Float.floatToIntBits(writeOps);
        result = 31 * result + Float.floatToIntBits(otherOps);
        result = 31 * result + Float.floatToIntBits(readLatency);
        result = 31 * result + Float.floatToIntBits(writeLatency);
        result = 31 * result + Float.floatToIntBits(otherLatency);
        result = 31 * result + Float.floatToIntBits(readData);
        result = 31 * result + Float.floatToIntBits(writeData);
        result = 31 * result + (int) (cpuNBladeResidenceTime ^ (cpuNBladeResidenceTime >>> 32));
        result = 31 * result + (int) (cpuNBladeUtilServiceTime ^ (cpuNBladeUtilServiceTime >>> 32));
        result = 31 * result + (int) (delayClusterInterconnectWaitTime ^ (delayClusterInterconnectWaitTime >>> 32));
        result = 31 * result + (int) (delayQosLimitWaitTime ^ (delayQosLimitWaitTime >>> 32));
        result = 31 * result + (int) (delayNetworkWaitTime ^ (delayNetworkWaitTime >>> 32));
        result = 31 * result + (int) (collectionPeriod ^ (collectionPeriod >>> 32));
        return result;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof QosWorkloadQueueNBladeMeasurements)) return false;

        QosWorkloadQueueNBladeMeasurements that = (QosWorkloadQueueNBladeMeasurements) o;

        return Float.compare(that.readOps, readOps) == 0
                && Float.compare(that.writeOps, writeOps) == 0
                && Float.compare(that.otherOps, otherOps) == 0
                && Float.compare(that.readLatency, readLatency) == 0
                && Float.compare(that.writeLatency, writeLatency) == 0
                && Float.compare(that.otherLatency, otherLatency) == 0
                && Float.compare(that.readData, readData) == 0
                && Float.compare(that.writeData, writeData) == 0
                && that.cpuNBladeResidenceTime == cpuNBladeResidenceTime
                && that.cpuNBladeUtilServiceTime == cpuNBladeUtilServiceTime
                && that.delayClusterInterconnectWaitTime == delayClusterInterconnectWaitTime
                && that.delayQosLimitWaitTime == delayQosLimitWaitTime
                && that.delayNetworkWaitTime == delayNetworkWaitTime
                && that.collectionPeriod == collectionPeriod;
    }


    @Override
    public String toString() {
        return new StringBuilder("QosWorkloadQueueNBladeMeasurements{")
                .append("readOps=").append(readOps)
                .append(", writeOps=").append(writeOps)
                .append(", otherOps=").append(otherOps)
                .append(", readLatency=").append(readLatency)
                .append(", writeLatency=").append(writeLatency)
                .append(", otherLatency=").append(otherLatency)
                .append(", readData=").append(readData)
                .append(", writeData=").append(writeData)
                .append(", cpuNBladeResidenceTime=").append(cpuNBladeResidenceTime)
                .append(", cpuNBladeUtilServiceTime=").append(cpuNBladeUtilServiceTime)
                .append(", delayClusterInterconnectWaitTime=").append(delayClusterInterconnectWaitTime)
                .append(", delayQosLimitWaitTime=").append(delayQosLimitWaitTime)
                .append(", delayNetworkWaitTime=").append(delayNetworkWaitTime)
                .append(", collectionPeriod=").append(collectionPeriod)
                .append('}').toString();
    }
}