/*
 * Copyright (c) 2014 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.qosworkloadnblade.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.qosworkloadnblade.interfaces.data.QosWorkloadQueueNBladeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Remote
public interface QosWorkloadQueueNBladePerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//QosWorkloadQueueNBladePerformanceQueryBean!com.netapp.oci.netapp.performance.qosworkloadnblade.interfaces.remote.QosWorkloadQueueNBladePerformanceQueryRemote";

    /**
     * Gets QoS workload queue nBlade samples for period of time
     *
     * @param qosWorkloadNodeRelationshipIds set of QoS workload node relationship ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of QoS workload node relationship id to list ordered by time of QoS workload queue nBlade samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<QosWorkloadQueueNBladeMeasurements>>> getSamples(Set<Long> qosWorkloadNodeRelationshipIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * Gets QoS workload queue nBlade samples for period of time
     *
     * @param clusterId                      cluster id
     * @param qosWorkloadNodeRelationshipIds set of QoS workload node relationship ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of QoS workload node relationship id to list ordered by time of QoS workload queue nBlade samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<QosWorkloadQueueNBladeMeasurements>>> getSamples(Long clusterId, Set<Long> qosWorkloadNodeRelationshipIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS workload queue nBlade samples for period of time
     *
     * @param clusterId                     cluster id
     * @param qosWorkloadNodeRelationshipId Qos Workload Node Relationship  id
     * @param fromTime                      beginning of the time period
     * @param toTime                        end of the time period
     * @return list ordered by time of QoS workload queue nBlade samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    List<SampleInfo<QosWorkloadQueueNBladeMeasurements>> getSamples(Long clusterId, Long qosWorkloadNodeRelationshipId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given qosVolume id's (qosVolumeIds). The map returned will contain only qosWorkloadQueueNBlade ids that has a sample.
     * qosWorkloadQueueNBlade ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the qosWorkloadQueueNBlade ids passed.
     *
     * @param clusterId                      cluster id
     * @param qosWorkloadNodeRelationshipIds set of QoS workload node relationship ids
     * @return map of qosWorkloadQueueNBlade id to latest sample SampleInfo<QosWorkloadQueueNBladeMeasurements>. qosWorkloadQueueNBlade ids with no samples will not be included in this map.
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, SampleInfo<QosWorkloadQueueNBladeMeasurements>> getLatestSample(Long clusterId, Set<Long> qosWorkloadNodeRelationshipIds) throws NetappInterfaceException;

    /**
     * Gets QoS workload queue nBlade summaries for QoS workload node relationship ids
     *
     * @param qosWorkloadNodeRelationshipIds set of QoS workload node relationship ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of QoS workload node relationship id to summary data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<Summary<QosWorkloadQueueNBladeMeasurements>>> getSummaries(Set<Long> qosWorkloadNodeRelationshipIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS workload queue nBlade summaries for QoS workload node relationship ids
     *
     * @param clusterId Cluster Id
     * @param qosWorkloadNodeRelationshipIds set of QoS workload node relationship ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of QoS workload node relationship id to summary data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<Summary<QosWorkloadQueueNBladeMeasurements>>> getSummaries(Long clusterId, Set<Long> qosWorkloadNodeRelationshipIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param clusterId
     * @param qosWorkloadNodeRelationshipId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long clusterId, Long qosWorkloadNodeRelationshipId, Date fromTime, Date toTime) throws NetappInterfaceException;
}
