package com.netapp.oci.netapp.performance.resourceheadroom;

import java.sql.ResultSet;
import java.sql.SQLException;

import com.netapp.oci.netapp.performance.resourceheadroom.interfaces.data.ResourceHeadRoomAggrMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

public class ResourceHeadRoomAggrMetaData{

	public static final Column[] SAMPLE_COLUMNS = new Column[]{
		new Column("currentUtil", "float"),
		new Column("currentUtilTotal", "bigint unsigned"),
		new Column("currentLatency", "float"),
		new Column("currentOps", "float"),
		new Column("serviceTimeSumOfSquares", "float"),
		new Column("serviceTimeSum", "float"),
		new Column("serviceTimeCount", "bigint unsigned"),
        new Column("interarrivalTimeSumOfSquares", "float"),
        new Column("interarrivalTimeSum", "float"),
        new Column("interarrivalTimeCount", "bigint unsigned"),
        new Column("waitTime", "bigint unsigned"),
        new Column("internalWorkloadUtilization", "float")
	};

	public static final Column[] SUMMARY_COLUMNS = new Column[]{
		new Column("currentUtil", "float"),
		new Column("currentUtilTotal", "bigint unsigned"),
		new Column("currentLatency", "float"),
		new Column("currentOps", "float"),
		new Column("serviceTimeSumOfSquares", "float"),
		new Column("serviceTimeSum", "float"),
		new Column("serviceTimeCount", "bigint unsigned"),
        new Column("interarrivalTimeSumOfSquares", "float"),
        new Column("interarrivalTimeSum", "float"),
        new Column("interarrivalTimeCount", "bigint unsigned"),
        new Column("waitTime", "bigint unsigned"),
		new Column("internalWorkloadUtilization", "float")

    };

    private static ObjectParser<ResourceHeadRoomAggrMeasurements> sampleParser;

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
    	{new RatioMethod("currentUtilTotal")},
    	{SqlMethod.AVG},
    	{new RatioMethod("currentOps")},
    	{SqlMethod.AVG},
    	{new RatioMethod("serviceTimeCount")},
    	{new RatioMethod("serviceTimeCount")},
    	{SqlMethod.SUM},
    	{new RatioMethod("interarrivalTimeCount")},
    	{new RatioMethod("interarrivalTimeCount")},
    	{SqlMethod.SUM},
    	{SqlMethod.SUM},
    	{SqlMethod.AVG}
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
    	new RatioMethod("currentUtilTotal"),
    	SqlMethod.AVG,
    	new RatioMethod("currentOps"),
    	SqlMethod.AVG,
    	new RatioMethod("serviceTimeCount"),
    	new RatioMethod("serviceTimeCount"),
    	SqlMethod.SUM,
    	new RatioMethod("interarrivalTimeCount"),
    	new RatioMethod("interarrivalTimeCount"),
    	SqlMethod.SUM,
    	SqlMethod.SUM,
    	SqlMethod.AVG
    };


    static {
        sampleParser = new ObjectParser<ResourceHeadRoomAggrMeasurements>() {
            @Override
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, ResourceHeadRoomAggrMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCurrentUtil());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCurrentUtilTotal());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCurrentLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCurrentOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getServiceTimeSumOfSquares());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getServiceTimeSum());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getServiceTimeCount());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getInterarrivalTimeSumOfSquares());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getInterarrivalTimeSum());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getInterarrivalTimeCount());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaitTime());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getInternalWorkloadUtilization());
            }

            @Override
            public ResourceHeadRoomAggrMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new ResourceHeadRoomAggrMeasurements()
                	.setCurrentUtil(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                	.setCurrentUtilTotal(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                	.setCurrentLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                	.setCurrentOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                	.setServiceTimeSumOfSquares(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                	.setServiceTimeSum(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                	.setServiceTimeCount(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                	.setInterarrivalTimeSumOfSquares(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                	.setInterarrivalTimeSum(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                	.setInterarrivalTimeCount(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                	.setWaitTime(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                	.setInternalWorkloadUtilization(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<ResourceHeadRoomAggrMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<ResourceHeadRoomAggrMeasurements> getSampleParser() {
        return sampleParser;
    }
}
