package com.netapp.oci.netapp.performance.resourceheadroom;

import java.sql.ResultSet;
import java.sql.SQLException;

import com.netapp.oci.netapp.performance.resourceheadroom.interfaces.data.ResourceHeadRoomCPUMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

public class ResourceHeadRoomCPUMetaData{

	public static final Column[] SAMPLE_COLUMNS = new Column[]{
		new Column("currentUtil", "float"),
		new Column("currentUtilTotal", "bigint unsigned"),
		new Column("currentLatency", "float"),
		new Column("currentOps", "float"),
		new Column("waflHiPriServiceTimeSumOfSquares", "float"),
		new Column("waflHiPriServiceTimeSum", "float"),
		new Column("waflHiPriServiceTimeCount", "bigint unsigned"),
		new Column("waflLoPriServiceTimeSumOfSquares", "float"),
		new Column("waflLoPriServiceTimeSum", "float"),
		new Column("waflLoPriServiceTimeCount", "bigint unsigned"),
		new Column("waflCpPriServiceTimeSumOfSquares", "float"),
		new Column("waflCpPriServiceTimeSum", "float"),
		new Column("waflCpPriServiceTimeCount", "bigint unsigned"),
		new Column("waflHiPriInterarrivalTimeVariance", "bigint unsigned"),
		new Column("waflHiPriInterarrivalTimeMean", "bigint unsigned"),
		new Column("waflHiPriInterarrivalTimeCount", "bigint unsigned"),
		new Column("waflLoPriInterarrivalTimeVariance", "bigint unsigned"),
		new Column("waflLoPriInterarrivalTimeMean", "bigint unsigned"),
		new Column("waflLoPriInterarrivalTimeCount", "bigint unsigned"),
		new Column("waflCpPriInterarrivalTimeVariance", "bigint unsigned"),
		new Column("waflCpPriInterarrivalTimeMean", "bigint unsigned"),
		new Column("waflCpPriInterarrivalTimeCount", "bigint unsigned"),
		new Column("inWorkloadUtilization", "float"),
		new Column("workloadSumDbladeUtil", "float"),
		new Column("userWorkloadSumDbladeUtil", "float"),
	};

	public static final Column[] SUMMARY_COLUMNS = new Column[]{
		new Column("currentUtil", "float"),
		new Column("currentUtilTotal", "bigint unsigned"),
		new Column("currentLatency", "float"),
		new Column("currentOps", "float"),
		new Column("waflHiPriServiceTimeSumOfSquares", "float"),
		new Column("waflHiPriServiceTimeSum", "float"),
		new Column("waflHiPriServiceTimeCount", "bigint unsigned"),
		new Column("waflLoPriServiceTimeSumOfSquares", "float"),
		new Column("waflLoPriServiceTimeSum", "float"),
		new Column("waflLoPriServiceTimeCount", "bigint unsigned"),
		new Column("waflCpPriServiceTimeSumOfSquares", "float"),
		new Column("waflCpPriServiceTimeSum", "float"),
		new Column("waflCpPriServiceTimeCount", "bigint unsigned"),
		new Column("waflHiPriInterarrivalTimeVariance", "bigint unsigned"),
		new Column("waflHiPriInterarrivalTimeMean", "bigint unsigned"),
		new Column("waflHiPriInterarrivalTimeCount", "bigint unsigned"),
		new Column("waflLoPriInterarrivalTimeVariance", "bigint unsigned"),
		new Column("waflLoPriInterarrivalTimeMean", "bigint unsigned"),
		new Column("waflLoPriInterarrivalTimeCount", "bigint unsigned"),
		new Column("waflCpPriInterarrivalTimeVariance", "bigint unsigned"),
		new Column("waflCpPriInterarrivalTimeMean", "bigint unsigned"),
		new Column("waflCpPriInterarrivalTimeCount", "bigint unsigned"),
		new Column("inWorkloadUtilization", "float"),
		new Column("workloadSumDbladeUtil", "float"),
		new Column("userWorkloadSumDbladeUtil", "float"),
    };

    private static ObjectParser<ResourceHeadRoomCPUMeasurements> sampleParser;

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
    	{new RatioMethod("currentUtilTotal")},
    	{SqlMethod.AVG},
    	{new RatioMethod("currentOps")},
    	{SqlMethod.AVG},
    	{new RatioMethod("waflHiPriServiceTimeCount")},
    	{new RatioMethod("waflHiPriServiceTimeCount")},
    	{SqlMethod.SUM},
    	{new RatioMethod("waflLoPriServiceTimeCount")},
    	{new RatioMethod("waflLoPriServiceTimeCount")},
    	{SqlMethod.SUM},
    	{new RatioMethod("waflCpPriServiceTimeCount")},
    	{new RatioMethod("waflCpPriServiceTimeCount")},
    	{SqlMethod.SUM},
    	{SqlMethod.SUM},
    	{SqlMethod.SUM},
    	{SqlMethod.SUM},
    	{SqlMethod.SUM},
    	{SqlMethod.SUM},
    	{SqlMethod.SUM},
    	{SqlMethod.SUM},
    	{SqlMethod.SUM},
    	{SqlMethod.SUM},
    	{SqlMethod.AVG},
    	{SqlMethod.AVG},
    	{SqlMethod.AVG},

    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
    	new RatioMethod("currentUtilTotal"),
    	SqlMethod.AVG,
    	new RatioMethod("currentOps"),
    	SqlMethod.AVG,
    	new RatioMethod("waflHiPriServiceTimeCount"),
    	new RatioMethod("waflHiPriServiceTimeCount"),
    	SqlMethod.SUM,
    	new RatioMethod("waflLoPriServiceTimeCount"),
    	new RatioMethod("waflLoPriServiceTimeCount"),
    	SqlMethod.SUM,
    	new RatioMethod("waflCpPriServiceTimeCount"),
    	new RatioMethod("waflCpPriServiceTimeCount"),
    	SqlMethod.SUM,
    	SqlMethod.SUM,
    	SqlMethod.SUM,
    	SqlMethod.SUM,
    	SqlMethod.SUM,
    	SqlMethod.SUM,
    	SqlMethod.SUM,
    	SqlMethod.SUM,
    	SqlMethod.SUM,
    	SqlMethod.SUM,
    	SqlMethod.AVG,
    	SqlMethod.AVG,
    	SqlMethod.AVG,

    };


    static {
        sampleParser = new ObjectParser<ResourceHeadRoomCPUMeasurements>() {
            @Override
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, ResourceHeadRoomCPUMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCurrentUtil());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCurrentUtilTotal());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCurrentLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCurrentOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflHiPriServiceTimeSumOfSquares());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflHiPriServiceTimeSum());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflHiPriServiceTimeCount());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflLoPriServiceTimeSumOfSquares());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflLoPriServiceTimeSum());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflLoPriServiceTimeCount());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflCpPriServiceTimeSumOfSqaures());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflCpPriServiceTimeSum());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflCpPriServiceTimeCount());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflHiPriInterArrivalTimeVariance());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflHiPriInterArrivalTimeMean());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflHiPriInterArrivalTimeCount());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflLoPriInterArrivalTimeVariance());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflLoPriInterArrivalTimeMean());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflLoPriInterArrivalTimeCount());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflCpPriInterArrivalTimeVariance());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflCpPriInterArrivalTimeMean());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflCpPriInterArrivalTimeCount());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getInWorkloadUtilization());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWorkloadSumDbladeUtil());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getUserWorkloadSumDbladeUtil());

            }

            @Override
            public ResourceHeadRoomCPUMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new ResourceHeadRoomCPUMeasurements()
                .setCurrentUtil(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setCurrentUtilTotal(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setCurrentLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setCurrentOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setWaflHiPriServiceTimeSumOfSquares(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setWaflHiPriServiceTimeSum(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setWaflHiPriServiceTimeCount(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setWaflLoPriServiceTimeSumOfSquares(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setWaflLoPriServiceTimeSum(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setWaflLoPriServiceTimeCount(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setWaflCpPriServiceTimeSumOfSqaures(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setWaflCpPriServiceTimeSum(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setWaflCpPriServiceTimeCount(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setWaflHiPriInterArrivalTimeVariance(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setWaflHiPriInterArrivalTimeMean(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setWaflHiPriInterArrivalTimeCount(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setWaflLoPriInterArrivalTimeVariance(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setWaflLoPriInterArrivalTimeMean(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setWaflLoPriInterArrivalTimeCount(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setWaflCpPriInterArrivalTimeVariance(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setWaflCpPriInterArrivalTimeMean(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setWaflCpPriInterArrivalTimeCount(PerformanceTableUtils.getLongNullable(resultSet, ++i))
                .setInWorkloadUtilization(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setWorkloadSumDbladeUtil(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setUserWorkloadSumDbladeUtil(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<ResourceHeadRoomCPUMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<ResourceHeadRoomCPUMeasurements> getSampleParser() {
        return sampleParser;
    }
}
