/*
 * Copyright (c) 2015 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.resourceheadroom.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.resourceheadroom.interfaces.data.ResourceHeadRoomAggrMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Remote
public interface ResourceHeadRoomAggrQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//ResourceHeadRoomAggrQueryBean!com.netapp.oci.netapp.performance.resourceheadroom.interfaces.remote.ResourceHeadRoomAggrQueryRemote";

    /**
     * Gets resource headroom aggr samples for period of time
     *
     * @param resourceHeadRoomAggrIds set of resource head room Aggr ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of resource headroom Aggr id to list ordered by time of resource headroom aggr samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<ResourceHeadRoomAggrMeasurements>>> getSamples(Set<Long> resourceHeadRoomAggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * Gets resource headroom aggr samples for period of time
     *
     * @param clusterId                      cluster id
     * @param resourceHeadRoomAggrIds 		 resource head room Aggr Ids 
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of resource headroom Aggr id to list ordered by time of resource headroom aggr samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<ResourceHeadRoomAggrMeasurements>>> getSamples(Long clusterId, Set<Long> resourceHeadRoomAggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets resource headroom aggr for period of time
     *
     * @param clusterId                     cluster id
     * @param resourceHeadRoomAggrId  		resourceHeadRoomAggrId
     * @param fromTime                      beginning of the time period
     * @param toTime                        end of the time period
     * @return list ordered by time of resource headroom aggr samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    List<SampleInfo<ResourceHeadRoomAggrMeasurements>> getSamples(Long clusterId, Long resourceHeadRoomAggrId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given resource headroom aggr Ids (resourceHeadRoomAggrIds). The map returned will contain only resource head room aggr Ids that has a sample.
     * resource headroom aggr ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the resource headroom aggr ids passed.
     *
     * @param clusterId                      cluster id
     * @param resourceHeadRoomAggrIds set of resource headroom aggr ids
     * @return map of resourceHeadRoom Aggr id to latest sample SampleInfo<ResourceHeadRoomAggrMeasurements>. resourceHeadRoom aggr ids with no samples will not be included in this map.
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, SampleInfo<ResourceHeadRoomAggrMeasurements>> getLatestSample(Long clusterId, Set<Long> resourceHeadRoomAggrIds) throws NetappInterfaceException;

    /**
     * Gets QoS workload queue nBlade summaries for resource headroom aggr ids
     *
     * @param resourceHeadRoomAggrIds set of resource headroom aggr Ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of resource headroom Aggr id to summary data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<Summary<ResourceHeadRoomAggrMeasurements>>> getSummaries(Set<Long> resourceHeadRoomAggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets QoS workload queue nBlade summaries for resource headroom aggr ids
     *
     * @param clusterId Cluster Id
     * @param resourceHeadRoomAggrIds set of resource headroom aggr Ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of resource headroom aggr id to summary data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<Summary<ResourceHeadRoomAggrMeasurements>>> getSummaries(Long clusterId, Set<Long> resourceHeadRoomAggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param clusterId
     * @param resourceHeadRoomAggrId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long clusterId, Long resourceHeadRoomAggrIds, Date fromTime, Date toTime) throws NetappInterfaceException;
}
