/*
 * Copyright (c) 2015 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.resourceheadroom.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.resourceheadroom.interfaces.data.ResourceHeadRoomCPUMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Remote
public interface ResourceHeadRoomCPUQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//ResourceHeadRoomCPUQueryBean!com.netapp.oci.netapp.performance.resourceheadroom.interfaces.remote.ResourceHeadRoomCPUQueryRemote";

    /**
     * Gets resource headroom CPU samples for period of time
     *
     * @param resourceHeadRoomCPUIds set of resource head room CPU ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of resource headroom CPU id to list ordered by time of resource headroom CPU samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<ResourceHeadRoomCPUMeasurements>>> getSamples(Set<Long> resourceHeadRoomCPUIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * Gets resource headroom CPU samples for period of time
     *
     * @param clusterId                      cluster id
     * @param resourceHeadRoomCPUIds 		 resource head room CPU Ids 
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of resource headroom CPU id to list ordered by time of resource headroom CPU samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<ResourceHeadRoomCPUMeasurements>>> getSamples(Long clusterId, Set<Long> resourceHeadRoomCPUIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets resource headroom CPU for period of time
     *
     * @param clusterId                     cluster id
     * @param resourceHeadRoomCPUId  		resourceHeadRoomCPUId
     * @param fromTime                      beginning of the time period
     * @param toTime                        end of the time period
     * @return list ordered by time of resource headroom cpu samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    List<SampleInfo<ResourceHeadRoomCPUMeasurements>> getSamples(Long clusterId, Long resourceHeadRoomCPUId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given resource headroom CPU Ids (resourceHeadRoomCPUIds). The map returned will contain only resource head room CPU Ids that has a sample.
     * resource headroom CPU ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the resource headroom CPU ids passed.
     *
     * @param clusterId                      cluster id
     * @param resourceHeadRoomCPUIds set of resource headroom CPU ids
     * @return map of resourceHeadRoom CPU id to latest sample SampleInfo<ResourceHeadRoomCPUMeasurements>. resourceHeadRoom CPU ids with no samples will not be included in this map.	
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, SampleInfo<ResourceHeadRoomCPUMeasurements>> getLatestSample(Long clusterId, Set<Long> resourceHeadRoomCPUIds) throws NetappInterfaceException;

    /**
     * Gets resource headroom CPU summaries for resource headroom CPU ids
     *
     * @param resourceHeadRoomCPUIds set of resource headroom CPU Ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of resource headroom CPU id to summary data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<Summary<ResourceHeadRoomCPUMeasurements>>> getSummaries(Set<Long> resourceHeadRoomCPUIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets resource headroom CPU summaries for resource headroom CPU ids
     *
     * @param clusterId Cluster Id
     * @param resourceHeadRoomCPUIds set of resource headroom CPU Ids
     * @param fromTime                       beginning of the time period
     * @param toTime                         end of the time period
     * @return map of resource headroom CPU id to summary data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<Summary<ResourceHeadRoomCPUMeasurements>>> getSummaries(Long clusterId, Set<Long> resourceHeadRoomCPUIds, Date fromTime, Date toTime) throws NetappInterfaceException;


    /**
     * returns true if samples exist over the time period
     *
     * @param clusterId
     * @param resourceHeadRoomCPUId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long clusterId, Long resourceHeadRoomCPUIds, Date fromTime, Date toTime) throws NetappInterfaceException;
}
