package com.netapp.oci.netapp.performance.shared;

import com.netapp.oci.netapp.Param;
import com.netapp.oci.netapp.performance.PerformanceConstants;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.framework.performance.tables.*;
import java.sql.Connection;
import java.util.regex.Pattern;
import org.slf4j.Logger;

/**
 * @author Anton Sergeev
 */
public class NonPartitionedPerformanceComponentsFactory<C, S> extends AbstractPerformanceComponentsFactory<C, S> {
    private static final Logger logger = org.slf4j.LoggerFactory.getLogger(NonPartitionedPerformanceComponentsFactory.class);

    //Sample Only
    public NonPartitionedPerformanceComponentsFactory(String packageName, Column[] sampleColumns, ObjectParser<C> sampleParser) {
        super(packageName, sampleColumns, sampleParser);
    }

    //Samples & Hourly Summary
    public NonPartitionedPerformanceComponentsFactory(String packageName, Column[] sampleColumns, Column[] summaryColumns,
                                                   ObjectParser<C> sampleParser, ObjectParser<S> houlySummaryParser,
                                                   SqlMethod[] summaryCommonMethods, SqlMethod[][] sampleToSummaryMethods) {
        super(packageName, sampleColumns, summaryColumns, sampleParser, houlySummaryParser, summaryCommonMethods,
                sampleToSummaryMethods);
    }

    //Samples, Hourly Summary & Daily Summary.
    public NonPartitionedPerformanceComponentsFactory(String packageName, Column[] sampleColumns,
                                                      Column[] hourlySummaryColumns, Column[] dailySummaryColumns,
                                                      ObjectParser<C> sampleParser,
                                                      ObjectParser<S> hourlySummaryParser, ObjectParser<C> dailySummaryParser,
                                                      SqlMethod[] hourlySummaryCommonMethods,
                                                      SqlMethod[][] sampleToHourlySummaryMethods,
                                                      SqlMethod[] dailySummaryMethods) {
        super(packageName, sampleColumns, hourlySummaryColumns, dailySummaryColumns, sampleParser, hourlySummaryParser,
                dailySummaryParser, hourlySummaryCommonMethods, sampleToHourlySummaryMethods, dailySummaryMethods);
    }


    public SampleTable<C> getSampleTable(Connection connection, Long containerId) {
        int maxPartitions = Pattern.matches(PerformanceConstants.SAMPLE_RAW_PACKAGE_PATTERN, packageName) ? PerformanceConstants
                .getTablesLifeSpan(Param.Key.SAMPLE_RAW_TABLES_LIFESPAN_DAYS) : PerformanceConstants
                .getTablesLifeSpan(Param.Key.SAMPLE_TABLES_LIFESPAN_DAYS);
        if (logger.isDebugEnabled()) {        
            logger.debug("getSampleTable: packageName = " + packageName + ", maxPartitions = " + maxPartitions);
        }
        SampleTable<C> table = new PartitionedSampleTable<C>(connection, sampleParser, getPackagePrefix(),
                sampleColumns, maxPartitions);
        table.setDatabase(PerformanceConstants.NETAPP_PERFORMANCE_DATABASE);
        return table;
    }

    public SummaryTable<S> getHourlySummaryTable(Connection connection, Long containerId) {
        SummaryTable<S> table = new InsertSelectSummaryTable<>(connection, getPackagePrefix(), hourlySummaryColumns,
                hourlySummaryParser);
        table.setDatabase(PerformanceConstants.NETAPP_PERFORMANCE_DATABASE);
        return table;
    }

    public DailySummaryTable<C> getDailySummaryTable(Connection connection, Long containerId) {
        DailySummaryTable<C> dailySummaryTable = null;
        if (dailySummaryColumns != null) {
            dailySummaryTable = new DailySummaryTable<>(connection, getPackagePrefix(), dailySummaryColumns,
                    dailySummaryMethods, dailySummaryParser);
        }
        return dailySummaryTable;
    }

    public String getPackagePrefix() {
        return packageName;
    }

    public String getPerformancePackageId(Long containerId) {
        return getPackagePrefix();
    }
}
