package com.netapp.oci.netapp.performance.shared;

import java.sql.Connection;
import java.util.regex.Pattern;

import com.netapp.oci.platform.framework.performance.tables.*;
import org.slf4j.Logger;
import com.netapp.oci.netapp.Param;
import com.netapp.oci.netapp.performance.PerformanceConstants;
import com.netapp.oci.platform.common.table.ObjectParser;

/**
 * @author Anton Sergeev
 */
public class PartitionedPerformanceComponentsFactory<C, S>  extends AbstractPerformanceComponentsFactory<C, S> {
    private static final Logger logger =org.slf4j.LoggerFactory.getLogger(PartitionedPerformanceComponentsFactory.class);
    
    public PartitionedPerformanceComponentsFactory(String packageName, Column[] sampleColumns, Column[] summaryColumns,
                                                   ObjectParser<C> sampleParser, ObjectParser<S> summaryParser,
                                                   SqlMethod[] summaryCommonMethods, SqlMethod[][] sampleToSummaryMethods) {
        super(packageName, sampleColumns, summaryColumns, sampleParser, summaryParser, summaryCommonMethods, sampleToSummaryMethods);
    }
    
    public PartitionedPerformanceComponentsFactory(String packageName, Column[] sampleColumns, ObjectParser<C> sampleParser) {
    	super(packageName, sampleColumns, sampleParser);
    }

    public PartitionedPerformanceComponentsFactory(String packageName, Column[] sampleColumns,
                                                   Column[] hourlySummaryColumns, Column[] dailySummaryColumns,
                                                   ObjectParser<C> sampleParser, ObjectParser<S> hourlySummaryParser,
                                                   ObjectParser<C> dailySummaryParser,
                                                   SqlMethod[] hourlySummaryCommonMethods,
                                                   SqlMethod[][] sampleToHourlySummaryMethods,
                                                   SqlMethod[] dailySummaryMethods) {
        super(packageName, sampleColumns, hourlySummaryColumns, dailySummaryColumns, sampleParser, hourlySummaryParser,
                dailySummaryParser, hourlySummaryCommonMethods, sampleToHourlySummaryMethods, dailySummaryMethods);
    }

    public SampleTable<C> getSampleTable(Connection connection, Long containerId) {
        int maxPartitions = Pattern.matches(PerformanceConstants.SAMPLE_RAW_PACKAGE_PATTERN, packageName) ? PerformanceConstants
                .getTablesLifeSpan(Param.Key.SAMPLE_RAW_TABLES_LIFESPAN_DAYS) : PerformanceConstants
                .getTablesLifeSpan(Param.Key.SAMPLE_TABLES_LIFESPAN_DAYS);
        if (logger.isDebugEnabled()) {
            logger.debug("getSampleTable: packageName = " + packageName + ", maxPartitions = " + maxPartitions);
        }
        SampleTable<C> table = new PartitionedSampleTable<C>(connection, sampleParser,
                getPackagePrefix() + containerId, sampleColumns, maxPartitions);
        table.setDatabase(PerformanceConstants.NETAPP_PERFORMANCE_DATABASE);
        return table;
    }

    public SummaryTable<S> getHourlySummaryTable(Connection connection, Long containerId) {
        SummaryTable<S> table = new InsertSelectSummaryTable<S>(connection, getPackagePrefix() + containerId,
                hourlySummaryColumns, hourlySummaryParser);
        table.setDatabase(PerformanceConstants.NETAPP_PERFORMANCE_DATABASE);
        return table;
    }

    public DailySummaryTable<C> getDailySummaryTable(Connection connection, Long containerId) {
        DailySummaryTable<C> dailySummaryTable = null;
        if (dailySummaryColumns != null) {
            dailySummaryTable = new DailySummaryTable<>(connection, getPackagePrefix() + containerId, dailySummaryColumns,
                    dailySummaryMethods, dailySummaryParser);
        }
        return dailySummaryTable;
    }

    public String getPackagePrefix() {
        return packageName + "_";
    }

    public String getPerformancePackageId(Long containerId) {
        return getPackagePrefix() + containerId;
    }
}
