package com.netapp.oci.netapp.performance.shared;

import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

/**
 * @author Anton Sergeev
 */
public class PerformanceTableUtils {
    private static double DOUBLE_NULL_VALUE = Double.MIN_VALUE;
    private static float FLOAT_NULL_VALUE = Float.MIN_VALUE;
    private static int INT_NULL_VALUE = Integer.MIN_VALUE;
    private static long LONG_NULL_VALUE = Long.MIN_VALUE;
    private static short SHORT_NULL_VALUE = Short.MIN_VALUE;
    public static BigInteger NULL_BIG_INT = BigInteger.valueOf(Long.MIN_VALUE);

    public static double getDoubleNullable(ResultSet resultSet, int fieldIndex) throws SQLException {
        double doubleValue = resultSet.getDouble(fieldIndex);
        if (resultSet.wasNull()) {
            doubleValue = DOUBLE_NULL_VALUE;
        }

        return doubleValue;
    }

    public static void setNextNullable(PreparedStatementWrapper preparedStatement, double doubleValue) throws TableException {
        if (doubleValue == DOUBLE_NULL_VALUE) {
            preparedStatement.setNextNull(Types.DOUBLE);
        } else {
            preparedStatement.setNext(doubleValue);
        }
    }
    
    public static int getIntNullable(ResultSet resultSet, int fieldIndex) throws SQLException {
        int intValue = resultSet.getInt(fieldIndex);
        if (resultSet.wasNull()) {
            intValue = INT_NULL_VALUE;
        }

        return intValue;
    }
    public static long getLongNullable(ResultSet resultSet, int fieldIndex) throws SQLException {
        long longValue = resultSet.getLong(fieldIndex);
        if (resultSet.wasNull()) {
            longValue = LONG_NULL_VALUE;
        }

        return longValue;
    }

    public static void setNextNullable(PreparedStatementWrapper preparedStatement, BigInteger bigIntValue) throws TableException {
    	if(bigIntValue == null || NULL_BIG_INT.equals(bigIntValue)){
    		preparedStatement.setNextNull(Types.BIGINT);
    	}
    	else{
    		preparedStatement.setNext(new BigDecimal(bigIntValue));
    	}
    }

    public static BigInteger getBigIntegerNullable(ResultSet resultSet, int fieldIndex) throws SQLException {
        return resultSet.getBigDecimal(fieldIndex).toBigInteger();
    }

    public static float getFloatNullable(ResultSet resultSet, int fieldIndex) throws SQLException {
        float floatValue = resultSet.getFloat(fieldIndex);
        if (resultSet.wasNull()) {
            floatValue = FLOAT_NULL_VALUE;
        }

        return floatValue;
    }

    public static short getShortNullable(ResultSet resultSet, int fieldIndex) throws SQLException {
        short shortValue = resultSet.getShort(fieldIndex);
        if (resultSet.wasNull()) {
            shortValue = SHORT_NULL_VALUE;
        }

        return shortValue;
    }

    public static String getStringNullable(ResultSet resultSet, int fieldIndex) throws SQLException {
        String stringValue = resultSet.getString(fieldIndex);
        if (resultSet.wasNull()) {
            stringValue = null;
        }

        return stringValue;
    }
    
    public static void setNextNullable(PreparedStatementWrapper preparedStatement, short shortValue) throws TableException {
        if (shortValue == SHORT_NULL_VALUE) {
            preparedStatement.setNextNull(Types.INTEGER);
        } else {
            preparedStatement.setNext(shortValue);
        }
    }

    public static void setNextNullable(PreparedStatementWrapper preparedStatement, int intValue) throws TableException {
        if (intValue == INT_NULL_VALUE) {
            preparedStatement.setNextNull(Types.INTEGER);
        } else {
            preparedStatement.setNext(intValue);
        }
    }

    public static void setNextNullable(PreparedStatementWrapper preparedStatement, long longValue) throws TableException {
        if (longValue == LONG_NULL_VALUE) {
            preparedStatement.setNextNull(Types.INTEGER);
        } else {
            preparedStatement.setNext(longValue);
        }
    }

    public static void setNextNullable(PreparedStatementWrapper preparedStatement, float floatValue) throws TableException {
        if (floatValue == FLOAT_NULL_VALUE) {
            preparedStatement.setNextNull(Types.FLOAT);
        } else {
            preparedStatement.setNext(floatValue);
        }
    }
    
    public static void setNextNullable(PreparedStatementWrapper preparedStatement, String stringValue) throws TableException {
        if (stringValue == null) {
            preparedStatement.setNextNull(Types.VARCHAR);
        } else {
            preparedStatement.setNext(stringValue);
        }
    }
    
}
