/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.shared;

import java.sql.Connection;

import com.netapp.oci.platform.framework.performance.tables.DailySummaryTable;
import org.slf4j.Logger;

import com.netapp.oci.netapp.Param;
import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.framework.performance.PersistenceStrategy;
import com.netapp.oci.platform.framework.performance.SampleFacade;
import com.netapp.oci.platform.framework.performance.SummaryFacade;
import com.netapp.oci.platform.framework.performance.TableManagement;
import com.netapp.oci.platform.framework.performance.tables.CurrentSampleTable;
import com.netapp.oci.platform.framework.performance.tables.SampleTable;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;
import com.netapp.oci.platform.framework.performance.util.AbstractPerformancePackageFactory;
import com.netapp.oci.platform.framework.performance.util.AbstractPersistenceStrategy;
import com.netapp.oci.platform.framework.performance.util.AbstractSampleFacade;
import com.netapp.oci.platform.framework.performance.util.AbstractTableManagement;

/**
 * 
 * Class to query and persist sample tables
 *
 */
public class SamplePerformancePackageFactory<C, S> extends AbstractPerformancePackageFactory<C, S>{
	
    private static final Logger logger =org.slf4j.LoggerFactory.getLogger(SamplePerformancePackageFactory.class);
    
    private AbstractPerformanceComponentsFactory<C, S> factory;

    public SamplePerformancePackageFactory(Long sampleContainerId, AbstractPerformanceComponentsFactory<C, S> factory) {
        this(null, sampleContainerId, factory);
    }

    public SamplePerformancePackageFactory(String sampleContainerKey, Long sampleContainerId, AbstractPerformanceComponentsFactory<C, S> factory) {
        super(sampleContainerKey, sampleContainerId,org.slf4j.LoggerFactory.getLogger("com.netapp.oci.netapp.performance." + factory.getPackageName() + "."
                + String.valueOf(sampleContainerId)));
        this.factory = factory;
    }

    public SampleFacade<C> getSampleFacade() {
        return new AbstractSampleFacade<C>() {
            @Override
            protected SqlMethod[] getGroupByTimeSqlMethod() {
            	if(factory.getHourlySummaryCommonMethods() != null){
                    return factory.getHourlySummaryCommonMethods();
            	}
            	else{
            		logger.error("There is no group methods facade for " + "com.netapp.oci.netapp.performance." + factory.getPackageName());
            		return null;
            	}
            }

            @Override
            protected SampleTable<C> getSampleTable(Connection connection) {
                return factory.getSampleTable(connection, getSampleContainerId());
            }
        };
    }

	@Override
	public SummaryFacade<S> getSummaryFacade() {
		logger.error("There is no summary for " + "com.netapp.oci.netapp.performance." + factory.getPackageName());
		return null;
	}

    public PersistenceStrategy<C> getPersistenceStrategy() {
    	return new AbstractPersistenceStrategy<C>(getLogger(), Param.INSTANCE,
    			getSampleContainerKey(), getSampleContainerId()) {

    		@Override
    		public void prepare(Connection connection, Long sampleTime) throws PerformanceException {

    		}

    		@Override
    		public void complete(Connection connection, Long sampleTime) throws PerformanceException {

    		}

    		@Override
    		protected SampleTable<C> getSampleTable(Connection connection) {
    			return factory.getSampleTable(connection, getSampleContainerId());
    		}

            @Override
            protected DailySummaryTable<C> getDailySummaryTable(Connection connection) {
                return factory.getDailySummaryTable(connection, getSampleContainerId());
            }

            @Override
    		protected CurrentSampleTable<C> getCurrentSampleTable(Connection connection) {
    			return factory.getCurrentSampleTable(connection, getSampleContainerKey(), getSampleContainerId());
    		}

    		public String getPerformancePackageId() {
    			return factory.getPerformancePackageId(getSampleContainerId());
    		}
    	};
    }

    public TableManagement getTableManagement() {
    	return new AbstractTableManagement<C>(getLogger()) {
            @Override
            protected SampleTable<C> getSampleTable(Connection connection) {
                return factory.getSampleTable(connection, getSampleContainerId());
            }
        };
    }
}
