/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.shared;

import java.sql.Connection;

import com.netapp.oci.platform.framework.performance.tables.*;
import org.slf4j.Logger;

import com.netapp.oci.netapp.Param;
import com.netapp.oci.netapp.performance.PerformanceConstants;
import com.netapp.oci.platform.framework.performance.PersistenceStrategy;
import com.netapp.oci.platform.framework.performance.SampleFacade;
import com.netapp.oci.platform.framework.performance.SummaryFacade;
import com.netapp.oci.platform.framework.performance.TableManagement;
import com.netapp.oci.platform.framework.performance.util.AbstractPerformancePackageFactory;
import com.netapp.oci.platform.framework.performance.util.AbstractSampleFacade;
import com.netapp.oci.platform.framework.performance.util.AbstractSummaryFacade;
import com.netapp.oci.platform.framework.performance.util.AbstractSummaryTableManagement;
import com.netapp.oci.platform.framework.performance.util.InsertSelectSummarizePersistenceStrategy;
import com.netapp.oci.platform.framework.performance.util.SampleTablePreparator;
import com.onaro.commons.types.Period;

/**
 * @author Anton Sergeev
 */
public class SimplePerformancePackageFactory<C, S> extends AbstractPerformancePackageFactory<C, S> {
    private AbstractPerformanceComponentsFactory<C, S> factory;

    public SimplePerformancePackageFactory(Long sampleContainerId, AbstractPerformanceComponentsFactory<C, S> factory) {
        this(null, sampleContainerId, factory);
    }

    public SimplePerformancePackageFactory(String sampleContainerKey, Long sampleContainerId, AbstractPerformanceComponentsFactory<C, S> factory) {
        super(sampleContainerKey, sampleContainerId,org.slf4j.LoggerFactory.getLogger("com.netapp.oci.netapp.performance." + factory.getPackageName() + "."
                + String.valueOf(sampleContainerId)));
        this.factory = factory;
    }

    public SampleFacade<C> getSampleFacade() {
        return new AbstractSampleFacade<C>() {
            @Override
            protected SqlMethod[] getGroupByTimeSqlMethod() {
                return factory.getHourlySummaryCommonMethods();
            }

            @Override
            protected SampleTable<C> getSampleTable(Connection connection) {
                return factory.getSampleTable(connection, getSampleContainerId());
            }
        };
    }

    public SummaryFacade<S> getSummaryFacade() {
        return new AbstractSummaryFacade<C, S>() {
            @Override
            protected SummaryTable<S> getSummaryTable(Connection connection) {
                return factory.getHourlySummaryTable(connection, getSampleContainerId());
            }

            @Override
            protected SampleTable<C> getSampleTable(Connection connection) {
                return factory.getSampleTable(connection, getSampleContainerId());
            }
        };
    }

    public PersistenceStrategy<C> getPersistenceStrategy() {
        return new InsertSelectSummarizePersistenceStrategy<C, S>(getLogger(), Param.INSTANCE,
                getSampleContainerKey(), getSampleContainerId()) {
            @Override
            protected long getTablesLifeSpan() {
                return PerformanceConstants.getTablesLifeSpan(Param.Key.SUMMARY_TABLES_LIFESPAN_DAYS)
                        * Period.DAY.milliseconds();
            }

            @Override
            protected SqlMethod[][] getSummarizationMethods() {
                return factory.getSampleToHourlySummaryMethods();
            }

            @Override
            protected SampleTablePreparator getSampleTablePreparator() {
                return factory.getSampleTablePreparator(getLogger());
            }

            @Override
            protected SummaryTable<S> getSummaryTable(Connection connection) {
                return factory.getHourlySummaryTable(connection, getSampleContainerId());
            }

            @Override
            protected SampleTable<C> getSampleTable(Connection connection) {
                return factory.getSampleTable(connection, getSampleContainerId());
            }

            @Override
            protected DailySummaryTable<C> getDailySummaryTable(Connection connection) {
                return factory.getDailySummaryTable(connection, getSampleContainerId());
            }

            @Override
            protected CurrentSampleTable<C> getCurrentSampleTable(Connection connection) {
                return factory.getCurrentSampleTable(connection, getSampleContainerKey(), getSampleContainerId());
            }

            public String getPerformancePackageId() {
                return factory.getPerformancePackageId(getSampleContainerId());
            }
        };
    }

    public TableManagement getTableManagement() {
        return new AbstractSummaryTableManagement<C, S>(getLogger()) {
            @Override
            protected SampleTable<C> getSampleTable(Connection connection) {
                return factory.getSampleTable(connection, getSampleContainerId());
            }

            @Override
            protected SummaryTable<S> getSummaryTable(Connection connection) {
                return factory.getHourlySummaryTable(connection, getSampleContainerId());
            }

            protected DailySummaryTable<C> getDailySummaryTable(Connection connection) {
                return factory.getDailySummaryTable(connection, getSampleContainerId());
            }

        };
    }

}
