/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.shared.interfaces.impl;

import com.netapp.oci.netapp.Param;
import com.netapp.oci.netapp.db.NetAppPerformance;
import com.netapp.oci.netapp.performance.PerformanceConstants;
import com.netapp.oci.platform.common.InternalServerException;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.onaro.commons.types.Period;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.Job;

import javax.annotation.security.RunAs;
import javax.sql.DataSource;

import java.sql.Connection;

/**
 * @author Anton Sergeev
 */
@RunAs("admin")
@DisallowConcurrentExecution
public class NetappPerformanceDailyJob implements Job {
    
    public static final String CRON_TRIGGER = "0 15 2 * * ?";
    public static final String JOB_NAME = "NetappPerformanceDailyJob";

    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        Connection connection = null;

        try {
            DataSource dataSource = (DataSource) jobExecutionContext.getJobDetail().getJobDataMap().get("dataSourceOnaroPool");
            connection = ConnectionHelper.newConnection(dataSource);
        	
            NetAppPerformance.INSTANCE.ensureUpdateHistoryLifeSpan(
                    connection,
                    System.currentTimeMillis(),
                    PerformanceConstants.getTablesLifeSpan(Param.Key.UPDATE_HISTORY_LIFESPAN_DAYS)
                            * Period.DAY.milliseconds());
        }
        catch (InternalServerException e) {
            throw new JobExecutionException(e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }

}
