/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.shared.interfaces.impl;

import com.netapp.oci.netapp.Param;
import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.shared.interfaces.local.NetappPerformanceDatasourceRegistry;
import com.netapp.oci.netapp.performance.shared.interfaces.remote.NetappPerformanceMaintenanceRemote;
import com.netapp.oci.platform.common.ServerConstant;
import com.netapp.oci.platform.common.param.ParamException;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.common.table.UpdateTable;
import com.netapp.oci.platform.framework.performance.util.AbstractTableManagement;
import com.netapp.oci.platform.framework.performance.util.PersistenceState;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.sql.DataSource;
import java.sql.Connection;

/**
 * @author Anton Sergeev
 */
@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@RolesAllowed({"admin"})
public class NetappPerformanceMaintenanceBean implements NetappPerformanceMaintenanceRemote {
    @Resource(name = "jdbc/OnaroDataSource", mappedName = "java:/onaro_pool", type = javax.sql.DataSource.class, authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    /**
     * use this bean to register reporting datasource
     */
    @EJB
    private NetappPerformanceDatasourceRegistry datasourceRegistry;

    public void clearAll() throws NetappInterfaceException {
        Connection con = null;
        try {
            con = dataSource.getConnection();
            UpdateTable updateHistoryTable = new UpdateTable(con, "netapp_performance", "update_history");
            updateHistoryTable.deleteAll();
        } catch (Exception ex) {
            throw new NetappInterfaceException("Failed to clear all", ex);
        } finally {
            ConnectionHelper.close(con);
        }

        AbstractTableManagement.clearCaches();
        PersistenceState.clearAll();
        datasourceRegistry.clearRegistery();
    }

    public void enablePerformanceCurrentSampleUpdate() throws NetappInterfaceException {
         configurePerformanceCurrentSampleUpdate(true);
     }

     public void disablePerformanceCurrentSampleUpdate() throws NetappInterfaceException {
         configurePerformanceCurrentSampleUpdate(false);
     }

     private void configurePerformanceCurrentSampleUpdate(boolean flag) throws NetappInterfaceException {
         Connection connection = null;
         try{
             connection = ConnectionHelper.newConnection(dataSource);
             Param.INSTANCE.set(connection, Param.Key.PERFORMANCE_UPDATE_CURRENT_SAMPLE.name(), flag? "1": "0");
         } catch (ParamException e) {
             throw new NetappInterfaceException("Failed to enable/disable current sample update", e);
         } finally{
             ConnectionHelper.close(connection);

         }
     }
}
