package com.netapp.oci.netapp.performance.shared.interfaces.impl;

import java.util.Map;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Singleton;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.interceptor.Interceptors;


import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.shared.SampleListenerRegistry;
import com.netapp.oci.netapp.performance.shared.interfaces.local.NetappPerformanceSampleListenerRegistry;
import com.netapp.oci.netapp.performance.shared.interfaces.local.PerformanceSampleListener;
import com.netapp.oci.platform.common.ServerInterfaceInterceptor;

/**
 * EJB local interface implementation for register/unregister/get the sample listeners.
 * 
 * @author Cheng Sun
 *
 */
@Singleton
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@RolesAllowed({"guest", "user", "admin", "scheduler", "acquisition"})
@Interceptors(ServerInterfaceInterceptor.class)
public class NetappPerformanceSampleListenerRegistryBean implements NetappPerformanceSampleListenerRegistry{

	private final static SampleListenerRegistry sampleListenerRegistry = new SampleListenerRegistry();

	@Override
	public void register(SampleType sampleType, String listerName, PerformanceSampleListener listener) throws NetappInterfaceException{
		sampleListenerRegistry.register(sampleType, listerName, listener);
	}

	@Override
	public void unregister(SampleType sampleType, String listerName) throws NetappInterfaceException{
		sampleListenerRegistry.unregister(sampleType, listerName);
	}
	
	@Override
	public Map<String, PerformanceSampleListener> getSampleListeners(SampleType sampleType) {
		return sampleListenerRegistry.getListeners(sampleType);
	}
	
}
