/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.shared.interfaces.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.security.RunAs;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageListener;

/**
 * @author Anton Sergeev
 */
@MessageDriven(activationConfig = {
        @ActivationConfigProperty(propertyName = "destination", propertyValue = "java:/topic/NotificationTopic"),
        @ActivationConfigProperty(propertyName = "acknowledgeMode", propertyValue = "Auto-acknowledge"),
        @ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Topic")})
@RunAs("admin")
public class PerformanceSampleCompleteListenerMDB implements MessageListener 
{
  private static final Logger logger = LoggerFactory.getLogger( PerformanceSampleCompleteListenerMDB.class );

  public void onMessage(Message inMessage)
  {
    if( logger.isDebugEnabled() ) logger.debug( "MESSAGE BEAN: Message received: " + inMessage );

    try
    {
      //Read message from the topic
      if( !(inMessage instanceof MapMessage) )
        logger.error( "Message of wrong type: " + inMessage.getClass().getName() );
      else
      {
        //Parse the message to see if it is this should trigger a performance summary
        MapMessage msg = (MapMessage) inMessage;
        String messageType = msg.getString("MessageType");
        String notificationType = msg.getString( "NotificationType" );
        String packageName = msg.getString( "PackageName" );
          if( messageType.equals( "PACKAGE_NOTIFICATION" ) && (notificationType.equals( "PACKAGE_COMPLETED" ) ||notificationType.equals( "PACKAGE_HISTORICAL_COMPLETED" )) && packageName.equals( "netappstorageperformance" ) )
        {
          long datasourceId = msg.getLong( "DatasourceID" );
          final long sampleTime = msg.getLong("SampleTime");
          new PerformanceSummaryJobStart( datasourceId, sampleTime).execute();
        }
      }
    }
    catch( Throwable t )
    {
      logger.error( "Error process message", t );
    }
  }
}



