/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.shared.interfaces.local;

import javax.ejb.Local;

/**
 * In every performance project (package) there can be only one datasource reporting about a given object at a given time.
 * for a given project the server will hold the following data
 * <ol start=0>
 * <li>	What is the datasource Id reporting on this Object (for example the storage in the storage performance project)
 * <li>	What was the last-report-time for this datasource report on this Object
 * </ol>
 * <p/>
 * If a different datasource trying to report on a specific storage, the sample should be ignore
 * If the last-report-time is greater then leaseTime the server will point to the new datasource as the 'reporting' datasource.
 * @author Anton Sergeev
 */
@Local
public interface NetappPerformanceDatasourceRegistry {
    String JNDI_NAME = "java:global/mega-app/server-core/NetappPerformanceDatasourceRegistryBean!com.netapp.oci.netapp.performance.shared.interfaces.local.NetappPerformanceDatasourceRegistry";

    /**
     * @param pckg         the packge that use this (in the storage performance project, it is "Storage")
     * @param key          the Object key (in the storage performance project, it is the storageIp - storage IP)
     * @param datasourceId the datasource Id
     * @param leaseTime    the time for datasource to be resgister in the in the registry (in milliseconds)
     * @return true if the datasource Id is registered for the given Object (in the given package)
     */
    boolean isRegistered(String pckg, String key, Long datasourceId, long leaseTime);

    /**
     * @param pckg         the packge that use this (in the storage performance project, it is "Storage")
     * @param key          the Object key (in the storage performance project, it is the storageIp - storage IP)
     * @param datasourceId the datasource Id
     * @param leaseTime    the time for datasource to be resgister in the in the registry (in milliseconds)
     * @return true if the register was successful, retuern false if failed to register,
     *         it failes if there is an additional datasource reporting on this Object (in the given package) in the last leaseTime
     */
    boolean register(String pckg, String key, Long datasourceId, long leaseTime);

    /**
     * @param pckg      the packge that use this (in the storage performance project, it is "Storage")
     * @param key       the Object key (in the storage performance project, it is the storageIp - storage IP)
     * @param leaseTime the time for datasource to be resgister in the in the registry (in milliseconds)
     * @return the datasource Id reporting on that object, if none exist return null
     */
    Long getRegisterDatasourceId(String pckg, String key, long leaseTime);

    /**
     * @param pckg         the package that uses this registry (in the storage performance project, it is "Storage")
     * @param key          the Object key (in the storage performance project, it is the storageIp - storage IP)
     * @param datasourceId the datasource Id
     * @return get the first registered time for the reporting datasource id, if none return -1;
     */
    long getFirstReportTime(String pckg, String key, Long datasourceId);

    /**
     * remove all the registr for all package
     */
    void clearRegistery();

    /**
     * remove all the registry for all package
     */
    void unregisterPackage(String pckg);
}
