/*
 * $Id$
 *
 * Copyright (c) 2017 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.oci.netapp.performance.shared.interfaces.local;

import java.util.Collection;
import java.util.Map;

import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateMeasurements;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverMeasurements;
import com.netapp.oci.netapp.performance.cluster.interfaces.data.ClusterMeasurements;
import com.netapp.oci.netapp.performance.disk.interfaces.data.DiskMeasurements;
import com.netapp.oci.netapp.performance.extcacheobj.interfaces.data.ExtCacheObjMeasurements;
import com.netapp.oci.netapp.performance.nvmffclif.interfaces.data.NvmfFcLifMeasurements;
import com.netapp.oci.netapp.performance.nvmffclif.interfaces.data.NvmfFcLifVserverMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverMeasurements;
import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifVserverMeasurements;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunMeasurements;
import com.netapp.oci.netapp.performance.namespace.interfaces.data.NamespaceMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverMeasurements;
import com.netapp.oci.netapp.performance.networkport.interfaces.data.NetworkPortMeasurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.NfsMeasurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv3Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv41Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv4Measurements;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicMeasurements;
import com.netapp.oci.netapp.performance.node.interfaces.data.NodeMeasurements;
import com.netapp.oci.netapp.performance.opmheadroom.interfaces.data.OpmHeadRoomAggrMeasurements;
import com.netapp.oci.netapp.performance.opmheadroom.interfaces.data.OpmHeadRoomCPUMeasurements;
import com.netapp.oci.netapp.performance.processor.interfaces.data.ProcessorMeasurements;
import com.netapp.oci.netapp.performance.qosservicecenter.interfaces.data.QosServiceCenterMeasurements;
import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosFileWorkloadMeasurements;
import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosLunWorkloadMeasurements;
import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosVolumeWorkloadMeasurements;
import com.netapp.oci.netapp.performance.qosworkloaddblade.interfaces.data.QosWorkloadQueueDBladeMeasurements;
import com.netapp.oci.netapp.performance.qosworkloaddetail.interfaces.data.QosWorkloadDetailMeasurements;
import com.netapp.oci.netapp.performance.qosworkloadnblade.interfaces.data.QosWorkloadQueueNBladeMeasurements;
import com.netapp.oci.netapp.performance.resourceheadroom.interfaces.data.ResourceHeadRoomAggrMeasurements;
import com.netapp.oci.netapp.performance.resourceheadroom.interfaces.data.ResourceHeadRoomCPUMeasurements;
import com.netapp.oci.netapp.performance.vserver.interfaces.data.VserverMeasurements;
import com.netapp.oci.netapp.performance.wafl.interfaces.data.WaflMeasurements;
import com.netapp.oci.platform.framework.performance.data.Sample;

/**
 * Defines the interfaces that a sample Collectionener needs to implement.
 *
 * @author Cheng Sun
 *
 */
public interface PerformanceSampleListener {

	void processAggr(Map<Long, Collection<Sample<AggregateMeasurements>>> samples, boolean isHistorical);

	void processCifsVserver(Map<Long, Collection<Sample<CifsVserverMeasurements>>> samples, boolean isHistorical);

	void processDisk(Map<Long, Collection<Sample<DiskMeasurements>>> samples, boolean isHistorical);

	void processFcpLif(Map<Long, Collection<Sample<FcpLifMeasurements>>> samples, boolean isHistorical);

	void processFcpLifVserver(Map<Long, Collection<Sample<FcpLifVserverMeasurements>>> samples, boolean isHistorical);

	void processFcpPort(Map<Long, Collection<Sample<FcpPortMeasurements>>> samples, boolean isHistorical);

	void processIscsiLif(Map<Long, Collection<Sample<IscsiLifMeasurements>>> samples, boolean isHistorical);

	void processIscsiLifVserver(Map<Long, Collection<Sample<IscsiLifVserverMeasurements>>> samples, boolean isHistorical);

	void processLun(Map<Long, Collection<Sample<LunMeasurements>>> samples, boolean isHistorical);

	void processNamespace(Map<Long, Collection<Sample<NamespaceMeasurements>>> samples, boolean isHistorical);

	void processNetworkLif(Map<Long, Collection<Sample<NetworkLifMeasurements>>> samples, boolean isHistorical);

	void processNetworkLifVserver(Map<Long, Collection<Sample<NetworkLifVserverMeasurements>>> samples, boolean isHistorical);

	void processNetworkPort(Map<Long, Collection<Sample<NetworkPortMeasurements>>> samples, boolean isHistorical);

	void processNfsv3(Map<Long, Collection<Sample<Nfsv3Measurements>>> samples, boolean isHistorical);

	void processNfsv4(Map<Long, Collection<Sample<Nfsv4Measurements>>> samples, boolean isHistorical);

	void processNfsv41(Map<Long, Collection<Sample<Nfsv41Measurements>>> samples, boolean isHistorical);

    void processNfs(Map<Long, Collection<Sample<NfsMeasurements>>> samples, boolean isHistorical);

	void processNode(Map<Long, Collection<Sample<NodeMeasurements>>> samples, boolean isHistorical);

	void processNic(Map<Long, Collection<Sample<NicMeasurements>>> samples, boolean isHistorical);

	void processNvmfFcLif(Map<Long, Collection<Sample<NvmfFcLifMeasurements>>> samples, boolean isHistorical);

	void processNvmfFcLifVserver(Map<Long, Collection<Sample<NvmfFcLifVserverMeasurements>>> samples, boolean isHistorical);

	void processProcessor(Map<Long, Collection<Sample<ProcessorMeasurements>>> samples, boolean isHistorical);

	void processVserver(Map<Long, Collection<Sample<VserverMeasurements>>> samples, boolean isHistorical);

	void processQosVolumeWorkload(Map<Long, Collection<Sample<QosVolumeWorkloadMeasurements>>> samples, boolean isHistorical);

    void processQosLunWorkload(Map<Long, Collection<Sample<QosLunWorkloadMeasurements>>> samples, boolean isHistorical);

    void processQosFileWorkload(Map<Long, Collection<Sample<QosFileWorkloadMeasurements>>> samples, boolean isHistorical);

	void processQosWorkloadDetail(Map<Long, Collection<Sample<QosWorkloadDetailMeasurements>>> samples, boolean isHistorical);

	void processQosServiceCenter(Map<Long, Collection<Sample<QosServiceCenterMeasurements>>> samples, boolean isHistorical);

	void processQosWorkloadQueueNBlade(Map<Long, Collection<Sample<QosWorkloadQueueNBladeMeasurements>>> samples, boolean isHistorical);

	void processQosWorkloadQueueDBlade(Map<Long, Collection<Sample<QosWorkloadQueueDBladeMeasurements>>> samples, boolean isHistorical);

	void processWafl(Map<Long, Collection<Sample<WaflMeasurements>>> samples, boolean isHistorical);

	void processCluster(Map<Long, Collection<Sample<ClusterMeasurements>>> samples, boolean isHistorical);

	void processResourceHeadRoomAggr(Map<Long, Collection<Sample<ResourceHeadRoomAggrMeasurements>>> samples, boolean isHistorical);

	void processExtCacheObj(Map<Long, Collection<Sample<ExtCacheObjMeasurements>>> samples, boolean isHistorical);
	void processResourceHeadRoomCPU(Map<Long, Collection<Sample<ResourceHeadRoomCPUMeasurements>>> samples, boolean isHistorical);

	void processOpmHeadRoomAggr(Map<Long, Collection<Sample<OpmHeadRoomAggrMeasurements>>> samples, boolean isHistorical);

	void processOpmHeadRoomCPU(Map<Long, Collection<Sample<OpmHeadRoomCPUMeasurements>>> samples, boolean isHistorical);

}

