/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.volumenode.interfaces.data;

import java.io.Serializable;

/**
 * @author Mike Johnson
 *         Date: June 14, 2013
 */
public class VolumeNodeMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * Average latency in microseconds for the WAFL filesystem to process all the operations on the volume;
     * not including request processing or network communication time
     */
    private float avgLatency;

    /**
     * Bytes read per second from the volume
     */
    private float readData;

    /**
     * Average latency in microseconds for the WAFL filesystem to process read request to the volume;
     * not including request processing or network communication time
     */
    private float readLatency;

    /**
     * Number of reads per second to the volume
     */
    private float readOps;

    /**
     * Number of operations per second serviced by the volume
     */
    private float totalOps;


    /**
     * Bytes written per second to the volume
     */
    private float writeData;

    /**
     * Average latency in microseconds for the WAFL filesystem to process write request to the volume;
     * not including request processing or network communication time
     */
    private float writeLatency;

    /**
     * Number of writes per second to the volume
     */
    private float writeOps;

    public VolumeNodeMeasurements() {
    }

    public VolumeNodeMeasurements(float avgLatency, float readData, float readLatency, float readOps, float totalOps, float writeData, float writeLatency, float writeOps) {
        this.avgLatency = avgLatency;
        this.readData = readData;
        this.readLatency = readLatency;
        this.readOps = readOps;
        this.totalOps = totalOps;
        this.writeData = writeData;
        this.writeLatency = writeLatency;
        this.writeOps = writeOps;
    }

    public float getAvgLatency() {
        return avgLatency;
    }

    public float getReadData() {
        return readData;
    }

    public float getReadLatency() {
        return readLatency;
    }

    public float getReadOps() {
        return readOps;
    }

    public float getWriteData() {
        return writeData;
    }

    public float getWriteLatency() {
        return writeLatency;
    }

    public float getWriteOps() {
        return writeOps;
    }

    @Override
    public int hashCode() {
        int result = (avgLatency != +0.0f ? Float.floatToIntBits(avgLatency) : 0);
        result = 31 * result + (readData != +0.0f ? Float.floatToIntBits(readData) : 0);
        result = 31 * result + (readLatency != +0.0f ? Float.floatToIntBits(readLatency) : 0);
        result = 31 * result + (readOps != +0.0f ? Float.floatToIntBits(readOps) : 0);
        result = 31 * result + (totalOps != +0.0f ? Float.floatToIntBits(totalOps) : 0);
        result = 31 * result + (writeData != +0.0f ? Float.floatToIntBits(writeData) : 0);
        result = 31 * result + (writeLatency != +0.0f ? Float.floatToIntBits(writeLatency) : 0);
        result = 31 * result + (writeOps != +0.0f ? Float.floatToIntBits(writeOps) : 0);
        return result;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof VolumeNodeMeasurements)) return false;

        VolumeNodeMeasurements that = (VolumeNodeMeasurements) o;

        if (Float.compare(that.avgLatency, avgLatency) != 0) return false;
        if (Float.compare(that.readData, readData) != 0) return false;
        if (Float.compare(that.readLatency, readLatency) != 0) return false;
        if (Float.compare(that.readOps, readOps) != 0) return false;
        if (Float.compare(that.totalOps, totalOps) != 0) return false;
        if (Float.compare(that.writeData, writeData) != 0) return false;
        if (Float.compare(that.writeLatency, writeLatency) != 0) return false;
        if (Float.compare(that.writeOps, writeOps) != 0) return false;

        return true;
    }

    @Override
    public String toString() {
        return "VolumeMeasurements{" +
                "avgLatency=" + avgLatency +
                ", readData=" + readData +
                ", readLatency=" + readLatency +
                ", readOps=" + readOps +
                ", totalOps=" + totalOps +
                ", writeData=" + writeData +
                ", writeLatency=" + writeLatency +
                ", writeOps=" + writeOps +
                '}';
    }
}
