package com.netapp.oci.netapp.performance.vserver;

import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.netapp.performance.vserver.interfaces.data.VserverMeasurements;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

public class VserverMetaData {

    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("opmOps", "float"),
            new Column("opmReadOps", "float"),
            new Column("opmWriteOps", "float"),
            new Column("opmThroughput", "float"),
            new Column("opmReadThroughput", "float"),
            new Column("opmWriteThroughput", "float"),
            new Column("opmLatency", "float"),
            new Column("opmReadLatency", "float"),
            new Column("opmWriteLatency", "float")
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("opmOps", "float"),
            new Column("opmReadOps", "float"),
            new Column("opmWriteOps", "float"),
            new Column("opmThroughput", "float"),
            new Column("opmReadThroughput", "float"),
            new Column("opmWriteThroughput", "float"),
            new Column("opmLatency", "float"),
            new Column("opmReadLatency", "float"),
            new Column("opmWriteLatency", "float")
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("opmOps")},
            {new RatioMethod("opmReadOps")},
            {new RatioMethod("opmWriteOps")}
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("opmOps"),
            new RatioMethod("opmReadOps"),
            new RatioMethod("opmWriteOps")
    };

    private static ObjectParser<VserverMeasurements> sampleParser;

    static {
        sampleParser = new ObjectParser<VserverMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, VserverMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmReadThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmWriteThroughput());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmWriteLatency());
            }

            public VserverMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new VserverMeasurements().setOpmOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmReadOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmWriteOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmThroughput(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmReadThroughput(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmWriteThroughput(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmReadLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOpmWriteLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<VserverMeasurements> getSampleParser() {
        return sampleParser;
    }

    public static ObjectParser<VserverMeasurements> getSummaryParser() {
        return sampleParser;
    }
}
