/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.wafl;

import java.sql.ResultSet;
import java.sql.SQLException;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.netapp.performance.wafl.interfaces.data.WaflMeasurements;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

public class WaflMetaData {

    public static final Column[] SAMPLE_COLUMNS = new Column[] {
        new Column("totalCpMsecs", "float"),
        new Column("p2Flush", "float"),
        new Column("waflCpBlocksRead", "float"),
        new Column("cpCount", "float")};
        //These were added after the collection period to avoid changes in the OPM 1.1 migration tool

    public static final Column[] SUMMARY_COLUMNS = new Column[] {
    	new Column("totalCpMsecs", "float"),
    	new Column("p2Flush", "float"),
    	new Column("waflCpBlocksRead", "float"),
    	new Column("cpCount", "float")};

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][] {
        {SqlMethod.SUM}, //
        {new RatioMethod("totalCpMsecs")}, //p2flush
        {SqlMethod.SUM}, //wafl_cp_blocks_read
        {SqlMethod.SUM}, //cp_count
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[] {
    	SqlMethod.SUM, //total_cp_msecs
        new RatioMethod("totalCpMsecs"), //p2flush
        SqlMethod.SUM, //wafl_cp_blocks_read
        SqlMethod.SUM, //cp_count
    };

    private static ObjectParser<WaflMeasurements> sampleParser;

    static {
        sampleParser = new ObjectParser<WaflMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement,
                    WaflMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalCpMsecs());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getP2Flush());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWaflCpBlksRead());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCpCount());
            }

            public WaflMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                final WaflMeasurements WaflMeasurements = new WaflMeasurements();
                int i = startIndex - 1;
                WaflMeasurements.setTotalCpMsecs(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                WaflMeasurements.setP2Flush(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                WaflMeasurements.setWaflCpBlksRead(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                WaflMeasurements.setCpCount(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                return WaflMeasurements;
            }
        };
    }

    public static ObjectParser<WaflMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<WaflMeasurements> getSampleParser() {
        return sampleParser;
    }

}
