/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.wafl.interfaces.data;

import java.io.Serializable;

@SuppressWarnings("UnusedDeclaration")
public class WaflMeasurements implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * Average latency in microseconds for the WFL Filesystem to process all the operations on the volume; not including
     * request processing or network communication
     */
    private float totalCpMsecs;

    /**
     *  Average latency in microseconds for the WFL Filesystem to process P2_flush operations on the volume; not including
     * request processing or network communication
     */
    private float p2Flush;
    private float waflCpBlksRead;
    private float cpCount;
    

    public WaflMeasurements() {
    }

    public WaflMeasurements(final float totalCpMsecs, final float p2Flush, final float waflCpBlksRead, final float cpCount) {
        this.totalCpMsecs = totalCpMsecs;
        this.p2Flush = p2Flush;
        this.waflCpBlksRead = waflCpBlksRead;
        this.cpCount = cpCount;
    }

    public float getTotalCpMsecs() {
        return totalCpMsecs;
    }

    public void setTotalCpMsecs(final float totalCpMsecs) {
        this.totalCpMsecs = totalCpMsecs;
    }

    public float getCpCount() {
        return cpCount;
    }

    public void setCpCount(final float cpCount) {
        this.cpCount = cpCount;
    }
    
    public float getWaflCpBlksRead() {
        return waflCpBlksRead;
    }

    public void setWaflCpBlksRead(final float waflCpBlksRead) {
        this.waflCpBlksRead = waflCpBlksRead;
    }
    
    public float getP2Flush() {
        return p2Flush;
    }

    public void setP2Flush(final float p2Flush) {
        this.p2Flush = p2Flush;
    }
    
    @Override
    public int hashCode() {
        int result = (totalCpMsecs != +0.0f ? Float.floatToIntBits(totalCpMsecs) : 0);
        result = 31 * result + (p2Flush != +0.0f ? Float.floatToIntBits(p2Flush) : 0);
        result = 31 * result + (waflCpBlksRead != +0.0f ? Float.floatToIntBits(waflCpBlksRead) : 0);
        result = 31 * result + (cpCount != +0.0f ? Float.floatToIntBits(cpCount) : 0);
        return result;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof WaflMeasurements)) {
            return false;
        }

        WaflMeasurements that = (WaflMeasurements) o;

        return Float.compare(that.totalCpMsecs, totalCpMsecs) == 0
               && Float.compare(that.p2Flush, p2Flush) == 0
               && Float.compare(that.waflCpBlksRead, waflCpBlksRead) == 0
               && Float.compare(that.cpCount, cpCount) == 0;
    }

    @Override
    public String toString() {
        return "WaflMeasurements{" +
               "totalCpMsecs=" + totalCpMsecs +
               ", p2FlushMsecs=" + p2Flush +
               ", waflCpBlksRead=" + waflCpBlksRead +
               ", cpCount=" + cpCount +
               '}';
    }
}
