/*
 * Copyright (c) 2015 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.wafl.interfaces.remote;


import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.qosservicecenter.interfaces.data.QosServiceCenterMeasurements;
import com.netapp.oci.netapp.performance.wafl.interfaces.data.WaflMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Remote
public interface WaflPerformanceQueryRemote {
    String JNDI_NAME = "ejb:mega-app/server-core//WaflPerformanceQueryBean!com.netapp.oci.netapp.performance.wafl.interfaces.remote.WaflPerformanceQueryRemote";

    /**
     * Gets Wafl samples for period of time
     *
     * @param waflIds set of wafl ids
     * @param fromTime            beginning of the time period
     * @param toTime              end of the time period
     * @return map of wafl id to list ordered by time of wafl samples data
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException
     *          if anything went wrong
     */
    Map<Long, List<SampleInfo<WaflMeasurements>>> getSamples(Set<Long> waflIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets Wafl samples for period of time
     *
     * @param containerId         container id
     * @param waflIds set of wafl ids
     * @param fromTime            beginning of the time period
     * @param toTime              end of the time period
     * @return map of wafl id to list ordered by time of wafl samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<WaflMeasurements>>> getSamples(Long containerId, Set<Long> waflIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets Wafl samples for period of time
     *
     * @param containerId        container id
     * @param waflId wafl id
     * @param fromTime           beginning of the time period
     * @param toTime             end of the time period
     * @return list ordered by time of wafl samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<WaflMeasurements>> getSamples(Long containerId, Long waflId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given wafl id's (waflIds).  The map returned will contain only wafl ids that has a sample.
     * wafl ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the wafl ids passed.
     *
     * @param clusterId cluster id
     * @param waflIds set of wafl ids
     * @return map of wafl id to latest sample SampleInfo<QosServiceCenterMeasurements>. wafl ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<WaflMeasurements>> getLatestSample(Long clusterId, Set<Long> waflIds) throws NetappInterfaceException;

    /**
     * Gets wafl summaries for wafl ids
     *
     * @param waflIds set of wafl ids
     * @param fromTime            beginning of the time period
     * @param toTime              end of the time period
     * @return map of wafl id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<WaflMeasurements>>> getSummaries(Set<Long> waflIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets wafl summaries for wafl ids
     *
     * @param clusterId Cluster Id
     * @param waflIds set of wafl ids
     * @param fromTime            beginning of the time period
     * @param toTime              end of the time period
     * @return map of wafl id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<WaflMeasurements>>> getSummaries(Long clusterId, Set<Long> waflIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;

}
