/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.upgrade.version_1;

import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.platform.upgrade.Upgrader;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author Al Velella
 *         Date: 3/11/13
 *         Time: 3:40 PM
 */
public class Upgrade_1_0_0_103 implements Upgrader {
    @Override
    public void run(UpgradeRunner upgradeRunner) throws TableException {
        dropColumns(upgradeRunner, "node",
                "avgProcessorBusy", "cifsOps", "diskDataRead", "diskDataWritten", "fcpOps", "iscsiOps", "netDataRecv",
                "netDataSent", "nfsOps", "cifsOtherLatency", "cifsOtherOps", "cifsReadData", "cifsReadLatency", "cifsReadOps",
                "cifsWriteData", "cifsWriteLatency", "cifsWriteOps", "fcpOtherLatency", "fcpOtherOps", "fcpReadData",
                "fcpReadLatency", "fcpReadOps", "fcpWriteData", "fcpWriteLatency", "fcpWriteOps", "iscsiOtherLatency",
                "iscsiOtherOps", "iscsiReadData", "iscsiReadLatency", "iscsiReadOps", "iscsiWriteData", "iscsiWriteLatency",
                "iscsiWriteOps", "nfsOtherLatency", "nfsOtherOps", "nfsReadData", "nfsReadLatency", "nfsReadOps",
                "nfsWriteData", "nfsWriteLatency", "nfsWriteOps");
        dropColumns(upgradeRunner, "networkport", "collisions", "recvDropPackets", "recvErrors", "recvMcasts", "sendErrors", "sendMcasts");
        dropColumns(upgradeRunner, "fcpport", "linkFailure", "recvErr");
    }

    private void dropColumns(UpgradeRunner upgradeRunner, String object, String... columns) throws TableException {
        String performanceDb = upgradeRunner.getPerformanceDb();

        Set<String> tables = new HashSet<String>(upgradeRunner.sqlQuery(
                " SELECT" +
                        "   TABLE_NAME" +
                        " FROM" +
                        "   information_schema.TABLES" +
                        " WHERE" +
                        "   table_schema='" + performanceDb + "'" +
                        " AND" +
                        "   (table_name = 'sample_" + object + "'" +
                        "   OR" +
                        "   table_name = 'summary_" + object + "')", TableResultParserDefault.STRING).executeQueryEx());
        for (String table : tables) {
            for (String column : columns) {
                List<String> names = upgradeRunner.sqlQuery(
                        " SELECT" +
                                "   COLUMN_NAME" +
                                " FROM" +
                                "   information_schema.COLUMNS" +
                                " WHERE" +
                                "   table_schema='" + performanceDb + "'" +
                                " AND" +
                                "   table_name = '" + table + "'" +
                                " AND" +
                                "   column_name = '" + column + "'", TableResultParserDefault.STRING).executeQueryEx();
                if (!names.isEmpty()) {
                    upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table + " DROP COLUMN " + column).executeUpdateEx();
                }
            }
        }
    }
}
