/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.upgrade.version_1;

import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.platform.upgrade.Upgrader;

import java.util.HashSet;
import java.util.Set;

/**
 * @author Al Velella
 *         Date: 2/19/13
 *         Time: 4:35 PM
 */
public class Upgrade_1_0_0_96 implements Upgrader {
    @Override
    public void run(UpgradeRunner upgradeRunner) throws TableException {
        dropColumns(upgradeRunner, "node",
                "avgProcessorBusy", "cifsOps", "diskDataRead", "diskDataWritten", "fcpOps", "iscsiOps", "netDataRecv",
                "netDataSent", "nfsOps", "cifsOtherLatency", "cifsOtherOps", "cifsReadData", "cifsReadLatency", "cifsReadOps",
                "cifsWriteData", "cifsWriteLatency", "cifsWriteOps", "fcpOtherLatency", "fcpOtherOps", "fcpReadData",
                "fcpReadLatency", "fcpReadOps", "fcpWriteData", "fcpWriteLatency", "fcpWriteOps", "iscsiOtherLatency",
                "iscsiOtherOps", "iscsiReadData", "iscsiReadLatency", "iscsiReadOps", "iscsiWriteData", "iscsiWriteLatency",
                "iscsiWriteOps", "nfsOtherLatency", "nfsOtherOps", "nfsReadData", "nfsReadLatency", "nfsReadOps",
                "nfsWriteData", "nfsWriteLatency", "nfsWriteOps");
        dropColumns(upgradeRunner, "networkport", "collisions", "recvDropPackets", "recvErrors", "recvMcasts", "sendErrors", "sendMcasts");
        dropColumns(upgradeRunner, "disk", "cpReadChain", "userReadChain", "userWriteChain");
        dropColumns(upgradeRunner, "fcpport", "linkFailure", "recvErr");

        dropTables(upgradeRunner, "aggregate");
    }

    private void dropColumns(UpgradeRunner upgradeRunner, String object, String... columns) throws TableException {
        String performanceDb = upgradeRunner.getPerformanceDb();
        Set<String> tables = new HashSet<String>(upgradeRunner.sqlQuery(
                " SELECT" +
                        "   TABLE_NAME" +
                        " FROM" +
                        "   information_schema.TABLES" +
                        " WHERE" +
                        "   table_schema='" + performanceDb + "'" +
                        " AND" +
                        "   (table_name LIKE 'sample_" + object + "_%'" +
                        "   OR" +
                        "   table_name LIKE 'summary_" + object + "_%'" +
                        "   OR" +
                        "   table_name LIKE 'sample_current_" + object + "')", TableResultParserDefault.STRING).executeQueryEx());
        for (String table : tables) {
            for (String column : columns) {
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table + " DROP COLUMN " + column).executeUpdateEx();
            }
        }
    }

    private void dropTables(UpgradeRunner upgradeRunner, String object) throws TableException {
        String performanceDb = upgradeRunner.getPerformanceDb();
        Set<String> tables = new HashSet<String>(upgradeRunner.sqlQuery(
                " SELECT" +
                        "   TABLE_NAME" +
                        " FROM" +
                        "   information_schema.TABLES" +
                        " WHERE" +
                        "   table_schema='" + performanceDb + "'" +
                        " AND" +
                        "   (table_name LIKE 'sample_" + object + "_%'" +
                        "   OR" +
                        "   table_name LIKE 'summary_" + object + "_%'" +
                        "   OR" +
                        "   table_name LIKE 'sample_current_" + object + "')", TableResultParserDefault.STRING).executeQueryEx());
        for (String table : tables) {
            upgradeRunner.sqlUpdate("DROP TABLE " + performanceDb + "." + table).executeUpdateEx();
        }
    }
}
