/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.upgrade.version_1_1;

import com.netapp.oci.platform.common.table.StringCache;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.platform.upgrade.Upgrader;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

/**
 * Upgrade quota keys to be more unique; requires hash based on set of configured quota users
 */
public class Upgrade_1_1_0_10 implements Upgrader {


    @Override
    public void run(final UpgradeRunner upgradeRunner) throws TableException {

        // Query the objid and uuid of all existing user quotas
        final List<Map<Long, String>> userQuotaRecords = upgradeRunner.sqlQuery(
                "SELECT objid, uuid FROM netapp_model.user_quota",
                TableResultParserDefault.newLongStringMap()).executeQueryEx();

        for (final Map<Long, String> userQuotaRecord : userQuotaRecords) {
            for (final Map.Entry<Long, String> userQuotaEntry : userQuotaRecord.entrySet()) {
                final Long userQuotaObjID = userQuotaEntry.getKey();
                final String userQuotaUUID = userQuotaEntry.getValue();

                // query the set of Users that are children of this quota.
                // Build a string concatenation of user IDs (must be the same algorithm that
                // Acquisition users)
                String quotaUserHashRaw = "";
                final List<String> quotaUserIDs = upgradeRunner.sqlQuery(
                        "SELECT quotaUserId FROM netapp_model.quota_user WHERE userQuotaId = " + userQuotaObjID,
                        TableResultParserDefault.STRING).executeQueryEx();
                for (final String quotaUserID : quotaUserIDs) {
                    quotaUserHashRaw += quotaUserID;
                }

                // New UUID has a new field at the end that is the hash calculated from
                // the string concatenation of user IDs
                final String quotaUserHash = String.valueOf(quotaUserHashRaw.hashCode());
                final String updatedUUID = userQuotaUUID + '.' + quotaUserHash;

                upgradeRunner.sqlUpdate(
                        "UPDATE netapp_model.user_quota" +
                                "  SET uuid='" + updatedUUID + "'" +
                                "  WHERE objid=" + userQuotaObjID
                ).executeUpdateEx();
            }
        }



    }
}
