/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.upgrade.version_1_1;

import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.platform.upgrade.Upgrader;

import java.util.*;

/**
 * @author Al Velella
 *         Date: 6/14/13
 *         Time: 1:53 PM
 */
public class Upgrade_1_1_0_6 implements Upgrader {
    @Override
    public void run(UpgradeRunner upgradeRunner) throws TableException {
        String performanceDb = upgradeRunner.getPerformanceDb();
        addColumnIfNotAbsent(upgradeRunner, performanceDb, "sample_processor", "summary_processor", "kahunaBusy", "nBladeBusy", "ALTER TABLE %1s.%2s ADD COLUMN %3s float DEFAULT NULL COMMENT 'Percentage of elapsed time that the processor spent in busy domain \\'kahuna\\'' AFTER %4s");
        addColumnIfNotAbsent(upgradeRunner, performanceDb, "sample_processor", "summary_processor", "storageBusy", "kahunaBusy", "ALTER TABLE %1s.%2s ADD COLUMN %3s float DEFAULT NULL COMMENT 'Percentage of elapsed time that the processor spent in busy domain \\'kahuna\\'' AFTER %4s");
        addColumnIfNotAbsent(upgradeRunner, performanceDb, "sample_processor", "summary_processor", "raidBusy", "storageBusy", "ALTER TABLE %1s.%2s ADD COLUMN %3s float DEFAULT NULL COMMENT 'Percentage of elapsed time that the processor spent in busy domain \\'kahuna\\'' AFTER %4s");
        addColumnIfNotAbsent(upgradeRunner, performanceDb, "sample_processor", "summary_processor", "targetBusy", "raidBusy", "ALTER TABLE %1s.%2s ADD COLUMN %3s float DEFAULT NULL COMMENT 'Percentage of elapsed time that the processor spent in busy domain \\'kahuna\\'' AFTER %4s");
        addColumnIfNotAbsent(upgradeRunner, performanceDb, "sample_processor", "summary_processor", "cifsBusy", "targetBusy", "ALTER TABLE %1s.%2s ADD COLUMN %3s float DEFAULT NULL COMMENT 'Percentage of elapsed time that the processor spent in busy domain \\'kahuna\\'' AFTER %4s");
        addColumnIfNotAbsent(upgradeRunner, performanceDb, "sample_processor", "summary_processor", "clusterBusy", "cifsBusy", "ALTER TABLE %1s.%2s ADD COLUMN %3s float DEFAULT NULL COMMENT 'Percentage of elapsed time that the processor spent in busy domain \\'kahuna\\'' AFTER %4s");
        addColumnIfNotAbsent(upgradeRunner, performanceDb, "sample_processor", "summary_processor", "protocolBusy", "clusterBusy", "ALTER TABLE %1s.%2s ADD COLUMN %3s float DEFAULT NULL COMMENT 'Percentage of elapsed time that the processor spent in busy domain \\'kahuna\\'' AFTER %4s");
        addColumnIfNotAbsent(upgradeRunner, performanceDb, "sample_processor", "summary_processor", "nwkExclusiveBusy", "protocolBusy", "ALTER TABLE %1s.%2s ADD COLUMN %3s float DEFAULT NULL COMMENT 'Percentage of elapsed time that the processor spent in busy domain \\'kahuna\\'' AFTER %4s");
        addColumnIfNotAbsent(upgradeRunner, performanceDb, "sample_processor", "summary_processor", "nwkLegacyBusy", "nwkExclusiveBusy", "ALTER TABLE %1s.%2s ADD COLUMN %3s float DEFAULT NULL COMMENT 'Percentage of elapsed time that the processor spent in busy domain \\'kahuna\\'' AFTER %4s");
    }

    private void addColumnIfNotAbsent(UpgradeRunner upgradeRunner, String performanceDb, String table1, String table2, String column, String afterColumn, String alterSql) throws TableException {
        addColumnIfNotAbsent(upgradeRunner, performanceDb, table1, column, afterColumn, alterSql);
        addColumnIfNotAbsent(upgradeRunner, performanceDb, table2, column, afterColumn, alterSql);
    }

    private void addColumnIfNotAbsent(UpgradeRunner upgradeRunner, String performanceDb, String table, String column, String afterColumn, String alterSql) throws TableException {
        List<String> tables = upgradeRunner.sqlQuery(
                " SELECT" +
                        "   COLUMN_NAME" +
                        " FROM" +
                        "   information_schema.COLUMNS" +
                        " WHERE" +
                        "   table_schema='" + performanceDb + "'" +
                        " AND" +
                        "   table_name = '" + table + "'", TableResultParserDefault.STRING).executeQueryEx();
        if (!tables.isEmpty()) {
            List<String> names = upgradeRunner.sqlQuery(
                    " SELECT" +
                            "   COLUMN_NAME" +
                            " FROM" +
                            "   information_schema.COLUMNS" +
                            " WHERE" +
                            "   table_schema='" + performanceDb + "'" +
                            " AND" +
                            "   table_name = '" + table + "'" +
                            " AND" +
                            "   column_name = '" + column + "'", TableResultParserDefault.STRING).executeQueryEx();
            if (names.isEmpty()) {
                upgradeRunner.sqlUpdate(String.format(alterSql, performanceDb, table, column, afterColumn)).executeUpdateEx();
            }
        }
    }
}
