/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.upgrade.version_1_2;

import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.platform.upgrade.Upgrader;

import java.util.List;


/**
 * Upgrader 1.1.0.25 was checked into 1.2, and NOT 1.1, after the branch from 1.1 to 1.2, and published in 1.2.
 * This upgrade is for the folks upgrading from 1.1 to 1.2, who would otherwise miss the 1.1.0.25 upgrade.
 * This upgrade checks to see if 1.1.0.25 has been run (by checking for a column created by 1.1.0.25).
 * If 1.1.0.25 has not run, this upgrade will perform the upgrade logic provided by 1.1.0.25.
 */
public class Upgrade_1_2_0_0 implements Upgrader {

    private static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(Upgrade_1_2_0_0.class);

    @Override
    public void run(UpgradeRunner upgradeRunner) throws TableException {
        final List<String> partnerNodeIdColumnName = upgradeRunner.sqlQuery(
                "SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE " +
                "TABLE_SCHEMA='netapp_model' and TABLE_NAME='storage_shelf' and COLUMN_NAME='partnerNodeId';",
                TableResultParserDefault.STRING).executeQueryEx();
        if (partnerNodeIdColumnName.isEmpty()) {
            // 1.1.0.25 has not run, do the required upgrade here
            upgradeRunner.sqlUpdate("ALTER TABLE netapp_model.storage_shelf ADD COLUMN partnerNodeId varchar(255) DEFAULT NULL COMMENT 'Partner Node UUID' AFTER nodeId;").executeUpdateEx();;
            upgradeRunner.sqlUpdate("ALTER TABLE netapp_model.storage_shelf ADD COLUMN partnerChannelName varchar(255) DEFAULT NULL COMMENT 'Partner Channel Name' AFTER channelName;").executeUpdateEx();;
        }
    }
}
