/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.upgrade.version_1_2;

import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.platform.upgrade.Upgrader;

import java.util.List;

/**
 * Populate disk_aggregate_relationship table using the foreign  key relation between the disk & aggregate table
 */
public class Upgrade_1_2_0_1 implements Upgrader {

    @Override
    public void run(final UpgradeRunner upgradeRunner) throws TableException {
        // Query the sanscreen.id tale for the objid sequence.
        final List<Long> objIdList = upgradeRunner.sqlQuery(
                "SELECT \n" +
                        "    value\n" +
                        "FROM\n" +
                        "    sanscreen.id\n" +
                        "WHERE\n" +
                        "    id = 1", TableResultParserDefault.LONG).executeQueryEx();
        if (objIdList.size() > 0) {
            final Long objId = objIdList.get(0);
            //Create and populate a temp table with the obj id's and the data
            final String tempTableName = "netapp_model.upgrade_1201_1";
            //Drop Temp table
            final String dropTempTable = "DROP TEMPORARY TABLE IF EXISTS " + tempTableName;
            upgradeRunner.sqlUpdate(dropTempTable).executeUpdateEx();
            //Create & populate Temp table
            final String createTempTable = "CREATE TEMPORARY TABLE " + tempTableName + " (\n" +
                    "   objid BIGINT(20) NOT NULL AUTO_INCREMENT,\n" +
                    "   originid BIGINT(20) NOT NULL,\n" +
                    "   uuid VARCHAR(255) NOT NULL ,\n" +
                    "   resourceKey VARCHAR(512)  NOT NULL,\n" +
                    "   clusterId BIGINT(20) NOT NULL,\n" +
                    "   diskId BIGINT(20) NOT NULL,\t\n" +
                    "   aggregateId BIGINT(20) NOT NULL,\t\n" +
                    "   checksum INT NOT NULL,\n" +
                    "   changeDetailChecksum INT NOT NULL,\n" +
                    "   createtime BIGINT(20) NOT NULL,\n" +
                    "   updatetime BIGINT(20) NOT NULL,\n" +
                    "   objState ENUM('LIVE','DEAD'),\n" +
                    "   PRIMARY KEY (objid)\n" +
                    ") AUTO_INCREMENT = " + objId + " select \n" +
                    "   d.originid originid,\n" +
                    "   CONCAT(d.uuid, '.', a.uuid) UUID,\n" +
                    "   CONCAT(d.uuid, ':type=disk_aggregate_relationship,aggregate_uuid=', a.uuid) resourceKey,\n" +
                    "    d.clusterId clusterId,\n" +
                    "    d.objid diskId,\n" +
                    "    a.objid aggregateId,\n" +
                    "    0 checksum,\n" +
                    "    0 changeDetailChecksum,\n" +
                    "    a.createtime createtime,\n" +
                    "    a.updatetime updatetime,\n" +
                    "    a.objState objState\n" +
                    "from\n" +
                    "    netapp_model.disk d\n" +
                    "        inner join\n" +
                    "    netapp_model.aggregate a ON (d.aggregateId = a.objid)\n";
            upgradeRunner.sqlUpdate(createTempTable).executeUpdateEx();
            //Insert Into disk aggregate relationship table.
            final String insertDARelation = "INSERT INTO netapp_model.disk_aggregate_relationship \n" +
                    "SELECT \n" +
                    "    objid,\n" +
                    "    originid,\n" +
                    "    uuid,\n" +
                    "    resourceKey,\n" +
                    "    clusterId,\n" +
                    "    diskId,\n" +
                    "    aggregateId,\n" +
                    "    checksum,\n" +
                    "    changeDetailChecksum,\n" +
                    "    createtime,\n" +
                    "    updatetime,\n" +
                    "    objState\n" +
                    "FROM\n" +
                    "    " + tempTableName;
            upgradeRunner.sqlUpdate(insertDARelation).executeUpdateEx();
            //Update the object id sequence with the used ids plus one. Object Ids are incremented in thousands. Round it to the next thousand.
            final String updateObjIdSeq = "UPDATE sanscreen.id \n" +
                    "SET \n" +
                    "    value = value + IFNULL((1000 * ((((SELECT \n" +
                    "            max(objid)\n" +
                    "        FROM\n" +
                    "            netapp_model.disk_aggregate_relationship) - value) DIV 1000) + 1)), 0)\n" +
                    "WHERE\n" +
                    "    id = 1";
            upgradeRunner.sqlUpdate(updateObjIdSeq).executeUpdateEx();
            // Remove the aggregate Id foreign key column from the disk table.
            upgradeRunner.sqlUpdate("ALTER TABLE netapp_model.disk DROP FOREIGN KEY disk_ibfk_3").executeUpdateEx();
            upgradeRunner.sqlUpdate("ALTER TABLE netapp_model.disk DROP COLUMN aggregateId").executeUpdateEx();
        } else {
            throw new TableException("Unable to query the sanscreen.id table.");
        }
    }
}
