/*
 * Copyright (c) 2014 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.upgrade.version_1_2;

import java.util.HashSet;
import java.util.Set;

import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.platform.upgrade.Upgrader;

/**
 * Upgrader to persist cacheMissRate counter values for volume workloads and workload_queue_dblade samples
 * Also drop all readIO* columns from volume workload samples as those stats are not required anymore.
 */
public class Upgrade_1_2_0_22 implements Upgrader {

    @Override
    public void run(UpgradeRunner upgradeRunner) throws TableException {
        upgradeQosWorkloadQueueDblade(upgradeRunner);
    }

    private void upgradeQosWorkloadQueueDblade(UpgradeRunner upgradeRunner) throws TableException {
        String performanceDb = upgradeRunner.getPerformanceDb();
        Set<String> tables = new HashSet<String>(upgradeRunner.sqlQuery(
                " SELECT" +
                        "   TABLE_NAME" +
                        " FROM" +
                        "   information_schema.TABLES" +
                        " WHERE" +
                        "   table_schema='" + performanceDb + "'" +
                        " AND" +
                        "   (table_name LIKE 'sample_qos_volume_workload_%'" +
                        "   OR" +
                        "   table_name LIKE 'summary_qos_volume_workload_%'" +
                        "   OR" +
                        "   table_name LIKE 'sample_qos_workload_queue_dblade_%'" +
                        "   OR" +
                        "   table_name LIKE 'summary_qos_workload_queue_dblade_%')", TableResultParserDefault.STRING).executeQueryEx());
        for (String table : tables) {
            if(table.startsWith("sample_qos_workload_queue_dblade_") || table.startsWith("summary_qos_workload_queue_dblade_")){
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table + " ADD COLUMN cacheMissRate float DEFAULT NULL COMMENT 'Percentage of cache misses. The sum of read_io_type DISK and HYA_HDD' AFTER delayNvlogTransferWaitTime").executeUpdateEx();
            }else{
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table + " DROP COLUMN readIOPercentCache").executeUpdateEx();
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table + " DROP COLUMN readIOPercentExtCache").executeUpdateEx();
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table + " DROP COLUMN readIOPercentDisk").executeUpdateEx();
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table + " DROP COLUMN readIOPercentBambooSsd").executeUpdateEx();
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table + " DROP COLUMN readIOPercentHyaHdd").executeUpdateEx();
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table + " DROP COLUMN readIOPercentHyaCache").executeUpdateEx();
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table + " DROP COLUMN readIOPercentHyaNonCache").executeUpdateEx();
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table + " ADD COLUMN cacheMissRate float DEFAULT NULL COMMENT 'Percentage of cache misses. The sum of read_io_type DISK and HYA_HDD' AFTER writeOps").executeUpdateEx();
            }
        }
    }
}
