/*
 * Copyright (c) 2014 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.upgrade.version_1_2;

import java.util.HashSet;
import java.util.Set;

import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.platform.upgrade.Upgrader;

public class Upgrade_1_2_0_25 implements Upgrader {

    @Override
    public void run(UpgradeRunner upgradeRunner) throws TableException {
        upgradeQosWorkloadQueueDblade(upgradeRunner);
    }

    private void upgradeQosWorkloadQueueDblade(UpgradeRunner upgradeRunner) throws TableException {
        String performanceDb = upgradeRunner.getPerformanceDb();
        Set<String> tables = new HashSet<String>(upgradeRunner.sqlQuery(
                " SELECT" +
                        "   TABLE_NAME" +
                        " FROM" +
                        "   information_schema.TABLES" +
                        " WHERE" +
                        "   table_schema='" + performanceDb + "'" +
                        " AND" +
                        "   (table_name LIKE 'sample_qos_workload_queue_dblade_%'" +
                        "   OR" +
                        "   table_name LIKE 'summary_qos_workload_queue_dblade_%')", TableResultParserDefault.STRING).executeQueryEx());
        for (String table : tables) {
            upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table +
                    " ADD COLUMN cpuDBladeBackgroundWAFLCPServiceTime bigint(20) DEFAULT NULL " +
                    "  COMMENT 'The raw CPU Dblade background service time. Wafl_cp represents service time " +
                    "     contributed by the WAFL_CP workload on behalf of this workload.'" +
                    "  AFTER cpuDBladeBackgroundServiceTime," +
                    " ADD COLUMN cpuDBladeBackgroundBackgroundDeleteServiceTime bigint(20) DEFAULT NULL " +
                    "  COMMENT 'The raw CPU Dblade background service time. Background_delete represents service time " +
                    "     contributed by the BACKGROUND_DELETE workload on behalf of this workload.' " +
                    "  AFTER cpuDBladeBackgroundWAFLCPServiceTime," +
                    " ADD COLUMN cpuDBladeBackgroundRAStripeXServiceTime bigint(20) DEFAULT NULL " +
                    "  COMMENT 'The raw CPU Dblade background service time. Readahead_x_stripe represents service time" +
                    "     contributed by the RA_STRIPE_X workload on behalf of this workload. Units are microseconds.' " +
                    "  AFTER cpuDBladeBackgroundBackgroundDeleteServiceTime," +
                    " ADD COLUMN diskHDDBackgroundWAFLCPServiceTime bigint(20) DEFAULT NULL " +
                    "  COMMENT 'The raw Disk HDD background service time. Wafl_cp represents service time contributed " +
                    "     by the WAFL_CP workload on behalf of this workload. Units are microseconds.' " +
                    "  AFTER diskHDDBackgroundServiceTime," +
                    " ADD COLUMN diskHDDBackgroundBackgroundDeleteServiceTime bigint(20) DEFAULT NULL " +
                    "  COMMENT 'The raw Disk HDD background service time. Background_delete represents service time " +
                    "     contributed by the BACKGROUND_DELETE workload on behalf of this workload.' " +
                    "  AFTER diskHDDBackgroundWAFLCPServiceTime," +
                    " ADD COLUMN diskHDDBackgroundRAStripeXServiceTime bigint(20) DEFAULT NULL " +
                    "  COMMENT 'The raw Disk HDD background service time. Readahead_x_stripe represents service time " +
                    "     contributed by the RA_STRIPE_X workload on behalf of this workload.' " +
                    "  AFTER diskHDDBackgroundBackgroundDeleteServiceTime," +
                    " ADD COLUMN diskSSDBackgroundWAFLCPServiceTime bigint(20) DEFAULT NULL " +
                    "  COMMENT 'The raw disk SSD background service time. Wafl_cp represents service time " +
                    "     contributed by the WAFL_CP workload on behalf of this workload.'" +
                    "  AFTER diskSSDBackgroundServiceTime," +
                    " ADD COLUMN diskSSDBackgroundBackgroundDeleteServiceTime bigint(20) DEFAULT NULL " +
                    "  COMMENT 'The raw disk SSD background service time. Background_delete represents service time " +
                    "     contributed by the BACKGROUND_DELETE workload on behalf of this workload.' " +
                    "  AFTER diskSSDBackgroundWAFLCPServiceTime," +
                    " ADD COLUMN diskSSDBackgroundRAStripeXServiceTime bigint(20) DEFAULT NULL " +
                    "  COMMENT 'The raw disk SSD background service time. Readahead_x_stripe represents service time " +
                    "     contributed by the RA_STRIPE_X workload on behalf of this workload' " +
                    "  AFTER diskSSDBackgroundBackgroundDeleteServiceTime"
            ).executeUpdateEx();
        }
    }
}
