/*
 * Copyright (c) 2014 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.upgrade.version_1_2;

import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.platform.upgrade.Upgrader;
import java.util.List;

/**
 * @author Michael Rydeen
 */
public class Upgrade_1_2_0_31 implements Upgrader {

    @Override
    public void run(UpgradeRunner upgradeRunner) throws TableException {
        // This upgrade, we will be adding in 2 new ds_type_attrs:
        //
        //      1.) netappstorageperformance.rawpersistence.enabled
        //      2.) netappstorageperformance.cookedpersistence.enabled
        //
        //      We need to make sure that any existing DS entries get these 
        //      new attributes and they are set to false/true respectively
        // 
        
        // First need to get the list of data sources ID's
        //
        final List<Long> dsIdList = upgradeRunner.sqlQuery(
                        "SELECT id from acquisition.ds", TableResultParserDefault.LONG).executeQueryEx();

        if ( dsIdList.size() > 0 ) {
            // Now loop over list and add to the ds_attrs table
            //
            for ( Long dsId: dsIdList ) {
                String cmd = "INSERT IGNORE INTO acquisition.ds_attrs (`ds_id`, `name`, `value`, `version`) VALUES (" +
                                      dsId + ", 'netappstorageperformance.rawpersistence.enabled', 'false', 0 )";
                upgradeRunner.sqlUpdate(cmd).executeUpdateEx();

                cmd = "INSERT IGNORE INTO acquisition.ds_attrs (`ds_id`, `name`, `value`, `version`) VALUES (" +
                                      dsId + ", 'netappstorageperformance.cookedpersistence.enabled', 'true', 0 )";
                upgradeRunner.sqlUpdate(cmd).executeUpdateEx();
            }
        }
    }
}
