package com.netapp.oci.netapp.upgrade.version_1_2;

import java.util.List;

import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.platform.upgrade.Upgrader;

/**
 * 
 * @author chengs
 *
 */
public class Upgrade_1_2_0_34 implements Upgrader{

	@Override
	public void run(UpgradeRunner upgradeRunner) throws TableException {
        // This upgrade, we will be adding in 2 columns to sample_lun_raw_xxx tables if the column doesn't exist:
        //      1.) read_partial_blocks
        //      2.) write_partial_blocks
		final String rpb = "read_partial_blocks";
		final String wpb = "write_partial_blocks";
		
		final List<String> rawLunTables = upgradeRunner.sqlQuery(
                "SELECT CONCAT(table_schema, '.', table_name) FROM information_schema.tables WHERE table_name like 'sample_lun_raw%'", 
                TableResultParserDefault.STRING).executeQueryEx();

		String sqlCheckColExist = "SELECT column_name FROM information_schema.columns WHERE table_name = '%s' AND column_name = '%s'";	
		
		String sqlAlterRpb = "ALTER TABLE %s ADD COLUMN " + rpb + " bigint(20) DEFAULT NULL COMMENT " +
				"'Reads whose size is not a multiple of WAFL block size'";
		
		String sqlAlterWpb = "ALTER TABLE %s ADD COLUMN " + wpb + " bigint(20) DEFAULT NULL COMMENT " +
				"'Writes whose size is not a multiple of WAFL block size'";

		if (rawLunTables != null && !rawLunTables.isEmpty()) {
			for (String tableName : rawLunTables) {
				// check to see if rpb column already exist
				List<String> cols = upgradeRunner.sqlQuery(String.format(sqlCheckColExist, tableName, rpb), 
						TableResultParserDefault.STRING).executeQueryEx(); 
				if (cols == null || cols.isEmpty()) {
					upgradeRunner.sqlUpdate(String.format(sqlAlterRpb, tableName)).executeUpdateEx();
				}
				
				// check to see if wpb column already exist
				cols = upgradeRunner.sqlQuery(String.format(sqlCheckColExist, tableName, wpb), 
						TableResultParserDefault.STRING).executeQueryEx(); 
				if (cols == null || cols.isEmpty()) {
					upgradeRunner.sqlUpdate(String.format(sqlAlterWpb, tableName)).executeUpdateEx();
				}
			}
		}
		
	}

}
