/*
 * Copyright (c) 2014 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.upgrade.version_1_2;

import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.platform.upgrade.Upgrader;

import java.util.HashSet;
import java.util.Set;

public class Upgrade_1_2_0_48 implements Upgrader {

    @Override
    public void run(UpgradeRunner upgradeRunner) throws TableException {
        upgradeQosWorkloadQueueDblade(upgradeRunner);
    }

    private void upgradeQosWorkloadQueueDblade(UpgradeRunner upgradeRunner) throws TableException {
        String performanceDb = upgradeRunner.getPerformanceDb();

        Set<String> tables = new HashSet<String>(upgradeRunner.sqlQuery(
                " SELECT" +
                        "   TABLE_NAME" +
                        " FROM" +
                        "   information_schema.TABLES" +
                        " WHERE" +
                        "   table_schema='" + performanceDb + "'" +
                        " AND" +
                        "   (table_name LIKE 'sample_qos_workload_queue_dblade_%'" +
                        "   OR" +
                        "   table_name LIKE 'sample_current_qos_workload_queue_dblade'" +
                        "   OR" +
                        "   table_name LIKE 'summary_qos_workload_queue_dblade_%'" +
                        "   OR " +
                        "   table_name LIKE 'sample_qos_service_center_%'" +
                        "   OR" +
                        "   table_name LIKE 'sample_current_qos_service_center'" +
                        "   OR" +
                        "   table_name LIKE 'summary_qos_service_center_%'" +
                        "   OR" +
                        "   table_name LIKE 'sample_qos_volume_workload_%'" +
                        "   OR" +
                        "   table_name LIKE 'sample_current_qos_volume_workload'" +
                        "   OR" +
                        "   table_name LIKE 'summary_qos_volume_workload_%')", TableResultParserDefault.STRING).executeQueryEx());
        for (String table : tables) {
            if (table.contains("qos_service_center")) {
                //Add cpuElapsedTime & numInstances to all qos_service_center tables
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table +
                        " ADD COLUMN cpuElapsedTime bigint(20) DEFAULT NULL " +
                        "  COMMENT 'Elapsed time since boot'," +
                        " ADD COLUMN numInstances bigint(20) DEFAULT NULL " +
                        "  COMMENT 'The number of instances (processors, disks, and so on) in the resource.'" +
                        "  AFTER cpuElapsedTime"
                ).executeUpdateEx();
            } else if (table.contains("qos_volume_workload")) {
                //Add cacheMissRateBase to all qos_volume_workload tables
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table +
                        " ADD COLUMN cacheMissRateBase bigint(20) DEFAULT NULL " +
                        "  COMMENT 'This is the total number of WAFL read requests. It is used as a base counter " +
                        "(or denominator) for the cache_miss_rate counter. This counter is exactly similar to the " +
                        " read_io_type_base counter from the workload object.'" +
                        "  AFTER cacheMissRate"
                ).executeUpdateEx();
            } else {
                //Add cacheMissRateBase to all qos_workload_queue_dblade tables
                upgradeRunner.sqlUpdate("ALTER TABLE " + performanceDb + "." + table +
                        " ADD COLUMN cacheMissRateBase bigint(20) DEFAULT NULL " +
                        "  COMMENT 'This is the total number of WAFL read requests. It is used as a base counter " +
                        "(or denominator) for the cache_miss_rate counter. This counter is exactly similar to the " +
                        " read_io_type_base counter from the workload object.'" +
                        "  AFTER collectionPeriod"
                ).executeUpdateEx();
            }
        }
    }
}
