package com.netapp.oci.netapp.upgrade.version_1_3;

import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.platform.upgrade.Upgrader;

import java.util.HashSet;
import java.util.Set;

/**
 * Convert node latency from milliseconds to microseconds for 8.2.X & 8.3.0
 */
public class Upgrade_1_3_0_76 implements Upgrader {

    @Override
    public void run(UpgradeRunner upgradeRunner) throws TableException {
        upgradeNodeLatency(upgradeRunner);
    }

    private void upgradeNodeLatency(UpgradeRunner upgradeRunner) throws TableException {
        String performanceDb = upgradeRunner.getPerformanceDb();

        StringBuilder sb = new StringBuilder();
        sb.append("SELECT TABLE_NAME FROM information_schema.TABLES ");
        sb.append("WHERE table_schema = '").append(performanceDb).append("' ");
        sb.append("  AND (table_name = 'sample_node' ");
        sb.append("       OR table_name = 'summary_node') ");

        Set<String> tables = new HashSet<>(upgradeRunner.sqlQuery(
                sb.toString(), TableResultParserDefault.STRING).executeQueryEx());

        // BURT  936091, convert node avg, read and write latency from milliseconds to microseconds seconds for SN and FS.0 clusters
        for (String table : tables) {
            upgradeRunner.sqlUpdate("update "+performanceDb+"."+table+" sn\n" +
                            "                inner join\n" +
                            "              netapp_model.node n on (n.objid = sn.objid)\n" +
                            "                inner join\n" +
                            "              netapp_model.cluster c on (c.objid = n.clusterId)\n" +
                            "              set sn.readLatency = sn.readLatency * 1000, " +
                            "                  sn.writeLatency = sn.writeLatency * 1000, " +
                            "                  sn.avgLatency = sn.avgLatency * 1000\n" +
                            "                where c.versionGeneration = 8 and (" +
                            "                        (c.versionMajor = 3 and c.versionMinor = 0) or " +
                            "                        (c.versionMajor = 2)" +
                            "                );"
            ).executeUpdateEx();
        }
    }
}