/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.acq.interfaces.data;

import java.util.*;

/**
 * @author Jason Sprague
 *         Date: 2/11/13
 *         Time: 2:45 PM
 */
public class PackagePollCompleteStatus extends PackagePollStatus {

    private static final long serialVersionUID = 1L;

    private boolean success;
    private int durationTime;
    private int zapiTime;
    private int reportTime;
    private List<Map<String, String>> errors;

    public static enum Error {
        CREDENTIAL_FAILURE,
        NETWORK_ACCESS_FAILURE,
        BAD_LIF_ERROR,
        BAD_ZAPI_RESPONSE,
        DUPLICATE_DATASOURCE,
        INTERNAL_ERROR
    }

    public PackagePollCompleteStatus(String datasourceName, Long datasourceId, String packageName,
                                     boolean success, long pollStartTime, long sampleTime, int durationTime, int pollInterval) {
        super(datasourceName, datasourceId, packageName, pollStartTime, sampleTime, pollInterval);
        this.success = success;
        this.durationTime = durationTime;
        this.zapiTime = -1;
        this.reportTime = -1;
        this.errors = new ArrayList<Map<String, String>>();
    }

    public PackagePollCompleteStatus(String datasourceName, Long datasourceId, String packageName,
                                     boolean success, long pollStartTime, long sampleTime, int durationTime, int pollInterval, int zapiTime, int reportTime) {
        super(datasourceName, datasourceId, packageName, pollStartTime, sampleTime, pollInterval);
        this.success = success;
        this.durationTime = durationTime;
        this.zapiTime = zapiTime;
        this.reportTime = reportTime;
        this.errors = new ArrayList<Map<String, String>>();
    }

    public void addError(Error error, Map<String, String> keyToValue) {
        HashMap<String, String> nameToValue = new HashMap<String, String>(keyToValue);
        nameToValue.put("Type", error.name());
        errors.add(nameToValue);
    }

    public boolean isSuccess() {
        return success;
    }

    public int getDurationTime() {
        return durationTime;
    }

    public int getZapiTime() {
        return zapiTime;
    }

    public int getReportTime() {
        return reportTime;
    }

    public List<Map<String, String>> getErrors() {
        return Collections.unmodifiableList(errors);
    }

}

