/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.acq.interfaces.impl;

import com.netapp.oci.platform.acq.interfaces.data.PackagePollCompleteStatus;
import com.netapp.oci.platform.acq.interfaces.data.PackagePollSkippedStatus;
import com.netapp.oci.platform.acq.interfaces.data.PackagePollStartStatus;
import com.netapp.oci.platform.acq.interfaces.data.PackagePollStatus;
import com.netapp.oci.platform.acq.interfaces.remote.AcquisitionStatusRemote;
import com.netapp.oci.platform.common.ServerInterfaceInterceptor;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.plugin.data.PlatformPlugin;
import com.netapp.oci.platform.plugin.remote.PlatformPluginRemote;
import com.onaro.sanscreen.server.interfaces.data.ServerInterfaceException;
import com.onaro.sanscreen.util.Constant;
import com.onaro.sanscreen.util.Schema;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.interceptor.Interceptors;
import javax.sql.DataSource;
import java.sql.Connection;

/**
 * @author Al Velella
 *         Date: 12/7/12
 *         Time: 4:57 PM
 */
@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
@RolesAllowed({"guest", "user", "admin", "scheduler", "acquisition"})
@Interceptors(ServerInterfaceInterceptor.class)
public class AcquisitionStatusBean implements AcquisitionStatusRemote {

    @Resource(name = "jdbc/OnaroDataSource",
              mappedName = "java:/onaro_pool",
              type = DataSource.class,
              authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    @EJB
    private PlatformPluginRemote platformPluginRemote;

    @Override
    public void packageCompleted(Schema schema, PackagePollCompleteStatus packagePollCompleteStatus) throws ServerInterfaceException {
        handleNotification(schema, packagePollCompleteStatus);
    }

    @Override
    public void packageStarted(Schema schema, PackagePollStartStatus packagePollStartStatus) throws ServerInterfaceException {
        handleNotification(schema, packagePollStartStatus);
    }

    @Override
    public void packageSkipped(Schema schema, PackagePollSkippedStatus packagePollSkippedStatus) throws ServerInterfaceException {
        handleNotification(schema, packagePollSkippedStatus);
    }

    private void handleNotification(Schema schema, PackagePollStatus packagePollStatus) throws ServerInterfaceException {
        PlatformPlugin plugin = platformPluginRemote.getPlugin(schema);
        if (plugin == null) {
            throw new ServerInterfaceException("Plugin " + schema + " not registered");
        }

        Connection connection = null;
        try {
            connection = dataSource.getConnection();
            plugin.handlePackageMessage(connection, packagePollStatus);
        }
        catch (Exception e) {
            throw new ServerInterfaceException("Failed to send package notification", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

}
