/*
 * Copyright (c) 2010 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.common;


import java.util.*;

/**
 * @author Nancy Walia
 */

/**
 * Use this class to save multiple N*M joins
 * <p/>
 * For example : getApplicationsByDisks
 * A: Application
 * B: Volume
 * C: Disk
 * <p/>
 * getBbyC : getVolumeByDisks
 * getAbyB : getApplicationsByVolume
 * Result : CByA : getApplicationsByDisk
 */

public abstract class JoinResult<A, B, K> {

    public abstract Map<K, Set<B>> getBbyC(Set<K> idsOfC) throws Exception;

    public abstract Map<K, Set<A>> getAbyB(Set<K> idsOfB) throws Exception;

    public abstract K getJoinKey(B b);
    
    public Map<K, Set<A>> joinAtoC(Set<K> keys) throws Exception {
        Map<K, Set<B>> cIdToB = getBbyC(keys);
        if (cIdToB.isEmpty()) {
            return new HashMap<K, Set<A>>();
        }

        // Initialize the result map with a set for each C ID
        Map<K, Set<A>> result = new HashMap<K, Set<A>>(cIdToB.keySet().size());

        Set<K> joinIds = new HashSet<K>(cIdToB.keySet().size());
        for (Set<B> bs : cIdToB.values()) {
            for (B b : bs) {
                joinIds.add(getJoinKey(b));
            }
        }

        Map<K, Set<A>> bIdToA = getAbyB(joinIds);
        for (Map.Entry<K, Set<B>> lEntry : cIdToB.entrySet()) {
            K cId = lEntry.getKey();
            Set<B> bs = lEntry.getValue();
            Set<A> aSet = null;
            for (B b : bs) {
                final Set<A> as = bIdToA.get(getJoinKey(b));
                if (as != null && !as.isEmpty()) {
                    if (aSet == null) {
                        aSet = new HashSet<A>();
                        result.put(cId, aSet);
                    }
                    aSet.addAll(as);
                }
            }
        }
        return result;
    }

    public static <A, K> Map<K, Set<A>> mergeResult(final Map<K, Set<A>>... maps) {
        Map<K, Set<A>> result = new HashMap<K, Set<A>>();
        for (Map<K, Set<A>> map : maps) {
            for (Map.Entry<K, Set<A>> entry : map.entrySet()) {
                final K key = entry.getKey();
                final Set<A> values = entry.getValue();
                Set<A> resultSet = result.get(key);
                if (resultSet == null) {
                    resultSet = new HashSet<A>(values.size());
                    result.put(key, resultSet);
                }
                resultSet.addAll(values);
            }
        }
        return result;
    }
}
