package com.netapp.oci.platform.common;

import com.netapp.oci.platform.common.table.Table;

import java.sql.Connection;

/**
 * Reflection based implementation of table factory
 *
 * @author Anton Sergeev
 */
public class ReflectionBasedTableFactory<K, T extends Table<K>> implements TableFactory<K, T> {
    private Class<T> tableClass;

    public ReflectionBasedTableFactory(Class<T> tableClass) {
        this.tableClass = tableClass;
    }

    public T createTable(Connection connection) {
        try {
            return tableClass.getConstructor(Connection.class).newInstance(connection);
        } catch (Exception ex) {
            throw new IllegalStateException("Failed to construct table for class " + tableClass);
        }
    }
}
