/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.common;

import java.net.InetAddress;

import org.slf4j.LoggerFactory;

/**
 * @author Al Velella
 *         Date: Apr 23, 2012
 *         Time: 11:35:41 AM
 */
public class ServerSystem {

    private static final org.slf4j.Logger logger = LoggerFactory.getLogger(ServerSystem.class);
    private static volatile Boolean isRunningInNetAppDomain = null;

    public static boolean isLinux() {
        return System.getProperty("os.name").equals("Linux");
    }

    public static String getMysqlExe(String sanscreenHome) {
        return isLinux() ? "/opt/mysql/bin/mysql" : sanscreenHome + "/mysql/bin/mysql.exe";
    }

    public static String getMyCnf(String sanscreenHome) {
        return isLinux() ? "/etc/mysql/my.cnf" : sanscreenHome + "/mysql/my.cnf";
    }

    public static String getMySqlDataDir(String sanscreenHome) {
        return isLinux() ? "/opt/netapp/data/mysql" : sanscreenHome + "/mysql/data";
    }

    public static String getMysqlDumpExe(String sanscreenHome) {
        return isLinux() ? "/opt/mysql/bin/mysqldump" : sanscreenHome + "/mysql/bin/mysqldump.exe";
    }

    public static String[] getIpConfigCommand() {
        return isLinux() ? new String[]{"ifconfig", "-a"} : new String[]{"ipconfig", "/all"};
    }

    public static String[] getSystemInformationCommand() {
        return isLinux() ? new String[]{"uname", "-a"} : new String[]{"systeminfo"};
    }

    public static String[] getDiskInfoCommand() {
        return isLinux() ? new String[]{"df", "-a"} : new String[]{};
    }

    public static String[] getRestartCommand(String sanscreenHome) {
        return isLinux() ? new String[]{"/etc/init.d/ocie", "restart"} : new String[]{sanscreenHome + "/bin/restart_server.cmd"};
    }

    public static String[] getStopCommand() {
        return isLinux() ? new String[]{"/etc/init.d/ocie", "stop"} : new String[]{};
    }

    /**
     * Is the current server machine running in the "NetApp domain"?   Checks various system parameters (e.g. hostname)
     * to make this determination.   Initialized on first call, then cached.   If initialization fails, default
     * is false for the server lifetime.
     *
     * @return true if server is determined to be running "netapp domain".
     */
    public static synchronized boolean isNetAppDomain() {
        if (isRunningInNetAppDomain == null) {
            //initialize.
            String user_dns_domain = System.getenv("USERDNSDOMAIN");
            if (user_dns_domain != null && user_dns_domain.contains("NETAPP.COM")) {
                isRunningInNetAppDomain = Boolean.TRUE;
            } else {
                try {
                    InetAddress localHost = InetAddress.getLocalHost();
                    String hostName = localHost.getCanonicalHostName();
                    hostName = hostName.toUpperCase();
                    isRunningInNetAppDomain = hostName.contains("NETAPP.COM") || hostName.contains("ONARO.COM");
                }
                catch (Exception e) {
                    logger.warn("Failed to determine if in NetApp domain for PhoneHomeJob.  Defaulting to false.", e);
                    isRunningInNetAppDomain = Boolean.FALSE;
                }
            }
        } //no else - fall through to return.

        logger.info("running in netapp domain: " + isRunningInNetAppDomain);
        return isRunningInNetAppDomain.booleanValue();
    }
}
