package com.netapp.oci.platform.common.interfaces.data.performance;

import java.io.Serializable;
import java.util.Date;

/**
 * Container for any performance chart sample data
 *
 * @author Anton Sergeev
 */
public class SampleInfo<C> implements Serializable, Comparable<SampleInfo<C>> {
    private static final long serialVersionUID = 1L;

    /**
     * The time of the chart sample
     */
    private Date time;

    /** Performance data to display in chart for given time */
    private C sampleData;

    public SampleInfo(Date time, C sampleData) {
        this.time = time;
        this.sampleData = sampleData;
    }

    public Date getTime() {
        return time;
    }

    public void setTime(Date time) {
        this.time = time;
    }

    public C getSampleData() {
        return sampleData;
    }

    public void setSampleData(C sampleData) {
        this.sampleData = sampleData;
    }

    public boolean isNull() {
        return sampleData == null;
    }

    public int compareTo(SampleInfo<C> o) {
        return time.compareTo(o.time);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof SampleInfo)) return false;

        SampleInfo that = (SampleInfo) o;

        if (sampleData != null ? !sampleData.equals(that.sampleData) : that.sampleData != null) return false;
        if (time != null ? !time.equals(that.time) : that.time != null) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = time != null ? time.hashCode() : 0;
        result = 31 * result + (sampleData != null ? sampleData.hashCode() : 0);
        return result;
    }
}
