package com.netapp.oci.platform.common.table;

import java.util.List;
import java.util.Set;
import java.util.HashSet;
import java.sql.*;

public class DBMetaDataUtil {

    public static Set<String> getAllTablesForDB(Connection connection, String dbName) throws TableException {
        String sql = "SELECT table_name FROM information_schema.tables WHERE table_schema=?";

        List<String> list= PreparedStatementWrapper.create(connection,sql, TableResultParserDefault.STRING)
                .setNext(dbName)
                .executeQueryEx();
        return new HashSet<String>(list);
    }



    public static Set<String> getAllTablesWith2Columns(Connection connection, String dbName, String column1, String column2) throws TableException {
        String sql = "SELECT distinct c1.table_name FROM information_schema.columns c1, information_schema.columns c2 WHERE " +
                     "c1.table_schema=? AND c2.table_schema=? AND c1.table_name=c2.table_name AND c1.column_name=? AND c2.column_name=?";
        List<String> list= PreparedStatementWrapper.create(connection,sql, TableResultParserDefault.STRING)
                .setNext(dbName)
                .setNext(dbName)
                .setNext(column1)
                .setNext(column2)
                .executeQueryEx();
        return new HashSet<String>(list);
    }

    public static Set<String> getAllTablesWithColumnName(Connection connection, String dbName, String column) throws TableException {
        String sql = "SELECT distinct c1.table_name FROM information_schema.columns c1 WHERE c1.table_schema=? AND c1.column_name=? " ;
        List<String> list= PreparedStatementWrapper.create(connection,sql, TableResultParserDefault.STRING)
                .setNext(dbName)
                .setNext(column)
                .executeQueryEx();
        return new HashSet<String>(list);
    }
    
     
    /**
     * Gets the table names in the database that have the given regexp in their name
     * @param connection
     * @param regexp
     * @return Set of all table names that have teh given regexp as part of tehir name
     * @throws java.sql.SQLException
     */
    public static Set<String> getTableWithNames(Connection connection, String dbName, String regexp) throws TableException {
        String sql = "SELECT table_name FROM information_schema.tables WHERE table_schema=? AND table_name LIKE ?";

        List<String> list= PreparedStatementWrapper.create(connection,sql, TableResultParserDefault.STRING)
                .setNext(dbName)
                .setNext("%" + regexp + "%")
                .executeQueryEx();
        return new HashSet<String>(list);
        
    }
}
