package com.netapp.oci.platform.common.table;

import com.onaro.commons.table.DbConfig;

import java.sql.Connection;
import java.util.List;

/**
 * @author Iti Behari
 *         Date: 1/14/13
 */
public class DBPrefixHelper {

    // this must have already been declared somewhere else.
    public static final String PREFIX_DB_TABLE_NAME = "prefix_db";

    public static final String DB_NAME_TO_PREFIX_PARAM_NAME = "DB_NAME_TO_PREFIX";
    public static final String PREFIX_NAME_PARAM_NAME = "PREFIX_NAME";

    /**
     * Configures application to use prefixed database schema iff it finds a value for 'PREFIX_NAME' param in management.prefix_db table.
     * Additionally, it looks for 'DB_NAME_TO_PREFIX' param to get a list of database names to prefix.
     *
     * DB_PREFIX table:
     * DESC MANAGEMENT.DB_PREFIX
        CREATE TABLE management.prefix_db (
        NAME VARCHAR(255) NOT NULL,
        VALUE VARCHAR(255) NOT NULL) ENGINE=INNODB DEFAULT CHARSET=utf8;

     INSERT INTO `prefix_db` (`NAME`, `VALUE`) VALUES('PREFIX_NAME','prefixName');
     INSERT INTO `prefix_db` (`NAME`, `VALUE`) VALUES('DB_NAME_TO_PREFIX','netapp_model');
     INSERT INTO `prefix_db` (`NAME`, `VALUE`) VALUES('DB_NAME_TO_PREFIX','netapp_performance');
     INSERT INTO `prefix_db` (`NAME`, `VALUE`) VALUES('DB_NAME_TO_PREFIX','netapp_model_view');
     INSERT INTO `prefix_db` (`NAME`, `VALUE`) VALUES('DB_NAME_TO_PREFIX','netapp_performance_view');

     * The table may contain one entry for 'PREFIX_NAME' name to specify which prefix to use..
     * The table may contain multiple entries for 'DB_NAME_TO_PREFIX' name.  Each entry specifies the database name to prefix.
     *
     * @param connection
     * @throws TableException
     */
    public void handleDbPrefix(Connection connection) throws TableException {
        String prefixToUse = readDbPrefixName(connection);
        if (prefixToUse != null) {
            configureConnectionToUsePrefix(connection, prefixToUse);
        }
    }

    private void configureConnectionToUsePrefix(Connection connection, String dbPrefixToUse) throws TableException {
        List<String> databasesToPrefix = PreparedStatementWrapper.create(connection, "SELECT value FROM " + DbConfig.getManagementDbName()  + "." + PREFIX_DB_TABLE_NAME
                + " WHERE name='" + DB_NAME_TO_PREFIX_PARAM_NAME + "'", TableResultParserDefault.STRING).executeQueryEx();
        PrefixConnection.setPrefixes(dbPrefixToUse, databasesToPrefix);
    }

    private String readDbPrefixName(Connection connection) throws TableException {
        final String management_db_name = DbConfig.getManagementDbName();
        if(Table.isTableExist(connection, management_db_name, DBPrefixHelper.PREFIX_DB_TABLE_NAME)){
            List<String> result = PreparedStatementWrapper.create(connection, "SELECT value FROM " + management_db_name + "." + PREFIX_DB_TABLE_NAME +
                    " WHERE name='" + PREFIX_NAME_PARAM_NAME + "'", TableResultParserDefault.STRING).executeQueryEx();
            if (result.size() == 1) {
                String prefix = result.iterator().next();
                if (prefix != null && prefix.length() > 0) {
                    return prefix;
                }
            }
            return null;
        }
        return null;
    }

}
