/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.common.table;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Parses object from result set
 *
 * @author Anton Sergeev
 */
public interface ObjectParser<S> {
    /**
     * Parses object data from result set starting specific index
     *
     * @param resultSet result set
     * @param startIndex index to start reading object data
     * @return object itself
     * @throws SQLException if failed to parse object data
     */
    S parseObjectData(ResultSet resultSet, int startIndex) throws SQLException;

    /**
     * Prepares object data for insert into table
     *
     * @param preparedStatement prepared statement to fill with parameter data
     * @param objectData object data
     * @throws TableException if failed to set up parameters
     */
    void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, S objectData) throws TableException;
}
