/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.common.table;

import com.onaro.commons.util.Pair;

import java.sql.*;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.Executor;
import java.util.regex.Pattern;

/**
 * @author Al Velella
 *         Date: 12/18/12
 *         Time: 10:25 AM
 */
public class PrefixConnection implements Connection {
    private static String prefix = null;
    private static Map<String, Pair<String, String>> prefixDatas = new HashMap<String, Pair<String, String>>();

    private ConnectionHelper connectionHelper;
    private Connection connection;
    private static final Pattern[] PATTERNS_TO_TRANSLATE = new Pattern[]{
            Pattern.compile("\\s*USE\\s+['`].*", Pattern.CASE_INSENSITIVE),
            Pattern.compile("\\s*DROP\\s+DATABASE\\s+.*", Pattern.CASE_INSENSITIVE),
            Pattern.compile("\\s*CREATE\\s+DATABASE\\s+.*", Pattern.CASE_INSENSITIVE),
            Pattern.compile("\\s*SHOW\\s+.*", Pattern.CASE_INSENSITIVE)};

    public PrefixConnection(ConnectionHelper connectionHelper) {
        this.connectionHelper = connectionHelper;
        this.connection = null;
    }

    private Connection getConnection() throws SQLException {
        if (connection == null) {
            connection = connectionHelper.getConnection();
        }
        return connection;
    }

    public static void setPrefixes(String prefix, List<String> databases) {
        PrefixConnection.prefix = prefix;
        for (String database : databases) {
            prefixDatas.put(database, new Pair<String, String>("\\s+" + database + "\\.", " " + prefix + "_" + database + "\\."));
        }
    }

    @Override
    public Statement createStatement() throws SQLException {
        return getConnection().createStatement();
    }

    @Override
    public PreparedStatement prepareStatement(String sql) throws SQLException {
        return getConnection().prepareStatement(translateSql(sql));
    }

    @Override
    public CallableStatement prepareCall(String sql) throws SQLException {
        return getConnection().prepareCall(translateSql(sql));
    }

    @Override
    public String nativeSQL(String sql) throws SQLException {
        return getConnection().nativeSQL(translateSql(sql));
    }

    @Override
    public void setAutoCommit(boolean autoCommit) throws SQLException {
        getConnection().setAutoCommit(autoCommit);
    }

    @Override
    public boolean getAutoCommit() throws SQLException {
        return getConnection().getAutoCommit();
    }

    @Override
    public void commit() throws SQLException {
        getConnection().commit();
    }

    @Override
    public void rollback() throws SQLException {
        getConnection().rollback();
    }

    @Override
    public void close() throws SQLException {
        Connection connection = getConnection();
        connectionHelper.close();
        connection.close();
    }

    @Override
    public boolean isClosed() throws SQLException {
        return getConnection().isClosed();
    }

    @Override
    public DatabaseMetaData getMetaData() throws SQLException {
        return getConnection().getMetaData();
    }

    @Override
    public void setReadOnly(boolean readOnly) throws SQLException {
        getConnection().setReadOnly(readOnly);
    }

    @Override
    public boolean isReadOnly() throws SQLException {
        return getConnection().isReadOnly();
    }

    @Override
    public void setCatalog(String catalog) throws SQLException {
        getConnection().setCatalog(catalog);
    }

    @Override
    public String getCatalog() throws SQLException {
        return getConnection().getCatalog();
    }

    @Override
    public void setTransactionIsolation(int level) throws SQLException {
        getConnection().setTransactionIsolation(level);
    }

    @Override
    public int getTransactionIsolation() throws SQLException {
        return getConnection().getTransactionIsolation();
    }

    @Override
    public SQLWarning getWarnings() throws SQLException {
        return getConnection().getWarnings();
    }

    @Override
    public void clearWarnings() throws SQLException {
        getConnection().clearWarnings();
    }

    @Override
    public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
        return getConnection().createStatement(resultSetType, resultSetConcurrency);
    }

    @Override
    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
        return getConnection().prepareStatement(translateSql(sql), resultSetType, resultSetConcurrency);
    }

    @Override
    public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
        return getConnection().prepareCall(translateSql(sql), resultSetType, resultSetConcurrency);
    }

    @Override
    public Map<String, Class<?>> getTypeMap() throws SQLException {
        return getConnection().getTypeMap();
    }

    @Override
    public void setTypeMap(Map<String, Class<?>> map) throws SQLException {
        getConnection().setTypeMap(map);
    }

    @Override
    public void setHoldability(int holdability) throws SQLException {
        getConnection().setHoldability(holdability);
    }

    @Override
    public int getHoldability() throws SQLException {
        return getConnection().getHoldability();
    }

    @Override
    public Savepoint setSavepoint() throws SQLException {
        return getConnection().setSavepoint();
    }

    @Override
    public Savepoint setSavepoint(String name) throws SQLException {
        return getConnection().setSavepoint(name);
    }

    @Override
    public void rollback(Savepoint savepoint) throws SQLException {
        getConnection().rollback(savepoint);
    }

    @Override
    public void releaseSavepoint(Savepoint savepoint) throws SQLException {
        getConnection().releaseSavepoint(savepoint);
    }

    @Override
    public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        return getConnection().createStatement(resultSetType, resultSetConcurrency, resultSetHoldability);
    }

    @Override
    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        return getConnection().prepareStatement(translateSql(sql), resultSetType, resultSetConcurrency, resultSetHoldability);
    }

    @Override
    public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        return getConnection().prepareCall(translateSql(sql), resultSetType, resultSetConcurrency, resultSetHoldability);
    }

    @Override
    public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
        return getConnection().prepareStatement(translateSql(sql), autoGeneratedKeys);
    }

    @Override
    public PreparedStatement prepareStatement(String sql, int[] columnIndexes) throws SQLException {
        return getConnection().prepareStatement(translateSql(sql), columnIndexes);
    }

    @Override
    public PreparedStatement prepareStatement(String sql, String[] columnNames) throws SQLException {
        return getConnection().prepareStatement(translateSql(sql), columnNames);
    }

    @Override
    public Clob createClob() throws SQLException {
        return getConnection().createClob();
    }

    @Override
    public Blob createBlob() throws SQLException {
        return getConnection().createBlob();
    }

    @Override
    public NClob createNClob() throws SQLException {
        return getConnection().createNClob();
    }

    @Override
    public SQLXML createSQLXML() throws SQLException {
        return getConnection().createSQLXML();
    }

    @Override
    public boolean isValid(int timeout) throws SQLException {
        return getConnection().isValid(timeout);
    }

    public boolean isUsingPrefix() {
        return prefix != null;
    }

    @Override
    public void setClientInfo(String name, String value) throws SQLClientInfoException {
        try {
            getConnection().setClientInfo(name, value);
        } catch (SQLException e) {
            throw new SQLClientInfoException();
        }
    }

    @Override
    public void setClientInfo(Properties properties) throws SQLClientInfoException {
        try {
            getConnection().setClientInfo(properties);
        } catch (SQLException e) {
            throw new SQLClientInfoException();
        }
    }

    @Override
    public String getClientInfo(String name) throws SQLException {
        return getConnection().getClientInfo(name);
    }

    @Override
    public Properties getClientInfo() throws SQLException {
        return getConnection().getClientInfo();
    }

    @Override
    public Array createArrayOf(String typeName, Object[] elements) throws SQLException {
        return getConnection().createArrayOf(typeName, elements);
    }

    @Override
    public Struct createStruct(String typeName, Object[] attributes) throws SQLException {
        return getConnection().createStruct(typeName, attributes);
    }

    @Override
    public <T> T unwrap(Class<T> iface) throws SQLException {
        return getConnection().unwrap(iface);
    }

    @Override
    public boolean isWrapperFor(Class<?> iface) throws SQLException {
        return getConnection().isWrapperFor(iface);
    }

    public static String translateSql(String sql) {
        for (Pair<String, String> prefixData : prefixDatas.values()) {
            sql = sql.replaceAll(prefixData.x, prefixData.y);
        }
        if (prefix != null) {

            for (Pattern pattern : PATTERNS_TO_TRANSLATE) {
                if (pattern.matcher(sql).matches()) {
                    for (String database : prefixDatas.keySet()) {
                        sql = sql.replaceFirst(database + "\\b", prefix + '_' + database);
                    }
                    break;
                }
            }
        }
        return sql;
    }

    public static String translateDbPrefix(String db) {
        if (prefix != null) {
            for (String database : prefixDatas.keySet()) {
                if (database.compareToIgnoreCase(db) == 0) {
                    return prefix + '_' + db;
                }
            }
        }
        return db;
    }

    @Override
    public int getNetworkTimeout() throws SQLException {
        return getConnection().getNetworkTimeout();
    }

    @Override
    public void setNetworkTimeout(Executor executor, int time) throws SQLException {
        getConnection().setNetworkTimeout(executor, time);
    }

    @Override
    public void abort(Executor executor) throws SQLException  {
        getConnection().abort(executor);
    }

    @Override
    public String getSchema() throws SQLException {
        return getConnection().getSchema();
    }

    @Override
    public void setSchema(String schema) throws SQLException {
         getConnection().setSchema(schema);
    }
}