package com.netapp.oci.platform.common.test;

import com.netapp.oci.platform.common.table.ConnectionHelper;
import junit.framework.TestCase;

import javax.sql.DataSource;
import java.util.Collections;
import java.sql.Statement;
import java.sql.Connection;


/**
 * This abstract test case takes care of common functionality like providing data source
 * so test case ca get connection or pass data source to components that require that data source.
 * <p/>
 * At the end of each test this class will make sure that all allocated connections, statements and result sets
 * were closed during test execution.
 * <p/>
 * Please DO NOT ADD more functionality in this class. The problem is right now unit tests taking too much
 * time to run and this class is supposed to be base of all test cases. So putting code to setUp() method
 * will increase time of execution of all tests in proportion with the number of tests.
 *
 * @author Anton Sergeev
 */
public class AbstractTestCase extends TestCase {
    private JdbcDataSourceStub testDataSource;

    public AbstractTestCase() {
    }

    public AbstractTestCase(String name) {
        super(name);
    }

    @Override
    protected void setUp() throws Exception {
        if (testDataSource == null) {
            testDataSource = new JdbcDataSourceStub();
        }

        testDataSource.reset();
    }

    public DataSource getDataSource() {
        return testDataSource;
    }

    protected void closeConnection(Connection connection) {
        ConnectionHelper.close(connection);
    }

    @Override
    protected void tearDown() throws Exception {
        assertEquals("Not all allocated connections were closed", testDataSource.getConnectionsOpened(), testDataSource.getConnectionsClosed());
        assertEquals("Not all allocated statements were closed", Collections.<Statement>emptyList(), testDataSource.getCurrentlyOpenedStatements());
        assertEquals("Not all allocated result sets were closed", testDataSource.getResultSetsOpened(), testDataSource.getResultSetsClosed());
    }
}
