/*
 * Copyright (c) 2010 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.common.test;


import com.netapp.oci.platform.common.table.ConnectionHelper;

import javax.sql.DataSource;
import java.io.PrintWriter;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.sql.*;
import java.util.*;
import java.util.concurrent.Executor;
import java.util.logging.Logger;

/**
 * Connection helper based data source to use for tests
 *
 * @author Anton Sergeev
 */
public class JdbcDataSourceStub implements DataSource {

    private Set<Statement> currentlyOpenedStatements = new HashSet<Statement>();

    private int connectionsOpened;
    private int connectionsClosed;

    private int resultSetsOpened;
    private int resultSetsClosed;


    public JdbcDataSourceStub() {
    }

    public List<Statement> getCurrentlyOpenedStatements() {
        return new ArrayList<Statement>(currentlyOpenedStatements);
    }

    public int getConnectionsOpened() {
        return connectionsOpened;
    }

    public int getConnectionsClosed() {
        return connectionsClosed;
    }

    public int getResultSetsOpened() {
        return resultSetsOpened;
    }

    public int getResultSetsClosed() {
        return resultSetsClosed;
    }

    /**
     * Resets all counters
     */
    public void reset() {
        currentlyOpenedStatements.clear();

        connectionsClosed = 0;
        connectionsOpened = 0;

        resultSetsClosed = 0;
        resultSetsOpened = 0;
    }

    @Override
    public Connection getConnection() throws SQLException {
        return new TestConnection(ConnectionHelper.newConnectionNoEjb());
    }

    @Override
    public Connection getConnection(String username, String password) throws SQLException {
        return null;
    }

    @Override
    public PrintWriter getLogWriter() throws SQLException {
        return null;
    }

    @Override
    public void setLogWriter(PrintWriter out) throws SQLException {
    }

    @Override
    public void setLoginTimeout(int seconds) throws SQLException {
    }

    @Override
    public int getLoginTimeout() throws SQLException {
        return 0;
    }

    private class TestConnection implements Connection {
        private Connection connection;

        public TestConnection(Connection connection) {
            this.connection = connection;
            connectionsOpened++;
        }

        @Override
        public Statement createStatement() throws SQLException {
            Statement statement = connection.createStatement();
            currentlyOpenedStatements.add(statement);
            return (Statement) Proxy.newProxyInstance(statement.getClass().getClassLoader(), new Class[]{Statement.class}, new StatementInvocationHander(statement));
        }

        @Override
        public PreparedStatement prepareStatement(String sql) throws SQLException {
            PreparedStatement preparedStatement = connection.prepareStatement(sql);
            currentlyOpenedStatements.add(preparedStatement);
            return (PreparedStatement) Proxy.newProxyInstance(preparedStatement.getClass().getClassLoader(), new Class[]{PreparedStatement.class}, new StatementInvocationHander(preparedStatement));
        }

        @Override
        public CallableStatement prepareCall(String sql) throws SQLException {
            CallableStatement callableStatement = connection.prepareCall(sql);
            currentlyOpenedStatements.add(callableStatement);
            return (CallableStatement) Proxy.newProxyInstance(callableStatement.getClass().getClassLoader(), new Class[]{CallableStatement.class}, new StatementInvocationHander(callableStatement));
        }

        @Override
        public String nativeSQL(String sql) throws SQLException {
            return connection.nativeSQL(sql);
        }

        @Override
        public void setAutoCommit(boolean autoCommit) throws SQLException {
            connection.setAutoCommit(autoCommit);
        }

        @Override
        public boolean getAutoCommit() throws SQLException {
            return connection.getAutoCommit();
        }

        @Override
        public void commit() throws SQLException {
            connection.commit();
        }

        @Override
        public void rollback() throws SQLException {
            connection.rollback();
        }

        @Override
        public void close() throws SQLException {
            connection.close();
            connectionsClosed++;
        }

        @Override
        public boolean isClosed() throws SQLException {
            return connection.isClosed();
        }

        @Override
        public DatabaseMetaData getMetaData() throws SQLException {
            return connection.getMetaData();
        }

        @Override
        public void setReadOnly(boolean readOnly) throws SQLException {
            connection.setReadOnly(readOnly);
        }

        @Override
        public boolean isReadOnly() throws SQLException {
            return connection.isReadOnly();
        }

        @Override
        public void setCatalog(String catalog) throws SQLException {
            connection.setCatalog(catalog);
        }

        @Override
        public String getCatalog() throws SQLException {
            return connection.getCatalog();
        }

        @Override
        public void setTransactionIsolation(int level) throws SQLException {
            connection.setTransactionIsolation(level);
        }

        @Override
        public int getTransactionIsolation() throws SQLException {
            return connection.getTransactionIsolation();
        }

        @Override
        public SQLWarning getWarnings() throws SQLException {
            return connection.getWarnings();
        }

        @Override
        public void clearWarnings() throws SQLException {
            connection.clearWarnings();
        }

        @Override
        public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
            Statement statement = connection.createStatement(resultSetType, resultSetConcurrency);
            currentlyOpenedStatements.add(statement);
            return (Statement) Proxy.newProxyInstance(statement.getClass().getClassLoader(), new Class[]{Statement.class}, new StatementInvocationHander(statement));
        }

        @Override
        public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
            PreparedStatement preparedStatement = connection.prepareStatement(sql, resultSetType, resultSetConcurrency);
            currentlyOpenedStatements.add(preparedStatement);
            return (PreparedStatement) Proxy.newProxyInstance(preparedStatement.getClass().getClassLoader(), new Class[]{PreparedStatement.class}, new StatementInvocationHander(preparedStatement));
        }

        @Override
        public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
            CallableStatement callableStatement = connection.prepareCall(sql, resultSetType, resultSetConcurrency);
            currentlyOpenedStatements.add(callableStatement);
            return (CallableStatement) Proxy.newProxyInstance(callableStatement.getClass().getClassLoader(), new Class[]{CallableStatement.class}, new StatementInvocationHander(callableStatement));
        }

        @Override
        public Map<String, Class<?>> getTypeMap() throws SQLException {
            return connection.getTypeMap();
        }

        @Override
        public void setTypeMap(Map<String, Class<?>> map) throws SQLException {
            connection.setTypeMap(map);
        }

        @Override
        public void setHoldability(int holdability) throws SQLException {
            connection.setHoldability(holdability);
        }

        @Override
        public int getHoldability() throws SQLException {
            return connection.getHoldability();
        }

        @Override
        public Savepoint setSavepoint() throws SQLException {
            return connection.setSavepoint();
        }

        @Override
        public Savepoint setSavepoint(String name) throws SQLException {
            return connection.setSavepoint(name);
        }

        @Override
        public void rollback(Savepoint savepoint) throws SQLException {
            connection.rollback(savepoint);
        }

        @Override
        public void releaseSavepoint(Savepoint savepoint) throws SQLException {
            connection.releaseSavepoint(savepoint);
        }

        @Override
        public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
            Statement statement = connection.createStatement(resultSetType, resultSetConcurrency, resultSetHoldability);
            currentlyOpenedStatements.add(statement);
            return (Statement) Proxy.newProxyInstance(statement.getClass().getClassLoader(), new Class[]{Statement.class}, new StatementInvocationHander(statement));
        }

        @Override
        public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
            PreparedStatement preparedStatement = connection.prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
            currentlyOpenedStatements.add(preparedStatement);
            return (PreparedStatement) Proxy.newProxyInstance(preparedStatement.getClass().getClassLoader(), new Class[]{PreparedStatement.class}, new StatementInvocationHander(preparedStatement));
        }

        @Override
        public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
            CallableStatement callableStatement = connection.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
            currentlyOpenedStatements.add(callableStatement);
            return (CallableStatement) Proxy.newProxyInstance(callableStatement.getClass().getClassLoader(), new Class[]{CallableStatement.class}, new StatementInvocationHander(callableStatement));
        }

        @Override
        public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
            PreparedStatement preparedStatement = connection.prepareStatement(sql, autoGeneratedKeys);
            currentlyOpenedStatements.add(preparedStatement);
            return (PreparedStatement) Proxy.newProxyInstance(this.getClass().getClassLoader(), new Class[]{PreparedStatement.class}, new StatementInvocationHander(preparedStatement));
        }

        @Override
        public PreparedStatement prepareStatement(String sql, int[] columnIndexes) throws SQLException {
            PreparedStatement preparedStatement = connection.prepareStatement(sql, columnIndexes);
            currentlyOpenedStatements.add(preparedStatement);
            return (PreparedStatement) Proxy.newProxyInstance(preparedStatement.getClass().getClassLoader(), new Class[]{PreparedStatement.class}, new StatementInvocationHander(preparedStatement));
        }

        @Override
        public PreparedStatement prepareStatement(String sql, String[] columnNames) throws SQLException {
            PreparedStatement preparedStatement = connection.prepareStatement(sql, columnNames);
            currentlyOpenedStatements.add(preparedStatement);
            return (PreparedStatement) Proxy.newProxyInstance(preparedStatement.getClass().getClassLoader(), new Class[]{PreparedStatement.class}, new StatementInvocationHander(preparedStatement));
        }

        @Override
        public Clob createClob() throws SQLException {
            return connection.createClob();
        }

        @Override
        public Blob createBlob() throws SQLException {
            return connection.createBlob();
        }

        @Override
        public NClob createNClob() throws SQLException {
            return connection.createNClob();
        }

        @Override
        public SQLXML createSQLXML() throws SQLException {
            return connection.createSQLXML();
        }

        @Override
        public boolean isValid(int timeout) throws SQLException {
            return connection.isValid(timeout);
        }

        @Override
        public void setClientInfo(String name, String value) throws SQLClientInfoException {
            connection.setClientInfo(name, value);
        }

        @Override
        public void setClientInfo(Properties properties) throws SQLClientInfoException {
            connection.setClientInfo(properties);
        }

        @Override
        public String getClientInfo(String name) throws SQLException {
            return connection.getClientInfo(name);
        }

        @Override
        public Properties getClientInfo() throws SQLException {
            return connection.getClientInfo();
        }

        @Override
        public Array createArrayOf(String typeName, Object[] elements) throws SQLException {
            return connection.createArrayOf(typeName, elements);
        }

        @Override
        public Struct createStruct(String typeName, Object[] attributes) throws SQLException {
            return connection.createStruct(typeName, attributes);
        }

        @Override
        public <T> T unwrap(Class<T> iface) throws SQLException {
            return connection.unwrap(iface);
        }

        @Override
        public boolean isWrapperFor(Class<?> iface) throws SQLException {
            return connection.isWrapperFor(iface);
        }

        @Override
        public int getNetworkTimeout() throws SQLException {
            return connection.getNetworkTimeout();
        }

        @Override
        public void setNetworkTimeout(Executor executor, int time) throws SQLException {
            connection.setNetworkTimeout(executor, time);
        }

        @Override
        public void abort(Executor executor) throws SQLException  {
            connection.abort(executor);
        }

        @Override
        public String getSchema() throws SQLException {
            return connection.getSchema();
        }

        @Override
        public void setSchema(String schema) throws SQLException {
            connection.setSchema(schema);
        }
    }

    @Override
    public <T> T unwrap(Class<T> iface) throws SQLException {
        return null;
    }

    @Override
    public boolean isWrapperFor(Class<?> iface) throws SQLException {
        return false;
    }

    private class StatementInvocationHander implements InvocationHandler {
        private Statement statement;

        private StatementInvocationHander(Statement statement) {
            this.statement = statement;
        }

        @Override
        public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
            if (method.getName().equals("close")) {
                currentlyOpenedStatements.remove(statement);
            }

            if (method.getName().equals("executeQuery") ||
                    method.getName().equals("getResultSet") ||
                    method.getName().equals("getGeneratedKeys")) {
                resultSetsOpened++;
                ResultSet resultSet = (ResultSet) method.invoke(statement, args);
                return Proxy.newProxyInstance(this.getClass().getClassLoader(), new Class[]{ResultSet.class}, new ResultSetInvocationHander(resultSet));
            }
            return method.invoke(statement, args);
        }
    }

    private class ResultSetInvocationHander implements InvocationHandler {
        private ResultSet resultSet;

        private ResultSetInvocationHander(ResultSet resultSet) {
            this.resultSet = resultSet;
        }

        @Override
        public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
            if (method.getName().equals("close")) {
                resultSetsClosed++;
            }

            return method.invoke(resultSet, args);
        }
    }

    @Override
    public Logger getParentLogger()  throws SQLFeatureNotSupportedException {
        // Since this class is used for test, ignore the parent logger.
        return null;
    }

}
