/*
 * Copyright (c) 2010 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.common.util;

import java.text.MessageFormat;

/**
 * Used to throw different kind of run time exceptions in case
 * method parameters or intermidiate variables do not belong to range of valid values.
 *
 * @author Anton Sergeev
 */
public class Assert {
    /**
     * Private constructor to prevent unwanted instances of this class
     */
    private Assert() {
    }

    /**
     * Throws Illegal argument exception if boolean expression is not <code>true</code>
     *
     * @param isLegal - boolean expression
     * @param message - exception message
     * @throws IllegalArgumentException - in case boolean expression doesn't evaluate to <code>true</code>
     */
    public static void isLegal(boolean isLegal, String message) throws IllegalArgumentException {
        if (!isLegal) {
            throw new IllegalArgumentException(message);
        }
    }

    public static void isLegal(boolean isLegal, String format, Object... args) throws IllegalArgumentException {
        if (!isLegal) {
            throw new IllegalArgumentException(MessageFormat.format(format, args));
        }
    }

    /**
     * Throws Illegal argument exception if object is null
     *
     * @param object  - object to check for null
     * @param message - exception message
     * @throws IllegalArgumentException - in case object is null
     */
    public static void isNotNull(Object object, String message) throws IllegalArgumentException {
        if (object == null) {
            throw new IllegalArgumentException(message);
        }
    }

    public static void isNotNull(Object object) throws IllegalArgumentException {
        if (object == null) {
            throw new IllegalArgumentException();
        }
    }

    public static void isNotNull(Object object, String format, Object... args) throws IllegalArgumentException {
        if (object == null) {
            throw new IllegalArgumentException(MessageFormat.format(format, args));
        }
    }

    /**
     * Throws Illegal argument exception if boolean expression is not <code>true</code>
     *
     * @param isLegal - boolean expression
     * @throws IllegalArgumentException - in case boolean expression doesn't evaluate to <code>true</code>
     */
    public static void isLegal(boolean isLegal) throws IllegalArgumentException {
        if (!isLegal) {
            throw new IllegalArgumentException();
        }
    }

    /**
     * Throws assertion failed exception if boolean expression is not <code>true</code>
     *
     * @param isTrue - boolean expression
     * @throws RuntimeException - in case boolean expression doesn't evaluate to <code>true</code>
     */
    public static void isTrue(boolean isTrue) throws RuntimeException {
        if (!isTrue) {
            throw new RuntimeException("Is not true");
        }
    }

    /**
     * Throws assertion failed exception if boolean expression is not <code>true</code>
     *
     * @param isTrue  - boolean expression
     * @param message - message to be thrown with exception
     * @throws RuntimeException - in case boolean expression doesn't evaluate to <code>true</code>
     */
    public static void isTrue(boolean isTrue, String message) throws RuntimeException {
        if (!isTrue) {
            throw new RuntimeException(message);
        }
    }

    public static void isTrue(boolean isTrue, String format, Object... args) throws RuntimeException {
        if (!isTrue) {
            throw new RuntimeException(MessageFormat.format(format, args));
        }
    }
}
