/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.db;

import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author Al Velella
 *         Date: Aug 22, 2012
 *         Time: 3:29:16 PM
 */
public class SqlFile {
    public static void executeFile(Connection connection, String file, Class<?> resourceClass, String dirBase, String path) throws IOException, TableException {
        List<String> statements = read(file, resourceClass, dirBase, path);
        for (String sql : statements) {
            PreparedStatementWrapper.createForUpdate(connection, sql).executeUpdateEx();
        }
    }

    public static List<String> loadContents(String file, Class<?> resourceClass, String dirBase, String path) throws IOException {
        return read(file, resourceClass, dirBase, path);
    }

    private static List<String> read(String file, Class<?> resourceClass, String dirBase, String path) throws IOException {
        Pattern delimiterFinder = Pattern.compile("^DELIMITER(.*)$");
        String customDelimiter = null;
        boolean customDelimiterUsed = false;
        List<String> statements = new ArrayList<String>();

        InputStream is = null;
        InputStreamReader isr = null;
        BufferedReader br = null;
        try {
            is = resourceClass.getResourceAsStream(path + file.substring(dirBase.length()));
            isr = new InputStreamReader(is);
            br = new BufferedReader(isr);
            StringBuilder sb = new StringBuilder();
            String line;
            while ((line = br.readLine()) != null) {
                boolean skipRestOfLine = false;
                boolean comment = false;
                boolean singleQuoteOpen = false;
                boolean doubleQuoteOpen = false;
                boolean escape = false;

                Matcher delimiterMatcher = delimiterFinder.matcher(line);
                if(delimiterMatcher.matches()) {
                    String delimiterStr = delimiterMatcher.group(1);
                    if(delimiterStr != null) {
                        if(";".equals(delimiterStr.trim())) {
                            customDelimiter = null;
                            customDelimiterUsed = false;
                        } else {
                            customDelimiter = delimiterStr.trim();
                            customDelimiterUsed = true;
                        }
                    }
                    continue;
                }
                if(customDelimiterUsed && customDelimiter.equals(line.trim())) {
                    // End of procedure;
                    sb = newStatement(statements, sb);
                    continue;
                }
                
                for (int i = 0; i < line.length(); i++) {
                    char character = line.charAt(i);
                    if (escape) {
                        escape = false;
                        sb.append(character);
                    } else {
                        if (comment && character != '-') {
                            sb.append('-');
                            comment = false;
                        }
                        switch (character) {
                            case '\\':
                                if (singleQuoteOpen || doubleQuoteOpen) {
                                    escape = true;
                                }
                                sb.append(character);
                                break;
                            case'\"':
                                if (!singleQuoteOpen) {
                                    doubleQuoteOpen = !doubleQuoteOpen;
                                }
                                sb.append(character);
                                break;
                            case'\'':
                                if (!doubleQuoteOpen) {
                                    singleQuoteOpen = !singleQuoteOpen;
                                }
                                sb.append(character);
                                break;
                            case '-':
                                if (singleQuoteOpen || doubleQuoteOpen) {
                                    sb.append(character);
                                } else if (comment) {
                                    skipRestOfLine = true;
                                } else {
                                    comment = true;
                                }
                                break;
                            case ';':
                                if (singleQuoteOpen || doubleQuoteOpen || customDelimiterUsed) {
                                    sb.append(character);
                                } else {
                                    sb = newStatement(statements, sb);
                                }
                                break;
                            default:
                                sb.append(character);
                        }
                    }
                    if (skipRestOfLine) {
                        break;
                    }
                }

                sb.append(' '); // replace end-of-line with space for multi-line statements
            }

            newStatement(statements, sb);
            return statements;
        } finally {
            if (br != null) {
                br.close();
            }
            if (isr != null) {
                isr.close();
            }
            if (is != null) {
                is.close();
            }
        }
    }

    private static StringBuilder newStatement(List<String> statements, StringBuilder sb) {
        String statement = sb.toString();
        if (!statement.matches("^\\s*$")) { // skip empty lines
            statements.add(statement);
        }
        sb = new StringBuilder();
        return sb;
    }
}
