package com.netapp.oci.platform.framework.performance;

import org.slf4j.Logger;

/**
 * Factory that creates all compoenent implementations of the standalone performance package/plugin/project.
 *
 * @author Dongjun Sun
 * @author Anton Sergeev
 */
public interface PerformancePackageFactory<C, S> {
    /**
     * Sample facade is resposible for returning samples for different time periods
     *
     * @return sample facade
     */
    SampleFacade<C> getSampleFacade();

    /**
     * Summary facae is resposible for returning summaries for different time periods
     *
     * @return summary facade
     */
    SummaryFacade<S> getSummaryFacade();

    /**
     * Persistence strategy is class that takes care of the task of persisting samples
     * in different forms
     *
     * @return persistence strategy
     */
    PersistenceStrategy<C> getPersistenceStrategy();

    /**
     * Tables management is resposible for lifecycle of the tables
     *
     * @return table management
     */
    TableManagement getTableManagement();

     /**
     * For better readable logging, use the logger - that will help differentiate between
     * different performance packages (such as AP or VM) in the log file
     *
     * @return Logger Object
     */
    Logger getLogger();    
}
