package com.netapp.oci.platform.framework.performance.data;

/**
 * Server-side presentation of samples recorded by the acquisition. Defers from {@link com.onaro.performance.interfaces.data.RawSample}
 * by adding the sample time and translating the object-key identifier to object-id (number). 
 */
public class Sample<C> implements Comparable<Sample>  {
    /**
     * The ID of object described by the counters.
     */
    private Long id;

    /**
     * The time the counters were retrieved by the server
     */
    private long time;

    /**
     * The value recorded by the acquisition. May be null to represent the first sample of a datasource.
     */
    private C sampleData;

    public Sample() {
        time = -1;
    }

    public Sample(Long id, long time) {
        this.id = id;
        this.time = time;
    }

    public Sample(Long id, long time, C sampleData) {
        this.id = id;
        this.time = time;
        this.sampleData = sampleData;
    }


    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public long getTime() {
        return time;
    }

    public void setTime(long time) {
        this.time = time;
    }

    public C getSampleData() {
        return sampleData;
    }

    public void setSampleData(C sampleData) {
        this.sampleData = sampleData;
    }

    public int compareTo(Sample sample2) {
        if (getTime() == sample2.getTime()) {
            return 0;
        }

        return getTime() < sample2.getTime() ? -1 : 1;
    }

    /**
     * Tests if this instance represetns the "null" counters - the first sample of a data-source.
     * @return true if no counters are set in this sample
     */
    public boolean isNull() {
        return sampleData == null;
    }


    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Sample sample = (Sample) o;

        if (!id.equals(sample.id)) return false;
        if (time != sample.time) return false;
        if (sampleData != null ? !sampleData.equals(sample.sampleData) : sample.sampleData != null) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = (int) (id ^ (id >>> 32));
        result = 31 * result + (int) (time ^ (time >>> 32));
        result = 31 * result + (sampleData != null ? sampleData.hashCode() : 0);
        return result;
    }

    public String toString() {
        return "Sample{" +
                "id=" + id +
                ", time=" + time +
                ", counters=" + sampleData +
                '}';
    }
}
