/*
 * Copyright (c) 2009 NetApp
 * All rights reserved
 */
package com.netapp.oci.platform.framework.performance.tables;

import com.netapp.oci.platform.common.table.*;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.onaro.commons.table.DbConfig;
import com.netapp.oci.platform.framework.performance.data.CurrentSampleStatus;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @author Anton Sergeev
 */
public class CurrentSampleStatusTable extends Table<CurrentSampleStatus> {
    public CurrentSampleStatusTable(Connection connection, String database) {
        super(connection, database, "sample_current_status");
    }

    @Override
    public void insert(CurrentSampleStatus status) throws TableException {
        PreparedStatementWrapper<CurrentSampleStatus> preparedStatement = null;
        String sql = "INSERT INTO " + getFullName() + "(package, containerkey, containerid, lastsampletime) VALUES(?, ?, ?, ?)"
            + " ON DUPLICATE KEY UPDATE lastsampletime=VALUES(lastsampletime)";

        try {
            preparedStatement = getPrepareStatementEx(sql);
            preparedStatement.setNext(status.getPerformancePackageId());
            preparedStatement.setNext(status.getSampleContainerKey());
            preparedStatement.setNext(status.getSampleContainerId());
            preparedStatement.setNext(status.getLastSampleTime());
            preparedStatement.executeUpdate();
        } catch (SQLException e) {
            throw new TableException("Failed on insert: " + sql, e);
        } finally {
            closeConnection(preparedStatement, null);
        }
    }

    @Override
    public CurrentSampleStatus parseObject(ResultSet resultSet) throws SQLException {
        int i = 1;
        return new CurrentSampleStatus(
                StringCache.get(resultSet.getString(i++)),
                StringCache.get(resultSet.getString(i++)),
                new Long(resultSet.getLong(i++)),
                resultSet.getLong(i)
                );
    }
}
