/*
 * Copyright (c) 2009 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.framework.performance.tables;

import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.data.Sample;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * Sample table to hold current sample data, base class
 *
 * @author Anton Sergeev
 */
public abstract class CurrentSampleTable<C> extends SampleTable<C> {
    public static String TABLE_NAME_PREFIX = "current_";

    private static Column [] CURRENT_SAMPLE_TABLE_COLUMNS = {
            new Column("containerkey", "varchar(255)"),
    };

    protected String containerKey;

    protected CurrentSampleTable(Connection connection, ObjectParser<C> sampleDataParser, String containerKey, String suffixName, Column[] columns) {
        super(connection, sampleDataParser, TABLE_NAME_PREFIX + suffixName, mergeColumns(CURRENT_SAMPLE_TABLE_COLUMNS, columns));
        this.containerKey = containerKey;
    }

    @Override
    protected int prepareSystemDataForInsert(PreparedStatementWrapper wrapper) throws TableException {
        int numberOfColumns = super.prepareSystemDataForInsert(wrapper);
        wrapper.setNext(containerKey);
        return numberOfColumns + 1;
    }

    @Override
    protected C parseSampleData(ResultSet resultSet, int startIndex) throws SQLException {
        return super.parseSampleData(resultSet, startIndex + 1);
    }

    public List<Sample<C>> findAllByContainerKey(String containerKey) throws TableException {
        return getPrepareStatementEx("SELECT * FROM " + getFullName() + " WHERE containerkey=?").setNext(containerKey).executeQueryEx();
    }
}
