package com.netapp.oci.platform.framework.performance.tables;

import java.util.List;
import java.util.LinkedList;
import java.util.Arrays;

/**
 * @author Anton Sergeev
 */
public class Index {
    private String name;

    boolean primaryKey;

    boolean unique;

    private List<Column> columns;

    public Index(boolean primaryKey, boolean unique, Column... columns) {
        this.primaryKey= primaryKey;
        this.unique= unique;
        this.columns = new LinkedList<Column>();
        this.columns.addAll(Arrays.asList(columns));
        StringBuilder str = new StringBuilder();
        // index name derive from columns name
        for (Column column :columns){
            str.append(column.getName()).append("_");
        }
        str.append("index");
        name = str.toString();

    }

    public Index(Column... columns) {
        this(false, false, columns);
    }

    public String getName() {
        return name;
    }

    public List<Column> getColumns() {
        return columns;
    }

    public String value() {
        StringBuilder builder = new StringBuilder();
        if (primaryKey) {
            builder.append("PRIMARY KEY ");
        } else {
            if (unique)
                builder.append("UNIQUE ");
            builder.append("KEY ").append(name);
        }
        builder.append("(");
        int i = 0;
        for (Column column: columns) {
            if (i != 0) {
                builder.append(",");
            }
            i++;
            builder.append(column.getName());
        }
        builder.append(")");
        return builder.toString();
    }

    public String toString() {
        return value();
    }
    
}
