package com.netapp.oci.platform.framework.performance.tables;

import com.netapp.oci.platform.common.util.Assert;

import java.text.MessageFormat;

/**
 * @author Anton Sergeev
 */
public class SqlMethod {
    public static SqlMethod SUM = new SqlMethod("SUM({0})");
    public static SqlMethod AVG = new SqlMethod("AVG({0})");
    public static SqlMethod MAX = new SqlMethod("MAX({0})");
    public static SqlMethod MIN = new SqlMethod("MIN({0})");
    //Cumulative moving average
    public static SqlMethod CMA = new SqlMethod("coalesce( (values({0}) + ({0} * noOfSamples)) / (noOfSamples + 1), {0}, values({0}) )");
    //Moving Max
    public static SqlMethod M_MAX = new SqlMethod("coalesce( greatest(values({0}), {0}), {0}, values({0}) )");
    //Moving Min
    public static SqlMethod M_MIN = new SqlMethod("coalesce( least(values({0}), {0}), {0}, values({0}) )");
    //Cumulative Sum
    public static SqlMethod C_SUM = new SqlMethod("coalesce( values({0}) + {0}, {0}, values({0}) )");

    private String methodBody; // in the form of SUM(0), AVG(0)

    public SqlMethod(String methodBody) {
        this.methodBody = methodBody;
    }

    public String wrapColumn(String columnName) {
        return MessageFormat.format(methodBody, columnName);
    }

    public static String getWrappedColumns(String prefix, Column[] columns, SqlMethod[] methods) {
        Assert.isNotNull(columns);
        Assert.isNotNull(methods);
        Assert.isTrue(columns.length == methods.length);

        StringBuilder sql = new StringBuilder();
        for (int i = 0; i < columns.length; i++) {
            Column column = columns[i];
            SqlMethod method = methods[i];
            sql.append(",\r\n    ");
            sql.append(method.wrapColumn(prefix + column.getName()));
            sql.append(" AS ").append(column.getName());
        }
        return sql.toString();
    }

    public static String getWrappedColumns(String prefix, Column[] columns, SqlMethod[][] methods) {
        Assert.isNotNull(columns);
        Assert.isNotNull(methods);
        Assert.isTrue(columns.length == methods.length);

        StringBuilder sql = new StringBuilder();
        for (int i = 0; i < columns.length; i++) {
            Column column = columns[i];
            SqlMethod[] columnMethods = methods[i];
            for (int j = 0; j < columnMethods.length; j++) {
                SqlMethod method = columnMethods[j];
                if (method == null){
                	continue;
                }
                sql.append(",\r\n    ");
                sql.append(method.wrapColumn(prefix + column.getName()));
                sql.append(" AS ").append(column.getName()).append('_').append(j);
            }
        }
        return sql.toString();
    }

}
