/*
 * Copyright (c) 2009 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.framework.performance.tables;

import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.data.Sample;

import java.sql.Connection;
import java.util.Collection;

/**
 * Implementation that uses update operation for current samples.
 * 
 * @author Anton Sergeev
 */
public class UpdateCurrentSampleTable<C> extends CurrentSampleTable<C> {
    public UpdateCurrentSampleTable(Connection connection, ObjectParser<C> sampleDataParser, String containerKey, String suffixName, Column[] columns) {
        super(connection, sampleDataParser, containerKey, suffixName, columns);
    }

    @Override
    public void insert(Collection<Sample<C>> samples) throws TableException {
        super.insert(samples);
        long sampleTime = samples.iterator().next().getTime();
        getPrepareStatementEx("DELETE FROM " + getFullName() + " WHERE containerkey=? AND time < ?")
                .setNext(containerKey).setNext(sampleTime).executeUpdateEx();
    }

    public String getInsertSql() {
        StringBuilder sql = new StringBuilder();
        sql.append("INSERT INTO ").append(getFullName()).append("(");
        int i = 0;
        for (Column column: getColumns()) {
            if (i != 0) {
                sql.append(", ");
            }
            i++;
            sql.append(column.getName());
        }
        sql.append(") VALUES (");
        for (int j = 0; j < getColumns().length; j++) {
            if (j != 0) {
                sql.append(", ");
            }
            sql.append("?");
        }
        sql.append(") ON DUPLICATE KEY UPDATE ");
        i = 0;
        for (Column column : getColumns()) {
            if (i != 0) {
                sql.append(", ");
            }
            sql.append(column.getName()).append("=VALUES(").append(column.getName()).append(")");
            i++;
        }
        return sql.toString();
    }
}
