package com.netapp.oci.platform.framework.performance.test;

import com.netapp.oci.platform.common.TimeSpan;
import com.netapp.oci.platform.framework.performance.data.Sample;
import junit.framework.Assert;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Map;

/**
 * Has some utilities to assert samples and summaries
 *
 * @author Anton Sergeev
 */
public class AssertUtil {

    private AssertUtil() {
    }

    public static <C> void assertEquals(Map<Long, List<Sample<C>>> idToExpectedSamples, Map<Long, List<Sample<C>>> idToSamplesToTest) {
        Assert.assertEquals("Samples map size should be the same", idToExpectedSamples.size(), idToSamplesToTest.size());

        for (Long id : idToExpectedSamples.keySet()) {
            Assert.assertEquals("Samples list size should be the same", idToExpectedSamples.get(id).size(), idToSamplesToTest.get(id).size());
        }

        for (Long id : idToExpectedSamples.keySet()) {
            List<Sample<C>> expectedSamples = idToExpectedSamples.get(id);
            List<Sample<C>> samplesToTest = idToSamplesToTest.get(id);
            for (int i = 0; i < idToExpectedSamples.get(id).size(); i++) {
                Sample expectedSample = expectedSamples.get(i);
                Sample sampleToTest = samplesToTest.get(i);

                Assert.assertEquals("Time is different in sample " + i, expectedSample.getTime(), sampleToTest.getTime());
                Assert.assertEquals("Id is different in sample " + i, expectedSample.getId(), sampleToTest.getId());
                Assert.assertEquals("Sample data is different in sample " + i, expectedSample.getSampleData(), sampleToTest.getSampleData());
            }
        }
    }

    public static <C> void assertEquals(List<Sample<C>> expectedSamples, List<Sample<C>> samplesToTest) {
        Assert.assertEquals("Samples list size should be the same", expectedSamples.size(), samplesToTest.size());

        for (int i = 0; i < expectedSamples.size(); i++) {
            Sample expectedSample = expectedSamples.get(i);
            Sample sampleToTest = samplesToTest.get(i);

            Assert.assertEquals("Time is different in sample " + i, expectedSample.getTime(), sampleToTest.getTime());
            Assert.assertEquals("Id is different in sample " + i, expectedSample.getId(), sampleToTest.getId());
            Assert.assertEquals("Sample data is different in sample " + i, expectedSample.getSampleData(), sampleToTest.getSampleData());
        }
    }

    /**
     * Validates time span of any summary or compressed sample for being
     * one hour or several hours that start with 0 minutes, seconds and millis
     *
     * @param timeSpan - time span to validate
     */
    public static void validateHourTimeSpan(TimeSpan timeSpan) {
        Calendar fromTime = new GregorianCalendar();
        fromTime.setTimeInMillis(timeSpan.getFromTime());

        Calendar toTime = new GregorianCalendar();
        toTime.setTimeInMillis(timeSpan.getToTime());

        Assert.assertEquals(fromTime.get(Calendar.MINUTE), 0);
        Assert.assertEquals(fromTime.get(Calendar.SECOND), 0);
        Assert.assertEquals(fromTime.get(Calendar.MILLISECOND), 0);

        Assert.assertEquals(toTime.get(Calendar.MINUTE), 59);
        Assert.assertEquals(toTime.get(Calendar.SECOND), 59);
        Assert.assertEquals(toTime.get(Calendar.MILLISECOND), 999);
    }
}
