/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.framework.performance.test;

import com.netapp.oci.platform.common.TimeSpan;
import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.framework.performance.data.Sample;
import com.netapp.oci.platform.framework.performance.data.Summary;
import com.netapp.oci.platform.framework.performance.util.SummaryCalculator;
import com.onaro.commons.types.Period;
import org.junit.Assert;
import org.junit.Test;

import java.util.List;

/**
 * @author Anton Sergeev
 */
public abstract class SummarizePerformancePackageTestCase<C, S> extends AbstractPerformancePackageTestCase<C, S> {
    protected SummaryCalculator<C, S> getSummaryCalculator() {
        return null;
    }

    @Test
    public void testSummaryCalculator() throws PerformanceException {
        Long id = Long.valueOf(1);

        SummaryCalculator<C, S> summaryCalculator = getSummaryCalculator();

        // not all packages have to implement summary calculator, some generate data from sample table directly
        // by using MAX, MIN and such methods
        if (summaryCalculator != null) {
            // generates summary
            long now = System.currentTimeMillis();
            TimeSpan hour = generateHourTimeSpan(now);
            List<Sample<C>> samples = generateSamples(id, hour, 10 * Period.MINUTE.milliseconds());
            Summary<S> summary = summaryCalculator.calculateSummary(samples, hour);

            Assert.assertEquals(summary.getId(), id);
            AssertUtil.validateHourTimeSpan(summary.getTimeSpan());
            Assert.assertEquals(summary.getTimeSpan(), hour);
        }
    }
}
