package com.netapp.oci.platform.framework.performance.util;

import com.netapp.oci.platform.common.TimeSpan;
import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.common.util.Assert;
import com.netapp.oci.platform.framework.performance.data.Summary;
import com.netapp.oci.platform.framework.performance.data.Sample;

import java.util.List;

/**
 * Provides some basic reusable code for summary calculators
 *
 * @author Anton Sergeev
 */
public abstract class AbstractSummaryCalculator<C, S> implements SummaryCalculator<C, S> {
    /**
     * @see SummaryCalculator#calculateSummary(java.util.List, com.netapp.oci.platform.common.TimeSpan)
     */
    public Summary<S> calculateSummary(List<Sample<C>> samples, TimeSpan timeSpan) throws PerformanceException {
        Assert.isLegal(samples != null && samples.size() > 0, " Samples can not be null or empty");
        if (samples.size() == 1)
            return null;

        Long id = samples.get(0).getId();

        // summaryData can be null if there are only two samples
        // and the the diff between the first sample and the last is negetive
        S summaryData = calculateSummary(samples, timeSpan, id);
        return new Summary<S>(id, timeSpan, summaryData);
    }

    protected abstract S calculateSummary(List<Sample<C>> samples, TimeSpan timeSpan, Long id) throws PerformanceException;

    // TODO-HV-HLP-MICHAEL - implement this in all the calculators

    public Summary<S> mergeSummaries(Summary<S> sum1, Summary<S> sum2) throws PerformanceException {
        return sum1;
    }
}
