package com.netapp.oci.platform.framework.performance.util;

import java.sql.Connection;

import com.netapp.oci.platform.framework.performance.tables.DailySummaryTable;
import org.slf4j.Logger;

import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.SummaryTable;


/**
 * Add summary to the abstract table management
 *
 * @param <C> Sample measurement
 * @param <S> Summary measurement
 */
public abstract class AbstractSummaryTableManagement<C,S> extends AbstractTableManagement <C>{

	protected AbstractSummaryTableManagement(Logger logger) {
		super(logger);
	}

    /**
     * Abstract method to get instance of summary table
     *
     * @param connection - database connection
     * @return summary table
     */
    protected abstract SummaryTable<S> getSummaryTable(Connection connection);

    /**
     * Abstract method to get instance of daily summary table
     *
     * @param connection - database connection
     * @return daily summary table
     */
    protected abstract DailySummaryTable<C> getDailySummaryTable(Connection connection);

    @Override
    protected void createMoreTables(Connection connection) throws TableException {
        SummaryTable<S> summaryTable = getSummaryTable(connection);
        summaryTable.createTable();
        final DailySummaryTable<C> dailySummaryTable = getDailySummaryTable(connection);
        if(dailySummaryTable != null) {
            dailySummaryTable.createTable();
        }
        logger.info("Table Management: Created table " + summaryTable.getFullName());
    }

    @Override
    protected void dropMoreTables(Connection connection) throws TableException {
        SummaryTable<S> summaryTable = getSummaryTable(connection);
        summaryTable.dropTable();
        final DailySummaryTable<C> dailySummaryTable = getDailySummaryTable(connection);
        if(dailySummaryTable != null) {
            dailySummaryTable.dropTable();
        }
        logger.info("Table Management: Dropped table " + summaryTable.getFullName());
    }
    
    protected boolean moreTablesExist(Connection connection) throws TableException {
        boolean moreTableExists;

        SummaryTable<S> summaryTable = getSummaryTable(connection);
        moreTableExists = summaryTable.isTableExist();

        final DailySummaryTable<C> dailySummaryTable = getDailySummaryTable(connection);
        if(dailySummaryTable != null) {
            moreTableExists = moreTableExists && dailySummaryTable.isTableExist();
        }
        return moreTableExists;
    }
}
