/*
 * Copyright (c) 2009 NetApp
 * All rights reserved
 */
package com.netapp.oci.platform.framework.performance.util;

import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.common.param.PluginParam;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.InsertSelectSummaryTable;
import com.netapp.oci.platform.framework.performance.tables.SampleTable;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;
import org.slf4j.Logger;

import java.sql.Connection;

/**
 * Summarization persistence strategy based on the usage of insert select summary table
 *
 * @author Anton Sergeev
 */
public abstract class InsertSelectSummarizePersistenceStrategy<C, S> extends SummarizePersistenceStrategy<C, S> {
    protected InsertSelectSummarizePersistenceStrategy(Logger logger, PluginParam param, String sampleContainerKey, Long sampleContainerId) {
        super(logger, param, sampleContainerKey, sampleContainerId);
    }

    protected abstract SqlMethod[][] getSummarizationMethods();

    protected void summarize(Connection connection, SampleTable<C> sampleTable, Long hourToSummarize) throws PerformanceException {
        int retryLimit = 10;
        InsertSelectSummaryTable<S> summaryTable = (InsertSelectSummaryTable<S>) getSummaryTable(connection);

        // burt 1070897, add 10 times retry to solve mysql deadlock issue
        for (int i = 0; i < retryLimit; i++) {
            try {
                summaryTable.insert(sampleTable, hourToSummarize, getSummarizationMethods());
                break;
            } catch (TableException ex) {
                if (i == (retryLimit - 1)) {
                    throw new PerformanceException("Failed to generate summaries from table " + sampleTable.getFullName(), ex);
                } else {
                    getLogger().error("Failed to generate summaries from table {} at {} try, will wait for 100 ms and retry. Error message: {}",
                            sampleTable.getFullName(), i, ex);
                    try {
                        Thread.sleep(100);
                    } catch (InterruptedException ie) {
                        getLogger().error("Exception from Thread.sleep method: {}", ie);
                    }
                }
            }
        }
    }
}
