package com.netapp.oci.platform.framework.performance.util;

import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Sample;
import com.netapp.oci.platform.framework.performance.data.Summary;
import java.util.*;

/**
 * Utility class for methods common to any performance package
 *
 * @author Anton Sergeev
 */
public class PerformanceUtils {
    /**
     * Collects ids of the performance sample objects and returns collection of ids
     *
     * @param samples - collection of performance samples
     * @return collection of sample ids
     */
    public static <C> Collection<Long> getSamplesIds(Collection<Sample<C>> samples) {
        Collection<Long> result = new HashSet<Long>();

        for (Sample sample: samples) {
            result.add(Long.valueOf(sample.getId()));
        }

        return result;
    }


    public static <C> Map<Long, List<SampleInfo<C>>> toSampleInfos(Map<Long, List<Sample<C>>> idToSamples) {
        Map<Long, List<SampleInfo<C>>> res = new HashMap<Long, List<SampleInfo<C>>>(idToSamples.size());
        for (Map.Entry<Long, List<Sample<C>>> entry: idToSamples.entrySet()) {
            List<SampleInfo<C>> list = new LinkedList<SampleInfo<C>>();
            res.put(entry.getKey(), list);
            for (Sample<C> sample : entry.getValue()) {
                if (sample.isNull()) {
                    list.add(new SampleInfo<C>(new Date(sample.getTime()), null));
                } else {
                    list.add(new SampleInfo<C>(new Date(sample.getTime()), sample.getSampleData()));
                }
            }
        }

        return res;
   }

    public static <C> Map<Long, SampleInfo<C>> toSampleInfo(Map<Long, Sample<C>> idToSamples) {
        Map<Long, SampleInfo<C>> res = new HashMap<Long, SampleInfo<C>>(idToSamples.size());
        for (Map.Entry<Long, Sample<C>> entry : idToSamples.entrySet()) {
            Sample<C> aSample = entry.getValue();
            res.put(entry.getKey(), new SampleInfo<C>(new Date(aSample.getTime()), aSample.getSampleData()));
        }
        return res;
    }

    public static <C> List<SampleInfo<C>> toChartSample(List<Sample<C>> samples) {
        List<SampleInfo<C>> res = new LinkedList<SampleInfo<C>>();
        for (Sample<C> sample : samples) {
            if (sample.isNull()) {
                res.add(new SampleInfo<C>(new Date(sample.getTime()), null));
            } else {
                res.add(new SampleInfo<C>(new Date(sample.getTime()), sample.getSampleData()));
            }
        }
        return res;
    }

    public static <S extends Summary> Map<Long, S> toMap(List<S> summaries) {
        HashMap<Long, S> map = new HashMap<Long, S>();
        for (S summary : summaries) {
            map.put(summary.getId(), summary);
        }
        return map;
    }
    
    public static <C> void addSummariesToMap(Map<Long, List<Summary<C>>> idToSummaries, List<Summary<C>> summaries) {
        for (Summary<C> summary : summaries) {
            if (idToSummaries.get(summary.getId()) == null) {
                idToSummaries.put(summary.getId(), new LinkedList<Summary<C>>());
            }
            idToSummaries.get(summary.getId()).add(summary);
        }
    }

    /**
     * Gets the time of today's midnight
     */
    public static long getMidnightTime() {
        Calendar cal = Calendar.getInstance();
        cal.set(Calendar.HOUR_OF_DAY, 24);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTimeInMillis();
    }

    public static long getMidnightTime(long sampleTime) {
        Calendar cal = Calendar.getInstance();
        cal.setTimeInMillis(sampleTime);
        cal.set(Calendar.HOUR_OF_DAY, 24);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        cal.set(Calendar.MILLISECOND, 0);
        return cal.getTimeInMillis();
    }
}
