package com.netapp.oci.platform.framework.performance.util;

import java.util.HashMap;
import java.util.Map;

/**
 * Stores the state of performance reporting status.
 *
 * @author Dongjun Sun
 */
public class PersistenceState {
    /**
     * Caching map, the map helps to trigger compression and summarization every time
     * there is new list of samples reported by acquisition.
     * Each new calendar hour compression will be triggered and this map will help
     * to remeber last reported time.
     * <p/>
     * Every sample table of any performance package will be part of this map
     */
    private final static Map<String, Long> keyToLastSampleTime = new HashMap<String, Long>();

    /**
     * this map is used to make sure only 1 thread is accessing the table during insert
     */
    private final static Map<String, Object> packageIdToSynchObj = new HashMap<String, Object>();

    static synchronized Long getLastSampleTime(String packageId) {
        return keyToLastSampleTime.get(packageId);
    }
    /**
     * Set the last report time
     *
     * @param packageId - performance package id
     * @param time - the insert report time
     */
    static synchronized void setLastSampleTime(String packageId, long time) {
        keyToLastSampleTime.put(packageId, time);
    }

    static synchronized Object getPackageSyncObject(String packageId) {
        Object synchObj = packageIdToSynchObj.get(packageId);
        if (synchObj == null) {
            synchObj = new Object();
            packageIdToSynchObj.put(packageId, synchObj);
        }

        return synchObj;
    }

    /**
     * For testing purpose only!!!
     */
    public static synchronized void clearAll() {
        packageIdToSynchObj.clear();
        keyToLastSampleTime.clear();
    }
}