package com.netapp.oci.platform.framework.performance.util;

import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.netapp.oci.platform.framework.performance.data.Sample;

import java.sql.Connection;
import java.util.Collection;

/**
 * Knows how to convert raw samples reported by acquisition to performance samples
 *
 * @author Anton Sergeev
 */
public interface RawSampleConverter<C> {
    /**
     * Converts raw-samples retrieved from the acquisiton into server-samples, setting the sample time and taking care
     * for creating zeroed samples for the first report.
     *
     * * @param rawSamples  sent by the acquisition, containing at most a single sample per monitored object (i.e. switch port,
     *                    or virtual-machine
     * @param time        the time that samples were retrieved by the server
     *@param firstReport true if this is the first report from the acqusition data-source, meaning that for each sample
 *                    the server will "invent" another zeroed sample immediately before the reported sample    @return the samples
     *  @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException if found multiple samples for the same ID
     */
    Collection<Sample<C>> toSamples(Collection<RawSample<C>> rawSamples, long time, boolean firstReport) throws PerformanceException;
}
