package com.netapp.oci.platform.framework.performance.util;

import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.framework.performance.tables.SampleTable;

/**
 * Sole responsibility of the component is to prepare sample data for potential compression and summarization (C&S)
 * in the temporary table. This abstracts the step of preparation for C&S for each performance component.
 *
 * @author Anton Sergeev
 */
public interface SampleTablePreparator {
    /**
     * Prepares sample data in temporary table
     *
     * @param sampleTable - sample table instance
     * @param tempTableName - the name of the temporary table
     * @parame sampleTime - the sample time
     *
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException if failed preparation step
     */
    void prepareSampleData(SampleTable sampleTable, String tempTableName, Long sampleTime) throws PerformanceException;

}
