package com.netapp.oci.platform.framework.performance.util;

import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.framework.performance.data.Summary;
import com.netapp.oci.platform.framework.performance.data.Sample;
import com.netapp.oci.platform.common.TimeSpan;

import java.util.List;

/**
 * Calculates summary based on list of samples
 *
 * @author Anton Sergeev
 */
public interface SummaryCalculator<C, S> {

    /**
     * Calculates summary object for list of samples
     * if number of sample is 1 return null;
     *
     * @param samples - list of samples
     * @param timeSpan - time span on which the summary will be valid
     *
     * @return summary object
     *
     * @throws PerformanceException if failed to calculate summary (samples is null or samples size is 0)
     */
    public Summary<S> calculateSummary(List<Sample<C>> samples, TimeSpan timeSpan) throws PerformanceException;

    /**
     * Calculates summary out of two summaries of the same objects refering to different time periods.
     * @param sum1 first summary
     * @param sum2 second summary
     * @return summary the summary/average of the two summaries
     * @throws PerformanceException if failed to calculate summary
     */
    public Summary<S> mergeSummaries(Summary<S> sum1, Summary<S> sum2) throws PerformanceException;

}
