package com.netapp.oci.platform.framework.performance.util.preparators;

import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.common.table.Table;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.SampleTable;
import com.netapp.oci.platform.framework.performance.util.AbstractTableManagement;
import com.netapp.oci.platform.framework.performance.util.SampleTablePreparator;
import com.netapp.oci.platform.common.table.UpdateTable;
import org.slf4j.Logger;

/**
 * The idea of this implementation lies in renaming original sample table
 * to a new temporary table and creating a new blank sample table for sample data.
 * This implementation should be used if sample data is being reported in prompt manner.
 *
 * @author Anton Sergeev
 */
public class RenamePreparator implements SampleTablePreparator {
    private Logger logger;

    public RenamePreparator(org.slf4j.Logger logger) {
        this.logger = logger;
    }

    public void prepareSampleData(SampleTable sampleTable, String tempTableName, Long sampleTime) throws PerformanceException {
        String tableName = sampleTable.getName();
        String database = sampleTable.getDatabase();

        Table table = new UpdateTable(sampleTable.getConnection(), database, tableName);

        try {
            AbstractTableManagement.unmarkCreated(sampleTable);
            // rename to a temporary table
            table.renameTable(tempTableName);
            sampleTable.createTable();
            // mark it created so "finally" will not try to recreate it
            AbstractTableManagement.markCreated(sampleTable);
        } catch (TableException ex) {
            throw new PerformanceException("Failed to rename/create table", ex);
        } finally {
            // try to recreate the table - try to recover if there was a failure
            if (!AbstractTableManagement.isMarkCreated(sampleTable)) {
                try {
                    sampleTable.createTable();
                    AbstractTableManagement.markCreated(sampleTable);
                } catch (TableException e) {
                    logger.info("Failed to recover, failed to recreate sample table ", e);
                }
            }
        }
    }
}
