/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.job;

import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import org.slf4j.Logger;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import javax.annotation.security.RunAs;
import javax.sql.DataSource;

import java.sql.Connection;

/**
 * @author Al Velella
 *         Date: 2/7/13
 *         Time: 3:10 PM
 */

/**
 * This job exists to workaround a jboss 6 issue where idle connection pools don't shutdown/start properly. We
 * workaround this by mandating a minimum pool size of 1. However, this causes a bad connection to be maintained
 * in the pool if mysql has shot the connection after a quiet period. Thus, this job pings the database every six hours
 * and attempts to keep the last connection alive.
 */

@RunAs("admin")
public class ConnectionPoolPingJob implements Job {
	
    public static final String CRON_TRIGGER = "0 0 0/6 * * ?";
    public static final String JOB_NAME = "ConnectionPoolPingJob";

    private static final org.slf4j.Logger logger =org.slf4j.LoggerFactory.getLogger(ConnectionPoolPingJob.class);

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        if (logger.isDebugEnabled()) {
            logger.debug("Starting Connection Pool Ping Job");
        }
		
        DataSource dataSourceOnaroPool = (DataSource) jobExecutionContext.getJobDetail().getJobDataMap().get("dataSourceOnaroPool");
        DataSource dataSourceAcqOnaroPool = (DataSource) jobExecutionContext.getJobDetail().getJobDataMap().get("dataSourceAcqOnaroPool");
        ping(dataSourceOnaroPool);
        ping(dataSourceAcqOnaroPool);
    }

    private void ping(DataSource dataSource) {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            PreparedStatementWrapper.executeFunctionQuery(connection, "SELECT COUNT(*) FROM management.version", -1L);
        } catch (TableException e) {
            logger.error("Ping error", e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }
}
