/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.job;

import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.plugin.data.PlatformPlugin;
import com.netapp.oci.platform.plugin.remote.PlatformPluginRemote;
import org.slf4j.Logger;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import javax.annotation.security.RunAs;
import javax.sql.DataSource;
import java.sql.Connection;

@RunAs("admin")
/**
 * The purpose of this job is to prune the expired database entries.
 */
@SuppressWarnings("unused")
public class PruneJob implements Job {

    public static final String CRON_TRIGGER = "0 0 * * * ?";
    public static final String JOB_NAME = "PruneJob";

    private static final Logger logger =org.slf4j.LoggerFactory.getLogger(PruneJob.class);

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        if (logger.isDebugEnabled()) {
            logger.debug("Starting Prune Job");
        }
        Connection connection = null;
        try {

            PlatformPluginRemote platformPluginRemote = (PlatformPluginRemote) jobExecutionContext.getJobDetail().getJobDataMap().get("platformPluginBean");
        
            DataSource dataSource = (DataSource) jobExecutionContext.getJobDetail().getJobDataMap().get("dataSourceOnaroPool");
            connection = ConnectionHelper.newConnection(dataSource);
            
            for (PlatformPlugin platformPlugin: platformPluginRemote.getPlugins()) {
                try {
                    platformPlugin.prune(connection, System.currentTimeMillis());
                } catch (Exception e) {
                    logger.error("Exception while pruning '" + platformPlugin + "' plugin", e);
                }
            }
        } catch (Exception e) {
            logger.error("PruneJob not completed. Exception while pruning.", e);
        } finally {
            ConnectionHelper.close(connection);
            if (logger.isDebugEnabled()) {
                logger.debug("Completed Prune Job");
            }
        }
    }
}
