package com.netapp.oci.platform.originator;

/**
 * Contains the details of the attribute value change such as its old value before the originator update, the new
 * value after the originator update, etc.
 *
 * @author Shan Ponnusamy
 */
public class AttributeChangeDetail {

    /**
     * Id of the object that contains this attribute.
     */
    private Long objId;

    /**
     * Object type of the object that contains this attribute.
     */
    private String xmlTagName;

    /**
     * Attribute name.
     */
    private String attribute;

    /**
     * Value of the attribute before the originator update.
     */
    private Object oldValue;

    /**
     * Value of the attribute after the originator update.
     */
    private Object newValue;

    public AttributeChangeDetail(Long objId, String xmlTagName, String attribute, Object oldValue, Object newValue) {

        this.objId = objId;
        this.xmlTagName = xmlTagName;
        this.attribute = attribute;
        this.oldValue = oldValue;
        this.newValue = newValue;
    }

    public Long getObjId() {
        return objId;
    }

    public String getXmlTagName() {
        return xmlTagName;
    }

    public String getAttribute() {
        return attribute;
    }

    public Object getOldValue() {
        return oldValue;
    }

    public Object getNewValue() {
        return newValue;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        AttributeChangeDetail that = (AttributeChangeDetail) o;

        if (!attribute.equals(that.attribute)) return false;
        if (newValue != null ? !newValue.equals(that.newValue) : that.newValue != null) return false;
        if (!objId.equals(that.objId)) return false;
        if (oldValue != null ? !oldValue.equals(that.oldValue) : that.oldValue != null) return false;
        if (!xmlTagName.equals(that.xmlTagName)) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = objId.hashCode();
        result = 31 * result + xmlTagName.hashCode();
        result = 31 * result + attribute.hashCode();
        result = 31 * result + (oldValue != null ? oldValue.hashCode() : 0);
        result = 31 * result + (newValue != null ? newValue.hashCode() : 0);
        return result;
    }
}
