package com.netapp.oci.platform.originator;

import com.netapp.oci.platform.originator.handler.ColumnData;
import com.netapp.oci.platform.originator.handler.RowData;

import java.util.*;

/**
 * Contains the list of columns of a table for which we need detailed change information when they change and the
 * values for those columns for rows which were found to have updated.
 *
 * @author Shan Ponnusamy
 */
class ChangeDetailQueryData {

    private String tagName;
    private String tableName;

    /**
     * This contains the obj id and the new values (in originator XML) for those attributes for which we need detailed
     * change information.  The values here will be compared against the values in DB to construct the detailed 
     * change information.
     */
    private Map<Long, List<Object>> changedAttributeValuesByObjIds;

    /**
     * Meta data about the columns that needed detailed change information such as the name of the column and its column index.
     */
    private List<String> columnNamesToRecord;

    public ChangeDetailQueryData(List<String> columnNamesToRecord, String tagName, String tableName) {
        this.tagName = tagName;
        this.tableName = tableName;
        this.columnNamesToRecord = columnNamesToRecord;
        changedAttributeValuesByObjIds = new HashMap<Long, List<Object>>();
    }

    public void addChangedRow(Long objId, List<Object> rowData) {
        changedAttributeValuesByObjIds.put(objId, rowData);
    }

    public List<String> getColumns() {
        return columnNamesToRecord;
    }

    public Map<Long, List<Object>> getChangedAttributeValues() {
        return changedAttributeValuesByObjIds;
    }

    public String getTagName() {
        return tagName;
    }

    public String getSql(String tempTableName) {
        StringBuilder sqlBuilder = new StringBuilder("SELECT ");
        for (String column: columnNamesToRecord) {
            sqlBuilder.append(column);
            sqlBuilder.append(",");
        }
        sqlBuilder.append(tempTableName + ".objId FROM ");
        sqlBuilder.append(tableName);
        sqlBuilder.append(" RIGHT JOIN ");
        sqlBuilder.append(tempTableName);
        sqlBuilder.append(" ON ");
        sqlBuilder.append(tableName);
        sqlBuilder.append(".objId = ");
        sqlBuilder.append(tempTableName);
        sqlBuilder.append(".objId");
        return sqlBuilder.toString();
    }
}
