/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.originator;

/**
 * @author Al Velella
 *         Date: Oct 3, 2011
 *         Time: 11:46:42 AM
 */
public class ObjState {
    private Long objid;

    /**
     * The check sum for the entire row.
     */
    private long checksum;

    /**
     * The check sum calculated only on those attributes for which we need detailed change information.
     */
    private long changeDetailChecksum;

    private ObjState.State state;

    public enum State {
        CREATED,
        NO_CHANGE,
        UPDATED,
        DELETED,
        PRESERVED, // object is preserved because it's parent object is marked offline
        NO_CHANGE_OFFLINE_OBJ,  // no change to the offline object
        UPDATED_TO_OFFLINE_OBJ, // object's state changed from online to offline
        DELETED_OFFLINE_OBJ  // offline object is initially marked 'DELETED_OFFLINE_OBJ' on a new orig report session. The object's state may change as parsing happens.  If the object still remains in 'DELETED_OFFLINE_OBJ' state once the parsing is finished, then the object is deleted and its dependent objects are un-preserved.
    }

    public ObjState(Long objid, State state, long checksum, long changeDetailChecksum) {
        this.objid = objid;
        this.state = state;
        this.checksum = checksum;
        this.changeDetailChecksum = changeDetailChecksum;
    }

    public Long getObjid() {
        return objid;
    }

    public long getChecksum() {
        return checksum;
    }

    public void setChecksum(long checksum) {
        this.checksum = checksum;
    }

    public void setState(ObjState.State state) {
        this.state = state;
    }

    public ObjState.State getState() {
        return state;
    }

    public long getChangeDetailChecksum() {
        return changeDetailChecksum;
    }

    public void setChangeDetailChecksum(long changeDetailChecksum) {
        this.changeDetailChecksum = changeDetailChecksum;
    }    
}