package com.netapp.oci.platform.originator;

import java.util.Map;
import java.util.Set;

/**
 * Encapsulates the changes caused by the originator update.
 */
public interface OriginatorChanges {

    /**
     * Originator Id.
     * @return originator id.
     */
    long originId();

    /**
     * Returns a map of added object's id grouped by the object's XML tag name.
     * @return map of added object's id.
     */
    Map<String, Set<Long>> addedObjs();

    /**
     * Returns a map of updated object's id grouped by the object's XML tag name.
     * @return map of updated object's id.
     */
    Map<String, Set<Long>> updatedObjs();

    /**
     * Returns a map of deleted object's id grouped by the object's XML tag name.
     * @return map of deleted object's id.
     */
    Map<String, Set<Long>> deletedObjs();

    /**
     * Returns a map of deleted history object's id grouped by the object's XML tag name.
     * Deleted history objects are soft deleted from the system, i.e. their active status is set to false. 
     * @return map of deleted history object's id.
     */
    Map<String, Set<Long>> deletedHistoryObjs();

    /**
     * Time of the originator update in milliseconds.
     * @return Time of the originator update in milliseconds.
     */
    long updateTime();

    /**
     * Detailed attribute change information for this originator update.
     * @return Detailed attribute change information
     */
    Set<AttributeChangeDetail> getAttributeChangeDetails();
}
