/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.originator;

import java.util.Map;
import java.util.Set;

/**
 * This encapsulates the changes that were made as a result of a data source XML import.
 *
 * @author Shan Ponnusamy
 */
public class OriginatorChangesImpl implements OriginatorChanges {

    /**
     * Added/updated/deleted objects grouped by tag name.
     */
    private OriginatorStateCache.ChangeData changes;

    /**
     * Data source ID
     */
    private long originId;

    /**
     * The time of data source data import.
     */
    private long updateTime;

    /**
     * Detailed attribute change information for this originator update.     
     */
    private Set<AttributeChangeDetail> attributeChangeDetails;

    public OriginatorChangesImpl(long originId, OriginatorStateCache.ChangeData changes, long updateTime,
                                 Set<AttributeChangeDetail> attributeChangeDetails) {
        this.originId = originId;
        this.changes = changes;
        this.updateTime = updateTime;
        this.attributeChangeDetails = attributeChangeDetails;
    }

    public Map<String, Set<Long>> addedObjs() {
        return changes.getAddedObjs();
    }

    public Map<String, Set<Long>> updatedObjs() {
        return changes.getUpdatedObjs();
    }

    public Map<String, Set<Long>> deletedObjs() {
        return changes.getDeletedObjs();
    }

    public Map<String, Set<Long>> deletedHistoryObjs() {
        return changes.getDeletedHistoryObjs();
    }

    public long originId() {
        return this.originId;
    }

    public long updateTime() {
        return this.updateTime;
    }

    public Set<AttributeChangeDetail> getAttributeChangeDetails() {
        return attributeChangeDetails;
    }

    public long getUpdateTime() {
        return updateTime;
    }    

    @Override
    public String toString() {
        return "OriginatorChangesImpl{" +
                "addedObjs=" + changes.getAddedObjs() +
                ", updatedObjs=" + changes.getUpdatedObjs() +
                ", deletedObjs=" + changes.getDeletedObjs() +
                ", deletedHistoryObjs=" + changes.getDeletedHistoryObjs() +
                ", originId=" + originId +
                ", updateTime=" + updateTime +
                ", attributeChangeDetails=" + attributeChangeDetails +
                '}';
    }
}
