/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.originator;

import com.onaro.sanscreen.server.originatorupdate.xml.Element;
import com.onaro.sanscreen.util.xml.DefaultHandler;
import org.slf4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import java.sql.Connection;
import java.util.Stack;

/**
 * @author Al Velella
 *         Date: Sep 21, 2011
 *         Time: 4:24:58 PM
 */
public class OriginatorUpdateHandler extends DefaultHandler {

    private static final org.slf4j.Logger logger =org.slf4j.LoggerFactory.getLogger("com.netapp.oci.platform.originator");
    private Connection connection; //DO NOT USE THIS CONNECTION.  THIS SHOULD ONLY BE USED TO PRESERVE OFFLINE OBJECTS

    /**
     * Holds element in stackElements
     */
    private Stack<Element> elementsStack = new Stack<Element>();
    private Stack<String> tagStack = new Stack<String>();

    private OriginatorUpdate originatorUpdate;

    public OriginatorUpdateHandler(OriginatorUpdate originatorUpdate, Connection connection) {
        this.originatorUpdate = originatorUpdate;
        this.connection = connection;
    }

    @Override
    public void startDocument() {
        elementsStack.clear();
        tagStack.clear();
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
        int size = attributes.getLength();
        if (size == 0 || "root".equals(localName) || "Info".equals(localName)) {
            return; // If the element those not have any attributes
        }
        // or this is the root continue

        Element element = new Element(localName, attributes);
        elementsStack.push(element);
        tagStack.push(localName);

        try {
            originatorUpdate.update(element);
        } catch (OriginatorUpdateException e) {
            logger.error("Fail update:\n" + this, e);
            throw new SAXException("Fail update:\n" + this, e);
        }

    }

    /**
     * Receive notification at the end of an element. pop the elementsStack
     */
    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException {
        if (tagStack.size() == 0 || tagStack.peek().compareToIgnoreCase(qName) != 0) {
            return;
        }
        
        tagStack.pop();
        elementsStack.pop();
        originatorUpdate.popObject();
    }

    @Override
    public void endDocument() throws SAXException {
        try {
            originatorUpdate.processDoNotDeleteList(connection);
            originatorUpdate.unpreserveOnlineObjects(connection);
            originatorUpdate.deletePreservedChildren(connection);
            originatorUpdate.preserveOfflineObjects(connection);
            connection = null;
            originatorUpdate.queueDelete();

        }
        catch (OriginatorUpdateException oue) {
            throw new SAXException("Failed in processDoNotDeleteList.", oue);
        }
        catch (Exception e) {
            logger.error("Fail delete:\n" + this, e);
            throw new SAXException("Fail delete:\n" + this, e);
        }
        super.endDocument();
    }

    public int getChangeCount() {
        return originatorUpdate.getChangeCount();
    }

    public boolean isChanged() {
        return originatorUpdate.isChanged();
    }
}
