/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.originator.db;

import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.Table;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.originator.ObjState;
import org.slf4j.Logger;

import java.sql.Connection;
import java.util.*;

/**
 * @author Al Velella
 *         Date: 1/16/13
 *         Time: 10:22 AM
 */
public abstract class AbstractTemporaryTableOperation implements DbOperation {

    private Logger logger =org.slf4j.LoggerFactory.getLogger("com.netapp.oci.platform.originator.db");

    private List<String> sqls;
    private String tempTableName;
    private Set<Long> ids;
    private boolean setTime;

    public AbstractTemporaryTableOperation(String tempTableName, List<ObjState> objStates, boolean setTime) {
        this(tempTableName, Collections.<Long>emptySet(), setTime);
        for (ObjState objState : objStates) {
            this.ids.add(objState.getObjid());
        }
    }

    public AbstractTemporaryTableOperation(String tempTableName, Set<Long> ids, boolean setTime) {
        this.sqls = new ArrayList<String>();
        this.tempTableName = tempTableName;
        this.ids = new HashSet<Long>(ids);
        this.setTime = setTime;
    }

    public void addSql(String sql) {
        sqls.add(sql);
    }

    public String getTempTableName() {
        return tempTableName;
    }

    protected Set<Long> getIds() {
        return ids;
    }


    public void addId(Long id) {
        ids.add(id);
    }

    @Override
    public void execute(Connection connection, long time) throws TableException {
        boolean debug = logger.isDebugEnabled();

        Table.createIdsTemporaryTableForJoin(connection, tempTableName, ids, debug ? logger : null);
        for (String sql : sqls) {
            if (debug) {
                logger.info(sql + ";");
            }
            PreparedStatementWrapper<Object> statement = PreparedStatementWrapper.createForUpdate(connection, sql);
            if (setTime) {
                statement.setNext(time);
            }
            statement.executeUpdateEx();
        }

        Table.dropTemporaryTable(connection, tempTableName, debug ? logger : null);
    }

    public int getOperationsCount(){
        return ids.size();
    }
}
