package com.netapp.oci.platform.originator.db;

import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.ColumnData;
import com.netapp.oci.platform.originator.handler.RowData;

import java.util.EnumSet;
import java.util.List;
import java.util.Set;

/**
 * @author Iti Behari
 *         Date: 8/14/13
 */
public class BaseInsertOperation extends AbstractBatchOperation {
    public BaseInsertOperation(final String baseTableName, ColumnData columnData) {
        super(columnData, true);

        // INSERT INTO
        //   table-name
        //   (objid, originid, [x, ...], createTime, updatetime, objState )
        // VALUES
        //   (?, ?, [?, ...] , ?, ?, ?)
        // ON DUPLICATE KEY UPDATE
        //   originid=VALUES(originid),
        //   [,x=VALUES(x)|x=COALESCE(VALUES(x),x)] ...
        // createtime=VALUES(createTime), updatetime=VALUES(updatetime),
        //   objState=VALUES(objState)

        StringBuilder sql = new StringBuilder("INSERT INTO ").append(baseTableName).append("(objid,originid,");
        final List<String> columnNames = columnData.getBaseNames();
        for (String columnName : columnNames) {
            sql.append(columnName).append(",");
        }
        sql.append("createtime, updatetime,objState) VALUES (?, ?,?,");

        for (int i = 0; i < columnNames.size(); i++) {
            sql.append("?,");
        }
        sql.append("?,'LIVE') ON DUPLICATE KEY UPDATE originid=VALUES(originid),");
        sql.append("createtime=VALUES(createtime), updatetime=VALUES(updatetime),objState=VALUES(objState)");

        List<EnumSet<ColumnAttr>> columnAttrs = columnData.getBaseAttributes();
        for (int i = 0; i < columnNames.size(); i++) {
            String name = columnNames.get(i);
            Set<ColumnAttr> attr = columnAttrs.get(i);

            sql.append(',').append(name);
            if (attr.contains(ColumnAttr.NULL_PERSISTS_LAST_VALUE)) {
                sql.append("=COALESCE(VALUES(").append(name).append("),");
            } else {
                sql.append("=VALUES(");
            }
            sql.append(name).append(')');
        }

        this.sql = sql.toString();
    }

    @Override
    protected void fillSqlStatement(PreparedStatementWrapper<Object> statement, RowData rowData, long time) throws TableException {
        statement.setNext(rowData.getObjid());
        statement.setNext(rowData.getOriginId());
        appendBaseValues(statement, rowData, null);
        statement.setNext(time);
        statement.setNext(time);

    }

}
