package com.netapp.oci.platform.originator.db;

import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.ColumnData;
import com.netapp.oci.platform.originator.handler.RowData;

import java.util.EnumSet;
import java.util.List;

/**
 * @author Iti Behari
 *         Date: 8/14/13
 */
public class BaseUpdateOperation  extends AbstractBatchOperation {

    public BaseUpdateOperation(final String tableName, ColumnData columnData) {
        super(columnData, true);

        // UPDATE
        //   table-name
        // SET
        //   originid=?,
        //   [x=?|x=COALESCE(?,x), ...]
        //   updatetime=?
        // WHERE
        //   objid=?

        StringBuilder sql = new StringBuilder("UPDATE ").append(tableName).append(" SET originid=?,");
        List<String> names = columnData.getBaseNames();
        List<Class<?>> types = columnData.getBaseTypes();
        List<EnumSet<ColumnAttr>> attributes = columnData.getBaseAttributes();
        for (int i = 0; i < types.size(); i++) {
            EnumSet<ColumnAttr> attribute = attributes.get(i);
            if (!attribute.contains(ColumnAttr.KEY)) {
                String columnName = names.get(i);
                sql.append(columnName);
                if (attribute.contains(ColumnAttr.NULL_PERSISTS_LAST_VALUE)) {
                    sql.append("=COALESCE(?,").append(columnName).append("),");
                } else {
                    sql.append("=?,");
                }
            }
        }

        sql.append("updatetime=? WHERE objid=?");

        this.sql = sql.toString();
    }

    @Override
    protected void fillSqlStatement(PreparedStatementWrapper<Object> statement, RowData rowData, long time) throws TableException {
        statement.setNext(rowData.getOriginId());
        appendBaseValues(statement, rowData, ColumnAttr.KEY);
        statement.setNext(time);
        statement.setNext(rowData.getObjid());

    }
}


