/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.originator.db;

import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.ColumnData;
import com.netapp.oci.platform.originator.handler.RowData;

import java.util.EnumSet;
import java.util.List;
import java.util.Set;

/**
 * @author Al Velella
 *         Date: Sep 23, 2011
 *         Time: 4:11:37 PM
 */
public class InsertOperation extends AbstractBatchOperation {

    public InsertOperation(final String tableName, final boolean isHistoryTable, ColumnData columnData) {
        super(columnData, true);

        // INSERT INTO
        //   table-name
        //   (objid, originid, [x, ...] inactive,checksum, changeDetailChecksum, createtime, updatetime, objState)
        // VALUES
        //   (?, ?, [?, ...] ?, ?, ?)
        // ON DUPLICATE KEY UPDATE
        //   originid=VALUES(originid),
        //   inactive=VALUES(inactive),  -- only if the table is history table
        //   checksum=VALUES(checksum),
        //   changeDetailChecksum=VALUES(changeDetailChecksum)
        //   updatetime=VALUES(updatetime),
        //   objState=VALUES(objState)
        //   [,x=VALUES(x)|x=COALESCE(VALUES(x),x)] ...

        StringBuilder sql = new StringBuilder("INSERT INTO ").append(tableName).append("(objid,originid,");
        for (String columnName : columnData.getNames()) {
            sql.append(columnName).append(",");
        }
        if (isHistoryTable) {
            sql.append("inactive,");
        }
        sql.append("checksum,changeDetailChecksum,createtime,updatetime,objState) VALUES (?,?,");
        for (int i = 0; i < columnData.getNames().size(); i++) {
            sql.append("?,");
        }
        if (isHistoryTable) {
            sql.append("false,");
        }
        sql.append("?,?,?,?,'LIVE') ON DUPLICATE KEY UPDATE originid=VALUES(originid),");
        if (isHistoryTable) {
            sql.append("inactive=VALUES(inactive),");
        }
        sql.append("checksum=VALUES(checksum),changeDetailChecksum=VALUES(changeDetailChecksum),updatetime=VALUES(updatetime),objState=VALUES(objState)");

        List<String> columnNames = columnData.getNames();
        List<EnumSet<ColumnAttr>> columnAttrs = columnData.getAttributes();
        for (int i = 0; i < columnNames.size(); i++) {
            String name = columnNames.get(i);
            Set<ColumnAttr> attr = columnAttrs.get(i);

            sql.append(',').append(name);
            if (attr.contains(ColumnAttr.NULL_PERSISTS_LAST_VALUE)) {
                sql.append("=COALESCE(VALUES(").append(name).append("),");
            } else {
                sql.append("=VALUES(");
            }
            sql.append(name).append(')');
        }

        this.sql = sql.toString();
    }

    @Override
    protected void fillSqlStatement(PreparedStatementWrapper<Object> statement, RowData rowData, long time) throws TableException {
        statement.setNext(rowData.getObjid());
        statement.setNext(rowData.getOriginId());
        append(statement, rowData, null);
        statement.setNext(rowData.getChecksum());
        statement.setNext(rowData.getChangeDetailChecksum());
        statement.setNext(time);
        statement.setNext(time);
    }
}
