/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.originator.db;

import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.originator.handler.ColumnAttr;
import com.netapp.oci.platform.originator.handler.ColumnData;
import com.netapp.oci.platform.originator.handler.RowData;
import com.netapp.oci.platform.originator.handler.XmlHandler;

import java.util.EnumSet;
import java.util.List;

/**
* @author Al Velella
*         Date: Sep 23, 2011
*         Time: 4:11:45 PM
*/
public class UpdateOperation extends AbstractBatchOperation {

    public UpdateOperation(final String tableName, final boolean isHistoryTable, ColumnData columnData) {
        super(columnData, true);

        // UPDATE
        //   table-name
        // SET
        //   originid=?,
        //   [x=?|x=COALESCE(?,x), ...]
        //   inactive=false,  -- only if the table is history table.
        //   checksum=?,
        //   changeDetailChecksum=?,
        //   updatetime=?
        // WHERE
        //   objid=?

        StringBuilder sql = new StringBuilder("UPDATE ").append(tableName).append(" SET originid=?,");
        List<String> names = columnData.getNames();
        List<Class<?>> types = columnData.getTypes();
        List<EnumSet<ColumnAttr>> attributes = columnData.getAttributes();
        for (int i = 0; i < types.size(); i++) {
            EnumSet<ColumnAttr> attribute = attributes.get(i);
            if (!attribute.contains(ColumnAttr.KEY)) {
                String columnName = names.get(i);
                sql.append(columnName);
                if (attribute.contains(ColumnAttr.NULL_PERSISTS_LAST_VALUE)) {
                    sql.append("=COALESCE(?,").append(columnName).append("),");
                } else {
                    sql.append("=?,");
                }
            }
        }
        if (isHistoryTable) {
            sql.append("inactive=false,");
        }
        sql.append("checksum=?,changeDetailChecksum=?,updatetime=? WHERE objid=?");

        this.sql = sql.toString();
    }

    @Override
    protected void fillSqlStatement(PreparedStatementWrapper<Object> statement, RowData rowData, long time) throws TableException {
        statement.setNext(rowData.getOriginId());
        append(statement, rowData, ColumnAttr.KEY);
        statement.setNext(rowData.getChecksum());
        statement.setNext(rowData.getChangeDetailChecksum());
        statement.setNext(time);
        statement.setNext(rowData.getObjid());
    }
}

