/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.originator.handler;

import java.util.ArrayList;
import java.util.Collections;
import java.util.EnumSet;
import java.util.List;

/**
 * @author Al Velella
 *         Date: Oct 3, 2011
 *         Time: 5:41:05 PM
 */
public class ColumnData {
    private List<String> names;
    private List<Class<?>> types;
    private List<EnumSet<ColumnAttr>> attributes;
    private ChangeDetailColumnData changeDetailColumnData;

    // For Hierarchy
    private List<String> baseNames;
    private List<Class<?>> baseTypes;
    private List<EnumSet<ColumnAttr>> baseAttributes;
    private ChangeDetailColumnData baseChangeDetailColumnData;

    public ColumnData() {
        this.names = Collections.emptyList();
        this.types = Collections.emptyList();
        this.attributes = Collections.emptyList();
        this.changeDetailColumnData = new ChangeDetailColumnData();
    }

    public ColumnData(List<String> names, List<Class<?>> types, List<EnumSet<ColumnAttr>> attributes) {
        this.names = names;
        this.types = types;
        this.attributes = attributes;
        this.changeDetailColumnData = new ChangeDetailColumnData();
    }

    public ColumnData(List<String> names, List<Class<?>> types, List<EnumSet<ColumnAttr>> attributes, List<String> baseNames, List<Class<?>> baseTypes, List<EnumSet<ColumnAttr>> baseAttributes) {
        this(names, types, attributes);
        this.baseNames = baseNames;
        this.baseTypes = baseTypes;
        this.baseAttributes = baseAttributes;
        this.baseChangeDetailColumnData = new ChangeDetailColumnData();
    }


    public void add(String name, Class<?> type, EnumSet<ColumnAttr> attribute, boolean changeDetailRequired) {
        if (changeDetailRequired) {
            changeDetailColumnData.add(name, names.size());
        }
        names.add(name);
        types.add(type);
        attributes.add(attribute);
    }

    public List<String> getNames() {
        return names;
    }

    public List<Class<?>> getTypes() {
        return types;
    }

    public List<EnumSet<ColumnAttr>> getAttributes() {
        return attributes;
    }

    public ChangeDetailColumnData getChangeDetailColumnData() {
        return changeDetailColumnData;
    }

    @SuppressWarnings("UnusedDeclaration")
    public void subAdd(String name, Class<?> type, EnumSet<ColumnAttr> attribute, boolean changeDetailRequired) {
        add(name, type, attribute, changeDetailRequired);
    }

    public void baseAdd(String name, Class<?> type, EnumSet<ColumnAttr> attribute, boolean changeDetailRequired) {
        if (changeDetailRequired) {
            baseChangeDetailColumnData.add(name, baseNames.size());
        }
        baseNames.add(name);
        baseTypes.add(type);
        baseAttributes.add(attribute);
    }

    public List<String> getBaseNames() {
        return baseNames;
    }

    public List<Class<?>> getBaseTypes() {
        return baseTypes;
    }

    public List<EnumSet<ColumnAttr>> getBaseAttributes() {
        return baseAttributes;
    }

    /**
     * An helper class to store meta data information about columns for which we need detailed change information.
     */
    public static class ChangeDetailColumnData {
        private List<String> columns;   // columns that require detailed change information.
        private List<Integer> columnIndex;  // the index of that columns in the table row        

        private ChangeDetailColumnData() {
            this.columns = new ArrayList<String>();
            this.columnIndex = new ArrayList<Integer>();
        }

        public void add(String columnName, int index) {
            columns.add(columnName);
            columnIndex.add(index);
        }

        public List<String> getColumns() {
            return columns;
        }

        public List<Integer> getColumnIndex() {
            return columnIndex;
        }
    }
}
