/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.originator.handler;

import com.netapp.oci.platform.originator.key.UuidKey;

import java.util.Collections;
import java.util.List;

/**
 * @author Al Velella
 *         Date: Sep 23, 2011
 *         Time: 4:12:03 PM
 */
public class RowData {
    private Long objid;
    private Long originId;
    private int checksum;
    private int changeDetailChecksum;
    private List<Object> values;
    private List<Object> baseValues;


    public RowData(Long objid) {
        this.objid = objid;
        this.originId = -1L;
        this.checksum = 0;
        this.changeDetailChecksum = 0;
        this.values = Collections.emptyList();
    }

    public RowData(Long objid, long originId, List<Object> values, int checksum, int changeDetailChecksum) {
        this.objid = objid;
        this.originId = originId;
        this.values = values;
        this.checksum = checksum;
        this.changeDetailChecksum = changeDetailChecksum;
    }

    public RowData(Long objid, long originId, List<Object> baseValues, List<Object> subValues, int checksum, int changeDetailChecksum) {
        this(objid, originId, subValues, checksum, changeDetailChecksum);
        this.baseValues = baseValues;
    }

    public void add(Object value) {
        values.add(value);
        checksum = getUpdatedCheckSum(checksum, value);
    }

    public void addSubValue(Object value) {
        add(value);
    }

    @SuppressWarnings("UnusedDeclaration")
    public void addSubChangeDetailReqValue(Object value) {
        addSubValue(value);
        changeDetailChecksum = getUpdatedCheckSum(changeDetailChecksum, value);
    }

    public void addBaseValue(Object value) {
        baseValues.add(value);
        checksum = getUpdatedCheckSum(checksum, value);
    }

    public void addBaseChangeDetailReqValue(Object value) {
        addBaseValue(value);
        changeDetailChecksum = getUpdatedCheckSum(changeDetailChecksum, value);
    }


    protected int getUpdatedCheckSum(int checksum, Object value) {
        return 31 * checksum + (value == null ? 0 : value.hashCode());
    }

    public Long getObjid() {
        return objid;
    }

    public Long getOriginId() {
        return originId;
    }

    public int getChecksum() {
        return checksum;
    }

    public List<Object> getValues() {
        return values;
    }

    public UuidKey getKey() {
        return (UuidKey) values.get(0);
    }

    @SuppressWarnings({"unchecked"})
    public <T> T getValue(int ix, Class<T> clazz) {
        return (T) values.get(ix);
    }


    public void addChangeDetailReqValue(Object value) {
        add(value);
        changeDetailChecksum = getUpdatedCheckSum(changeDetailChecksum, value);
    }

    public int getChangeDetailChecksum() {
        return changeDetailChecksum;
    }


    public List<Object> getBaseValues() {
        return baseValues;
    }
}
