/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.originator.handler;

import java.util.Collections;
import java.util.Map;
import java.lang.StringBuilder;

public class XmlChildTagDetails {
    private String childTagName;
    private String childTable;
    // Represents the relationship between child table and the parent table.
    private Map<String, String> dependentTableToForeignKeyColumn;

    private String parentTable;
    private String parentFK;
    private String parentTag;

    public XmlChildTagDetails(String childTagName, String childTable, Map<String, String> tableToForeignKeyColumn) {
        this(childTagName, childTable, tableToForeignKeyColumn, null, null, null);
    }

    public XmlChildTagDetails(String childTagName, String childTable, Map<String, String> tableToForeignKeyColumn, String parentTable, String parentFK, String parentTag) {
        this.childTagName = childTagName;
        this.childTable = childTable;
        this.dependentTableToForeignKeyColumn = Collections.unmodifiableMap(tableToForeignKeyColumn);

        // these are optional
        this.parentTable = parentTable;
        this.parentFK = parentFK;
        this.parentTag = parentTag;
    }

    public String getChildTagName() {
        return childTagName;
    }

    public String getChildTable() {
        return childTable;
    }

    public Map<String, String> getDependentTableToForeignKeyColumn() {
        return dependentTableToForeignKeyColumn;
    }

    public String getParentTable() {
        return parentTable;
    }

    public String getParentFK() {
        return parentFK;
    }

    public String getParentTag() {
        return parentTag;
    }

    public String toString() {
        StringBuilder out = new StringBuilder();
        out.append(String.format("{childTagName=%s", childTagName));
        out.append(String.format(", childTable=%s", childTable));
        out.append(String.format(", dependentTableToForeignKeyColumn=%s", dependentTableToForeignKeyColumn.toString()));
        out.append(String.format(", parentTable=%s", parentTable));
        out.append(String.format(", parentFK=%s", parentFK));
        out.append(String.format(", parentTag=%s}", parentTag));

        return out.toString();
    }
}
