/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.originator.key;

import com.netapp.oci.platform.common.table.StringCache;

import java.util.Arrays;

/**
 * @author Al Velella
 *         Date: 3/1/13
 *         Time: 11:13 AM
 */
public abstract class DelimitedUuidKey implements UuidKey {
    private String[] parts;

    public DelimitedUuidKey(String uuid, String delimiter) {
        // Make sure that client classes escape the delimiter so it doesn't get interpreted as a regex
        this.parts = uuid.split(delimiter);
        if (this.parts.length == 0) {
            this.parts = new String[] {StringCache.get(uuid)};
        } else {
            for (int i = 0; i < this.parts.length; i++) {
                this.parts[i] = StringCache.get(this.parts[i]);
            }
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        DelimitedUuidKey that = (DelimitedUuidKey) o;

        return Arrays.equals(parts, that.parts);
    }

    @Override
    public int hashCode() {
        return Arrays.hashCode(parts);
    }

    public String toString(String delimiter) {
        StringBuilder builder = new StringBuilder();
        int count = 0;
        for (String part : parts) {
            if (count++ > 0) {
                builder.append(delimiter);
            }
            builder.append(part);
        }
        return builder.toString();
    }
}
