/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.originator.runner;

import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.LoggerFactory;

import com.netapp.oci.platform.db.SqlFile;
import com.netapp.oci.platform.originator.db.RunnerOperation;

/**
 * @author Al Velella
 *         Date: Nov 9, 2011
 *         Time: 11:25:39 AM
 */
public class DbRunner {
    private static final org.slf4j.Logger logger =
            LoggerFactory
            .getLogger("com.netapp.oci.platform.originator.runner");

    private String tagname;
    private Class<?> resourceClass;
    private Map<String, List<String>> updateSqlFileToSqls;
    private Map<String, List<String>> deleteSqlFileToSqls;
    private boolean isInited;

    public DbRunner(String tagname, Class<?> resourceClass, List<String> updateSqls, List<String> deleteSqls) {
        this.tagname = tagname;
        this.resourceClass = resourceClass;

        // defer load of sql to after construction to avoid startup condition that produced a class loader problem
        // while attempting to read *.sql resources
        this.updateSqlFileToSqls = new LinkedHashMap<String, List<String>>();
        for (String sqlFile : updateSqls) {
            this.updateSqlFileToSqls.put(sqlFile, new ArrayList<String>());
        }

        this.deleteSqlFileToSqls = new LinkedHashMap<String, List<String>>();
        for (String sqlFile : deleteSqls) {
            this.deleteSqlFileToSqls.put(sqlFile, new ArrayList<String>());
        }

        loadSqlFile(updateSqlFileToSqls);
        loadSqlFile(deleteSqlFileToSqls);
    }

    private void loadSqlFile(Map<String, List<String>> sqlFileToSqls) {
        for (String sqlFile : sqlFileToSqls.keySet()) {
            try {
                String baseDir = getBaseDir(sqlFile);

                sqlFileToSqls.get(sqlFile).addAll(SqlFile.loadContents(sqlFile, resourceClass, baseDir, "runner/"));
            } catch (IOException e) {
                logger.error("Failed to load " + sqlFile, e);
            }
        }
    }

    public RunnerOperation getUpdateExec(String tempDatabase, Set<Long> ids) {
        return getRunnerOp(tempDatabase, ids, updateSqlFileToSqls);
    }

    public RunnerOperation getDeleteExec(String tempDatabase, Set<Long> ids) {
        return getRunnerOp(tempDatabase, ids, deleteSqlFileToSqls);
    }

    private RunnerOperation getRunnerOp(String tempDatabase, Set<Long> ids, Map<String, List<String>> sqlFileToSqls) {
        List<String> sqls = new ArrayList<String>();
        for (Map.Entry<String, List<String>> entry : sqlFileToSqls.entrySet()) {
            sqls.addAll(entry.getValue());
        }
        return new RunnerOperation(tempDatabase, tagname, ids, sqls);
    }

    private String getBaseDir(String sqlFile) {
        return sqlFile.substring(0, sqlFile.lastIndexOf('/') + 1);
    }
}
