/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.plugin.impl;

import com.netapp.oci.platform.common.ServerConstant;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.netapp.oci.platform.framework.performance.util.AbstractTableManagement;
import com.netapp.oci.platform.originator.key.KeyCache;
import com.netapp.oci.platform.plugin.data.PlatformPlugin;
import com.netapp.oci.platform.plugin.remote.PlatformPluginRemote;
import com.netapp.oci.server.TransactionContext;
import com.onaro.sanscreen.acquisition.sessions.AcquisitionFacadeSessionLocal;
import com.onaro.sanscreen.server.interfaces.data.ServerInterfaceException;
import com.onaro.sanscreen.util.Schema;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Resource;
import javax.annotation.security.PermitAll;
import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.sql.DataSource;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Collection;
import java.util.EnumMap;
import java.util.Map;

/**
 * @author Al Velella
 *         Date: Nov 10, 2011
 *         Time: 5:18:17 PM
 */
@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@Resource(name = "jdbc/OnaroDataSource", mappedName = "java:/onaro_pool", type = javax.sql.DataSource.class, authenticationType = Resource.AuthenticationType.CONTAINER)
@RolesAllowed({"admin", "acquisition"})
public class PlatformPluginBean implements PlatformPluginRemote {
    @Resource(name = "jdbc/OnaroDataSource", mappedName = "java:/onaro_pool", type = DataSource.class, authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    @EJB
    private AcquisitionFacadeSessionLocal acquisitionLocal;

    private static final Logger LOG = LoggerFactory.getLogger(PlatformPluginBean.class);

    private static final Map<Schema, PlatformPlugin> schemaToPlugin = new EnumMap<Schema, PlatformPlugin>(Schema.class);

    @Override
    public void register(PlatformPlugin plugin) throws ServerInterfaceException {
        Connection connection = null;
        try {
            // init from sanscreen.update_history
            TransactionContext.getLastUpdateTime();

            // update last update time with latest time in plugin update_history
            connection = ConnectionHelper.newConnection(dataSource);
            long updateTime = PreparedStatementWrapper.create(connection,
                    "SELECT MAX(createtime) FROM " + plugin.getUpdateHistoryTableName(),
                    TableResultParserDefault.LONG).executeFunctionQueryEx();

            TransactionContext.setLastUpdateTime(updateTime);

            // register plugin
            KeyCache.init(connection, plugin);
            schemaToPlugin.put(plugin.getSchema(), plugin);
            AbstractTableManagement.init(connection);
            LOG.info("Register plugin: {}, build: {}",
                    plugin.getSchema(), plugin.getBuildNumber());

            // Workaround for circular dependency. We want to deploy acquisition in the onaro.ear startup service
            // but under essentials this must wait until oci-netapp.ear finishes deployment. Try deploying again assuming
            // the first deploy fails.
            LOG.info("Deploying Acquisition Datasource meta-data");
            acquisitionLocal.deploy(false);
        } catch (Exception e) {
            throw new ServerInterfaceException("Failed to register plugin", e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public PlatformPlugin getPlugin(Schema schema) {
        return schemaToPlugin.get(schema);
    }

    @Override
    public Collection<PlatformPlugin> getPlugins() {
        return new ArrayList<PlatformPlugin>(schemaToPlugin.values());
    }

    @Override
    @PermitAll
    public boolean isPluginRegisteredForSchema(final Schema schema) {
        return schemaToPlugin.containsKey(schema);
    }
}
