/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.platform.upgrade;

import com.netapp.oci.platform.common.table.TableException;

/**
 * @author Al Velella
 *         Date: Nov 17, 2011
 *         Time: 2:47:35 PM
 */
public class FinishUpgrader implements Upgrader {

    public void run(UpgradeRunner upgradeRunner) throws TableException {
        Version initialVersion = upgradeRunner.getInitialVersion();
        Version currentVersion = upgradeRunner.getCurrentVersion();
        String configurationDb = upgradeRunner.getConfigurationDb();
        long starttime = upgradeRunner.getStartTime();

        StringBuilder sb = new StringBuilder();
        sb.append("UPDATE ").append(configurationDb).append(".upgrade_history SET duration=?, toMajor=?, toMinor=?, toMaintenance=?, toLevel=? WHERE starttime=? ");
        int rowUpdated = upgradeRunner.sqlUpdate(sb.toString())
                             .setNext(System.currentTimeMillis() - starttime)
                             .setNext(currentVersion.getMajor())
                             .setNext(currentVersion.getMinor())
                             .setNext(currentVersion.getMaintenance())
                             .setNext(currentVersion.getPatch())
                             .setNext(starttime)
                             .executeUpdateEx();
        
        if (rowUpdated == 0) {
            upgradeRunner.sqlUpdate("INSERT INTO " +
                configurationDb + ".upgrade_history" +
                "  (starttime, duration, fromMajor, fromMinor, fromMaintenance, fromLevel, toMajor, toMinor, toMaintenance, toLevel)" +
                " VALUES" +
                "  (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)")
                .setNext(starttime)
                .setNext(System.currentTimeMillis() - starttime)
                .setNext(initialVersion.getMajor())
                .setNext(initialVersion.getMinor())
                .setNext(initialVersion.getMaintenance())
                .setNext(initialVersion.getPatch())
                .setNext(currentVersion.getMajor())
                .setNext(currentVersion.getMinor())
                .setNext(currentVersion.getMaintenance())
                .setNext(currentVersion.getPatch())
                .executeUpdateEx();
        }
        
        upgradeRunner.sqlUpdate("UPDATE " + configurationDb + ".version SET major=?, minor=?, maintenance=?, patch=?")
                .setNext(currentVersion.getMajor())
                .setNext(currentVersion.getMinor())
                .setNext(currentVersion.getMaintenance())
                .setNext(currentVersion.getPatch()).executeUpdateEx();
    }
    
}
