package com.netapp.oci.platform.upgrade;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author Nancy Walia
 *         Date: Sep 10, 2012
 */
public abstract class UpgradeRegistry {

    public static class Entry {
        private String sqlFile;
        private Upgrader upgrader;
        private String description;

        public Entry(String sqlFile, Upgrader upgrader, String description) {
            this.sqlFile = sqlFile;
            this.upgrader = upgrader;
            this.description = description;
        }

        public String getSqlFile() {
            return sqlFile;
        }

        public Upgrader getUpgrader() {
            return upgrader;
        }

        public String getDescription() {
            return description;
        }
    }

    private List<String> createDBs;
    private Map<Version, Entry> versionToEntry;
    private String configurationDb;
    private String performanceDb;

    protected UpgradeRegistry(String configurationDb, String performanceDb) {
        this.createDBs = new ArrayList<String>();
        this.versionToEntry = new HashMap<Version, Entry>();
        this.configurationDb = configurationDb;
        this.performanceDb = performanceDb;
    }

    public void createDB(String createDB) {
        createDBs.add(createDB);
    }

    public void register(String upgradeFile, String description) {
        register(upgradeFile, null, description);
    }

    public void register(String upgradeFile, Upgrader upgrader, String description) {
        Pattern pattern = Pattern.compile(".*upgrade_(\\d+)_(\\d+)_(\\d+)_(\\d+)\\.sql$");
        Matcher matcher = pattern.matcher(upgradeFile);
        matcher.matches();
        Version version = new Version(Integer.valueOf(matcher.group(1)).intValue(),
                Integer.valueOf(matcher.group(2)).intValue(),
                Integer.valueOf(matcher.group(3)).intValue(),
                Integer.valueOf(matcher.group(4)).intValue());
        versionToEntry.put(version, new Entry(upgradeFile, upgrader, description));
    }

    public abstract Entry getDropViewsUpgrader();

    public abstract Entry getCreateViewsUpgrader() ;

    public abstract String getSqlDirBase();

    public Entry getFinishUpgrader() {
        return new Entry(null, new FinishUpgrader(), "Finishing upgrade");
    }

    public Entry getUpgrader(Version version) {
        return versionToEntry.get(version);
    }

    public Version getLatestVersion() {
        List<Version> versions = getVersions();
        return versions.get(versions.size() - 1);
    }

    public List<Version> getVersions() {
        ArrayList<Version> versions = new ArrayList<Version>(versionToEntry.keySet());
        Collections.sort(versions);
        return versions;
    }

    public List<String> getCreateDBs() {
        return new ArrayList<String>(createDBs);
    }

    public String getConfigurationDb() {
        return configurationDb;
    }

    public String getPerformanceDb() {
        return performanceDb;
    }
}