package com.netapp.oci.platform.util.schema;

/**
 * ColumnInfo stores information on a column in a database table
 *
 * @author Iti Behari
 *         Date: Sep 7, 2012
 */
public class ColumnInfo implements Comparable<ColumnInfo>, MapKeyContainer {
    private String tableSchema;
    private String tableName;
    private String columnName;
    private String ordinalPosition;
    private String isNullable;
    private String columnType;

    public ColumnInfo(String tableSchema, String tableName, String columnName, String ordinalPosition, String nullable, String columnType) {
        this.tableSchema = tableSchema;
        this.tableName = tableName;
        this.columnName = columnName != null ? columnName.toLowerCase() : null;
        this.ordinalPosition = ordinalPosition;
        isNullable = nullable;
        this.columnType = columnType;
    }

    public MapKey getMapKey() {
        return new MapKey(tableSchema, tableName, columnName);
    }

    public String getTableSchema() {
        return tableSchema;
    }

    public String getTableName() {
        return tableName;
    }

    public String getColumnName() {
        return columnName;
    }

    public String getOrdinalPosition() {
        return ordinalPosition;
    }

    public String getNullable() {
        return isNullable;
    }

    public String getColumnType() {
        return columnType;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        ColumnInfo that = (ColumnInfo) o;

        return !(columnName != null ? !columnName.equals(that.columnName) : that.columnName != null)
                && !(columnType != null ? !columnType.equals(that.columnType) : that.columnType != null)
                && !(isNullable != null ? !isNullable.equals(that.isNullable) : that.isNullable != null)
                && !(ordinalPosition != null ? !ordinalPosition.equals(that.ordinalPosition) : that.ordinalPosition != null)
                && !(tableName != null ? !tableName.equals(that.tableName) : that.tableName != null)
                && !(tableSchema != null ? !tableSchema.equals(that.tableSchema) : that.tableSchema != null);
    }

    @Override
    public int hashCode() {
        int result = columnName != null ? columnName.hashCode() : 0;
        result = 31 * result + (columnType != null ? columnType.hashCode() : 0);
        result = 31 * result + (isNullable != null ? isNullable.hashCode() : 0);
        result = 31 * result + (ordinalPosition != null ? ordinalPosition.hashCode() : 0);
        result = 31 * result + (tableName != null ? tableName.hashCode() : 0);
        result = 31 * result + (tableSchema != null ? tableSchema.hashCode() : 0);
        return result;
    }


    public int compareTo(ColumnInfo that) {
        final int EQUAL = 0;
        final int LESS_THAN = -1;

        int comparison = columnName != null ? columnName.compareTo(that.columnName) : that.columnName == null ? EQUAL : LESS_THAN;
        if (comparison != EQUAL) return comparison;

        comparison = columnType != null ? columnType.compareTo(that.columnType) : that.columnType == null ? EQUAL : LESS_THAN;
        if (comparison != EQUAL) return comparison;

        comparison = isNullable != null ? isNullable.compareTo(that.isNullable) : that.isNullable == null ? EQUAL : LESS_THAN;
        if (comparison != EQUAL) return comparison;

        comparison = ordinalPosition != null ? ordinalPosition.compareTo(that.ordinalPosition) : that.ordinalPosition == null ? EQUAL : LESS_THAN;
        if (comparison != EQUAL) return comparison;

        comparison = tableName != null ? tableName.compareTo(that.tableName) : that.tableName == null ? EQUAL : LESS_THAN;
        if (comparison != EQUAL) return comparison;

        return tableSchema != null ? tableSchema.compareTo(that.tableSchema) : that.tableSchema == null ? EQUAL : LESS_THAN;

    }


    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("tableSchema=").append(tableSchema).append(" | ");
        sb.append("tableName=").append(tableName).append(" | ");
        sb.append("columnName=").append(columnName).append(" | ");
        sb.append("ordinalPosition=").append(ordinalPosition).append(" | ");
        sb.append("isNullable=").append(isNullable).append(" | ");
        sb.append("columnType=").append(columnType);
        return sb.toString();
    }

}