package com.netapp.oci.platform.util.schema;

/**
 * KeyColumnInfo stores information on key columns with constraints
 *
 * @author Iti Behari
 *         Date: Sep 7, 2012
 *         <p/>
 */
public class KeyColumnInfo implements Comparable<KeyColumnInfo>, MapKeyContainer {
    private String tableSchema;
    private String tableName;
    private String columnName;
    private String ordinalPosition;
    private String constraintName;
    private String referencedTableName;
    private String referencedColumnName;

    public KeyColumnInfo(String tableSchema, String tableName, String columnName, String ordinalPosition, String constraintName, String referencedTableName, String referencedColumnName) {
        this.tableSchema = tableSchema;
        this.tableName = tableName;
        this.columnName = columnName != null ? columnName.toLowerCase() : null;
        this.ordinalPosition = ordinalPosition;
        this.constraintName = constraintName != null ? constraintName.toLowerCase() : null;
        this.referencedTableName = referencedTableName;
        this.referencedColumnName = referencedColumnName != null ? referencedColumnName.toLowerCase() : null;
    }

    public MapKey getMapKey() {
        return new MapKey(tableSchema, tableName, columnName);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        KeyColumnInfo that = (KeyColumnInfo) o;

        return !(columnName != null ? !columnName.equals(that.columnName) : that.columnName != null)
                && !(constraintName != null ? !constraintName.equals(that.constraintName) : that.constraintName != null)
                && !(ordinalPosition != null ? !ordinalPosition.equals(that.ordinalPosition) : that.ordinalPosition != null)
                && !(referencedColumnName != null ? !referencedColumnName.equals(that.referencedColumnName) : that.referencedColumnName != null)
                && !(referencedTableName != null ? !referencedTableName.equals(that.referencedTableName) : that.referencedTableName != null)
                && !(tableName != null ? !tableName.equals(that.tableName) : that.tableName != null)
                && !(tableSchema != null ? !tableSchema.equals(that.tableSchema) : that.tableSchema != null);
    }

    @Override
    public int hashCode() {
        int result = columnName != null ? columnName.hashCode() : 0;
        result = 31 * result + (constraintName != null ? constraintName.hashCode() : 0);
        result = 31 * result + (ordinalPosition != null ? ordinalPosition.hashCode() : 0);
        result = 31 * result + (referencedColumnName != null ? referencedColumnName.hashCode() : 0);
        result = 31 * result + (referencedTableName != null ? referencedTableName.hashCode() : 0);
        result = 31 * result + (tableName != null ? tableName.hashCode() : 0);
        result = 31 * result + (tableSchema != null ? tableSchema.hashCode() : 0);
        return result;
    }

    public int compareTo(KeyColumnInfo that) {
        final int EQUAL = 0;
        final int LESS_THAN = -1;

        int comparison = columnName != null ? columnName.compareTo(that.columnName) : that.columnName == null ? EQUAL : LESS_THAN;
        if (comparison != EQUAL) return comparison;

        comparison = constraintName != null ? constraintName.compareTo(that.constraintName) : that.constraintName == null ? EQUAL : LESS_THAN;
        if (comparison != EQUAL) return comparison;

        comparison = ordinalPosition != null ? ordinalPosition.compareTo(that.ordinalPosition) : that.ordinalPosition == null ? EQUAL : LESS_THAN;
        if (comparison != EQUAL) return comparison;

        comparison = referencedColumnName != null ? referencedColumnName.compareTo(that.referencedColumnName) : that.referencedColumnName == null ? EQUAL : LESS_THAN;
        if (comparison != EQUAL) return comparison;

        comparison = referencedTableName != null ? referencedTableName.compareTo(that.referencedTableName) : that.referencedTableName == null ? EQUAL : LESS_THAN;
        if (comparison != EQUAL) return comparison;

        comparison = tableName != null ? tableName.compareTo(that.tableName) : that.tableName == null ? EQUAL : LESS_THAN;
        if (comparison != EQUAL) return comparison;

        return tableSchema != null ? tableSchema.compareTo(that.tableSchema) : that.tableSchema == null ? EQUAL : LESS_THAN;

    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("tableSchema=").append(tableSchema).append(" | ");
        sb.append("tableName=").append(tableName).append(" | ");
        sb.append("columnName=").append(columnName).append(" | ");
        sb.append("ordinalPosition=").append(ordinalPosition).append(" | ");
        sb.append("constraintName=").append(constraintName).append(" | ");
        sb.append("referencedTableName=").append(referencedTableName).append(" | ");
        sb.append("referencedColumnName=").append(referencedColumnName);
        return sb.toString();
    }

}