/*
 * Copyright (c) 2009 NetApp
 * All rights reserved
 */

package com.netapp.oci.server;

import org.slf4j.Logger;

import java.io.Serializable;

/**
 * This interface used for separate between DB transaction and none DB transaction Class job.
 * The methods run() is caled by the UpdateTaskExceuter object and it is part
 * of the DB transaction.
 */
public abstract class UpdateTask implements Serializable {

    static protected transient final Logger logger = org.slf4j.LoggerFactory.getLogger("com.onaro.sanscreen.server");

    private static final long serialVersionUID = 1L;

    protected String originatorName;

    protected Long originatorId;

    protected String taskClassName;

    protected long updateTime;

    protected String loggerPrefix = "";

    protected UpdateTask(String originatorName, Long originId) throws UpdateTaskException {
        this.originatorName = originatorName;
        this.originatorId = originId;
        setLoggerPrefix(originatorId.longValue() > 0 ? " originId:" + originatorId + "(" + getOriginatorName() + ")" : " user:" + getOriginatorName());
        this.taskClassName = getClass().getSimpleName();
        this.updateTime = -1L;
    }

    public boolean runTask(TransactionContext transactionContext) throws UpdateTaskException {
        logger.info("<<<<<<<<<<<<<<<<  Start Transaction " + taskClassName + " : " + loggerPrefix + "  >>>>>>>>>>>>>>>>");
        this.updateTime = transactionContext.getTime();
        boolean update;
        try {
            update = run(transactionContext);
        } catch (UpdateTaskException e) {
            logger.error(">>>>>>>>>>>>>>>>  Failed Transaction " + taskClassName + " : " + loggerPrefix + "  <<<<<<<<<<<<<<<<");
            throw new UpdateTaskException("Failed to run " + taskClassName + loggerPrefix, e);
        }

        logger.info("UpdateTime: " + getUpdateTime());
        logger.info(">>>>>>>>>>>>>>>>  End Transaction " + taskClassName + " : " + loggerPrefix + "  <<<<<<<<<<<<<<<<");
        return update;
    }

    public abstract boolean run(TransactionContext transactionContext) throws UpdateTaskException;

    public String getOriginatorName() {
        return originatorName;
    }

    public Long getOriginatorId() {
        return originatorId;
    }

    public long getUpdateTime() {
        return updateTime;
    }

    protected void setLoggerPrefix(String prefix){
        this.loggerPrefix = prefix;
    }
}
