/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.server;

/**
 * The UpdateTaskExecuter is singleton object with synchronizes method run.
 * For getting a synchronize view of the SAN inventory, path and violation,
 * and to avoid data integrity error, every call of run method will be
 * "stack" in the synchronize
 * The run method is running in the DB transaction
 */
public class UpdateTaskExecute {

    /**
     * The UpdateTaskExecute singelton instance;
     */
    private static final UpdateTaskExecute INSTANCE = new UpdateTaskExecute();

    /**
     * return the UpdateTaskExecute singleton instance.
     *
     * @return the UpdateTaskExecute singleton instance
     */
    public static UpdateTaskExecute getInstance() {
        return INSTANCE;
    }

    /**
     * Use this synchronized method to run UpdateTask, synchronized for getting
     * a synchronize view of the SAN.
     *
     * @param task the task to run
     * @throws UpdateTaskException
     * @return true if the execution of the UpdateTask resulted in an update
     */
    public synchronized boolean run(UpdateTask task) throws UpdateTaskException {
        TransactionContext transactionContext = null;
        try {
            transactionContext = new TransactionContext(task.getClass());
            return task.runTask(transactionContext);
        } catch (TransactionContextException tce) {
            throw new UpdateTaskException("Failed to run task " + task.getClass().getName(), tce);
        } finally {
            TransactionContext.close(transactionContext);
        }
    }

}

