/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.server;

import com.onaro.sanscreen.server.sessions.SessionFactory;
import com.onaro.sanscreen.server.sessions.UpdateTaskExecuteSession;


/**
 * The UpdateTaskSynchronizer is singleton object with synchronizes method run.
 * For getting a synchronize view of the SAN inventory, path and violation,
 * and to avoid data integrity error, every call of run method will be
 * "stack" in the synchronize
 * The run method is running in the DB transaction
 */
public class UpdateTaskSynchronizer implements UpdateTaskExecutor {

    /**
     * The UpdateTaskSynchronizer singleton instance;
     */
    private static UpdateTaskSynchronizer instance = null;

    /**
     * return the UpdateTaskSynchronizer singleton instance.
     *
     * @return the UpdateTaskSynchronizer singleton instance
     * @throws com.netapp.oci.server.UpdateTaskException
     *          If Fail create IdGenerator throws UpdateTaskException
     */
    public synchronized static UpdateTaskSynchronizer getInstance() throws UpdateTaskException {
        if (instance == null) {
            instance = new UpdateTaskSynchronizer();
        }

        return instance;
    }

    private UpdateTaskSynchronizer() {
    }


    /**
     * Use this synchronized method to run UpdateTask, synchronized for getting
     * a synchronize view of the SAN.
     *
     * @param task the task to run
     * @throws com.onaro.sanscreen.server.UpdateTaskException
     *
     */
    public synchronized boolean execute(UpdateTask task) throws UpdateTaskException {
        try {
            UpdateTaskExecuteSession updateTaskExecuteSession = SessionFactory.createSession(UpdateTaskExecuteSession.JNDI_NAME);
            return updateTaskExecuteSession.execute(task);
            
        } catch (Exception e) {
            throw new UpdateTaskException("Failed execute task: " + task.getClass(), e);
        }
    }
}

