/*
 * Copyright (c) 2009 NetApp
 * All rights reserved
 */

package com.onaro.management.session;

import com.netapp.oci.platform.common.ServerConstant;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.server.TransactionContext;
import com.onaro.commons.table.DbConfig;
import com.onaro.commons.types.Period;
import com.onaro.commons.util.CollectionPrinter;
import com.onaro.management.Scale;

import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;

import java.util.Map;

/**
 * @author Al Velella
 *         Date: Mar 8, 2010
 *         Time: 5:07:17 PM
 */
@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
public class ManagementBean implements ManagementRemote {

    @Override
    public void run() throws ManagementException {
        TransactionContext transactionContext = null;
        try {
            transactionContext = new TransactionContext(getClass());
            final long time = transactionContext.getTime();
            PreparedStatementWrapper.createForUpdate(transactionContext.getConnection(), "DELETE FROM " + DbConfig.getManagementDbName() + ".scale WHERE time < ?")
                    .setNext(time - Period.YEAR.milliseconds()).executeUpdateEx();
            String sql = "INSERT INTO " + DbConfig.getManagementDbName() + ".scale VALUES " + new CollectionPrinter<Map.Entry<String, Number>>() {
                @Override
                public String text(Map.Entry<String, Number> entry) {
                    return "(" + time + ",'" + entry.getKey() + "', " + entry.getValue() + ")";
                }
            }.toString(new Scale().getCurrentScale(transactionContext).entrySet());

            PreparedStatementWrapper.createForUpdate(transactionContext.getConnection(), sql).executeUpdateEx();
        } catch (Exception e) {
            throw new ManagementException("Failed to calculate scale", e);
        } finally {
            TransactionContext.close(transactionContext);
        }
    }
}
