package com.onaro.management.session;

import com.onaro.commons.table.DbConfig;
import com.onaro.management.serverresource.SystemInfoHelper;
import com.onaro.management.serverresource.benchmarktask.BenchMarkTask;
import com.onaro.management.serverresource.benchmarktask.SqlTask;
import com.onaro.management.serverresource.interfaces.data.ServerResourceData;
import org.slf4j.Logger;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * The job to measure server CPU, memory, disk resources. It runs at 12AM everyday.
 * <p/>
 * Created by weis.
 * Date: Aug 11, 2010
 * Time: 10:44:01 AM
 */

@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@RolesAllowed({"admin"})
public class ServerResourceStatusSessionEJB implements ServerResourceStatusSession {
    static final org.slf4j.Logger logger =org.slf4j.LoggerFactory.getLogger("com.onaro.management.session");

    @Resource(name = "jdbc/OnaroDataSource", mappedName = "java:/onaro_pool", type = javax.sql.DataSource.class,
            authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    public ServerResourceData getServerResourceData(boolean runSQL) throws ManagementException {

        ServerResourceData serverResourceData = new ServerResourceData();
        SystemInfoHelper systemInfoHelper = new SystemInfoHelper();
        Connection connection = null;
        try {
            systemInfoHelper.getSystemInfo(serverResourceData);

            connection = dataSource.getConnection();
            List<BenchMarkTask> tasks = getTaskList(connection);

            if (runSQL) {
                for (BenchMarkTask task : tasks) {
                    task.run(serverResourceData, connection);
                }
            } else {
                for (BenchMarkTask task : tasks) {
                    serverResourceData.addSQLInfo(task.getName(), "N/A");
                }
            }
        } catch (Exception e) {
            logger.error("Error in ServerResourceStatusSessionEJB.getServerResourceData. Details:" + e.getMessage());
            throw new ManagementException("ServerResourceStatusSessionEJB.getServerResourceData", e);
        } finally {
            if (connection != null) {
                try {
                    connection.close();
                } catch (SQLException e) {
                    logger.error("Error when close connection in ServerResourceStatusSessionEJB.getServerResourceData. Details:" + e.getMessage());
                }
            }
        }
        return serverResourceData;
    }

    private List<BenchMarkTask> getTaskList(Connection connection) {
        List<BenchMarkTask> tasks = new ArrayList<BenchMarkTask>();
        tasks.add(new SqlTask("Path SQL", connection, "SELECT * FROM " + DbConfig.getSanscreenDbName() + ".fc_logical_current"));
        tasks.add(new BenchMarkTask("VM SQL") {
            @Override
            protected void runDetails() throws Exception {
                Thread.sleep(3000);
            }
        });
        return tasks;
    }

}
