package com.onaro.performance.interfaces.data;

import com.onaro.util.enumeration.EnumPresentation;

public enum MetricType {

    TX_UTILIZATION(MetricUnit.PERCENTAGE),
    RX_UTILIZATION(MetricUnit.PERCENTAGE),
    UTILIZATION(MetricUnit.PERCENTAGE),
    CRC_ERROR_RATE(MetricUnit.PERCENTAGE),
    ERROR_RATE(MetricUnit.PERCENTAGE),
    SYNC_LOSS_COUNT(MetricUnit.COUNT),
    SIGNAL_LOSS_COUNT(MetricUnit.COUNT),
    CLASS_3_DISCARDS(MetricUnit.COUNT),
    FRAME_TOO_LONG(MetricUnit.COUNT),
    FRAME_TOO_SHORT(MetricUnit.COUNT),
    BB_CREDIT_ERROR_COUNT(MetricUnit.COUNT),    

    // Traffic types
    TRAFFIC(MetricUnit.MB),
    TX_TRAFFIC(MetricUnit.MB),
    RX_TRAFFIC(MetricUnit.MB),
    
    TRAFFIC_RATE(MetricUnit.MBS),
    TX_TRAFFIC_RATE(MetricUnit.MBS),
    RX_TRAFFIC_RATE(MetricUnit.MBS),

//    UTILIZATION_DEVIATION(MetricUnit.Percentage),

    //Array Performance Disk
    AP_DISK_IOPS(MetricUnit.IOS),
    AP_DISK_IOPS_MAX(MetricUnit.IOS),
    AP_DISK_READ_IOPS(MetricUnit.IOS),
    AP_DISK_WRITE_IOPS(MetricUnit.IOS),
    AP_DISK_THROUGHPUT(MetricUnit.MBS),
    AP_DISK_THROUGHPUT_MAX(MetricUnit.MBS),
    AP_DISK_READ_THROUGHPUT(MetricUnit.MBS),
    AP_DISK_WRITE_THROUGHPUT(MetricUnit.MBS),
    AP_DISK_UTILIZATION(MetricUnit.PERCENTAGE),
    AP_DISK_UTILIZATION_MAX(MetricUnit.PERCENTAGE),
    AP_DISK_READ_UTILIZATION(MetricUnit.PERCENTAGE),
    AP_DISK_WRITE_UTILIZATION(MetricUnit.PERCENTAGE),

    //Array Performance Volume
    AP_VOLUME_IOPS(MetricUnit.IOS),
    AP_VOLUME_IOPS_MAX(MetricUnit.IOS),
    AP_VOLUME_READ_IOPS(MetricUnit.IOS),
    AP_VOLUME_WRITE_IOPS(MetricUnit.IOS),
    AP_VOLUME_THROUGHPUT(MetricUnit.MBS),
    AP_VOLUME_THROUGHPUT_MAX(MetricUnit.MBS),
    AP_VOLUME_READ_THROUGHPUT(MetricUnit.MBS),
    AP_VOLUME_WRITE_THROUGHPUT(MetricUnit.MBS),
    AP_VOLUME_RESPONSE_TIME(MetricUnit.MILLISECONDS),
    AP_VOLUME_RESPONSE_TIME_MAX(MetricUnit.MILLISECONDS),
    AP_VOLUME_READ_RESPONSE_TIME(MetricUnit.MILLISECONDS),
    AP_VOLUME_WRITE_RESPONSE_TIME(MetricUnit.MILLISECONDS),
    AP_VOLUME_READ_WRITE_HIT_RATIO(MetricUnit.PERCENTAGE),
    AP_VOLUME_READ_HIT_RATIO(MetricUnit.PERCENTAGE),
    AP_VOLUME_WRITE_HIT_RATIO(MetricUnit.PERCENTAGE),
    AP_VOLUME_PARTIAL_BLOCKS_RATIO(MetricUnit.PERCENTAGE),

    //Array Performance Storage Resource used by VirtualDiskMeasurementsEx (VM Package)
    AP_STORAGE_RESOURCE_IOPS(MetricUnit.IOS),
    AP_STORAGE_RESOURCE_READ_IOPS(MetricUnit.IOS),
    AP_STORAGE_RESOURCE_WRITE_IOPS(MetricUnit.IOS),
    AP_STORAGE_RESOURCE_THROUGHPUT(MetricUnit.MBS),
    AP_STORAGE_RESOURCE_READ_THROUGHPUT(MetricUnit.MBS),
    AP_STORAGE_RESOURCE_WRITE_THROUGHPUT(MetricUnit.MBS),
    AP_STORAGE_RESOURCE_RESPONSE_TIME(MetricUnit.MILLISECONDS),
    AP_STORAGE_RESOURCE_READ_RESPONSE_TIME(MetricUnit.MILLISECONDS),
    AP_STORAGE_RESOURCE_WRITE_RESPONSE_TIME(MetricUnit.MILLISECONDS),

    // Array Performance Internal Volume
    AP_INTERNAL_VOLUME_IOPS ( MetricUnit.IOS),
    AP_INTERNAL_VOLUME_IOPS_MAX ( MetricUnit.IOS),
    AP_INTERNAL_VOLUME_READ_IOPS ( MetricUnit.IOS),
    AP_INTERNAL_VOLUME_WRITE_IOPS ( MetricUnit.IOS),
    AP_INTERNAL_VOLUME_THROUGHPUT(MetricUnit.MBS),
    AP_INTERNAL_VOLUME_THROUGHPUT_MAX(MetricUnit.MBS),
    AP_INTERNAL_VOLUME_READ_THROUGHPUT(MetricUnit.MBS),
    AP_INTERNAL_VOLUME_WRITE_THROUGHPUT ( MetricUnit.MBS),
    AP_INTERNAL_VOLUME_RESPONSE_TIME ( MetricUnit.MILLISECONDS),
    AP_INTERNAL_VOLUME_RESPONSE_TIME_MAX ( MetricUnit.MILLISECONDS),
    AP_INTERNAL_VOLUME_READ_RESPONSE_TIME ( MetricUnit.MILLISECONDS),
    AP_INTERNAL_VOLUME_WRITE_RESPONSE_TIME ( MetricUnit.MILLISECONDS),


    //Host Virtualization
    HV_VIRTUAL_MACHINE_DISK_IOPS(MetricUnit.IOS),
    HV_VIRTUAL_MACHINE_DISK_READ_IOPS(MetricUnit.IOS),
    HV_VIRTUAL_MACHINE_DISK_WRITE_IOPS(MetricUnit.IOS),
    HV_VIRTUAL_MACHINE_DISK_IOPS_MAX(MetricUnit.IOS),
    HV_VIRTUAL_MACHINE_DISK_THROUGHPUT(MetricUnit.MBS),
    HV_VIRTUAL_MACHINE_DISK_READ_THROUGHPUT(MetricUnit.MBS),
    HV_VIRTUAL_MACHINE_DISK_WRITE_THROUGHPUT(MetricUnit.MBS),
    HV_VIRTUAL_MACHINE_DISK_THROUGHPUT_MAX(MetricUnit.MBS),
    HV_VIRTUAL_MACHINE_IP_THROUGHPUT(MetricUnit.MBS),
    HV_VIRTUAL_MACHINE_DISK_LATENCY(MetricUnit.MILLISECONDS),
    HV_VIRTUAL_MACHINE_DISK_READ_LATENCY(MetricUnit.MILLISECONDS),
    HV_VIRTUAL_MACHINE_DISK_WRITE_LATENCY(MetricUnit.MILLISECONDS),
    HV_VIRTUAL_MACHINE_DISK_LATENCY_MAX(MetricUnit.MILLISECONDS),
    HV_VIRTUAL_MACHINE_CPU_UTILIZATION(MetricUnit.PERCENTAGE),
    HV_VIRTUAL_MACHINE_MEMORY_UTILIZATION(MetricUnit.PERCENTAGE),
    
    HV_HYPERVISOR_DISK_IOPS(MetricUnit.IOS),
    HV_HYPERVISOR_DISK_READ_IOPS(MetricUnit.IOS),
    HV_HYPERVISOR_DISK_WRITE_IOPS(MetricUnit.IOS),
    HV_HYPERVISOR_DISK_IOPS_MAX(MetricUnit.IOS),
    HV_HYPERVISOR_DISK_THROUGHPUT(MetricUnit.MBS),
    HV_HYPERVISOR_DISK_READ_THROUGHPUT(MetricUnit.MBS),
    HV_HYPERVISOR_DISK_WRITE_THROUGHPUT(MetricUnit.MBS),
    HV_HYPERVISOR_DISK_THROUGHPUT_MAX(MetricUnit.MBS),
    HV_HYPERVISOR_IP_THROUGHPUT(MetricUnit.MBS),
    HV_HYPERVISOR_DISK_LATENCY(MetricUnit.MILLISECONDS),
    HV_HYPERVISOR_DISK_READ_LATENCY(MetricUnit.MILLISECONDS),
    HV_HYPERVISOR_DISK_WRITE_LATENCY(MetricUnit.MILLISECONDS),
    HV_HYPERVISOR_DISK_LATENCY_MAX(MetricUnit.MILLISECONDS),
    HV_HYPERVISOR_CPU_UTILIZATION(MetricUnit.PERCENTAGE),
    HV_HYPERVISOR_MEMORY_UTILIZATION(MetricUnit.PERCENTAGE);

    private MetricUnit unit;

    MetricType(MetricUnit unit) {
        this.unit = unit;
    }
    
    public MetricUnit getUnit() {
        return unit;
    }
    
    public String getEnumPresentation() {
        EnumPresentation.Attributes attributes = EnumPresentation.getDefaults().getAttributes(getClass().getName(),
                name());
        if (attributes != null) {
            return attributes.name;
        }
        return name();
    }  
}

