/*
 * Copyright (c) 2009 NetApp
 * All rights reserved
 */

package com.onaro.performance.interfaces.data;

import static com.onaro.performance.interfaces.data.MetricType.*;

import java.util.Arrays;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;


public enum MetricTypesScope{
    SWITCH(
            TX_UTILIZATION, 
            RX_UTILIZATION, 
            UTILIZATION, 
            SYNC_LOSS_COUNT, 
            SIGNAL_LOSS_COUNT, 
            CRC_ERROR_RATE, 
            ERROR_RATE, 
            CLASS_3_DISCARDS, 
            FRAME_TOO_LONG, 
            FRAME_TOO_SHORT,
            BB_CREDIT_ERROR_COUNT
            ),
    
    SWITCH_ACCUMULATIVE(
            TRAFFIC,
            TX_TRAFFIC,
            RX_TRAFFIC
            ),
    
    HOST_VIRTUALIZATION_VIRTUAL_MACHINE(
            HV_VIRTUAL_MACHINE_DISK_THROUGHPUT,
            HV_VIRTUAL_MACHINE_DISK_READ_THROUGHPUT,
            HV_VIRTUAL_MACHINE_DISK_WRITE_THROUGHPUT,

            HV_VIRTUAL_MACHINE_DISK_IOPS,
            HV_VIRTUAL_MACHINE_DISK_READ_IOPS,
            HV_VIRTUAL_MACHINE_DISK_WRITE_IOPS,

            HV_VIRTUAL_MACHINE_DISK_LATENCY,
            HV_VIRTUAL_MACHINE_DISK_READ_LATENCY,
            HV_VIRTUAL_MACHINE_DISK_WRITE_LATENCY,

            HV_VIRTUAL_MACHINE_CPU_UTILIZATION,
            HV_VIRTUAL_MACHINE_MEMORY_UTILIZATION
            ),
            
    HOST_VIRTUALIZATION_VIRTUAL_MACHINE_DISK(
            HV_VIRTUAL_MACHINE_DISK_THROUGHPUT,
            HV_VIRTUAL_MACHINE_DISK_READ_THROUGHPUT,
            HV_VIRTUAL_MACHINE_DISK_WRITE_THROUGHPUT,
            
            HV_VIRTUAL_MACHINE_DISK_IOPS,
            HV_VIRTUAL_MACHINE_DISK_READ_IOPS,
            HV_VIRTUAL_MACHINE_DISK_WRITE_IOPS,
            
            HV_VIRTUAL_MACHINE_DISK_LATENCY,
            HV_VIRTUAL_MACHINE_DISK_READ_LATENCY,
            HV_VIRTUAL_MACHINE_DISK_WRITE_LATENCY
            ),
            
    HOST_VIRTUALIZATION_HYPERVISOR(
            HV_HYPERVISOR_DISK_THROUGHPUT,
            HV_HYPERVISOR_DISK_READ_THROUGHPUT,
            HV_HYPERVISOR_DISK_WRITE_THROUGHPUT,
            HV_HYPERVISOR_DISK_IOPS, 
            HV_HYPERVISOR_DISK_READ_IOPS,
            HV_HYPERVISOR_DISK_WRITE_IOPS,
            HV_HYPERVISOR_DISK_LATENCY,
            HV_HYPERVISOR_DISK_READ_LATENCY,
            HV_HYPERVISOR_DISK_WRITE_LATENCY,
            HV_HYPERVISOR_IP_THROUGHPUT,
            HV_HYPERVISOR_CPU_UTILIZATION,
            HV_HYPERVISOR_MEMORY_UTILIZATION
            ),
            
    ARRAY_PERFORMANCE_VOLUME(
            AP_VOLUME_THROUGHPUT,
            AP_VOLUME_READ_THROUGHPUT,
            AP_VOLUME_WRITE_THROUGHPUT,
            
            AP_VOLUME_IOPS,
            AP_VOLUME_READ_IOPS,
            AP_VOLUME_WRITE_IOPS,

            AP_VOLUME_READ_WRITE_HIT_RATIO,
            AP_VOLUME_READ_HIT_RATIO,
            AP_VOLUME_WRITE_HIT_RATIO,

            AP_VOLUME_RESPONSE_TIME,
            AP_VOLUME_READ_RESPONSE_TIME,
            AP_VOLUME_WRITE_RESPONSE_TIME,
            
            AP_VOLUME_PARTIAL_BLOCKS_RATIO
    ),
    
    ARRAY_PERFORMANCE_INTERNAL_VOLUME(
            AP_INTERNAL_VOLUME_THROUGHPUT,
            AP_INTERNAL_VOLUME_READ_THROUGHPUT,
            AP_INTERNAL_VOLUME_WRITE_THROUGHPUT,
            
            AP_INTERNAL_VOLUME_IOPS,
            AP_INTERNAL_VOLUME_READ_IOPS,
            AP_INTERNAL_VOLUME_WRITE_IOPS,

            AP_INTERNAL_VOLUME_RESPONSE_TIME,
            AP_INTERNAL_VOLUME_READ_RESPONSE_TIME,
            AP_INTERNAL_VOLUME_WRITE_RESPONSE_TIME
    ),

    ARRAY_PERFORMANCE_DISK(
            AP_DISK_THROUGHPUT,
            AP_DISK_READ_THROUGHPUT,
            AP_DISK_WRITE_THROUGHPUT,

            AP_DISK_IOPS,
            AP_DISK_READ_IOPS,
            AP_DISK_WRITE_IOPS,
            
            AP_DISK_UTILIZATION,
            AP_DISK_READ_UTILIZATION,
            AP_DISK_WRITE_UTILIZATION            
    ),

    ARRAY_PERFORMANCE_STORAGE_RESOURCE(
            AP_STORAGE_RESOURCE_THROUGHPUT,
            AP_STORAGE_RESOURCE_READ_THROUGHPUT,
            AP_STORAGE_RESOURCE_WRITE_THROUGHPUT,

            AP_STORAGE_RESOURCE_IOPS,
            AP_STORAGE_RESOURCE_READ_IOPS,
            AP_STORAGE_RESOURCE_WRITE_IOPS,

            AP_STORAGE_RESOURCE_RESPONSE_TIME,
            AP_STORAGE_RESOURCE_READ_RESPONSE_TIME,
            AP_STORAGE_RESOURCE_WRITE_RESPONSE_TIME
    ),

    WITH_THRESHOLD(SWITCH);

    private final Set<MetricType> scopeTypes;

    private MetricTypesScope(MetricType ... types) {
        this.scopeTypes = Collections.unmodifiableSet(new LinkedHashSet<MetricType>(Arrays.asList(types)));
    }

    private MetricTypesScope(MetricTypesScope... otherScopes) {
        Set<MetricType> tempSet = new LinkedHashSet<MetricType>();
        for (MetricTypesScope otherScope : otherScopes) {
            tempSet.addAll(otherScope.scopeTypes);
        }
        this.scopeTypes = Collections.unmodifiableSet(tempSet);
    }

    public boolean inScope(MetricType type){
        return scopeTypes.contains(type);
    }

    public Set<MetricType> getMetricTypes() {
        return scopeTypes;
    }

    public static MetricTypesScope getScope(MetricType type){
        for (MetricTypesScope scope : values()) {
            if(scope.inScope(type)) return scope;
        }
        return null;
    }
}
