/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.onaro.performance.interfaces.impl;

import com.netapp.oci.platform.framework.performance.util.DatasourceRegistry;
import com.onaro.performance.interfaces.local.AppPerformanceDatasourceRegistry;

import javax.ejb.Stateless;

/**
 * In every perofrmance project (package) there can be only one datasource reporting about a given object at a given time.
 * for a given project the server will hold the following data
 * <ol start=0>
 * <li>	What is the datasource Id reporting on this Object (for example the storage in the storage performance project)
 * <li>	What was the last-report-time for this datasource report on this Object
 * </ol>
 * <p/>
 * If a different datasource trying to report on a specific storage, the sample should be ignore
 * If the last-report-time is greater then 30 min the server will point to the new datasource as the 'reporting' datasource.
 */
@Stateless
public class AppPerformanceDatasourceRegistryBean implements AppPerformanceDatasourceRegistry {
    private final static DatasourceRegistry datasourceRegistry = new DatasourceRegistry();

    public boolean isRegistered(String pckg, String key, Long datasourceId, long leaseTime) {
        return datasourceRegistry.isRegistered(pckg, key, datasourceId, leaseTime);
    }

    public boolean register(String pckg, String key, Long datasourceId, long leaseTime) {
        return datasourceRegistry.register(pckg, key, datasourceId, leaseTime);
    }

    public Long getRegisterDatasourceId(String pckg, String key, long leaseTime) {
        return datasourceRegistry.getRegisterDatasourceId(pckg, key, leaseTime);
    }

    public long getFirstReportTime(String pckg, String key, Long datasourceId) {
        return datasourceRegistry.getFirstReportTime(pckg, key, datasourceId);
    }

    public void clearRegistery() {
        datasourceRegistry.clearRegistery();
    }

    public void unregisterPackage(String pckg) {
        datasourceRegistry.unregisterPackage(pckg);
    }
}
