/*
 * Copyright (c) 2009 NetApp
 * All rights reserved
 */
package com.onaro.performance.maintenance;

import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;

import javax.ejb.Local;
import java.util.Set;

/**
 * Performs drop tables for any of the performance tables' .
 *
 * @author Ran Gilboa
 */
@Local
public interface TableDropper {
    /**
     * invoked by test or user to drop all tables perfromance tables from the database
     * with a given suffix, for example with the suffix 'volume_' delete:
     *
     * all SampleTable              (sample_volume_xxx)
     * all SummaryTable             (summary_volume_xxx)
     * all CompressedSampleTable    (compressed_volume_xxx)
     *
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException if failed to drop
     */
    void dropAll(String suffix) throws PerformanceException;

    /**
     * Drops table using suffix but excludes those specified in the set
     *
     * @param suffix - table suffix
     * @param excludingTables - set of tables not to drop
     * @throws PerformanceException if failed to drop
     */
    void dropAllExcluding(String suffix, Set<String> excludingTables) throws PerformanceException;
}
