/*
 * Copyright (c) 2009 NetApp
 * All rights reserved
 */
package com.onaro.performance.maintenance;

import com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.common.table.SimpleTypeQuery;

import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.sql.DataSource;
import java.sql.Connection;
import java.util.Collections;
import java.util.List;
import java.util.Set;

/**
 * Performs maintenance of performance tables' size.
 */
@Stateless
public class TableDropperBean implements TableDropper {


    @Resource(name = "jdbc/OnaroDataSource", mappedName = "java:/onaro_pool", type = javax.sql.DataSource.class, authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger("com.onaro.performance.maintenance");

    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    public void dropAll(String suffix) throws PerformanceException {
        dropAllExcluding(suffix, Collections.<String>emptySet());
    }

    public void dropAllExcluding(String suffix, Set<String> excludingTables) throws PerformanceException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            dropAll(connection, suffix, excludingTables);
        } catch (Exception e) {
            logger.error("Failed to to drop all performance table with suffix " + suffix, e);
            throw new PerformanceException("Failed to to drop all performance table with suffix " + suffix, e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }

    private void dropAll(Connection connection, String subtext, Set<String> excludingTables) throws Exception {
        String sql = "SHOW TABLES FROM netapp_performance LIKE '%" + subtext + "%';";
        SimpleTypeQuery<String> simpleTypeQuery = new SimpleTypeQuery<String>(connection);
        List<String> tableNames = simpleTypeQuery.getPrepareStatementEx(sql).executeQueryEx();
        for (String tableName : tableNames) {
            if (!excludingTables.contains(tableName)) {
                logger.info("Run DROP TABLE " + tableName);
                simpleTypeQuery.getPrepareStatementEx("DROP TABLE netapp_performance." + tableName).executeUpdateEx();
            }
        }
    }
}
