package com.onaro.sanscreen.acquisition.identify;

import com.netapp.oci.platform.common.table.TableException;
import com.onaro.sanscreen.server.tables.acquisition.AcquisitionTable;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;


public final class AcqStorageIdentifyTable extends AcquisitionTable<String> {

    private static final String TABLE_NAME =  "acquisition_storage_identify";

    private String SQL_SELECT_BY_STORAGE;
    private String SQL_SET;

    public AcqStorageIdentifyTable(Connection connection) {
        super(connection, TABLE_NAME);

        SQL_SELECT_BY_STORAGE = "SELECT storageIp FROM " + getFullName() + " WHERE storageIdentifier=?";
        SQL_SET = "INSERT INTO " + getFullName() + " (storageIdentifier, storageIp) VALUES(?, ?) ON DUPLICATE KEY UPDATE storageIp=?";
    }

    public String parseObject(final ResultSet resultSet) throws SQLException {
        throw new SQLException("parseResult should not implemented");
    }

    public String getStorageIp(String storageIdentifier) throws TableException {
      PreparedStatement preparedStatement = null;
        ResultSet resultSet = null;
        try {

            preparedStatement = getPrepareStatement(SQL_SELECT_BY_STORAGE);
            preparedStatement.setString(1, storageIdentifier.toUpperCase());
            resultSet = preparedStatement.executeQuery();
            if (resultSet.next())
                return resultSet.getString(1);
            return null;
        } catch (SQLException e) {
            throw new TableException("Failed to run " + SQL_SELECT_BY_STORAGE, e);
        } finally {
            closeConnection(preparedStatement, resultSet);
        }
    }

    public void setStorageIp(String storageIdentifier, String storageIp) throws TableException {
        PreparedStatement preparedStatement = null;
        try {
            preparedStatement = getPrepareStatement(SQL_SET);
            preparedStatement.setString(1, storageIdentifier.toUpperCase());
            preparedStatement.setString(2, storageIp);
            preparedStatement.setString(3, storageIp);
            preparedStatement.executeUpdate();
        } catch (SQLException e) {
            throw new TableException("Failed to run " + SQL_SET, e);
        } finally {
            closeConnection(preparedStatement, null);
        }
    }
}
