package com.onaro.sanscreen.acquisition.identify;

import com.netapp.oci.platform.common.table.TableException;
import com.onaro.sanscreen.server.tables.acquisition.AcquisitionTable;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;


public final class AcqVolumeIdentifyTable extends AcquisitionTable<String> {

    private static final String TABLE_NAME = "acquisition_volume_identify";

    private String SQL_SELECT_BY_STORAGE;
    private String SQL_SELECT_BY_STORAGE_AND_VOLUME;
    private String SQL_SET;

    public AcqVolumeIdentifyTable(Connection connection) {
        super(connection, TABLE_NAME);

        SQL_SELECT_BY_STORAGE = "SELECT volumeIdentifier, volumeName FROM " + getFullName() + " WHERE storageIdentifier=?";
        SQL_SELECT_BY_STORAGE_AND_VOLUME = "SELECT volumeName FROM " + getFullName() + " WHERE storageIdentifier=? AND volumeIdentifier=?";
        SQL_SET = "INSERT INTO " + getFullName() + " (storageIdentifier, volumeIdentifier, volumeName) VALUES(?, ?, ?) ON DUPLICATE KEY UPDATE volumeName=?";
    }

    public String parseObject(final ResultSet resultSet) throws SQLException {
        throw new SQLException("parseResult should not implemented");
    }


    public String findVolumeName(String storageIdentifier, String volumeIdentifier) throws TableException {
        PreparedStatement preparedStatement = null;
        ResultSet resultSet = null;
        try {

            preparedStatement = getPrepareStatement(SQL_SELECT_BY_STORAGE_AND_VOLUME);
            preparedStatement.setString(1, storageIdentifier.toUpperCase());
            preparedStatement.setString(2, volumeIdentifier.toUpperCase());
            resultSet = preparedStatement.executeQuery();
            if (resultSet.next())
                return resultSet.getString(1);
            return null;
        } catch (SQLException e) {
            throw new TableException("Failed to run " + SQL_SELECT_BY_STORAGE_AND_VOLUME, e);
        } finally {
            closeConnection(preparedStatement, resultSet);
        }
    }

    public void setVolumeName(String storageIdentifier, String volumeIdentifier, String volumeName) throws TableException {
        String curVolumeName = findVolumeName(storageIdentifier, volumeIdentifier);
        if (curVolumeName != null && curVolumeName.equals(volumeName))
            return;

        PreparedStatement preparedStatement = null;
        try {
            preparedStatement = getPrepareStatement(SQL_SET);
            preparedStatement.setString(1, storageIdentifier.toUpperCase());
            preparedStatement.setString(2, volumeIdentifier.toUpperCase());
            preparedStatement.setString(3, volumeName);
            preparedStatement.setString(4, volumeName);
            preparedStatement.executeUpdate();
        } catch (SQLException e) {
            throw new TableException("Failed to run " + SQL_SET, e);
        } finally {
            closeConnection(preparedStatement, null);
        }
    }

    public Map<String,String> getVolumeName(String storageIdentifier) throws TableException {
        PreparedStatement preparedStatement = null;
        ResultSet resultSet = null;
        try {
            Map<String,String> res = new HashMap<String, String>();
            preparedStatement = getPrepareStatement(SQL_SELECT_BY_STORAGE);
            preparedStatement.setString(1, storageIdentifier.toUpperCase());
            resultSet = preparedStatement.executeQuery();
            while (resultSet.next()) {
                res.put(resultSet.getString(1), resultSet.getString(2));
            }
            return res;
        } catch (SQLException e) {
            throw new TableException("Failed to run " + SQL_SELECT_BY_STORAGE, e);
        } finally {
            closeConnection(preparedStatement, resultSet);
        }
    }

    public void setVolumeIdentifierToName(String storageIdentifier, Map volumeIdentifierToName) throws TableException {
        Map<String,String> curVolumeIdentifierToName = getVolumeName(storageIdentifier);

        PreparedStatement preparedStatement = null;
        try {
            preparedStatement = getPrepareStatement(SQL_SET);
            preparedStatement.setString(1, storageIdentifier.toUpperCase());
            for (Iterator iterator = volumeIdentifierToName.entrySet().iterator(); iterator.hasNext();) {
                Map.Entry entry = (Map.Entry) iterator.next();
                String volumeIdentifier = (String) entry.getKey();
                String volumeName = (String) entry.getValue();
                String curVolumeName = curVolumeIdentifierToName.get(volumeIdentifier);
                if (curVolumeName != null && curVolumeName.equals(volumeName))
                    continue;

                preparedStatement.setString(2, volumeIdentifier.toUpperCase());
                preparedStatement.setString(3, volumeName);
                preparedStatement.setString(4, volumeName);
                preparedStatement.executeUpdate();
            }
        } catch (SQLException e) {
            throw new TableException("Failed to run " + SQL_SET, e);
        } finally {
            closeConnection(preparedStatement, null);
        }
    }
}
