/*
 * Copyright (c) 2009-2018 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.acquisition.sessions;

import java.beans.XMLEncoder;
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.zip.DataFormatException;

import javax.annotation.Resource;
import javax.annotation.security.PermitAll;
import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.ejb.EJBException;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.naming.NamingException;

import com.onaro.commons.types.Period;
import com.onaro.commons.util.PropertiesEncoderHelper;
import com.onaro.sanscreen.server.dataobject.acquisition.AcquisitionAuditMessage;
import com.onaro.sanscreen.server.dataobject.acquisition.AcquisitionUnit;
import com.onaro.sanscreen.server.dataobject.acquisition.AcquisitionUnitNew;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSource;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceAudit;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceDefaults;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceNew;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceStatus;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceType;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceTypeVendorModel;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceVendorModel;
import com.onaro.sanscreen.server.dataobject.acquisition.DsTypeAttribute;
import com.onaro.sanscreen.server.sessions.OriginatorUpdateImpl;
import com.onaro.sanscreen.server.sessions.OriginatorUpdateSession;
import com.onaro.sanscreen.server.sessions.OriginatorUpdateSessionLocal;
import com.onaro.sanscreen.server.sessions.SessionFactory;
import com.onaro.sanscreen.server.sessions.SummarySessionEJB;
import com.onaro.sanscreen.server.startup.StartUpServiceImpl;
import com.onaro.sanscreen.server.tables.acquisition.AcquisitionUnitTableNew;
import com.onaro.sanscreen.server.tables.acquisition.DataSourceAttrsTable;
import com.onaro.sanscreen.server.tables.acquisition.DataSourceAuditDetailsTable;
import com.onaro.sanscreen.server.tables.acquisition.DataSourceTableNew;
import com.onaro.sanscreen.server.tables.acquisition.DataSourceTypeAttrsTable;
import com.onaro.sanscreen.server.tables.acquisition.DataSourceTypeTable;
import com.onaro.sanscreen.server.tables.acquisition.DataSourceTypeVendorModelTable;
import com.onaro.sanscreen.server.tables.acquisition.Originator;
import com.onaro.sanscreen.server.tables.acquisition.OriginatorTable;
import com.onaro.sanscreen.types.ByteArrayWrapper;
import com.onaro.sanscreen.util.Schema;
import com.onaro.sanscreen.util.xml.XMLException;
import com.onaro.sanscreen.util.xml.XMLUtil;
import com.onaro.util.Compression;

import com.netapp.acquisition.aquisitionserverside.interfaces.data.AcquisitionServerException;
import com.netapp.acquisition.aquisitionserverside.interfaces.data.DataSourceDoesNotExistException;
import com.netapp.acquisition.aquisitionserverside.interfaces.local.AcquisitionServerSessionLocal;
import com.netapp.acquisition.aquisitionserverside.interfaces.remote.AcquisitionServerSession;
import com.netapp.oci.netapp.client.interfaces.data.EmsZapiManagerException;
import com.netapp.oci.netapp.db.NetAppModel;
import com.netapp.oci.netapp.ems.shared.interfaces.data.NetAppEmsEventException;
import com.netapp.oci.netapp.ems.shared.interfaces.local.EmsMsgListener;
import com.netapp.oci.netapp.ems.shared.interfaces.local.NetAppEmsEventPersistence;
import com.netapp.oci.netapp.interfaces.data.NetAppEmsConfig;
import com.netapp.oci.platform.common.ServerConstant;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParser;
import com.netapp.oci.platform.plugin.remote.PlatformPluginRemote;
import com.netapp.oci.server.UpdateTaskException;
import com.netapp.sanscreen.server.api.EmsPullProcessor;

@SuppressWarnings({"ConstantConditions"})
@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@Resource(name = "jdbc/OnaroDataSource", mappedName = "java:/onaro_pool", type = javax.sql.DataSource.class, authenticationType = Resource.AuthenticationType.CONTAINER)
@RolesAllowed({"guest", "user", "admin", "scheduler", "acquisition"})
public class AcquisitionFacadeSessionEJB implements AcquisitionFacadeSession, AcquisitionFacadeSessionLocal {
    private static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(AcquisitionFacadeSessionEJB.class);
    private static final String OCIE_PASSWORD = "netappfoundation.password";
    private static final String EMS_DESTINATION_PROP = "um.ems.rest.destination";


    @EJB
    private OriginatorUpdateSession originatorUpdateSession;

    @EJB
    private AcquisitionServerSession acquisitionServerSession;

    @EJB
    private AcquisitionServerSessionLocal acquisitionServerSessionLocal;

    @EJB
    private PlatformPluginRemote platformPluginRemote;

    @EJB
    private NetAppEmsEventPersistence netAppEmsEventPersistence;

    @Resource
    private SessionContext currentContext;

    @Resource(name = "jdbc/OnaroDataSource", mappedName = "java:/onaro_pool", type = javax.sql.DataSource.class, authenticationType = Resource.AuthenticationType.CONTAINER)
    private javax.sql.DataSource dataSource;

    private static long lastUpdate = 0;
    private static boolean deployed = false;

    private static DataSourceAuditManager auditManager;

    static {
        auditManager = new DataSourceAuditManager();
        auditManager.initialize();
    }

    private static final int DEFAULT_ORIGINATOR_EXPIRATION_PERIOD_IN_DAYS = 60;
    private static final int MAX_THREADS = 8;
    private static final String DUPLICATE_DATASOURCE_ERROR_MESSAGE = "Another datasource was found acquiring the same device (deviceKey - %s). To correct the problem, please remove one of the duplicate datasources: %s.";

    /**
     * Contains all the EMS names which have been added to ems_config table which is initialized by StartUpServiceImpl.checkEmsRegistrationForUprade.
     */
    private static List<String> addedEmsNameCache = null;

    @Override
    @PermitAll
    public void deploy(boolean force) {
        synchronized (AcquisitionFacadeSessionEJB.class) {
            if (!force && deployed) {
                return;
            }

            // Delegate the actual deploy work to the acq-server-side
            try {
                acquisitionServerSession.deploy(force);
                deployed = true;
            }
            catch (AcquisitionServerException e) {
                throw new EJBException("Failed to deploy acquisition", e);
            }
        }
    }

    /**
     * Invoked by {@link StartUpServiceImpl} to check if there is any DS to register EMS REST destination.
     * If dsAttr {@link EMS_DESTINATION_PROP} doesn't exist:
     *      it means we need to verify/register EMS REST destination. This should take care of already-added clusters
     *      during server upgrade, each DS added pre-KB doesn't have this DS attribute.
     * else
     *      it means the Acquisition server has registered with the DS as an EMS REST destination. We need to check
     *      if there is any EmsConfig entries for toBeAdded and toBeDeleted, and make corresponding actions.
     *
     * @throws TableException
     * @return empty list - registered with all the DS successfully
     *         List of DataSourceNew - the list of data sources that couldn't be registered for EMS
     */
    @Override
    @PermitAll
    public List<DataSourceNew> checkEmsRegistrationForUpgrade() throws TableException {
        Connection connection = null;
        List<DataSourceNew> failedDss = new ArrayList<>();
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            final DataSourceAttrsTable dataSourceAttrsTable = new DataSourceAttrsTable(connection);
            final List<DataSourceNew> dss = dataSourceTable.getAll();
            final List<NetAppEmsConfig> emsToBeAdded = NetAppModel.INSTANCE.getEmsConfigByState(connection, NetAppEmsConfig.EmsMessageState.TO_BE_ADDED);
            final List<NetAppEmsConfig> emsToBeDeleted = NetAppModel.INSTANCE.getEmsConfigByState(connection, NetAppEmsConfig.EmsMessageState.TO_BE_DELETED);
            final List<NetAppEmsConfig> emsConfigAll = NetAppModel.INSTANCE.getEmsConfigs(connection);
            final String hostname = AcquisitionFacadeUtil.getHostName();

            for (DataSourceNew ds : dss) {
                Properties props = dataSourceAttrsTable.findByDsId(ds.getId());
                if (!props.containsKey(EMS_DESTINATION_PROP)) {
                    DataSourceNew unregisteredDS = registerDsForEms(hostname, ds, props, connection, emsConfigAll);
                    if (unregisteredDS != null) {
                        failedDss.add(unregisteredDS);
                    } else {
                        logger.info("Registered this server as an EMS REST destination to Cluster '{}'.", ds.getName());
                    }
                } else {
                    // check if there is any emsConfig entries needed to be added or deleted from the cluster
                    AcquisitionFacadeUtil.updateEmsToDs(hostname, ds, props, emsToBeAdded, emsToBeDeleted, emsConfigAll);
                }
            }

            // update EmsConfig table toBeDeleted to Deleted and toBeAdded to Added
            if (!emsToBeAdded.isEmpty()) {
                NetAppModel.INSTANCE.updateStateEmsConfig(connection, NetAppEmsConfig.EmsMessageState.TO_BE_ADDED, NetAppEmsConfig.EmsMessageState.ADDED);
            }

            if (!emsToBeDeleted.isEmpty()) {
                NetAppModel.INSTANCE.updateStateEmsConfig(connection, NetAppEmsConfig.EmsMessageState.TO_BE_DELETED, NetAppEmsConfig.EmsMessageState.DELETED);
            }

            // initialize the cache for added EMSs
            addedEmsNameCache = NetAppModel.INSTANCE.getEmsNamesAdded(connection);
        }
        finally {
            ConnectionHelper.close(connection);
        }

        return failedDss;
    }

    @Override
    @PermitAll
    public void handleEmsRegistryForCertRegeneration() {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            final DataSourceAttrsTable dataSourceAttrsTable = new DataSourceAttrsTable(connection);
            final List<DataSourceNew> dss = dataSourceTable.getAll();
            final String certCN = AcquisitionFacadeUtil.getHostName();

            for (DataSourceNew ds : dss) {
                Properties props = dataSourceAttrsTable.findByDsId(ds.getId());

                if (props.containsKey(EMS_DESTINATION_PROP)) {
                    final String currentEmsDest = props.getProperty(EMS_DESTINATION_PROP);
                    if (certCN.equals(currentEmsDest)) {
                        // certificate regenerated only, CN is not changed.
                        // We only need to reload the certificate
                        AcquisitionFacadeUtil.reloadSecurityCertificate(currentEmsDest, props);

                    } else {
                        // certificate is regenerated with a new CN
                        // unregister with the DS
                        AcquisitionFacadeUtil.unRegisterEms(currentEmsDest, props);

                        // register with the new destination and certificate
                        final List<NetAppEmsConfig> emsConfigAll = NetAppModel.INSTANCE.getEmsConfigs(connection);
                        AcquisitionFacadeUtil.registerEms(certCN, props, emsConfigAll);

                        // update dataSource properties with new EMS destination
                        Properties attrs = new Properties();
                        attrs.put(EMS_DESTINATION_PROP, certCN);
                        dataSourceAttrsTable.update(ds.getId(), attrs);
                        updateLastUpdate();

                    }
                }
            }
        } catch (TableException e) {
            logger.error("Database error ocurred when retrieving data sources;", e);
        } catch (EmsZapiManagerException e) {
            logger.error("ZAPI invocation to the cluster failed; ", e);
        } finally {
            if (connection != null) {
                ConnectionHelper.close(connection);
            }
        }
    }

    @Override
    public boolean isEmsSupported(String emsName) {
        return addedEmsNameCache == null ? false : addedEmsNameCache.contains(emsName);
    }

    @Override
    public void getZapiEmsEvents(List<EmsMsgListener> listeners) {
        Connection connection = null;
        ThreadPoolExecutor executor = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            List<DataSourceNew> dss = dataSourceTable.getAll();
            if (dss != null && !dss.isEmpty()) {
                // Multiple threads will be launched - to pull EMS events, parse them and insert them in the netapp_model db schema
                int noOfThreads = dss.size() < MAX_THREADS ? dss.size() : MAX_THREADS;
                executor = (ThreadPoolExecutor) Executors.newFixedThreadPool(noOfThreads);
                for (int i = 0; i < dss.size(); i++) {
                    DataSourceNew ds = dss.get(i);
                    executor.execute(new EmsPullProcessor(ds.getId(),  ConnectionHelper.newConnection(dataSource), netAppEmsEventPersistence, listeners));
                }
            }
        } catch (TableException e) {
            logger.error("Error fetching datasources", e);
        } catch (Exception e) {
            logger.error("Caught exception while running EMS pull zapi calls for datasources", e);
        } finally {
            if (executor != null) {
                //TODO - After performance testing, revisit the timeout value
                try {
                    executor.shutdown();  // disable new tasks being submitted
                    boolean isTerminated = executor.awaitTermination(120L, TimeUnit.SECONDS);
                    if (!isTerminated) {
                        logger.error("Timed out while collecting Ems events through zapi");
                    }
                } catch (InterruptedException e) {
                    logger.error("Interrupted while waiting for Ems zapi pull to finish", e);
                }
            }
            if (connection != null) {
                ConnectionHelper.close(connection);
            }
        }
    }



    @Override
    public long getOriginatorID(final long ds_objid, final String deviceKey, final String originatorName) throws AcquisitionUnitException {
        return getOriginatorID(ds_objid, deviceKey, originatorName, false);
    }

    @Override
    public long getOriginatorID(final long ds_objid, final String deviceKey, final String originatorName, final boolean performDuplicateDatasourceCheck) throws AcquisitionUnitException {
        if (originatorName == null) {
            throw new AcquisitionUnitException("Originator name can not be NULL ! ds_id=" + ds_objid + ";key=" + deviceKey);
        }

        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            OriginatorTable originatorTable = new OriginatorTable(connection);
            if (performDuplicateDatasourceCheck && deviceKey != null) {
                performDuplicateDatasourceCheck(connection, ds_objid, deviceKey);
            }
            Long originatorId = originatorTable.getByDsIdAndDeviceKey(ds_objid, deviceKey);

            if (originatorId != null) {
                // The record was found in the DB.
                originatorTable.update(originatorId, deviceKey, originatorName);
                if (logger.isDebugEnabled()) {
                    logger.debug("EJB.getOriginatorID(ds_objid=" + ds_objid + ";key=" + deviceKey + ";name=" + originatorName + ") - return found id=" + originatorId);
                }

                return originatorId;
            }

            // No record was found in the DB.
            long currentTimeInMillis = System.currentTimeMillis();

            if (deviceKey == null) {
                // Check to see if they gave us a valid DatasourceID. fetchDatasourceById throws an exception if no datasource is found.
                final DataSourceNew ds = fetchDatasourceById(connection, ds_objid);
                originatorId = createOriginator(originatorTable, ds_objid, deviceKey, originatorName, ds.getName(), currentTimeInMillis);
            }
            else {
                originatorId = createOriginator(originatorTable, ds_objid, deviceKey, originatorName, null, currentTimeInMillis);
            }

            if (logger.isDebugEnabled()) {
                logger.debug("EJB.getOriginatorID(ds_objid=" + ds_objid + ";key=" + deviceKey + ";name=" + originatorName + ") - return new id=" + originatorId);
            }

            return originatorId;
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to connect to database", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    private Long createOriginator(OriginatorTable originatorTable, long ds_objid, String deviceKey, String originatorName, String dataSourceName, long currentTimeInMillis) throws AcquisitionUnitException, TableException {
        String originatorIDName = figureOutOriginatorName(deviceKey, originatorName, dataSourceName);
        if (logger.isDebugEnabled()) {
            logger.debug("finding (originatorName:" + originatorName + "), (deviceKey:" + deviceKey + "), (originatorIDName:" + originatorIDName + ")");
        }
        Originator record = new Originator(-1, ds_objid, deviceKey, originatorIDName, currentTimeInMillis);
        originatorTable.insert(record);
        return record.getId();
    }

    /**
     * Retrieves datasource by id.  If it does not exist in the database, throws AcquisitionUnitException.
     *
     * @param connection
     * @param ds_objid
     *
     * @return
     *
     * @throws TableException
     * @throws AcquisitionUnitException
     */
    private DataSourceNew fetchDatasourceById(Connection connection, long ds_objid) throws TableException, AcquisitionUnitException {
        final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
        final DataSourceNew ds = dataSourceTable.getById(ds_objid);
        if (ds == null) {
            throw new AcquisitionUnitException("Could not find datasourceID in datasource table");
        }
        return ds;
    }

    /**
     * performs duplicate datasource check of same type and same device key.
     * Looks up datasourceType for given datsource.  If given datasource does not exist, throws an exception
     * Looks for all datasources of same datasourceType that are configured for given devicekey.
     * If multiple datasources found, then throw a AcquisitionUnitException
     * If a different datasource found, then throw a AcquisitionUnitException
     *
     * @param ds_objid
     * @param deviceKey
     *
     * @return
     *
     * @throws TableException
     * @throws AcquisitionUnitException
     */
    private void performDuplicateDatasourceCheck(Connection connection, long ds_objid, String deviceKey) throws TableException, AcquisitionUnitException {
        final DataSourceNew ds = fetchDatasourceById(connection, ds_objid);
        final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
        // Finds all datasources by datasourceType and devicekey.
        final List<DataSourceNew> datasourcesOfSameType = dataSourceTable.findByTypeIdAndDeviceKey(ds.getDsTypeId(), deviceKey);

        if (datasourcesOfSameType.size() > 1) {
            Set<String> datasourceNamesOfSameType = new HashSet<String>(datasourcesOfSameType.size());
            for (DataSourceNew datasourceOfSameType : datasourcesOfSameType) {
                datasourceNamesOfSameType.add(datasourceOfSameType.getName());
            }
            throw new AcquisitionUnitException(String.format(DUPLICATE_DATASOURCE_ERROR_MESSAGE, deviceKey, datasourceNamesOfSameType));
        }
        else if (datasourcesOfSameType.size() == 1) {
            DataSourceNew datasourceOfSameType = datasourcesOfSameType.get(0);
            if (datasourceOfSameType.getId() != ds_objid) {
                Set<String> datasourceNamesOfSameType = new HashSet<String>(2);
                datasourceNamesOfSameType.add(datasourceOfSameType.getName());
                datasourceNamesOfSameType.add(ds.getName());
                throw new AcquisitionUnitException(String.format(DUPLICATE_DATASOURCE_ERROR_MESSAGE, deviceKey, datasourceNamesOfSameType));
            }

        }
    }

    @Override
    public void deleteOriginatorID(long datasourceID, final String deviceKey) throws AcquisitionUnitException {
        Connection connection = null;

        try {
            connection = ConnectionHelper.newConnection(dataSource);
            OriginatorTable originatorTable = new OriginatorTable(connection);
            final Long originatorID = originatorTable.getByDsIdAndDeviceKey(datasourceID, deviceKey);

            // If this originator ID does not exist, return as quickly as possible
            if (originatorID == null) {
                logger.info("Not found originatorId for datasourceId: [" + datasourceID + "] deviceKey: [" + deviceKey + "]");
                return;
            }

            // This originator ID exists and has to be removed. Send an empty
            stopReportOriginatorID(originatorID);

            // Now delete the record from the originator table (which will cascade delete the originator audit records
            // as well).
            originatorTable.delete(originatorID);
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to connect to database", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public void deleteOriginatorID(Set<Long> originatorIDs) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            OriginatorTable originatorTable = new OriginatorTable(connection);

            for (Long originatorID : originatorIDs) {
                // This originator ID exists and has to be removed. Send an empty
                stopReportOriginatorID(originatorID);

                // Now delete the record from the originator table (which will cascade delete the originator audit records
                // as well).
                originatorTable.delete(originatorID);
            }
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to connect to database", e);
        }
        finally {
            updateLastUpdate();
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public void stopReportOriginatorID(Set<Long> originatorIDs) throws AcquisitionUnitException {
        for (Long originatorID : originatorIDs) {
            stopReportOriginatorID(originatorID);
        }

        updateLastUpdate();
    }

    /**
     * This method stops reporting on one originator ID only. It facilitated MOID upgrades
     *
     * @param originatorId
     *
     * @throws AcquisitionUnitException
     */
    public void stopReportOriginatorID(long originatorId) throws AcquisitionUnitException {
        try {
            originatorUpdateSession.removeOriginUpdate(originatorId);
            logger.info("Stop report from originatorId:" + originatorId);
        }
        catch (UpdateTaskException e) {
            logger.error("Failed to stop report from originatorId:" + originatorId, e);
            throw new AcquisitionUnitException("Fail Stop report from Originator:" + originatorId, e);
        }
    }

    @Override
    public void retireExpiredOriginators() throws AcquisitionUnitException {
        // Get the cutoff period and calculate the cutoff time in milliseconds
        final int originatorExpirationPeriod = DEFAULT_ORIGINATOR_EXPIRATION_PERIOD_IN_DAYS;
        final long originatorExpirationPeriodMillis = originatorExpirationPeriod * Period.DAY.milliseconds();
        final long originatorCutoffTimeMillis = System.currentTimeMillis() - originatorExpirationPeriodMillis;

        List<Originator> originators = null;
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            // Get the originator table
            OriginatorTable originatorTable = new OriginatorTable(connection);

            // Get all the originators older than the given cutoff
            originators = originatorTable.getExpiredOriginators(originatorCutoffTimeMillis);

            logger.info("Removing originators size: " + originators.size() + "  originatorCutoffTimeMillis " + originatorCutoffTimeMillis);
            for (final Originator originator : originators) {
                deleteOriginatorID(originator.getDsId(), originator.getDeviceKey());

                logger.info("Deleted originator: " + originator);
                // Add a DataSource audit to appear in the DataSource event log under the "System" package.
                final String message = "Stopped reporting device " + originator.getName();

                final Long auditSessionId = auditManager.createDsAudit(originator.getDsId(), "Inventory", message);
                auditManager.closeDsAudit(auditSessionId, message, null);
            }
        }
        catch (Exception e) {
            logger.info("Failed to remove originators " + originators, e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    /**
     * Deletes all originators for the data source that are not in the activeOriginatorIDs set.
     *
     * @param activeOriginatorIDs Originators successfully reported by this data source
     * @param dataSourceId        ID of data source
     *
     * @throws AcquisitionUnitException
     */
    @Override
    public void retireUnreportedOriginators(final Set<Long> activeOriginatorIDs, final Long dataSourceId) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            // Get the originator table
            OriginatorTable originatorTable = new OriginatorTable(connection);

            // find originators with last access time earlier than the cutoff
            final Set<Long> originatorIdsToDelete = originatorTable.findByDsId(dataSourceId);
            originatorIdsToDelete.removeAll(activeOriginatorIDs);

            if (!originatorIdsToDelete.isEmpty()) {
                StringBuilder activeIdsBuilder = new StringBuilder();
                for (Long id : activeOriginatorIDs) {
                    activeIdsBuilder.append(id);
                    activeIdsBuilder.append(",");
                }
                if(activeIdsBuilder.length() > 0) { //BURT 1182788--- this can be empty
                    activeIdsBuilder.deleteCharAt(activeIdsBuilder.length() - 1); // delete the last ","
                }
                StringBuilder deleteIdsBuilder = new StringBuilder();
                for (Long id : originatorIdsToDelete) {
                    deleteIdsBuilder.append(id);
                    deleteIdsBuilder.append(",");
                }
                if(deleteIdsBuilder.length() > 0) {//BURT 1182788--- this can be empty
                    deleteIdsBuilder.deleteCharAt(deleteIdsBuilder.length() - 1); // delete the last ","
                }
                logger.warn("Originator to delete set not empty, active ids=" + activeIdsBuilder.toString() + ", originator ids to delete=" + deleteIdsBuilder.toString());
            }

            for (final Long originatorIdToDelete : originatorIdsToDelete) {
                final Originator originatorToDelete = originatorTable.find(originatorIdToDelete);
                deleteOriginatorID(dataSourceId, originatorToDelete.getDeviceKey());

                final String message = "Stopped reporting device " + originatorToDelete.getName();
                final Long auditSessionId = auditManager.createDsAudit(dataSourceId, "Inventory", message);
                auditManager.closeDsAudit(auditSessionId, message, null);
            }

            updateLastUpdate();
        }
        catch (TableException e) {
            throw new AcquisitionUnitException(e.getMessage(), e);
        }
        catch (AcquisitionUnitException e) {
            throw new AcquisitionUnitException(e.getMessage(), e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public Long addDS(final String auName,
                      final String name,
                      final String vendor,
                      final String model,
                      final Integer datasourceTypeId,
                      final Integer manual,
                      final Boolean isActive,
                      final Properties attrs) throws AcquisitionUnitException {

        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);

            // Check if there is another datasource with the same name. If there is one, fail
            if (dataSourceTable.findByName(name).size() > 0) {
                throw new AcquisitionUnitException("Name " + name + " already used by another datasource");
            }

            final AcquisitionUnitTableNew acquisitionUnitTable = new AcquisitionUnitTableNew(connection);

            final AcquisitionUnitNew au = acquisitionUnitTable.getByName(auName);
            if (au == null) {
                logger.error("Fail Find acquisitionUnitName:" + auName);
                throw new AcquisitionUnitException("Fail Find acquisitionUnitName:" + auName);
            }

            // Prepare attributes
            encryptOCIEPasswordInAttrs(attrs);      // OCIE clients pass OCIE password in cleartext, password must be encrypted here

            attrs.remove("____");
            attrs.setProperty("domain.id", auName);

            // fix "foundation.poll" attribute - change minutes to seconds !
            // I, Dmitry Erkin, failed to change the GUI to do it, so I'm fixing it here.
            // TODO: in new GUI framework, it should be easier to do.
            final String foundationPoll = attrs.getProperty("foundation.poll");
            if (foundationPoll != null) {
                final int v = Integer.parseInt(foundationPoll);
                attrs.setProperty("foundation.poll", Integer.toString(v * 60));
            }

            final String nasPoll = attrs.getProperty("nas.poll");
            if (nasPoll != null) {
                final int v = Integer.parseInt(nasPoll);
                attrs.setProperty("nas.poll", Integer.toString(v * 60));
            }

            if (logger.isDebugEnabled()) {
                logger.debug("name of new datasource:" + name);
            }

            // TA38128  Acquisition US15248: OPM - Fixed tick netappstorageperformance polling
            // Make sure the "netappstorageperformance.clock.polling.enabled" property exists
            if (!attrs.containsKey("netappstorageperformance.clock.polling.enabled")) {
                attrs.setProperty("netappstorageperformance.clock.polling.enabled", "false");
            }
            initializedOCIEPerformancePolling(auName, new Long(datasourceTypeId), attrs, connection);

            final DataSourceNew dataSource = new DataSourceNew(
                    au.getId(),
                    datasourceTypeId,
                    -1,
                    name,
                    attrs,
                    vendor,
                    model,
                    "",
                    "",
                    0,
                    manual,
                    isActive);

            dataSourceTable.insert(dataSource);

            final DataSourceAttrsTable dataSourceAttrsTable = new DataSourceAttrsTable(connection);

            // DS added in KB will have this attribute set which is used to detect server upgrade
            final String hostname = AcquisitionFacadeUtil.getHostName();
            attrs.put(EMS_DESTINATION_PROP, hostname);

            dataSourceAttrsTable.insert(dataSource.getId(), attrs);

            updateLastUpdate();

            long dsId = dataSource.getId();
            List<NetAppEmsConfig> emsConfigs = NetAppModel.INSTANCE.getEmsConfigs(connection);
            if (!emsConfigs.isEmpty()) {
                AcquisitionFacadeUtil.registerEms(hostname, attrs, emsConfigs);
            }

            return dsId;
        } catch (Exception e) {
            StringWriter writer = new StringWriter();
            attrs.list(new PrintWriter(writer));
            String message = String
                    .format("Fail to add data source for acquisitionUnitName: %s, dataSourceName: %s, vendor:%s, model: %s, datasourceTypeId: %d, manual: %d, isActive: %b, attrs: %s",
                            auName, name, vendor, model, datasourceTypeId, manual, isActive, writer.toString());
            logger.error(message, e);

            if (e instanceof EmsZapiManagerException) {
                //Throwing AcquisitionUnitException with message and error code.
                throw new AcquisitionUnitException("Fail to add a new DS", e, ((EmsZapiManagerException) e).getReason());
            } else {
                throw new AcquisitionUnitException("Fail to add a new DS", e);
            }
        } finally {
            ConnectionHelper.close(connection);
        }
    }

    /**
     * OCIE performance polls need to be synchronized with a specific wall clock minute.
     * This method gets the generates the target Minute based on a random offset between 0
     * and the performance poll period (netappstorageperformance.clock.polling.offset).
     * @param dsName the data source anme (for logging)
     * @param datasourceTypeId the data source tyep (this logic only applies to type 91: netapp_ocie
     * @param attrs datasource attributes to add netappstorageperformance.clock.polling.offset to
     * @param connection the database connection
     * @throws TableException if we can't read the performance poll period from the database
     * @throws AcquisitionUnitException if the datasource performance poll period is not parsable (shouldn't happent)
     */
    private void initializedOCIEPerformancePolling(final String dsName, final Long datasourceTypeId,
                                                   final Properties attrs, Connection connection) throws TableException, AcquisitionUnitException {

        // TA38128  Acquisition US15248: OPM - Fixed tick netappstorageperformance polling
        if (!datasourceTypeId.equals(91L)) {
            return;
        }

        // Pick a random offset between 0 and the performance poll period in minutes
        // First, get the poll period.
        String performancePollValue = attrs.getProperty("netappstorageperformance.poll");
        if (performancePollValue == null) {
            final DataSourceTypeAttrsTable typeAttrsTable = new DataSourceTypeAttrsTable(connection);
            final List<DsTypeAttribute> typeAttrs = typeAttrsTable.findByDsTypeId(91L);
            for (final DsTypeAttribute dsTypeAttribute : typeAttrs) {
                if ("netappstorageperformance.poll".equals(dsTypeAttribute.getName())) {
                    performancePollValue = dsTypeAttribute.getValue();
                    break;
                }
            }
        }
        // Then choose an offset to evenly distribute polling offsets
        try {
            final String clockPollingEnabled = attrs.getProperty("netappstorageperformance.clock.polling.enabled");
            final String previousOffset = attrs.getProperty("netappstorageperformance.clock.polling.offset");
            final int performancePollSeconds = Integer.parseInt(performancePollValue);
            final int performancePollMinutes = performancePollSeconds / 60;

            DataSourceAttrsTable dataSourceAttrsTable = new DataSourceAttrsTable(connection);
            HashMap<Long, String> dsIdOffsetMap = dataSourceAttrsTable.getDsIdValueMap("netappstorageperformance.clock.polling.offset");

            HashMap<Integer, Long> offsetCounts = new HashMap<Integer, Long>();

            for (int i = 0; i < performancePollMinutes; i++) {
                offsetCounts.put(i, 0L);
            }

            for (Map.Entry<Long, String> entry : dsIdOffsetMap.entrySet()) {
                Integer offset = Integer.parseInt(entry.getValue());

                if (offset < performancePollMinutes) {
                    offsetCounts.put(offset, offsetCounts.get(offset) + 1L);
                }
            }

            long minCount = Long.MAX_VALUE;
            int minCountOffset = 0;
            for (int i = 0; i < performancePollMinutes; i++) {
                if (offsetCounts.get(i) < minCount) {
                    minCount = offsetCounts.get(i);
                    minCountOffset = i;
                }
            }

            int offsetMinute = minCountOffset;
            int offsetSecond = (int)(Math.random() * 60);
            attrs.setProperty("netappstorageperformance.clock.polling.offset", String.valueOf(offsetMinute));
            attrs.setProperty("netappstorageperformance.clock.polling.offset.second", String.valueOf(offsetSecond));
            logger.warn("Set new performance poll offset for " + dsName + ", new performance poll " + performancePollValue
                    + "s, new offset: " + offsetMinute
                    + "m, previous offset: " + previousOffset + " m, clock polling enabled: " + clockPollingEnabled);
        } catch (NumberFormatException nfe) {
            throw new AcquisitionUnitException("Invalid netappstorageperformance.poll value "
                + performancePollValue + " for datasource " + dsName);
        }
    }

    @Override
    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public boolean waitForDataSourceStart(final Long datasourceId) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            final AcquisitionUnitTableNew auTable = new AcquisitionUnitTableNew(connection);
            final int WAIT_TIME = 5000;
            int elapsed = 0;
            boolean started = false;
            do {
                final DataSourceNew dataSourceNew = dataSourceTable.get(datasourceId);
                final AcquisitionUnitNew au = dataSourceNew != null ? auTable.get(dataSourceNew.getAuId()) : null;
                if (au==null || dataSourceNew == null) {
                    //we don't want to keep looping here - may have been deleted in the mean time.  But it didn't really start either.
                    break;
                } else {
                    //started counts only if AU is active.  If so, then check if DS is active.
                    started = au.isActive() && dataSourceNew.isActive();
                }
                if (!started) {
                    try {
                        Thread.sleep(1000);
                    } catch (InterruptedException ie) {
                        //fall through.
                    }
                    elapsed += 1000;
                }
            } while (elapsed < WAIT_TIME && !started);

            if (elapsed >= WAIT_TIME) {
                logger.warn("Could not wait for DataSource ID=" + datasourceId + " to start.");
            }
            return started;
        }
        catch (Exception e) {
            throw new AcquisitionUnitException("Fail to check start status of DataSource id=" + datasourceId, e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public void updateManageURL(Long datasourceId, String manageURL) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            final DataSourceAttrsTable dataSourceAttrsTable = new DataSourceAttrsTable(connection);
            dataSourceAttrsTable.create(datasourceId, "foundation.manage.url", manageURL);
        }
        catch (TableException e) {
            logger.error("Failed to change the manageURL for datasource: " + datasourceId, e);
            throw new AcquisitionUnitException("Failed to change the manageURL for datasource: " + datasourceId, e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public List<AcquisitionAuditMessage> getAuditMessages(final Long audit_id) throws AcquisitionUnitException {

        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceAuditDetailsTable dsadt = new DataSourceAuditDetailsTable(connection);
            return dsadt.getAuditMessagesByAuditID(audit_id);
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get Audit messages by id:" + audit_id, e);
        }
        finally {
            ConnectionHelper.close(connection);
        }

    }

    @Override
    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public DataSourceNew getDsById(final long ds_objid) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            final DataSourceNew dataSource = dataSourceTable.get(ds_objid);
            fillDataSourceWithAttrs(dataSource, connection);
            return dataSource;
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to getById id:" + ds_objid, e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }


    @Override
    public void updateDsIsActiveForAu(AcquisitionUnitNew au, boolean active) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            final List<DataSourceNew> datasources = dataSourceTable.findByAuName(au.getName());
            for (DataSourceNew dataSource : datasources) {
                dataSourceTable.updateIsActive(dataSource.getId(), active);
            }
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to getAllOpenForDomain. au name:" + au.getName(), e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    /**
     * This method is used by the data source to report that it finished an acquisition cycle and it has nothing new to
     * report. The reason for such a method is to act as a ping receiver between the acquisition and the server, with
     * out this method jboss disconnect the connection between them.
     */
    @Override
    public void noChangeSinceLastReport(AcquisitionUnit au) throws AcquisitionUnitException {
    }

    @Override
    public Boolean update(String xml, Schema schema) throws AcquisitionUnitException {
        if (xml == null) {
            return Boolean.FALSE;
        }

        try {
            Long originId = XMLUtil.getLongElementVal(xml, "Originator");
            String dataSourceName = XMLUtil.getElementAttributeVal(xml, "Info", "DataSourceName");
            return getOriginatorUpdateSessionLocal().getUpdate(new StringReader(xml), schema, originId, dataSourceName).isChanged();
        }
        catch (UpdateTaskException e) {
            throw new AcquisitionUnitException(e.getMessage(), e);
        }
        catch (XMLException e) {
            throw new AcquisitionUnitException(e.getMessage(), e);
        }
    }

    @Override
    public Boolean updateCompressed(byte[] compressedXML, Schema schema) throws AcquisitionUnitException {
        byte[] uncompressedXML;

        try {
            uncompressedXML = Compression.decompressData(compressedXML);
            return update(new String(uncompressedXML, "utf-8"), schema);
        }
        catch (DataFormatException e) {
            logger.error("Failed to uncompress oData", e);
            throw new AcquisitionUnitException("Failed to uncompress oData", e);
        }
        catch (UnsupportedEncodingException e) {
            //should not happen, as utf-8 is supported
            logger.error("Failed to create a string from the bytes", e);
            throw new AcquisitionUnitException("Failed to create a string from the bytes", e);
        }
    }

    @Override
    public void updateOriginatorLastAccessTime(final long originatorId) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            OriginatorTable originatorTable = new OriginatorTable(connection);

            originatorTable.updateOriginatorLastAccessTime(originatorId);

        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to update originator lastaccesstime in database.", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }


    @Override
    public void restartDS(final long ds_objid) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnectionNoEjb();

            final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            dataSourceTable.updateIsActive(ds_objid, false);

            acquisitionServerSession.notifyDsConfigurationChange(ds_objid);
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to restart ds:" + ds_objid, e);
        }
        catch (AcquisitionServerException e) {
            throw new AcquisitionUnitException("Failed to restart ds:" + ds_objid, e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public void updateManual(final long ds_objid, final int manual) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            final DataSourceNew dataSource = dataSourceTable.get(ds_objid);
            if (dataSource.getManual() == manual) {
                return;
            }
            dataSourceTable.updateManual(ds_objid, manual);
            acquisitionServerSession.notifyDsConfigurationChange(ds_objid);

        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to update id:" + ds_objid, e);
        }
        catch (AcquisitionServerException e) {
            throw new AcquisitionUnitException("Failed to update id:" + ds_objid, e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public void updateProp(String auName, long ds_objid, Properties attrs) throws AcquisitionUnitException {
        updateProp(auName, ds_objid, serializeProperties(attrs));
    }

    @Override
    public void updateProp(final String auName, final long ds_objid, final String prop) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            if (logger.isDebugEnabled()) {
                logger.debug("AcquisitionFacadeSessionEJB.updateProp(au=" + auName + ";ds=" + ds_objid + ")");
            }

            connection = ConnectionHelper.newConnection(dataSource);
            final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            final DataSourceAttrsTable dataSourceAttrsTable = new DataSourceAttrsTable(connection);
            final AcquisitionUnitTableNew acquisitionUnitTable = new AcquisitionUnitTableNew(connection);

            final AcquisitionUnitNew auNew = acquisitionUnitTable.getByName(auName);
            if (auNew == null) {
                logger.error("Fail Find acquisitionUnitName:" + auName);
                throw new AcquisitionUnitException("Fail Find acquisitionUnitName:" + auName);
            }

            // Find the data source to update (using the ID)
            final DataSourceNew dataSource = dataSourceTable.get(ds_objid);
            fillDataSourceWithAttrs(dataSource, connection);

            final Properties originalAttrs = dataSource.getAttrs();

            final Properties incomingAttrs = PropertiesEncoderHelper.deserializeProperties(prop);
            incomingAttrs.remove("____");

            // Encrypt the incoming OCIE password
            encryptOCIEPasswordInAttrs(incomingAttrs);

            // TA38128  Acquisition US15248: OPM - Fixed tick netappstorageperformance polling
            // Do we need to calculate a new clock poll offset?

            // Is the clock polling offset feature transitioning to on?
            final boolean originalOffsetEnabled = Boolean.parseBoolean(originalAttrs.getProperty("netappstorageperformance.clock.polling.enabled", "false"));
            final boolean incomingOffsetEnabled = incomingAttrs.containsKey("netappstorageperformance.clock.polling.enabled")
                    && Boolean.parseBoolean(incomingAttrs.getProperty("netappstorageperformance.clock.polling.enabled)"));
            final boolean offsetEnabled = incomingOffsetEnabled && !originalOffsetEnabled;      // transitioned from false to true

            // Do we already have an existing poll offset?
            final String originalOffsetString = originalAttrs.getProperty("netappstorageperformance.clock.polling.offset");
            Integer originalOffset = -1;
            if (originalOffsetString != null) {
                try {
                    originalOffset = Integer.parseInt(originalOffsetString);
                } catch (NumberFormatException ignore) {}
            }
            final boolean haveOriginalOffset = !originalOffset.equals(-1);

            // Did the user specify an offset directly?
            final String incomingOffsetString = incomingAttrs.getProperty("netappstorageperformance.clock.polling.offset");
            Integer incomingOffset = -1;
            if (incomingOffsetString != null) {
                try {
                    incomingOffset = Integer.parseInt(incomingOffsetString);
                } catch (NumberFormatException ignore) {}
            }
            final boolean offsetSpecified = (!incomingOffset.equals(-1));

            // Did the performance poll change?
            final String previousPerformancePoll = originalAttrs.getProperty("netappstorageperformance.poll");
            final boolean perfPollChanged = incomingAttrs.containsKey("netappstorageperformance.poll") &&
                    !previousPerformancePoll.equals(incomingAttrs.getProperty("netappstorageperformance.poll"));

            final boolean needOCIEClockOffset =
                    !offsetSpecified &&                                     // User specified offset overrides need to calculate one
                        (perfPollChanged ||                                 // Performance poll change requires new offset
                                (offsetEnabled && !haveOriginalOffset));    // Feature enabled for first time requires new offset

            if (needOCIEClockOffset) {
                // NOTE: incomingAttrs is initialized by read all properties from database, then override the
                //       updated properties value to the properties hash.
                //       This line will only execute when offset is needed but missing in database.
                initializedOCIEPerformancePolling(dataSource.getName(), dataSource.getDsTypeId(), incomingAttrs, connection);
            }


            // fix "foundation.poll" attribute - change minutes to seconds !
            // I, Dmitry Erkin, failed to change the GUI to do it, so I'm fixing it here.
            // TODO: in new GUI framework, it should be easier to do.
            final String foundationPollStr = incomingAttrs.getProperty("foundation.poll");
            if (foundationPollStr != null) {
                final int foundationPoll = Integer.parseInt(foundationPollStr);
                incomingAttrs.setProperty("foundation.poll", Integer.toString(foundationPoll * 60));
            }

            final String nasPollStr = incomingAttrs.getProperty("nas.poll");
            if (nasPollStr != null) {
                final int nasPoll = Integer.parseInt(nasPollStr);
                incomingAttrs.setProperty("nas.poll", Integer.toString(nasPoll * 60));
            }

            final long au_objid = dataSource.getAuId();

            // Checks if acq. unit name has changed
            if (auNew.getId() != au_objid) {
                dataSource.setAuId(auNew.getId());
            }

            for (Enumeration enumeration = incomingAttrs.keys(); enumeration.hasMoreElements(); ) {
                final String name = (String) enumeration.nextElement();

                // HEADSUP We do not audit the following property changes- they are considered metadata:
                // transport.record
                // transport.crawl
                final String[] nameParts = name.split("\\.");
                if (nameParts.length > 2 && nameParts[1].equals("transport")) {
                    continue;
                }

                // First get each new property and check to see if its different from the origional
                final String originalValue = originalAttrs.getProperty(name);
                if (originalValue == null) {
                    // A property was added. This is a rare occurance and not due to user interaction
                    if (logger.isDebugEnabled()) {
                        logger.debug("AcquisitionFacadeSessionEJB.updateProp() - new prop was added : name=" + name + ";originalValue=" + originalValue + ";new value=" + incomingAttrs.get(name));
                    }
                }
            }

            incomingAttrs.setProperty("domain.id", auNew.getName());
            dataSource.setAttrs(incomingAttrs);

            // Update the data source record in the database
            //            dataSource.setActive(false);
            dataSourceTable.update(dataSource);
            dataSourceAttrsTable.update(ds_objid, dataSource.getAttrs());

            acquisitionServerSession.notifyDsConfigurationChange(ds_objid);

        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to update Prop", e);
        }
        catch (AcquisitionServerException e) {
            throw new AcquisitionUnitException("Failed to update Prop", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    /**
     * For the given array of DataSource IDs, find all corresponding originators and "stop reporting" on
     * each.   This will cause each relevant device to be flagged as "dead" and greyed out in the UI.  This is
     * usually triggered by a DataSource deletion request.
     */
    @Override
    public void stopReport(final long[] ids) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            for (long ds_objid : ids) {
                if (logger.isDebugEnabled()) {
                    logger.debug("stopReport() - (datasourceId:" + ds_objid + ")");
                }

                OriginatorTable originatorTable = new OriginatorTable(connection);
                Set<Long> origionatorIds = originatorTable.findByDsId(ds_objid);
                for (final Long originatorId : origionatorIds) {
                    stopReportOriginatorID(originatorId);
                }
            }

            updateLastUpdate();
        }
        catch (TableException e) {
            throw new AcquisitionUnitException(e.getMessage(), e);
        }

        finally {
            ConnectionHelper.close(connection);
        }
    }

    private OriginatorUpdateSessionLocal getOriginatorUpdateSessionLocal() {
        try {
            return SessionFactory.createSession(OriginatorUpdateSessionLocal.JNDI_NAME);
        }
        catch (NamingException e) {
            logger.error("Failed to createSession " + OriginatorUpdateSessionLocal.JNDI_NAME, e);
            throw new EJBException("Failed to createSession " + OriginatorUpdateSessionLocal.JNDI_NAME, e);
        }
    }

    /**
     * Returns the most appropiated Originator ID's name using a sequence of posible options.  The sequece of
     * if-statements is important based on priority.
     *
     * @param deviceKey
     * @param originatorName
     * @param dataSourceName
     *
     * @throws AcquisitionUnitException
     */
    private String figureOutOriginatorName(final String deviceKey, final String originatorName, final String dataSourceName) throws AcquisitionUnitException {
        if (originatorName != null) {
            return originatorName;
        }

        if (deviceKey != null) {
            return deviceKey;
        }

        if (dataSourceName != null) {
            return dataSourceName;
        }

        throw new AcquisitionUnitException("figureOutOriginatorName() failed.  Impossible figure out a name for this Originator ID");
    }

    private void updateLastUpdate() {
        lastUpdate = System.currentTimeMillis();
        SummarySessionEJB.updateTimeDatasources = lastUpdate;
    }

    private DataSource convertFromNewToOld(final DataSourceNew dataSource, final Connection connection) throws TableException {
        if (dataSource == null) {
            return null;
        }

        // put type attrs first
        final DataSourceTypeAttrsTable dsTypeAttrsTable = new DataSourceTypeAttrsTable(connection);
        final List<DsTypeAttribute> dstAttrs = dsTypeAttrsTable.findByDsTypeId(dataSource.getDsTypeId());
        final Properties attrs = new Properties();

        for (final DsTypeAttribute dstAttr : dstAttrs) {
            DsTypeAttribute acqAttr = dstAttr;
            attrs.setProperty(acqAttr.getName(), acqAttr.getValue());
        }

        // override with ds attrs
        DataSourceAttrsTable dsAttrsTable = new DataSourceAttrsTable(connection);
        attrs.putAll(dsAttrsTable.findByDsId(dataSource.getId()));

        // fix "port" if needed
        final String port = attrs.getProperty("foundation.port");
        if (port == null || port.length() == 0) {
            attrs.setProperty("foundation.port", "-2147483648");
        }

        // fix "foundation.poll" attribute - change seconds to minutes !
        // I, Dmitry Erkin, failed to change the GUI to do it, so I'm fixing it here.
        // TODO: in new GUI framework, it should be easier to do.
        final String foundationPollStr = attrs.getProperty("foundation.poll");
        if (foundationPollStr != null) {

            //final int v = Integer.parseInt(foundationPollStr);
            // TODO: temp *incorrect* fix. redo!
            final int v = (int) Float.parseFloat(foundationPollStr);

            attrs.setProperty("foundation.poll", Integer.toString(v / 60));
        }

        final String nasPollStr = attrs.getProperty("nas.poll");
        if (nasPollStr != null) {
            final int v = Integer.parseInt(nasPollStr);
            attrs.setProperty("nas.poll", Integer.toString(v / 60));
        }

        // add metadata string
        attrs.setProperty("____", buildMetaDataString(dstAttrs, true));

        dataSource.setAttrs(attrs);

        return new DataSource(
                dataSource.getId(),
                0,
                Long.toString(dataSource.getDsTypeId()),
                Integer.parseInt(dataSource.getAttrs().getProperty("foundation.port")),
                getIp(dataSource.getAttrs()),
                dataSource.getName(),
                dataSource.getVendor(),
                dataSource.getModel(),
                dataSource.getAttrs().getProperty("domain.id"),
                (int) dataSource.getDsTypeId(),
                dataSource.getManual(),
                dataSource.isActive(),
                dataSource.getTime(),
                ServerConstant.END_DATE,
                PropertiesEncoderHelper.serializeProperties(attrs),
                dataSource.getStatus(),
                dataSource.getAttrs().getProperty("manage.url"));
    }

    private String buildMetaDataString(final List<DsTypeAttribute> dstAttrs, final boolean adjustPollingInterval) {
        // Marshal the attributes in an XML format
        StringBuilder xml = new StringBuilder(500);
        xml.append("<properties>");

        for (final DsTypeAttribute attribute : dstAttrs) {
            // fix "poll" attribute - change seconds to minutes !
            // I, Dmitry Erkin, failed to change the GUI to do it, so I'm fixing it here.
            // TODO: in new GUI framework, it should be easier to do.
            final String attrName = attribute.getName();
            final String attrValue = attribute.getValue();

            if (adjustPollingInterval) {

                if (attrValue != null && attrName.equals("foundation.poll")) {
                    final int v = Integer.parseInt(attrValue);
                    attribute.setValue(Integer.toString(v / 60));
                }

                if (attrValue != null && attrName.equals("nas.poll")) {
                    final int v = Integer.parseInt(attrValue);
                    attribute.setValue(Integer.toString(v / 60));
                }
            }

            attribute.toXML(xml);
        }
        xml.append("</properties>");

        return xml.toString();
    }

    @Override
    @RolesAllowed({"admin", "acquisition"})
    @TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
    public void removeDSs(final long[] ids) throws AcquisitionUnitException {
        Connection connection = null;
        final List<Long> missingIds = new ArrayList<Long>();
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            for (long id : ids) {

                if (getDataSourceNameByDatasourceId(id) == null) {
                    missingIds.add(id);
                    continue;
                }

                // Clear the audit history for this data source
                this.removeDsAudit(id);

                DataSourceNew dataSourceNew = dataSourceTable.get(id);
                // Now we can remove the data source itself
                dataSourceTable.deleteById(id);

                acquisitionServerSession.notifyDsRemoval(dataSourceNew.getAuId(), dataSourceNew.getName());
            }
        }
        catch (TableException | AcquisitionServerException e) {
            throw new AcquisitionUnitException("Failed to remove datasources", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }

        //catch case of missing IDs passed in
        if (!missingIds.isEmpty()) {
            throw new AcquisitionUnitException(DataSourceDoesNotExistException.buildString(missingIds));
        }

    }

    @Override
    @RolesAllowed({"admin", "acquisition"})
    public void removeAllDataSource() throws AcquisitionUnitException {
        final long[] ids;

        // Obtain the list of all data sources
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);

            final List<DataSourceNew> dataSources = dataSourceTable.getAll();

            ids = new long[dataSources.size()];

            int i = 0;
            for (DataSourceNew ds : dataSources) {
                ids[i++] = ds.getId();
            }
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to remove all datasources", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }

        // And now remove these DS
        removeDSs(ids);
    }

    /**
     * Is DataSource safe to delete?   Means either the AU is down, or that the manual state is 2 and it is no
     * actively polling.
     *
     * @param id         object ID of DataSource
     * @param connection existing DB connection.
     *
     * @return true if safe for delete
     *
     * @throws TableException DB errors.
     */
    private boolean isDataSourceDeleteSafe(final long id, Connection connection) throws TableException {
        final DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
        final AcquisitionUnitTableNew auTable = new AcquisitionUnitTableNew(connection);
        final DataSourceNew dataSourceNew = dataSourceTable.get(id);
        final AcquisitionUnitNew au = auTable.get(dataSourceNew.getAuId());

        //Safe for delete if AU is down, else if DataSource is updateManual of 2 and active = false.
        return !au.isActive() || (dataSourceNew.getManual() == 2 && !dataSourceNew.isActive());
    }

    @Override
    @RolesAllowed({"admin", "acquisition"})
    @TransactionAttribute(TransactionAttributeType.NOT_SUPPORTED)
    public void removeDataSource(final long id) throws AcquisitionUnitException {

        //Note that this must make several changes to Acq tables immediately, so it should not run in its own
        //transaction (NOT_SUPPORTED) to avoid caching.  Those tables are also updated by AU unit.

        Connection connection = null;
        try {

            connection = ConnectionHelper.newConnection(dataSource);

            //0. get the clusterUUID before it's deleted
            final String clusterUuid = NetAppModel.INSTANCE.getClusterUUIDByDatasourceID(connection, id);

            //1.  stop the DataSource first
            //wait for it to stop.  Wait maximum of 20 seconds.

            if (!isDataSourceDeleteSafe(id, connection)) {
                int time = 0;
                do {
                    updateManual(id, 2);
                    try {
                        Thread.sleep(2000);
                    }
                    catch (InterruptedException ie) {
                        //ignore
                    }
                    time += 2;
                }
                while (!isDataSourceDeleteSafe(id, connection) && time < 20);

                if (!isDataSourceDeleteSafe(id, connection)) {
                    //This is a quirky situation.   Since I don't trust the AU side to always set isActive to
                    //false, we want to avoid getting permenantly stuck where we can't delete.   So, just warn
                    //and try to delete (we will have waited for the loop above anyways)
                    logger.warn("Could not cleanly stop DataSource with ID=" + id);
                }
            }

            final long[] idArray = new long[]{id};

            synchronized (OriginatorUpdateImpl.INSTANCE.getOriginatorSyncObject(id)) {
                // burt 1097636: stopReport (clean netapp_model data) and removeDSs (remove acquisition.ds table entry)
                // should to be locked together.

                //2.clear (stop) the originator reports.
                stopReport(idArray);

                List<String> filterRules = NetAppModel.INSTANCE.getEMSMessageFilterRules(connection);
                Properties dsProperties = NetAppModel.INSTANCE.getDatasourceProperties(connection, id);
                if (!filterRules.isEmpty()) {
                    AcquisitionFacadeUtil.unRegisterEms(AcquisitionFacadeUtil.getHostName(), dsProperties);
                }
                //3.finally remove the DS.
                removeDSs(idArray);
            }

            //4. remove EMS event and its parameters associated with this DS
            if (clusterUuid != null) {
                netAppEmsEventPersistence.deleteEmsEventsByCluserUuid(clusterUuid);
            }

        } catch (TableException | EmsZapiManagerException ex) {
            throw new AcquisitionUnitException("Could not delete DataSource ID " + id + ": " + ex);
        } catch (NetAppEmsEventException ex) {
            throw new AcquisitionUnitException("Could not delete EMS events associated with DataSource " + id + ": " + ex);
        } finally { //close the connection for sure.
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @RolesAllowed({"admin", "acquisition", "scheduler"})
    @TransactionAttribute(TransactionAttributeType.NOT_SUPPORTED)
    public void removeDsAudit(long[] dsIDs) throws AcquisitionUnitException {
        removeDsAuditOnly(dsIDs);
        updateDS(dsIDs);
    }

    @Override
    public void removeDsAuditOnly(long[] dsIDs) throws AcquisitionUnitException {
        for (long dsID : dsIDs) {
            acquisitionServerSession.clearDsAudit(dsID);
        }
    }

    @Override
    public void updateDS(long[] dsIDs) throws AcquisitionUnitException {
        // Find all data sources for this acquisition unit
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            for (long dsID : dsIDs) {
                auditManager.updateDS(dsID, connection);
            }
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to update ds", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    /**
     * Delete all the data source audit records for a specific data source
     */
    @Override
    public void removeDsAudit(long dsID) throws AcquisitionUnitException {
        acquisitionServerSession.clearDsAudit(dsID);
    }

    /**
     * Deletes all audit records for all data sources belonging for this acq unit
     */
    @Override
    public void removeDsAuditForAu(final String acquisitionUnitName) throws AcquisitionUnitException {
        acquisitionServerSession.clearDsAuditForAu(acquisitionUnitName);
    }

    @Override
    public void updateDsName(long id, String newName) throws AcquisitionUnitException {

        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);

            // Find the data source to update (using the ID)
            DataSourceNew dataSource = dataSourceTable.get(id);

            if (!dataSource.getName().equals(newName)) {
                // Remember the old name and set the new one
                dataSource.setName(newName);
            }
            dataSourceTable.update(dataSource);

            this.acquisitionServerSession.notifyDsConfigurationChange(id);

        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to update ds name", e);
        }
        catch (AcquisitionServerException e) {
            throw new AcquisitionUnitException("Failed to update ds name", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public DataSource getDataSource(String ip, int deviceTypeId, long time) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);

            // Find the data source to update (using the ID)
            DataSourceNew dataSource = dataSourceTable.getDS(ip, deviceTypeId, time);
            return convertFromNewToOld(dataSource, connection);
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to update ds name", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public long getLastUpdate() throws AcquisitionUnitException {
        return lastUpdate;
    }

    @Override
    public DataSource[] getDsForAu(String acqUnitName) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);

            List<DataSourceNew> dataSources = dataSourceTable.findByAuName(acqUnitName);
            return convertFromNewToOld(dataSources, connection);
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get all open", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public List<DataSourceNew> getDsForAuNew(String acqUnitName) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);

            List<DataSourceNew> dataSources = dataSourceTable.findByAuName(acqUnitName);
            fillDataSourcesWithAttrs(dataSources, connection);
            return dataSources;
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get all open", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    private DataSource[] convertFromNewToOld(final List<DataSourceNew> dataSources, final Connection connection) throws TableException {
        DataSource[] result = new DataSource[dataSources.size()];

        final Iterator<DataSourceNew> iterator = dataSources.iterator();
        int i = 0;
        while (iterator.hasNext()) {
            DataSourceNew ds = iterator.next();
            result[i++] = convertFromNewToOld(ds, connection);
        }

        return result;
    }

    @Override
    public DataSource[] getAllDs() throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);

            List<DataSourceNew> dataSources = dataSourceTable.findAll();
            return convertFromNewToOld(dataSources, connection);
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get all open", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public List<DataSourceNew> getAllDsNew() throws AcquisitionUnitException {
        return getAllDsNew(true);
    }

    @Override
    public List<DataSourceNew> getAllDsNew(boolean includeAttributes) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);

            final List<DataSourceNew> dataSources = dataSourceTable.findAll();
            if (includeAttributes) {
                fillDataSourcesWithAttrs(dataSources, connection);
                for (DataSourceNew ds : dataSources) {
                    final String foundationPoll = ds.getAttrs().getProperty("foundation.poll");
                    if (foundationPoll != null) {

                        // TODO: temp *incorrect* fix. redo!
                        final int v = (int) Float.parseFloat(foundationPoll);
                        ds.getAttrs().setProperty("foundation.poll", Integer.toString(v / 60));
                    }

                    final String nasPoll = ds.getAttrs().getProperty("nas.poll");
                    if (nasPoll != null) {
                        final int v = Integer.parseInt(nasPoll);
                        ds.getAttrs().setProperty("nas.poll", Integer.toString(v / 60));
                    }
                }
            }

            return dataSources;
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get all open", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    private void fillDataSourcesWithAttrs(final List<DataSourceNew> dataSources, final Connection connection) throws TableException, AcquisitionUnitException {
        for (DataSourceNew ds : dataSources) {
            fillDataSourceWithAttrs(ds, connection);
        }
    }

    private void fillDataSourceWithAttrs(final DataSourceNew ds, final Connection connection) throws TableException, AcquisitionUnitException {
        final Properties attrs = new Properties();

        String maxIgnoreDynamicFor = "6";

        // Add default value for maximum time to suppress
        attrs.setProperty("foundation.max.ignore.dynamic.for.hours", maxIgnoreDynamicFor);

        // put Type attrs first
        final DataSourceTypeAttrsTable typeAttrsTable = new DataSourceTypeAttrsTable(connection);
        final List<DsTypeAttribute> typeAttrs = typeAttrsTable.findByDsTypeId(ds.getDsTypeId());
        for (Object typeAttr1 : typeAttrs) {
            DsTypeAttribute typeAttr = (DsTypeAttribute) typeAttr1;
            attrs.setProperty(typeAttr.getName(), typeAttr.getValue());
        }

        // overwrite/add datasource attrs over type attrs
        final DataSourceAttrsTable attrsTable = new DataSourceAttrsTable(connection);
        attrs.putAll(attrsTable.findByDsId(ds.getId()));

        // decrypt the OCIE password before sending back to clients
        decryptOCIEPasswordInAttrs(attrs);

        // done
        ds.setAttrs(attrs);
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public DataSource[] getAllDsByTime(long date) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);

            List<DataSourceNew> dataSources = dataSourceTable.findByTime(date);
            return convertFromNewToOld(dataSources, connection);
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get all open", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public DataSource[] getDsByIds(long[] ids) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);

            List<DataSourceNew> dataSources = new LinkedList<DataSourceNew>();
            for (long id : ids) {
                DataSourceNew ds = dataSourceTable.getById(id);
                dataSources.add(ds);
            }

            return convertFromNewToOld(dataSources, connection);
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get all open", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    /**
     * Get the name of the originator. Return the name defined in the device key of the originator table. If the name is
     * not defined (happens when there is only one originator in the data source) - return the the name of the data
     * source.
     */
    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public String getOriginatorNameOriginatorId(Long originId) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            OriginatorTable originatorTable = new OriginatorTable(connection);
            Originator originatorRecord = originatorTable.find(originId.longValue());
            return originatorRecord != null ? originatorRecord.getName() : null;
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to getOriginatorName(), originId: [" + originId + "]", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public String getDataSourceNameByDatasourceId(Long dataSourceId) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            DataSourceNew ds = dataSourceTable.getById(dataSourceId);
            if (ds == null) {
                return null;
            }

            return ds.getName();
        }
        catch (Exception e) {
            throw new AcquisitionUnitException("Failed to find ds name", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public DataSource getOpenById(long id) throws VendorModelNotFoundException, AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            DataSourceNew ds = dataSourceTable.getById(id);
            if (ds == null) {
                return null;
            }

            return convertFromNewToOld(ds, connection);
        }
        catch (Exception e) {
            throw new AcquisitionUnitException("Failed to find ds", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public DataSource getById(long id, long date) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);
            DataSourceNew ds = dataSourceTable.getByIdAndTime(id, date);
            if (ds == null) {
                return null;
            }

            return convertFromNewToOld(ds, connection);
        }
        catch (Exception e) {
            throw new AcquisitionUnitException("Failed to find ds", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public DataSourceAudit[] getAllDsHistoryById(long dsID) throws AcquisitionUnitException {
        return auditManager.getAuditHistory(dsID);
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public AcquisitionUnitNew getAcquisitionUnit(final String acquisitionUnitName) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            AcquisitionUnitTableNew acquisitionUnitTable = new AcquisitionUnitTableNew(connection);
            final Collection<AcquisitionUnitNew> col = acquisitionUnitTable.findByName(acquisitionUnitName);

            if (col.size() != 1) {
                return null;
            }

            return col.iterator().next();
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to getAcquisitionUnit acquisitionUnitName:" + acquisitionUnitName, e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public Collection<AcquisitionUnit> getAllAcquisitionUnit() throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            AcquisitionUnitTableNew acquisitionUnitTable = new AcquisitionUnitTableNew(connection);
            final List<AcquisitionUnitNew> col = acquisitionUnitTable.findAll();

            List<AcquisitionUnit> result = new LinkedList<AcquisitionUnit>();
            for (AcquisitionUnitNew au : col) {
                result.add(au.getAcquisitionUnit());
            }

            return result;
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to getAcquisitionUnits", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public AcquisitionUnit[] getAcquisitionUnitsByNames(String[] names, int status, boolean remoteOnly) throws AcquisitionUnitException {
        Collection<AcquisitionUnit> allAcquisitionUnit = getAllAcquisitionUnit();
        Set<String> nameSet = new HashSet<String>(Arrays.asList(names));
        if (remoteOnly) {
            nameSet.remove("local");
        }

        List<AcquisitionUnit> acqUnitsByName = new LinkedList<AcquisitionUnit>();
        for (AcquisitionUnit acqUnit : allAcquisitionUnit) {
            if (nameSet.contains(acqUnit.getName()) && acqUnit.getStatusWithLeaseCheck() == status) {
                acqUnitsByName.add(acqUnit);
            }
        }

        return acqUnitsByName.toArray(new AcquisitionUnit[acqUnitsByName.size()]);
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public void removeAcqUnitsByNames(String[] names) throws AcquisitionUnitException {

        final List<String> failures = new ArrayList<String>();
        for (final String name : names) {
            try {
                this.acquisitionServerSession.removeAcquisitionUnitByName(name);
            }
            catch (final AcquisitionServerException ase) {
                failures.add(name + ": " + ase.getMessage());
            }
        }

        if (failures.size() > 0) {
            final StringBuffer temp = new StringBuffer("Failed to removeAcqUnitsByNames: ");
            for (final String failure : failures) {
                temp.append(failure).append("; ");
            }
            throw new AcquisitionUnitException(temp.substring(0, temp.length() - 1));
        }

    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public Collection<DataSourceVendorModel> geAllDatasourceTypeVendorModel() throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            final List<DataSourceTypeVendorModel> models = new DataSourceTypeVendorModelTable(connection).findAll();
            List<DataSourceVendorModel> result = new LinkedList<DataSourceVendorModel>();
            for (DataSourceTypeVendorModel dataSourceTypeVendorModel : models) {
                result.add(dataSourceTypeVendorModel.getDataSourceVendorModel());
            }

            return result;
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get all VendorModels", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public Map<Long, DataSourceNew> getOriginIdToDatasource() throws AcquisitionUnitException {
        Connection connection = null;
        try {
            // First we check if deviceKey is null. If so, we return the oldOriginatorID from the datasource Table
            connection = ConnectionHelper.newConnection(dataSource);
            Map<Long, DataSourceNew> originIdToDatasource = new HashMap<Long, DataSourceNew>();
            Map<Long, DataSourceNew> datasourceIdToDatasource = new HashMap<Long, DataSourceNew>();
            OriginatorTable originatorTable = new OriginatorTable(connection);
            DataSourceTableNew dataSourceTable = new DataSourceTableNew(connection);

            List<DataSourceNew> datasources = dataSourceTable.findAll();
            for (DataSourceNew dataSource : datasources) {
                datasourceIdToDatasource.put(dataSource.getId(), dataSource);
            }

            List<Originator> originators = originatorTable.findAll();
            for (Originator originatorRecord : originators) {
                DataSourceNew dataSource = datasourceIdToDatasource.get(originatorRecord.getDsId());
                if (dataSource != null) {
                    originIdToDatasource.put(originatorRecord.getId(), dataSource);
                }
            }

            return originIdToDatasource;
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to connect to database", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @RolesAllowed({"guest", "admin", "acquisition"})
    public Map<String, AcquisitionUnit> getAcquisitionUnitsByName() throws AcquisitionUnitException {
        Collection<AcquisitionUnit> allAcquisitionUnit = getAllAcquisitionUnit();

        Map<String, AcquisitionUnit> nameToAcquisitionUnit = new HashMap<String, AcquisitionUnit>();
        for (AcquisitionUnit acquisitionUnitData : allAcquisitionUnit) {
            nameToAcquisitionUnit.put(acquisitionUnitData.getName(), acquisitionUnitData);
        }

        return nameToAcquisitionUnit;
    }

    @Override
    @RolesAllowed({"guest", "admin", "acquisition"})
    public Map<Long, AcquisitionUnitNew> getAcquisitionUnitsByID() throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            AcquisitionUnitTableNew acquisitionUnitTable = new AcquisitionUnitTableNew(connection);

            final Map<Long, AcquisitionUnitNew> acqUnitsByID = new HashMap<Long, AcquisitionUnitNew>();

            final List<AcquisitionUnitNew> col = acquisitionUnitTable.findAll();
            for (AcquisitionUnitNew au : col) {
                acqUnitsByID.put(au.getId(), au);
            }

            return acqUnitsByID;
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to getAcquisitionUnits", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    /**
     * Returns an array of {@link DataSourceStatus} instances representing the current data source's status.
     * <p/>
     * The implementation of this method is encapsulated in {@link DataSourceStatusQueryImpl}, which the needed logic
     * has been coded to retrive Data source's status, based on document named:
     * "Data-source alerts Software Requirement Specification"
     */
    @Override
    public DataSourceStatus[] getDataSourceStatusQuery(int timeOutInHours, int timeOutForSinglePollInHours, int numOfAttemps) throws AcquisitionUnitException {
        return new DataSourceStatusQueryImpl().getDataSourceStatusQuery(timeOutInHours, timeOutForSinglePollInHours, numOfAttemps);
    }

    /**
     * Returns an array of {@link DataSourceStatus} instances representing the current data source's status.
     * <p/>
     * This method will take into account default settings used in the logic to determine if a specific data source
     * is in a Success or Failed state.
     * <p/>
     * The implementation of this method is encapsulated in {@link DataSourceStatusQueryImpl}
     */
    @Override
    public DataSourceStatus[] getDataSourceStatusQuery() throws AcquisitionUnitException {
        return new DataSourceStatusQueryImpl().getDataSourceStatusQuery();
    }

    @Override
    public DataSourceVendorModel[] findAllDataSourceVendorModels() throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTypeVendorModelTable dsvtmTable = new DataSourceTypeVendorModelTable(connection);
            List<DataSourceTypeVendorModel> all = dsvtmTable.findAll();
            return convertNewDstvmToOld(all);
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Finder Exception", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    private DataSourceVendorModel[] convertNewDstvmToOld(final List<DataSourceTypeVendorModel> all) {
        DataSourceVendorModel[] result = new DataSourceVendorModel[all.size()];
        int i = 0;
        for (Iterator<DataSourceTypeVendorModel> iterator = all.iterator(); iterator.hasNext(); i++) {
            DataSourceTypeVendorModel model = iterator.next();
            result[i] = model.getDataSourceVendorModel();
        }
        return result;
    }

    @Override
    public DataSourceVendorModel[] findAllVmByDataSourceType(long dataSourceType) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTypeVendorModelTable dsvtmTable = new DataSourceTypeVendorModelTable(connection);
            List<DataSourceTypeVendorModel> all = dsvtmTable.findByDsTypeId(dataSourceType);
            return convertNewDstvmToOld(all);
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Finder Exception", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public DataSourceVendorModel[] findAllVmByVendor(final String vendor) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTypeVendorModelTable dsvtmTable = new DataSourceTypeVendorModelTable(connection);
            List<DataSourceTypeVendorModel> all = dsvtmTable.findByVendor(vendor);
            return convertNewDstvmToOld(all);
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Finder Exception", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public DataSourceVendorModel[] findVmByDataSourceTypeAndVendor(long dataSourceType, String vendor) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTypeVendorModelTable dsvtmTable = new DataSourceTypeVendorModelTable(connection);
            List<DataSourceTypeVendorModel> all = dsvtmTable.findByIdVendor(dataSourceType, vendor);
            return convertNewDstvmToOld(all);
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Finder Exception", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public DataSourceVendorModel findVmByVendorModel(final String vendor, final String model) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTypeVendorModelTable dsvtmTable = new DataSourceTypeVendorModelTable(connection);
            return dsvtmTable.getByVendorModel(vendor.toUpperCase(), model.toUpperCase()).getDataSourceVendorModel();
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Finder Exception", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public String getDsTypeProperty(long dataSourceType, final String name) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTypeAttrsTable table = new DataSourceTypeAttrsTable(connection);
            final DsTypeAttribute attr = table.getAttribute(dataSourceType, name);
            return attr.getValue();
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get default properties dataSourceType:" + dataSourceType, e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public Collection<DataSourceDefaults> getDsDefaults() throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            final List<DataSourceDefaults> result = new LinkedList<DataSourceDefaults>();

            final DataSourceTypeTable dstTable = new DataSourceTypeTable(connection);
            final DataSourceTypeAttrsTable table = new DataSourceTypeAttrsTable(connection);

            final List<DataSourceType> types = dstTable.findAll();
            for (DataSourceType dst : types) {
                List<DsTypeAttribute> attrs = table.findByDsTypeId(dst.getId());

                Properties props = new Properties();
                for (final DsTypeAttribute attr : attrs) {
                    props.setProperty(attr.getName(), attr.getValue());
                }

                props.setProperty("____", buildMetaDataString(attrs, false));

                DataSourceDefaults defaults = new DataSourceDefaults((int) dst.getId(), dst.getName(), dst.getDescription(), PropertiesEncoderHelper.serializeProperties(props));
                result.add(defaults);
            }

            return result;
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get ds defaults", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public ByteArrayWrapper getDsPolicy(long dataSourceType) throws AcquisitionUnitException {
        return null;
    }

    @Override
    public String getDsPolicyClassName(long dataSourceType) throws AcquisitionUnitException {
        return null;
    }

    @Override
    public String getDsDefaultName(long dataSourceType) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTypeTable table = new DataSourceTypeTable(connection);
            DataSourceType dst = table.getById(dataSourceType);
            return dst.getName();
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get default name dataSourceType:" + dataSourceType, e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public String getDefaultDescription(long dataSourceType) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            DataSourceTypeTable table = new DataSourceTypeTable(connection);
            DataSourceType dst = table.getById(dataSourceType);
            return dst.getDescription();
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get default description dataSourceType:" + dataSourceType, e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    @Override
    public Map<Long, Originator> getOriginatorIdToOriginator() throws AcquisitionUnitException {
        Connection connection = null;
        try {
            Map<Long, Originator> res
                    = new HashMap<Long, Originator>();
            connection = ConnectionHelper.newConnection(dataSource);
            OriginatorTable table = new OriginatorTable(connection);
            for (Originator originator : table.getAll()) {
                res.put(originator.getId(), originator);
            }
            return res;
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to getgetOriginatorIdToOriginator", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    /**
     * @todo Revisit this during Bug Fixing
     */
    @Override
    public Map<Long, com.netapp.acquisition.aquisitionserverside.interfaces.data.Originator> getOriginatorIdToAcqOriginator() throws AcquisitionUnitException {
        Connection connection = null;
        try {
            Map<Long, com.netapp.acquisition.aquisitionserverside.interfaces.data.Originator> res
                    = new HashMap<Long, com.netapp.acquisition.aquisitionserverside.interfaces.data.Originator>();
            connection = ConnectionHelper.newConnection(dataSource);
            OriginatorTable table = new OriginatorTable(connection);
            for (Originator originator : table.getAll()) {
                com.netapp.acquisition.aquisitionserverside.interfaces.data.Originator acqOriginator = new
                        com.netapp.acquisition.aquisitionserverside.interfaces.data.Originator();

                acqOriginator.setId(originator.getId());
                acqOriginator.setDsId(originator.getDsId());
                acqOriginator.setDeviceKey(originator.getDeviceKey());
                acqOriginator.setName(originator.getName());
                acqOriginator.setLastAccessTime(originator.getLastAccessTime());

                res.put(originator.getId(), acqOriginator);
            }
            return res;
        }
        catch (TableException e) {
            throw new AcquisitionUnitException("Failed to getgetOriginatorIdToOriginator", e);
        }
        finally {
            ConnectionHelper.close(connection);
        }
    }

    private String getIp(Properties attrs) {
        String ipProperty = attrs.getProperty("foundation.ip");
        if (ipProperty == null) {
            // Maybe it's not foundation.ip but <something>.ip?
            for (String propertyName : attrs.stringPropertyNames()) {
                if (propertyName.endsWith(".ip")) {
                    ipProperty = attrs.getProperty(propertyName);
                    break;
                }
            }
        }
        return ipProperty;
    }

    /**
     * This method is taken from the acquisition module. It encodes the properties into a single stream in exactly the
     * same way as the acquisition framework does.
     *
     * @param properties Properties to encode
     *
     * @return A string representation of the property set.
     */
    private String serializeProperties(final Properties properties) {
        try {
            final ByteArrayOutputStream baos = new ByteArrayOutputStream();
            final XMLEncoder encoder = new XMLEncoder(baos);
            try {
                encoder.writeObject(properties);
            }
            finally {
                encoder.close();
            }

            return new String(baos.toByteArray(), "utf-8").replaceAll("[\\n\\r]", "");
        }
        catch (UnsupportedEncodingException e1) {
            throw new RuntimeException(e1);
        }
    }


    /**
     * Encrypt the OCIE netappfoundation.password in the given data source properties.
     * OCIE contract dictates that password encryption happen in the server, and NOT
     * in clients (acq, cli, gui...).   This is a different contract than OCI and OCR.
     *
     * @param dsAttrs the data source properties
     */
    private void encryptOCIEPasswordInAttrs(final Properties dsAttrs) throws AcquisitionUnitException {
        if (dsAttrs.containsKey(OCIE_PASSWORD)) {
            // OCIE netappfoundation password is passed in clear text must be encrypted here
            final String ocieClearTextPassword = dsAttrs.getProperty(OCIE_PASSWORD);
            try {
                final String ocieEncryptedPassword = acquisitionServerSessionLocal.encryptPassword(ocieClearTextPassword);
                dsAttrs.setProperty(OCIE_PASSWORD, ocieEncryptedPassword);
            } catch (AcquisitionServerException e) {
                throw new AcquisitionUnitException("Failed to encrypt netappfoundation.password: " + e.getMessage(), e);
            }
        }

    }

    /**
     * Decrypt the OCIE netappfoundation.password in the given data source properties.
     * OCIE contract dictates that password decryption happen in the server, and NOT
     * in clients (acq, cli, gui...).   This is a different contract than OCI and OCR.
     *
     * @param dsAttrs the data source properties
     */
    private void decryptOCIEPasswordInAttrs(final Properties dsAttrs) throws AcquisitionUnitException {
        if (dsAttrs.containsKey(OCIE_PASSWORD)) {
            // OCIE netappfoundation password is passed in clear text must be encrypted here
            final String ocieEncryptedPassword = dsAttrs.getProperty(OCIE_PASSWORD);
            try {
                final String ocieClearTextPassword = acquisitionServerSessionLocal.decryptPassword(ocieEncryptedPassword);
                dsAttrs.setProperty(OCIE_PASSWORD, ocieClearTextPassword);
            } catch (AcquisitionServerException e) {
                throw new AcquisitionUnitException("Failed to decrypt netappfoundation.password: " + e.getMessage(), e);
            }
        }
    }

    /**
     *
     * @param dataSourceId
     * @return
     * @throws AcquisitionUnitException
     */
    @Override
    public long getLatestPerformancePollTimestamp(final long dataSourceId) throws AcquisitionUnitException {
        if(dataSourceId <0 )  {
            throw new AcquisitionUnitException("DataSourceID can not be negative ! dataSourceId=" + dataSourceId);
        }
        long latestPerformancePollTS = -1L;

        Connection connection = null;
        final List<Long> result = new ArrayList<>();
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            String sql = String.format("SELECT coalesce(max(time), -1) from netapp_performance.update_history where datasourceId=%s;", dataSourceId);
            TableResultParser<Long> resultParser = new TableResultParser<Long>() {
                @Override
                public List<Long> parseResult(ResultSet resultSet) throws SQLException {
                    while(resultSet.next()) {
                        long timestamp = resultSet.getLong(1);
                        result.add(timestamp);
                        return result;
                    }
                    return null;
                }
            };

            PreparedStatementWrapper statementWrapper = PreparedStatementWrapper.create(connection, sql, resultParser);
            List<Long> timeList = statementWrapper.executeQueryEx();
            if(timeList!=null) {
                latestPerformancePollTS = timeList.get(0);
            }
        } catch(TableException ex) {
            throw new AcquisitionUnitException("Failed to get latest performance poll timestamp: " + ex.getMessage(), ex);
        } finally {
            ConnectionHelper.close(connection);
        }

        return latestPerformancePollTS;

    }

    /**
     * Register with the ds to receive EMS REST push, and mark the ds for EMS destination enabled.
     *
     * @param ds
     * @param dsAttrs
     * @param connection
     * @return null - register DS successfully
     *         DataSourceNew - couldn't register this DS
     */
    private DataSourceNew registerDsForEms(String hostname, DataSourceNew ds, Properties dsAttrs, Connection connection, List<NetAppEmsConfig> emsConfigs) {
        try {
            if(!emsConfigs.isEmpty()) {
                AcquisitionFacadeUtil.registerEms(hostname, dsAttrs, emsConfigs);
            }

            // update ds attribute for EMS destination enabled
            final DataSourceAttrsTable dataSourceAttrsTable = new DataSourceAttrsTable(connection);
            Properties attrs = new Properties();
            attrs.put(EMS_DESTINATION_PROP, hostname);
            dataSourceAttrsTable.insert(ds.getId(), attrs);
            updateLastUpdate();
            return null;
        } catch (TableException | EmsZapiManagerException e) {
            logger.error("Unable to register EMS destination to cluster " + ds.getName(), e);
            return ds;
        }
    }

}
