package com.onaro.sanscreen.acquisition.sessions;

import java.util.List;

import javax.ejb.Local;

import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceNew;

import com.netapp.oci.netapp.ems.shared.interfaces.local.EmsMsgListener;
import com.netapp.oci.platform.common.table.TableException;

/**
 * Acquisition Facade local interaction
 *
 * @author Jason Sprague
 */
@Local
public interface AcquisitionFacadeSessionLocal extends AcquisitionFacadeSession {

    String JNDI_NAME = "java:global/mega-app/server-core/AcquisitionFacadeSessionBean!com.onaro.sanscreen.acquisition.sessions.AcquisitionFacadeSessionLocal";

    /**
     * Sets up acquisition server side for use.   Sets appropriate licensing information and deploys DataSource
     * meta-data to acquisition tables.  This prepares system for an AU to check in as well as for the client to
     * properly show Datasource UI screens.
     *
     * <p>This is enforced to happen exactly once per Server environment (current from @link{StartUpServerMBean}).   Further calls will be no-ops.
     *
     * @param force  force the deployment (or re-deployment).  Typically only true in test simulators.
     * @throws AcquisitionUnitException
     */
    void deploy(boolean force) throws AcquisitionUnitException;

    /**
     * This is enforced to happen exactly once per server restart (current from @link{StartUpServerMBean}). This function is NOT intended to be called by other beans.
     *
     * @throws TableException
     * @return empty list - register DS successfully
     *         List of DataSourceNew - the list of data sources that couldn't be registered for EMS
     *
     */
    List<DataSourceNew> checkEmsRegistrationForUpgrade() throws TableException;

    /**
     * To pull missed EMS events by making a zapi call
     * for all the datasources
     * @param listeners
     */
    void getZapiEmsEvents(List<EmsMsgListener> listeners);

    /**
     * Handles the EMS registration when server certificate is regenerated.
     */
    void handleEmsRegistryForCertRegeneration();

    /**
     * Tells if the input EMS is supported by MEGA.
     *
     * @param emsName
     * @return
     */
    boolean isEmsSupported(String emsName);

}
