package com.onaro.sanscreen.acquisition.sessions;

import com.netapp.acquisition.aquisitionserverside.interfaces.data.AcquisitionUpdateType;
import com.onaro.sanscreen.server.sessions.SummarySessionEJB;

import javax.annotation.security.RunAs;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageListener;

/**
 * MDB to hook into acquisition server side EAR update messaging.
 */

@MessageDriven(activationConfig = {
        @ActivationConfigProperty(
                propertyName = "destinationType",
                propertyValue = "javax.jms.Topic"
        ),
        @ActivationConfigProperty(
                propertyName = "destination",
                propertyValue = "java:/topic/AcquisitionUpdateTopic"
        ),
        @ActivationConfigProperty(
                propertyName = "acknowledgeMode",
                propertyValue = "Auto-acknowledge"
        )
})
@RunAs("guest")
public class AcquisitionUpdateMDB implements MessageListener {

    /**
     * Receive message, process, and send call back into main server update time tracker system.
     *
     * Runtime errors thrown for various pre-condition failures (wrong message type, null message, etc.) as well
     * as data validation errors.
     *
     * @param message  the message to process.
     */
    @Override
    public void onMessage(Message message) {
        if (message == null) {
            throw new IllegalArgumentException("No message received.");
        }

        if (!(message instanceof MapMessage)) {
            throw new IllegalArgumentException("AcquisitionUpdateMDB only supports MapMessage - received: " + message.getClass().getName());
        }

        final MapMessage mapMessage = ((MapMessage) message);

        final AcquisitionUpdateType acqUpdateType;
        final long updateTime;
        try {
            acqUpdateType = AcquisitionUpdateType.valueOf(mapMessage.getString("UPDATE_TYPE"));
            updateTime = mapMessage.getLong("UPDATE_TIME");
        }
        catch (Exception e) {
            throw new RuntimeException("Encountered a problem processing an Acquisition update message: " + e.getMessage());
        }

        switch (acqUpdateType) {
            case DATA_SOURCE_ADD:
            case DATA_SOURCE_UPDATE:
            case DATA_SOURCE_REMOVE:
            case ACQUISITION_UNIT_REGISTER:
            case ACQUISITION_UNIT_UPDATE:
            case ACQUISITION_UNIT_UNREGISTER:
                SummarySessionEJB.updateTimeDatasources = updateTime;
                break;
            case DATA_SOURCE_AUDIT_CREATE:
            case DATA_SOURCE_AUDIT_UPDATE:
            case DATA_SOURCE_AUDIT_CLOSE:
                SummarySessionEJB.updateTimeDatasourceAudit = updateTime;
                break;
            default:
                //ignore for now.   We do not want to do an update for other types.
                break;
        }
    }

}
