/*
 * Copyright (c) 2010 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.acquisition.sessions;

import com.netapp.oci.platform.common.table.ConnectionHelper;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.Job;

import javax.annotation.security.RunAs;
import javax.sql.DataSource;

import java.sql.Connection;
import java.sql.SQLException;


/**
 * @author Glen Nuckolls
 */
@RunAs("admin")
@DisallowConcurrentExecution
public class DSAuditsPurgeOldEntries implements Job {
    
    public static final String CRON_TRIGGER = "0 15 1 1/1 * ? *";
    public static final String JOB_NAME = "DSAuditsPurgeOldEntries";

    public void execute(final JobExecutionContext jobExecutionContext) throws JobExecutionException {
        Connection connection = null;
        try {
        	
            DataSource dataSource = (DataSource) jobExecutionContext.getJobDetail().getJobDataMap().get("dataSourceOnaroPool");
            connection = ConnectionHelper.newConnection(dataSource);

            try {

                DataSourceAuditManager.clearOldAudits(60, connection);

                // reset the audit cache
                final DataSourceAuditManager dsAuditManager = new DataSourceAuditManager();
                dsAuditManager.initialize();

            }
            catch (SQLException e) {
                throw new JobExecutionException(e);
            }

        }
        finally {
            ConnectionHelper.close(connection);
        }
    }
}
