package com.onaro.sanscreen.acquisition.sessions;

import com.onaro.sanscreen.acquisition.identify.AcqIdentifyException;

import javax.ejb.Remote;
import java.util.Map;

/**
 * @author carl.chan-aldebol
 */
@Remote
public interface IdentifySession {
    String JNDI_NAME = "ejb:mega-app/server-core//IdentifySessionEJB!com.onaro.sanscreen.acquisition.sessions.IdentifySession";

    /**
     * @param storageIdentifier - the storage identifier (could be the serial number)
     * @return The storage IP associated with the identifier. if the identifier does not exist return null.
     * @throws DataSourceException
     */
    public String getStorageIp(String storageIdentifier) throws AcqIdentifyException;

    /**
     * Set the storage ip for a given storage identifier
     *
     * @param storageIdentifier - The storage identifier (could be the serial number or WWN )
     * @param storageIp         - The storage IP associated with the identifier
     * @throws AcqIdentifyException
     */
    public void setStorageIp(String storageIdentifier, String storageIp) throws AcqIdentifyException;

    /**
     * @param storageIdentifier the storage isentifier
     * @param volumeIdentifier  the volume identifier
     * @return the volume name if does not exist return null
     */
    public String getVolumeName(String storageIdentifier, String volumeIdentifier) throws AcqIdentifyException;

    /**
     * Set the voluem name for a giving storage identifier and volume identifier
     *
     * @param storageIdentifier the storage isentifier
     * @param volumeIdentifier  the volume identifier
     * @param volumeName        the volume name
     */
    public void setVolumeName(String storageIdentifier, String volumeIdentifier, String volumeName) throws AcqIdentifyException;

    /**
     * @param storageIdentifier the storage isentifier
     * @return a Map between volume identifier and volume Name if the storage does not exist return empty HashMap;
     */
    public Map getVolumeIdentifierToName(String storageIdentifier) throws AcqIdentifyException;

    /**
     *  Set the volume identifier to name for a giving storage
     * @param storageIdentifier the storage isentifier
     * @param volumeIdentifierToName a Map between volume identifier and volume Name
     */
    public void setVolumeIdentifierToName(String storageIdentifier, Map volumeIdentifierToName) throws AcqIdentifyException;

    /**
     * Delete all the data from the tables
     */
    public void deleteAll();
}
