package com.onaro.sanscreen.server.dataobject.acquisition;


import java.io.Serializable;

/**
 * This class represents Audit message
 * <li>id= id of the message (PK)</li>
 * <li>device name = user friendly name of the device/managment console</li>
 * <li>type = type of the message e.g Error , Warn etc ...</li>
 * <li>auditId = sample audit id corrispond to auditID in ds_audit table </li>
 * <li>title = title of the message  </li>
 * <li>message = the message currently in html format. </li>
 */
public class AcquisitionAuditMessage implements Serializable {

    private final static long serialVersionUID = 1;

    private Long id;
    private String deviceName;
    private String type;
    private Long auditId;
    private String title;
    private String message;

    public Long getId() {
        return id;
    }

    public void setId(final Long id) {
        this.id = id;
    }

    public AcquisitionAuditMessage(final String deviceName, final String type, final String title, final String message) {
        this.deviceName = deviceName;
        this.type = type;
        this.title = title;
        this.message = message;
    }


    public AcquisitionAuditMessage(final Long id, final Long auditId, final String deviceName, final String type, final String title, final String message) {
        this.id = id;
        this.auditId = auditId;
        this.deviceName = deviceName;
        this.type = type;
        this.title = title;
        this.message = message;
    }

    public AcquisitionAuditMessage() {
    }

    public AcquisitionAuditMessage(final AcquisitionAuditMessage other) {
        setAuditId(other.getAuditId());
        setDeviceName(other.getDeviceName());
        setId(other.getId());
        setTitle(other.getTitle());
        setType(other.getType());
        setMessage(other.getMessage());
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(final String message) {
        this.message = message;
    }

    public String getDeviceName() {
        return deviceName;
    }

    public void setDeviceName(final String deviceName) {
        this.deviceName = deviceName;
    }

    public String getType() {
        return type;
    }

    public void setType(final String type) {
        this.type = type;
    }

    public Long getAuditId() {
        return auditId;
    }

    public void setAuditId(final Long auditId) {
        this.auditId = auditId;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(final String title) {
        this.title = title;
    }

    public String toString() {
        return "Device Key:" + deviceName
               + "\nMessage Type:" + type
               + "\nTitle:" + title
               + "\nMessage:" + message;
    }

    public String toString4CRC() {
        return deviceName + type + title + message;
    }

    public boolean equals(final Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        final AcquisitionAuditMessage that = (AcquisitionAuditMessage) o;

        if (deviceName != null ? !deviceName.equals(that.deviceName) : that.deviceName != null) return false;
        if (message != null ? !message.equals(that.message) : that.message != null) return false;
        if (title != null ? !title.equals(that.title) : that.title != null) return false;
        if (type != null ? !type.equals(that.type) : that.type != null) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = (id != null ? id.hashCode() : 0);
        result = 31 * result + (deviceName != null ? deviceName.hashCode() : 0);
        result = 31 * result + (type != null ? type.hashCode() : 0);
        result = 31 * result + (auditId != null ? auditId.hashCode() : 0);
        result = 31 * result + (title != null ? title.hashCode() : 0);
        result = 31 * result + (message != null ? message.hashCode() : 0);
        return result;
    }
}
