package com.onaro.sanscreen.server.dataobject.acquisition;

import java.io.Serializable;

/**
 * Data object for AcquisitionUnit.
 * @deprecated use AcquisitionUnitNew 
 */
public class AcquisitionUnit implements Serializable {
    private static final long serialVersionUID = 1L;
    public transient static final int CONNECTED = 1;
    public transient static final int NOT_CONNECTED = -1;
    public transient static final int WARN = 2;

    private String name = null;
    private String ip = null;
    private int status;
    private int leasePeriod;
    private long nextLeaseRenewal;

    public AcquisitionUnit() {
    }

    public AcquisitionUnit(final String name, final String ip) {
        setName(name);
        setIp(ip);
    }

    public AcquisitionUnit(final String name, final String ip, int status, int leasePeriod, long nextLeaseRenewal) {
        setName(name);
        setIp(ip);
        setStatus(status);
        setLeasePeriod(leasePeriod);
        setNextLeaseRenewal(nextLeaseRenewal);
    }

    public String getName() {
        return this.name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getIp() {
        return this.ip;
    }

    public void setIp(String ip) {
        this.ip = ip;
    }

    /**
     * The status takes into account the status of the acquisition unit (based on the data base row) and the lease
     * status
     */

    public int getStatus() {
        return status ;
    }

    public int getStatusWithLeaseCheck() {
        if (this.status != NOT_CONNECTED) {
            if (System.currentTimeMillis() > getNextLeaseRenewal()) {
                // We didn't receive a liveness report from this acquisition unit for too long
                return NOT_CONNECTED;
            }
        }

        return this.status;
    }

    public void setStatus(int status) {
        this.status = status;
    }

    public int getLeasePeriod() {
        return this.leasePeriod;
    }

    public void setLeasePeriod(int leasePeriod) {
        this.leasePeriod = leasePeriod;
    }

    public long getNextLeaseRenewal() {
        return this.nextLeaseRenewal;
    }

    public void setNextLeaseRenewal(long nextLeaseRenewal) {
        this.nextLeaseRenewal = nextLeaseRenewal;
    }

    /**
     * @return true if the acquisition unit is active (status is active and lease not expired)
     */
    public boolean isActive() {
        if (status == NOT_CONNECTED)
            return false;

        if (nextLeaseRenewal - System.currentTimeMillis() < 0)
            return false;

        return true;
    }

    public String getAcqStatus() {
        if (status == NOT_CONNECTED)
            return "Not Connected";

        if (status == WARN)
            return "WARNING";

        if (status == CONNECTED)
            return "OK";

        return "";
    }

    public String calculateDataSourceState(boolean active, int manual) {
        if (!active && manual == 2)
            return "Stopped";

        if (!isActive())
            return "Acquisition not running";

        if (active && manual == 1)
            return "Running";

        if (!active && manual == 1)
            return "Starting";

        if (active && manual == 2)
            return "Stopping";

        return "";
    }

    public String toString() {
        return "AcquisitionUnit{" +
                "name='" + name + '\'' +
                ", ip='" + ip + '\'' +
                ", status=" + status +
                '}';
    }
}
