package com.onaro.sanscreen.server.dataobject.acquisition;

import java.io.Serializable;

public abstract class DataSourceAttribute implements Comparable<DataSourceAttribute>, Serializable {
    private AttributeType type;
    private String name;
    private String description;
    private String label;
    private String role;
    private boolean editable;
    private boolean encrypt;
    private int guiorder;
    private Package.PackageName packageName;

    public enum AttributeType {
        STRING("string"),
        INTEGER("integer"),
        FLOAT("float"),
        BOOLEAN("boolean"),
        ENUM("enum"),
        LIST("list");

        private String e;

        private AttributeType(final String e) {
            this.e = e;
        }

        public static AttributeType fromString(final String s) {
            if (s == null || s.length() == 0)
                return STRING;

            for (final AttributeType type : AttributeType.values()) {
                if (type.e.equals(s))
                    return type;
            }

            throw new IllegalArgumentException("Invalid argument type '" + s + "'");
        }

        public String toString() {
            return e;
        }
    }


    public Package.PackageName getPackageName() {
        return packageName;
    }

    public void setPackageName(final Package.PackageName packageName) {
        this.packageName = packageName;
    }

    protected DataSourceAttribute(final AttributeType type,
                                  final String name,
                                  final String description,
                                  final String label,
                                  final String role,
                                  final boolean editable,
                                  final boolean encrypt,
                                  final int guiorder) {
        this.type = type;
        this.name = name;
        this.description = description;
        this.label = label;
        this.role = role;
        this.editable = editable;
        this.encrypt = encrypt;
        this.guiorder = guiorder;
    }

    // copy constructor
    DataSourceAttribute(final DataSourceAttribute original) {
        this.type = original.type;
        this.name = original.name;
        this.description = original.description;
        this.label = original.label;
        this.role = original.role;
        this.editable = original.editable;
        this.encrypt = original.encrypt;
        this.guiorder = original.guiorder;

    }

    public AttributeType getType() {
        return type;
    }

    public String getName() {
        return name;
    }

    public String getDescription() {
        return description;
    }

    public String getLabel() {
        return label;
    }

    public boolean isMandatory() {
        return role.equals("mandatory");
    }

    public boolean isHidden() {
        return role.equals("hidden");
    }

    public boolean isEditable() {
        return editable;
    }

    public boolean isEncrypt() {
        return encrypt;
    }

    public int getGuiorder() {
        return guiorder;
    }

    public int compareTo(final DataSourceAttribute otherAttribute) {
        return this.guiorder - otherAttribute.getGuiorder();
    }

    public abstract String getDefaultValueAsString();

    public abstract String getValueAsString();

    public abstract void setValueFromString(String value);

}
