/*
 * Generated by XDoclet - Do not edit!
 */
package com.onaro.sanscreen.server.dataobject.acquisition;

import com.onaro.sanscreen.acquisition.sessions.AcquisitionUnitException;

import java.io.Serializable;
import java.util.*;

/**
 * Data object for DataSource.
 */
public class DataSourceConfigurationImpl implements DataSourceConfiguration, Serializable {
    private final static long serialVersionUID = 1L;

    private long auId;
    private long dsTypeId;
    private long id;
    private String name;
    private String vendor;
    private String model;

    // used to determine which properties with the same name in different packages
    // are treated as the same property
    static private List<String> excludeFromUnifying = new ArrayList<String>();
    static private List<String> hideWhenUnifying = new ArrayList<String>();

    static {
        excludeFromUnifying.add("poll");
        excludeFromUnifying.add("enabled");
        excludeFromUnifying.add("port");

        hideWhenUnifying.add("ip");
        hideWhenUnifying.add("user");
        hideWhenUnifying.add("password");
        hideWhenUnifying.add("partner.ip");
        hideWhenUnifying.add("partner.user");
        hideWhenUnifying.add("partner.password");
    }

    final Map<String, Package> packages = new TreeMap<String, Package>();

    public DataSourceConfigurationImpl(final long dsTypeId, final String vendor, final String model, final List<DsTypeAttribute> attrs) throws AcquisitionUnitException {
        this.dsTypeId = dsTypeId;
        this.vendor = vendor;
        this.model = model;

        createPackagesFromDBProperties(attrs);
    }

    // Get attributes from database and decide which one belongs to what package
    private void createPackagesFromDBProperties(final List<DsTypeAttribute> attrs) throws AcquisitionUnitException {
        // create DataSourceAttributes from DsTypeAttributes
        for (final DsTypeAttribute attr : attrs) {

            // DSTypeAttribute name should start with the package name
            final String[] nameParts = attr.getName().split("\\.");

            // special case for list properties
            if (nameParts[1].equalsIgnoreCase("LIST")) {
                // list property format: package.LIST.listname
                // list column property format: package.LIST.listname.0.columnName
                // attributes are ordered from DB, so the list property comes before the column property
                createListAttributeFromProperties(attr, nameParts);
                continue;
            }

            // special case to handle package dependcies
            if (nameParts[1].equalsIgnoreCase("REQUIRES")) {
                // create a dependency and add to package
                final Package pkg = getOrCreatePackage(nameParts[0]);
                // get the dependency packageName enum
                final Package.PackageName dependencyName = Package.PackageName.fromName(attr.getValue());
                pkg.getDependencies().add(dependencyName);
                continue;
            }

            // all other properties
            createStandardAttributeFromDBProperty(attr, nameParts);
        }
    }

    private void createStandardAttributeFromDBProperty(final DsTypeAttribute attr, final String[] nameParts) throws AcquisitionUnitException {
        // leave off the package name for the property name
        // since the attribute is stored under its named package
        final String propertyName = buildAttributeNameFromDBPropertyString(nameParts);

        final String packageName = nameParts[0];

        // convert poll from seconds as stored in the database, to minutes for the gui
        handleFoundationAndNasPoll(attr, propertyName, packageName);

        final DataSourceAttribute dsa = createDataSourceSimpleAttributeFromDB(attr, propertyName);

        // now add to the appropriate package, creating if needed
        final Package pkg = getOrCreatePackage(packageName);
        pkg.getAttributesMapByName().put(dsa.getName(), dsa);
    }

    private void handleFoundationAndNasPoll(final DsTypeAttribute attr, final String propertyName, final String packageName) {
        if (propertyName.equals("poll") && (packageName.equals("foundation") || packageName.equals("nas") || packageName.equals("netappfoundation"))) {
            // reset the type and value: convert from minutes as a float to seconds as an integer
            final String pollStringInMinutes = convertIntegerSecondsToFloatMinutes(attr.getValue());
            attr.setValue(pollStringInMinutes);
            attr.setType(DsTypeAttribute.Type.FLOAT);
        }
    }

    /**
     * Takes a property like packageName.LIST.listName or packageName.LIST.listName.0.column.name
     * and constructs the correstponding part of a DataSourceListAttribute
     * Note: this method assumes that it will be called with parameter attr having been sorted ascending by name
     *
     * @param attr
     * @param nameParts
     *
     * @throws AcquisitionUnitException
     */
    private void createListAttributeFromProperties(final DsTypeAttribute attr, final String[] nameParts) throws AcquisitionUnitException {
        final String pkgName = nameParts[0];
        final String listName = nameParts[2];

        final Package pkg = getOrCreatePackage(pkgName);

        // list type property looks like
        //      package.LIST.listName
        // and thus has 3 parts:
        if (nameParts.length == 3) {
            // create the list attribute from the property
            // package.LIST.listName
            final List<DataSourceSimpleAttribute> columns = new ArrayList<DataSourceSimpleAttribute>();
            final DataSourceListAttribute listAttr = new DataSourceListAttribute(listName,
                                                                                 attr.getDescription(),
                                                                                 attr.getLabel(),
                                                                                 attr.getRole(),
                                                                                 attr.isEditable(),
                                                                                 attr.isEncrypt(),
                                                                                 attr.getGuiorder(),
                                                                                 columns);
            pkg.getAttributesMapByName().put(listName, listAttr);
            return;
        }
        // list type property looks like
        //      package.LIST.listName.0.columnName
        // and thus has > 4 parts:
        if (nameParts.length > 4) {
            final DataSourceListAttribute listAttr = (DataSourceListAttribute) pkg.getAttributesMapByName().get(listName);
            // get the column name
            final String columnName = buildColumnNameFromPropertyName(nameParts);
            final DataSourceSimpleAttribute columnAttribute = createDataSourceSimpleAttributeFromDB(attr, columnName);
            listAttr.getColumns().add(columnAttribute);
            return;
        }

        // property does not have the correct form or is in the wrong order
        throw new AcquisitionUnitException("LIST's property entry with parts " + nameParts.toString()
                                           + " has the wrong form appeared before list header for property : " + listName);
    }

    private static String buildAttributeNameFromDBPropertyString(final String[] nameParts) {
        return buildNameFromRemainingParts(nameParts, 1);
    }

    /**
     * Builds a string from the remainder of an array of strings
     * Starts with the index given by parameter start.
     *
     * @param nameParts
     *
     * @return
     */
    private static String buildNameFromRemainingParts(final String[] nameParts, final int start) {
        final StringBuffer sb = new StringBuffer();
        for (int i = start; i < nameParts.length; i++) {
            sb.append(nameParts[i]).append(".");
        }
        sb.deleteCharAt(sb.length() - 1);
        return sb.toString();
    }


    private String buildColumnNameFromPropertyName(final String[] nameParts) {
        return buildNameFromRemainingParts(nameParts, 4);
    }


    private Package getOrCreatePackage(final String packageString) throws AcquisitionUnitException {
        Package pkg = getPackagesMapByName().get(packageString);
        if (pkg == null) {
            pkg = new Package(packageString);
            getPackagesMapByName().put(packageString, pkg);
        }
        else {
            // package exists
        }
        return pkg;
    }

    /**
     * Create an Attribute for package from flat attribute from DB
     * @param dsta
     * @param name
     * @return
     * @throws AcquisitionUnitException
     */
    private DataSourceSimpleAttribute createDataSourceSimpleAttributeFromDB(final DsTypeAttribute dsta, final String name) throws AcquisitionUnitException {
        DataSourceSimpleAttribute result = null;

        // first determine type
        // TODO unify these two type enums!
        final DataSourceAttribute.AttributeType type = DataSourceAttribute.AttributeType.fromString(dsta.getType().toString());

        // common fields
        //final String name = dsta.getName();

        final String description = dsta.getDescription();
        final String label = dsta.getLabel();
        final String role = dsta.getRole();
        final boolean editable = dsta.isEditable();
        final boolean encrypt = dsta.isEncrypt();
        final int guiorder = dsta.getGuiorder();


        if (type.equals(DataSourceAttribute.AttributeType.BOOLEAN)) {
            final Boolean defaultValue = getSafeBoolean(dsta.getValue());
            result = new DataSourceBooleanAttribute(name, description, label, role, editable, encrypt, guiorder, defaultValue);

        }
        else if (type.equals(DataSourceAttribute.AttributeType.ENUM)) {
            final String defaultValue = dsta.getValue();
            final String enumList = dsta.getEnumsStr();
            result = new DataSourceEnumAttribute(name, description, label, role, editable, encrypt, guiorder, defaultValue, enumList);

        }
        else if (type.equals(DataSourceAttribute.AttributeType.FLOAT)) {
            final Float defaultValue = getSafeFloat(dsta.getValue());
            final Float min = getSafeFloat(dsta.getMin());
            final Float max = getSafeFloat(dsta.getMax());
            result = new DataSourceFloatAttribute(name, description, label, role, editable, encrypt, guiorder, defaultValue, min, max);
        }
        else if (type.equals(DataSourceAttribute.AttributeType.INTEGER)) {
            final Integer defaultValue = getSafeInteger(dsta.getValue());
            final Integer min = getSafeInteger(dsta.getMin());
            final Integer max = getSafeInteger(dsta.getMax());
            result = new DataSourceIntegerAttribute(name, description, label, role, editable, encrypt, guiorder, defaultValue, min, max);

        }
        else if (type.equals(DataSourceAttribute.AttributeType.STRING)) {
            final String defaultValue = dsta.getValue();
            result = new DataSourceStringAttribute(name, description, label, role, editable, encrypt, guiorder, defaultValue);

        }
        else if (type.equals(DataSourceAttribute.AttributeType.LIST)) {
            // cannot process a list type attribute
            throw new AcquisitionUnitException("Cannot create a DataSourceSimpleAttribute of Type LIST");
        }
        else {
            // unknown type
            throw new AcquisitionUnitException("Unrecognized DataSource Attribute Type: " + type.name());
        }

        return result;
    }

    private Float getSafeFloat(final String valueString) {
        Float returnValue = null;
        if (valueString != null && !valueString.trim().equals("")) {
            returnValue = Float.parseFloat(valueString);
        }
        return returnValue;
    }

    private Integer getSafeInteger(final String valueString) {
        Integer returnValue = null;
        if (valueString != null && !valueString.trim().equals("")) {
            returnValue = Integer.parseInt(valueString);
        }
        return returnValue;
    }

    private Boolean getSafeBoolean(final String valueString) {
        Boolean returnValue = null;
        if (valueString != null && !valueString.trim().equals("")) {
            returnValue = Boolean.parseBoolean(valueString);
        }
        return returnValue;
    }

    public void processUnknowProperties(final Properties properties) throws AcquisitionUnitException {
        createOrUpdateAttributesFromDBProperties(properties);
    }

    public void createOrUpdateAttributesFromDBProperties(final Properties properties) throws AcquisitionUnitException {
        for (final Map.Entry<Object, Object> property : properties.entrySet()) {

            // the property we will use to create DataSourceAttributes
            final String propertyKey = (String) property.getKey();
            final String[] parts = propertyKey.split("\\.");
            final String packageNameString = parts[0];
            // the property name is
            final String attributeName = buildAttributeNameFromDBPropertyString(parts);
            final String value = ((String) property.getValue());

            final Package.PackageName packageName = Package.PackageName.fromName(packageNameString);

            // get the package
            final Package pkg = getPackagesMapByName().get(packageName.toString());

            // Sometimes, because of sins of the past, DB contains properties from non-existing packages.
            // We decided to ignore it, because writing an upgrader is too much of an effort for that.
            if ( pkg == null )
                continue ;

            final Map<String, DataSourceAttribute> attrMap = pkg.getAttributesMapByName();
            DataSourceSimpleAttribute simpleAttrToSet = null;

            // list properties and regular properties are handled separately
            if (parts[1].equalsIgnoreCase("LIST")) {
                handleListProperty(parts, value, attrMap);
                continue;
            }

            simpleAttrToSet = ((DataSourceSimpleAttribute) attrMap.get(attributeName));
            // It can happen if user imported a property that is not defined in xml
            if (simpleAttrToSet == null) {
                simpleAttrToSet = new DataSourceStringAttribute(attributeName, null, null, "hidden", false, false, -1, null);
                pkg.getAttributesMapByName().put(attributeName,simpleAttrToSet);
            }

            // convert foundation.poll and nas.poll from seconds to minutes
            if (attributeName.equals("poll") && (packageNameString.equals("foundation") || packageNameString.equals("nas") || packageNameString.equals("netappfoundation"))) {
                final String pollStringInMinutes = convertIntegerSecondsToFloatMinutes(value);
                simpleAttrToSet.setValueFromString(pollStringInMinutes);
                continue;
            }

            simpleAttrToSet.prepareValueFromDatabase(value);
        }
    }

    private String convertIntegerSecondsToFloatMinutes(final String value) {
        final Float pollInSeconds = (float) Integer.parseInt(value);
        final Float pollInMinutes = (pollInSeconds / 60);
        return pollInMinutes.toString();
    }

    private void handleListProperty(final String[] parts, final String value, final Map<String, DataSourceAttribute> attrMap) {
        // the format for list properties is
        // packageName.LIST.listName.rowNumber.row.name
        final String listName = parts[2];
        final DataSourceListAttribute list = ((DataSourceListAttribute) attrMap.get(listName));
        final Integer rowNumber = Integer.parseInt(parts[3]);
        final String name = buildColumnNameFromPropertyName(parts);
        if (rowNumber == 0) {
            // this is a column, skip it
        }
        else {
            DataSourceListAttribute.Row row = null;
            if (list.getRows().size() < rowNumber) {
                // create new rows and add up to rowNumber -1
                // so that row is now the last one created and at the correct index
                for (int i = list.getRows().size(); i < rowNumber; i++) {
                    row = list.createRow();
                    list.getRows().add(row);
                }
            }
            else {
                // get the row by index. Row number starts at 1, rows start at 0, so adjust.
                row = list.getRows().get(rowNumber - 1);
            }
            // set the value on the right field
            row.getRowFieldsAsMapByName().get(name).prepareValueFromDatabase(value);
        }
    }

    public List<String> getExcludeFromUnifying() {
        return excludeFromUnifying;
    }

    public List<String> getHideWhenUnifying() {
        return hideWhenUnifying;
    }

    public void setAuId(final long auId) {
        this.auId = auId;
    }

    public void setDsTypeId(final long dsTypeId) {
        this.dsTypeId = dsTypeId;
    }

    public void setId(final long id) {
        this.id = id;
    }

    public void setVendor(final String vendor) {
        this.vendor = vendor;
    }

    public void setModel(final String model) {
        this.model = model;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public long getAuId() {
        return auId;
    }

    public long getDsTypeId() {
        return dsTypeId;
    }

    public long getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public String getVendor() {
        return vendor;
    }

    public String getModel() {
        return model;
    }

    public List<Package> getPackages() {
        final List<Package> pkgList = new ArrayList<Package>();
        pkgList.addAll(packages.values());
        return Collections.unmodifiableList(pkgList);
    }

    public Map<String, Package> getPackagesMapByName() {
        return packages;
    }


} 
