package com.onaro.sanscreen.server.dataobject.acquisition;

import org.slf4j.Logger;

import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

public class DataSourceEnumAttribute extends DataSourceSimpleAttribute<String> {
    private static final long serialVersionUID = 1L;
    private static final Logger logger =org.slf4j.LoggerFactory.getLogger("com.onaro.sanscreen.server.dataobject.acquisition");

    private final Set<String> enums;

    public Set<String> getEnums() {
        return enums;
    }

    public DataSourceEnumAttribute(final String name, final String description, final String label, final String role, final boolean editable, final boolean encrypt, final int guiorder, final String defaultValue, final String enumString) {
        super(AttributeType.ENUM, name, description, label, role, editable, encrypt, guiorder, defaultValue);
        enums = parseEnums(enumString);
    }

    // copy constructor
    DataSourceEnumAttribute(final DataSourceEnumAttribute original) {
        super(original);
        // copy the enums to a new set
        this.enums = new TreeSet(original.enums);
    }

    private Set<String> parseEnums(final String enumString) {
        final String[] values = enumString.split(";");
        final List<String> valueList = Arrays.asList(values);
        return (Set<String>) new TreeSet(valueList);
    }

    public void setValueFromString(final String value) throws IllegalArgumentException {
        if ( !enums.contains(value) ) {
            // we do not want to fail if we encounter an unknown enum string
            // so we set to the default
            setValue(getDefaultValue());
            logger.warn("Value " + value + "is not one of the values " + "defined for enum " + getName() + ". " +
                        "Assigning default value of " + getDefaultValue() + ". Defined values are: [" + enums.toString() + "].");
        } else {
            setValue(value);
        }
    }
}
