package com.onaro.sanscreen.server.dataobject.acquisition;

import org.slf4j.Logger;

public class DataSourceFloatAttribute extends DataSourceNumericAttribute<Float> {
    private static final long serialVersionUID = 1L;

    private static final org.slf4j.Logger logger =org.slf4j.LoggerFactory.getLogger("com.onaro.sanscreen.server.dataobject.acquisition");
    
    public DataSourceFloatAttribute(final String name, final String description, final String label, final String role, final boolean editable, final boolean encrypt, final int guiorder, final Float defaultValue, final Float min, final Float max) {
        super(AttributeType.FLOAT, name, description, label, role, editable, encrypt, guiorder, defaultValue, min, max);
    }

    DataSourceFloatAttribute(final DataSourceFloatAttribute original) {
        super(original);
    }

    public void setValueFromString(final String value) throws IllegalArgumentException {
        if (value == null || value.equals("")) {
            setValue(null);
        } else {
            try {
                final float i = Float.parseFloat(value);
                setValue(i);
            }
            catch (NumberFormatException e) {
                setValue(getDefaultValue());
                logger.warn("Value [" + value + "] is not a float value. A float value is required for property [" + getName() + "]. " +
                            "Assigning default value of " + getDefaultValue());
            }
        }
    }

    public String getValueAsString() {
        if (getValue() != null) {
            final String valueString = getValue().toString();
            return removeZeroAfterDecimal(valueString);
        } else {
            return null;
        }
    }

    public String getDefaultValueAsString() {
        if (getDefaultValue() != null) {
            final String valueString = getDefaultValue().toString();
            return removeZeroAfterDecimal(valueString);
        } else {
            return null;
        }
    }

    private String removeZeroAfterDecimal(final String valueString) {
        final int valueLength = valueString.length();
        if (valueString.endsWith(".0")) {
            return valueString.substring(0, valueLength - 2);
        }
        else {
            return valueString;
        }
    }


}
