package com.onaro.sanscreen.server.dataobject.acquisition;

import org.slf4j.Logger;

public abstract class DataSourceNumericAttribute<T extends Number> extends DataSourceSimpleAttribute<T> {

    private static final Logger logger = org.slf4j.LoggerFactory.getLogger("com.onaro.sanscreen.server.dataobject.acquisition");
    

    private T min;
    private T max;


    protected DataSourceNumericAttribute(final AttributeType type, final String name, final String description, final String label, final String role, final boolean editable, final boolean encrypt, final int guiorder, final T defaultValue, final T min, final T max) {
        super(type, name, description, label, role, editable, encrypt, guiorder, defaultValue);
        this.min = min;
        this.max = max;
    }

    // copy constructor
    DataSourceNumericAttribute(final DataSourceNumericAttribute<T> original) {
        super(original);
        this.min = original.min;
        this.max = original.max;
    }

    public T getMin() {
        return min;
    }

    public T getMax() {
        return max;
    }

    public void setValue(final T value) {
        if (value != null) {
            if ( min != null && value.floatValue() < min.floatValue() )  {
                super.setValue(min);
                logger.warn("Value " + value + " must be at least the minimum of " + getMin() + ". " +
                            "Assigning the minimum value to the property " + getName());
                return;
            } 
            if ( max != null && value.floatValue() > max.floatValue() )  {
                super.setValue(max);
                logger.warn("Value " + value + " cannot be more than the maximun of " + getMax() + ". " +
                            "Assigning the maximum value to the property " + getName());
                return;
            }
            super.setValue(value);
        }
    }
}
