/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.server.dataobject.acquisition;

public abstract class DataSourceSimpleAttribute<T> extends DataSourceAttribute {
    private static final long serialVersionUID = 1L;

    private T value;
    private T defaultValue;

    protected DataSourceSimpleAttribute(final AttributeType type,
                                        final String name,
                                        final String description,
                                        final String label,
                                        final String role,
                                        final boolean editable,
                                        final boolean encrypt,
                                        final int guiorder,
                                        final T defaultValue) {
        super(type, name, description, label, role, editable, encrypt, guiorder);
        this.defaultValue = defaultValue;
    }

    // copy constructor
    protected DataSourceSimpleAttribute(final DataSourceSimpleAttribute<T> original) {
        super(original);
        // we should probably be cloning these values, but we don't know how to clone arbitrary types
        this.value = original.value;
        this.defaultValue = original.defaultValue;
    }

    public T getValue() {
        return value;
    }

    public void setValue(final T value) {
        this.value = value;
    }

    public abstract void setValueFromString(final String value) throws IllegalArgumentException;

    public T getDefaultValue() {
        return defaultValue;
    }

    public String getValueAsString() {
        if (value != null) {
            return value.toString();
        } else {
            return null;
        }
    }

    public String prepareValueForDatabase(){
        return getValueAsString();
    }

    public String getDefaultValueAsString() {
        if (defaultValue != null) {
            return defaultValue.toString();
        } else {
            return null;
        }
    }

    public void prepareValueFromDatabase(final String value) {
        setValueFromString(value);
    }
}
