package com.onaro.sanscreen.server.dataobject.acquisition;

import com.onaro.sanscreen.acquisition.sessions.AcquisitionUnitException;

import java.util.*;
import java.io.Serializable;

public class Package implements Serializable {
    private static final long serialVersionUID = 1L;

    final PackageName name;
    final List<PackageName> dependencies = new LinkedList<PackageName>(); // list of packages this package depends on.
    final boolean isMandatory;
    final Map<String, DataSourceAttribute> attributes = new TreeMap<String, DataSourceAttribute>();

    public enum PackageName {
        FOUNDATION("foundation", "Inventory", true),
        HOST_VIRTUALIZATION("hostvirtualization", "VM Performance", false),
        NAS("nas", "NAS Inventory", false),
        PERFORMANCE("performance", "Switch Performance", false),
        RPO("rpo", "Replication", false),
        STORAGE_PERFORMANCE("storageperformance", "Array Performance", false),
        //todo: fix this
        //TODO:Remove me
        NER("nerfoundation", "Reporting", false),
        NETAPP_FOUNDATION("netappfoundation","netappfoundation",false),
        NETAPP_STORAGE_PERFORMANCE("netappstorageperformance","netappstorageperformance",false),
        CLOUD("cloud","Cloud",false);

        private static PackageName[] validEnums = {FOUNDATION,HOST_VIRTUALIZATION,NAS,PERFORMANCE,RPO,STORAGE_PERFORMANCE,NER,NETAPP_FOUNDATION,NETAPP_STORAGE_PERFORMANCE, CLOUD};

        private final String name;
        private final String displayName;
        private final boolean isMandatory;
        
        PackageName(final String name, final String displayName, final boolean isMandatory){
            this.name = name;
            this.displayName = displayName;
            this.isMandatory = isMandatory;
        }

        public static PackageName fromName(final String name) throws AcquisitionUnitException {
            for (final PackageName pkg : validEnums) {
                if (pkg.name.equals(name)) {
                    return pkg;
                }
            }
            throw new AcquisitionUnitException("Package name " + name + " is not a valid acquisition package.");
        }

        public String getDisplayName() {
            return displayName;
        }

        public boolean isMandatory() {
            return isMandatory;
        }

        public String toString() {
            return name;
        }
    }

    public Package(final PackageName name) {
        this.name = name;
        this.isMandatory = name.isMandatory();
        // TODO: When/if dependencies are defined, decide if it affects isMandatory.
    }

    public Package(final String name) throws AcquisitionUnitException {
        this (PackageName.fromName(name));
    }

    public PackageName getName() {
        return name;
    }

    public String getDisplayName() {
        return name.getDisplayName();
    }

    public boolean isMandatory() {
        return isMandatory;
    }

    public List<PackageName> getDependencies() {
        return dependencies;
    }

    public List<DataSourceAttribute> getAttributes() {
        final List<DataSourceAttribute> attributeList = new ArrayList<DataSourceAttribute>();
        attributeList.addAll(attributes.values());

        // Sort the list. DataSourceAttribute implements Comparable 
        // by comparing guiorder.
        Collections.sort(attributeList);
        return Collections.unmodifiableList(attributeList);
    }

    public Map<String, DataSourceAttribute> getAttributesMapByName() {
        return attributes;
    }

}
