package com.onaro.sanscreen.server.interfaces.data;

import com.netapp.oci.platform.common.ServerConstant;

import java.util.Date;
import java.io.Serializable;

/**
 * Context of the request to server interfaces
 *
 * @author Anton Sergeev
 */
public abstract class Context implements Serializable {
    /**
     * Id of the context
     */
    private long id;

    /**
     * The time context was created
     */
    private Date createTime;

    /**
     * Context type: Task, Current, Time, History or what not
     */
    private ContextType type;

    /**
     * Id of the user that created context
     */
    private String userId;

    /**
     * the client IP of this Context
     */
    private String userIp;

    public Context(long id, Date createTime, ContextType type, String userId) {
        this.id = id;
        this.createTime = createTime;
        this.type = type;
        this.userId = userId;
        this.userIp = getClientIpAddress();
    }

    public long getId() {
        return id;
    }

    public Date getCreateTime() {
        return createTime;
    }

    public ContextType getType() {
        return type;
    }

    public String getUserId() {
        return userId;
    }

    public String getUserIp() {
        return userIp;
    }

    public void setUserIp(String userIp) {
        this.userIp = userIp;
    }

    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (o == null || getClass() != o.getClass())
            return false;

        Context context = (Context) o;

        if (id != context.id)
            return false;
        if (!createTime.equals(context.createTime))
            return false;
        if (type != context.type)
            return false;
        if (userId != null ? !userId.equals(context.userId) : context.userId != null)
            return false;
        if (userIp != null ? !userIp.equals(context.userIp) : context.userIp != null)
            return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = (int) (id ^ (id >>> 32));
        result = 31 * result + createTime.hashCode();
        result = 31 * result + type.hashCode();
        result = 31 * result + (userId != null ? userId.hashCode() : 0);
        result = 31 * result + (userIp != null ? userIp.hashCode() : 0);
        return result;
    }

    public static long getTime(Context context) throws UnsupportedOperationException {
        if (context.getType() == ContextType.TIME) {
            return ((TimeContext) context).getTime().getTime();
        } else if (context.getType() == ContextType.CURRENT) {
            return ServerConstant.END_DATE - 1;
        } else {
            throw new UnsupportedOperationException(" context does not support get time. " + context.getClass());
        }
    }

    private static String getClientIpAddress() {
        String currentThreadName = Thread.currentThread().getName();
        int begin = currentThreadName.indexOf('[') + 1;
        if (begin > 0) {
            int end = currentThreadName.indexOf(']') - 1;
            String[] ipAddress = currentThreadName.substring(begin, end).split(":");
            return ipAddress[0];
        } else {
            return currentThreadName;
        }

    }


}
