package com.onaro.sanscreen.server.interfaces.data;

import javax.persistence.*;
import java.io.Serializable;

/**
 * Base for all interface objects (id based objects)
 *
 * @author Anton Sergeev
 */
@Entity
@Inheritance(strategy = InheritanceType.TABLE_PER_CLASS)
public abstract class DataObject implements Serializable {
    private static final long serialVersionUID = 1L;

    /** Object id */
    @Id
    @Column(name = "objid")
    private Long id;

    public DataObject(long id) {
        this.id = Long.valueOf(id);
    }

    protected DataObject(Long id) {
        this.id = id;
    }

    public DataObject() {}

    public Long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = Long.valueOf(id);
    }

    public void setId(Long id) {
        this.id = id;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + (int) (id.longValue() ^ (id.longValue() >>> 32));
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null) return false;
        if (getClass() != obj.getClass()) return false;
        DataObject other = (DataObject) obj;
        if (id.longValue() != other.id.longValue()) return false;
        return true;
    }

    @Override
    public String toString() {
        return "DataObject{" +
                "id=" + id +
                '}';
    }
}
