package com.onaro.sanscreen.server.interfaces.data;

import com.netapp.oci.platform.common.util.Assert;

import java.io.Serializable;


/**
 * Useful reference mechanism to use in server interfaces when ids of multiple classes are mixed in the return.
 * This object is immutable.  
 *
 * @author Anton Sergeev
 */
public class DataObjectReference implements Serializable {
    private static final long serialVersionUID = 1L;

    /** Object id */
    private final Long id;

    /** Data object class */
    private final Class<? extends DataObject> dataObjectClass;

    public DataObjectReference(Long id, Class<? extends DataObject> dataObjectClass) {
        Assert.isNotNull(id);
        Assert.isNotNull(dataObjectClass);

        this.id = id;
        this.dataObjectClass = dataObjectClass;
    }

    public Long getId() {
        return id;
    }

    public Class<? extends DataObject> getDataObjectClass() {
        return dataObjectClass;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof DataObjectReference)) return false;

        DataObjectReference that = (DataObjectReference) o;

        if (id != null ? !id.equals(that.id) : that.id != null) return false;
        if (dataObjectClass != null ? !dataObjectClass.equals(that.dataObjectClass) : that.dataObjectClass != null)
            return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = id != null ? id.hashCode() : 0;
        result = 31 * result + (dataObjectClass != null ? dataObjectClass.hashCode() : 0);
        return result;
    }

    @Override
    public String toString() {
        return "DataObjectReference[id=" + id + ", dataObjectClass=" + dataObjectClass + "]";
    }
}
