package com.onaro.sanscreen.server.interfaces.impl;

import com.netapp.oci.platform.common.ServerConstant;
import com.netapp.oci.platform.job.JobSchedulerBean;
import com.onaro.sanscreen.server.interfaces.remote.SchedulerManagement;

import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.impl.StdSchedulerFactory;
import org.quartz.impl.matchers.GroupMatcher;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

/**
 * Manages quartz scheduler
 * <p/>
 * Manages 'JBossQuartzScheduler'. JBossQuartzScheduler is used by JBoss and all the OCI jobs are managed by this scheduler.
 * If you try to access the scheduler out of JBoss JVM, for example while running unit tests, you won't be able to access to
 * this scheduler. So, this ejb needs to be used in order to access the scheduler.
 * JBossQuartzScheduler is shipped with JBoss and configured in
 *    quartz-ra.rar\quartz-ra.jar\org\jboss\resource\adapter\quartz\inflow\quartz.properties.
 * quartz-ra.rar is located in the deploy folder of JBoss.
 *
 * @author Onur Keskin
 */
@Stateless
@RolesAllowed({"guest", "user", "admin"})
public class SchedulerManagementBean implements SchedulerManagement {

    public static final String DEFAULT_SCHEDULER_NAME = "JBossQuartzScheduler";
    
    private Scheduler scheduler;
    
    public SchedulerManagementBean() {
    	try {
    	    scheduler = new StdSchedulerFactory().getScheduler();
    	} catch (SchedulerException e) {
    	    throw new RuntimeException(e);
    	}
    }

    public void pauseScheduler() throws SchedulerException {
    	scheduler.pauseAll();
    }

    public void resumeScheduler() throws SchedulerException {
    	scheduler.resumeAll();
    }

    public void shutdown(boolean waitForJobsToComplete) throws Exception {
    	scheduler.shutdown(waitForJobsToComplete);
    }

    public void triggerJob(String jobName, String groupName) throws SchedulerException {
    	scheduler.triggerJob(new JobKey(jobName, groupName));
    }

    public Collection<String> getJobGroups() throws SchedulerException {
    	return scheduler.getJobGroupNames();
    }

    public Collection<String> getJobs(String group) throws SchedulerException {
    	Set<JobKey> jobKeys = scheduler.getJobKeys(GroupMatcher.jobGroupEquals(JobSchedulerBean.QRTZ_JOB_GROUP));
    	List<String> jobNames = new ArrayList<String>();
    	for (JobKey key : jobKeys) {
    	    jobNames.add(key.getName());
    	}
    	return jobNames;
    }
}
