package com.onaro.sanscreen.server.interfaces.impl.viewprofile;

import com.netapp.oci.platform.common.ServerConstant;
import com.onaro.sanscreen.server.interfaces.data.viewprofile.ViewProfile;
import com.onaro.sanscreen.server.interfaces.remote.ViewProfileRemote;
import com.onaro.sanscreen.server.sessions.UpdateId;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.List;

/**
 * Date: Aug 20, 2007
 * Time: 5:57:45 PM
 *
 * @author yarom.gabay
 */
@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@RolesAllowed({"guest", "user", "admin", "scheduler"})
public class ViewProfileBean implements ViewProfileRemote {

    private static long updateTime = UpdateId.get();

    private static String OBJECT_NAME = ViewProfile.class.getName();

    @PersistenceContext(unitName = "onaro_persistence")
    private EntityManager entityManager;

    public long createViewProfile(ViewProfile viewProfile) {
        entityManager.persist(viewProfile);
        Long newId = viewProfile.getId();
        incrementUpdateId();
        return newId.longValue();
    }


    public void removeViewProfile(long id) {
        ViewProfile viewProfile = getViewProfileById(id);
        if (viewProfile != null) {
            entityManager.merge(viewProfile);
            entityManager.remove(viewProfile);
            incrementUpdateId();
        }
    }

    public void updateViewProfile(ViewProfile viewProfile) {
        entityManager.merge(viewProfile);
        incrementUpdateId();
    }

    public ViewProfile getViewProfileById(long id) {
        return entityManager.find(ViewProfile.class, Long.valueOf(id));
    }


    @SuppressWarnings("unchecked")
    public List<ViewProfile> getAllViewProfiles() {
        return entityManager.createQuery("FROM " + OBJECT_NAME).getResultList();
    }

    @SuppressWarnings("unchecked")
    public List<ViewProfile> getGlobalViewProfiles() {
        return entityManager.createQuery("SELECT vp FROM " + OBJECT_NAME + " vp WHERE vp.global=true").getResultList();
    }


    @SuppressWarnings("unchecked")
    public List<ViewProfile> getViewProfilesByOwnerNonGlobal(String owner) {
        Query query = entityManager.createQuery("SELECT vp FROM " + OBJECT_NAME + " vp WHERE vp.global=false AND vp.owner=?1");
        query.setParameter(1, owner);
        return query.getResultList();
    }


    public Long getViewProfileUpdateTime() {
        return new Long(getUpdateId());
    }


    /**
     * updates the updateId to be now. Apply when there are changes to viewProfiles.
     */
    private static synchronized void incrementUpdateId() {
        updateTime = UpdateId.get();
    }

    /**
     * @return the current updateId (or updateTime)
     */
    private static synchronized long getUpdateId() {
        return updateTime;
    }
}
