package com.onaro.sanscreen.server.interfaces.remote;

import org.quartz.SchedulerException;

import javax.ejb.Remote;
import java.util.Collection;

/**
 * This interface is used primarily by unit tests that need
 * scheduler to pause activity while certain shceduled job is
 * being tested.
 *
 * @author Anton Sergeev
 */
@Remote
public interface SchedulerManagement {
    String JNDI_NAME = "ejb:mega-app/server-core//SchedulerManagementBean!com.onaro.sanscreen.server.interfaces.remote.SchedulerManagement";

    /**
     * Pauses work of scheduler
     *
     * @throws SchedulerException if failed to pause
     */
    void pauseScheduler() throws SchedulerException;

    /**
     * Resumes work of scheduler
     *
     * @throws SchedulerException if failed to resume scheduler work
     */
    void resumeScheduler() throws SchedulerException;

    /**
     * Halts the Scheduler's firing of Triggers, and cleans up all resources associated with the Scheduler.
     *
     * @param  waitForJobsToComplete if true the scheduler will not allow this method to return until all
     *                               currently executing jobs have completed.
     * @throws SchedulerException if failed to resume scheduler work
     */
    void shutdown(boolean waitForJobsToComplete) throws Exception;

    /**
     * Triggers the identified JobDetail  (execute it now) - the generated trigger will be volatile.
     *
     * @param jobName the name of the Job to trigger
     * @param groupName the group name of the Job to trigger
     * @throws SchedulerException
     */
    void triggerJob(String jobName, String groupName) throws SchedulerException;


    /**
     * Finds the job groups for the default scheduler
     * @return A collection of all the job groups in the scheduler
     * @throws SchedulerException if fails to access the scheduler
     */
    Collection<String> getJobGroups() throws SchedulerException;

    /**
     * Finds all the jobs for a given group
     * @param group the group name to look up for
     * @return A collection of all the jobs for the group
     * @throws SchedulerException if fails to access the scheduler
     */
    Collection<String> getJobs(String group) throws SchedulerException;
}
