/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.server.originatorupdate.xml;

import com.netapp.oci.platform.common.table.StringCache;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import java.util.HashMap;
import java.util.Map;

/**
 * Every XML element holds the element name and
 * the element properties (attributes)
 * 
 * @author Anton Sergeev
 */
public class Element {
    public String tagName;
    public Map<String, String> prop;

    public Element(String sTagName, Attributes attributes) throws SAXException {
        tagName = sTagName;
        if (tagName == null) {
            throw new SAXException("No object type found for " + sTagName);
        }

        int size = attributes.getLength();
        prop = new HashMap<String, String>();
        for (int i = 0; i < size; i++) {
            String name = attributes.getLocalName(i);

            // Sometimes leading whitespace is significant, we should NOT trim strings,
            // we should always trust Acquisition
            String val = attributes.getValue(i);
            prop.put(StringCache.get(name), StringCache.get(val));
        }
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Element)) return false;

        final Element element = (Element) o;

        return !(tagName != null ? !tagName.equals(element.tagName) : element.tagName != null) && !(prop != null ? !prop.equals(element.prop) : element.prop != null);
    }

    public int hashCode() {
        int result;
        result = (tagName != null ? tagName.hashCode() : 0);
        result = 29 * result + (prop != null ? prop.hashCode() : 0);
        return result;
    }

    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("</").append(tagName);
        for (Map.Entry<String, String> entry : prop.entrySet()) {
            String key = entry.getKey();
            String val = entry.getValue();
            sb.append(" ").append(key).append("=\"").append(val).append("\"");
        }
        sb.append(">");
        return sb.toString();
    }
}
