package com.onaro.sanscreen.server.sessions;

import com.netapp.acquisition.aquisitionserverside.interfaces.data.AcquisitionServerException;
import com.netapp.acquisition.aquisitionserverside.interfaces.data.DataSourceDoesNotExistException;
import com.netapp.acquisition.aquisitionserverside.interfaces.remote.AcquisitionServerSession;
import com.netapp.oci.platform.common.ServerConstant;
import com.onaro.sanscreen.acquisition.sessions.AcquisitionUnitException;
import com.onaro.sanscreen.acquisition.sessions.DataSourceException;

import javax.annotation.security.RolesAllowed;
import javax.annotation.security.RunAs;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@RolesAllowed({"user", "admin", "scheduler", "acquisition"})
@RunAs("acquisition")
public class ForceAcqEJB implements ForceAcqSession, ForceAcqSessionLocal {
    @EJB
    private AcquisitionServerSession acquisitionServerSession;

    public void forceAcquisition(final long[] dsIds ) throws AcquisitionUnitException, DataSourceException {
      forceAcquisition( dsIds, null );
    }

    public void forceAcquisition(final long[] dsIds, final Properties properties ) throws AcquisitionUnitException, DataSourceException {

        List<Long> missingIds = new ArrayList<Long>();
        try {
            for (final long dsId : dsIds) {
                try {
                    forceAcquisition(dsId, properties);
                }
                catch (DataSourceDoesNotExistException ddnee) {
                    missingIds.addAll(ddnee.getMissingIds());
                }
            }

        }
        catch (AcquisitionServerException e) {
            throw new AcquisitionUnitException("Failed to force acq", e);
        }

        if (!missingIds.isEmpty()) {
            throw new AcquisitionUnitException(DataSourceDoesNotExistException.buildString(missingIds));
        }

    }

    public void forceCrawl(final long[] dsIds) throws AcquisitionUnitException, DataSourceException {

        List<Long> missingIds = new ArrayList<Long>();
        try {
            for (final long dsId : dsIds) {
                try {
                    forceCrawl(dsId);
                }
                catch (DataSourceDoesNotExistException ddnee) {
                    missingIds.addAll(ddnee.getMissingIds());
                }
            }

        }
        catch (AcquisitionServerException e) {
            throw new AcquisitionUnitException("Failed to force crawl", e);
        }

        if (!missingIds.isEmpty()) {
            throw new AcquisitionUnitException(DataSourceDoesNotExistException.buildString(missingIds));
        }

    }

    protected void forceAcquisition(final long dsId, final Properties properties) throws AcquisitionUnitException, DataSourceException, AcquisitionServerException {
        acquisitionServerSession.forceAcquisition(dsId, "", properties );
    }

    protected void forceCrawl(long dsId) throws AcquisitionUnitException, DataSourceException, AcquisitionServerException {
        acquisitionServerSession.forceCrawl(dsId, "foundation");
    }
}
