/*
 * Copyright (c) 2009 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.server.sessions;

import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.server.UpdateTaskException;
import com.onaro.commons.table.DbConfig;
import org.slf4j.Logger;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.annotation.security.RunAs;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.sql.DataSource;
import java.sql.Connection;

/**
 * @author Al Velella
 *         Date: Aug 13, 2009
 *         Time: 2:22:48 PM
 */
@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
@RolesAllowed({"guest", "acquisition", "admin"})
@RunAs("guest")
public class ImplicitCommitCheckBean implements ImplicitCommitCheckRemote {
    @Resource(name = "jdbc/OnaroDataSource",
            mappedName = "java:/onaro_pool",
            type = DataSource.class,
            authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    static protected transient final org.slf4j.Logger logger =org.slf4j.LoggerFactory.getLogger("com.onaro.sanscreen.server");

    public void assertImplicitCommitCheck(long time, boolean throwException) throws UpdateTaskException {
        Connection otherConnection = null;
        try {
            otherConnection = ConnectionHelper.newConnection(dataSource);
            String sql = "SELECT starttime FROM " + DbConfig.getSanscreenDbName() + ".implicit_commit";
            long timeFromOtherConnection = PreparedStatementWrapper.executeFunctionQuery(otherConnection, sql, 0);
            if (timeFromOtherConnection == time) {
                logger.error("Detected implicit commit: " + timeFromOtherConnection + "/" + time);
                if (throwException) {
                    throw new UpdateTaskException("Detected implicit commit: " + timeFromOtherConnection);
                }
            }
        } catch (TableException e) {
            logger.error("Error checking implicit commit", e);
            if (throwException) {
                throw new UpdateTaskException("Error checking implicit commit", e);
            }
        } finally {
            ConnectionHelper.close(otherConnection);
        }
    }
}
