package com.onaro.sanscreen.server.sessions;

import com.netapp.oci.platform.common.param.ParamException;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import javax.annotation.security.RunAs;

/**
 * Triggers every day at 7AM
 *
 * @author carl.chan-aldebol
 */
@RunAs("admin")
@DisallowConcurrentExecution
public class LogSchedulerJob implements Job {
    
    public static final String CRON_TRIGGER = "0 0 7 * * ?";
    public static final String JOB_NAME = "LogSchedulerJob";
    
    private static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(LogSchedulerJob.class);

    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        try {
            LogSchedulerSession logSchedulerSession = (LogSchedulerSession) jobExecutionContext.getJobDetail().getJobDataMap().get("logSchedulerBean");
            logSchedulerSession.checkLogs(new Long(System.currentTimeMillis()));
        } catch (ParamException pe) {
            logger.error("Param Exception", pe);
            throw new JobExecutionException(pe);
        }
    }
}