/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.server.sessions;

import com.netapp.oci.platform.common.param.ParamException;
import com.onaro.commons.types.Period;
import org.slf4j.Logger;

import javax.ejb.Stateless;
import java.io.File;
import java.io.FilenameFilter;

/**
 * @author Nancy Walia
 */
@Stateless
public class LogsSchedulerSessionEJB implements LogSchedulerSession {
    static transient final org.slf4j.Logger logger =org.slf4j.LoggerFactory.getLogger("com.onaro.sanscreen.server.sessions");

    private String onaroHome = System.getProperty("onaro.home", "c:/program files/sanscreen");
    private static final int MB_TO_BYTES = 1024 * 1024;
    private static final int DELETE_AFTER_DAY = 1; //delete everything after 1 day.

    /**
     * This method will be called at each time tick to check if any acquisition unit should be removed.
     */
    public void checkLogs(Long time) throws ParamException {
        long maxSizeBytes = 1024L * MB_TO_BYTES;
        int historyDays = 3;

        if (logger.isDebugEnabled())
            logger.debug("Checking the logs directory...");

        File logsdir = new File(onaroHome + "/log");
        if (!logsdir.exists()) {
            logger.debug("The logs direcrory does not exist!");
            return;
        }

        final File[] zipFiles = logsdir.listFiles(new LogFilenameFilter());
        if ((zipFiles == null) || (zipFiles.length == 0)) {
            logger.debug("No files found in the logs directory");
            return;
        }

        // Calculate current size consumed by the log files
        long totalSize = 0;


        for (File zipFile : zipFiles) {
            totalSize += zipFile.length();
        }

        long thresholdTime;
        int deleteCount = 0;
        if (totalSize > maxSizeBytes) {
            thresholdTime = Period.DAY.subtract(System.currentTimeMillis(), DELETE_AFTER_DAY);
        } else {
            thresholdTime = Period.DAY.subtract(System.currentTimeMillis(), historyDays);
        }

        for (File zipFile : zipFiles) {
            long lastModifiedTime = zipFile.lastModified();
            if (thresholdTime > lastModifiedTime) {
                zipFile.delete();
                if (logger.isDebugEnabled())
                    logger.debug("Deleting file '" + zipFile.getName() + "'");
                deleteCount++;
            }
        }

        if (logger.isDebugEnabled())
            logger.debug("Finished checking log directory (deleted " + deleteCount + " files)");
    }

    public static void main(String[] args) throws ParamException {
        LogsSchedulerSessionEJB lse = new LogsSchedulerSessionEJB();
        lse.checkLogs(null);
    }

    /**
     * Filter that returns only zip files
     */
    private static class LogFilenameFilter implements FilenameFilter {
        public boolean accept(File dir, String name) {
            return (!name.endsWith(".log"));
        }
    }
}

