/*
 * Copyright (c) 2010 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.server.sessions;

import com.netapp.oci.server.UpdateTaskException;
import com.onaro.sanscreen.server.originatorupdate.OriginatorCacheChanges;
import com.onaro.sanscreen.util.Schema;
import com.onaro.sanscreen.util.xml.XMLException;

import javax.ejb.Remote;

/**
 * @author carl.chan-aldebol
 */
@Remote
public interface OriginatorUpdateSession {
    String JNDI_NAME = "ejb:mega-app/server-core//OriginatorUpdateSessionEJB!com.onaro.sanscreen.server.sessions.OriginatorUpdateSession";

    /**
     * Send update to server
     * @param xml xml data
     * @param schema schema type
     * @return list of changes
     * @throws UpdateTaskException if failed to execute
     */
    OriginatorCacheChanges getUpdate(String xml, Schema schema) throws UpdateTaskException;

    /**
     * Remove all data for the originator
     * @param originId originator ID
     * @return list of changes
     * @throws UpdateTaskException if failed to execute
     */
    OriginatorCacheChanges removeOriginUpdate(Long originId) throws UpdateTaskException;

    /**
     * Validates xml against schema
     * @param xml xml data
     * @param schema schema type
     * @throws XMLException if failed to execute
     */
    void validate(String xml, Schema schema) throws XMLException;

    /**
     * Clear all originator caches
     * @throws UpdateTaskException if failed to execute
     */
    void resetAllCache() throws UpdateTaskException;
}
