package com.onaro.sanscreen.server.sessions;

import com.netapp.acquisition.aquisitionserverside.interfaces.data.DataSourceDoesNotExistException;
import com.netapp.acquisition.aquisitionserverside.interfaces.remote.AcquisitionServerSession;
import com.netapp.oci.netapp.performance.shared.interfaces.impl.PerformanceSummaryJobStart;

import com.onaro.sanscreen.acquisition.sessions.AcquisitionUnitException;
import com.onaro.sanscreen.acquisition.sessions.DataSourceException;

import javax.annotation.security.RolesAllowed;
import javax.annotation.security.RunAs;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import java.util.*;

@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@RolesAllowed({"user", "admin", "scheduler", "acquisition"})
@RunAs("acquisition")
public class PerformanceSummaryEJB implements PerformanceSummarySession
{
  private static final org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(PerformanceSummaryEJB.class);

  @EJB
  private AcquisitionServerSession acquisitionServerSession;

  public void execute(final long[] dsIds, final Properties properties) throws AcquisitionUnitException, DataSourceException
  {
    logger.info( "Executing Performance Summary for " + Arrays.toString( dsIds ) );

    List<Long> missingIds = new ArrayList<>();
    try
    {
      for( final long dsId : dsIds )
      {
        try
        {
          String timeStr = properties.getProperty( "time" );
          Date time;
          if( timeStr == null )
            time = new Date();
          else
            time = new Date( Long.valueOf( timeStr ) );
          new PerformanceSummaryJobStart( dsId, time.getTime(), true ).execute();
        }
        catch( Throwable t )
        {
          logger.error( "Error Performing summary for " + dsId, t );
        }
      }
    }
    catch( Throwable e )
    {
      throw new AcquisitionUnitException( "Failed to execute performance summary.", e );
    }
    finally
    {
      logger.info( "Executing Performance Summary for " + Arrays.toString( dsIds ) );
    }

    if( !missingIds.isEmpty() )
    {
      throw new AcquisitionUnitException( DataSourceDoesNotExistException.buildString( missingIds ) );
    }
  }
}
