/*
 * Copyright (c) 2010 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.server.sessions;

import com.netapp.oci.platform.common.ServerConstant;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableResultParserDefault;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.sql.DataSource;
import java.sql.Connection;
import java.util.*;

@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@RolesAllowed({"guest", "user", "admin", "scheduler", "acquisition"})
public class SummarySessionEJB implements SummarySession {

    public static long updateTimeDatasources = 0L;
    public static long updateTimeDatasourceAudit = 0L;

    @Resource(name = "jdbc/OnaroDataSource",
            mappedName = "java:/onaro_pool",
            type = DataSource.class,
            authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    /**
     * Gets the version of the server's code. The version is read from the file
     * "About.properties" which should be in the root of the configuration jar.
     */
    @Override
    public String getCodeVersion() throws SummaryException {
        return getAboutResource("version");
    }

    /**
     * Gets the build-number of the server's code. The build number is read from the file
     * "About.properties" which should be in the root of the configuration jar.
     */
    @Override
    public String getCodeBuildNumber() throws SummaryException {
        return getAboutResource("build");
    }

    /**
     * Gets the current site name as specified by the "phone home" settings. If the name was never set, it'll return
     * the name of the server computer.<br>
     *
     * @return the current site name
     */
    @Override
    public String getSiteName() throws SummaryException {
        return "MEG Acquisition";
    }

    /**
     * Gets the current site name as specified by the "phone home" settings. If the name was never set, it'll return
     * the name of the server computer.<br>
     *
     * @return the current site name
     */
    @Override
    public String getDatabaseVersion() throws SummaryException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            return PreparedStatementWrapper.create(connection,
                    "SELECT CONCAT(major,'.',minor,'.',maintenance,'.',patch) FROM netapp_model.version",
                    TableResultParserDefault.STRING).executeQueryEx().iterator().next();
        } catch (Exception e) {
            throw new SummaryException("Failed to get version", e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }

    /**
     * Gets current time
     *
     * @return the current time
     */
    @Override
    public long getCurrentTime() {
        return System.currentTimeMillis();
    }

    private String getAboutResource(final String key) throws SummaryException {
        try {
            final ResourceBundle bundle = ResourceBundle.getBundle("About");
            return bundle.getString(key);
        } catch (MissingResourceException e) {
            throw new SummaryException("Couldn't find '" + key + "' in 'About.properties'", e);
        }
    }

}
