package com.onaro.sanscreen.server.startup;

import java.lang.management.ManagementFactory;
import java.sql.Connection;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.sql.DataSource;

import com.onaro.sanscreen.acquisition.sessions.AcquisitionFacadeSessionLocal;
import com.onaro.sanscreen.server.tables.idgenerator.sessions.IdGeneratorSession;
import org.jboss.ejb3.annotation.SecurityDomain;
import org.jboss.security.RunAsIdentity;
import org.jboss.security.SecurityContextAssociation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.netapp.oci.netapp.startup.NetAppPlugin;
import com.netapp.oci.netapp.upgrade.NetAppUpgradeRegistry;
import com.netapp.oci.platform.common.InternalServerException;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.common.table.DBPrefixHelper;
import com.netapp.oci.platform.plugin.remote.PlatformPluginRemote;
import com.netapp.oci.platform.upgrade.UpgradeRunner;
import com.netapp.oci.profiler.ProfilerService;
import com.netapp.oci.profiler.ProfilerServiceFactory;

/**
 * @author Nancy Walia
 *         Date: Dec 16, 2009
 *         This MBean is used to run methods after jboss starts.
 */
@TransactionManagement(TransactionManagementType.BEAN)
@SecurityDomain("onaro-login")
@SuppressWarnings({"UnusedDeclaration"})
@Startup
@Singleton
public class StartUpServiceImpl implements StartUpServiceImplMBean {

    private static final Logger logger = LoggerFactory.getLogger("com.onaro.sanscreen.server.startup");

    @Resource(name = "jdbc/OnaroDataSource", mappedName = "java:/onaro_pool", type = javax.sql.DataSource.class, authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    @EJB 
    private AcquisitionFacadeSessionLocal acquisitionLocal;

    @EJB
    private PlatformPluginRemote platformPluginRemote;

    @EJB
    private IdGeneratorSession idGeneratorSession; // hack to preload id generator session, cause policy local will use it behind scenes

    @PostConstruct
    public void registerInJMX() {
        try {
            ObjectName objectName = new ObjectName("sampleJMX:service=ociServer");
            MBeanServer platformMBeanServer = ManagementFactory.getPlatformMBeanServer();
            platformMBeanServer.registerMBean(this, objectName);
	    start();
        } catch (Exception e) {
            throw new IllegalStateException("Problem during registration of StartupServiceMbean into JMX:" + e);
        }
    }

    /**
     * This method should not throw any exceptions.
     */
    @Override
    public void start() {

        try {
            // annotation RunAs did not work for some reason
            SecurityContextAssociation.getSecurityContext().setOutgoingRunAs(new RunAsIdentity("admin", "admin"));

            logger.info("Deploying Acquisition Datasource meta-data");
            acquisitionLocal.deploy(false);

            Connection connection = null;
            try {
                connection = ConnectionHelper.newConnection(dataSource);

                new DBPrefixHelper().handleDbPrefix(connection);

                logger.info("Checking OCIE database");
                try {
                    new UpgradeRunner(NetAppUpgradeRegistry.INSTANCE, connection).run();
                } catch (InternalServerException e) {
                    logger.error("Failed to check database", e);
                }

                startProfilerService();

                logger.info("Aquisition initializing");
                platformPluginRemote.register(new NetAppPlugin());
            } finally {
                ConnectionHelper.close(connection);
            }

        } catch (Throwable ex) {
            logger.error("Exception during server startup", ex);
        }
    }

    private void startProfilerService() {
        final ProfilerService profilerService = ProfilerServiceFactory.getInstance();
        if (profilerService.isEnabled()) {
            profilerService.start();
            Runtime.getRuntime().addShutdownHook(new Thread() {
                @Override
                public void run() {
                    profilerService.stop();
                }
            });
        }
    }
}
