package com.onaro.sanscreen.server.tables.acquisition;

import com.netapp.oci.platform.common.table.Table;
import com.netapp.oci.platform.common.table.TableException;
import com.onaro.commons.table.DbConfig;

import java.sql.Connection;
import java.util.List;

/**
 * Base class for the acquisition units.
 * This class contains general queries that are common to the acquisition table classes.
 */
abstract public class AcquisitionTable<T> extends Table<T> {
    protected AcquisitionTable(Connection connection, String name) {
        super(connection, DbConfig.getAcquisitionDbName(), name);
    }

    long getAutoGeneratedId() throws TableException {
        return getPrepareStatementEx("SELECT LAST_INSERT_ID()").executeFunctionQueryEx();
    }

    public List<T> getAll() throws TableException {
        return getPrepareStatementEx("SELECT * FROM " + getFullName()).executeQueryEx();
    }

    /**
     * @param id
     * @return the object
     * @throws TableException if the object is null or database error
     */
    public T get(long id) throws TableException {
        T object = find(id);
        if (object == null)
            throw new TableException("Failed to find id:" + id);
        
        return object;
    }

    /**
     * @param id
     * @return null if object not found
     * @throws TableException if there is database error
     */
    public T find(long id) throws TableException {
        List<T> rows = getPrepareStatementEx("SELECT * FROM " + getFullName() + " WHERE id=?").setNext(id).executeQueryEx();
        if (rows.size() == 0)
            return null;

        return rows.iterator().next();
    }

    @Override
    public void insert(T tableObject) throws TableException {
        throw new UnsupportedOperationException(this.getFullName() + " does not support insert. Use AcquisitionServerSession instead");        
    }

    /**
     * delete rows (by id) from table
     *
     * @throws com.netapp.oci.platform.common.table.TableException
     */
    public void deleteById(long id) throws TableException {
        getPrepareStatementEx("DELETE FROM " + getFullName() + " WHERE id=?").setNext(id).executeUpdateEx();
    }

    /**
     * @param id
     * @return number of rows for id
     * @throws TableException
     */
    public int getCountById(long id) throws TableException {
        final String sql = "SELECT COUNT(*) FROM " + getFullName() + " WHERE id=?";
        return (int) getPrepareStatementEx(sql).setNext(id).executeFunctionQueryEx();
    }

    public long getMaxObjId() throws TableException {
        final String sql = "SELECT MAX(objId) FROM " + getFullName();

        return getPrepareStatementEx(sql).executeFunctionQueryEx();
    }
}
