package com.onaro.sanscreen.server.tables.acquisition;

import com.netapp.oci.platform.common.table.TableException;
import com.onaro.commons.table.DbConfig;
import com.onaro.sanscreen.server.dataobject.acquisition.AcquisitionUnitNew;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

public class AcquisitionUnitTableNew extends AcquisitionTable<AcquisitionUnitNew> {

    private static final String TABLE_NAME = "au";

    public AcquisitionUnitTableNew(Connection connection) {
        super(connection, TABLE_NAME);
    }

    public AcquisitionUnitNew parseObject(final ResultSet resultSet) throws SQLException {
        return new AcquisitionUnitNew(
                resultSet.getLong("id"),
                resultSet.getString("name"),
                resultSet.getString("ip"),
                AcquisitionUnitNew.Status.parseString(resultSet.getString("status")),
                resultSet.getLong("leaseperiod"),
                resultSet.getLong("nextleaserenewal"));
    }

    /**
     * This static method returns the full table name without initializing an instance. It will be aware of changes in customer prefix {@link com.onaro.commons.table.DbConfig}
     * @return
     */
    public static String FULL_TABLE_NAME() {
        return DbConfig.getAcquisitionDbName() + "." + TABLE_NAME;
    }

    public void update(final AcquisitionUnitNew acqUnit) throws TableException {
        throw new UnsupportedOperationException( "AcquisitionUnitTableNew does not support update. Use AcquisitionServerSession instead");
    }

    public List<AcquisitionUnitNew> findAllBrokenLeases() throws TableException {
        return findAllBrokenLeases(System.currentTimeMillis());
    }

    public List<AcquisitionUnitNew> findAllBrokenLeases(long time) throws TableException {
        return getPrepareStatementEx("SELECT * FROM " + getFullName() + " WHERE nextleaserenewal < ? and status <> '-1'")
                .setNext(time)
                .executeQueryEx();
    }

    public void deleteAll() throws TableException {
        getPrepareStatementEx("DELETE FROM " + getFullName() + " WHERE name<>'local'").executeUpdateEx();
    }

    public AcquisitionUnitNew getByName(final String name) throws TableException {
        List<AcquisitionUnitNew> col = getPrepareStatementEx("SELECT * FROM " + getFullName() + " WHERE name=?").setNext(name).executeQueryEx();

        if (col == null || (col.size() != 1))
            return null;

        return col.get(0);
    }

    public void deleteByName(final String name) throws TableException {
        getPrepareStatementEx("DELETE FROM " + getFullName() + " WHERE name=?").setNext(name).executeUpdateEx();
    }

    public List<AcquisitionUnitNew> findByName(final String name) throws TableException {
        return getPrepareStatementEx("SELECT * FROM " + getFullName() + " WHERE name=?")
                .setNext(name)
                .executeQueryEx();
    }
}
