package com.onaro.sanscreen.server.tables.acquisition;

import com.netapp.oci.platform.common.table.TableException;
import com.onaro.sanscreen.server.dataobject.acquisition.DsTypeAttribute;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

public class DataSourceTypeAttrsTable extends AcquisitionTable<DsTypeAttribute> {
    public static final String TABLE_NAME = "ds_type_attrs";

    public DataSourceTypeAttrsTable(Connection connection) {
        super(connection, DataSourceTypeAttrsTable.TABLE_NAME);
    }

    public void insert(final Map<String, DsTypeAttribute> attrs) throws TableException {
        for (final DsTypeAttribute guiAttribute : attrs.values()) {
            insert(guiAttribute.getDsTypeId(), guiAttribute);
        }
    }

    public void update(final Map<String, DsTypeAttribute> attrs) throws TableException {
        for (final DsTypeAttribute guiAttribute : attrs.values()) {
            update(guiAttribute);
        }
    }

    private void update(final DsTypeAttribute attr) throws TableException {
        final String sql = "UPDATE " + getFullName() + " SET name=?, " +
                           "value=?, type=?, role=?, encrypt=?, editable=?, description=?, label=?, min=?, max=?, enum=?, guiorder=? WHERE ds_type_id=?";
        getPrepareStatementEx(sql)
                .setNext(attr.getName())
                .setNext(attr.getValue())
                .setNext(attr.getType().toString())
                .setNext(attr.getRole())
                .setNext(attr.isEncrypt())
                .setNext(attr.isEditable())
                .setNext(attr.getDescription())
                .setNext(attr.getLabel())
                .setNext(attr.getMin())
                .setNext(attr.getMax())
                .setNext(attr.getEnumsStr())
                .setNext(attr.getGuiorder())
                .setNext(attr.getDsTypeId())
                .executeUpdateEx();
    }

    public void insert(final long ds_type_id, final DsTypeAttribute attr) throws TableException {
        final String sql = "INSERT INTO " + getFullName() +
                           " (ds_type_id, name, value, type, role, encrypt, editable, description, label, min, max, enum, guiorder) " +
                           "VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
        getPrepareStatementEx(sql)
                .setNext(ds_type_id)
                .setNext(attr.getName())
                .setNext(attr.getValue())
                .setNext(attr.getType().toString())
                .setNext(attr.getRole())
                .setNext(attr.isEncrypt())
                .setNext(attr.isEditable())
                .setNext(attr.getDescription())
                .setNext(attr.getLabel())
                .setNext(attr.getMin())
                .setNext(attr.getMax())
                .setNext(attr.getEnumsStr())
                .setNext(attr.getGuiorder())
                .executeUpdateEx();
    }

    public DsTypeAttribute parseObject(final ResultSet resultSet) throws SQLException {
        final long ds_type_id = Long.parseLong(resultSet.getString("ds_type_id"));
        final Boolean editable = resultSet.getBoolean("editable");
        final Boolean encrypt = resultSet.getBoolean("encrypt");
        final Integer guiorder = resultSet.getInt("guiorder");

        return new DsTypeAttribute(ds_type_id,
                                   resultSet.getString("name"),
                                   resultSet.getString("value"),
                                   resultSet.getString("description"),
                                   resultSet.getString("label"),
                                   resultSet.getString("type"),
                                   resultSet.getString("min"),
                                   resultSet.getString("max"),
                                   resultSet.getString("enum"),
                                   resultSet.getString("role"),
                                   editable.toString(),
                                   encrypt.toString(),
                                   guiorder.toString());
    }


    public List<DsTypeAttribute> findByDsTypeId(final long ds_type_id) throws TableException {
        return getPrepareStatementEx("SELECT * FROM " + getFullName() + " WHERE ds_type_id=? ORDER BY guiorder")
                .setNext(ds_type_id)
                .executeQueryEx();
    }

    public List<DsTypeAttribute> findByDsTypeIdOrderByName(final long ds_type_id) throws TableException {
        return getPrepareStatementEx("SELECT * FROM " + getFullName() + " WHERE ds_type_id=? ORDER BY name")
                .setNext(ds_type_id)
                .executeQueryEx();
    }

    public DsTypeAttribute getAttribute(final long ds_type_id, final String name) throws TableException {
        final List attrs = getPrepareStatementEx("SELECT * FROM " + getFullName() + " where ds_type_id=? and name=?")
                .setNext(ds_type_id)
                .setNext(name)
                .executeQueryEx();
        if (attrs == null || attrs.size() != 1)
            return null;
        return (DsTypeAttribute) attrs.get(0);
    }
}
