package com.onaro.sanscreen.server.tables.acquisition;

import com.netapp.oci.platform.common.table.TableException;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceType;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

public class DataSourceTypeTable extends AcquisitionTable<DataSourceType> {
    private static final String TABLE_NAME = "ds_type";

    public DataSourceTypeTable(Connection connection) {
        super(connection, TABLE_NAME);
    }

    public DataSourceType parseObject(final ResultSet resultSet) throws SQLException {
        return new DataSourceType(resultSet.getInt("id"),
                                  resultSet.getString("name"),
                                  resultSet.getString("description"));
    }

    public void insert(final DataSourceType dst) throws TableException {
        getPrepareStatementEx("INSERT INTO " + getFullName() + " (id, name, description) VALUES(?, ?, ?)")
                .setNext(dst.getId())
                .setNext(dst.getName())
                .setNext(dst.getDescription())
                .executeUpdateEx();
    }

    public void update(final DataSourceType dst) throws TableException {
        getPrepareStatementEx("UPDATE " + getFullName() + " SET name=?, description=? WHERE id=?")
                .setNext(dst.getName())
                .setNext(dst.getDescription())
                .setNext(dst.getId())
                .executeUpdateEx();
    }

    public void deleteById(final long dataSourceType) throws TableException {
        getPrepareStatementEx("DELETE FROM " + getFullName() + " WHERE id=?").setNext(dataSourceType).executeUpdateEx();
    }

    public DataSourceType getById(final long dataSourceType) throws TableException {
        List<DataSourceType> all = getPrepareStatementEx("SELECT * FROM " + getFullName() + " WHERE id=?").setNext(dataSourceType).executeQueryEx();
        if (all.size() == 1)
            return all.get(0);
        return null;
    }
}
