package com.onaro.sanscreen.server.tables.acquisition;

import com.netapp.oci.platform.common.table.TableException;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceTypeVendorModel;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

public class DataSourceTypeVendorModelTable extends AcquisitionTable<DataSourceTypeVendorModel> {
    public static final String TABLE_NAME = "ds_type_vendormodel";

    public DataSourceTypeVendorModelTable(final Connection connection) {
        super(connection, DataSourceTypeVendorModelTable.TABLE_NAME);
    }

    public void insert(final DataSourceTypeVendorModel vm) throws TableException {
        getPrepareStatementEx("INSERT INTO " + getFullName() + " (ds_type_id, vendorname, vendordescription, modelname, modeldescription) " +
                "VALUES(?, ?, ?, ?, ?)")
                .setNext(vm.getDsTypeId())
                .setNext(vm.getVendorName())
                .setNext(vm.getVendorDescription())
                .setNext(vm.getModelName())
                .setNext(vm.getModelDescription())
                .executeUpdateEx();
    }

    public DataSourceTypeVendorModel parseObject(final ResultSet resultSet) throws SQLException {
        return new DataSourceTypeVendorModel(
                    resultSet.getLong("ds_type_id"),
                    resultSet.getString("vendorname"),
                    resultSet.getString("vendordescription"),
                    resultSet.getString("modelname"),
                    resultSet.getString("modeldescription"));
    }

    public List<DataSourceTypeVendorModel> findByDsTypeId(final long ds_type_id) throws TableException {
        return getPrepareStatementEx("SELECT * FROM " + getFullName() + " WHERE ds_type_id=?").setNext(ds_type_id).executeQueryEx();
    }

    public List<DataSourceTypeVendorModel> findByVendor(final String vendor) throws TableException {
        return getPrepareStatementEx("SELECT * FROM " + getFullName() + " WHERE vendorname=?").setNext(vendor).executeQueryEx();
    }

    public List<DataSourceTypeVendorModel> findByIdVendor(final long ds_type_id, final String vendor) throws TableException {
        return getPrepareStatementEx("SELECT * FROM " + getFullName() + " WHERE ds_type_id=? AND vendorname=?").setNext(ds_type_id).setNext(vendor).executeQueryEx();
    }

    public DataSourceTypeVendorModel getByVendorModel(final String vendor, final String model) throws TableException {
        final List<DataSourceTypeVendorModel> dataSourceTypeVendorModels = getPrepareStatementEx("SELECT * FROM " + getFullName() + " WHERE vendorname=? AND modelname=?").setNext(vendor).setNext(model).executeQueryEx();
        if ( dataSourceTypeVendorModels == null || dataSourceTypeVendorModels.size() != 1)
            throw new TableException("Failed to find single vendor model");
        return dataSourceTypeVendorModels.get(0);
    }

    public void deleteByDataSourceType(final long ds_type_id) throws TableException {
        getPrepareStatementEx("DELETE FROM " + getFullName() + " WHERE ds_type_id=?").setNext(ds_type_id).executeUpdateEx();
    }
}
