/*
 * Copyright (c) 2010 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.server.tables.idgenerator.sessions;


import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.ServerConstant;
import com.onaro.sanscreen.server.interfaces.data.ServerInterfaceException;
import com.onaro.sanscreen.server.tables.idgenerator.IdGenerator;
import com.onaro.sanscreen.server.tables.idgenerator.IdGeneratorException;
import com.onaro.sanscreen.server.tables.idgenerator.IdTable;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.sql.DataSource;
import java.sql.Connection;


@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRES_NEW)
@RolesAllowed({"guest", "user", "admin", "scheduler"})
public class IdGeneratorSessionEJB implements IdGeneratorSession {

    @Resource(name = "jdbc/OnaroDataSource",
            mappedName = "java:/onaro_pool",
            type = DataSource.class,
            authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    public long getNextSeedNumber() throws ServerInterfaceException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            IdTable idTable = new IdTable(connection);
            return idTable.getNextSeedNumber();
        } catch (TableException e) {
            throw new ServerInterfaceException("Failed to generate NextSeedNumber ", e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }


    public void reset() throws ServerInterfaceException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);
            IdTable idTable = new IdTable(connection);
            idTable.reset();
        } catch (Exception e) {
            throw new ServerInterfaceException("Failed to reset", e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }

    public void resetState() throws ServerInterfaceException {
        try {
            IdGenerator.getInstance().reset();
        } catch (Exception e) {
            throw new ServerInterfaceException("Failed to reset", e);
        }
    }
}
