package com.onaro.sanscreen.server.tables.param;

import com.netapp.oci.platform.common.param.ParamException;
import com.netapp.oci.platform.common.table.StringCache;
import com.netapp.oci.platform.common.table.Table;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.common.table.TableResultParserDefault;
import com.onaro.commons.table.DbConfig;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * @author Anton Sergeev
 */
public class InfoTable extends Table<InfoTable.KeyToValue> {

    private static final String TABLE_NAME = "info";

    public InfoTable(Connection connection) {
        super(connection, DbConfig.getManagementDbName(), TABLE_NAME);
    }

    private int update(String param, String value) throws TableException {
        return getPrepareStatementEx("INSERT INTO " + getFullName() + " VALUES(?, ?) ON DUPLICATE KEY UPDATE value=?")
                .setNext(param)
                .setNext(value)
                .setNext(value)
                .executeUpdateEx();
    }

    private String select(String param) throws TableException {
        List<String> values = getPrepareStatementEx("SELECT value FROM " + getFullName() + " WHERE info=?", TableResultParserDefault.STRING)
                .setNext(param)
                .executeQueryEx();
        return values.isEmpty() ? null : StringCache.get(values.iterator().next());
    }

    public void set(String param, String value) throws TableException {
        update(param, value);
    }

    public KeyToValue parseObject(ResultSet resultSet) throws SQLException {
        String name = resultSet.getString(1);
        String value = StringCache.get(resultSet.getString(2));

        return new KeyToValue(name, value);
    }

    public String get(String param) throws TableException {
        return select(param);
    }

    public void deletePrefix(String prefix) throws ParamException {
        String sql = "DELETE FROM " + getFullName() + " WHERE info like \"" + prefix + "%\";";
        try {
            getPrepareStatementEx(sql).executeUpdateEx();
        } catch (TableException e) {
            throw new ParamException("Failed to run " + sql, e);
        }
    }

    public class KeyToValue {
        String key;
        String value;

        public KeyToValue(String key, String value) {
            this.key = key;
            this.value = value;
        }

        public String getKey() {
            return key;
        }

        public String getValue() {
            return value;
        }

        public String toString() {
            return key + " = " + value;
        }
    }
}
