package com.onaro.sanscreen.server.userregister;

import com.netapp.oci.platform.common.table.Table;
import com.netapp.oci.platform.common.table.TableException;
import com.onaro.commons.table.DbConfig;
import org.slf4j.Logger;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

public class UserRegisterTable extends Table<UserRegister> {

    private static final String TABLE_NAME = "user_registration";

    private String SQL_INSERT;
    private String SQL_DELETE;
    private String SQL_SELECT_BY_USER;

    static transient final Logger logger =org.slf4j.LoggerFactory.getLogger("com.onaro.sanscreen.server.userregister");

    public UserRegisterTable(Connection connection) {
        super(connection, DbConfig.getSanscreenDbName(), TABLE_NAME);

        SQL_INSERT = "INSERT INTO " + getFullName() + " VALUES(?, ?)";
        SQL_DELETE = "DELETE FROM " + getFullName() + " WHERE userId=? AND deviceId=?";
        SQL_SELECT_BY_USER = "SELECT * FROM " + getFullName() + " WHERE userId=?";
    }

    public UserRegister parseObject(ResultSet resultSet) throws SQLException {
        return new UserRegister(new Long(resultSet.getLong(1)), new Long(resultSet.getLong(2)));
    }

    public void insert(UserRegister userRegister) throws TableException {
        getPrepareStatementEx(SQL_INSERT).setNext(userRegister.getUserId()).setNext(userRegister.getDeviceId()).executeUpdateEx();
    }

    public void delete(Long userId, Long deviceId) throws TableException {
        getPrepareStatementEx(SQL_DELETE).setNext(userId).setNext(deviceId).executeUpdateEx();
    }

    public List<UserRegister> findByUserId(Long userId) throws TableException {
        return getPrepareStatementEx(SQL_SELECT_BY_USER).setNext(userId).executeQueryEx();
    }

}

