/*
 * Copyright (c) 2010 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.util;

import com.netapp.oci.platform.common.ServerConstant;

@SuppressWarnings("nls")
public class Constant {

    public static final String EMPTY_STRING = "";
    static public final long NULL_ID = -1;
    static public final Long LONG_NULL_ID = Long.valueOf(NULL_ID);
    static public final Long DEFAULT_CAPACITY_MB = Long.valueOf(-1);
    static public final Long DEFAULT_RAW_CAPACITY_MB = Long.valueOf(0);
    static public final long END_DATE = 5000000000000L; // Fri Jun 11 04:53:20 EDT 2128

    static public final Long MERGE_ORIGINATOR = Long.valueOf(-1);
    static public final int DEFAULT_ACTION_ORDER_INDEX = -1;
    static public final long NULL_TEMPLATE_ID = 0;

    public static final long LAST_KNOWN_ACCESS_TIME_NOT_SUPPORT_VALUE = -2;
    public static final long LAST_KNOWN_ACCESS_TIME_DATASOURCE_INIT_VALUE = -1;
    
    public static final int DISTRIBUTION_TIME_SPAN = 4 * 60 * 60 * 1000;

    public static final transient int SWITCH_DEFAULT_PRIORITY = 2;
    public static final transient int LOGICAL_SWITCH_DEFAULT_PRIORITY = 128;

    public static final transient String UNKNOWN_ORIGINATOR_NAME = "Unknown";

    //TODO : Acq and CLinet uses these constants. We need to remove them eventually
    public static final transient String HTTPS_JNDI_BINDING_PREFIX = ServerConstant.HTTPS_JNDI_BINDING_PREFIX;

    /**
     * Private constructor prevent this class from being created by mistake
     */
    private Constant() {
    }

    /**
     * Return true if the given id equals the LONG_NULL_ID constant,
     * false otherwise.
     *
     * @param id the id to check
     * @return true if the id is null, false otherwise.
     * @see #notNullId(Long)
     */
    public static boolean isNullId(Long id) {
        return LONG_NULL_ID.equals(id);
    }

    /**
     * Return false if the given id equals the LONG_NULL_ID constant,
     * true otherwise.
     *
     * @param id the id to check
     * @return false if the id is null, true otherwise.
     * @see #isNullId(Long)
     */
    public static boolean notNullId(Long id) {
        return !isNullId(id);
    }

    /**
     * Return true if the given value equals the DEFAULT_CAPACITY_MB constant,
     * false otherwise.
     *
     * @param megabytes the value to check
     * @return true if the value represents an undetermined capacity value, false otherwise.
     */
    public static boolean isUndeterminedCapacity(long megabytes) {
        return DEFAULT_CAPACITY_MB.longValue() == megabytes;
    }

    /**
     * Return true if the given value is null or
     * if it equals the DEFAULT_CAPACITY_MB constant,
     * false otherwise.
     *
     * @param megabytes the value to check
     * @return true if the value represents an undetermined capacity value, false otherwise.
     */
    public static boolean isUndeterminedCapacity(Number megabytes) {
        return megabytes == null || isUndeterminedCapacity (megabytes.longValue());
    }

    /**
     * Return true if the given value equals the DEFAULT_RAW_CAPACITY_MB constant,
     * false otherwise.
     *
     * @param megabytes the value to check
     * @return true if the value represents an undetermined raw capacity value, false otherwise.
     */
    public static boolean isUndeterminedRawCapacity(long megabytes) {
        return DEFAULT_RAW_CAPACITY_MB.longValue() == megabytes;
    }

    /**
     * Return true if the given value is null or 
     * if it equals the DEFAULT_RAW_CAPACITY_MB constant,
     * false otherwise.
     *
     * @param megabytes the value to check
     * @return true if the value represents an undetermined raw capacity value, false otherwise.
     */
    public static boolean isUndeterminedRawCapacity(Number megabytes) {
        return megabytes == null || isUndeterminedRawCapacity (megabytes.longValue());
    }
}
