/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2001</p>
 * <p>Company: Onaro</p>
 * @author unascribed
 * @version 1.0
 */

package com.onaro.sanscreen.util;

import java.io.*;
import java.util.*;


public class CsvTableParser {

	private String fileName;

	private List<LinkedList<String>> rowsList;

	private Map<String, Integer> headerIndexes;

	private boolean useHeader;

	public CsvTableParser(boolean useHeader) {
		this.rowsList = new LinkedList<LinkedList<String>>();
		this.headerIndexes = new HashMap<String, Integer>();
		this.useHeader = useHeader;
	}

	public void loadResource(String resourceName) throws IOException {
		this.fileName = resourceName;

		InputStream is = this.getClass().getResourceAsStream(fileName);
		if (is == null)
			throw new IOException("Fail get Resource \"" + resourceName + "\"");
		load(is, null);
	}

	public void load(String fileName, ColumnModifier[] modifiers) throws IOException {
		this.fileName = fileName;
		load(new FileInputStream(fileName), modifiers);
	}

	public void load(InputStream is, ColumnModifier[] modifiers) throws IOException {
        BufferedReader br = getCSVReader(is);

		StringBuilder builder = new StringBuilder();
        String strLine;
        while ((strLine = br.readLine()) != null) {
            builder.append(strLine);
            builder.append("\n\r");
        }

		loadBuffer(builder.toString(), modifiers);
	}

    private BufferedReader getCSVReader(InputStream is) throws IOException {
        PushbackInputStream pis = new PushbackInputStream(is, 4);
        BufferedReader br = getUnicodeReader(pis);
        if (br == null) {
            br = new BufferedReader(new InputStreamReader(pis));
        }

        return br;
    }

    private BufferedReader getUnicodeReader(PushbackInputStream pis) throws IOException {
        int c1 = pis.read();
        int c2 = pis.read();
        if (c1 == 0xFF && c2 == 0xFE) {
            int c3 = pis.read();
            int c4 = pis.read();
            if (c3 == 0 && c4 == 0) {
                return new BufferedReader(new InputStreamReader(pis, "UTF-32LE"));
            } else {
                pis.unread(c4);
                pis.unread(c3);
                return new BufferedReader(new InputStreamReader(pis, "UTF-16LE"));
            }
        } else if (c1 == 0xFE && c2 == 0xFF) {
            return new BufferedReader(new InputStreamReader(pis, "UTF-16BE"));
        } else if (c1 == 0 && c2 == 0) {
            int c3 = pis.read();
            int c4 = pis.read();
            if (c3 == 0xFE && c4 == 0xFF) {
                return new BufferedReader(new InputStreamReader(pis, "UTF-32BE"));
            }

            pis.unread(c4);
            pis.unread(c3);
        } else if (c1 == 0xEF && c2 == 0xBB) {
            int c3 = pis.read();
            if (c3 == 0xBF) {
                return new BufferedReader(new InputStreamReader(pis, "UTF-8"));
            }

            pis.unread(c3);
        }

        pis.unread(c2);
        pis.unread(c1);
        return null;
    }

    private LinkedList<String> parseLine(String line, ColumnModifier[] modifiers) {
		int size = line.length();
		boolean isApostrof = false;
		boolean tokenEnd = true;
		int apostrofCount = 0;
		char c = '\0';
		boolean isFirstLine = headerIndexes.size() == 0;
		StringBuffer token = new StringBuffer();
		StringBuffer fulToken = new StringBuffer();
		LinkedList<String> row = new LinkedList<String>();
		for (int i = 0; i <= size; i++) {
			if (i < size) {
				c = line.charAt(i);
			}
			if ((tokenEnd == true) && (isApostrof == false) && i < size
					&& line.charAt(i) == '"'
					&& (i == 0 || line.charAt(i - 1) == ','))
				isApostrof = true;

			if ((c == ',') || (i == size)) {
				if ((i == size) || (isApostrof == false)
						|| (isApostrof == true) && apostrofCount % 2 == 1) {

					String val = token.toString();
					if (isApostrof == true && val.indexOf(',') == -1)
						val = fulToken.toString();

					if (modifiers != null) {
						int index = row.size();
						if (modifiers.length > index) {
							ColumnModifier modifier = modifiers[index];
							if (modifier != null) {
								val = modifier.modify(val);
							}
						}
					}
					row.add(val);

					if (isFirstLine)
						// Bug # 12892.  
						// The problem was generated as described in DevTrack, by extra spaces at the end of the 
						// header line.  These spaces were wrongly included in the "key" string and then saved into <code> headerIndexes </code>
						// The exception was produced by trying to match the key-string against a enum type, making 
						// the comparison failed due to these extra spaces.  Once eliminated the unwanted spaces the rest 
						// of the code worked properly.
						headerIndexes.put(val.trim().toUpperCase(), new Integer(
								headerIndexes.size()));
					token = new StringBuffer();
					fulToken = new StringBuffer();
					tokenEnd = true;
					isApostrof = false;
					apostrofCount = 0;
					continue;
				}

			}

			tokenEnd = false;

			if (c != '"')
				apostrofCount = 0;

			if (isApostrof == true) {
				if (c == '"') {
					apostrofCount++;
				}
				if (apostrofCount % 2 == 0)
					token.append(c);

				fulToken.append(c);
			} else {
				token.append(c);
			}
		}
		return row;
	}

	public void loadBuffer(String buffer) {
		loadBuffer(buffer, null);
	}

	public void loadBuffer(String buffer, ColumnModifier[] modifiers) {
		rowsList.clear();
		StringTokenizer st = new StringTokenizer(buffer, "\n\r");

		while (st.hasMoreTokens()) {
			String line = st.nextToken();
			for (int i = 0; i < line.length(); i++) {
				if (line.charAt(i) != '\t' && line.charAt(i) != ' '
						&& line.charAt(i) != '\n') {
					rowsList.add(parseLine(line, modifiers));
					break;
				}
			}
		}
	}

	public int size() {
		if (useHeader)
			return rowsList.size() - 1;
		return rowsList.size();
	}

	//public String getValue(int rowIndex, String key) {
	/**
	 * Gets the value of the column specified by <code> key </code> in the 
	 * given <code> rowIndex </code>. <br>
	 * 
	 * This method has private access due to the fact that it is meant to be called 
	 * by the Iterator of this class only. This is to prevent situations in which callers used to call this method passing zero as 
	 * <code> rowIndex </code> retriving the "header" and then incorrectly use it as real data.  <br>
	 * 
	 * This is a confusing situation manifested in bug# 10670.  The Iterator solves the confusion
	 * freeing the user of managing the rowIndex when <code>useHeader</code> has set to true.
	 */
	public String getValue(int rowIndex, String key) {
		if (!useHeader)
			throw new IllegalArgumentException(
					"Method ggetKeyVal can be used onle when useHeader = true");

		String upperCaseKey = key.toUpperCase();
		Integer i = headerIndexes.get(upperCaseKey);
		if (i == null)
			throw new IllegalArgumentException("key " + upperCaseKey
					+ "  does not exist in " + fileName + " the keys are: "
					+ headerIndexes);

		return getValue(rowIndex, i);
	}

	public String getValue(int rowIndex, int columnIndex) {
		return getValue(rowIndex, Integer.valueOf(columnIndex));
	}

	public String getValue(int rowIndex, Integer columnIndex) {
		List<String> row = rowsList.get(rowIndex);
		if (row == null)
			throw new IllegalArgumentException("Row index " + rowIndex
					+ " out of bound");

		if (columnIndex.intValue() >= row.size())
			return "";

		return row.get(columnIndex.intValue());
	}

	public String toString() {
		StringBuffer str = new StringBuffer();
		for (LinkedList<String> row : rowsList) {
			Iterator<String> rowItr = row.iterator();
			while (rowItr.hasNext()) {
				String val = rowItr.next();
				if (val.indexOf(',') >= 0) {
					str.append('\"');
					str.append(val.replaceAll("\"", "\"\""));
					str.append('\"');
				} else {
					str.append(val);
				}
				if (rowItr.hasNext())
					str.append(",");
			}
			str.append("\n");
		}
		return str.toString();
	}

	public static interface ColumnModifier {
		public String modify(String value);
	}

	public static class ToUpperCase implements ColumnModifier {
		public String modify(String value) {
			return value != null ? value.toUpperCase() : null;
		}
	}

	public boolean exitColumnName(final String columnName){
		return headerIndexes.containsKey(columnName);
	}
	
	public CsvTableParser.CsvTableParserIterator createIterator() {
		return new CsvTableParserIterator(this);
	}

	/**
	 * The intention of creating an Iterator for {@see CsvTableParser} class is to overcome a confusing situation
	 * when <code> useHeader </code> is set.<br>
	 * 
	 * Based on the existing code the use of this variable is not consistent.  For example {@link CsvTableParser#size()} will subtract the first row when
	 * <code> useHeader = true </code>.
	 * 
	 * If the index of the row is zero and the <code> useHeader = true </code> you get the row that represents the Header. This has been source of problems 
	 * and confusion (bug #10643, #10735) when callers of this class has treat the first column as data.
	 * 
	 * @author john.cohen
	 *
	 */
	public static class CsvTableParserIterator {
		private int rowId;  // Controls the rowId based on "useHeader"
		private CsvTableParser csvTableParser; // Iterates over this instance.
		private int totalNumberOfDataRows; // total number of rows with data, not including the header.

		/**
		 * Creates an instance of this class setting the right row index based on 
		 * {@link CsvTableParser#useHeader} member variable.
		 */
		private CsvTableParserIterator(final CsvTableParser csvTableParser) {
			this.csvTableParser = csvTableParser;

			// Sets the correct index used to extract the information from the data model. 
			if (csvTableParser.useHeader){
				rowId = 0;
			    totalNumberOfDataRows = csvTableParser.rowsList.size() - 1;
			}
			else{
				rowId = -1;
				totalNumberOfDataRows = csvTableParser.rowsList.size();				
			}
		}

		/**
		 * Gets the next index that corresponds to the next row to be processed.
		 * 
		 * @return TRUE if you have more rows otherwise FALSE.
		 */
		public boolean getNextRow(){
			if (!hasNextRow())
			   return false;
			
			rowId++;
			return true;
		}
		
		public int getRowId(){
			return rowId;
		}
		
		/**
		 * Returns the next value base on a given <code> key </code>
		 */
		public String getColumn(final String key) {			
			return csvTableParser.getValue(rowId, key);
		}
		
		/**
		 * Returns TRUE if more data is available, otherwise FALSE.
		 * @return
		 */
		private boolean hasNextRow(){
			return totalNumberOfDataRows > rowId; 
		}
	}
}