package com.onaro.sanscreen.util.xml;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;


public class SaxParserWrapper {

    private DefaultHandler handler;

    private String schemaFileUri;

    static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
    static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema";
    static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";

    public SaxParserWrapper() {
    }

    public SaxParserWrapper(DefaultHandler handler) {
        setSaxHandler(handler);
    }
    
    public void setSaxHandler(DefaultHandler handler) {
        this.handler = handler;
    }

    public void run(InputSource is) throws XMLException {
        if (handler == null) {
            throw new XMLException("Fail parse input stream SaxHandler = null");
        }

        if (is == null) {
            throw new XMLException("Fail parse, input stream is is null");
        }

        try {
            SAXParserFactory factory = SAXParserFactory.newInstance();
            factory.setNamespaceAware(true);

            if (schemaFileUri != null) {
                factory.setValidating(true);
            }

            SAXParser parser = factory.newSAXParser();

            if (schemaFileUri != null) {
                parser.setProperty(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
                InputStream resourceAsStream = this.getClass().getResourceAsStream(schemaFileUri);
                if (resourceAsStream == null) {
                    throw new XMLException("Failed to load resource " + schemaFileUri);
                }
                parser.setProperty(JAXP_SCHEMA_SOURCE, resourceAsStream);
                handler.setFailOnError(true);
            }

            parser.parse(is, handler);

        } catch (SAXException e) {
            if (e instanceof SAXParseException) {
                SAXParseException saxParseException = (SAXParseException) e;
                StringBuilder str = new StringBuilder();
                str.append("Error while parsing XML - line:").append(saxParseException.getLineNumber()).append(", ");
                str.append("column:").append(saxParseException.getColumnNumber()).append("\r\n").append(e.getMessage());
                throw new XMLException(str.toString(), e);
            } else {
                throw new XMLException(e.getMessage(), e);
            }
        } catch (IOException e) {
            throw new XMLException("Got Error while parsing the xml", e);
        } catch (ParserConfigurationException e) {
            throw new XMLException("Parser Configuration error, should indicate incorrect version xml parser\\n use JAXP 2.1 supported parser", e);
        }
    }

    public void setSchemaFileUri(String schemaFileUri) {
        this.schemaFileUri = schemaFileUri;
    }

    public void run(InputStream is) throws XMLException {
        if (is == null) {
            throw new XMLException("Fail parse, input stream is is null");
        }

        run(new InputSource(is));
    }

    public void run(String fileName) throws XMLException {
        try {
            InputStream is = new FileInputStream(fileName);
            run(is);
        } catch (FileNotFoundException e) {
            throw new XMLException("Fail create FileInputStream from " + fileName, e);
        }
    }

    public void parseXmlString(String xml) throws XMLException {
        run(new InputSource(new StringReader(xml)));
    }

    public static void main(String[] args) throws XMLException {

        DefaultHandler h = new SaxPrintHandler();
        SaxParserWrapper parser = new SaxParserWrapper();
        parser.setSaxHandler(h);
        parser.run("C:\\Dev\\Scenario\\Attribute\\runBigScenario_2.xml");
    }
}







