package com.onaro.sanscreen.util.xml;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.StringTokenizer;

public class XMLUtil {


    /**
     * Special character Special meaning Entity encoding
     * > Begins a tag. &gt;
     * < Ends a tag. &lt;
     * " Quotation mark. &quot;
     * ' Apostrophe. &apos;
     * & Ampersand. &amp;
     *
     * @param value to be encoded
     *
     * @return engoding value.
     */
    static public String encode(String value) {
        if (value == null) {
            return null;
        }
        if (value.length() == 0) {
            return value;
        }

        StringTokenizer st = new StringTokenizer(value, "><\"'&", true);
        StringBuilder sb = new StringBuilder();
        while (st.hasMoreTokens()) {
            String val = st.nextToken();
            if (">".equals(val)) {
                sb.append("&gt;");
            } else if ("<".equals(val)) {
                sb.append("&lt;");
            } else if ("\"".equals(val)) {
                sb.append("&quot;");
            } else if ("'".equals(val)) {
                sb.append("&apos;");
            } else if ("&".equals(val)) {
                sb.append("&amp;");
            } else {
                sb.append(val);
            }
        }

        return sb.toString();
    }

    static public String getElementVal(String xml, String element) throws XMLException {
        int fpos = xml.indexOf("<" + element + ">");
        if (fpos >= 0) {
            int fendPos = xml.indexOf("</" + element + ">");
            if (fendPos <= 0) {
                throw new XMLException("BAD XML !!!  Fail find close element </" + element + ">    xml");
            }
            return xml.substring(fpos + element.length() + 2, fendPos);
        }
        return null;
    }

    static public Integer getIntegerElementVal(String xml, String element) throws XMLException {
        String elmentVal = getElementVal(xml, element);
        try {
            return Integer.valueOf(elmentVal);
        } catch (NumberFormatException e) {
            throw new XMLException("Element <" + element + "> value '" + elmentVal + "' is not a valid Integer format", e);
        }
    }

    static public Long getLongElementVal(String xml, String element) throws XMLException {
        String elmentVal = getElementVal(xml, element);
        try {
            return Long.valueOf(elmentVal);
        } catch (NumberFormatException e) {
            throw new XMLException("Element <" + element + "> value '" + elmentVal + "' is not a valid Long format", e);
        }
    }

    static public String getElementAttributeVal(String xml, String element, String attribute) throws XMLException {
        int fpos = xml.indexOf("<" + element);
        if (fpos > 0) {
            int fendPos = xml.indexOf(">", fpos + element.length());
            if (fendPos <= 0) {
                throw new XMLException("BAD XML !!!  Fail find close element </" + element + ">    xml");
            }

            fpos = xml.indexOf(attribute, fpos);
            if (fpos > 0 && fpos < fendPos) {
                fpos = xml.indexOf("\"", fpos);
                if (fpos == -1) {
                    fpos = xml.indexOf("'", fpos);
                }
                fendPos = xml.indexOf("\"", fpos + 1);
                if (fpos == -1) {
                    fendPos = xml.indexOf("'", fpos + 1);
                }
                if (fpos < fendPos) {
                    return xml.substring(fpos + 1, fendPos);
                }
                if (fpos == fendPos) {
                    return "";
                }
            }
        }
        return null;
    }


    static public String encode(Throwable throable) {
        StringWriter stackTrace = new StringWriter();
        throable.printStackTrace(new PrintWriter(stackTrace));
        return encode(stackTrace.toString());
    }

}
