package com.onaro.util;

import java.io.Serializable;
import java.util.*;

public class CounterMap<K, E> implements Serializable {

    private static final long serialVersionUID = 1L;

    private Map<K, Counter<E>> map = new HashMap<K, Counter<E>>();

    private Comparator<E> counterComparator = null;

    public CounterMap() {
        this(null);
    }

    public CounterMap(Comparator<E> counterComparator) {
        this.counterComparator = counterComparator;
    }

    public long get(K key, E obj) {
        Counter<E> counter = map.get(key);
        return counter != null ? counter.get(obj) : 0;
    }

    public Set<K> getKeys() {
        return map.keySet();
    }

    public int size() {
        return map.size();
    }

    public Counter<E> getCounter(K key) {
        return map.get(key);
    }

    public long inc(K key, E obj) {
        return inc(key, obj, 1);
    }

    public long inc(K key, E obj, long number) {
        Counter<E> counter = map.get(key);
        if (counter == null) {
            counter = new Counter<E>(counterComparator);
            map.put(key, counter);
        }

        return counter.inc(obj, number);
    }

    public void merge(K key, Counter<E> otherCounter) {
        Counter<E> currentCounter = map.get(key);
        if (currentCounter == null) {
            //copy the counter, otherwise it might change in the next merge
            map.put(key, new Counter<E>(otherCounter));
        } else {
            currentCounter.merge(otherCounter);
        }
    }


    public void merge(K key, Set<E> set) {
        Counter<E> currentCounter = map.get(key);
        if (currentCounter == null) {
            //copy the counter, otherwise it might change in the next merge
            map.put(key, new Counter<E>(set));
        } else {
            currentCounter.merge(set);
        }
    }
}
