package com.onaro.util;

import java.util.LinkedList;
import java.util.List;

/**
 * String related utility methods
 *
 * @author Anton Sergeev
 */
public class Strings {
    /**
     * Checks if string is empty
     *
     * @param test - test string
     * @return <code>true</code> if string is empty, <code> false</code> otherwise
     */
    public static boolean isEmpty(String test) {
        return test == null || test.trim().length() == 0;
    }

    public static <T> String toCommaSeparatedString(List<T> values) {
        StringBuilder builder = new StringBuilder();
        for (T value : values) {
            if (builder.length() > 0) {
                builder.append(",");
            }
            builder.append(value.toString());
        }
        return builder.toString();
    }

/**
     * Joins an array of strings into a string using a separator
     * @param strings array of strings to join
     * @param separator separator to use
     * @return string created from an array of strings using a separator
     * @author Daniel Nuriyev
     */
    public static String toString(List<String> strings, String separator) {

        if(strings == null) return null;
        if(separator == null) separator = "";

        StringBuilder builder = new StringBuilder();
        for (String string : strings) {
            if (builder.length() > 0) {
                builder.append(separator);
            }
            builder.append(string);
        }

        return builder.toString();
    }

    public static List<String> commaSeparatedStringToList(String commaSeparated) {
        return commaSeparatedStringToList(commaSeparated, new Converter<String>() {
            public String valueOf(String s) {
                return s;
            }
        });
    }

    public static <T> List<T> commaSeparatedStringToList(String commaSeparated, Converter<T> converter) {
        List<T> parts = new LinkedList<T>();
        for (String part : commaSeparated.split(",")) {
            part = part.trim();
            if (!part.isEmpty()) {
                parts.add(converter.valueOf(part));
            }
        }
        return parts;
    }


    public static interface Converter<T> {
        T valueOf(String s);
    }

}
