package com.onaro.util.enumeration;

import java.util.*;
import java.io.*;

public class EnumValueMap<K extends Enum,V> implements Serializable {
    private static final long serialVersionUID = 1L;
	private V[] values;

    public EnumValueMap(K[] keysUniverse) {
        setKeysUniverse(keysUniverse);
    }

    private void setKeysUniverse(K[] keysUniverse) {
        assert keysUniverse != null && keysUniverse.length > 0 : "array of possible enumeration values is either null or empty";

        int maxIndex = 0;
        for (K key : keysUniverse) {
            maxIndex = Math.max(maxIndex, key.ordinal());
        }

        values = (V[]) new Object[maxIndex + 1];
    }

    public void put(K key, V value) {
        int index = key.ordinal();
        assert index < values.length : "usig enum '" + key + "' with ordinal '" + index + "'. but the map was initialized with without it";

        values[index] = value;
    }

    public V get(K key) {
        int index = key.ordinal();
        assert index < values.length : "usig enum '" + key + "' with ordinal '" + index + "'. but the map was initialized with without it";

        return values[index];
    }

    public Collection<V> getValues() {
        ArrayList<V> rvValues = new ArrayList<V>();
        for(V value: values) {
            rvValues.add(value);
        }
        return rvValues;
    }
}
